Package: IOBR
Title: Immune Oncology Biological Research
Version: 2.2.3
Authors@R: c(
    person("Dongqiang", "Zeng", , "interlaken@smu.edu.cn", role = "aut"),
    person("Yiran", "Fang", , "fyr_nate@163.com", role = "aut"),
    person("Shixiang", "Wang", , "w_shixiang@163.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-9855-7357")),
    person("Qingcong", "Luo", , "qingcongl@163.com", role = "aut"),
    person("Hongqian", "Qian", , "536769786@qq.com", role = "aut")
  )
Description: Provides six modules for tumor microenvironment (TME)
    analysis based on multi-omics data. These modules cover data
    preprocessing, TME estimation, TME infiltrating patterns, cellular
    interactions, genome and TME interaction, and visualization for TME
    relevant features, as well as modelling based on key features. It
    integrates multiple microenvironmental analysis algorithms and
    signature estimation methods, simplifying the analysis and downstream
    visualization of the TME. In addition to providing a quick and easy
    way to construct gene signatures from single-cell RNA-seq data, it
    also provides a way to construct a reference matrix for TME
    deconvolution from single-cell RNA-seq data. The analysis pipeline and
    feature visualization are user-friendly and provide a comprehensive
    description of the complex TME, offering insights into tumour-immune
    interactions (Zeng D, et al. (2024)
    <doi:10.1016/j.crmeth.2024.100910>.  Fang Y, et al. (2025)
    <doi:10.1002/mdr2.70001>).
License: GPL-3
URL: https://doi.org/10.3389/fimmu.2021.687975 (paper),
        https://iobr.github.io/book/ (docs),
        https://iobr.github.io/IOBR/
BugReports: https://github.com/IOBR/IOBR/issues
Depends: R (>= 3.6.0)
Imports: cli, dplyr, ggplot2, glmnet, GSVA, methods, purrr, rlang,
        stringr, survival, survminer, tibble, tidyr
Suggests: BiocManager, BiocParallel, biomaRt, circlize,
        clusterProfiler, ComplexHeatmap, corrplot, DESeq2, doParallel,
        DOSE, e1071, easier, enrichplot, factoextra, FactoMineR,
        foreach, ggdensity, ggpp, ggpubr, ggsci, gridExtra, Hmisc,
        knitr, limma, limSolve, maftools, MASS, Matrix, msigdbr,
        NbClust, org.Hs.eg.db, org.Mm.eg.db, patchwork, PMCMRplus,
        pracma, preprocessCore, prettydoc, pROC, psych, RColorBrewer,
        reshape2, rmarkdown, ROCR, sampling, scales, Seurat,
        SeuratObject, sva, testthat (>= 3.0.0), tidyHeatmap, timeROC,
        webr, WGCNA
VignetteBuilder: knitr
Additional_repositories: https://bioconductor.org/packages/3.22/bioc,
        https://bioconductor.org/packages/3.22/data/annotation,
        https://bioconductor.org/packages/3.22/data/experiment,
        https://bioconductor.org/packages/3.23/bioc,
        https://bioconductor.org/packages/3.23/data/annotation,
        https://bioconductor.org/packages/3.23/data/experiment
biocViews: GeneExpression, DifferentialExpression, ImmunoOncology,
        Transcriptomics, Clustering, Survival, Visualization
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: FALSE
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-30 14:20:59 UTC; wsx
Author: Dongqiang Zeng [aut],
  Yiran Fang [aut],
  Shixiang Wang [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9855-7357>),
  Qingcong Luo [aut],
  Hongqian Qian [aut]
Maintainer: Shixiang Wang <w_shixiang@163.com>
Repository: CRAN
Date/Publication: 2026-05-30 15:30:02 UTC
Built: R 4.5.2; ; 2026-05-30 16:30:26 UTC; unix

Package: IOBR
Title: Immune Oncology Biological Research
Version: 2.2.3
Authors@R: c(
    person("Dongqiang", "Zeng", , "interlaken@smu.edu.cn", role = "aut"),
    person("Yiran", "Fang", , "fyr_nate@163.com", role = "aut"),
    person("Shixiang", "Wang", , "w_shixiang@163.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-9855-7357")),
    person("Qingcong", "Luo", , "qingcongl@163.com", role = "aut"),
    person("Hongqian", "Qian", , "536769786@qq.com", role = "aut")
  )
Description: Provides six modules for tumor microenvironment (TME)
    analysis based on multi-omics data. These modules cover data
    preprocessing, TME estimation, TME infiltrating patterns, cellular
    interactions, genome and TME interaction, and visualization for TME
    relevant features, as well as modelling based on key features. It
    integrates multiple microenvironmental analysis algorithms and
    signature estimation methods, simplifying the analysis and downstream
    visualization of the TME. In addition to providing a quick and easy
    way to construct gene signatures from single-cell RNA-seq data, it
    also provides a way to construct a reference matrix for TME
    deconvolution from single-cell RNA-seq data. The analysis pipeline and
    feature visualization are user-friendly and provide a comprehensive
    description of the complex TME, offering insights into tumour-immune
    interactions (Zeng D, et al. (2024)
    <doi:10.1016/j.crmeth.2024.100910>.  Fang Y, et al. (2025)
    <doi:10.1002/mdr2.70001>).
License: GPL-3
URL: https://doi.org/10.3389/fimmu.2021.687975 (paper),
        https://iobr.github.io/book/ (docs),
        https://iobr.github.io/IOBR/
BugReports: https://github.com/IOBR/IOBR/issues
Depends: R (>= 3.6.0)
Imports: cli, dplyr, ggplot2, glmnet, GSVA, methods, purrr, rlang,
        stringr, survival, survminer, tibble, tidyr
Suggests: BiocManager, BiocParallel, biomaRt, circlize,
        clusterProfiler, ComplexHeatmap, corrplot, DESeq2, doParallel,
        DOSE, e1071, easier, enrichplot, factoextra, FactoMineR,
        foreach, ggdensity, ggpp, ggpubr, ggsci, gridExtra, Hmisc,
        knitr, limma, limSolve, maftools, MASS, Matrix, msigdbr,
        NbClust, org.Hs.eg.db, org.Mm.eg.db, patchwork, PMCMRplus,
        pracma, preprocessCore, prettydoc, pROC, psych, RColorBrewer,
        reshape2, rmarkdown, ROCR, sampling, scales, Seurat,
        SeuratObject, sva, testthat (>= 3.0.0), tidyHeatmap, timeROC,
        webr, WGCNA
VignetteBuilder: knitr
Additional_repositories: https://bioconductor.org/packages/3.22/bioc,
        https://bioconductor.org/packages/3.22/data/annotation,
        https://bioconductor.org/packages/3.22/data/experiment,
        https://bioconductor.org/packages/3.23/bioc,
        https://bioconductor.org/packages/3.23/data/annotation,
        https://bioconductor.org/packages/3.23/data/experiment
biocViews: GeneExpression, DifferentialExpression, ImmunoOncology,
        Transcriptomics, Clustering, Survival, Visualization
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: FALSE
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-30 14:20:59 UTC; wsx
Author: Dongqiang Zeng [aut],
  Yiran Fang [aut],
  Shixiang Wang [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9855-7357>),
  Qingcong Luo [aut],
  Hongqian Qian [aut]
Maintainer: Shixiang Wang <w_shixiang@163.com>
Repository: CRAN
Date/Publication: 2026-05-30 15:30:02 UTC
Built: R 4.6.0; ; 2026-05-30 17:24:01 UTC; unix

# Generated by roxygen2: do not edit by hand

export(BinomialAUC)
export(BinomialModel)
export(CIBERSORT)
export(CalculatePref)
export(CalculateTimeROC)
export(Construct_con)
export(ConvertRownameToLoci)
export(CoreAlg)
export(DrawQQPlot)
export(EPIC)
export(Enet)
export(GetFractions.Abbas)
export(GetOutlierGenes)
export(IPS_calculation)
export(LR_cal)
export(MCPcounter.estimate)
export(ParseInputExpression)
export(PlotAUC)
export(PlotTimeROC)
export(ProcessingData)
export(PrognosticAUC)
export(PrognosticModel)
export(PrognosticResult)
export(RegressionResult)
export(RemoveBatchEffect)
export(SplitTrainTest)
export(Top_probe)
export(add_iobr_mirror)
export(add_riskscore)
export(anno_eset)
export(assimilate_data)
export(batch_cor)
export(batch_kruskal)
export(batch_pcc)
export(batch_sig_surv_plot)
export(batch_surv)
export(batch_wilcoxon)
export(best_cutoff)
export(best_cutoff2)
export(calculate_break_month)
export(calculate_sig_score)
export(calculate_sig_score_integration)
export(calculate_sig_score_pca)
export(calculate_sig_score_ssgsea)
export(calculate_sig_score_zscore)
export(cell_bar_plot)
export(check_cancer_types)
export(check_eset)
export(clear_iobr_cache)
export(combine_pd_eset)
export(count2tpm)
export(creat_folder)
export(deconvo_cibersort)
export(deconvo_epic)
export(deconvo_estimate)
export(deconvo_ips)
export(deconvo_mcpcounter)
export(deconvo_quantiseq)
export(deconvo_ref)
export(deconvo_timer)
export(deconvo_tme)
export(deconvo_xcell)
export(deconvolute_quantiseq.default)
export(deconvolute_timer.default)
export(design_mytheme)
export(doPerm)
export(download_iobr_data)
export(enrichment_barplot)
export(eset_distribution)
export(estimateScore)
export(exact_pvalue)
export(extract_sc_data)
export(feature_manipulation)
export(feature_select)
export(filterCommonGenes)
export(find_markers_in_bulk)
export(find_mutations)
export(find_outlier_samples)
export(find_variable_genes)
export(format_msigdb)
export(format_signatures)
export(generateRef)
export(generateRef_DEseq2)
export(generateRef_limma)
export(generateRef_rnaseq)
export(generateRef_seurat)
export(getHRandCIfromCoxph)
export(get_cols)
export(get_cor)
export(get_cor_matrix)
export(get_iobr_cache_dir)
export(get_sig_sc)
export(high_var_fea)
export(iobr_cor_plot)
export(iobr_deconvo_pipeline)
export(iobr_deg)
export(iobr_pca)
export(ipsmap)
export(lasso_select)
export(limma.dif)
export(list_github_datasets)
export(list_iobr_mirrors)
export(load_data)
export(log2eset)
export(make_mut_matrix)
export(mapbw)
export(mapcolors)
export(merge_duplicate)
export(merge_eset)
export(mouse2human_eset)
export(outputGCT)
export(output_sig)
export(palettes)
export(parallel_doperm)
export(patterns_to_na)
export(percent_bar_plot)
export(pie_chart)
export(plotPurity)
export(random_strata_cells)
export(rbind_iobr)
export(remove_batcheffect)
export(remove_duplicate_genes)
export(remove_names)
export(reset_iobr_cache_dir)
export(reset_iobr_mirrors)
export(roc_time)
export(scale_matrix)
export(select_method)
export(set_iobr_cache_dir)
export(sigScore)
export(sig_box)
export(sig_box_batch)
export(sig_forest)
export(sig_gsea)
export(sig_heatmap)
export(sig_pheatmap)
export(sig_roc)
export(sig_surv_plot)
export(signature_score_calculation_methods)
export(subgroup_survival)
export(surv_group)
export(tcga_rna_preps)
export(test_for_infiltration)
export(timer_available_cancers)
export(timer_info)
export(tme_cluster)
export(tme_deconvolution_methods)
export(transform_data)
import(dplyr)
import(ggplot2)
import(glmnet)
import(grid)
import(parallel)
import(purrr)
import(stringr)
import(survival)
import(tibble)
import(tidyr)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,curve)
importFrom(graphics,legend)
importFrom(graphics,matlines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,stars)
importFrom(methods,as)
importFrom(methods,is)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,dist)
importFrom(stats,end)
importFrom(stats,glm)
importFrom(stats,hclust)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,lsfit)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,prcomp)
importFrom(stats,pt)
importFrom(stats,qqplot)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,sd)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(survminer,surv_cutpoint)
importFrom(tibble,column_to_rownames)
importFrom(utils,adist)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,menu)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,write.csv)
importFrom(utils,write.table)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# IOBR: Immuno-Oncology Biological Research

[![R-CMD-check.yaml](https://github.com/IOBR/IOBR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/IOBR/IOBR/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/IOBR)](https://cran.r-project.org/package=IOBR)
[![CRAN
downloads](https://cranlogs.r-pkg.org/badges/grand-total/IOBR?color=blue)](https://cran.r-project.org/package=IOBR)
[![GitHub
stars](https://img.shields.io/github/stars/IOBR/IOBR.svg)](https://GitHub.com/IOBR/IOBR/stargazers/)
[![GitHub
issues](https://img.shields.io/github/issues/IOBR/IOBR.svg)](https://GitHub.com/IOBR/IOBR/issues/)
[![License](https://img.shields.io/badge/license-GPL%203.0-blue.svg)](https://opensource.org/licenses/GPL-3.0)

IOBR is a comprehensive R package designed for immuno-oncology research,
providing a one-stop solution for tumor microenvironment (TME)
deconvolution, signature analysis, and integrated visualization. It
integrates multiple state-of-the-art algorithms and curated gene sets to
facilitate in-depth analysis of tumor immunity.

## 📊 Package Workflow

<figure>
<img src="./man/figures/IOBR-Workflow.png" alt="IOBR Workflow" />
<figcaption aria-hidden="true">IOBR Workflow</figcaption>
</figure>

<figure>
<img src="./man/figures/IOBR-Package.png" alt="IOBR Package" />
<figcaption aria-hidden="true">IOBR Package</figcaption>
</figure>

## 🎯 Key Features

### 1. Extensive Signature Collection

- **322+ published signature gene sets** covering TME, metabolism, m6A,
  exosomes, microsatellite instability, tertiary lymphoid structure, and
  more
- Easy access to signature genes and source citations
- Flexible signature management and customization

### 2. Multi-algorithm TME Deconvolution

Integrates 8 cutting-edge TME decoding methodologies:

- `CIBERSORT` - Cell-type identification by estimating relative subsets
  of RNA transcripts
- `TIMER` - Tumor Immune Estimation Resource
- `xCell` - Digital portrayal of tissue cellular heterogeneity
- `MCPcounter` - Estimation of immune and stromal cell populations
- `ESTIMATE` - Inference of tumor purity and stromal/immune cell
  admixture
- `EPIC` - Enumeration of cancer and immune cell types
- `IPS` - Immunophenoscore calculation
- `quanTIseq` - Quantification of tumor-infiltrating immune cells

### 3. Signature Score Calculation

Three robust computational methods for signature scoring:

- `PCA` - Principal Component Analysis
- `z-score` - Standardized expression scoring
- `ssGSEA` - Single-sample Gene Set Enrichment Analysis

### 4. Integrated Analysis Pipeline

- Data preprocessing and normalization
- Batch effect correction
- Feature selection and dimensionality reduction
- Survival analysis and risk modeling
- Statistical testing and visualization

### 5. Rich Visualization Tools

- Batch survival analysis plots
- Subgroup characteristic visualization
- Correlation heatmaps and scatter plots
- Forest plots for biomarker validation
- Customizable themes and color palettes

## ⚠️ Important Note

To better integrate IOBR into the bioinformatics community, we have
recently made extensive changes to optimize the code and enhance
documentation. However, these improvements may introduce code
instability and inconsistency. If you need to reproduce previous
analysis results, please install and use the previously tagged versions
from <https://github.com/IOBR/IOBR/tags>.

## 📦 Installation

### Prerequisites

- R version 3.6.0 or higher
- Bioconductor version 3.10 or higher

``` r
# Install BiocManager if not already installed
if (!requireNamespace("BiocManager", quietly = TRUE)) {
  install.packages("BiocManager")
}
```

### Install from GitHub

``` r
# Install IOBR from GitHub
BiocManager::install("IOBR/IOBR")
```

#### For Chinese Users (Faster Download Maybe)

``` r
# Install remotes if not already installed
if (!requireNamespace("remotes", quietly = TRUE)) {
  install.packages("remotes")
}

# Install IOBR using a mirror for faster download
remotes::install_git("https://ghfast.top/https://github.com/IOBR/IOBR")
```

### Install from CRAN

``` r
# Install IOBR from GitHub
BiocManager::install("IOBR")
```

> Note: install with `dependencies = TRUE` is recommended to ensure all
> required packages are installed for full functionality.
>
> `BiocManager::install("IOBR", dependencies = TRUE)`

### Install via Conda/Mamba

Using Conda or Mamba provides a clean, isolated environment with all
system dependencies pre-installed:

``` bash
# Create a new conda environment with R and essential dependencies
mamba create -n r45 -c conda-forge -c bioconda \
  r-base=4.5 r-xml r-magick r-cairo r-gdtools r-remotes r-tidyverse r-biocmanager

# Activate the environment
mamba activate r45
```

Then install IOBR in R:

``` r
BiocManager::install("IOBR/IOBR")
```

### Use Docker

> Only works for Linux.

``` bash
# Pull the image
docker pull ghcr.io/iobr/iobr:latest
# China: docker pull ghcr.nju.edu.cn/iobr/iobr:latest

# Run with RStudio Server
docker run -d -p 8787:8787 -e PASSWORD=yourpassword ghcr.io/iobr/iobr:latest

# Access RStudio at http://localhost:8787
# Username: rstudio
# Password: yourpassword
```

### Load the Package

``` r
library(IOBR)
```

## 💾 Data Cache Management

IOBR downloads large datasets from [GitHub
releases](https://github.com/IOBR/IOBR/releases/tag/data-v1.0/) and
caches them locally. You can customize the cache location:

``` r
# Check current cache directory
get_iobr_cache_dir()

# Set a custom cache directory (persists for current R session)
set_iobr_cache_dir("~/my_iobr_cache")

# Or set via options (useful in .Rprofile for permanent setting)
options(IOBR.cache_dir = "~/my_iobr_cache")

# Clear cached data
clear_iobr_cache()

# Reset to default cache location
reset_iobr_cache_dir()
```

To permanently set a custom cache directory, add this to your
`~/.Rprofile`:

``` r
options(IOBR.cache_dir = "/path/to/your/cache")
```

> When R fails to download, you can manually download the data from
> GitHub or GitHub acceleration sites, and then configure the cache
> directory.

## 🚀 Quick Start

### TME Deconvolution

``` r
# List available TME deconvolution methods
tme_deconvolution_methods

# Perform TME deconvolution using multiple methods
# Assuming you have an expression set object 'eset'
tme_result <- deconvo_tme(eset, 
                          methods = c("cibersort", "timer", "xcell"),
                          output_format = "data.frame")
```

### Signature Score Calculation

``` r
# List available signature score calculation methods
signature_score_calculation_methods

# Calculate signature scores using ssGSEA
# Assuming you have an expression set object 'eset' and signature list 'sig_list'
sig_scores <- sigScore(eset, 
                       signature = sig_list,
                       method = "ssgsea")
```

## 📖 Documentation

### IOBR Book

For detailed tutorials and case studies, please refer to the [IOBR
Book](https://iobr.github.io/book/), which provides comprehensive
guidance on:

- Installation and setup
- Data preprocessing
- TME deconvolution
- Signature analysis
- Survival analysis
- Visualization techniques

### Package Vignettes

Vignettes are available within the package and can be accessed using:

``` r
browseVignettes("IOBR")
```

## 🛠️ Available Methods

### TME Deconvolution Methods

``` r
tme_deconvolution_methods
#>         MCPcounter               EPIC              xCell          CIBERSORT 
#>       "mcpcounter"             "epic"            "xcell"        "cibersort" 
#> CIBERSORT Absolute                IPS           ESTIMATE                SVR 
#>    "cibersort_abs"              "ips"         "estimate"              "svr" 
#>               lsei              TIMER          quanTIseq 
#>             "lsei"            "timer"        "quantiseq"
```

### Signature Score Calculation Methods

``` r
signature_score_calculation_methods
#>           PCA        ssGSEA       z-score   Integration 
#>         "pca"      "ssgsea"      "zscore" "integration"
```

### Signature Collection

``` r
signature_collection <- load_data("signature_collection")
# Number of available signatures
length(signature_collection)
#> [1] 323

head(signature_collection)
#> $CD_8_T_effector
#> [1] "CD8A"   "GZMA"   "GZMB"   "IFNG"   "CXCL9"  "CXCL10" "PRF1"   "TBX21" 
#> 
#> $DDR
#>   [1] "UNG"     "SMUG1"   "MBD4"    "OGG1"    "MUTYH"   "NTHL1"   "MPG"    
#>   [8] "NEIL1"   "NEIL2"   "NEIL3"   "APEX1"   "APEX2"   "LIG3"    "XRCC1"  
#>  [15] "PNKP"    "APLF"    "PARP1"   "PARP2"   "PARP3"   "MGMT"    "ALKBH2" 
#>  [22] "ALKBH3"  "TDP1"    "TDP2"    "MSH2"    "MSH3"    "MSH6"    "MLH1"   
#>  [29] "PMS2"    "MSH4"    "MSH5"    "MLH3"    "PMS1"    "XPC"     "RAD23B" 
#>  [36] "CETN2"   "RAD23A"  "XPA"     "DDB1"    "DDB2"    "RPA1"    "RPA2"   
#>  [43] "RPA3"    "ERCC3"   "ERCC2"   "GTF2H1"  "GTF2H2"  "GTF2H3"  "GTF2H4" 
#>  [50] "GTF2H5"  "CDK7"    "CCNH"    "MNAT1"   "ERCC5"   "ERCC1"   "ERCC4"  
#>  [57] "LIG1"    "ERCC8"   "ERCC6"   "UVSSA"   "XAB2"    "MMS19"   "RAD51"  
#>  [64] "RAD51B"  "RAD51D"  "DMC1"    "XRCC2"   "XRCC3"   "RAD52"   "RAD54L" 
#>  [71] "RAD54B"  "BRCA1"   "SHFM1"   "RAD50"   "MRE11A"  "NBN"     "RBBP8"  
#>  [78] "MUS81"   "EME1"    "EME2"    "GEN1"    "FANCA"   "FANCB"   "FANCC"  
#>  [85] "BRCA2"   "FANCD2"  "FANCE"   "FANCF"   "FANCG"   "FANCI"   "BRIP1"  
#>  [92] "FANCL"   "FANCM"   "PALB2"   "RAD51C"  "XRCC6"   "XRCC5"   "PRKDC"  
#>  [99] "LIG4"    "XRCC4"   "DCLRE1C" "NHEJ1"   "NUDT1"   "DUT"     "RRM2B"  
#> [106] "POLB"    "POLG"    "POLD1"   "POLE"    "PCNA"    "REV3L"   "MAD2L2" 
#> [113] "POLH"    "POLI"    "POLQ"    "POLK"    "POLL"    "POLM"    "POLN"   
#> [120] "FEN1"    "FAN1"    "TREX1"   "EXO1"    "APTX"    "ENDOV"   "UBE2A"  
#> [127] "UBE2B"   "RAD18"   "SHPRH"   "HLTF"    "RNF168"  "SPRTN"   "RNF8"   
#> [134] "RNF4"    "UBE2V2"  "UBE2N"   "H2AFX"   "CHAF1A"  "SETMAR"  "BLM"    
#> [141] "WRN"     "RECQL4"  "ATM"     "DCLRE1A" "DCLRE1B" "RPA4"    "PRPF19" 
#> [148] "RECQL"   "RECQL5"  "HELQ"    "RDM1"    "ATR"     "ATRIP"   "MDC1"   
#> [155] "RAD1"    "RAD9A"   "HUS1"    "RAD17"   "CHEK1"   "CHEK2"   "TP53"   
#> [162] "TP53BP1" "RIF1"    "TOPBP1"  "CLK2"    "PER1"   
#> 
#> $APM
#> [1] "B2M"   "HLA-A" "HLA-B" "HLA-C" "TAP1"  "TAP2" 
#> 
#> $Immune_Checkpoint
#> [1] "CD274"    "PDCD1LG2" "CTLA4"    "PDCD1"    "LAG3"     "HAVCR2"   "TIGIT"   
#> 
#> $CellCycle_Reg
#>  [1] "ATM"    "CDKN1A" "CDKN2A" "MDM2"   "TP53"   "CCND1"  "RB1"    "CCNE1" 
#>  [9] "FBXW7"  "E2F3"  
#> 
#> $Pan_F_TBRs
#>  [1] "ACTA2"    "ACTG2"    "ADAM12"   "ADAM19"   "CNN1"     "COL4A1"  
#>  [7] "CTGF"     "CTPS1"    "FAM101B"  "FSTL3"    "HSPB1"    "IGFBP3"  
#> [13] "PXDC1"    "SEMA7A"   "SH3PXD2A" "TAGLN"    "TGFBI"    "TNS1"    
#> [19] "TPM1"

signature_collection_citation <- load_data("signature_collection_citation")
head(signature_collection_citation)
#> # A tibble: 6 × 6
#>   Signatures        `Published year` Journal Title                   PMID  DOI  
#>   <chr>                        <dbl> <chr>   <chr>                   <chr> <chr>
#> 1 CD_8_T_effector               2018 Nature  TGFβ attenuates tumour… 2944… 10.1…
#> 2 DDR                           2018 Nature  TGFβ attenuates tumour… 2944… 10.1…
#> 3 APM                           2018 Nature  TGFβ attenuates tumour… 2944… 10.1…
#> 4 Immune_Checkpoint             2018 Nature  TGFβ attenuates tumour… 2944… 10.1…
#> 5 CellCycle_Reg                 2018 Nature  TGFβ attenuates tumour… 2944… 10.1…
#> 6 Pan_F_TBRs                    2018 Nature  TGFβ attenuates tumour… 2944… 10.1…

sig_group <- load_data("sig_group")
sig_group[1:3]
#> $tumor_signature
#>  [1] "CellCycle_Reg"                            
#>  [2] "Cell_cycle"                               
#>  [3] "DDR"                                      
#>  [4] "Mismatch_Repair"                          
#>  [5] "Histones"                                 
#>  [6] "Homologous_recombination"                 
#>  [7] "Nature_metabolism_Hypoxia"                
#>  [8] "Molecular_Cancer_m6A"                     
#>  [9] "MT_exosome"                               
#> [10] "Positive_regulation_of_exosomal_secretion"
#> [11] "Ferroptosis"                              
#> [12] "EV_Cell_2020"                             
#> 
#> $EMT
#> [1] "Pan_F_TBRs" "EMT1"       "EMT2"       "EMT3"       "WNT_target"
#> 
#> $io_biomarkers
#>  [1] "TMEscore_CIR"                    "TMEscoreA_CIR"                  
#>  [3] "TMEscoreB_CIR"                   "T_cell_inflamed_GEP_Ayers_et_al"
#>  [5] "CD_8_T_effector"                 "IPS_IPS"                        
#>  [7] "Immune_Checkpoint"               "Exhausted_CD8_Danaher_et_al"    
#>  [9] "Pan_F_TBRs"                      "Mismatch_Repair"                
#> [11] "APM"
```

## 📊 Licenses and Citations

### TME Deconvolution Methods

| Method | License | Citation |
|----|----|----|
| [CIBERSORT](https://cibersortx.stanford.edu/) | Free for non-commercial use only | Newman, A. M., et al. (2015). Nature Methods, 12(5), 453–457. <https://doi.org/10.1038/nmeth.3337> |
| [ESTIMATE](https://bioinformatics.mdanderson.org/public-software/estimate/) | Free ([GPL2.0](https://bioinformatics.mdanderson.org/estimate/)) | Vegesna R, et al. (2013). Nature Communications, 4, 2612. <https://doi.org/10.1038/ncomms3612> |
| [quanTIseq](https://icbi.i-med.ac.at/software/quantiseq/doc/index.html) | Free ([GPL2.0](https://github.com/omnideconv/immunedeconv/blob/master/LICENSE)) | Finotello, F., et al. (2019). Genome Medicine, 11(1), 34. <https://doi.org/10.1186/s13073-019-0638-6> |
| [TIMER](https://github.com/hanfeisun/TIMER) | Free ([GPL 2.0](https://github.com/hanfeisun/TIMER)) | Li, B., et al. (2016). Genome Biology, 17(1), 174. <https://doi.org/10.1186/s13059-016-1028-7> |
| [IPS](https://github.com/icbi-lab/Immunophenogram) | Free ([BSD](https://github.com/icbi-lab/Immunophenogram/blob/master/LICENSE)) | Charoentong P, et al. (2017). Cell Reports, 18, 248-262. <https://doi.org/10.1016/j.celrep.2016.12.019> |
| [MCPCounter](https://github.com/ebecht/MCPcounter) | Free ([GPL 3.0](https://github.com/ebecht/MCPcounter/blob/master/Source/License)) | Becht, E., et al. (2016). Genome Biology, 17(1), 218. <https://doi.org/10.1186/s13059-016-1070-5> |
| [xCell](https://github.com/dviraran/xCell) | Free ([GPL 3.0](https://github.com/dviraran/xCell/blob/master/DESCRIPTION)) | Aran, D., et al. (2017). Genome Biology, 18(1), 220. <https://doi.org/10.1186/s13059-017-1349-1> |
| [EPIC](https://github.com/GfellerLab/EPIC) | Free for non-commercial use only ([Academic License](https://github.com/GfellerLab/EPIC/blob/master/LICENSE)) | Racle, J., et al. (2017). eLife, 6, e26476. <https://doi.org/10.7554/eLife.26476> |

### Signature Estimation Methods

| Method | License | Citation |
|----|----|----|
| [GSVA](https://www.bioconductor.org/packages/release/bioc/html/GSVA.html) | Free ([GPL (\>= 2)](https://github.com/rcastelo/GSVA)) | Hänzelmann S, et al. (2013). BMC Bioinformatics, 14, 7. <https://doi.org/10.1186/1471-2105-14-7> |

## 📝 Citing IOBR

If you use IOBR in your research, please cite both the IOBR package and
the specific methods you employ.

1.  Zeng DQ, Fang YR, …, Liao WJ. Enhancing Immuno-Oncology
    Investigations Through Multidimensional Decoding of Tumour
    Microenvironment with IOBR 2.0, **Cell Reports Methods**, 2024
    <https://doi.org/10.1016/j.crmeth.2024.100910>

2.  Fang YR, …, Liao WJ, Zeng DQ, Systematic Investigation of Tumor
    Microenvironment and Antitumor Immunity With IOBR, **Med Research**,
    2025 <https://onlinelibrary.wiley.com/doi/epdf/10.1002/mdr2.70001>

## 🤝 Contributing

We welcome contributions to IOBR! If you’re interested in contributing,
please:

1.  Fork the repository
2.  Create a feature branch
3.  Make your changes
4.  Submit a pull request

Please ensure your code follows the project’s coding standards and
includes appropriate documentation and tests.

## 🐛 Reporting Bugs

If you encounter any bugs or issues, please report them to the [GitHub
issues page](https://github.com/IOBR/IOBR/issues). When reporting bugs,
please include:

- A clear and descriptive title
- A detailed description of the issue
- Minimal reproducible example (if possible)
- Your R and IOBR versions
- Any error messages or warnings

## 📧 Contact

For questions or inquiries, please contact:

- Dr. Deqiang Zeng: <interlaken@smu.edu.cn>
- Dr. Yiran Fang: <fyr_nate@163.com>

## 📄 License

IOBR is released under the GNU General Public License.

#' Create a Percent Bar Plot
#'
#' @description
#' Generates a bar plot visualizing the percentage distribution of a variable
#' grouped by another variable.
#'
#' @param input Input data frame.
#' @param x Name of the x-axis variable.
#' @param y Name of the y-axis (grouping) variable.
#' @param subset.x Optional subset of x-axis values.
#' @param color Optional color palette.
#' @param palette Optional palette type.
#' @param title Optional plot title.
#' @param axis_angle Angle for axis labels (0-90). Default is 0.
#' @param coord_flip Logical to flip coordinates. Default is FALSE.
#' @param add_Freq Logical to add frequency count. Default is TRUE.
#' @param Freq Name of frequency column.
#' @param size_freq Size of frequency labels. Default is 8.
#' @param legend.size Size of legend. Default is 0.5.
#' @param legend.size.text Size of legend text. Default is 10.
#' @param add_sum Logical to add sum to x-axis labels. Default is TRUE.
#' @param print_result Logical to print result data frame. Default is TRUE.
#' @param round.num Decimal places for proportion. Default is 2.
#'
#' @return A ggplot object.
#'
#' @export
#' @author Dongqiang Zeng
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_data <- data.frame(
#'   Subtype = sample(c("EBV", "GS", "MSI", "CIN"), 100, replace = TRUE),
#'   Lauren = sample(c("Diffuse", "Intestinal", "Mixed"), 100, replace = TRUE)
#' )
#' 
#' # Create percent bar plot
#' p <- percent_bar_plot(
#'   input = sim_data, x = "Subtype", y = "Lauren",
#'   axis_angle = 60
#' )
#' if (!is.null(p)) print(p)
percent_bar_plot <- function(input, x, y,
                             subset.x = NULL,
                             color = NULL,
                             palette = NULL,
                             title = NULL,
                             axis_angle = 0,
                             coord_flip = FALSE,
                             add_Freq = TRUE,
                             Freq = "Proportion",
                             size_freq = 8,
                             legend.size = 0.5,
                             legend.size.text = 10,
                             add_sum = TRUE,
                             print_result = TRUE,
                             round.num = 2) {
  if (is.null(input)) return(NULL)
  # Check if required columns exist
  input <- as.data.frame(input[, colnames(input) %in% c(x, y)])
  rlang::check_installed("scales")

  if (!is.null(subset.x)) {
    input <- input[input[, x] %in% subset.x, ]
  }

  if (add_Freq) {
    input$Freq <- 1
  } else {
    colnames(input)[which(colnames(input) == Freq)] <- "Freq"
  }

  df_sum <- input %>%
    dplyr::group_by(!!rlang::sym(x), !!rlang::sym(y)) %>%
    dplyr::summarise(Freq = sum(.data$Freq), .groups = "drop_last") %>%
    dplyr::group_by(!!rlang::sym(x)) %>%
    dplyr::mutate(
      Prop = round(.data$Freq / sum(.data$Freq), round.num),
      count = round(sum(.data$Freq), 0)
    )

  if (print_result) message(paste(capture.output(df_sum), collapse = "\n"))

  # Get colors
  if (is.null(color)) {
    color <- if (is.null(palette)) {
      palettes(category = "random", show_col = TRUE, show_message = TRUE)
    } else {
      palettes(category = "box", palette = palette, show_col = TRUE, show_message = TRUE)
    }
  }

  if (add_sum) {
    df_sum <- as.data.frame(df_sum)
    df_sum[, 1] <- paste0(as.character(df_sum[, 1]), "\n(", df_sum$count, ")")
  }

  hjust <- if (axis_angle == 0) 0.5 else 1

  pp <- ggplot2::ggplot(df_sum, ggplot2::aes(
    x = !!rlang::sym(x), y = .data$Prop, fill = !!rlang::sym(y)
  )) +
    ggplot2::geom_bar(stat = "identity", position = "fill", width = 0.85) +
    ggplot2::geom_text(
      ggplot2::aes(label = scales::percent(.data$Prop, suffix = "%", accuracy = 1)),
      position = ggplot2::position_stack(0.5), size = size_freq
    ) +
    ggplot2::ggtitle(title) +
    ggplot2::scale_fill_manual(values = color) +
    ggplot2::xlab(NULL) +
    design_mytheme(
      axis_text_size = 10, axis_angle = axis_angle, hjust = hjust,
      legend.size = legend.size, legend.size.text = legend.size.text
    )

  if (coord_flip) {
    pp <- pp + ggplot2::coord_flip()
  }

  if (interactive()) print(pp)
  pp
}


#' Create Pie or Donut Charts
#'
#' @description
#' Generates a pie chart or donut chart from input data.
#'
#' @param input Input dataframe.
#' @param var Variable for the chart.
#' @param var2 Secondary variable for donut chart (type = 3).
#' @param type Chart type: 1 (pie), 2 (donut), 3 (PieDonut via webr).
#' @param show_freq Logical to show frequencies. Default is FALSE.
#' @param color Optional color palette.
#' @param palette Color palette name. Default is "jama".
#' @param title Plot title. Default is NULL.
#' @param text_size Text size. Default is 10.
#' @param title_size Title size. Default is 20.
#' @param add_sum Logical to add sum to labels. Default is FALSE.
#'
#' @return A ggplot object.
#'
#' @export
#' @author Dongqiang Zeng
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_data <- data.frame(
#'   Subtype = sample(c("EBV", "GS", "MSI", "CIN"), 100, replace = TRUE)
#' )
#' 
#' # Create pie chart
#' p1 <- pie_chart(input = sim_data, var = "Subtype", palette = "jama")
#' if (!is.null(p1)) print(p1)
#' 
#' # Create donut chart
#' p2 <- pie_chart(input = sim_data, var = "Subtype", type = 2)
#' if (!is.null(p2)) print(p2)
pie_chart <- function(input, var, var2 = NULL, type = 2,
                      show_freq = FALSE, color = NULL, palette = "jama",
                      title = NULL, text_size = 10, title_size = 20,
                      add_sum = FALSE) {
  if (is.null(input)) return(NULL)
  # Check if variable exists
  input <- input[!is.na(input[, var]), , drop = FALSE]
  input <- as.data.frame(input)
  input[, var] <- as.character(input[, var])

  input2 <- input
  input <- as.data.frame(table(input[, var]))
  colnames(input)[1] <- "var"

  input <- input %>%
    dplyr::mutate(
      percent_weight = round(.data$Freq / sum(.data$Freq) * 100, 1)
    ) %>%
    dplyr::arrange(dplyr::desc(.data$Freq)) %>%
    dplyr::mutate(lab.ypos = cumsum(.data$percent_weight) - 0.5 * .data$percent_weight)

  if (interactive()) print(input)

  if (add_sum) {
    input[, 1] <- paste0(as.character(input[, 1]), "(", input$Freq, ")")
  }

  if (is.null(color)) {
    color <- if (is.null(palette)) {
      palettes(category = "random", show_col = FALSE, show_message = TRUE)
    } else {
      palettes(category = "box", palette = palette, show_col = FALSE, show_message = TRUE)
    }
  }

  pp <- switch(as.character(type),
    "1" = .pie_chart_basic(input, color, title, title_size),
    "2" = .pie_chart_donut(input, color, title, text_size, show_freq),
    "3" = .pie_chart_piedonut(input2, var, var2),
    cli::cli_abort("type must be 1, 2, or 3")
  )

  if (type %in% c(1, 2) && interactive()) print(pp)

  pp
}

#' @keywords internal
.pie_chart_basic <- function(input, color, title, title_size) {
  input <- input[order(input$Freq, decreasing = FALSE), ]

  ggplot2::ggplot(input, ggplot2::aes(x = 2, y = .data$percent_weight, fill = .data$var)) +
    ggplot2::geom_bar(stat = "identity", color = "white") +
    ggplot2::coord_polar(theta = "y", start = 0, direction = 1) +
    ggplot2::geom_text(
      ggplot2::aes(x = 2, y = .data$lab.ypos, label = .data$percent_weight),
      color = "white", size = 10
    ) +
    ggplot2::labs(x = NULL, y = NULL, fill = NULL) +
    ggplot2::ggtitle(title) +
    ggplot2::scale_fill_manual(values = color) +
    ggplot2::theme_void() +
    ggplot2::xlim(0.5, 2.5) +
    ggplot2::theme(
      axis.line = ggplot2::element_blank(),
      axis.text = ggplot2::element_blank(),
      axis.ticks = ggplot2::element_blank(),
      plot.title = ggplot2::element_text(hjust = 0.5, color = "#666666", size = title_size)
    )
}

#' @keywords internal
.pie_chart_donut <- function(input, color, title, text_size, show_freq) {
  label_col <- if (show_freq) "Freq" else "percent_weight"
  label_fun <- function(x) if (show_freq) x else paste0(x, "%")

  ggplot2::ggplot(input, ggplot2::aes(x = 2, y = .data$percent_weight, fill = .data$var)) +
    ggplot2::geom_bar(stat = "identity", color = "white") +
    ggplot2::coord_polar(theta = "y", start = 0) +
    ggplot2::geom_text(
      ggplot2::aes(label = label_fun(!!rlang::sym(label_col))),
      color = "white", size = text_size, position = ggplot2::position_stack(vjust = 0.5)
    ) +
    ggplot2::labs(x = NULL, y = NULL, fill = NULL) +
    ggplot2::ggtitle(title) +
    ggplot2::scale_fill_manual(values = color) +
    ggplot2::theme_classic() +
    ggplot2::theme(
      axis.line = ggplot2::element_blank(),
      axis.text = ggplot2::element_blank(),
      axis.ticks = ggplot2::element_blank(),
      plot.title = ggplot2::element_text(hjust = 0.5, color = "#666666", size = 20)
    )
}

#' @keywords internal
.pie_chart_piedonut <- function(input, var, var2) {
  if (is.null(var2)) {
    cli::cli_abort("var2 must be defined for type = 3")
  }
  rlang::check_installed("webr", reason = "to create PieDonut plots")

  webr::PieDonut(input, ggplot2::aes(pies = !!rlang::sym(var), donuts = !!rlang::sym(var2)),
    explode = 1, pieLabelSize = 7, donutLabelSize = 5
  )
}

#' Default Pattern List for Name Cleaning
#'
#' @description
#' A character vector of common substrings to remove from feature names.
#' Used in [remove_names()] and other helper functions.
#'
#' @format A character vector of length 12.
#'
#' @return Character vector of patterns to remove.
#'
#' @export
#'
#' @examples
#' # View default patterns
#' patterns_to_na
patterns_to_na <- c(
  "_cibersort", "xCell", "_EPIC", "_TIMER", "_quantiseq", "_MCP",
  "HALLMARK_", "_CIBERSORT", "xcell", "_timer", "_mcp", "_epic"
)


#' Remove Patterns from Column Names or Variables
#'
#' @description
#' Modifies column names or specified variables in a data frame by replacing
#' specified patterns with empty strings or spaces.
#'
#' @param input_df Data frame. Input data to modify.
#' @param variable Character. Column to modify: "colnames" for column names,
#'   or a specific column name. Default is "colnames".
#' @param patterns_to_na Character vector. Patterns to replace with empty
#'   string. Default uses [patterns_to_na].
#' @param patterns_space Character vector or `NULL`. Patterns to replace with
#'   spaces. Default is `NULL`.
#'
#' @return Modified data frame with patterns replaced.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' df <- data.frame(
#'   "CellA_cibersort" = 1:5,
#'   "CellB_xCell" = 6:10,
#'   "CellC_TIMER" = 11:15
#' )
#' result <- remove_names(df, variable = "colnames", patterns_to_na = patterns_to_na)
#' colnames(result)
remove_names <- function(input_df,
                         variable = "colnames",
                         patterns_to_na = patterns_to_na,
                         patterns_space = NULL) {
  if (!is.data.frame(input_df)) {
    cli::cli_abort("{.arg input_df} must be a data frame.")
  }

  if (variable != "colnames" && !variable %in% colnames(input_df)) {
    cli::cli_abort("Column {.val {variable}} not found in {.arg input_df}.")
  }

  if (variable == "colnames") {
    for (pattern in patterns_to_na) {
      colnames(input_df) <- gsub(colnames(input_df), pattern = pattern, replacement = "")
    }

    if (!is.null(patterns_space)) {
      for (pattern in patterns_space) {
        colnames(input_df) <- gsub(colnames(input_df), pattern = pattern, replacement = " ")
      }
    }
  } else {
    for (pattern in patterns_to_na) {
      input_df[[variable]] <- gsub(input_df[[variable]], pattern = pattern, replacement = "")
    }

    if (!is.null(patterns_space)) {
      for (pattern in patterns_space) {
        input_df[[variable]] <- gsub(input_df[[variable]], pattern = pattern, replacement = " ")
      }
    }
  }

  input_df
}

#' Generate Kaplan-Meier Survival Plot for Signature
#'
#' @description
#' Creates Kaplan-Meier survival plots for a given signature or gene, with
#' automatic cutoff determination. Generates three types of plots: optimal
#' cutoff (best cutoff), tertile-based (3 groups), and median split (2 groups).
#'
#' @param input_pdata Data frame with survival data and signature scores.
#' @param signature Character string. Column name of the target signature.
#' @param project Character string. Project name for output. Default is `"KM"`.
#' @param time Character string. Column name for survival time. Default is `"time"`.
#' @param status Character string. Column name for survival status. Default is `"status"`.
#' @param time_type Character string. Time unit (`"month"` or `"day"`).
#'   Default is `"month"`.
#' @param break_month Numeric or `"auto"`. Time axis breaks. Default is `"auto"`.
#' @param palette Character string. Color palette if `cols` not provided.
#'   Default is `"jama"`.
#' @param save_path Character string or `NULL`. Directory for saving plots.
#'   If `NULL`, plots are not saved. Default is `NULL`.
#' @param mini_sig Character string. Label for low score group. Default is `"score"`.
#' @param show_col Logical indicating whether to show colors. Default is `TRUE`.
#' @param index Integer. Index for multiple plots. Default is `1`.
#' @param cols Character vector. Optional custom colors.
#' @param fig.type Character string. File format. Default is `"png"`.
#' @param ID Character string. Column name for sample IDs. Default is `"ID"`.
#'
#' @return A list containing:
#' \describe{
#'   \item{data}{Processed input data with group assignments}
#'   \item{plots}{Combined survival plots}
#' }
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' # Create small example data
#' set.seed(123)
#' test_data <- data.frame(
#'   time = runif(100, 0, 100),
#'   OS_status = sample(c(0, 1), 100, replace = TRUE),
#'   TMEscore_plus = rnorm(100),
#'   ID = paste0("S", 1:100)
#' )
#' sig_surv_plot(
#'   input_pdata = test_data,
#'   signature = "TMEscore_plus",
#'   time = "time",
#'   status = "OS_status"
#' )
sig_surv_plot <- function(input_pdata,
                          signature,
                          project = "KM",
                          ID = "ID",
                          time = "time",
                          status = "status",
                          time_type = "month",
                          break_month = "auto",
                          cols = NULL,
                          palette = "jama",
                          show_col = TRUE,
                          mini_sig = "score",
                          fig.type = "png",
                          save_path = NULL,
                          index = 1) {
  if (is.null(input_pdata)) return(NULL)
  rlang::check_installed("survminer")

  # TODO
  # Ignoring unknown labels:
  # colour : "Strata"


  # Input validation
  if (!signature %in% colnames(input_pdata)) {
    cli::cli_abort("Signature {.val {signature}} not found in input_pdata")
  }

  # Create output directory
  abspath <- NULL
  if (!is.null(save_path)) {
    cli::cli_alert_info("Saving plots to: {.val {save_path}}")
    if (!dir.exists(save_path)) dir.create(save_path, recursive = TRUE)
    abspath <- file.path(normalizePath(save_path, winslash = "/", mustWork = FALSE), "")
  }

  # Select relevant columns
  input_pdata <- as.data.frame(input_pdata[, colnames(input_pdata) %in% c(ID, time, status, signature)])

  # Standardize column names
  colnames(input_pdata)[colnames(input_pdata) == time] <- "time"
  colnames(input_pdata)[colnames(input_pdata) == status] <- "status"
  colnames(input_pdata)[colnames(input_pdata) == ID] <- "ID"

  # Convert and filter data
  input_pdata$time <- as.numeric(input_pdata$time)
  input_pdata$status <- as.numeric(as.character(input_pdata$status))

  input_pdata <- input_pdata %>%
    dplyr::filter(!is.na(.data$time)) %>%
    dplyr::filter(!is.na(.data$status)) %>%
    dplyr::filter(.data$time > 0)

  input_pdata$ID <- as.character(input_pdata$ID)

  # Transform follow-up time to months if needed
  if (time_type == "day") {
    input_pdata$time <- input_pdata$time / 30
  }

  # Create tertile-based groups
  q1 <- stats::quantile(input_pdata[[signature]], probs = 1 / 3, na.rm = TRUE)
  q2 <- stats::quantile(input_pdata[[signature]], probs = 2 / 3, na.rm = TRUE)

  input_pdata$group3 <- ifelse(
    input_pdata[[signature]] <= q1, "Low",
    ifelse(input_pdata[[signature]] >= q2, "High", "Middle")
  )

  # Create mean-based groups
  mean_val <- mean(input_pdata[[signature]], na.rm = TRUE)
  input_pdata$group2 <- ifelse(input_pdata[[signature]] <= mean_val, "Low", "High")

  cli::cli_alert_info(
    "Survival follow-up time range: {round(min(input_pdata$time, na.rm = TRUE), 2)} to {round(max(input_pdata$time, na.rm = TRUE), 2)} months"
  )

  # Calculate best cutoff
  res.cut <- survminer::surv_cutpoint(
    input_pdata,
    time = "time", event = "status", variables = signature
  )
  res.cut <- res.cut$cutpoint[[1]]
  cli::cli_alert_info("Best cutoff for {.val {signature}}: {round(res.cut, 2)}")

  # Apply best cutoff
  input_pdata <- best_cutoff(
    pdata = input_pdata, variable = signature,
    time = "time", status = "status", print_result = FALSE
  )

  colnames(input_pdata)[colnames(input_pdata) == paste0(signature, "_binary")] <- "bestcutoff"

  cli::cli_alert_info("High {signature}: {table(input_pdata$bestcutoff)['High']}")
  cli::cli_alert_info("Low {signature}: {table(input_pdata$bestcutoff)['Low']}")

  # Save data if path provided
  if (!is.null(save_path)) {
    save(input_pdata, file = file.path(
      save_path,
      paste0(index, "-0-", project, "-", signature, "-survival-analysis-input.RData")
    ))
  }

  # Prepare data for Cox model (High = 1)
  input_pdata$bestcutoff <- ifelse(input_pdata$bestcutoff == "High", 1, 0)

  # Calculate HR and CI
  cox_fit <- survival::coxph(
    survival::Surv(time = input_pdata$time, event = input_pdata$status) ~ bestcutoff,
    data = input_pdata
  )
  pvalue <- getHRandCIfromCoxph(cox_fit)

  HR <- paste0("Hazard Ratio = ", round(pvalue[, 2], 2))
  CI <- paste0("95% CI: ", round(pvalue[, 3], 2), " - ", round(pvalue[, 4], 2))
  cut_off <- paste0("cutoff = ", round(res.cut, 3))

  # Create survival fit
  sfit <- survminer::surv_fit(
    survival::Surv(time = input_pdata$time, event = input_pdata$status) ~ bestcutoff,
    data = input_pdata
  )

  # Define break time
  break_month_val <- break_month
  if (identical(break_month_val, "auto")) {
    break_month_val <- calculate_break_month(input = input_pdata$time, block = 6)
  }
  break_month_val <- as.numeric(break_month_val)
  max_month <- break_month_val * 6

  # Get colors
  if (is.null(cols)) {
    cols <- palettes(category = "box", palette = palette, show_col = FALSE)
  }

  # Plot 1: Best cutoff
  pp1 <- survminer::ggsurvplot(
    sfit,
    data = input_pdata, censor = TRUE, ncensor.plot = FALSE, conf.int = FALSE,
    xlim = c(0, max_month), break.time.by = break_month_val,
    xlab = "Months after diagnosis", surv.median.line = "h",
    legend.labs = c(paste0("Low ", mini_sig), paste0("High ", mini_sig)),
    submain = paste0(signature, "-in-", project),
    risk.table = TRUE, tables.height = 0.20, palette = cols, pval.size = 8,
    pval = paste(
      ifelse(pvalue[, 1] < 0.0001, "P < 0.0001",
        paste0("P = ", round(pvalue[, 1], 4))
      ),
      HR, CI, cut_off,
      sep = "\n"
    ),
    size = 0.4
  )

  res1 <- survminer::arrange_ggsurvplots(list(pp1), print = FALSE, ncol = 1, nrow = 1)

  if (!is.null(save_path)) {
    ggplot2::ggsave(
      plot = res1,
      filename = paste0(
        index, "-1-KMplot-best-cutoff-", signature, "-",
        project, ".", fig.type
      ),
      width = 6, height = 6.5, path = save_path
    )
  }

  # Plot 2: Three groups (tertiles)
  input_pdata$bestcutoff <- ifelse(input_pdata$bestcutoff == 1, "High", "Low")

  input_pdata <- input_pdata[!is.na(input_pdata$group3), ]
  sfit <- survminer::surv_fit(
    survival::Surv(input_pdata$time, input_pdata$status) ~ group3,
    data = input_pdata
  )

  names(sfit$strata) <- gsub("group3=", "", names(sfit$strata))

  pp2 <- survminer::ggsurvplot(
    sfit,
    data = input_pdata, censor = TRUE, ncensor.plot = FALSE,
    conf.int = FALSE, xlim = c(0, max_month), break.time.by = break_month_val,
    xlab = "Months after diagnosis",
    submain = paste0(signature, "-in-", project),
    surv.median.line = "h", risk.table = TRUE, tables.height = 0.25,
    palette = cols, pval.size = 8, size = 0.4
  )

  # Log-rank test
  fitd <- survival::survdiff(
    survival::Surv(time, status) ~ group3,
    data = input_pdata,
    na.action = stats::na.exclude
  )
  p.val <- 1 - stats::pchisq(fitd$chisq, length(fitd$n) - 1)

  p.lab <- paste0(
    "Overall P",
    ifelse(p.val < 0.001, " < 0.001", paste0(" = ", round(p.val, 3)))
  )

  pp2$plot <- pp2$plot + ggplot2::annotate(
    "text",
    x = 0, y = 0.55, hjust = 0, fontface = 3, label = p.lab
  )

  # Pairwise comparisons
  ps <- survminer::pairwise_survdiff(
    survival::Surv(time, status) ~ group3,
    data = input_pdata,
    p.adjust.method = "none"
  )

  addTab <- as.data.frame(as.matrix(ifelse(round(ps$p.value, 3) < 0.001,
    "<0.001", round(ps$p.value, 3)
  )))
  addTab[is.na(addTab)] <- "-"

  df <- tibble::tibble(x = 0, y = 0, tb = list(addTab))

  rlang::check_installed("gridExtra")
  tb_grob <- gridExtra::tableGrob(df$tb,
    rows = TRUE,
    theme = gridExtra::ttheme_minimal(base_size = 6)
  )

  pp2$plot <- pp2$plot +
    ggplot2::geom_text(ggplot2::aes(x = .data$x, y = .data$y, label = ""),
      data = df, size = 0
    ) +
    ggplot2::annotation_custom(tb_grob,
      xmin = df$x, xmax = df$x,
      ymin = df$y, ymax = df$y
    )

  res2 <- survminer::arrange_ggsurvplots(list(pp2), print = FALSE, ncol = 1, nrow = 1)

  if (!is.null(save_path)) {
    ggplot2::ggsave(
      plot = res2,
      filename = paste0(
        index, "-2-KMplot-3group-", signature, "-",
        project, ".", fig.type
      ),
      width = 6, height = 6.5, path = save_path
    )
  }

  # Plot 3: Two groups (median)
  input_pdata <- input_pdata[!is.na(input_pdata$group2), ]
  input_pdata$group2 <- ifelse(input_pdata$group2 == "High", 1, 0)

  cox_fit <- survival::coxph(
    survival::Surv(input_pdata$time, input_pdata$status) ~ group2,
    data = input_pdata
  )
  pvalue <- getHRandCIfromCoxph(cox_fit)
  HR <- paste0("Hazard Ratio = ", round(pvalue[, 2], 2))
  CI <- paste0("95% CI: ", round(pvalue[, 3], 2), " - ", round(pvalue[, 4], 2))

  sfit <- survminer::surv_fit(
    survival::Surv(input_pdata$time, input_pdata$status) ~ group2,
    data = input_pdata
  )

  pp3 <- survminer::ggsurvplot(
    sfit,
    data = input_pdata, censor = TRUE, ncensor.plot = FALSE,
    conf.int = FALSE, xlim = c(0, max_month), break.time.by = break_month_val,
    xlab = "Months after diagnosis",
    legend.labs = c(paste0("Low ", mini_sig), paste0("High ", mini_sig)),
    submain = paste0(signature, "-in-", project),
    surv.median.line = "h", risk.table = TRUE, tables.height = 0.20,
    palette = cols, pval.size = 8,
    pval = paste(
      ifelse(pvalue[, 1] < 0.0001, "P < 0.0001",
        paste0("P = ", round(pvalue[, 1], 4))
      ),
      HR, CI,
      sep = "\n"
    ),
    size = 0.4
  )

  res3 <- survminer::arrange_ggsurvplots(list(pp3), print = FALSE, ncol = 1, nrow = 1)

  if (!is.null(save_path)) {
    ggplot2::ggsave(
      plot = res3,
      filename = paste0(
        index, "-3-KMplot-2group-", signature, "-",
        project, ".", fig.type
      ),
      width = 6, height = 6.5, path = save_path
    )
  }

  # Combine all plots
  input_pdata$group2 <- ifelse(input_pdata$group2 == 1, "High", "Low")

  plots <- list(pp1, pp3, pp2)
  combined_plots <- survminer::arrange_ggsurvplots(plots, print = FALSE, ncol = 3, nrow = 1)

  list(data = input_pdata, plots = combined_plots)
}

#' Calculate break points for survival plots
#' @keywords internal
#' @noRd
calculate_break_month <- function(input, block = 6) {
  max_follow_up <- max(input, na.rm = TRUE)
  max_follow_up <- ceiling(max_follow_up)
  break_month_val <- max_follow_up / block
  break_month_val <- ceiling(break_month_val)
  break_month_val
}
# TODO: a same exported function in surv_group.R

#' Format Input Signatures from MSigDB
#'
#' @description
#' Reads a GMT file from MSigDB and converts it into a named list of gene sets
#' suitable for IOBR functions.
#'
#' @param gmt Character string. Path to a GMT file.
#' @param ont Character string. Name of the signature/set column in the parsed
#'   GMT table. Default is `"term"`.
#' @param gene Character string. Name of the gene column in the parsed GMT
#'   table. Default is `"gene"`.
#'
#' @return Named list of character vectors, where each element contains the
#'   genes belonging to one signature.
#'
#' @export
#'
#' @examples
#' tf <- tempfile(fileext = ".gmt")
#' writeLines(
#'   c(
#'     "HALLMARK_TNFA_SIGNALING_VIA_NFKB\tNA\tTNF\tNFKB1\tNFKB2",
#'     "HALLMARK_P53_PATHWAY\tNA\tTP53\tMDM2\tCDKN1A"
#'   ),
#'   con = tf
#' )
#'
#' sig_list <- format_msigdb(tf, ont = "term", gene = "gene")
#' names(sig_list)
#' sig_list[[1]]
format_msigdb <- function(gmt, ont = "term", gene = "gene") {
  if (!file.exists(gmt)) {
    cli::cli_abort("File {.file {gmt}} does not exist")
  }

  rlang::check_installed("clusterProfiler")
  sig <- clusterProfiler::read.gmt(gmt)
  sig <- as.data.frame(sig)

  colnames(sig)[colnames(sig) == ont] <- "ont"
  colnames(sig)[colnames(sig) == gene] <- "gene"

  sig_list <- split(sig[, c("ont", "gene")], sig$ont)
  sig_list <- lapply(sig_list, function(x) x$gene)

  sig_list
}

#' Forest Plot for Survival Analysis Results
#'
#' @description
#' Generates a forest plot to visualize hazard ratios, confidence intervals,
#' and p-values for gene signatures or features from survival analysis.
#'
#' @param data Data frame with survival analysis results including p-values,
#'   hazard ratios, and confidence intervals.
#' @param signature Character string. Column name for signatures or feature names.
#' @param pvalue Character string. Column name for p-values. Default is `"P"`.
#' @param HR Character string. Column name for hazard ratios. Default is `"HR"`.
#' @param CI_low_0.95 Character string. Column name for lower CI bound.
#'   Default is `"CI_low_0.95"`.
#' @param CI_up_0.95 Character string. Column name for upper CI bound.
#'   Default is `"CI_up_0.95"`.
#' @param n Integer. Maximum number of signatures to display. Default is `10`.
#' @param max_character Integer. Maximum characters for labels before wrapping.
#'   Default is `25`.
#' @param discrete_width Integer. Width for discretizing long labels. Default is `35`.
#' @param color_option Integer. Color option for p-value gradient (1, 2, or 3).
#'   Default is `1`.
#' @param cols Character vector. Custom colors for p-value gradient (low to high).
#'   Default is `NULL`.
#' @param text.size Numeric. Text size for y-axis labels. Default is `13`.
#'
#' @return A ggplot2 object of the forest plot.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' # Example with sample survival results
#' sample_results <- data.frame(
#'   ID = c("Sig1", "Sig2", "Sig3"),
#'   HR = c(1.5, 0.8, 2.0),
#'   P = c(0.01, 0.05, 0.001),
#'   CI_low_0.95 = c(1.1, 0.6, 1.5),
#'   CI_up_0.95 = c(2.0, 1.0, 2.8)
#' )
#' sig_forest(data = sample_results, signature = "ID")
sig_forest <- function(data,
                       signature,
                       pvalue = "P",
                       HR = "HR",
                       CI_low_0.95 = "CI_low_0.95",
                       CI_up_0.95 = "CI_up_0.95",
                       n = 10,
                       max_character = 25,
                       discrete_width = 35,
                       color_option = 1,
                       cols = NULL,
                       text.size = 13) {
  rlang::check_installed("stringr")

  data <- as.data.frame(data)

  # Validate required columns
  required_cols <- c(signature, pvalue, HR, CI_low_0.95, CI_up_0.95)
  missing_cols <- required_cols[!required_cols %in% colnames(data)]
  if (length(missing_cols) > 0) {
    cli::cli_abort("Missing required columns: {.val {missing_cols}}")
  }

  # Standardize column names
  colnames(data)[colnames(data) == signature] <- "signature"
  colnames(data)[colnames(data) == pvalue] <- "P"
  colnames(data)[colnames(data) == HR] <- "HR"
  colnames(data)[colnames(data) == CI_low_0.95] <- "CI_low_0.95"
  colnames(data)[colnames(data) == CI_up_0.95] <- "CI_up_0.95"

  # Convert to numeric and remove incomplete cases
  data[, c("P", "HR", "CI_low_0.95", "CI_up_0.95")] <- lapply(
    data[, c("P", "HR", "CI_low_0.95", "CI_up_0.95")],
    as.numeric
  )
  data <- data[stats::complete.cases(data), ]

  if (nrow(data) == 0) {
    cli::cli_abort("No valid data after removing incomplete cases")
  }

  # Select top signatures
  if (nrow(data) > n) {
    cli::cli_alert_info("Showing top {n} signatures")

    data$HR_statistic <- data$HR - 1
    good_features <- high_var_fea(
      result = data,
      target = "signature",
      name_padj = "P",
      padj_cutoff = 1,
      name_logfc = "HR_statistic",
      logfc_cutoff = 0,
      n = n / 2
    )
    data <- data[data$signature %in% good_features, ]
    data <- data[order(data$HR, decreasing = FALSE), ]
    data$signature <- as.character(data$signature)
  }

  # Process long labels
  if (max(nchar(as.character(data$signature))) > max_character) {
    long_idx <- nchar(as.character(data$signature)) > max_character
    data$signature[long_idx] <- gsub("_", " ", as.character(data$signature[long_idx]))
  }

  # Set factor order based on HR
  data$signature <- factor(data$signature, levels = data$signature)

  # Set gradient colors
  if (!is.null(cols)) {
    if (length(cols) < 2) {
      cli::cli_abort("{.arg cols} must have at least 2 colors for gradient")
    }
    gradient_colors <- grDevices::colorRampPalette(cols)(256)
  } else {
    gradient_colors <- grDevices::colorRampPalette(c(
      "#000004FF", "#51127CFF", "#B63679FF", "#FCA50AFF", "#F7F419FF"
    ))(256)
  }

  # Create forest plot
  pp <- ggplot2::ggplot(
    data = data,
    ggplot2::aes(x = .data$HR, y = .data$signature, color = .data$P)
  ) +
    ggplot2::geom_errorbarh(
      ggplot2::aes(xmax = .data$CI_up_0.95, xmin = .data$CI_low_0.95),
      color = "black",
      height = 0,
      linewidth = 1.2
    ) +
    ggplot2::geom_point(
      ggplot2::aes(x = .data$HR, y = .data$signature),
      size = 4.5,
      shape = 16
    ) +
    ggplot2::geom_vline(
      xintercept = 1,
      linetype = "dashed",
      linewidth = 0.8
    ) +
    ggplot2::scale_x_continuous(breaks = c(0.5, 1, 1.50)) +
    ggplot2::coord_trans(x = "log2") +
    ggplot2::ylab("Features") +
    ggplot2::xlab("Hazard Ratios of Features") +
    ggplot2::labs(color = "P value") +
    ggplot2::scale_color_gradientn(colors = gradient_colors, name = "P value") +
    ggplot2::theme_light() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(
        size = 15, color = "black", vjust = 0.5, hjust = 0.5
      ),
      axis.text.y = ggplot2::element_text(
        size = text.size, color = "black", vjust = 0.5, hjust = 1
      ),
      plot.title = ggplot2::element_text(
        size = 15, colour = "black", vjust = 0.5, hjust = 0.5
      )
    ) +
    ggplot2::scale_y_discrete(
      labels = function(x) stringr::str_wrap(x, width = discrete_width)
    )

  pp
}

#' Identify Outlier Samples in Gene Expression Data
#'
#' @description
#' Analyzes gene expression data to identify potential outlier samples using
#' connectivity analysis via the WGCNA package. Calculates normalized
#' adjacency and connectivity z-scores for each sample, generates connectivity
#' plots, and optionally performs hierarchical clustering.
#'
#' @param eset Numeric matrix. Gene expression data with genes as rows and
#'   samples as columns.
#' @param yinter Numeric. Z-score threshold for identifying outliers.
#'   Default is -3.
#' @param project Character or `NULL`. Output directory path for saving plots.
#'   Required if `save = TRUE`. Default is `NULL`.
#' @param plot_hculst Logical. Whether to plot hierarchical clustering.
#'   Default is `FALSE`.
#' @param show_plot Logical. Whether to display the connectivity plot.
#'   Default is `TRUE`.
#' @param index Integer or `NULL`. Index for output file naming.
#'   Default is `NULL`.
#' @param save Logical. Whether to save plots to files. Default is `FALSE`.
#'
#' @return Character vector of sample names identified as potential outliers.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_eset <- matrix(rnorm(100 * 10), 100, 10)
#' rownames(sim_eset) <- paste0("Gene", 1:100)
#' colnames(sim_eset) <- paste0("Sample", 1:10)
#' 
#' # Add one extreme outlier
#' sim_eset[, 10] <- sim_eset[, 10] + 50
#'
#' # Identify outliers
#' if (requireNamespace("WGCNA", quietly = TRUE)) {
#'   outs <- find_outlier_samples(eset = sim_eset, show_plot = FALSE)
#'   print(outs)
#' }
find_outlier_samples <- function(eset, yinter = -3, project = NULL,
                                 plot_hculst = FALSE, show_plot = TRUE,
                                 index = NULL, save = FALSE) {
  if (is.null(eset)) return(NULL)
  rlang::check_installed("WGCNA")

  if (!is.matrix(eset) && !is.data.frame(eset)) {
    cli::cli_abort("{.arg eset} must be a matrix or data frame")
  }
  if (nrow(eset) < 2 || ncol(eset) < 2) {
    cli::cli_abort("{.arg eset} must have at least 2 rows and 2 columns")
  }
  if (!is.numeric(yinter) || length(yinter) != 1) {
    cli::cli_abort("{.arg yinter} must be a single numeric value")
  }

  if (save && is.null(project)) {
    cli::cli_abort("{.arg project} must be provided when save = TRUE")
  }

  path <- if (save) creat_folder(project) else NULL
  if (is.null(index)) index <- 1

  if (save && plot_hculst) {
    tree.combat <- eset %>%
      t() %>%
      stats::dist() %>%
      stats::hclust(method = "average")

    grDevices::pdf(
      paste0(path$abspath, index, "-1-clusteringplot.pdf"),
      width = 20, height = 10
    )
    plot(tree.combat, main = paste0("1-", "Hierarchical Clustering Samples"))
    grDevices::dev.off()
  }

  normalized.adjacency <- (0.5 + 0.5 * WGCNA::bicor(eset))^2
  network.summary <- WGCNA::fundamentalNetworkConcepts(normalized.adjacency)
  connectivity <- network.summary$Connectivity
  connectivity.zscore <- (connectivity - mean(connectivity)) / sqrt(stats::var(connectivity))

  connectivity.plot <- data.frame(
    Sample.Name = names(connectivity.zscore),
    Z.score = connectivity.zscore,
    Sample.Num = seq_along(connectivity.zscore)
  )

  p <- ggplot2::ggplot(
    connectivity.plot,
    ggplot2::aes(x = .data$Sample.Num, y = .data$Z.score, label = .data$Sample.Name)
  ) +
    ggplot2::geom_text(size = 4, colour = "red") +
    ggplot2::geom_hline(ggplot2::aes(yintercept = yinter)) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      panel.grid.major = ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank()
    ) +
    ggplot2::xlab("Sample Number") +
    ggplot2::ylab("Z score") +
    ggplot2::ggtitle("Sample Connectivity") +
    design_mytheme(axis_angle = 0, axis_text_size = 12, axis_title_size = 2)

  if (show_plot) print(p)

  if (save) {
    ggplot2::ggsave(
      filename = paste0(index, "-2-connectivityplot.pdf"),
      plot = p,
      width = 8, height = 8,
      path = path$folder_name
    )
  }

  names_eset_rmout <- colnames(eset)[abs(connectivity.zscore) > abs(yinter)]

  cli::cli_alert_info("When yinter = {yinter}")
  cli::cli_alert_info("Potential outliers: {.val {names_eset_rmout}}")

  names_eset_rmout
}

#' Add Risk Score to Dataset
#'
#' @description
#' Computes a risk score for each observation based on Cox proportional hazards
#' regression or binary logistic regression. The function fits the specified
#' model and returns the dataset with an added risk score column.
#'
#' @param input Data frame containing the variables for analysis.
#' @param family Character string specifying the model family: `"cox"` for Cox
#'   proportional hazards regression or `"binary"` for logistic regression.
#'   Default is `"cox"`.
#' @param target Character string specifying the target variable name. Required
#'   when `family = "binary"`.
#' @param time Character string specifying the time-to-event variable name.
#'   Required when `family = "cox"`.
#' @param status Character string specifying the event status variable name.
#'   Required when `family = "cox"`.
#' @param vars Character vector of variable names to include in the model.
#' @param new_var_name Character string specifying the name for the new risk
#'   score column. Default is `"riskscore"`.
#'
#' @return Data frame identical to `input` with an additional column containing
#'   risk scores (linear predictors for Cox models or predicted probabilities
#'   for logistic models).
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' set.seed(123)
#' input_data <- data.frame(
#'   time = rexp(100),
#'   status = rbinom(100, 1, 0.5),
#'   age = rnorm(100, 60, 10),
#'   score1 = rnorm(100),
#'   score2 = rnorm(100)
#' )
#' result <- add_riskscore(
#'   input_data,
#'   time = "time", status = "status",
#'   vars = c("age", "score1", "score2")
#' )
#' head(result$riskscore)
add_riskscore <- function(input, family = c("cox", "binary"), target = NULL,
                          time = NULL, status = NULL, vars,
                          new_var_name = "riskscore") {
  family <- rlang::arg_match(family)

  if (!is.data.frame(input)) {
    cli::cli_abort("{.arg input} must be a data frame")
  }
  if (nrow(input) == 0) {
    cli::cli_abort("{.arg input} has no rows")
  }
  if (!is.character(vars) || length(vars) == 0) {
    cli::cli_abort("{.arg vars} must be a non-empty character vector")
  }

  missing_vars <- setdiff(vars, colnames(input))
  if (length(missing_vars) > 0) {
    cli::cli_abort("Variables not found in input: {.val {missing_vars}}")
  }

  if (family == "cox") {
    if (is.null(time) || !time %in% colnames(input)) {
      cli::cli_abort("Cox model requires {.arg time} to be a valid column name")
    }
    if (is.null(status) || !status %in% colnames(input)) {
      cli::cli_abort("Cox model requires {.arg status} to be a valid column name")
    }

    formula <- stats::as.formula(
      paste(
        "survival::Surv(", time, ",", status, ") ~",
        paste(vars, collapse = " + ")
      )
    )
    model <- survival::coxph(formula, data = input)
    input[[new_var_name]] <- stats::predict(model, newdata = input, type = "lp")
  } else {
    if (is.null(target) || !target %in% colnames(input)) {
      cli::cli_abort("Binary model requires {.arg target} to be a valid column name")
    }
    if (length(unique(input[[target]])) != 2) {
      cli::cli_abort("Target variable must have exactly 2 unique values")
    }

    input[[target]] <- as.numeric(factor(input[[target]])) - 1
    formula <- stats::as.formula(
      paste(target, "~", paste(vars, collapse = " + "))
    )
    model <- stats::glm(formula, data = input, family = stats::binomial())
    input[[new_var_name]] <- stats::predict(model, newdata = input, type = "response")
  }

  if (interactive()) {
    print(summary(model))
  }

  input
}

#' Generate Reference Signature Matrix Using Limma
#'
#' @description
#' Performs differential expression analysis using the limma package to
#' identify significantly expressed genes across different cell types.
#' Computes median expression levels of these significant genes to create
#' a reference signature matrix.
#'
#' @param dat Matrix or data frame. Gene probes in rows and samples in columns.
#' @param pheno Character vector. Cell type class of the samples.
#' @param FDR Numeric. Genes with BH adjusted p-value < FDR are considered
#'   significant. Default is 0.05.
#'
#' @return List containing:
#'   - `reference_matrix`: Data frame of median expression values for
#'     significantly expressed genes.
#'   - `G`: Number of probes used that resulted in the lowest condition number.
#'   - `condition_number`: Minimum condition number obtained.
#'   - `whole_matrix`: Matrix of median values across all samples.
#'
#' @export
#'
#' @examples
#' dat <- matrix(rnorm(2000), nrow = 100)
#' rownames(dat) <- paste("Gene", 1:100, sep = "_")
#' colnames(dat) <- paste("Sample", 1:20, sep = "_")
#' pheno <- sample(c("Type1", "Type2", "Type3"), 20, replace = TRUE)
#' results <- generateRef_limma(dat, pheno)
#' print(results)
generateRef_limma <- function(dat, pheno, FDR = 0.05) {
  rlang::check_installed("limma")

  if (!is.matrix(dat) && !is.data.frame(dat)) {
    cli::cli_abort("{.arg dat} must be a matrix or data frame")
  }
  if (length(pheno) != ncol(dat)) {
    cli::cli_abort("Length of {.arg pheno} must match number of columns in {.arg dat}")
  }

  pheno <- factor(pheno)
  design <- stats::model.matrix(~ 0 + pheno)
  colnames(design) <- stringr::str_sub(colnames(design), 6)
  rownames(design) <- colnames(dat)

  con <- Construct_con(pheno = pheno)

  fit <- limma::lmFit(dat, design) %>%
    limma::contrasts.fit(con) %>%
    limma::eBayes()

  dif <- colnames(con) %>%
    purrr::map(function(x) limma::topTable(fit, adjust.method = "BH", coef = x, number = Inf)) %>%
    lapply(function(x) data.frame(probe = rownames(x), x)) %>%
    purrr::map(function(x) dplyr::filter(x, .data$adj.P.Val < FDR)) %>%
    purrr::map(function(x) dplyr::arrange(x, desc(.data$logFC)))

  median_value <- t(dat) %>%
    as.data.frame() %>%
    split(pheno) %>%
    purrr::map(function(x) apply(as.matrix(x), 2, stats::median, na.rm = TRUE))
  median_value <- do.call(cbind, median_value)
  rownames(median_value) <- rownames(dat)

  con_nums <- numeric(151)
  for (i in 50:200) {
    probes <- dif %>%
      purrr::map(function(x) Top_probe(dat = x, i = i)) %>%
      unlist() %>%
      unique()
    tmpdat <- median_value[probes, , drop = FALSE]
    con_nums[i - 49] <- kappa(tmpdat)
  }

  i <- 49 + which.min(con_nums)
  probes <- dif %>%
    purrr::map(function(x) Top_probe(dat = x, i = i)) %>%
    unlist() %>%
    unique()
  reference <- median_value[probes, , drop = FALSE]
  reference <- data.frame(NAME = rownames(reference), reference)
  G <- i
  condition_number <- min(con_nums)

  list(
    reference_matrix = reference,
    G = G,
    condition_number = condition_number,
    whole_matrix = median_value
  )
}


#' Construct Contrast Matrix
#'
#' @description
#' Creates a contrast matrix for differential analysis, where each phenotype
#' level is contrasted against all other levels combined.
#'
#' @param pheno Factor with different levels representing groups to contrast.
#'
#' @return Square matrix with dimensions equal to the number of levels in
#'   `pheno`. Each row represents a contrast where the corresponding level
#'   is compared against the average of others.
#'
#' @export
#'
#' @examples
#' pheno <- factor(c("A", "B", "C", "D"))
#' contrast_matrix <- Construct_con(pheno)
#' print(contrast_matrix)
Construct_con <- function(pheno) {
  n <- length(levels(pheno))
  con <- matrix(-1, n, n)
  diag(con) <- 1
  rownames(con) <- levels(pheno)
  colnames(con) <- paste0(rownames(con), " - others")
  con
}

#' Construct Mutation Matrices from MAF Data
#'
#' Builds mutation presence/absence matrices from MAF input (file path or MAF
#' object). Supports multiple categories: all mutations, SNPs, indels, and
#' frameshift mutations. When category = "multi", returns a list of matrices
#' for each category. Compatible with TCGA-formatted data.
#'
#' @param maf Character or MAF object. Path to MAF file or an already loaded MAF object.
#' @param mut_data Data frame or NULL. Preloaded MAF-like data (used if 'maf' is NULL).
#' @param isTCGA Logical. Whether the MAF follows TCGA conventions. Default is TRUE.
#' @param category Character. Mutation category: "all", "snp", "indel", "frameshift", or "multi". Default is "multi".
#' @param Tumor_Sample_Barcode Character. Column name for tumor sample IDs. Default is "Tumor_Sample_Barcode".
#' @param Hugo_Symbol Character. Column name for gene symbols. Default is "Hugo_Symbol".
#' @param Variant_Classification Character. Column name for variant classification (e.g., Frame_Shift_Del). Default is "Variant_Classification".
#' @param Variant_Type Character. Column name for variant type (e.g., SNP, INS, DEL). Default is "Variant_Type".
#'
#' @return List of mutation matrices (if category = "multi") or a single matrix for the specified category.
#' @note
#' Some users may encounter errors from upstream data import (e.g.
#' "Can't combine ..$Tumor_Seq_Allele2" when using TCGAbiolinks or
#' TCGAmutations).
#' This is due to inconsistent column types in the source MAF tables,
#' not an issue of this function.
#' Please ensure your MAF or merged data frame uses consistent column types
#' (e.g. convert allele columns to character before input).
#'
#' @export
#' @author Dongqian Zeng
#' @author Shixiang Huang
#'
#' @examples
#' \dontrun{
#' # See maftools or TCGAbiolinks documentation for obtaining MAF input
#' # Example: Download MAF file from TCGA portal
#' mut_list <- make_mut_matrix(maf = "path_to_maf_file.maf", isTCGA = TRUE, category = "multi")
#' }
make_mut_matrix <- function(maf = NULL, mut_data = NULL, isTCGA = TRUE,
                            category = c("multi", "all", "snp", "indel", "frameshift"),
                            Tumor_Sample_Barcode = "Tumor_Sample_Barcode",
                            Hugo_Symbol = "Hugo_Symbol",
                            Variant_Classification = "Variant_Classification",
                            Variant_Type = "Variant_Type") {
  category <- rlang::arg_match(category)

  if (is.null(maf) && is.null(mut_data)) {
    cli::cli_abort("Either {.arg maf} or {.arg mut_data} must be provided.")
  }

  if (!is.null(maf)) {
    mut_maf <- maftools::read.maf(maf = maf, useAll = TRUE, isTCGA = isTCGA)

    cli::cli_alert_info("Variant Classification summary:")
    print(summary(mut_maf@data$Variant_Classification))
    cli::cli_alert_info("Variant Type summary:")
    print(summary(mut_maf@data$Variant_Type))

    mut <- mut_maf@data
  } else {
    mut <- as.data.frame(mut_data)

    required_cols <- c(Tumor_Sample_Barcode, Hugo_Symbol, Variant_Classification)
    missing_cols <- setdiff(required_cols, colnames(mut))
    if (length(missing_cols) > 0) {
      cli::cli_abort("Missing required column(s): {.val {missing_cols}}")
    }

    colnames(mut)[colnames(mut) == Tumor_Sample_Barcode] <- "Tumor_Sample_Barcode"
    colnames(mut)[colnames(mut) == Hugo_Symbol] <- "Hugo_Symbol"
    colnames(mut)[colnames(mut) == Variant_Classification] <- "Variant_Classification"

    if (!Variant_Type %in% colnames(mut)) {
      if ("filter" %in% colnames(mut)) mut <- mut[mut$filter == "PASS", ]
      mut$Variant_Type <- mut$Variant_Classification
      mut <- mut[!grepl("synonymous", mut$Variant_Type, ignore.case = TRUE), ]

      mut$Variant_Type <- dplyr::case_when(
        stringr::str_detect(tolower(mut$Variant_Type), "missense") ~ "SNP",
        stringr::str_detect(tolower(mut$Variant_Type), "frameshift") ~ "Frame_Shift",
        stringr::str_detect(tolower(mut$Variant_Type), "insert") ~ "INS",
        stringr::str_detect(tolower(mut$Variant_Type), "delet") ~ "DEL",
        TRUE ~ mut$Variant_Type
      )

      mut <- mut[mut$Variant_Type %in% c("SNP", "INS", "DEL", "Frame_Shift"), ]
    } else {
      colnames(mut)[colnames(mut) == Variant_Type] <- "Variant_Type"
    }
  }

  mut <- mut[, c("Tumor_Sample_Barcode", "Hugo_Symbol", "Variant_Classification", "Variant_Type")]

  mut_list <- .build_mut_matrices(mut, category)

  return(mut_list)
}

#' Build Mutation Matrices from MAF Data
#'
#' @param mut Data frame with mutation data.
#' @param category Character. Mutation category.
#'
#' @return List of mutation matrices.
#'
#' @keywords internal
.build_mut_matrices <- function(mut, category) {
  if (category == "multi") {
    mut_all <- .create_mut_matrix(mut)
    mut_snp <- .create_mut_matrix(mut[mut$Variant_Type == "SNP", ])
    mut_indel <- .create_mut_matrix(mut[mut$Variant_Type %in% c("DEL", "INS"), ])
    mut_frameshift <- .create_mut_matrix(mut[grepl("frame", tolower(mut$Variant_Classification)), ])

    mut_list <- list(
      all = t(mut_all),
      snp = t(mut_snp),
      indel = t(mut_indel),
      frameshift = t(mut_frameshift)
    )
    rlang::check_installed("ComplexHeatmap")
    mut_list <- ComplexHeatmap::unify_mat_list(mut_list)
  } else {
    mut_all <- .create_mut_matrix(mut)
    mut_list <- list(all = t(mut_all))
  }

  mut_list
}

#' Create Single Mutation Matrix
#'
#' @param mut Data frame with mutation data.
#'
#' @return Mutation matrix.
#'
#' @keywords internal
.create_mut_matrix <- function(mut) {
  if (nrow(mut) == 0) {
    cli::cli_warn("No mutations found for the specified category.")
    return(matrix(nrow = 0, ncol = 0))
  }

  rlang::check_installed("reshape2")
  mut_mat <- reshape2::dcast(mut, Hugo_Symbol ~ Tumor_Sample_Barcode,
    value.var = "Variant_Classification"
  )
  mut_mat <- tibble::column_to_rownames(mut_mat, var = "Hugo_Symbol")
  mut_mat <- as.matrix(mut_mat)
  mut_mat <- matrix(as.numeric(mut_mat),
    nrow = nrow(mut_mat), ncol = ncol(mut_mat),
    dimnames = dimnames(mut_mat)
  )
  mut_mat
}

#' Feature Selection for Predictive or Prognostic Models Using LASSO Regression
#'
#' Applies LASSO (Least Absolute Shrinkage and Selection Operator) regression
#' to construct predictive or prognostic models. Supports both binary and
#' survival response variables, utilizing cross-validation for optimal model
#' selection.
#'
#' @param x A numeric matrix. Features (e.g., gene symbols or CGI) as row names and samples as column names.
#' @param y A response variable vector. Can be binary (0/1) or survival data (e.g., survival time and event status).
#' @param type Character. Model type: "binary" for binary response or "survival" for survival analysis. Default is "binary".
#' @param nfold Integer. Number of folds for cross-validation. Default is 10.
#' @param lambda Character. Regularization parameter selection: "lambda.min" (minimum mean cross-validated error) or "lambda.1se" (one standard error from minimum). Default is "lambda.min".
#'
#' @return Character vector of selected feature names with non-zero coefficients in the optimal LASSO model.
#' @export
#' @author Dongqiang Zeng
#' @examples
#' set.seed(123)
#' gene_expression <- matrix(rnorm(100 * 20), nrow = 100, ncol = 20)
#' rownames(gene_expression) <- paste0("Gene", 1:100)
#' colnames(gene_expression) <- paste0("Sample", 1:20)
#'
#' # Binary response example
#' binary_outcome <- sample(c(0, 1), 20, replace = TRUE)
#' lasso_select(
#'   x = gene_expression,
#'   y = binary_outcome,
#'   type = "binary",
#'   nfold = 5
#' )
lasso_select <- function(x, y, type = c("binary", "survival"), nfold = 10,
                         lambda = c("lambda.min", "lambda.1se")) {
  type <- match.arg(type)
  lambda <- match.arg(lambda)
  x <- t(x)

  if (type == "binary") {
    cvfit <- glmnet::cv.glmnet(
      x = x,
      y = y,
      nfolds = nfold,
      alpha = 1,
      family = "binomial",
      type.measure = "class"
    )
  } else {
    cvfit <- glmnet::cv.glmnet(
      x = x,
      y = y,
      nfolds = nfold,
      alpha = 1,
      family = "cox"
    )
  }

  myCoefs <- stats::coef(cvfit, s = lambda)
  myCoefs <- as.matrix(myCoefs)
  lasso_fea <- rownames(myCoefs)[which(myCoefs != 0)]
  lasso_fea <- setdiff(lasso_fea, "(Intercept)")

  return(lasso_fea)
}

#' Package Startup and Initialization
#'
#' @description Internal startup hooks for the IOBR package.
#' The `.onLoad` hook performs non-intrusive checks for optional (suggested)
#' namespaces using `requireNamespace()`, avoiding hard dependencies. The
#' `.onAttach` hook prints a concise startup message that includes the
#' package version, tutorial URL, issue tracker link, and recommended citation
#' information for published use.
#'
#' @details These functions are for internal package initialization and are not
#' intended for direct invocation by end users. The availability checks executed
#' in `.onLoad` are silent and do not attach or load optional packages;
#' they only detect presence so runtime code can adapt accordingly.
#'
#' @param libname Character string giving the library directory where
#'   the package defining the package was found.
#' @param pkgname Character string giving the name of the package.
#'
#' @keywords internal
#' @noRd
#' @importFrom utils packageDescription

.onLoad <- function(libname, pkgname) {
  # # Check that suggested packages are available (silently)
  # # These are optional dependencies used by various functions
  # suggested_pkgs <- c(
  #   "ComplexHeatmap", "tidyHeatmap", "clusterProfiler", "tibble",
  #   "tidyverse", "survival", "survminer", "ggplot2",
  #   "ggpubr", "limma", "limSolve", "preprocessCore", "e1071", "GSVA"
  # )

  # invisible(suppressPackageStartupMessages(
  #   vapply(
  #     suggested_pkgs,
  #     requireNamespace,
  #     logical(1),
  #     quietly = TRUE
  #   )
  # ))
}


.onAttach <- function(libname, pkgname) {
  pkgVersion <- utils::packageDescription(pkgname, fields = "Version")

  msg <- paste0(
    "==========================================================================\n",
    "  ", pkgname, " v", pkgVersion, "  Immuno-Oncology Biological Research ", "\n",
    "  For Documentation: https://iobr.github.io/IOBR/", "\n",
    "  For Tutorial: https://iobr.github.io/book/", "\n",
    "  For Help: https://github.com/IOBR/IOBR/issues", "\n\n"
  )

  citation <- paste0(
    " If you use ", pkgname, " in published research, please cite:\n",
    " DQ Zeng, YR Fang, ..., GC Yu*, WJ Liao*, \n",
    " Enhancing immuno-oncology investigations through multidimensional decoding \n",
    " of tumor microenvironment with IOBR 2.0. Cell Rep Methods 4, 100910 (2024). \n",
    " \u0026  \n",
    " YR Fang, ..., WJ Liao*, DQ Zeng*, \n",
    " Systematic Investigation of Tumor Microenvironment and \n",
    " Antitumor Immunity With IOBR, Med Research (2025). \n",
    " https://onlinelibrary.wiley.com/doi/epdf/10.1002/mdr2.70001 \n",
    "=========================================================================="
  )

  packageStartupMessage(msg, citation)
}

#' Identify Variable Genes in Expression Data
#'
#' @description
#' Identifies variable genes from a gene expression dataset using specified
#' selection criteria. Supports multiple methods, including expression
#' thresholding and variability estimation via median absolute deviation (MAD).
#'
#' @param eset Numeric matrix. Gene expression data (genes as rows,
#'   samples as columns).
#' @param data_type Character. Type of data: `"count"` or `"normalized"`.
#'   Default is `"count"`.
#' @param methods Character vector. Methods for gene selection: `"low"`,
#'   `"mad"`. Default is `c("low", "mad")`.
#' @param prop Numeric. Proportion of samples in which a gene must be expressed.
#'   Default is 0.7.
#' @param quantile Numeric. Quantile threshold for minimum MAD (0.25, 0.5, 0.75).
#'   Default is 0.75.
#' @param min.mad Numeric. Minimum allowable MAD value. Default is 0.1.
#' @param feas Character vector or `NULL`. Additional features to include.
#'   Default is `NULL`.
#'
#' @return Matrix subset of `eset` containing variable genes.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_eset <- matrix(rnorm(100 * 20), 100, 20)
#' rownames(sim_eset) <- paste0("Gene", 1:100)
#' colnames(sim_eset) <- paste0("Sample", 1:20)
#' 
#' # Identify variable genes
#' eset_var <- find_variable_genes(
#'   eset = sim_eset,
#'   data_type = "normalized",
#'   methods = "mad",
#'   quantile = 0.25
#' )
#' head(eset_var)
find_variable_genes <- function(eset,
                                data_type = c("count", "normalized"),
                                methods = c("low", "mad"),
                                prop = 0.7,
                                quantile = 0.75,
                                min.mad = 0.1,
                                feas = NULL) {
  if (is.null(eset)) return(NULL)
  data_type <- match.arg(data_type)
  methods <- match.arg(methods, c("low", "mad"), several.ok = TRUE)
  quantile <- match.arg(as.character(quantile), c("0.25", "0.5", "0.75"))
  quantile <- as.numeric(quantile)

  eset <- as.matrix(eset)
  feas0 <- feature_manipulation(data = eset, is_matrix = TRUE)
  eset <- eset[feas0, , drop = FALSE]

  feas1 <- NULL
  if (data_type == "count" && "low" %in% methods) {
    cli::cli_alert_info(
      "Genes expressed in more than {prop * 100}% of samples"
    )
    keep <- rowSums(eset == 0) < ncol(eset) * prop
    message(paste(capture.output(table(keep)), collapse = "\n"))
    feas1 <- rownames(eset)[keep]
  }

  feas2 <- NULL
  if ("mad" %in% methods) {
    eset_log <- log2eset(eset)
    cli::cli_alert_info("min.mad = {min.mad}")
    m.mad <- apply(eset_log, 1, stats::mad)
    cli::cli_alert_info("Range of MAD: {paste(round(range(m.mad), 2), collapse = ' to ')}")

    index <- switch(as.character(quantile),
      "0.75" = 4,
      "0.5" = 3,
      "0.25" = 2
    )

    cli::cli_alert_info("{quantile * 100}% of variables will be filtered out...")
    mad_quantiles <- quantile(m.mad, probs = seq(0, 1, 0.25))
    threshold <- max(mad_quantiles[index], min.mad)
    feas2 <- rownames(eset_log)[which(m.mad > threshold)]
  }

  feas <- unique(c(feas1, feas2, feas))
  feas <- feas[!is.na(feas)]

  eset[rownames(eset) %in% feas, , drop = FALSE]
}

#' Batch Correlation Analysis
#'
#' @description
#' Performs correlation analysis between a target variable and multiple feature
#' variables. Computes correlation coefficients, p-values, and adjusts for
#' multiple testing using the Benjamini-Hochberg method.
#'
#' @param data Data frame containing the target and feature variables.
#' @param target Character string specifying the name of the target variable.
#' @param feature Character vector specifying the names of feature variables to
#'   correlate with the target.
#' @param method Character string specifying the correlation method. Options are
#'   `"spearman"`, `"pearson"`, or `"kendall"`. Default is `"spearman"`.
#'
#' @return Tibble containing the following columns for each feature:
#' \describe{
#'   \item{sig_names}{Feature name}
#'   \item{p.value}{Raw p-value}
#'   \item{statistic}{Correlation coefficient}
#'   \item{p.adj}{Adjusted p-value (Benjamini-Hochberg)}
#'   \item{log10pvalue}{Negative log10-transformed p-value}
#'   \item{stars}{Significance stars: **** p<0.0001, *** p<0.001,
#'     ** p<0.01, * p<0.05, + p<0.5}
#' }
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' # Create a small example dataset
#' set.seed(123)
#' data_df <- as.data.frame(matrix(runif(100 * 10), 100, 10))
#' colnames(data_df) <- paste0("Signature", 1:10)
#'
#' # Perform batch correlation
#' results <- batch_cor(
#'   data = data_df,
#'   target = "Signature1",
#'   feature = colnames(data_df)[2:10]
#' )
#' head(results)
batch_cor <- function(data,
                      target,
                      feature,
                      method = c("spearman", "pearson", "kendall")) {
  # Input validation
  if (is.null(data)) return(NULL)
  if (is.null(data) || !is.data.frame(data)) {
    cli::cli_abort("{.arg data} must be a non-null data frame.")
  }
  if (nrow(data) == 0) {
    cli::cli_abort("{.arg data} has no rows.")
  }
  if (!target %in% colnames(data)) {
    cli::cli_abort("Target {.val {target}} not found in data columns.")
  }
  if (!is.character(feature) || length(feature) == 0) {
    cli::cli_abort("{.arg feature} must be a non-empty character vector.")
  }

  method <- rlang::arg_match(method)

  # Filter valid features
  valid_features <- feature[feature %in% colnames(data)]
  invalid_features <- setdiff(feature, colnames(data))

  if (length(invalid_features) > 0) {
    cli::cli_alert_warning(paste(
      "Ignoring {length(invalid_features)} invalid feature{?s}:",
      "{.val {invalid_features}}"
    ))
  }

  if (length(valid_features) == 0) {
    cli::cli_abort("No valid features found in data.")
  }

  valid_features <- setdiff(valid_features, target)

  # Remove rows with NA in target
  data <- data[!is.na(data[[target]]), , drop = FALSE]

  if (nrow(data) < 3) {
    cli::cli_abort("Insufficient data after removing NA values (need >= 3).")
  }

  # Filter features with zero variance
  valid_features <- valid_features[
    vapply(
      data[, valid_features, drop = FALSE],
      function(x) stats::sd(x, na.rm = TRUE) > 0,
      logical(1)
    )
  ]

  if (length(valid_features) == 0) {
    cli::cli_abort("All features have zero variance.")
  }

  cli::cli_alert_info(
    "Computing {method} correlation for {length(valid_features)} feature{?s}"
  )

  # Vectorized correlation analysis
  results <- vapply(valid_features, function(feat) {
    test <- stats::cor.test(data[[feat]], data[[target]],
      method = method, use = "complete.obs"
    )
    p_val <- exact_pvalue(data[[feat]], data[[target]], method)
    c(p_value = p_val, estimate = as.numeric(test$estimate))
  }, numeric(2))

  # Build results data frame
  cc <- data.frame(
    sig_names = valid_features,
    p.value = results["p_value", ],
    statistic = results["estimate", ],
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  cc$p.adj <- stats::p.adjust(cc$p.value, method = "BH")
  cc$log10pvalue <- -log10(cc$p.value)
  cc$stars <- cut(cc$p.value,
    breaks = c(-Inf, 0.0001, 0.001, 0.01, 0.05, 0.5, Inf),
    labels = c("****", "***", "**", "*", "+", "")
  )

  cc <- cc[order(cc$p.value), , drop = FALSE]
  rownames(cc) <- NULL

  cli::cli_alert_success("Correlation analysis complete")

  tibble::as_tibble(cc)
}


#' Calculate Exact P-Value for Correlation
#'
#' @description
#' Computes the exact p-value for the correlation between two numeric variables
#' using a specified correlation method.
#'
#' @param x Numeric vector representing the first variable.
#' @param y Numeric vector representing the second variable.
#' @param method Character string specifying the correlation method:
#'   `"spearman"`, `"pearson"`, or `"kendall"`.
#'
#' @return Numeric value representing the exact p-value.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' x <- rnorm(100)
#' y <- rnorm(100)
#' p_val <- exact_pvalue(x = x, y = y, method = "spearman")
#' print(p_val)
exact_pvalue <- function(x, y, method) {
  l <- sum(!is.na(x) & !is.na(y))

  if (l < 3) {
    return(NA_real_)
  }

  r <- stats::cor(x = x, y = y, method = method, use = "complete.obs")

  # t-statistic for correlation
  t_stat <- r * sqrt((l - 2) / (1 - r^2))

  # Two-tailed p-value
  p <- 2 * stats::pt(q = abs(t_stat), df = l - 2, lower.tail = FALSE)

  p
}

#' Signature Box Plot with Statistical Comparisons
#'
#' @description
#' Creates box plots to visualize signature distributions across groups with
#' optional statistical pairwise comparisons. Supports both data frames and
#' Seurat objects for single-cell data visualization.
#'
#' @param data Data frame or Seurat object containing the signature and
#'   grouping variable.
#' @param signature Character string specifying the column name (or feature name
#'   in Seurat) for the signature values to plot on the y-axis.
#' @param variable Character string specifying the grouping variable column name
#'   for the x-axis.
#' @param palette Character string specifying the color palette name.
#'   Default is `"nrc"`.
#' @param cols Character vector of custom fill colors. If `NULL`, palette is
#'   used. Default is `NULL`.
#' @param jitter Logical indicating whether to add jittered points to the box
#'   plot. Default is `FALSE`.
#' @param point_size Numeric value specifying the size of jittered points.
#'   Default is 5.
#' @param angle_x_text Numeric value specifying the rotation angle for x-axis
#'   labels (in degrees). Default is 0.
#' @param hjust Numeric value specifying the horizontal justification of x-axis
#'   labels. Default is 0.5.
#' @param show_pairwise_p Logical indicating whether to display pairwise
#'   comparison p-values between groups. Default is `TRUE`.
#' @param show_overall_p Logical indicating whether to display the overall
#'   group difference p-value. Default is `FALSE`.
#' @param return_stat_res Logical indicating whether to return statistical test
#'   results instead of the plot. Default is `FALSE`.
#' @param size_of_pvalue Numeric value specifying the font size for p-values.
#'   Default is 6.
#' @param size_of_font Numeric value specifying the base font size.
#'   Default is 10.
#' @param assay Character string specifying the assay name (for Seurat objects).
#'   Default is `NULL`.
#' @param slot Character string specifying the slot name (for Seurat objects).
#'   Default is `"scale.data"`.
#' @param scale Logical indicating whether to scale signature values (z-score
#'   transformation). Default is `FALSE`.
#'
#' @return If `return_stat_res = FALSE`, returns a ggplot2 object. If
#'   `return_stat_res = TRUE`, returns a data frame containing statistical test
#'   results.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' # Create small example data
#' set.seed(123)
#' test_data <- data.frame(
#'   subtype = rep(c("A", "B"), each = 50),
#'   TMEscore_plus = rnorm(100)
#' )
#' sig_box(
#'   data = test_data,
#'   signature = "TMEscore_plus",
#'   variable = "subtype",
#'   jitter = TRUE,
#'   palette = "jco"
#' )
sig_box <- function(data,
                    signature,
                    variable,
                    palette = "nrc",
                    cols = NULL,
                    jitter = FALSE,
                    point_size = 5,
                    angle_x_text = 0,
                    hjust = 0.5,
                    show_pairwise_p = TRUE,
                    show_overall_p = FALSE,
                    return_stat_res = FALSE,
                    size_of_pvalue = 6,
                    size_of_font = 10,
                    assay = NULL,
                    slot = "scale.data",
                    scale = FALSE) {
  if (is.null(data)) return(NULL)
  rlang::check_installed("ggpubr")

  # Handle Seurat object
  if (inherits(data, "Seurat")) {
    cli::cli_alert_info("Extracting data from Seurat object...")
    data <- extract_sc_data(
      sce = data,
      vars = signature,
      assay = assay,
      slot = slot,
      combine_meta_data = TRUE
    )
  }

  # Input validation
  if (!is.data.frame(data)) {
    cli::cli_abort("{.arg data} must be a data frame or Seurat object")
  }
  if (!signature %in% colnames(data)) {
    cli::cli_abort("Signature {.val {signature}} not found in data")
  }
  if (!variable %in% colnames(data)) {
    cli::cli_abort("Variable {.val {variable}} not found in data")
  }

  data <- as.data.frame(data)
  data <- data[, c(variable, signature), drop = FALSE]
  colnames(data)[colnames(data) == variable] <- "variable"
  colnames(data)[colnames(data) == signature] <- "signature"

  # Scale if requested
  if (scale) {
    data$signature <- as.numeric(scale(data$signature))
  }

  # Remove NA in grouping variable or signature
  data <- data[!is.na(data$variable) & !is.na(data$signature), , drop = FALSE]

  if (nrow(data) == 0) {
    cli::cli_abort("No valid data after removing NA values")
  }

  # Get colors
  if (is.null(cols)) {
    cols <- palettes(
      category = "box",
      palette = palette,
      show_message = FALSE,
      show_col = FALSE
    )
  }

  # Create base plot
  p <- ggplot2::ggplot(data, ggplot2::aes(
    x = .data$variable,
    y = .data$signature,
    fill = .data$variable
  )) +
    ggplot2::geom_boxplot(
      notch = FALSE,
      outlier.shape = NA
    ) +
    ggplot2::scale_fill_manual(values = cols) +
    ggplot2::ylab(signature) +
    ggplot2::xlab(variable)

    # Count samples per group
  group_count <- table(data$variable)
  valid_groups <- names(group_count[group_count >= 2])

  # Generate comparisons safely
  if (length(valid_groups) >= 2) {
    comparisons <- utils::combn(valid_groups, 2, simplify = FALSE)
  } else {
    comparisons <- list()
  }

  # Adjust hjust for angled text
  if (angle_x_text %in% c(30, 45, 60)) {
    hjust <- 1
  }

  # Apply theme
  size_font <- size_of_font * 0.2
  p <- p +
    ggplot2::theme_light() +
    ggplot2::theme(
      axis.title.y = ggplot2::element_text(size = ggplot2::rel(size_font)),
      axis.title.x = ggplot2::element_text(size = ggplot2::rel(size_font)),
      axis.text = ggplot2::element_text(size = ggplot2::rel(size_font)),
      axis.text.x = ggplot2::element_text(
        face = "plain",
        angle = angle_x_text,
        hjust = hjust,
        color = "black"
      ),
      axis.line = ggplot2::element_line(color = "grey", linewidth = 0.05),
      legend.position = "none"
    )

  # Add statistical comparisons
  if (show_pairwise_p) {
    if (length(comparisons) > 0) {
      p <- p + ggpubr::stat_compare_means(
        comparisons = comparisons,
        size = size_of_pvalue
      )
    }
  }

  if (show_overall_p) {
    if (length(valid_groups) >= 2) {
      stat_data <- data[data$variable %in% valid_groups, , drop = FALSE]
      p <- p + ggpubr::stat_compare_means(
        data = stat_data,
        size = size_of_pvalue
      )
    }
  }

  # Calculate and print statistics
  res <- NULL
  if (length(valid_groups) >= 2) {
    stat_data <- data[data$variable %in% valid_groups, , drop = FALSE]

    res <- tryCatch({
      ggpubr::compare_means(
        stats::reformulate("variable", "signature"),
        data = stat_data
      )
    }, error = function(e) {
      NULL
    })

    if (interactive() && !is.null(res)) print(res)
  }
  
   # Add jitter points if requested
  if (jitter) {
    p <- p + ggplot2::geom_point(
      shape = 21,
      size = point_size,
      position = ggplot2::position_jitterdodge(dodge.width = 0.2),
      alpha = 0.5
    )
  }

  if (return_stat_res) {
    invisible(res)
  } else {
    invisible(p)
  }
}

#' Extract Top Marker Genes from Single-Cell Differential Results
#'
#' @description
#' Selects the top N marker genes per cluster from a ranked differential
#' expression result table.
#'
#' @param deg Data frame or matrix. Ranked marker statistics.
#' @param cluster Character. Column name containing cluster identifiers.
#'   Default is `"cluster"`.
#' @param gene Character. Column name containing gene identifiers.
#'   Default is `"gene"`.
#' @param n Integer. Number of top markers per cluster. Default is 100.
#' @param avg_log2FC Character. Column name for average log2 fold change.
#'   Default is `"avg_log2FC"`.
#'
#' @return List of character vectors; each element contains the top N genes
#'   for a cluster.
#'
#' @export
#'
#' @examples
#' # Simulate marker data
#' set.seed(123)
#' sim_deg <- data.frame(
#'   cluster = rep(c("A", "B"), each = 50),
#'   gene = paste0("Gene", 1:100),
#'   avg_log2FC = rnorm(100, 2, 1)
#' )
#' 
#' # Extract top 5 markers per cluster
#' markers <- get_sig_sc(sim_deg, n = 5)
#' print(markers)
get_sig_sc <- function(deg, cluster = "cluster", gene = "gene", avg_log2FC = "avg_log2FC", n = 100) {
  if (is.null(deg)) return(NULL)
  if (!is.data.frame(deg) && !is.matrix(deg)) {
    cli::cli_abort("{.arg deg} must be a data frame or matrix")
  }
  if (!cluster %in% colnames(deg)) {
    cli::cli_abort("Column {.val {cluster}} not found in {.arg deg}")
  }
  if (!gene %in% colnames(deg)) {
    cli::cli_abort("Column {.val {gene}} not found in {.arg deg}")
  }
  if (!avg_log2FC %in% colnames(deg)) {
    cli::cli_abort("Column {.val {avg_log2FC}} not found in {.arg deg}")
  }

  deg <- as.data.frame(deg)

  deg_top <- deg %>%
    dplyr::group_by(.data[[cluster]]) %>%
    dplyr::top_n(n, .data[[avg_log2FC]])

  feas <- split(deg_top, deg_top[[cluster]])
  feas <- lapply(feas, function(x) {
    genes <- as.character(x[[gene]])
    head(genes, n)
  })

  feas
}

#' Identification of TME Cluster
#'
#' @description
#' Performs TME (Tumor Microenvironment) clustering analysis using various
#' clustering methods. Supports feature selection, scaling, and automatic
#' determination of optimal cluster number.
#'
#' @param input Data frame containing the input dataset.
#' @param features Vector of features to use for clustering. Default is NULL
#'   (uses all columns or pattern-selected columns).
#' @param pattern Regular expression pattern for selecting features.
#'   Default is NULL.
#' @param id Column name for identifiers. Default is NULL (uses row names).
#' @param method Clustering method. Default is "kmeans".
#' @param min_nc Minimum number of clusters to evaluate. Default is 2.
#' @param max.nc Maximum number of clusters to evaluate. Default is 6.
#' @param scale Logical indicating whether to scale features. Default is TRUE.
#'
#' @return Data frame with cluster assignments appended.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' input_data <- data.frame(
#'   ID = paste0("Sample", 1:20),
#'   xCell_Tcells = rnorm(20),
#'   xCell_Bcells = rnorm(20),
#'   xCell_Macrophages = rnorm(20),
#'   Other_feature = rnorm(20)
#' )
#' result <- tme_cluster(
#'   input = input_data,
#'   pattern = "xCell",
#'   id = "ID",
#'   method = "kmeans"
#' )
#' table(result$cluster)
tme_cluster <- function(input, features = NULL, pattern = NULL, id = NULL,
                        scale = TRUE, method = "kmeans", min_nc = 2, max.nc = 6) {
  input <- as.data.frame(input)

  # Extract IDs
  sample_ids <- if (is.null(id)) {
    rownames(input)
  } else {
    as.character(input[[id]])
  }

  # Select features
  if (is.null(features)) {
    if (!is.null(pattern)) {
      features <- colnames(input)[stringr::str_detect(colnames(input), pattern)]
    } else {
      features <- setdiff(colnames(input), id)
    }
  } else {
    features <- intersect(features, colnames(input))
  }

  if (length(features) == 0) {
    cli::cli_abort("No valid features selected for clustering")
  }

  input <- input[, features, drop = FALSE]

  # Scale features
  if (scale) {
    input <- as.data.frame(scale(input, scale = TRUE, center = TRUE))
    features <- feature_manipulation(data = input, feature = features, print_result = FALSE)
    input <- input[, features, drop = FALSE]
  }

  # Perform clustering
  rlang::check_installed("NbClust")
  res <- NbClust::NbClust(
    data = input,
    diss = NULL,
    distance = "euclidean",
    min.nc = min_nc,
    max.nc = max.nc,
    method = method,
    index = "kl",
    alphaBeale = 0.1
  )

  nc <- res$Best.nc
  cli::cli_alert_info("Best number of TME clusters: {nc[1]}")

  clusters <- res$Best.partition
  cli::cli_alert_info("Cluster distribution:")
  print(summary(as.factor(clusters)))

  out <- data.frame(ID = sample_ids, cluster = paste0("TME", clusters))
  input$ID <- sample_ids

  merge(out, input, by = "ID")
}

#' NULL Model Coefficients for MCPcounter
#'
#' @format A `data.frame` with cell types in rows and coefficients in columns.
#' @usage data(null_models)
#' @keywords datasets
#'
#' @examples
#' data(null_models)
#' head(null_models)
"null_models"


#' Toy STAD Phenotype Data
#'
#' A data frame containing clinical and pathological annotations for the
#' TCGA stomach adenocarcinoma (STAD) cohort. Each row corresponds to one
#' tumour sample and can be matched to the columns of \code{eset_stad} via
#' the \code{ID} column. This dataset is typically used together with
#' \code{eset_stad} in examples of survival analysis, subgroup comparison
#' and immune deconvolution in the IOBR package.
#'
#' @format A data frame with one row per TCGA-STAD sample and 8 variables:
#' \describe{
#'   \item{ID}{Character. TCGA sample barcode, matching the column names
#'   of \code{eset_stad}.}
#'   \item{stage}{Factor. Pathological stage (e.g. \code{"Stage_I"},
#'   \code{"Stage_II"}, \code{"Stage_III"}, \code{"Stage_IV"}).}
#'   \item{status}{Factor. Vital status at last follow-up
#'   (\code{"Alive"} or \code{"Dead"}).}
#'   \item{Lauren}{Factor. Lauren classification of gastric cancer
#'   (\code{"Intestinal"}, \code{"Diffuse"}, \code{"Mixed"} or \code{NA}).}
#'   \item{subtype}{Factor. Molecular subtype (e.g. \code{"CIN"}, \code{"EBV"},
#'   \code{"GS"}, \code{"MSI"}).}
#'   \item{EBV}{Factor. EBV status of the tumour (\code{"Positive"} or
#'   \code{"Negative"}).}
#'   \item{time}{Numeric. Overall survival or follow-up time, typically
#'   measured in months.}
#'   \item{OS_status}{Integer/binary. Overall survival status indicator.
#'   (\code{1} = death, \code{0} = censored)}
#' }
#'
#' @usage data(pdata_stad)
#' @keywords datasets
#'
#' @examples
#' data(pdata_stad)
#' head(pdata_stad)
"pdata_stad"


#' Gene signature collection for pathway and immune analysis
#'
#' A named list of gene signatures used in the IOBR package for immune
#' deconvolution, pathway scoring, functional annotation, and tumour
#' microenvironment (TME) characterization. Each element corresponds to a
#' predefined biological signature and contains a character vector of HGNC
#' gene symbols.
#'
#' @format A named list of length 323. Each element is a character vector of
#'   gene symbols. Representative entries include:
#'   \describe{
#'     \item{CD_8_T_effector}{Markers of CD8\eqn{^{+}} effector T cells.}
#'     \item{DDR}{DNA damage response and repair genes.}
#'     \item{Immune_Checkpoint}{Immune checkpoint molecules.}
#'     \item{CellCycle_Reg}{Core regulators of cell-cycle progression.}
#'     \item{Mismatch_Repair}{Mismatch-repair pathway genes.}
#'     \item{TMEsocreA_CIR}{TME-related signature used in TMEscore analysis.}
#'     \item{...}{Additional signatures are included in the list but are not
#'       individually listed here; all follow the same structure.}
#'   }
#'
#' @usage data(signature_collection)
#' @keywords datasets
#'
#' @examples
#' data(signature_collection)
#' head(signature_collection)
"signature_collection"


#' Grouped gene signatures for IOBR analysis
#'
#' A named list that organizes gene signatures into functional or biological
#' categories. Each element of the list is a character vector containing the
#' names of gene signatures defined in \code{signature_collection}. A total of
#' 43 signature groups are included, covering tumour intrinsic pathways,
#' immune-related processes, stromal activity, TME characteristics and
#' immuno-oncology biomarkers. These groups are used in IOBR to conveniently
#' select sets of signatures for scoring and visualization.
#'
#' @format A named list of length 43. Each element is a character vector of
#'   signature names. Representative groups include:
#'   \describe{
#'     \item{tumor_signature}{Signatures related to intrinsic tumour biology
#'       such as cell cycle, DNA damage repair and histone regulation.}
#'     \item{EMT}{Epithelial–mesenchymal transition (EMT)–associated
#'     signatures.}
#'     \item{io_biomarkers}{Immuno-oncology biomarker–related signatures.}
#'     \item{immu_microenvironment}{Immune microenvironment–related signatures.}
#'     \item{immu_suppression}{Immune suppression–related signatures.}
#'     \item{immu_exclusion}{Signatures associated with immune exclusion and
#'       stromal barriers.}
#'     \item{TCR_BCR}{T-cell and B-cell receptor pathway signatures.}
#'     \item{tme_signatures1}{Tumour microenvironment signature panel (set 1).}
#'     \item{tme_signatures2}{Tumour microenvironment signature panel (set 2).}
#'     \item{...}{Additional groups are included (43 total), but not listed
#'       individually here; all groups follow the same structure.}
#'   }
#'
#' @usage data(sig_group)
#' @keywords datasets
#'
#' @examples
#' data(sig_group)
#' head(sig_group)
"sig_group"

#' TCGA-STAD Clinical and Molecular Annotation Data
#'
#' Clinical, molecular, and signature score data for TCGA stomach adenocarcinoma
#' (STAD) samples. Includes patient demographics, tumor characteristics,
#' molecular subtypes, and pre-computed signature scores.
#'
#' @format A data frame with samples as rows and variables as columns:
#' \itemize{
#'   \item ID – TCGA sample barcode
#'   \item stage – tumor stage (Stage_I, Stage_II, etc.)
#'   \item status – vital status (Alive/Dead)
#'   \item Lauren – histological classification
#'   \item subtype – molecular subtype (EBV, MSI, GS, CN, ...)
#'   \item EBV – EBV infection status
#'   \item TMEscore_plus – continuous tumor-micro-environment score
#'   \item TMEscore_plus_binary – High/Low TME classification
#'   \item time – follow-up time (months)
#'   \item OS_status – 0 = alive, 1 = dead
#'   \item ARID1A, PIK3CA – driver-gene mutation status
#'   \item MALAT1 – lncRNA expression
#'   \item remaining columns – gene-expression values and additional
#'   clinical/molecular annotations
#' }
#'
#'
#' @usage data(tcga_stad_pdata)
#'
#' @source The Cancer Genome Atlas Stomach Adenocarcinoma (TCGA-STAD)
#'
#' @references
#' Cancer Genome Atlas Research Network. Comprehensive molecular
#' characterization of gastric adenocarcinoma. Nature 513, 202-209 (2014).
#' doi:10.1038/nature13480
#'
#' @keywords datasets
#'
#' @examples
#' data(tcga_stad_pdata)
#' head(tcga_stad_pdata)
"tcga_stad_pdata"


#' IMvigor210 Bladder Cancer Immunotherapy Cohort Data
#'
#' Clinical and biomarker data from the IMvigor210 clinical trial cohort.
#' Includes treatment response, survival outcomes, and immune biomarker
#' measurements for bladder cancer patients treated with atezolizumab.
#'
#' @format A data frame with patients as rows and variables as columns:
#' \describe{
#'   \item{ID}{Patient sample identifier}
#'   \item{BOR}{Best overall response (CR, PR, SD, PD, NA)}
#'   \item{BOR_binary}{Binary response classification (R=responder,
#'   NR=non-responder)}
#'   \item{OS_days}{Overall survival time in days}
#'   \item{OS_status}{Overall survival status (0=alive, 1=dead)}
#'   \item{Mutation_Load}{Tumor mutation burden}
#'   \item{Neo_antigen_Load}{Neoantigen load}
#'   \item{CD_8_T_effector}{CD8+ T effector signature score}
#'   \item{Immune_Checkpoint}{Immune checkpoint signature score}
#'   \item{Pan_F_TBRs}{Pan-fibroblast TGF-\eqn{\beta} response signature}
#'   \item{Mismatch_Repair}{Mismatch repair status or signature}
#'   \item{TumorPurity}{Estimated tumor purity}
#' }
#'
#' @usage data(imvigor210_pdata)
#'
#' @source IMvigor210 clinical trial (NCT02108652)
#'
#' @references
#' Mariathasan S et al. TGF\eqn{\beta} attenuates tumour response to PD-L1 blockade
#' by contributing to exclusion of T cells. Nature 554, 544-548 (2018).
#' doi:10.1038/nature25501
#'
#' @keywords datasets
#'
#' @examples
#' data(imvigor210_pdata)
#' head(imvigor210_pdata)
"imvigor210_pdata"


#' Example Dataset for Subgroup Survival Analysis
#'
#' An example dataset demonstrating the data structure required for subgroup
#' survival analysis using the \code{\link{subgroup_survival}} function.
#' Contains simulated clinical and biomarker data with survival outcomes.
#'
#' @format A data frame with clinical variables and biomarker scores:
#' \describe{
#'   \item{Patient_ID}{Unique patient identifier}
#'   \item{ProjectID}{Study or dataset identifier (e.g., "Dataset1")}
#'   \item{AJCC_stage}{AJCC pathological stage (2, 3, 4)}
#'   \item{status}{Event status (0=censored, 1=event)}
#'   \item{time}{Follow-up time in months}
#'   \item{score}{Continuous biomarker score (numeric values)}
#'   \item{score_binary}{Binary biomarker classification ("High", "Low")}
#' }
#'
#' @usage data(subgroup_data)
#'
#' @keywords datasets
#'
#' @examples
#' data(subgroup_data)
#' head(subgroup_data)
#' @docType data
"subgroup_data"

#' Example Clinical Data for TCGA-STAD Gastric Cancer Analysis
#'
#' A small example dataset demonstrating the clinical data structure required
#' for TCGA Stomach Adenocarcinoma (STAD) analysis using IOBR package functions.
#' Contains simulated clinical variables, molecular subtypes, and survival data.
#'
#' @format A data frame with patient samples as rows and clinical variables as columns:
#' \describe{
#'   \item{ID}{Unique patient identifier (TCGA barcode format)}
#'   \item{stage}{AJCC pathological stage (Stage_II, Stage_III, Stage_IV)}
#'   \item{status}{Patient vital status (Alive, Dead, NA)}
#'   \item{Lauren}{Lauren histological classification (Intestinal, Diffuse,
#'   Mixed)}
#'   \item{subtype}{Molecular subtype classification (EBV, GS)}
#'   \item{EBV}{Epstein-Barr virus status (Positive, Negative)}
#'   \item{time}{Overall survival time in months}
#'   \item{OS_status}{Overall survival status (0=alive, 1=dead)}
#' }
#'
#' @usage data(stad_group)
#'
#' @keywords datasets
#'
#' @examples
#' data(stad_group)
#' head(stad_group)
#' @docType data
"stad_group"

#' Calculate Signature Score Using PCA, Mean, or Z-score Methods
#'
#' @description
#' Computes signature scores from gene expression data using Principal
#' Component Analysis (PCA), mean-based, or z-score approaches.
#'
#' @param eset Normalized expression matrix with genes (signature) as rows and
#'   samples as columns.
#' @param methods Scoring method: `"PCA"` (default) for principal component 1,
#'   `"mean"` for mean expression, or `"zscore"` for z-score normalized mean.
#'
#' @return Numeric vector of length `ncol(eset)`; a score summarizing the rows
#'   of `eset`.
#'
#' @author Dorothee Nickles, Dongqiang Zeng
#' @export
#'
#' @examples
#' # Create small example expression matrix
#' set.seed(123)
#' test_eset <- matrix(rnorm(1000), nrow = 10, ncol = 100)
#' rownames(test_eset) <- paste0("Gene", 1:10)
#' colnames(test_eset) <- paste0("Sample", 1:100)
#'
#' # Calculate scores
#' score_pca <- sigScore(eset = test_eset, methods = "PCA")
#' score_mean <- sigScore(eset = test_eset, methods = "mean")
#' score_zscore <- sigScore(eset = test_eset, methods = "zscore")
sigScore <- function(eset, methods = c("PCA", "mean", "zscore")) {
  if (is.null(eset)) return(NULL)
  methods <- rlang::arg_match(methods)

  eset <- as.matrix(eset)

  if (nrow(eset) == 0 || ncol(eset) == 0) {
    cli::cli_abort(c(
      "Expression matrix is empty.",
      "i" = "Check that input genes exist in the expression matrix.",
      "*" = "Current dimensions: {nrow(eset)} rows x {ncol(eset)} columns"
    ))
  }

  if (nrow(eset) < 2) {
    cli::cli_abort(c(
      "At least 2 genes are required for PCA method.",
      "i" = "Current number of genes: {nrow(eset)}"
    ))
  }

  col_vars <- apply(eset, 2, stats::var, na.rm = TRUE)
  if (all(col_vars == 0, na.rm = TRUE)) {
    cli::cli_abort("All samples have zero variance.")
  }

  if (methods == "PCA") {
    pc <- stats::prcomp(t(eset), scale. = TRUE)
    sigs <- pc$x[, 1] *
      sign(stats::cor(pc$x[, 1], colMeans(eset, na.rm = TRUE)))
  } else if (methods == "zscore") {
    eset_z <- t(scale(t(eset)))
    eset_z[is.na(eset_z)] <- 0
    sigs <- colMeans(eset_z, na.rm = TRUE)
  } else {
    sigs <- colMeans(eset, na.rm = TRUE)
  }

  sigs
}

#' Log2 Transformation of Gene Expression Matrix
#'
#' @description
#' Determines whether a gene expression matrix requires log2 transformation
#' based on the distribution of values, and applies it if necessary. This is
#' useful for automatically detecting raw counts or linear-scale data that
#' should be log-transformed for downstream analysis.
#'
#' @param eset Numeric matrix. Gene expression data with genes as rows and
#'   samples in columns.
#'
#' @return Numeric matrix. Log2-transformed gene expression data (if
#'   transformation was needed), or the original data otherwise.
#'
#' @export
#'
#' @examples
#' set.seed(123)
#' eset <- matrix(rnorm(1000, mean = 10, sd = 2), nrow = 100, ncol = 10)
#' rownames(eset) <- paste0("Gene", 1:100)
#' colnames(eset) <- paste0("Sample", 1:10)
#' eset_transformed <- log2eset(eset)
log2eset <- function(eset) {
  # Input validation
  if (is.null(eset)) {
    cli::cli_abort("{.arg eset} cannot be NULL.")
  }

  if (!is.matrix(eset) && !is.data.frame(eset)) {
    cli::cli_abort(c(
      "Invalid input type.",
      "i" = "Expected a matrix or data frame, got {.cls {class(eset)}}."
    ))
  }

  # Ensure numeric matrix
  if (!is.matrix(eset)) {
    eset <- as.matrix(eset)
  }

  if (!is.numeric(eset)) {
    cli::cli_abort("Expression matrix must contain numeric values.")
  }

  # Calculate quantiles for log-judge heuristic
  qx <- stats::quantile(
    eset,
    probs = c(0, 0.25, 0.5, 0.75, 0.99, 1.0),
    na.rm = TRUE
  )

  # Log-judge conditions:
  # 1. 99th percentile > 100 (large values typical of counts)
  # 2. Range > 50 AND Q1 > 0 (wide distribution, positive values)
  # 3. 0 < Q1 < 1 AND 1 < Q4 < 2 (likely already log-transformed)
  needs_log <- (qx[5] > 100) ||
    (qx[6] - qx[1] > 50 && qx[2] > 0) ||
    (qx[2] > 0 && qx[2] < 1 && qx[4] > 1 && qx[4] < 2)

  if (needs_log) {
    # Set negative values to 0 before log transformation
    eset[eset < 0] <- 0
    eset <- log2(eset + 1)
    cli::cli_alert_success("Applied log2 transformation")
  } else {
    cli::cli_alert_info(
      paste(
        "Log2 transformation not necessary",
        "(data appears to already be log-scaled)"
      )
    )
  }

  eset
}

#' Generate Kaplan-Meier Survival Plots for Categorical Groups
#'
#' @description
#' Creates Kaplan-Meier survival plots for data grouped by a categorical variable.
#' Handles both binary and multi-level categorical groups with customizable plot
#' aesthetics.
#'
#' @param input_pdata Data frame containing survival data and grouping variables.
#' @param target_group Name of column containing the grouping variable.
#' @param ID Name of column with unique identifiers. Default is "ID".
#' @param levels Names for levels of target_group (for binary groups).
#'   Default is c("High", "Low").
#' @param reference_group Reference level for binary comparison. Default is NULL.
#' @param project Optional title for plot. Default is NULL.
#' @param time Name of column with follow-up times. Default is "time".
#' @param status Name of column with event indicators. Default is "status".
#' @param time_type Units: "month" or "day". Default is "month".
#' @param break_month X-axis break interval. If "auto", calculated automatically.
#'   Default is "auto".
#' @param cols Color vector for plot lines. Default is NULL.
#' @param palette Color palette name. Default is "jama".
#' @param mini_sig Prefix label for variables. Default is "score".
#' @param save_path Directory for saving plot. Default is NULL.
#' @param fig.type File format: "pdf" or "png". Default is "pdf".
#' @param index Identifier for file naming. Default is 1.
#' @param width Plot width. Default is 6.
#' @param height Plot height. Default is 6.5.
#' @param font.size.table Font size for risk table. Default is 3.
#'
#' @return Kaplan-Meier plot object.
#' @importFrom utils capture.output
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_pdata <- data.frame(
#'   ID = paste0("Sample", 1:100),
#'   Lauren = sample(c("Intestinal", "Diffuse"), 100, replace = TRUE),
#'   time = runif(100, 1, 60),
#'   OS_status = sample(0:1, 100, replace = TRUE)
#' )
#' # Run survival analysis (survival and survminer are imported packages)
#' result <- surv_group(
#'   input_pdata = sim_pdata,
#'   target_group = "Lauren",
#'   time = "time",
#'   status = "OS_status",
#'   save_path = NULL
#' )
#' if (!is.null(result)) print(result)
surv_group <- function(input_pdata,
                       target_group,
                       ID = "ID",
                       levels = c("High", "Low"),
                       reference_group = NULL,
                       project = NULL,
                       time = "time",
                       status = "status",
                       time_type = "month",
                       break_month = "auto",
                       cols = NULL,
                       palette = "jama",
                       mini_sig = "score",
                       save_path = NULL,
                       fig.type = "pdf",
                       index = 1,
                       width = 6,
                       height = 6.5,
                       font.size.table = 3) {
  if (!target_group %in% colnames(input_pdata)) {
    cli::cli_abort("{.arg target_group} must be a column in {.arg input_pdata}")
  }

  # Setup save path
  if (!is.null(save_path)) {
    if (!dir.exists(save_path)) dir.create(save_path, recursive = TRUE)
    abspath <- file.path(normalizePath(save_path, winslash = "/", mustWork = FALSE), "")
  } else {
    abspath <- NULL
  }

  # Prepare data
  input_pd <- input_pdata[, c(ID, target_group, time, status)]
  colnames(input_pd)[which(colnames(input_pd) == time)] <- "time"
  colnames(input_pd)[which(colnames(input_pd) == status)] <- "status"
  colnames(input_pd)[which(colnames(input_pd) == ID)] <- "ID"

  # Convert and filter
  input_pd$time <- as.numeric(as.character(input_pd$time))
  input_pd$status <- as.numeric(as.character(input_pd$status))

  input_pd <- input_pd %>%
    dplyr::filter(!is.na(.data$time), !is.na(.data$status)) %>%
    dplyr::filter(.data$time > 0)

  # Transform time if needed
  if (time_type == "day") {
    input_pd$time <- input_pd$time / 30
  }

  cli::cli_alert_info(
    "Follow-up time ranges from {paste(round(summary(input_pd$time), 2)[c(1, 6)], collapse = ' to ')} months"
  )

  if (!is.null(save_path)) {
    save(input_pd, file = file.path(abspath, paste0("0-", project, "-", target_group, "-survival.RData")))
  }

  colnames(input_pd)[which(colnames(input_pd) == target_group)] <- "target_group"
  input_pd <- input_pd[!is.na(input_pd$target_group), ]
  message(paste(capture.output(summary(as.factor(input_pd$target_group))), collapse = "\n"))

  # Define break time and colors
  break_month_val <- break_month
  if (identical(break_month_val, "auto")) {
    break_month_val <- calculate_break_month(input = input_pd$time, block = 6)
  }
  break_month_val <- as.numeric(break_month_val)
  max_month <- break_month_val * 6

  if (is.null(cols)) {
    cols <- palettes(category = "box", palette = palette, show_col = FALSE)
  }

  # Create plot based on number of groups
  n_groups <- length(unique(input_pd$target_group))

  if (n_groups > 2) {
    pp <- .surv_plot_multi(
      input_pd, max_month, break_month_val, cols,
      target_group, project, font.size.table
    )
  } else {
    pp <- .surv_plot_binary(
      input_pd, max_month, break_month_val, cols,
      target_group, project, reference_group, levels
    )
  }

  pp <- list(pp)
  res <- survminer::arrange_ggsurvplots(pp, print = FALSE, ncol = 1, nrow = 1)

  if (!is.null(save_path)) {
    ggplot2::ggsave(
      filename = paste0(index, "-KMplot-", target_group, "-", project, ".", fig.type),
      plot = res,
      width = width, height = height, path = save_path
    )
  }

  res
}

#' @keywords internal
.surv_plot_multi <- function(input_pd, max_month, break_month, cols,
                             target_group, project, font.size.table) {
  sfit <- survminer::surv_fit(
    survival::Surv(time, status) ~ target_group,
    data = input_pd
  )

  names(sfit$strata) <- gsub("target_group=", "", names(sfit$strata))

  pp <- survminer::ggsurvplot(sfit,
    data = input_pd,
    censor = TRUE,
    ncensor.plot = FALSE,
    conf.int = FALSE,
    xlim = c(0, max_month),
    break.time.by = break_month,
    xlab = "Months after diagnosis",
    submain = paste0(target_group, " ", project),
    surv.median.line = "h",
    risk.table = TRUE,
    tables.height = 0.25,
    palette = cols,
    pval.size = 8
  )

  # Log-rank test
  fitd <- survival::survdiff(
    survival::Surv(time, status) ~ target_group,
    data = input_pd,
    na.action = stats::na.exclude
  )
  p.val <- 1 - stats::pchisq(fitd$chisq, length(fitd$n) - 1)

  p.lab <- paste0(
    "Overall P",
    ifelse(p.val < 0.001, " < 0.001", paste0(" = ", round(p.val, 3)))
  )

  pp$plot <- pp$plot + ggplot2::annotate("text",
    x = 0, y = 0.55,
    hjust = 0,
    fontface = 3,
    label = p.lab
  )

  # Pairwise comparison table
  ps <- survminer::pairwise_survdiff(
    survival::Surv(time, status) ~ target_group,
    data = input_pd,
    p.adjust.method = "none"
  )

  addTab <- as.data.frame(as.matrix(ifelse(
    round(ps$p.value, 3) < 0.001, "<0.001", round(ps$p.value, 3)
  )))
  addTab[is.na(addTab)] <- "-"

  rlang::check_installed("ggpp")
  df <- tibble::tibble(x = 0, y = 0, tb = list(addTab))
  pp$plot <- pp$plot +
    ggpp::geom_table(
      data = df,
      ggplot2::aes(x = x, y = y, label = tb),
      table.rownames = TRUE,
      size = font.size.table
    )

  pp
}

#' @keywords internal
.surv_plot_binary <- function(input_pd, max_month, break_month, cols,
                              target_group, project, reference_group, levels) {
  levels <- unique(input_pd$target_group)
  levels <- levels[order(levels)]

  if (!is.null(reference_group)) {
    if (!reference_group %in% input_pd$target_group) {
      cli::cli_abort("{.arg reference_group} must be one of target_group levels")
    }
    if (reference_group != levels[1]) {
      levels <- c(levels[2], levels[1])
    }
    input_pd$target_group <- ifelse(input_pd$target_group == reference_group, 1, 0)
  } else {
    cli::cli_alert_info("Reference group not defined, using alphabetical order")
  }

  pvalue <- getHRandCIfromCoxph(survival::coxph(
    survival::Surv(time, status) ~ target_group,
    data = input_pd
  ))

  HR <- paste("Hazard Ratio =", round(pvalue[, 2], 2))
  CI <- paste("95% CI:", paste(round(pvalue[, 3], 2), round(pvalue[, 4], 2), sep = " - "))

  sfit <- survminer::surv_fit(
    survival::Surv(time, status) ~ target_group,
    data = input_pd
  )

  survminer::ggsurvplot(sfit,
    data = input_pd,
    censor = TRUE,
    ncensor.plot = FALSE,
    conf.int = FALSE,
    xlim = c(0, max_month),
    break.time.by = break_month,
    xlab = "Months after diagnosis",
    legend.labs = c(levels[2], levels[1]),
    submain = paste0(target_group, " ", project),
    risk.table = TRUE,
    tables.height = 0.20,
    palette = cols,
    pval.size = 8,
    pval = paste(
      ifelse(pvalue[, 1] < 0.0001, "P < 0.0001", paste("P =", round(pvalue[, 1], 4))),
      HR, CI,
      sep = "\n"
    )
  )
}

#' Break Time Into Blocks
#'
#' @description
#' Divides time duration into specified blocks for analysis.
#'
#' @param input Numeric vector of time durations.
#' @param block Number of blocks. Default is 6.
#' @param time_type Units: "month" or "day". Default is "month".
#'
#' @return Numeric vector of breakpoints, rounded to nearest multiple of 5.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' time_data <- c(24, 36, 12, 48)
#' blocks <- calculate_break_month(input = time_data)
calculate_break_month <- function(input, block = 6, time_type = c("month", "day")) {
  time_type <- rlang::arg_match(time_type)

  max_time <- max(input, na.rm = TRUE)
  if (time_type == "day") {
    max_time <- max_time / 30
  }

  cli::cli_alert_info(
    "Maximum follow-up time is {round(max_time, 1)} months; divided into {block} sections"
  )

  round((max_time %/% block) / 5) * 5
}

#' Build Prognostic Models Using LASSO and Ridge Regression
#'
#' @description
#' Prepares data, splits it into training and testing sets, and fits LASSO and Ridge
#' regression models for survival analysis. Evaluates model performance using
#' cross-validation and optionally generates time-dependent ROC curves for visual
#' assessment of predictive accuracy.
#'
#' @param x A matrix or data frame of predictor variables (features).
#' @param y A data frame of survival outcomes with two columns: survival time and event status.
#' @param scale Logical indicating whether to scale predictor variables. Default is `FALSE`.
#' @param seed Integer seed for random number generation to ensure reproducibility.
#'   Default is `123456`.
#' @param train_ratio Numeric proportion of data for training (e.g., 0.7). Default is `0.7`.
#' @param nfold Integer number of folds for cross-validation. Default is `10`.
#' @param plot Logical indicating whether to plot ROC curves. Default is `TRUE`.
#' @param cols Optional vector of colors for ROC curves. If `NULL`, uses default palette.
#' @param palette String specifying color palette. Default is `"jama"`.
#'
#' @return A list containing:
#' \describe{
#'   \item{lasso_result}{Results from LASSO model including coefficients and AUC}
#'   \item{ridge_result}{Results from Ridge model including coefficients and AUC}
#'   \item{train.x}{Training data with sample IDs}
#' }
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE) &&
#'   requireNamespace("survival", quietly = TRUE)) {
#'   library(survival)
#'   set.seed(123)
#'   # Create small example data (first column must be ID)
#'   x_sim <- as.data.frame(matrix(rnorm(100 * 5), 100, 5))
#'   colnames(x_sim) <- paste0("Sig", 1:5)
#'   x_sim$ID <- paste0("S", 1:100)
#'   x_sim <- x_sim[, c(6, 1:5)] # Move ID to first column
#'
#'   y_sim <- data.frame(
#'     ID = paste0("S", 1:100),
#'     OS_days = rexp(100, 0.01),
#'     OS_status = rbinom(100, 1, 0.5)
#'   )
#'   prognostic_result <- PrognosticModel(
#'     x = x_sim, y = y_sim,
#'     scale = TRUE, seed = 123456,
#'     train_ratio = 0.7, nfold = 3, plot = FALSE
#'   )
#'   if (!is.null(prognostic_result)) head(prognostic_result)
#' }
PrognosticModel <- function(x, y, scale = FALSE, seed = 123456, train_ratio = 0.7,
                            nfold = 10, plot = TRUE, palette = "jama", cols = NULL) {
  if (is.null(x)) return(NULL)
  rlang::check_installed("glmnet")

  x <- as.data.frame(x)
  y <- as.data.frame(y)

  cli::cli_alert_info("Processing data")
  processdat <- ProcessingData(x = x, y = y, scale = scale, type = "survival")
  x_scale <- processdat$x_scale
  y <- processdat$y
  x_ID <- processdat$x_ID

  cli::cli_alert_info("Splitting data into training and validation sets")
  train_test <- SplitTrainTest(
    x = x_scale, y = y, train_ratio = train_ratio,
    type = "survival", seed = seed
  )
  train.x <- train_test$train.x
  train.y <- train_test$train.y
  test.x <- train_test$test.x
  test.y <- train_test$test.y
  train_sample <- train_test$train_sample
  return.x <- data.frame(ID = x_ID[train_sample], train.x)

  cli::cli_alert_info("Running LASSO")
  set.seed(seed)
  lasso_model <- glmnet::cv.glmnet(
    x = train.x, y = as.matrix(train.y),
    family = "cox", alpha = 1, nfolds = nfold
  )
  lasso_result <- PrognosticResult(model = lasso_model, train.x, train.y, test.x, test.y)

  if (plot) {
    p1 <- PlotTimeROC(
      train.x = train.x, train.y = train.y,
      test.x = test.x, test.y = test.y, model = lasso_model, modelname = "LASSO"
    )
    if (interactive()) print(p1)
  }

  cli::cli_alert_info("Running RIDGE REGRESSION")
  set.seed(seed)
  ridge_model <- glmnet::cv.glmnet(
    x = train.x, y = as.matrix(train.y),
    family = "cox", alpha = 0, nfolds = nfold
  )
  ridge_result <- PrognosticResult(model = ridge_model, train.x, train.y, test.x, test.y)

  if (plot) {
    p2 <- PlotTimeROC(
      train.x = train.x, train.y = train.y, cols = cols, palette = palette,
      test.x = test.x, test.y = test.y, model = ridge_model, modelname = "RIDGE"
    )
    if (interactive()) print(p2)
  }

  cli::cli_alert_success("Model fitting complete")

  list(
    lasso_result = lasso_result, ridge_result = ridge_result, train.x = return.x,
    plots = if (plot) list(lasso = p1, ridge = p2) else NULL
  )
}

#' Compute Prognostic Results for Survival Models
#'
#' @description
#' Computes and compiles prognostic results from a survival model fitted with `glmnet`.
#' Extracts model coefficients at optimal lambda values (`lambda.min` and `lambda.1se`)
#' and calculates time-dependent AUC metrics for both training and testing datasets.
#'
#' @param model A fitted survival model object (e.g., from `glmnet::cv.glmnet`).
#' @param train.x Matrix or data frame of training predictors.
#' @param train.y Training dataset survival outcomes (time and status).
#' @param test.x Matrix or data frame of testing predictors.
#' @param test.y Testing dataset survival outcomes (time and status).
#'
#' @return A list containing:
#' \describe{
#'   \item{model}{The fitted model object}
#'   \item{coefs}{Data frame of coefficients at `lambda.min` and `lambda.1se`}
#'   \item{AUC}{Data frame with AUC values for train/test at both lambda values}
#' }
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE) &&
#'   requireNamespace("survival", quietly = TRUE) &&
#'   requireNamespace("timeROC", quietly = TRUE)) {
#'   library(survival)
#'   set.seed(123)
#'   train_x <- matrix(rnorm(100 * 10), ncol = 10)
#'   train_y <- data.frame(time = rexp(100), status = rbinom(100, 1, 0.5))
#'   test_x <- matrix(rnorm(50 * 10), ncol = 10)
#'   test_y <- data.frame(time = rexp(50), status = rbinom(50, 1, 0.5))
#'   fit <- glmnet::cv.glmnet(train_x, Surv(train_y$time, train_y$status), family = "cox")
#'   results <- PrognosticResult(
#'     model = fit, train.x = train_x, train.y = train_y,
#'     test.x = test_x, test.y = test_y
#'   )
#' }
PrognosticResult <- function(model, train.x, train.y, test.x, test.y) {
  coefs <- cbind(
    stats::coef(model, s = "lambda.min"),
    stats::coef(model, s = "lambda.1se")
  )
  coefs <- data.frame(
    feature = rownames(coefs),
    lambda.min = coefs[, 1],
    lambda.1se = coefs[, 2],
    row.names = NULL
  )

  datasets <- list(
    list(data = train.x, outcome = train.y, lambda = "lambda.min"),
    list(data = train.x, outcome = train.y, lambda = "lambda.1se"),
    list(data = test.x, outcome = test.y, lambda = "lambda.min"),
    list(data = test.x, outcome = test.y, lambda = "lambda.1se")
  )

  auc_list <- lapply(datasets, function(d) {
    PrognosticAUC(model = model, newx = d$data, s = d$lambda, acture.y = d$outcome)
  })

  auc <- dplyr::bind_rows(auc_list)
  rownames(auc) <- c(
    "Train_lambda.min", "Train_lambda.1se",
    "Test_lambda.min", "Test_lambda.1se"
  )

  list(model = model, coefs = coefs, AUC = auc)
}

#' Calculate Time-Dependent AUC for Survival Models
#'
#' @description
#' Evaluates prognostic ability of a survival model by calculating time-dependent AUC
#' at the 30th and 90th percentiles of survival time. These thresholds assess
#' short-term and long-term predictive accuracy.
#'
#' @param model A fitted survival model object capable of generating risk scores.
#' @param newx A matrix or data frame of new data for prediction.
#' @param s Lambda value for prediction. Can be numeric or `"lambda.min"`/`"lambda.1se"`.
#' @param acture.y Data frame with `time` and `status` columns.
#'
#' @return A data frame with AUC values at 30th (`probs.3`) and 90th (`probs.9`) percentiles.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE) &&
#'   requireNamespace("survival", quietly = TRUE) &&
#'   requireNamespace("timeROC", quietly = TRUE)) {
#'   library(survival)
#'   set.seed(123)
#'   x <- matrix(rnorm(100 * 5), ncol = 5)
#'   y <- Surv(rexp(100), rbinom(100, 1, 0.5))
#'   fit <- glmnet::cv.glmnet(x, y, family = "cox")
#'   acture_y <- data.frame(time = y[, 1], status = y[, 2])
#'   auc_results <- PrognosticAUC(fit, newx = x, s = "lambda.min", acture.y = acture_y)
#' }
PrognosticAUC <- function(model, newx, s, acture.y) {
  rlang::check_installed("timeROC")
  riskscore <- stats::predict(model, newx = newx, s = s)
  timerocDat <- data.frame(risk = riskscore[, 1], acture.y)

  ROC <- with(
    timerocDat,
    timeROC::timeROC(
      T = time, delta = status,
      marker = risk, cause = 1,
      weighting = "marginal",
      time = quantile(time, probs = c(0.3, 0.9)),
      ROC = TRUE,
      iid = TRUE
    )
  )

  data.frame(probs.3 = ROC$AUC[1], probs.9 = ROC$AUC[2])
}

#' Calculate Time-Dependent ROC Curve
#'
#' @description
#' Computes time-dependent ROC curve for survival models using the `timeROC` package.
#' Evaluates predictive accuracy at a specified time quantile.
#'
#' @param model A fitted survival model object.
#' @param newx A matrix or data frame of new data for prediction.
#' @param s Lambda value for prediction.
#' @param acture.y Data frame with `time` and `status` columns.
#' @param modelname Character string for model identification.
#' @param time_prob Numeric quantile for ROC calculation. Default is `0.9`.
#'
#' @return An object of class `timeROC` containing ROC curve information.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE) &&
#'   requireNamespace("survival", quietly = TRUE) &&
#'   requireNamespace("timeROC", quietly = TRUE)) {
#'   library(survival)
#'   dat <- na.omit(lung[, c("time", "status", "age", "sex", "ph.ecog")])
#'   dat$status <- dat$status - 1
#'   x <- as.matrix(dat[, c("age", "sex", "ph.ecog")])
#'   y <- Surv(dat$time, dat$status)
#'   fit <- glmnet::glmnet(x, y, family = "cox")
#'   actual_outcome <- data.frame(time = dat$time, status = dat$status)
#'   roc_info <- CalculateTimeROC(
#'     model = fit, newx = x, s = 0.01, acture.y = actual_outcome,
#'     modelname = "glmnet Cox Model", time_prob = 0.5
#'   )
#'   print(roc_info$AUC)
#' }
CalculateTimeROC <- function(model, newx, s, acture.y, modelname, time_prob = 0.9) {
  rlang::check_installed("timeROC")
  riskscore <- stats::predict(model, newx = newx, s = s)
  timerocDat <- data.frame(risk = riskscore[, 1], acture.y)

  ROC <- with(
    timerocDat,
    timeROC::timeROC(
      T = time, delta = status,
      marker = risk, cause = 1,
      weighting = "marginal",
      time = quantile(time, probs = time_prob),
      ROC = TRUE,
      iid = TRUE
    )
  )

  ROC
}

#' Plot Time-Dependent ROC Curves
#'
#' @description
#' Generates time-dependent ROC curves for evaluating prognostic accuracy of survival models.
#' Plots training and testing ROC curves at the 90th percentile survival time.
#'
#' @param train.x Matrix or data frame of training predictors.
#' @param train.y Training survival outcomes (time and status).
#' @param test.x Matrix or data frame of testing predictors.
#' @param test.y Testing survival outcomes (time and status).
#' @param model Fitted survival model object.
#' @param modelname Character string for model identification.
#' @param cols Optional vector of colors for plotting.
#' @param palette Character string specifying color palette. Default is `"jama"`.
#'
#' @return A `ggplot` object representing the ROC curve plot.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE) &&
#'   requireNamespace("survival", quietly = TRUE) &&
#'   requireNamespace("timeROC", quietly = TRUE)) {
#'   library(survival)
#'   set.seed(123)
#'   train_x <- matrix(rnorm(100 * 5), ncol = 5)
#'   train_y <- data.frame(time = rexp(100), status = rbinom(100, 1, 0.5))
#'   test_x <- matrix(rnorm(50 * 5), ncol = 5)
#'   test_y <- data.frame(time = rexp(50), status = rbinom(50, 1, 0.5))
#'   fit <- glmnet::cv.glmnet(train_x, Surv(train_y$time, train_y$status), family = "cox")
#'   p <- PlotTimeROC(train_x, train_y, test_x, test_y, fit, "Cox Model")
#'   print(p)
#' }
PlotTimeROC <- function(train.x, train.y, test.x, test.y, model, modelname,
                        cols = NULL, palette = "jama") {
  if (is.null(cols)) {
    cols <- palettes(category = "box", palette = palette, show_message = FALSE, show_col = FALSE)
  }

  datasets <- list(
    list(data = train.x, outcome = train.y, lambda = "lambda.min"),
    list(data = train.x, outcome = train.y, lambda = "lambda.1se"),
    list(data = test.x, outcome = test.y, lambda = "lambda.min"),
    list(data = test.x, outcome = test.y, lambda = "lambda.1se")
  )

  auc_list <- lapply(datasets, function(d) {
    PrognosticAUC(model = model, newx = d$data, s = d$lambda, acture.y = d$outcome)
  })

  auc <- dplyr::bind_rows(auc_list)
  rownames(auc) <- c(
    "Train_lambda.min", "Train_lambda.1se",
    "Test_lambda.min", "Test_lambda.1se"
  )

  roclist <- lapply(datasets, function(d) {
    CalculateTimeROC(
      model = model, newx = d$data, s = d$lambda,
      acture.y = d$outcome, modelname = modelname
    )
  })

  aucs <- round(auc$probs.9, 2)
  legend.name <- paste(
    c("train_lambda.min", "train_lambda.1se", "test_lambda.min", "test_lambda.1se"),
    "AUC", aucs,
    sep = " "
  )
  names(roclist) <- c(
    "train_lambda.min", "train_lambda.1se",
    "test_lambda.min", "test_lambda.1se"
  )

  plotdat <- do.call(rbind, lapply(names(roclist), function(nm) {
    data.frame(
      s = nm,
      x = roclist[[nm]]$FP[, 2],
      y = roclist[[nm]]$TP[, 2]
    )
  }))

  plotdat$s <- factor(plotdat$s, levels = names(roclist))

  ggplot2::ggplot(plotdat, ggplot2::aes(x = .data$x, y = .data$y, color = .data$s)) +
    ggplot2::geom_path(linewidth = 1) +
    ggplot2::geom_abline(intercept = 0, slope = 1, linetype = "dashed") +
    ggplot2::xlab("False positive rate") +
    ggplot2::ylab("True positive rate") +
    ggplot2::theme_bw() +
    ggplot2::scale_color_manual(values = cols, labels = legend.name) +
    ggplot2::ggtitle(paste0(stringr::str_replace(modelname, "_", " "), "\nROC at time quantile 0.9")) +
    ggplot2::theme(
      legend.title = ggplot2::element_blank(),
      plot.title = ggplot2::element_text(size = ggplot2::rel(1.5), hjust = 0.5),
      axis.text.x = ggplot2::element_text(face = "plain", angle = 0, hjust = 1, color = "black"),
      axis.text.y = ggplot2::element_text(face = "plain", angle = 0, hjust = 1, color = "black")
    )
}

#' Feature Selection via Correlation or Differential Expression
#'
#' Selects informative features using either correlation with a quantitative
#' response or differential expression (limma) for binary/continuous
#' responses.
#'
#' @param x Numeric matrix. Features (rows) by samples (columns).
#' @param y Numeric or factor. Response vector (quantitative or binary).
#' @param method Character. "cor" (correlation) or "dif" (differential expression). Default c("cor","dif").
#' @param family Character. Correlation method if method = "cor": "spearman" or "pearson".
#' @param cutoff Numeric. Absolute correlation (for cor) or |log2FC| (for dif) threshold.
#' @param padjcut Numeric. Adjusted p-value cutoff.
#'
#' @import dplyr
#' @import tibble
#' @import tidyr
#' @import purrr
#' @import stringr
#' @import glmnet
#'
#' @return Character vector of selected feature names.
#' @export
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_eset <- matrix(rnorm(100 * 50), 100, 50)
#' rownames(sim_eset) <- c("PDCD1", paste0("Gene", 2:100))
#' colnames(sim_eset) <- paste0("Sample", 1:50)
#' pd1 <- as.numeric(sim_eset["PDCD1", ])
#' group <- ifelse(pd1 > mean(pd1), "high", "low")
#' 
#' # Correlation method
#' pd1_cor <- feature_select(
#'   x = sim_eset, y = pd1, method = "cor",
#'   family = "pearson", padjcut = 0.05, cutoff = 0.5
#' )
#' 
#' # Differential expression method
#' pd1_dif <- feature_select(
#'   x = sim_eset, y = pd1, method = "dif",
#'   padjcut = 0.05, cutoff = 2
#' )
#' pd1_dif_2 <- feature_select(
#'   x = sim_eset, y = group,
#'   method = "dif", padjcut = 0.05, cutoff = 2
#' )
feature_select <- function(x, y, method = c("cor", "dif"),
                           family = c("spearman", "pearson"),
                           cutoff = NULL, padjcut = NULL) {
  method <- match.arg(method)
  if (length(unique(y)) == 1) {
    stop("There are only one constant value in y, y must be binary or quantitative value.")
  }
  type <- ifelse(length(unique(y)) == 2, "Binary", "quantitative")
  if (length(unique(y)) == 2) {
    message("Deteching two levels in y, we will treat y as a binary varibale")
  }
  if (length(unique(y)) > 2) {
    message("Deteching more than two levels in y, we will treat y as a quantitative varibale")
  }

  if (method == "cor") {
    if (type != "quantitative") {
      stop("Correlation between x and y, y must be quantitative")
    }
    Gene <- rownames(x)
    x <- tibble::as_tibble(t(x))
    tmp <- x %>%
      map(cor.test, y, method = family)
    pvalue <- tmp %>% map_dbl("p.value")
    estimate <- tmp %>% map_dbl("estimate")
    P.adj <- p.adjust(as.numeric(pvalue), method = "fdr")
    cor_feature <- tibble(Gene = Gene, P.adj = P.adj, Estimate = estimate)
    cor_feature <- cor_feature %>%
      filter(P.adj < padjcut, abs(Estimate) > cutoff) %>%
      select("Gene")
    feature <- cor_feature$Gene %>% as.character()
  }
  if (method == "dif") {
    if (type == "quantitative") {
      message("For quantitative varibale, upper 25% and bottom 25% samples
              were treated as upregulated group and downregulated group.")
      up <- which(y > quantile(y, 0.75))
      down <- which(y < quantile(y, 0.25))
      pdata <- data.frame(
        samples = c(colnames(x)[up], colnames(x)[down]),
        group = c(rep("up", length(up)), rep("down", length(down)))
      )
      exprdata <- x[, c(up, down)]
      contrastfml <- c("up - down")
    }
    if (type == "Binary") {
      if (length(unique(y)) != 2) {
        stop("y must be binary feature, please check your data carefully")
      }
      pdata <- data.frame(samples = colnames(x), group = y)
      contrastfml <- paste(unique(y)[1], "-", unique(y)[2])
      exprdata <- x
    }
    dif <- limma.dif(exprdata = exprdata, pdata = pdata, contrastfml = contrastfml)
    dif <- data.frame(Probe = rownames(dif), dif)
    dif_feature <- dif %>%
      as_tibble() %>%
      filter(abs(logFC) > cutoff, adj.P.Val < padjcut) %>%
      select("Probe")
    feature <- dif_feature$Probe %>% as.character()
  }
  return(feature)
}

#' Differential Expression Analysis Using Limma
#'
#' Performs differential expression analysis using the limma package on a
#' given gene expression dataset.
#' Constructs a design matrix from phenotype data, fits a linear model,
#' applies contrasts, and computes
#' statistics for differential expression.
#'
#' @param exprdata A matrix with rownames as features like gene symbols or cgi, and colnames as samples.
#' @param pdata A two-column dataframe where the first column matches the colnames of exprdata and the second column contains the grouping variable.
#' @param contrastfml A character vector for contrasts to be tested (see ?makeContrasts for more details).
#'
#' @return Returns a dataframe from limma::topTable, which includes genes as rows and columns like genelist, logFC, AveExpr, etc.
#' @export
#' @examples
#' # Toy example with 100 genes and 6 samples
#' set.seed(123)
#' exprdata <- matrix(
#'   rnorm(100 * 6),
#'   nrow = 100,
#'   ncol = 6,
#'   dimnames = list(
#'     paste0("gene", 1:100),
#'     paste0("sample", 1:6)
#'   )
#' )
#'
#' # Phenotype data: 3 vs 3
#' pdata <- data.frame(
#'   sample = colnames(exprdata),
#'   group = rep(c("group1", "group2"), each = 3),
#'   stringsAsFactors = FALSE
#' )
#'
#' # Differential expression: group1 vs group2
#' res <- limma.dif(
#'   exprdata = exprdata,
#'   pdata = pdata,
#'   contrastfml = "group1 - group2"
#' )
#' head(res)
limma.dif <- function(exprdata, pdata, contrastfml) {
  group_list <- as.character(pdata[, 2])
  design <- model.matrix(~ 0 + factor(group_list))
  colnames(design) <- levels(as.factor(pdata[, 2]))
  rownames(design) <- colnames(exprdata)
  if (!all(colnames(exprdata) == pdata[, 1])) {
    stop(" expression data do not match pdata")
  }
  rlang::check_installed("limma")
  contrast.matrix <- limma::makeContrasts(contrasts = contrastfml, levels = design)
  fit <- limma::lmFit(exprdata, design)
  fit <- limma::contrasts.fit(fit, contrast.matrix)
  fit <- limma::eBayes(fit)
  dif <- limma::topTable(fit, adjust.method = "BH", coef = contrastfml, number = Inf)
  return(dif)
}

#' Preprocess TCGA RNA-seq Data
#'
#' @description
#' Preprocesses TCGA RNA-seq data by modifying sample types, transforming data,
#' and annotating genes based on specified parameters.
#'
#' @param eset Matrix or data frame. RNA-seq gene expression matrix from TCGA.
#' @param id_type Character. Gene identifier type: "ensembl" or "symbol".
#'   Default is "ensembl".
#' @param input_type Character. Input data type: "log2count" or "count".
#'   Default is "log2count".
#' @param output Character. Sample type: "tumor" or "tumor_normal".
#'   Default is "tumor".
#' @param output_type Character. Output data type: "tpm", "log2tpm", or "count".
#'   Default is "tpm".
#' @param annotation Logical. Whether to perform gene annotation. Default is TRUE.
#'
#' @return Preprocessed gene expression matrix.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' # Simulate TCGA-style data with Ensembl IDs
#' set.seed(123)
#' sim_eset <- matrix(
#'   abs(rnorm(500)),
#'   nrow = 100,
#'   ncol = 5,
#'   dimnames = list(
#'     paste0("ENSG00000", 101:200),
#'     paste0("TCGA-AB-123", 4:8, "-01A")
#'   )
#' )
#' # Process tumor samples only (output as count for offline testing)
#' eset <- tcga_rna_preps(
#'   eset = sim_eset, id_type = "ensembl", input_type = "count",
#'   output = "tumor", output_type = "count", annotation = FALSE
#' )
#' print(dim(eset))
tcga_rna_preps <- function(eset,
                           id_type = c("ensembl", "symbol"),
                           input_type = c("log2count", "count"),
                           output = c("tumor", "tumor_normal"),
                           output_type = c("tpm", "log2tpm", "count"),
                           annotation = TRUE) {
  id_type <- rlang::arg_match(id_type)
  input_type <- rlang::arg_match(input_type)
  output <- rlang::arg_match(output)
  output_type <- rlang::arg_match(output_type)

  if (!is.matrix(eset) && !is.data.frame(eset)) {
    cli::cli_abort("{.arg eset} must be a matrix or data frame.")
  }

  if (id_type == "ensembl") {
    rownames(eset) <- substring(rownames(eset), 1, 15)
  }

  if (input_type == "log2count") {
    eset <- (2^eset) - 1
  }

  if (output == "tumor") {
    cli::cli_alert_info("Sample type distribution:")
    message(paste(capture.output(table(substring(colnames(eset), 14, 16))), collapse = "\n"))

    cli::cli_alert_info("Adjacent normal samples removed")
    eset <- eset[, !substring(colnames(eset), 14, 16) == "11A"]

    cli::cli_alert_info("TCGA barcode reduced to 12 digits")
    colnames(eset) <- substring(colnames(eset), 1, 12)

    cli::cli_alert_info("Duplicate barcode check:")
    message(paste(capture.output(summary(duplicated(colnames(eset)))), collapse = "\n"))
    eset <- eset[, !duplicated(colnames(eset))]
  } else {
    cli::cli_alert_info(
      "Both tumor and paracancerous samples retained; TCGA barcode kept to 16 digits"
    )
  }

  if (output_type == "tpm") {
    cli::cli_alert_info("Converting count to TPM")
    eset_tpm <- count2tpm(countMat = eset, idType = id_type)
  } else if (output_type == "log2tpm") {
    cli::cli_alert_info("Converting count to TPM and applying log2 transformation")
    eset_tpm <- count2tpm(countMat = eset, idType = id_type)
    eset_tpm <- log2eset(eset_tpm)
  } else {
    eset_tpm <- eset
    if (id_type == "ensembl" && annotation) {
      cli::cli_alert_info("Annotating genes")
      anno_grch38 <- load_data("anno_grch38")
      eset_tpm <- anno_eset(
        eset = eset_tpm,
        annotation = anno_grch38,
        probe = "id",
        symbol = "symbol"
      )
    }
  }

  eset_tpm
}

#' Stratified Random Sampling of Cells
#'
#' @description
#' Performs stratified random sampling of cells from single-cell data,
#' ensuring proportional representation of each cell type while respecting
#' minimum and maximum count constraints.
#'
#' @param input A data frame or Seurat object containing cell annotations.
#' @param group Character string specifying the column name for cell type
#'   grouping.
#' @param proportion Numeric value between 0 and 1 specifying the sampling
#'   proportion. Default is 0.1.
#' @param minimum_count_include Integer specifying the minimum count threshold
#'   for a cell type to be included in sampling. Default is 300.
#' @param minimum_count Integer specifying the minimum number of cells to
#'   sample per cell type. Default is 200.
#' @param maximum_count Integer specifying the maximum number of cells to
#'   sample per cell type. Default is 1000.
#' @param sub_cluster Optional character string specifying a sub-cluster
#'   column for filtering. Default is NULL.
#' @param cell_type Optional character string specifying the cell type value
#'   to filter when `sub_cluster` is provided. Default is NULL.
#'
#' @return A data frame containing the sampled cells with preserved cell type
#'   proportions.
#'
#' @export
#'
#' @examples
#' # Create simulated cell annotation data
#' set.seed(123)
#' sim_cells <- data.frame(
#'   cell_id = paste0("Cell", 1:500),
#'   cell_type = sample(c("T_cell", "B_cell", "NK_cell", "Macrophage"), 500, replace = TRUE)
#' )
#' # Sample cells with stratified random sampling
#' sampled <- random_strata_cells(
#'   input = sim_cells,
#'   group = "cell_type",
#'   proportion = 0.2,
#'   minimum_count_include = 50,
#'   minimum_count = 20,
#'   maximum_count = 100
#' )
#' if (!is.null(sampled)) head(sampled)
random_strata_cells <- function(input,
                                group,
                                proportion = 0.1,
                                minimum_count_include = 300,
                                minimum_count = 200,
                                maximum_count = 1000,
                                sub_cluster = NULL,
                                cell_type = NULL) {
  # Validate inputs
  stopifnot(
    "input must be a data frame or Seurat object" =
      is.data.frame(input) || inherits(input, "Seurat"),
    "group must be a character string" = is.character(group) && length(group) == 1,
    "proportion must be between 0 and 1" =
      is.numeric(proportion) && proportion > 0 && proportion < 1,
    "minimum_count_include must be positive" =
      is.numeric(minimum_count_include) && minimum_count_include > 0,
    "minimum_count must be positive" =
      is.numeric(minimum_count) && minimum_count > 0,
    "maximum_count must be greater than minimum_count" =
      is.numeric(maximum_count) && maximum_count > minimum_count
  )

  # Extract metadata from Seurat object
  if (inherits(input, "Seurat")) {
    input <- input@meta.data
  }

  # Check if group column exists
  if (!group %in% colnames(input)) {
    cli::cli_abort("Column '{group}' not found in input data.")
  }

  # Filter invalid entries
  input <- input[!is.na(input[[group]]), ]
  input <- input[!input[[group]] %in% c("Undetermined", "NA", " ", ""), ]

  # Apply sub-cluster filter if specified
  if (!is.null(sub_cluster) && !is.null(cell_type)) {
    if (!sub_cluster %in% colnames(input)) {
      cli::cli_abort("Column '{sub_cluster}' not found in input data.")
    }
    input <- input[input[[sub_cluster]] == cell_type, ]
  }

  # Calculate cell type frequencies
  cell_freq <- as.data.frame(table(input[[group]]))
  colnames(cell_freq) <- c("CellType", "Count")

  cli::cli_alert_info("Cell type counts before sampling:")
  if (interactive()) print(cell_freq)

  # Select cell types meeting minimum count threshold
  valid_types <- as.character(
    cell_freq[cell_freq$Count > minimum_count_include, "CellType"]
  )
  valid_types <- valid_types[!valid_types %in% c("Undetermined", "NA", " ", "")]

  if (length(valid_types) == 0) {
    cli::cli_abort("No cell types meet the minimum count threshold of {minimum_count_include}.")
  }

  cli::cli_alert_info("Cell types included in sampling:")
  if (interactive()) print(valid_types)

  # Filter to valid cell types
  input <- input[input[[group]] %in% valid_types, ]
  input <- input[order(input[[group]]), ]

  # Check minimum count constraint
  filtered_freq <- as.data.frame(table(input[[group]]))
  if (minimum_count > min(filtered_freq$Freq)) {
    cli::cli_abort(c(
      "minimum_count ({minimum_count}) exceeds the smallest cell type count.",
      "i" = "Smallest count: {min(filtered_freq$Freq)}",
      "*" = "Reduce minimum_count or adjust minimum_count_include."
    ))
  }

  # Calculate sample sizes
  sample_sizes <- round(table(input[[group]]) * proportion)

  cli::cli_alert_info("Initial sample sizes (proportion = {proportion}):")
  if (interactive()) print(sample_sizes)

  # Perform stratified sampling
  rlang::check_installed("sampling")
  strata_result <- sampling::strata(
    input,
    group,
    size = as.numeric(sample_sizes),
    method = "srswor"
  )
  sampled_data <- sampling::getdata(input, strata_result)
  sampled_data <- input[rownames(input) %in% rownames(sampled_data), ]

  # Adjust cell types below minimum count
  current_freq <- as.data.frame(table(sampled_data[[group]]))
  colnames(current_freq) <- c("CellType", "Count")

  below_min <- current_freq[current_freq$Count < minimum_count, "CellType"]
  above_max <- current_freq[current_freq$Count > maximum_count, "CellType"]

  # Upsample cell types below minimum
  if (length(below_min) > 0) {
    cli::cli_alert_info("Upsampling cell types below minimum_count ({minimum_count}):")
    print(as.character(below_min))

    for (cell in below_min) {
      cell_data <- input[input[[group]] == cell, ]
      sampled_data <- sampled_data[sampled_data[[group]] != cell, ]
      upsampled <- cell_data[sample(rownames(cell_data), minimum_count), ]
      sampled_data <- rbind(sampled_data, upsampled)
    }
  }

  # Downsample cell types above maximum
  if (length(above_max) > 0) {
    cli::cli_alert_info("Downsampling cell types above maximum_count ({maximum_count}):")
    print(as.character(above_max))

    for (cell in above_max) {
      cell_data <- input[input[[group]] == cell, ]
      sampled_data <- sampled_data[sampled_data[[group]] != cell, ]
      downsampled <- cell_data[sample(rownames(cell_data), maximum_count), ]
      sampled_data <- rbind(sampled_data, downsampled)
    }
  }

  # Report final counts
  final_freq <- as.data.frame(table(sampled_data[[group]]))
  colnames(final_freq) <- c("CellType", "Count")

  cli::cli_alert_info("Cell type counts after sampling:")
  if (interactive()) print(final_freq)

  return(sampled_data)
}

#' Save Signature Data to File
#'
#' @description
#' Saves signature data to a specified file format, supporting CSV or RData.
#' Handles single signatures or lists of signatures, converting them to a
#' data frame for storage.
#'
#' @param signatures Signature data: a list or single string of signatures.
#' @param format Character. Output format: "csv" or "rdata". Default is "csv".
#' @param file.name Character. Name of the output file without extension.
#'
#' @return Data frame containing the processed signature data, also saved to
#'   the specified file.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' # Simulate signatures
#' set.seed(123)
#' sim_sigs <- list(
#'   Signature1 = paste0("Gene", 1:50),
#'   Signature2 = paste0("Gene", 51:100),
#'   Signature3 = paste0("Gene", 101:150)
#' )
#' tmpfile <- tempfile(fileext = ".csv")
#' output_sig(
#'   signatures = sim_sigs, format = "csv",
#'   file.name = tools::file_path_sans_ext(tmpfile)
#' )
output_sig <- function(signatures, format = c("csv", "rdata"), file.name) {
  format <- rlang::arg_match(format)

  if (missing(file.name)) {
    cli::cli_abort("{.arg file.name} must be provided.")
  }

  if (is.null(signatures) || length(signatures) == 0) {
    cli::cli_abort("{.arg signatures} cannot be NULL or empty.")
  }

  if (length(signatures) == 1) {
    signatures <- as.data.frame(signatures)
    colnames(signatures) <- "signature genes"
  } else {
    signatures <- as.data.frame(t(do.call("rbind", signatures)))

    for (i in seq_len(ncol(signatures))) {
      signatures[duplicated(signatures[, i]), i] <- NA
    }
  }

  if (format == "csv") {
    write.csv(signatures, file = paste0(file.name, ".csv"), row.names = FALSE)
    cli::cli_alert_success("Signature data saved to {.file {file.name}.csv}")
  } else {
    save(signatures, file = paste0(file.name, ".RData"))
    cli::cli_alert_success("Signature data saved to {.file {file.name}.RData}")
  }

  signatures
}

#' Download IOBR Data from GitHub with Mirror Support
#'
#' @description
#' Downloads large datasets from GitHub releases to avoid CRAN size limits.
#' Supports multiple download mirrors for users in different regions.
#' Data is cached locally after first download. Cache directory can be
#' customized via `options(IOBR.cache_dir = "your/path")`.
#'
#' @param name Character string. Name of the dataset to download.
#' @param force Logical. Whether to force re-download even if cached. Default: FALSE.
#' @param verbose Logical. Whether to print progress messages. Default: TRUE.
#' @param mirrors Character vector. URLs of mirrors to try. Default uses
#'   get_default_mirrors().
#' @param cache_dir Character string. Custom cache directory. If NULL, uses
#'   the option `IOBR.cache_dir` or the default system cache location.
#'
#' @return The requested dataset.
#' @export
#'
#' @examples
#' \dontrun{
#' tcga_sig <- download_iobr_data("tcga_stad_sig")
#' }
download_iobr_data <- function(name, force = FALSE, verbose = TRUE,
                               mirrors = get_default_mirrors(),
                               cache_dir = NULL) {
  # Get all available GitHub datasets
  github_data <- list_github_datasets()

  if (!name %in% github_data) {
    stop(sprintf(
      "Dataset '%s' not available for download. Available: %s",
      name, paste(github_data, collapse = ", ")
    ))
  }

  # Set up cache directory (priority: argument > option > default)
  cache_dir <- get_iobr_cache_dir(cache_dir)
  if (!dir.exists(cache_dir)) {
    dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  }

  cache_file <- file.path(cache_dir, sprintf("%s.rda", name))

  # Check if already cached
  if (file.exists(cache_file) && !force) {
    if (verbose) cli::cli_alert_info("Loading cached data: {.val {name}}")
    env <- new.env()
    load(cache_file, envir = env)
    # Return the object (might be named 'data' or the actual name)
    obj_names <- ls(env)
    if (length(obj_names) == 1) {
      return(env[[obj_names[1]]])
    } else {
      return(env[[name]])
    }
  }

  # Check for internet connection if no local file exists
  if (!file.exists(cache_file)) {
    has_internet <- function() {
      # Test both Google and Baidu, return TRUE if either works
      # Using sequential approach to avoid fork() issues on macOS
      urls <- c("https://www.google.com", "https://www.baidu.com")
      for (url_str in urls) {
        result <- tryCatch({
          con <- url(url_str)
          tryCatch({
            readLines(con, n = 1)
            TRUE
          }, error = function(e) FALSE,
            finally = {
              tryCatch(close(con), error = function(e) NULL)
            })
        }, error = function(e) FALSE)
        if (result) return(TRUE)
      }
      FALSE
    }

    if (!has_internet()) {
      cli::cli_alert_warning("No internet connection available and dataset {.val {name}} is not in cache.")
      cli::cli_alert_info("To comply with CRAN policies, returning {.code NULL} instead of an error.")
      return(NULL)
    }
  }

  # Try each mirror until success
  last_error <- NULL
  for (i in seq_along(mirrors)) {
    mirror <- mirrors[i]

    # Construct URL based on mirror type
    if (grepl("github\\.com", mirror)) {
      # GitHub or GitHub proxy
      url <- sprintf(
        "%s/IOBR/IOBR/releases/download/data-v1.0/%s.rda",
        mirror, name
      )
    } else {
      # Standalone mirror (e.g., biotree.top)
      # Ensure there's a slash between mirror and name
      url <- sprintf("%s/%s.rda", sub("/$", "", mirror), name)
    }

    if (verbose) {
      cli::cli_alert_info("Trying mirror {i}/{length(mirrors)}: {.url {mirror}}")
    }

    tryCatch(
      {
        utils::download.file(url, cache_file, mode = "wb", quiet = !verbose)

        if (file.exists(cache_file) && file.size(cache_file) > 0) {
          if (verbose) cli::cli_alert_success("Download complete: {.val {name}}")

          env <- new.env()
          load(cache_file, envir = env)
          obj_names <- ls(env)
          if (length(obj_names) == 1) {
            return(env[[obj_names[1]]])
          } else {
            return(env[[name]])
          }
        }
      },
      error = function(e) {
        last_error <<- e
        if (verbose) cli::cli_alert_warning("Mirror {i} failed: {e$message}")
        # Clean up partial download
        if (file.exists(cache_file)) file.remove(cache_file)
      }
    )
  }

  # All mirrors failed
  cache_dir <- get_iobr_cache_dir()
  manual_url <- sprintf(
    "https://github.com/IOBR/IOBR/releases/download/data-v1.0/%s.rda", name
  )
  cli::cli_alert_danger("All download mirrors failed for dataset: {.val {name}}")
  cli::cli_alert_info("Please try the following manual download steps:")
  cli::cli_ul(c(
    "1. Download the file from: {.url {manual_url}}",
    sprintf("2. Save it to: {.path %s}", file.path(cache_dir, paste0(name, ".rda"))),
    "3. Run your code again - the data will be loaded from cache"
  ))
  cli::cli_alert_info("To comply with CRAN policies, returning {.code NULL} instead of an error.")
  return(NULL)
}

#' Get Default Download Mirrors
#'
#' @description Returns the default list of download mirrors.
#' @return Character vector of mirror URLs.
#' @keywords internal
get_default_mirrors <- function() {
  c(
    # Original GitHub (default)
    "https://github.com",
    # Chinese mirrors (Verified working)
    "https://gh-proxy.com/https://github.com",
    "https://ghproxy.net/https://github.com",
    "https://moeyy.cn/gh-proxy/https://github.com",
    "https://github.akams.cn/https://github.com",
    "http://toolwa.com/github/https://github.com",
    "https://v6.gh-proxy.org/https://github.com",
    "https://gh-proxy.org/https://github.com",
    "https://ghfast.top/https://github.com",
    "https://download.githubcdn.com?url=https://github.com",
    "https://proxy.gitwarp.top/https://github.com",
    # Biotree mirror (Fallback)
    "http://biotree.top:49004/d/IOBR_data"
  )
}

#' Add Custom Download Mirror
#'
#' @description
#' Adds a custom mirror URL to the default mirrors for the current session.
#' The mirror URL should be a base URL. If it contains 'github.com', it will
#' be treated as a GitHub proxy and the relative path to IOBR releases will
#' be appended. Otherwise, it will be treated as a direct repository path.
#'
#' @param url Character string. The mirror URL to add.
#' @param position Character. Where to add the mirror: "first", "last", or
#'   "before_github". Default: "first".
#'
#' @return Invisibly returns the updated mirror list.
#' @export
#'
#' @examples
#' \dontrun{
#' add_iobr_mirror("https://my-mirror.com/https://github.com")
#' }
add_iobr_mirror <- function(url, position = c("first", "last", "before_github")) {
  position <- match.arg(position)

  # Validate URL
  if (!grepl("^https?://", url)) {
    stop("Invalid URL. Must start with http:// or https://")
  }

  # Get current mirrors from option or default
  current_mirrors <- getOption("IOBR.download_mirrors", get_default_mirrors())

  # Remove trailing slash if present
  url <- sub("/$", "", url)

  # Add URL if not already present
  if (url %in% current_mirrors) {
    cli::cli_alert_info("Mirror {.url {url}} already exists in the list")
    return(invisible(current_mirrors))
  }

  # Add at specified position
  new_mirrors <- switch(position,
    "first" = c(url, current_mirrors),
    "last" = c(current_mirrors, url),
    "before_github" = {
      # Insert before the default GitHub URL
      github_idx <- which(current_mirrors == "https://github.com")
      if (length(github_idx) == 0) {
        c(current_mirrors, url)
      } else {
        c(current_mirrors[1:(github_idx - 1)], url, current_mirrors[github_idx:length(current_mirrors)])
      }
    }
  )

  # Store in options
  options(IOBR.download_mirrors = new_mirrors)

  cli::cli_alert_success("Added mirror {.url {url}} to position: {.val {position}}")
  cli::cli_alert_info("Current mirrors: {.val {length(new_mirrors)}} total")

  invisible(new_mirrors)
}

#' List Current Download Mirrors
#'
#' @description Returns the current list of download mirrors.
#' @return Character vector of mirror URLs.
#' @export
#'
#' @examples
#' list_iobr_mirrors()
list_iobr_mirrors <- function() {
  mirrors <- getOption("IOBR.download_mirrors", get_default_mirrors())

  cat("Current IOBR download mirrors:\n")
  cat("==============================\n")
  for (i in seq_along(mirrors)) {
    cat(sprintf("%d. %s\n", i, mirrors[i]))
  }
  cat("==============================\n")
  cat(sprintf("Total: %d mirrors\n", length(mirrors)))

  invisible(mirrors)
}

#' Reset Download Mirrors to Default
#'
#' @description Resets the download mirror list to the default values.
#' @return Invisibly returns the default mirror list.
#' @export
#'
#' @examples
#' reset_iobr_mirrors()
reset_iobr_mirrors <- function() {
  options(IOBR.download_mirrors = NULL)
  cli::cli_alert_success("Download mirrors reset to default")
  invisible(get_default_mirrors())
}

#' Get IOBR Cache Directory
#'
#' @description Returns the current cache directory for IOBR downloaded data.
#' To comply with CRAN policies, the default cache directory is a session-specific
#' temporary directory. Users can opt-in to a persistent cache by setting
#' `options(IOBR.cache_dir = "your/path")` or using `set_iobr_cache_dir()`.
#'
#' The cache directory is determined in the following priority order:
#' 1. Function argument `cache_dir` (if provided)
#' 2. Option `IOBR.cache_dir` (if set via `options()`)
#' 3. Default: A session-specific temporary directory (`file.path(tempdir(), "IOBR_cache")`)
#'
#' @param cache_dir Optional character string to override the current setting.
#' @return Character string with the cache directory path.
#' @export
#'
#' @examples
#' # Get current cache directory (defaults to tempdir)
#' get_iobr_cache_dir()
#'
#' # Set custom cache directory via options (use tempdir() for examples)
#' options(IOBR.cache_dir = tempdir())
#' get_iobr_cache_dir()
get_iobr_cache_dir <- function(cache_dir = NULL) {
  # Priority: argument > option > default
  if (!is.null(cache_dir)) {
    if (!is.character(cache_dir) || length(cache_dir) != 1) {
      stop("'cache_dir' must be a single character string")
    }
    return(path.expand(cache_dir))
  }

  option_cache <- getOption("IOBR.cache_dir")
  if (!is.null(option_cache)) {
    return(path.expand(option_cache))
  }

  # Default to temporary directory to satisfy CRAN policies
  file.path(tempdir(), "IOBR_cache")
}

#' Set IOBR Cache Directory
#'
#' @description Sets a custom cache directory for IOBR downloaded data.
#' This is useful when you want to store cached data in a specific location,
#' such as a shared network drive or a custom directory.
#'
#' To use the standard system cache location (persistent across sessions),
#' you can use: `set_iobr_cache_dir(tools::R_user_dir("IOBR", which = "cache"))`.
#'
#' @param path Character string. The path to the cache directory.
#' @param create Logical. Whether to create the directory if it doesn't exist.
#'   Default: TRUE.
#'
#' @return Invisibly returns the cache directory path.
#' @export
#'
#' @examples
#' \donttest{
#' # Set a custom cache directory (use tempdir() for examples)
#' set_iobr_cache_dir(tempdir())
#'
#' # Use standard system cache (persistent)
#' # set_iobr_cache_dir(tools::R_user_dir("IOBR", which = "cache"))
#'
#' # Check the current cache directory
#' get_iobr_cache_dir()
#'
#' # Download data will now use the custom cache
#' data <- download_iobr_data("lm22")
#' }
set_iobr_cache_dir <- function(path, create = TRUE) {
  if (!is.character(path) || length(path) != 1) {
    stop("'path' must be a single character string")
  }

  path <- path.expand(path)

  if (create && !dir.exists(path)) {
    dir.create(path, recursive = TRUE, showWarnings = FALSE)
    if (!dir.exists(path)) {
      stop(sprintf("Failed to create cache directory: %s", path))
    }
    cli::cli_alert_success("Created cache directory: {.path {path}}")
  }

  options(IOBR.cache_dir = path)
  cli::cli_alert_success("IOBR cache directory set to: {.path {path}}")

  invisible(path)
}

#' Reset IOBR Cache Directory to Default
#'
#' @description Resets the cache directory to the default session-specific
#' temporary directory.
#' @return Invisibly returns the default cache directory path.
#' @export
#'
#' @examples
#' reset_iobr_cache_dir()
reset_iobr_cache_dir <- function() {
  options(IOBR.cache_dir = NULL)
  default_cache <- file.path(tempdir(), "IOBR_cache")
  cli::cli_alert_success("Cache directory reset to session-specific temporary directory: {.path {default_cache}}")
  invisible(default_cache)
}

#' List Available GitHub Datasets
#'
#' @return Character vector of available dataset names.
#' @export
#'
#' @examples
#' list_github_datasets()
list_github_datasets <- function() {
  c(
    # From data/ directory - Reference matrices
    "BRef", "TRef", "lm22",

    # From data/ directory - Annotation files
    "anno_gc_vm32", "anno_grch38", "anno_hug133plus2", "anno_illumina", "anno_rnaseq",

    # From data/ directory - Example datasets
    "tcga_stad_sig", "imvigor210_sig", "eset_stad", "sig_stad",
    "eset_gse62254", "eset_tme_stad", "eset_blca", "deg",

    # From sysdata - Gene sets
    "hallmark", "kegg", "go_bp", "go_cc", "go_mf", "reactome",
    "msig_immune", "msig_sc",

    # From sysdata - Reference data
    "xCell.data", "quantiseq_data",

    # From sysdata - Cell markers and gene lists
    "cancer_type_genes", "cellmarkers", "common_genes", "immuneCuratedData",
    "ips_gene_set", "length_ensembl", "mRNA_cell_default",
    "mus_human_gene_symbol", "onco_sig", "PurityDataAffy", "SI_geneset",

    # From sysdata - Signatures
    "signature_collection_citation", "signature_metabolism",
    "signature_sc", "signature_tumor",

    # From sysdata - Example datasets
    "imvigor210_eset", "melanoma_data",
    "pdata_acrg", "pdata_GSE63557", "pdata_sig_tme",
    "pdata_sig_tme_binary", "pdata_tme_binary", "tcga_stad_var"
  )
}

#' Clear IOBR Data Cache
#'
#' @description Removes all cached data files downloaded from GitHub.
#' @param cache_dir Character string. Custom cache directory. If NULL, uses
#'   the option `IOBR.cache_dir` or the default system cache location.
#' @return Invisible NULL. Called for side effects of clearing the cache.
#' @export
#'
#' @examples
#' clear_iobr_cache()
clear_iobr_cache <- function(cache_dir = NULL) {
  cache_dir <- get_iobr_cache_dir(cache_dir)
  if (dir.exists(cache_dir)) {
    files <- list.files(cache_dir, full.names = TRUE)
    if (length(files) > 0) {
      file.remove(files)
      cli::cli_alert_success("Cache cleared: {.val {length(files)}} file(s) removed from {.path {cache_dir}}")
    } else {
      cli::cli_alert_info("Cache is already empty: {.path {cache_dir}}")
    }
  } else {
    cli::cli_alert_info("Cache directory does not exist: {.path {cache_dir}}")
  }
  invisible(NULL)
}

###
### $Id: estimateScore.R 13 2016-09-28 19:32:16Z proebuck $

## -----------------------------------------------------------------------------
#' estimateScore
#'
#' This function reads a gene expression dataset in GCT format, calculates
#' enrichment scores
#' for specific gene sets, and writes the computed scores to an output file.
#' It supports multiple
#' platform types and performs platform-specific calculations if necessary.
#'
#' @param input.ds A character string specifying the path to the input dataset file in GCT format.
#'                 The file should have gene expression data with appropriate
#'                 headers.
#' @param output.ds A character string specifying the path to the output dataset file, where
#'                  the calculated scores will be written.
#' @param platform A character vector indicating the platform type. Must be one of "affymetrix",
#'                 "agilent", or "illumina". Platform-specific calculations
#'                 are performed
#'                 based on this parameter.
#'
#' @return This function does not return a value but writes the computed scores to the specified
#'         output file in GCT format.
#' @export
#'
#' @examples
#' \dontrun{
#' set.seed(123)
#' si_geneset_data <- load_data("SI_geneset")
#' if (!is.null(si_geneset_data)) {
#'   gene_names <- unique(c(si_geneset_data[1, -1], si_geneset_data[2, -1]))
#'   gene_names <- gene_names[!is.na(gene_names) & gene_names != ""]
#'   gene_names <- head(gene_names, 200)
#'   n_genes <- length(gene_names)
#'   eset_sim <- as.data.frame(matrix(rnorm(n_genes * 2, mean = 5, sd = 1), n_genes, 2))
#'   rownames(eset_sim) <- gene_names
#'   colnames(eset_sim) <- c("Sample1", "Sample2")
#'   eset_sim <- tibble::rownames_to_column(eset_sim, var = "symbol")
#'   input_file <- tempfile(pattern = "estimate_", fileext = ".gct")
#'   output_file <- tempfile(pattern = "estimate_score_", fileext = ".gct")
#'   writeLines(c("#1.2", paste(nrow(eset_sim), ncol(eset_sim) - 1, sep = "\t")), input_file)
#'   utils::write.table(eset_sim, input_file, sep = "\t", row.names = FALSE,
#'                     col.names = TRUE, append = TRUE, quote = FALSE)
#'   score_res <- estimateScore(input.ds = input_file, output.ds = output_file,
#'                              platform = "affymetrix")
#'   if (!isFALSE(score_res) && file.exists(output_file)) {
#'     head(read.table(output_file, skip = 2, header = TRUE, sep = "\t"))
#'   }
#' }
#' }
estimateScore <- function(input.ds,
                          output.ds,
                          platform = c("affymetrix", "agilent", "illumina")) {
  ## Check arguments
  stopifnot(is.character(input.ds) && length(input.ds) == 1 && nzchar(input.ds))
  stopifnot(is.character(output.ds) && length(output.ds) == 1 && nzchar(output.ds))
  platform <- match.arg(platform)

  ## Read input dataset(GCT format)
  ds <- read.delim(input.ds,
    header = TRUE,
    sep = "\t",
    skip = 2,
    row.names = 1,
    blank.lines.skip = TRUE,
    as.is = TRUE,
    na.strings = ""
  )
  descs <- ds[, 1]
  ds <- ds[-1]
  row.names <- row.names(ds)
  names <- names(ds)
  dataset <- list(
    ds = ds,
    row.names = row.names,
    descs = descs,
    names = names
  )

  m <- data.matrix(dataset$ds)
  gene.names <- dataset$row.names
  sample.names <- dataset$names
  Ns <- length(m[1, ]) # Number of genes
  Ng <- length(m[, 1]) # Number of samples
  temp <- strsplit(input.ds, split = "/")
  s <- length(temp[[1]])
  input.file.name <- temp[[1]][s]
  temp <- strsplit(input.file.name, split = ".gct")
  input.file.prefix <- temp[[1]][1]

  ## Sample rank normalization
  for (j in 1:Ns) {
    m[, j] <- rank(m[, j], ties.method = "average")
  }
  m <- 10000 * m / Ng

  ## SI_geneset
  si_geneset_data <- load_data("SI_geneset")
  if (is.null(si_geneset_data)) return(FALSE)
  gs <- as.matrix(si_geneset_data[, -1], dimnames = NULL)
  N.gs <- 2
  gs.names <- row.names(si_geneset_data)

  ## Loop over gene sets
  score.matrix <- matrix(0, nrow = N.gs, ncol = Ns)
  for (gs.i in 1:N.gs) {
    gene.set <- gs[gs.i, ]
    gene.overlap <- intersect(gene.set, gene.names)
    message(gs.i, " gene set: ", gs.names[gs.i], " overlap=", length(gene.overlap))
    if (length(gene.overlap) == 0) {
      score.matrix[gs.i, ] <- rep(NA, Ns)
      next
    } else {
      ES.vector <- vector(length = Ns)

      ## Enrichment score
      for (S.index in 1:Ns) {
        gene.list <- order(m[, S.index], decreasing = TRUE)
        gene.set2 <- match(gene.overlap, gene.names)
        correl.vector <- m[gene.list, S.index]

        TAG <- sign(match(gene.list, gene.set2, nomatch = 0)) # 1 (TAG) & 0 (no.TAG)
        no.TAG <- 1 - TAG
        N <- length(gene.list)
        Nh <- length(gene.set2)
        Nm <- N - Nh
        correl.vector <- abs(correl.vector)^0.25
        sum.correl <- sum(correl.vector[TAG == 1])
        P0 <- no.TAG / Nm
        F0 <- cumsum(P0)
        Pn <- TAG * correl.vector / sum.correl
        Fn <- cumsum(Pn)
        RES <- Fn - F0
        max.ES <- max(RES)
        min.ES <- min(RES)
        if (max.ES > -min.ES) {
          arg.ES <- which.max(RES)
        } else {
          arg.ES <- which.min(RES)
        }
        ES <- sum(RES)
        EnrichmentScore <- list(
          ES = ES,
          arg.ES = arg.ES,
          RES = RES,
          indicator = TAG
        )
        ES.vector[S.index] <- EnrichmentScore$ES
      }
      score.matrix[gs.i, ] <- ES.vector
    }
  }

  score.data <- data.frame(score.matrix)
  names(score.data) <- sample.names
  row.names(score.data) <- gs.names
  estimate.score <- apply(score.data, 2, sum)

  if (platform != "affymetrix") {
    score.data <- rbind(score.data, estimate.score)
    rownames(score.data) <- c(
      "StromalScore",
      "ImmuneScore",
      "ESTIMATEScore"
    )
  } else {
    ## ---------------------------------------------------------------------
    ## Calculate ESTIMATE-based tumor purity (Affymetrix-specific)
    convert_row_estimate_score_to_tumor_purity <- function(x) {
      stopifnot(is.numeric(x))
      cos(0.6049872018 + 0.0001467884 * x)
    }

    est.new <- NULL
    for (i in 1:length(estimate.score)) {
      est_i <- convert_row_estimate_score_to_tumor_purity(estimate.score[i])
      est.new <- rbind(est.new, est_i)
      if (est_i >= 0) {
        next
      } else {
        message(paste(sample.names[i], ": out of bounds", sep = ""))
      }
    }
    colnames(est.new) <- c("TumorPurity")
    estimate.t1 <- cbind(estimate.score, est.new)
    x.bad.tumor.purities <- estimate.t1[, "TumorPurity"] < 0
    estimate.t1[x.bad.tumor.purities, "TumorPurity"] <- NA
    score.data <- rbind(score.data, t(estimate.t1))
    rownames(score.data) <- c(
      "StromalScore",
      "ImmuneScore",
      "ESTIMATEScore",
      "TumorPurity"
    )
  }
  outputGCT(score.data, output.ds)
}


#' filterCommonGenes
#'
#' This function filters and merges a dataset with a set of common genes.
#'
#' @param input.f A character string specifying the path to the input file or a connection object. The file should be a tab-separated table with row names.
#' @param output.f A character string specifying the path to the output file.
#' @param id A character string indicating the type of gene identifier to use. Can be either "GeneSymbol" or "EntrezID".
#'
#' @return No return value. The function writes the merged dataset to the specified output file.
#' @export
#'
#' @examples
#' \dontrun{
#' input_data <- data.frame(
#'   GeneSymbol = c("BRCA1", "TP53", "EGFR", "NOTCH1"),
#'   Value = c(10, 15, 8, 12),
#'   stringsAsFactors = FALSE
#' )
#' input_file <- tempfile(fileext = ".txt")
#' output_file <- tempfile(fileext = ".txt")
#' write.table(input_data, file = input_file, sep = "\t", row.names = TRUE, quote = FALSE)
#' filterCommonGenes(input_file, output_file, id = "GeneSymbol")
#' }
filterCommonGenes <- function(input.f,
                              output.f,
                              id = c("GeneSymbol", "EntrezID")) {
  ## Check arguments
  stopifnot((is.character(input.f) && length(input.f) == 1 && nzchar(input.f)) ||
    (inherits(input.f, "connection") && isOpen(input.f, "r")))
  stopifnot((is.character(output.f) && length(output.f) == 1 && nzchar(output.f)))
  id <- match.arg(id)

  ## Read input data
  input.df <- read.table(input.f,
    header = TRUE,
    row.names = 1,
    sep = "\t",
    quote = "",
    stringsAsFactors = FALSE
  )

  common_genes_data <- load_data("common_genes")
  if (is.null(common_genes_data)) return(FALSE)
  merged.df <- merge(common_genes_data, input.df, by.x = id, by.y = "row.names")
  rownames(merged.df) <- merged.df$GeneSymbol
  merged.df <- merged.df[, -1:-ncol(common_genes_data)]
  message(sprintf(
    "Merged dataset includes %d genes (%d mismatched).",
    nrow(merged.df),
    nrow(common_genes_data) - nrow(merged.df)
  ))
  outputGCT(merged.df, output.f)
  return(TRUE)
}


#' outputGCT
#'
#' This function converts a gene expression dataset to a GCT format file.
#'
#' @param input.f A data frame or a character string specifying the path to the input file. If a character string, the file should be a tab-separated table with row names.
#' @param output.f A character string specifying the path to the output file.
#'
#' @return No return value. The function writes the dataset to the specified output file in GCT format.
#' @export
#'
#' @examples
#' # Create a sample input data frame
#' sample_data <- data.frame(
#'   Gene = c("BRCA1", "TP53", "EGFR"),
#'   Sample1 = c(10, 15, 8),
#'   Sample2 = c(12, 18, 7),
#'   stringsAsFactors = FALSE
#' )
#' rownames(sample_data) <- sample_data$Gene
#' sample_data <- sample_data[, -1]
#'
#' # Convert the input data frame to GCT format and save to temporary file
#' output_file <- tempfile(fileext = ".gct")
#' outputGCT(sample_data, output_file)
outputGCT <- function(input.f,
                      output.f) {
  ## Check arguments
  ## input.f - must be character string or connection
  ## output.f - must be character string

  if (is.data.frame(input.f) == TRUE) {
    exp.data <- input.f
  } else {
    exp.data <- read.table(input.f, header = TRUE, row.names = 1, sep = "\t", quote = "")
  }

  exp.data1 <- data.frame(NAME = rownames(exp.data), Description = rownames(exp.data), exp.data)
  column1 <- colnames(exp.data1)
  column1[1] <- "NAME"
  column1[2] <- "Description"
  exp.data1$NAME <- factor(exp.data1$NAME)
  exp.data1$Description <- factor(exp.data1$Description)
  levels(exp.data1[, 1]) <- c(levels(exp.data1[, 1]), "NAME")
  levels(exp.data1[, 2]) <- c(levels(exp.data1[, 2]), "Description")
  exp.data2 <- rbind(column1, exp.data1)

  row1 <- rep("", length(1:ncol(exp.data)))
  row1_2 <- data.frame(row1, row1)
  row1_2 <- t(row1_2)
  No_gene <- nrow(exp.data1)
  No_sample <- (ncol(exp.data1) - 2)
  GCT <- matrix(c("#1.2", No_gene, "", No_sample), nrow = 2, ncol = 2)
  gct <- cbind(GCT, row1_2)
  colnames(gct) <- colnames(exp.data2)
  tmp <- rbind(gct, exp.data2)
  write.table(tmp, output.f, sep = "\t", row.names = FALSE, col.names = FALSE, quote = FALSE)
  invisible(NULL)
}


#' plotPurity
#'
#' This function generates scatterplots of tumor purity based on ESTIMATE
#' scores for given samples.
#'
#' @param scores A character string specifying the path to the input file containing ESTIMATE scores. The file should be a tab-separated table with appropriate headers.
#' @param samples A character vector specifying the sample names to plot. The default is "all_samples", which plots all samples in the input file.
#' @param platform A character string specifying the platform used for data collection. Can be "affymetrix", "agilent", or "illumina". Currently, only "affymetrix" is implemented.
#' @param output.dir A character string specifying the directory to save the output plots. If `NULL`, plots are not saved. Default is `NULL`.
#'
#' @return No return value. The function generates and saves scatterplots in the specified output directory.
#' @export
#'
#' @examples
#' # Create a sample ESTIMATE score matrix
#' scores_data <- data.frame(
#'   Sample1 = c(100, 200, 500, 0.80),
#'   Sample2 = c(120, 220, 450, 0.70),
#'   Sample3 = c(140, 240, 600, 0.90),
#'   row.names = c(
#'     "StromalScore", "ImmuneScore", "ESTIMATEScore",
#'     "TumorPurity"
#'   ),
#'   check.names = FALSE
#' )
#'
#' # Write to a temporary GCT file
#' scores_file <- tempfile(fileext = ".gct")
#' outputGCT(scores_data, scores_file)
plotPurity <- function(scores,
                       samples = "all_samples",
                       platform = c("affymetrix", "agilent", "illumina"),
                       output.dir = NULL) {
  ## Check arguments
  stopifnot((is.character(scores) && length(scores) == 1 && nzchar(scores)) ||
    (inherits(scores, "connection") && isOpen(scores, "r")))
  if (!is.null(output.dir)) {
    stopifnot(is.character(output.dir) && length(output.dir) == 1 && nzchar(output.dir))
  }
  platform <- match.arg(platform, choices = c("affymetrix", "agilent", "illumina"))

  if (platform != "affymetrix") {
    stop("not implemented")
  }

  ## Begin processing

  ## -------------------------------------------------------------------------
  get_estimates_df <- function(scores) {
    # estimate <- read.table(scores, skip=2, header=TRUE, row.names=1, sep="\t")
    estimate <- read.delim(scores, skip = 2, row.names = 1)
    as.data.frame(t(estimate[, -1]))
  }

  ## -------------------------------------------------------------------------
  convert_row_estimate_score_to_tumor_purity <- function(x) {
    stopifnot(is.numeric(x))
    cos(0.6049872018 + 0.0001467884 * x)
  }

  ## Read ESTIMATE data file
  estimate.df <- get_estimates_df(scores)
  samplenames <- rownames(estimate.df)
  Affy.model <- load_data("PurityDataAffy")
  pred.p <- Affy.model[, 5:7]
  est <- estimate.df[, 3]
  est.new <- estimate.df[, 4]

  ## Create output directory
  if (!is.null(output.dir)) {
    dir.create(output.dir, showWarnings = FALSE, recursive = TRUE)
  }

  ## ESTIMATE based tumor purity in scatterplot with prediction interval
  if (!is.null(output.dir)) {
    message("Plotting tumor purity based on ESTIMATE score")
  }

  max.af <- max(Affy.model$ESTIMATEScore)
  min.af <- min(Affy.model$ESTIMATEScore)

  if (samples[1] == "all_samples") {
    Num.S <- nrow(estimate.df)
  } else {
    Num.S <- as.numeric(length(samples))
  }

  for (i in 1:Num.S) {
    if (samples[1] == "all_samples") {
      samplename <- samplenames[i]
    } else {
      samplename <- samples[i]
    }

    if (!is.null(output.dir)) {
      png.filename <- file.path(output.dir, sprintf("%s.png", samplename))
      png(filename = png.filename, width = 480, height = 480)
    }

    geMin <- est[i] >= min.af
    leMax <- est[i] <= max.af
    withinMinMax <- geMin && leMax

    xlim <- if (!withinMinMax) {
      ## Expands plot boundary
      adjustment <- 500 # Arbitrary
      if (geMin) {
        from <- min.af
        to <- est[i] + adjustment
      } else {
        from <- est[i] - adjustment
        to <- max.af
      }
      c(from, to)
    } else {
      NULL
    }

    plot(Affy.model$tumor.purity ~ Affy.model$ESTIMATEScore, Affy.model,
      main = samplename,
      type = "n",
      xlab = "ESTIMATE score",
      xlim = xlim,
      ylab = "Tumor purity",
      ylim = c(0, 1)
    )
    # Store original par settings and restore on exit
    old_par <- par(no.readonly = TRUE)
    on.exit(par(old_par))
    par(new = TRUE)
    points(Affy.model$ESTIMATEScore, Affy.model$tumor.purity, cex = 0.75, col = "lightgrey")
    if (withinMinMax) {
      ## Prediction interval
      matlines(Affy.model$ESTIMATEScore, pred.p, lty = c(1, 2, 2), col = "darkgrey")
    } else {
      matlines(Affy.model$ESTIMATEScore, pred.p, lty = c(1, 2, 2), col = "darkgrey")
      par(new = TRUE)
      curve(convert_row_estimate_score_to_tumor_purity,
        from, to,
        n = 10000, col = "grey", ylim = c(0, 1), xlab = "", ylab = ""
      )
    }
    points(est[i], est.new[i], pch = 19, cex = 1.25)
    abline(h = est.new[i], col = "black", lty = 2)
    abline(v = est[i], col = "black", lty = 2)

    if (!is.null(output.dir)) {
      dev.off()
    }
  }
}

#' Harmonize Two Data Frames by Column Structure
#'
#' @description
#' Adds missing columns (filled with `NA`) to a secondary data frame so that
#' its column set and order match a reference data frame. This is useful when
#' combining data frames from different sources that should have the same
#' structure but may be missing some columns.
#'
#' @param data_a Data frame. Reference data frame whose column structure
#'   should be matched.
#' @param data_b Data frame. Data frame to be conformed to `data_a`.
#'
#' @return Data frame `data_b` with added missing columns (NA-filled) and
#'   reordered to match `data_a`.
#'
#' @export
#'
#' @examples
#' # Create reference data frame
#' pdata_a <- data.frame(
#'   A = 1:5, B = 2:6, C = 3:7, D = 4:8, E = 5:9
#' )
#'
#' # Create data frame with subset of columns
#' pdata_b <- data.frame(A = 1:3, C = 4:6, E = 7:9)
#'
#' # Harmonize pdata_b to match pdata_a structure
#' pdata_b_harmonized <- assimilate_data(data_a = pdata_a, data_b = pdata_b)
#' print(names(pdata_b_harmonized)) # Now has A, B, C, D, E
assimilate_data <- function(data_a, data_b) {
  if (is.matrix(data_a)) {
    data_a <- as.data.frame(data_a)
  }
  if (is.matrix(data_b)) {
    data_b <- as.data.frame(data_b)
  }

  # Input validation
  if (is.null(data_a) || !is.data.frame(data_a)) {
    cli::cli_abort("{.arg data_a} must be a non-null data frame.")
  }
  if (is.null(data_b) || !is.data.frame(data_b)) {
    cli::cli_abort("{.arg data_b} must be a non-null data frame.")
  }

  if (nrow(data_a) == 0) {
    cli::cli_warn("{.arg data_a} has 0 rows.")
  }
  if (nrow(data_b) == 0) {
    cli::cli_warn("{.arg data_b} has 0 rows.")
  }

  # Identify missing columns
  missing_cols <- setdiff(names(data_a), names(data_b))

  # Add missing columns with NA
  if (length(missing_cols) > 0) {
    cli::cli_alert_info(
      "Adding {length(missing_cols)} missing column{?s}: {.val {missing_cols}}"
    )

    missing_data <- stats::setNames(
      lapply(missing_cols, function(x) rep(NA, nrow(data_b))),
      missing_cols
    )
    data_b <- cbind(data_b, as.data.frame(missing_data))
  }

  # Reorder columns to match data_a
  data_b[, names(data_a), drop = FALSE]
}

# R/imports.R
#' @importFrom grDevices colorRampPalette dev.off pdf png rgb
#' @importFrom graphics abline barplot box curve legend matlines mtext par points stars
#' @importFrom methods as is
#' @importFrom stats aggregate as.formula binomial coef complete.cases cor cor.test dist end glm hclust kruskal.test lm lsfit mad median model.matrix na.exclude na.omit p.adjust pchisq pnorm prcomp pt qqplot quantile reorder sd start time var wilcox.test
#' @importFrom utils adist combn data head menu packageVersion read.csv read.delim read.table tail write.csv write.table
NULL

#' Convert Read Counts to Transcripts Per Million (TPM)
#'
#' @description
#' Transforms gene expression count data into Transcripts Per Million (TPM)
#' values, normalizing for gene length and library size. Supports multiple gene
#' ID types and can retrieve gene length information from BioMart or use local
#' datasets.
#'
#' @param countMat Numeric matrix of raw read counts with genes in rows and
#'   samples in columns.
#' @param idType Character string specifying the gene identifier type. Options
#'   are `"Ensembl"`, `"Entrez"`, or `"Symbol"`. Default is `"Ensembl"`.
#' @param org Character string specifying the organism. Options include
#'   `"hsa"` (human) or `"mmus"` (mouse). Default is `"hsa"`.
#' @param source Character string specifying the source for gene length
#'   information. Options are `"biomart"` (retrieve from Ensembl BioMart) or
#'   `"local"` (use local dataset). Default is `"local"`.
#' @param effLength Data frame containing effective gene length information. If
#'   `NULL`, lengths are retrieved based on `source`. Default is `NULL`.
#' @param id Character string specifying the column name in `effLength`
#'   containing gene identifiers. Default is `"id"`.
#' @param gene_symbol Character string specifying the column name in `effLength`
#'   containing gene symbols. Default is `"symbol"`.
#' @param length Character string specifying the column name in `effLength`
#'   containing gene lengths. Default is `"eff_length"`.
#' @param check_data Logical indicating whether to check for missing values in
#'   the count matrix. Default is `FALSE`.
#'
#' @return Data frame of TPM-normalized expression values with genes in rows
#'   and samples in columns. Gene identifiers are converted to gene symbols
#'   in the output, regardless of the input `idType`.
#'
#' @author Wubing Zhang, Dongqiang Zeng, Yiran Fang
#' @export
#'
#' @examples
#' # Simulated count data
#' set.seed(123)
#' count_matrix <- matrix(
#'   rpois(100, lambda = 10),
#'   ncol = 5,
#'   dimnames = list(
#'     paste0("ENSG00000", 101:120),
#'     paste0("Sample", 1:5)
#'   )
#' )
#' # Simulated effective length data
#' eff_len <- data.frame(
#'   id = paste0("ENSG00000", 101:120),
#'   symbol = paste0("Gene", 1:20),
#'   eff_length = runif(20, 500, 5000)
#' )
#' # Transform to TPM using local gene annotation
#' eset <- count2tpm(
#'   countMat = count_matrix, effLength = eff_len,
#'   id = "id", length = "eff_length", gene_symbol = "symbol"
#' )
#' head(eset)
count2tpm <- function(countMat,
                      idType = "Ensembl",
                      org = c("hsa", "mmus"),
                      source = c("local", "biomart"),
                      effLength = NULL,
                      id = "id",
                      gene_symbol = "symbol",
                      length = "eff_length",
                      check_data = FALSE) {

  if (is.null(countMat)) return(NULL)
  # Validate arguments
  org <- rlang::arg_match(org)
  source <- rlang::arg_match(source)

  # Ensure matrix format
  if (!is.matrix(countMat)) {
    countMat <- as.matrix(countMat)
  }
  storage.mode(countMat) <- "numeric"

  # Check for missing values
  if (check_data || sum(is.na(countMat)) > 0) {
    na_count <- sum(is.na(countMat))
    if (na_count > 0) {
      cli::cli_alert_warning(
        "Found {na_count} missing value{?s}; removing affected genes"
      )
      feas <- feature_manipulation(data = countMat, is_matrix = TRUE)
      countMat <- countMat[rownames(countMat) %in% feas, , drop = FALSE]
    }
  }

  # Get gene lengths and symbols
  if (!is.null(effLength)) {
    result <- .get_lengths_from_df(countMat, effLength, id, gene_symbol, length)
  } else if (source == "biomart") {
    result <- .get_lengths_from_biomart(countMat, idType, org)
  } else {
    result <- .get_lengths_local(countMat, idType, org)
  }

  # Handle NULL result (offline environment)
  if (is.null(result)) {
    cli::cli_alert_info("Returning NULL - annotation data not available offline")
    return(NULL)
  }

  countMat <- result$countMat
  len <- result$lengths

  # Remove genes without length information
  valid_idx <- !is.na(len)
  if (!all(valid_idx)) {
    n_omit <- sum(!valid_idx)
    cli::cli_alert_warning("Omitting {n_omit} genes without length information")
    countMat <- countMat[valid_idx, , drop = FALSE]
    len <- len[valid_idx]
  }

  # Calculate TPM
  TPM <- .calculate_tpm(countMat, len)

  # Remove duplicates and clean up
  TPM <- TPM[!is.na(rownames(TPM)) & !rownames(TPM) == " ", , drop = FALSE]

  symbol.id <- rownames(TPM)
  TPM <- as.data.frame(TPM)
  TPM$symbol <- symbol.id

  TPM <- remove_duplicate_genes(eset = TPM, column_of_symbol = "symbol")

  as.data.frame(TPM)
}


# Helper: Calculate TPM values
.calculate_tpm <- function(counts, lengths) {
  # RPK: reads per kilobase
  rpk <- counts / c(lengths / 1000)
  # TPM: transcripts per million
  tpm <- 1e6 * t(t(rpk) / colSums(rpk, na.rm = TRUE))
  tpm
}

# Helper: Get lengths from data frame
.get_lengths_from_df <- function(countMat, effLength, id, gene_symbol, length) {
  effLength <- as.data.frame(effLength)
  colnames(effLength)[colnames(effLength) == id] <- "id"
  colnames(effLength)[colnames(effLength) == length] <- "eff_length"
  effLength <- effLength[!duplicated(effLength$id), ]

  countMat <- countMat[rownames(countMat) %in% effLength$id, , drop = FALSE]

  if (nrow(countMat) == 0) {
    cli::cli_abort("No matching identifiers found in effLength data")
  }

  effLength <- effLength[effLength$id %in% rownames(countMat), ]

  if (id != gene_symbol) {
    colnames(effLength)[colnames(effLength) == gene_symbol] <- "gene_symbol"
  } else {
    effLength$gene_symbol <- effLength$id
  }

  idx <- match(rownames(countMat), effLength$id)
  rownames(countMat) <- effLength[idx, "gene_symbol"]
  lengths <- effLength[idx, "eff_length"]

  list(countMat = countMat, lengths = lengths)
}

# Helper: Get lengths from BioMart
.get_lengths_from_biomart <- function(countMat, idType, org) {
  cli::cli_alert_warning(
    "BioMart source is being deprecated. Consider using source='local'"
  )

  rlang::check_installed("biomaRt")

  datasets <- paste0(c(
    "hsapiens", "mmusculus", "btaurus", "cfamiliaris",
    "ptroglodytes", "rnorvegicus", "sscrofa"
  ), "_gene_ensembl")

  type <- c(
    "ensembl_gene_id", "entrezgene_id", "hgnc_symbol",
    "start_position", "end_position"
  )
  if (org == "mmus") type[3] <- "mgi_symbol"

  ds <- datasets[grepl(org, datasets)]

  # Define mirror hosts for failover
  mirrors <- c(
    "https://www.ensembl.org",
    "https://useast.ensembl.org",
    "https://asia.ensembl.org"
  )

  # Try each mirror until success
  mart <- NULL
  last_error <- NULL
  for (mirror in mirrors) {
    tryCatch(
      {
        cli::cli_alert_info("Trying Ensembl mirror: {.url {mirror}}")
        mart <- biomaRt::useMart(
          host = mirror,
          biomart = "ENSEMBL_MART_ENSEMBL",
          dataset = ds
        )
        cli::cli_alert_success("Connected to Ensembl mirror: {.url {mirror}}")
        break
      },
      error = function(e) {
        last_error <<- e
        cli::cli_alert_warning("Failed to connect to {.url {mirror}}: {e$message}")
      }
    )
  }

  if (is.null(mart)) {
    cli::cli_abort(
      "Failed to connect to all Ensembl mirrors. Please try {.code source='local'}"
    )
  }

  ensembl <- biomaRt::getBM(attributes = type, mart = mart)
  ensembl$Length <- abs(ensembl$end_position - ensembl$start_position)

  id_col <- switch(toupper(idType),
    "ENSEMBL" = "ensembl_gene_id",
    "ENTREZ"  = "entrezgene_id",
    "SYMBOL"  = type[3],
    cli::cli_abort("Invalid idType: {.val {idType}}")
  )

  idx <- match(rownames(countMat), ensembl[[id_col]])
  rownames(countMat) <- ensembl[idx, type[3]]
  lengths <- ensembl$Length[idx]

  list(countMat = countMat, lengths = lengths)
}

# Helper: Get lengths from local annotation
.get_lengths_local <- function(countMat, idType, org) {
  idType_lower <- tolower(idType)

  if (org == "hsa") {
    .get_lengths_human(countMat, idType_lower)
  } else if (org == "mmus") {
    .get_lengths_mouse(countMat, idType_lower)
  }
}

# Helper: Get lengths for human genes
.get_lengths_human <- function(countMat, idType) {
  anno_grch38 <- load_data("anno_grch38")

  if (is.null(anno_grch38)) {
    cli::cli_alert_warning("No annotation data available (anno_grch38)")
    cli::cli_alert_info("Returning NULL for offline environment")
    return(NULL)
  }

  cli::cli_alert_info(
    "Using local annotation (anno_grch38) for TPM conversion"
  )

  if (idType == "ensembl") {
    rownames(countMat) <- substring(rownames(countMat), 1, 15)
    length_df <- anno_grch38[, c("id", "eff_length", "symbol")]
  } else if (idType == "entrez") {
    cli::cli_alert_warning(
      "Entrez IDs may yield fuzzy results. Consider using Ensembl IDs"
    )
    length_df <- anno_grch38[, c("entrez", "eff_length", "symbol")]
    colnames(length_df)[1] <- "id"
    length_df <- length_df[!duplicated(length_df$id), ]
  } else if (idType == "symbol") {
    cli::cli_alert_warning(
      "Gene symbols may yield fuzzy results. Consider using Ensembl IDs"
    )
    length_df <- anno_grch38[, c("symbol", "eff_length", "symbol")]
    colnames(length_df)[1] <- "id"
    length_df <- length_df[!duplicated(length_df$id), ]
  } else {
    cli::cli_abort("Invalid idType for local source: {.val {idType}}")
  }

  .match_and_extract_lengths(countMat, length_df)
}

# Helper: Get lengths for mouse genes
.get_lengths_mouse <- function(countMat, idType) {
  anno_gc_vm32 <- load_data("anno_gc_vm32")

  if (is.null(anno_gc_vm32)) {
    cli::cli_alert_warning("No annotation data available (anno_gc_vm32)")
    cli::cli_alert_info("Returning NULL for offline environment")
    return(NULL)
  }

  cli::cli_alert_info(
    "Using local annotation (anno_gc_vm32) for TPM conversion"
  )

  if (idType == "ensembl") {
    length_df <- anno_gc_vm32[, c("id", "eff_length", "symbol")]
  } else if (idType == "mgi") {
    cli::cli_alert_warning(
      "MGI IDs may yield fuzzy results. Consider using Ensembl IDs"
    )
    length_df <- anno_gc_vm32[, c("mgi_id", "eff_length", "symbol")]
    colnames(length_df)[1] <- "id"
    length_df <- length_df[!duplicated(length_df$id), ]
  } else if (idType == "symbol") {
    cli::cli_alert_warning(
      "Gene symbols may yield fuzzy results. Consider using Ensembl IDs"
    )
    length_df <- anno_gc_vm32[, c("symbol", "eff_length", "symbol")]
    colnames(length_df)[1] <- "id"
    length_df <- length_df[!duplicated(length_df$id), ]
  } else {
    cli::cli_abort("Invalid idType for mouse: {.val {idType}}")
  }

  .match_and_extract_lengths(countMat, length_df)
}

# Helper: Match identifiers and extract lengths
.match_and_extract_lengths <- function(countMat, length_df) {
  # Sort by length (descending) for duplicate handling
  length_df <- length_df[order(length_df$eff_length, decreasing = TRUE), ]

  # Filter to common genes
  common_genes <- intersect(rownames(countMat), length_df$id)

  if (length(common_genes) == 0) {
    cli::cli_abort(
      "No matching identifiers found between count matrix and annotation"
    )
  }

  countMat <- countMat[rownames(countMat) %in% common_genes, , drop = FALSE]
  length_df <- length_df[length_df$id %in% common_genes, ]

  # Match and extract lengths
  idx <- match(rownames(countMat), length_df$id)
  lengths <- length_df$eff_length[idx]

  # Update rownames to symbols (third column is symbol)
  rownames(countMat) <- length_df[idx, 3]

  list(countMat = countMat, lengths = lengths)
}

#' Enrichment Bar Plot with Two Directions
#'
#' @description
#' Creates a bar plot visualizing enrichment results for up-regulated and
#' down-regulated terms, using -log10(p-values) to indicate significance.
#'
#' @param up_terms Data frame for up-regulated terms.
#' @param down_terms Data frame for down-regulated terms.
#' @param terms Column name for term descriptions. Default is "Description".
#' @param pvalue Column name for p-values. Default is "pvalue".
#' @param group Column name for group indicator. Default is "group".
#' @param palette Color palette. Default is "jama".
#' @param cols Character vector. Custom colors for bars. If NULL, uses palette.
#'   Default is NULL.
#' @param title Plot title. Default is "Gene Ontology Enrichment".
#' @param width_wrap Maximum width for wrapping pathway names. Default is 30.
#' @param font_terms Font size for axis labels. Default is 15.
#'
#' @return A ggplot object of the enrichment bar plot.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' up_terms <- data.frame(
#'   Description = c("Pathway1", "Pathway2"),
#'   pvalue = c(0.001, 0.01)
#' )
#' down_terms <- data.frame(
#'   Description = c("Pathway4", "Pathway5"),
#'   pvalue = c(0.005, 0.02)
#' )
#' p <- enrichment_barplot(
#'   up_terms = up_terms,
#'   down_terms = down_terms,
#'   title = "Custom Enrichment Plot"
#' )
#' p
enrichment_barplot <- function(up_terms, down_terms,
                               terms = "Description",
                               pvalue = "pvalue",
                               group = "group",
                               palette = "jama",
                               cols = NULL,
                               title = "Gene Ontology Enrichment",
                               width_wrap = 30,
                               font_terms = 15) {
  up_terms <- as.data.frame(up_terms)
  down_terms <- as.data.frame(down_terms)
  dat <- rbind(up_terms, down_terms)

  # Rename columns for internal use
  colnames(dat)[which(colnames(dat) == terms)] <- "terms"
  colnames(dat)[which(colnames(dat) == pvalue)] <- "pvalue"

  # Set up group indicator
  if (!group %in% colnames(dat)) {
    dat$group <- ifelse(dat$terms %in% up_terms[, terms], 1, -1)
  } else {
    colnames(dat)[which(colnames(dat) == group)] <- "group"
  }

  dat$terms <- gsub(dat$terms, pattern = "_", replacement = " ")
  dat$pvalue <- -log10(as.numeric(dat$pvalue))
  dat$pvalue <- dat$pvalue * dat$group
  dat <- dat[order(dat$pvalue, decreasing = FALSE), ]

  # Get colors
  if (is.null(cols)) {
    color <- palettes(category = "box", palette = palette, show_col = FALSE)
  } else {
    if (length(cols) < 2) {
      cli::cli_abort("{.arg cols} must have at least 2 colors")
    }
    color <- cols[1:2]
  }

  ggplot2::ggplot(dat, ggplot2::aes(
    x = reorder(.data$terms, order(.data$pvalue, decreasing = FALSE)),
    y = .data$pvalue, fill = .data$group
  )) +
    ggplot2::geom_bar(stat = "identity") +
    ggplot2::scale_fill_gradient(low = color[1], high = color[2], guide = "none") +
    ggplot2::scale_x_discrete(
      name = "Pathway names",
      labels = function(x) stringr::str_wrap(x, width = width_wrap)
    ) +
    ggplot2::scale_y_continuous(name = "-log10(P.value)") +
    ggplot2::coord_flip() +
    ggplot2::theme_light() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(hjust = 0),
      axis.title = ggplot2::element_text(size = ggplot2::rel(1.3)),
      axis.text.x = ggplot2::element_text(
        face = "plain", size = font_terms, angle = 0, color = "black"
      ),
      axis.text.y = ggplot2::element_text(
        face = "plain", size = font_terms, angle = 0, color = "black"
      )
    ) +
    ggplot2::ggtitle(title)
}

#' Batch Signature Survival Plot
#'
#' @description
#' Generates Kaplan-Meier survival plots for multiple projects or cohorts based
#' on signature scores. Automatically determines optimal cutpoints for signature
#' stratification and creates publication-ready survival curves.
#'
#' @param input_pdata Data frame containing survival data and signature scores.
#' @param signature Character string specifying the column name of the target
#'   signature for survival analysis.
#' @param id Character string specifying the column name containing unique
#'   identifiers. Default is `"ID"`.
#' @param column_of_project Character string specifying the column name
#'   containing project identifiers. Default is `"ProjectID"`.
#' @param project Character string or vector specifying project name(s) to
#'   analyze. If `NULL`, all projects are analyzed. Default is `NULL`.
#' @param time Character string specifying the column name containing
#'   time-to-event data. Default is `"time"`.
#' @param status Character string specifying the column name containing event
#'   status. Default is `"status"`.
#' @param time_type Character string specifying the time unit. Options are
#'   `"day"` or `"month"`. Default is `"day"`.
#' @param break_month Numeric value or `"auto"` specifying the interval for
#'   time axis breaks in months. Default is `"auto"`.
#' @param palette Character string specifying the color palette. Default is
#'   `"jama"`.
#' @param cols Character vector of custom colors. If `NULL`, palette is used.
#'   Default is `NULL`.
#' @param mini_sig Character string for the signature label in the legend.
#'   Default is `"score"`.
#' @param save_path Character string or `NULL`. Directory path for saving plots.
#'   If `NULL`, plots are not saved. Default is `NULL`.
#' @param show_col Logical indicating whether to display color information.
#'   Default is `TRUE`.
#' @param fig_type Character string specifying the output file format
#'   (`"pdf"`, `"png"`, etc.). Default is `"pdf"`.
#'
#' @return Data frame containing combined survival analysis results from all
#'   projects.
#'
#' @author Dongqiang Zeng
#' @export
#' @importFrom survminer surv_cutpoint
#' @import survival
#'
#' @examples
#' set.seed(123)
#' test_pdata <- data.frame(
#'   ID = paste0("S", 1:20),
#'   ProjectID = rep("P1", 20),
#'   OS_time = runif(20, 1, 60),
#'   OS_status = sample(c(0, 1), 20, replace = TRUE),
#'   Marker = rnorm(20)
#' )
#' result <- batch_sig_surv_plot(
#'   input_pdata = test_pdata, signature = "Marker",
#'   id = "ID", column_of_project = "ProjectID",
#'   time = "OS_time", status = "OS_status", time_type = "month"
#' )
#' if (!is.null(result)) head(result)
batch_sig_surv_plot <- function(
  input_pdata,
  signature,
  id = "ID",
  column_of_project = "ProjectID",
  project = NULL,
  time = "time",
  status = "status",
  time_type = "day",
  break_month = "auto",
  palette = "jama",
  cols = NULL,
  mini_sig = "score",
  save_path = NULL,
  show_col = TRUE,
  fig_type = "pdf"
) {
  if (is.null(input_pdata)) return(NULL)
  if (!is.data.frame(input_pdata)) {
    cli::cli_abort("{.arg input_pdata} must be a data frame")
  }
  if (!signature %in% colnames(input_pdata)) {
    cli::cli_abort("Signature column {.val {signature}} not found in input_pdata")
  }
  if (!column_of_project %in% colnames(input_pdata)) {
    cli::cli_abort("Project column {.val {column_of_project}} not found in input_pdata")
  }

  input_pdata <- as.data.frame(input_pdata)
  colnames(input_pdata)[colnames(input_pdata) == column_of_project] <- "ProjectID"
  input_pdata$ProjectID <- as.character(input_pdata$ProjectID)

  goi <- if (is.null(project)) {
    unique(input_pdata$ProjectID)
  } else {
    intersect(project, unique(input_pdata$ProjectID))
  }

  if (length(goi) == 0) {
    cli::cli_abort("No valid projects found")
  }

  results <- lapply(seq_along(goi), function(i) {
    var <- goi[i]
    cli::cli_alert_info("Processing project: {.val {var}}")

    pd <- input_pdata[input_pdata$ProjectID == var, , drop = FALSE]
    if (nrow(pd) == 0) {
      cli::cli_alert_warning("No data for project {.val {var}}, skipping")
      return(NULL)
    }

    sig_surv_plot(
      input_pdata = pd,
      signature = signature,
      project = var,
      ID = id,
      time = time,
      status = status,
      time_type = time_type,
      break_month = break_month,
      cols = cols,
      palette = palette,
      show_col = show_col,
      mini_sig = mini_sig,
      fig.type = fig_type,
      save_path = save_path,
      index = i
    )$data
  })

  invisible(dplyr::bind_rows(results))
}

#' Extract Best Cutoff and Add Binary Variable to Data Frame
#'
#' @description
#' Determines the optimal cutoff point for a continuous variable in survival
#' analysis using the maximally selected rank statistics method. Creates a
#' binary variable based on the identified cutoff and adds it to the input data
#' frame.
#'
#' @param pdata Data frame containing survival information and the continuous
#'   variable.
#' @param variable Character string specifying the name of the continuous
#'   variable for which the optimal cutoff should be determined.
#' @param time Character string specifying the column name containing
#'   time-to-event data. Default is `"time"`.
#' @param status Character string specifying the column name containing event
#'   status (censoring information). Default is `"status"`.
#' @param print_result Logical indicating whether to print detailed results
#'   including cutoff value and Cox model summaries. Default is `TRUE`.
#'
#' @return Data frame identical to `pdata` with an additional binary column
#'   named `<variable>_binary` containing "High" and "Low" categories based on
#'   the optimal cutoff.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' pdata <- data.frame(
#'   time = rexp(100),
#'   status = rbinom(100, 1, 0.5),
#'   score = rnorm(100, mean = 50, sd = 10)
#' )
#' result <- best_cutoff(pdata, variable = "score", print_result = FALSE)
#' table(result$score_binary)
best_cutoff <- function(pdata, variable, time = "time",
                        status = "status", print_result = TRUE) {
  # Input validation
  if (!is.data.frame(pdata)) {
    cli::cli_abort("{.arg pdata} must be a data frame")
  }
  if (!variable %in% colnames(pdata)) {
    cli::cli_abort("Variable {.val {variable}} not found in pdata")
  }
  if (!time %in% colnames(pdata)) {
    cli::cli_abort("Time column {.val {time}} not found in pdata")
  }
  if (!status %in% colnames(pdata)) {
    cli::cli_abort("Status column {.val {status}} not found in pdata")
  }

  pdata <- as.data.frame(pdata)

  # Rename columns temporarily
  colnames(pdata)[colnames(pdata) == time] <- "time_iobr"
  colnames(pdata)[colnames(pdata) == status] <- "status_iobr"

  # Remove NA values
  pdata <- pdata[!is.na(pdata$time_iobr) & !is.na(pdata$status_iobr), ]

  if (nrow(pdata) == 0) {
    cli::cli_abort("No valid data after removing NA values")
  }

  # Ensure numeric
  pdata$time_iobr <- as.numeric(pdata$time_iobr)
  pdata$status_iobr <- as.numeric(pdata$status_iobr)

  # Find best cutoff
  surv_obj <- survival::Surv(pdata$time_iobr, pdata$status_iobr)

  iscutoff <- survminer::surv_cutpoint(
    pdata,
    time = "time_iobr",
    event = "status_iobr",
    variables = variable
  )

  cutoff_value <- iscutoff$cutpoint$cutpoint
  cli::cli_alert_success(paste(
    "Best cutoff for {.val {variable}}:",
    "{round(cutoff_value, 3)}"
  ))

  # Cox model for continuous variable
  cox_cont <- summary(survival::coxph(
    surv_obj ~ pdata[[variable]],
    data = pdata
  ))

  # Create binary variable
  variable2 <- paste0(variable, "_binary")
  pdata[[variable2]] <- ifelse(
    pdata[[variable]] <= cutoff_value,
    "Low",
    "High"
  )
  pdata[[variable2]] <- factor(pdata[[variable2]], levels = c("Low", "High"))

  # Summary statistics
  binary_summary <- summary(pdata[[variable2]])

  # Cox model for binary variable
  cox_binary <- summary(survival::coxph(
    surv_obj ~ pdata[[variable2]],
    data = pdata
  ))

  if (print_result) {
    print(list(
      best_cutoff = paste0("The best cutoff is = ", round(cutoff_value, 3)),
      cox_continuous_object = cox_cont,
      summary_binary_variable = binary_summary,
      cox_binary_object = cox_binary
    ))
  }

  # Restore original column names
  colnames(pdata)[colnames(pdata) == "time_iobr"] <- time
  colnames(pdata)[colnames(pdata) == "status_iobr"] <- status

  pdata
}

#' Calculate and Visualize Correlation Matrix Between Two Variable Sets
#'
#' @description
#' Calculates and visualizes the correlation matrix between two sets of variables.
#' Supports Pearson, Spearman, and Kendall correlation methods. The function
#' generates a customizable heatmap with significance stars.
#'
#' @param data Input data frame or matrix. Variables should be in columns.
#' @param feas1 Character vector of variable names for the first set.
#' @param feas2 Character vector of variable names for the second set.
#' @param method Correlation method: `"pearson"`, `"spearman"`, or `"kendall"`.
#'   Default is `"pearson"`.
#' @param path Directory to save the plot. If `NULL`, plot is not saved.
#'   Default is `NULL`.
#' @param index Numeric prefix for output filename. Default is 1.
#' @param fig.type File format: `"pdf"`, `"png"`, etc. Default is `"pdf"`.
#' @param width Plot width in inches. Auto-calculated if `NULL`.
#' @param height Plot height in inches. Auto-calculated if `NULL`.
#' @param project Project name for plot title. Default is `NULL`.
#' @param is.matrix Logical: if `TRUE`, data is transposed. Default is `FALSE`.
#' @param scale Logical: scale variables before correlation. Default is `TRUE`.
#' @param font.size Font size for axis labels. Default is 15.
#' @param fill_by_cor Logical: show correlation values instead of stars.
#'   Default is `FALSE`.
#' @param round.num Decimal places for correlation values. Default is 1.
#' @param font.size.star Font size for significance stars. Default is 8.
#' @param cols Custom colors for gradient (low, mid, high). If `NULL`, uses
#'   blue-white-red. Default is `NULL`.
#'
#' @return ggplot object displaying the correlation matrix heatmap.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' data <- as.data.frame(matrix(rnorm(1000), nrow = 100, ncol = 10))
#' colnames(data) <- paste0("Gene_", 1:10)
#'
#' feas1 <- c("Gene_1", "Gene_2", "Gene_3")
#' feas2 <- c("Gene_4", "Gene_5", "Gene_6")
#'
#' cor_plot <- get_cor_matrix(
#'   data = data,
#'   feas1 = feas1,
#'   feas2 = feas2,
#'   method = "spearman",
#'   project = "Example Correlation"
#' )
get_cor_matrix <- function(data,
                           feas1,
                           feas2,
                           method = c("pearson", "spearman", "kendall"),
                           path = NULL,
                           index = 1,
                           fig.type = "pdf",
                           width = NULL,
                           height = NULL,
                           project = NULL,
                           is.matrix = FALSE,
                           scale = TRUE,
                           font.size = 15,
                           fill_by_cor = FALSE,
                           round.num = 1,
                           font.size.star = 8,
                           cols = NULL) {
  # Validate method
  method <- rlang::arg_match(method)

  # Create output folder if requested
  if (!is.null(path)) {
    path <- creat_folder(path)
  }

  # Handle matrix input
  if (is.matrix) data <- as.data.frame(t(data))

  # Validate features
  if (!is.character(feas1) || length(feas1) == 0) {
    cli::cli_abort("{.arg feas1} must be a non-empty character vector.")
  }
  if (!is.character(feas2) || length(feas2) == 0) {
    cli::cli_abort("{.arg feas2} must be a non-empty character vector.")
  }

  # Filter to existing features
  feas1 <- feas1[feas1 %in% colnames(data)]
  feas2 <- feas2[feas2 %in% colnames(data)]

  if (length(feas1) == 0) {
    cli::cli_abort("No features from {.arg feas1} found in data.")
  }
  if (length(feas2) == 0) {
    cli::cli_abort("No features from {.arg feas2} found in data.")
  }

  cli::cli_alert_info(
    "Calculating {method} correlation: {length(feas1)} x {length(feas2)}"
  )

  # Scale data if requested
  data_vars <- data[, unique(c(feas1, feas2)), drop = FALSE]
  if (scale) {
    data_vars <- scale(data_vars)
  }

  # Calculate correlation
  rlang::check_installed("psych", reason = "to calculate correlation matrices")
  result <- psych::corr.test(
    data_vars[, feas1, drop = FALSE],
    data_vars[, feas2, drop = FALSE],
    method = method
  )

  # Reshape for plotting
  rlang::check_installed("reshape2")
  heat <- cbind(
    reshape2::melt(result$r),
    reshape2::melt(result$p)
  )[, c(1, 2, 3, 6)]
  colnames(heat) <- c("ID1", "ID2", "cor", "pvalue")

  # Clean labels for display (replace underscores with spaces)
  feas1_clean <- gsub("_", " ", feas1)
  feas2_clean <- gsub("_", " ", feas2)

  heat$ID1 <- gsub("_", " ", as.character(heat$ID1))
  heat$ID2 <- gsub("_", " ", as.character(heat$ID2))

  # Set factor levels to preserve order
  heat$ID1 <- factor(heat$ID1, levels = feas1_clean)
  heat$ID2 <- factor(heat$ID2, levels = rev(feas2_clean))

  # Create labels
  if (fill_by_cor) {
    heat$stars <- round(heat$cor, round.num)
  } else {
    heat$stars <- cut(
      heat$pvalue,
      breaks = c(-Inf, 0.001, 0.01, 0.05, 0.5, Inf),
      labels = c("***", "**", "*", "+", "")
    )
  }

  # Define colors
  if (is.null(cols)) {
    low_col <- "#2C7BB6" # Blue
    mid_col <- "white"
    high_col <- "#D7191C" # Red
  } else {
    if (length(cols) < 2) {
      cli::cli_abort("{.arg cols} must have at least 2 colors (low, high)")
    }
    low_col <- cols[1]
    high_col <- cols[length(cols)]
    mid_col <- if (length(cols) >= 3) cols[2] else "white"
  }

  # Create plot
  # cor_plot <- p +
  # ggplot2::geom_tile() +
  # ggplot2::scale_fill_gradient2(
  #   low = low_col,
  #   mid = mid_col,
  #   high = high_col,
  #   name = "Coefficient"
  # ) +
  cor_plot <- ggplot2::ggplot(
    heat,
    ggplot2::aes(x = ID1, y = ID2, fill = cor)
  ) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_gradient2(
      low = low_col,
      mid = mid_col,
      high = high_col,
      name = "Coefficient"
  ) +
  ggplot2::geom_text(
    ggplot2::aes(label = .data$stars),
    color = "black",
    size = font.size.star
  ) +
  ggplot2::scale_x_discrete(
    labels = function(x) stringr::str_trunc(x, width = 28)
  ) +
  ggplot2::scale_y_discrete(
    labels = function(y) stringr::str_wrap(y, width = 40)
  ) +
  ggplot2::labs(x = NULL, y = NULL) +
  ggplot2::coord_cartesian(clip = "off") +
  ggplot2::theme_bw() +
  ggplot2::theme(
    axis.text.x = ggplot2::element_text(
      angle = -45,
      hjust = 0,
      vjust = 1,
      size = font.size,
      margin = ggplot2::margin(t = 8)
    ),
    axis.text.y = ggplot2::element_text(
      angle = 0,
      hjust = 1,
      size = font.size
    ),
    axis.title = ggplot2::element_text(
      size = font.size + 4
    ),
    plot.margin = ggplot2::margin(
      t = 10, r = 20, b = 80, l = 20
    )
  )

  if (!is.null(project)) {
    cor_plot <- cor_plot + ggplot2::ggtitle(label = project)
  }

  # Auto-calculate dimensions
  if (is.null(width)) width <- length(feas1) * 0.55 + 6.5
  if (is.null(height)) height <- length(feas2) * 0.35 + 4.5

  # Save if path provided
  if (!is.null(path)) {
    ggplot2::ggsave(
      filename = paste0(index, "-", project %||% "cor", "-cor_plot.", fig.type),
      plot = cor_plot,
      width = width,
      height = height,
      path = path$folder_name
    )
  }

  if (interactive()) print(cor_plot)
  invisible(cor_plot)
}

# CIBERSORT R script v1.04 (last updated 10-24-2016)
# Note: Signature matrix construction is not currently available;
#   use java version for full functionality.
# Author: Aaron M. Newman, Stanford University (amnewman@stanford.edu)
# Requirements:
#   R v3.0 or later.
# License: https://cibersortx.stanford.edu/CIBERSORT_License.txt

#' Core Algorithm for CIBERSORT Deconvolution
#'
#' @description
#' Performs nu-regression using support vector machines (SVM) to estimate
#' cell type proportions. This is the core computational engine of CIBERSORT,
#' using nu-SVR with linear kernel to decompose mixed gene expression signals.
#'
#' @param X Matrix or data frame containing signature matrix (predictor variables).
#'   Rows are genes, columns are cell types.
#' @param y Numeric vector containing the mixture sample expression (response variable).
#' @param absolute Logical indicating whether to use absolute space for weights.
#'   Default is FALSE (relative proportions).
#' @param abs_method String specifying the method for absolute space weights:
#'   `"sig.score"` or `"no.sumto1"`.
#'
#' @return List containing:
#' \describe{
#'   \item{w}{Estimated cell type weights/proportions}
#'   \item{mix_rmse}{Root mean squared error of the deconvolution}
#'   \item{mix_r}{Correlation coefficient between observed and predicted mixture}
#' }
#'
#' @export
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' X <- matrix(rnorm(100), nrow = 10)
#' rownames(X) <- paste0("Gene", 1:10)
#' colnames(X) <- paste0("Cell", 1:10)
#' y <- rnorm(10)
#' names(y) <- paste0("Gene", 1:10)
#'
#' # Run core algorithm
#' result <- CoreAlg(X, y, absolute = FALSE, abs_method = "sig.score")
#' if (!is.null(result)) str(result)
CoreAlg <- function(X, y, absolute, abs_method) {
  if (is.null(X)) return(NULL)
  rlang::check_installed("e1071")

  svn_itor <- 3
  nus <- c(0.25, 0.5, 0.75)

  res <- function(i) {
    model <- e1071::svm(X, y,
      type = "nu-regression", kernel = "linear",
      nu = nus[i], scale = FALSE
    )
    model
  }

  if (Sys.info()["sysname"] == "Windows") {
    out <- parallel::mclapply(1:svn_itor, res, mc.cores = 1)
  } else {
    out <- parallel::mclapply(1:svn_itor, res, mc.cores = 1)
  }

  nusvm <- numeric(svn_itor)
  corrv <- numeric(svn_itor)

  for (t in seq_len(svn_itor)) {
    weights <- t(out[[t]]$coefs) %*% out[[t]]$SV
    weights[which(weights < 0)] <- 0
    w <- weights / sum(weights)
    u <- sweep(X, MARGIN = 2, w, "*")
    k <- apply(u, 1, sum)
    nusvm[t] <- sqrt(mean((k - y)^2))
    corrv[t] <- cor(k, y)
  }

  mn <- which.min(nusvm)
  model <- out[[mn]]

  q <- t(model$coefs) %*% model$SV
  q[which(q < 0)] <- 0

  if (!absolute || abs_method == "sig.score") {
    w <- q / sum(q)
  } else if (absolute && abs_method == "no.sumto1") {
    w <- q
  }

  list(w = w, mix_rmse = nusvm[mn], mix_r = corrv[mn])
}

#' Permutation Test for CIBERSORT
#'
#' @description
#' Performs permutation-based sampling to generate an empirical null distribution
#' of correlation coefficients for p-value calculation in CIBERSORT analysis.
#' Randomly samples from the mixture data to create null distributions.
#'
#' @param perm Integer. Number of permutations to perform (\eqn{\ge}100 recommended for
#'   reliable p-value estimation).
#' @param X Matrix or data frame containing signature matrix (predictor variables).
#' @param Y Numeric vector containing the mixture sample expression.
#' @param absolute Logical indicating whether to use absolute space for weights.
#' @param abs_method String specifying the method for absolute space weights:
#'   `"sig.score"` or `"no.sumto1"`.
#' @param seed Integer. Random seed for reproducibility. If NULL (default),
#'   uses current random state.
#'
#' @return List containing:
#' \describe{
#'   \item{dist}{Numeric vector of correlation coefficients from permutations}
#' }
#'
#' @export
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' X <- matrix(rnorm(100), nrow = 10)
#' rownames(X) <- paste0("Gene", 1:10)
#' colnames(X) <- paste0("Cell", 1:10)
#' Y <- rnorm(10)
#' names(Y) <- paste0("Gene", 1:10)
#'
#' # Run permutation test
#' result <- doPerm(10, X, Y, absolute = FALSE, abs_method = "sig.score")
#' if (!is.null(result)) str(result)
doPerm <- function(perm, X, Y, absolute, abs_method, seed = NULL) {
  if (is.null(perm)) return(NULL)
  if (!is.null(seed)) set.seed(seed)

  if (perm < 1) {
    cli::cli_abort("{.arg perm} must be at least 1, got {perm}")
  }

  Ylist <- as.list(data.matrix(Y))
  dist <- numeric(perm)

  for (itor in seq_len(perm)) {
    yr <- as.numeric(Ylist[sample(length(Ylist), dim(X)[1])])
    yr <- (yr - mean(yr)) / sd(yr)
    result <- CoreAlg(X, yr, absolute, abs_method)
    dist[itor] <- result$mix_r
  }

  list(dist = dist)
}


#' Parallel Permutation Test for CIBERSORT
#'
#' @description
#' Parallel version of doPerm. Performs permutation-based sampling and runs
#' the CoreAlg function iteratively using multiple CPU cores to accelerate
#' computation. This function generates an empirical null distribution of
#' correlation coefficients for p-value calculation in CIBERSORT analysis.
#'
#' @param perm1 Integer. Number of permutations to perform (\eqn{\ge}100 recommended).
#' @param X1 Matrix or data frame. Signature matrix (cell type GEP barcode).
#' @param Y1 Matrix. Mixture file containing gene expression profiles.
#' @param absolute1 Logical. Whether to run in absolute mode (default: FALSE).
#' @param abs_method1 String. Method for absolute mode: 'sig.score' or 'no.sumto1'.
#' @param num_cores1 Integer. Number of CPU cores for parallel computation (default: 2).
#' @param seed Integer. Random seed for reproducibility. If NULL (default),
#'   uses current random state. Set to a specific value (e.g., 123) for
#'   reproducible results across runs.
#'
#' @return List containing:
#' \describe{
#'   \item{dist}{Numeric vector of correlation coefficients from permutations,
#'     representing the empirical null distribution.}
#' }
#'
#' @details
#' This function utilizes the \code{foreach} and \code{doParallel} packages
#' to distribute permutation iterations across multiple cores. It automatically
#' handles cluster setup/teardown via \code{on.exit()} to prevent resource leaks.
#'
#' Note: Windows users may experience slower performance due to socket-based
#' parallelization (PSOCK) versus forking on Unix systems.
#'
#' @seealso \code{\link{doPerm}} for the sequential version, \code{\link{CoreAlg}}, \code{\link{CIBERSORT}}
#'
#' @importFrom parallel makeCluster stopCluster
#' @export
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' X <- matrix(rnorm(1000), nrow = 100)
#' Y <- matrix(rnorm(500), nrow = 100)
#' rownames(X) <- rownames(Y) <- paste0("Gene", 1:100)
#' colnames(X) <- paste0("Cell", 1:10)
#' colnames(Y) <- paste0("Sample", 1:5)
#'
#' # Run parallel permutation test
#' result <- parallel_doperm(
#'   perm1 = 10, X1 = X, Y1 = Y,
#'   absolute1 = FALSE, abs_method1 = "sig.score", num_cores1 = 1
#' )
#' if (!is.null(result)) str(result$dist)
parallel_doperm <- function(perm1, X1, Y1, absolute1, abs_method1,
                            num_cores1 = 2, seed = NULL) {
  if (is.null(perm1)) return(NULL)
  rlang::check_installed("foreach", reason = "for parallel permutation")
  rlang::check_installed("doParallel", reason = "for parallel backend")

  if (perm1 < 1) {
    cli::cli_abort("{.arg perm1} must be at least 1, got {perm1}")
  }
  if (num_cores1 < 1) {
    cli::cli_abort("{.arg num_cores1} must be at least 1, got {num_cores1}")
  }

  `%dopar%` <- foreach::`%dopar%`

  if (!is.null(seed)) set.seed(seed)
  n <- dim(X1)[1]
  Ylist_len <- length(as.list(data.matrix(Y1)))
  perm_indices <- lapply(seq_len(perm1), function(i) sample(Ylist_len, n))

  cl <- parallel::makeCluster(num_cores1)
  on.exit(
    {
      parallel::stopCluster(cl)
      foreach::registerDoSEQ()
    },
    add = TRUE
  )

  doParallel::registerDoParallel(cl)

  dist <- foreach::foreach(
    itor = seq_len(perm1),
    .combine = rbind,
    .export = c("perm_indices", "Y1", "X1", "absolute1", "abs_method1"),
    .packages = "e1071"
  ) %dopar% {
    Ylist <- as.list(data.matrix(Y1))
    yr <- as.numeric(Ylist[perm_indices[[itor]]])
    yr <- (yr - mean(yr)) / sd(yr)

    result <- CoreAlg(X1, yr, absolute1, abs_method1)
    result$mix_r
  }

  list(dist = dist)
}


#' CIBERSORT Deconvolution Algorithm
#'
#' An analytical tool to estimate cell type abundances in mixed cell populations
#' using gene expression data.

#' @param sig_matrix Cell type GEP barcode matrix: row 1 = sample labels;
#'   column 1 = gene symbols; no missing values; default = LM22.txt download
#'   from CIBERSORT (https://cibersortx.stanford.edu/runcibersort.php)
#' @param mixture_file GEP matrix: row 1 = sample labels; column 1 = gene
#'   symbols; no missing values
#' @param perm Set permutations for statistical analysis
#'   (>=100 permutations recommended).
#' @param QN Quantile normalization of input mixture (default = TRUE)
#' @param absolute Run CIBERSORT in absolute mode (default = FALSE)
#'   - note that cell subsets will be scaled by their absolute levels and
#'     will not be represented as fractions (to derive the default output,
#'     normalize absolute levels such that they sum to 1 for each mixture sample)
#'   - the sum of all cell subsets in each mixture sample will be added to the
#'     output ('Absolute score'). If LM22 is used, this score will capture
#'     total immune content.
#' @param abs_method If absolute is set to TRUE, choose method:
#'   'no.sumto1' or 'sig.score'
#'   - sig.score = for each mixture sample, define S as the median expression
#'     level of all genes in the signature matrix divided by the median
#'     expression level of all genes in the mixture. Multiple cell subset
#'     fractions by S.
#'   - no.sumto1 = remove sum to 1 constraint
#' @param parallel Logical. Enable parallel execution? (default = FALSE)
#' @param num_cores Integer. Number of cores to use when \code{parallel = TRUE}
#'   (default = 2)
#' @param seed Integer. Random seed for reproducible permutation testing.
#'   If \code{NULL} (default), uses current random state. Set to a specific
#'   value (e.g., 123) for reproducible results across runs. Applies to both
#'   parallel and serial permutation.
#' @author Aaron M. Newman, Stanford University (amnewman@stanford.edu)
#' @return A matrix object containing the estimated cibersort-cell fractions,
#'   p-values, correlation coefficients, and RMSE values.
#' @export
#' @import parallel
#' @import dplyr
#' @import tibble
#' @import tidyr
#' @import purrr
#' @import stringr
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_sig <- matrix(rnorm(100 * 5), 100, 5)
#' rownames(sim_sig) <- paste0("Gene", 1:100)
#' colnames(sim_sig) <- paste0("Cell", 1:5)
#'
#' sim_mixture <- as.data.frame(matrix(
#'   rnorm(100 * 3), 100, 3
#' ))
#' rownames(sim_mixture) <- paste0("Gene", 1:100)
#' colnames(sim_mixture) <- paste0("Sample", 1:3)
#'
#' # Run deconvolution
#' result <- CIBERSORT(
#'   sig_matrix = sim_sig,
#'   mixture_file = sim_mixture,
#'   perm = 10, QN = FALSE, absolute = FALSE,
#'   parallel = FALSE
#' )
#' if (!is.null(result)) head(result)
CIBERSORT <- function(sig_matrix = NULL, mixture_file, perm, QN = TRUE,
                      absolute = FALSE, abs_method = "sig.score",
                      parallel = FALSE, num_cores = 2, seed = NULL) {
  if (is.null(mixture_file)) return(NULL)
  if (is.null(sig_matrix)) {
    sig_matrix <- load_data("lm22")
    if (is.null(sig_matrix)) return(NULL)
  }
  # Input validation
  if (length(intersect(rownames(mixture_file), rownames(sig_matrix))) == 0) {
    stop("None identical gene between eset and reference had been found.
         Check your eset using: intersect(rownames(eset), rownames(reference))")
  }

  if (absolute && abs_method != "no.sumto1" && abs_method != "sig.score") {
    stop("abs_method must be set to either 'sig.score' or 'no.sumto1'")
  }

  # Read in data
  X <- sig_matrix
  Y <- rownames_to_column(mixture_file, var = "symbol")

  # Handle duplicated gene symbols
  dups <- dim(Y)[1] - length(unique(Y[, 1]))
  if (dups > 0) {
    warning(paste(dups, "duplicated gene symbol(s) found in mixture file!"))
    rownames(Y) <- make.names(Y[, 1], unique = TRUE)
  } else {
    rownames(Y) <- Y[, 1]
  }
  Y <- Y[, -1, drop = FALSE]

  X <- data.matrix(X)
  Y <- data.matrix(Y)

  # Order
  X <- X[order(rownames(X)), , drop = FALSE]
  Y <- Y[order(rownames(Y)), , drop = FALSE]

  P <- perm

  # Anti-log if max < 50 in mixture file
  if (max(Y) < 50) {
    Y <- 2^Y
  }

  # Quantile normalization of mixture file
  if (QN == TRUE) {
    tmpc <- colnames(Y)
    tmpr <- rownames(Y)
    rlang::check_installed("preprocessCore")
    Yq <- preprocessCore::normalize.quantiles(Y)
    Y <- matrix(Yq,
      nrow = length(tmpr), ncol = length(tmpc),
      dimnames = list(tmpr, tmpc)
    )
  }

  # Store original mixtures
  Yorig <- Y
  Ymedian <- max(median(Yorig), 1)

  # Intersect genes
  Xgns <- row.names(X)
  Ygns <- row.names(Y)
  YintX <- Ygns %in% Xgns
  Y <- Y[YintX, , drop = FALSE]
  XintY <- Xgns %in% row.names(Y)
  X <- X[XintY, , drop = FALSE]

  # Sanity check
  if (length(dim(Y)) != 2) {
    stop(
      "`mixture_file` became non-2D after preprocessing. ",
      "This usually occurs when it has only one column and subsetting ",
      "dropped dimensions. ",
      "Please update package or ensure `mixture_file` is a matrix with drop = FALSE."
    )
  }

  # Standardize sig matrix
  X <- (X - mean(X)) / sd(as.vector(X))

  # Parallel setup with safety checks
  use_parallel_mixtures <- parallel

  if (parallel) {
    rlang::check_installed("BiocParallel", reason = "for parallel mode")

    # Cap cores to prevent system overload
    max_available <- parallel::detectCores()
    if (is.na(max_available)) max_available <- 2
    safe_cores <- max(1, max_available - 1)

    if (num_cores > safe_cores) {
      warning(
        "Requested ", num_cores, " cores but only ", safe_cores,
        " available/safe. Reducing to ", safe_cores, "."
      )
      num_cores <- safe_cores
    }

    BPPARAM <- BiocParallel::MulticoreParam(workers = num_cores)
  }

  # Empirical null distribution of correlation coefficients
  if (P > 0) {
    if (parallel && exists("parallel_doperm", mode = "function")) {
      # Use parallel permutation (consumes parallel resources)
      kk <- parallel_doperm(
        perm1 = P, X1 = X, Y1 = Y,
        absolute1 = absolute, abs_method1 = abs_method,
        num_cores1 = num_cores, seed = seed
      )
      nulldist <- sort(kk$dist)
      # CRITICAL: Disable parallel for mixtures to avoid nested parallelism
      use_parallel_mixtures <- FALSE
    } else {
      # Serial permutation
      nulldist <- sort(doPerm(P, X, Y, absolute, abs_method, seed = seed)$dist)
    }
  } else {
    nulldist <- NULL
  }

  header <- c("Mixture", colnames(X), "P-value", "Correlation", "RMSE")
  if (absolute) {
    header <- c(header, paste0("Absolute score (", abs_method, ")"))
  }

  # Core job function with explicit parameters (Windows-safe)
  coreJob <- function(itor, Y, X, absolute, abs_method, Ymedian, P, nulldist) {
    y <- Y[, itor, drop = TRUE]
    y <- (y - mean(y)) / sd(y)
    result <- CoreAlg(X, y, absolute, abs_method)
    w <- result$w
    mix_r <- result$mix_r
    mix_rmse <- result$mix_rmse
    if (absolute && abs_method == "sig.score") {
      w <- w * median(Y[, itor]) / Ymedian
    }
    pval <- if (P > 0) {
      1 - which.min(abs(nulldist - mix_r)) / length(nulldist)
    } else {
      9999
    }

    out <- c(colnames(Y)[itor], w, pval, mix_r, mix_rmse)
    if (absolute) out <- c(out, sum(w))
    out
  }

  mixtures <- ncol(Y)

  # Process mixtures
  if (use_parallel_mixtures && mixtures > 1) {
    # Parallel processing (Windows-safe via explicit variable passing)
    output_list <- BiocParallel::bplapply(
      seq_len(mixtures),
      function(i) {
        coreJob(i,
          Y = Y, X = X, absolute = absolute,
          abs_method = abs_method, Ymedian = Ymedian,
          P = P, nulldist = nulldist
        )
      },
      BPPARAM = BPPARAM
    )
    output <- do.call(rbind, output_list)
  } else {
    # Serial processing (safe fallback)
    output_list <- lapply(
      seq_len(mixtures),
      function(i) {
        coreJob(i,
          Y = Y, X = X, absolute = absolute,
          abs_method = abs_method, Ymedian = Ymedian,
          P = P, nulldist = nulldist
        )
      }
    )
    output <- do.call(rbind, output_list)
  }

  # Return matrix object containing all results
  obj <- rbind(header, output)
  obj <- obj[, -1, drop = FALSE]
  obj <- obj[-1, , drop = FALSE]
  obj <- matrix(as.numeric(unlist(obj)), nrow = nrow(obj))
  rownames(obj) <- colnames(Y)
  if (!absolute) {
    colnames(obj) <- c(colnames(X), "P-value", "Correlation", "RMSE")
  } else {
    colnames(obj) <- c(
      colnames(X), "P-value", "Correlation", "RMSE",
      paste0("Absolute score (", abs_method, ")")
    )
  }
  obj
}

#' Batch Kruskal-Wallis Test
#'
#' @description
#' Performs Kruskal-Wallis rank sum tests on multiple continuous features across
#' different groups. Computes p-values, adjusts for multiple testing, and ranks
#' features by significance.
#'
#' @param data Data frame containing the dataset for analysis.
#' @param group Character string specifying the name of the grouping variable.
#' @param feature Character vector specifying the names of feature variables to
#'   test. If `NULL`, the user is prompted to select features (interactive mode
#'   only). Default is `NULL`.
#' @param feature_manipulation Logical indicating whether to apply feature
#'   manipulation to filter valid features. Default is `FALSE`.
#'
#' @return Tibble containing:
#' \describe{
#'   \item{sig_names}{Feature name}
#'   \item{p.value}{Raw p-value from Kruskal-Wallis test}
#'   \item{statistic}{Test statistic (chi-squared)}
#'   \item{p.adj}{Adjusted p-value (Benjamini-Hochberg)}
#'   \item{log10pvalue}{Negative log10-transformed p-value}
#'   \item{stars}{Significance stars: **** p<0.0001, *** p<0.001,
#'     ** p<0.01, * p<0.05, + p<0.5}
#'   \item{group columns}{Mean-centered values for each group}
#' }
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' # Create small example data
#' set.seed(123)
#' test_data <- data.frame(
#'   Gender = rep(c("Male", "Female"), each = 50),
#'   Signature1 = rnorm(100),
#'   Signature2 = rnorm(100)
#' )
#' # Test features by gender
#' res <- batch_kruskal(
#'   data = test_data,
#'   group = "Gender",
#'   feature = c("Signature1", "Signature2")
#' )
#' head(res)
batch_kruskal <- function(data,
                          group,
                          feature = NULL,
                          feature_manipulation = FALSE) {
  if (is.null(data)) return(NULL)
  # Input validation
  if (!is.data.frame(data)) {
    cli::cli_abort("{.arg data} must be a non-null data frame.")
  }
  if (nrow(data) == 0) {
    cli::cli_abort("{.arg data} has no rows.")
  }
  if (!group %in% colnames(data)) {
    cli::cli_abort("Group variable {.val {group}} not found in data.")
  }

  # Prepare data
  data <- as.data.frame(data)
  data$group <- as.character(data[[group]])
  data <- data[!is.na(data$group) & data$group != "", , drop = FALSE]

  group_names <- unique(data$group)
  if (length(group_names) < 2) {
    cli::cli_abort("Group variable must have at least 2 unique non-NA values.")
  }

  # Feature selection
  if (is.null(feature)) {
    if (!interactive()) {
      cli::cli_abort(
        "{.arg feature} must be specified in non-interactive mode."
      )
    }
    cli::cli_alert_info("Select features for analysis")
    index <- utils::menu(
      c("All continuous features", "Specify manually"),
      title = "Feature selection:"
    )
    if (index == 1) {
      feature <- names(data)[vapply(data, is.numeric, logical(1))]
    } else {
      cli::cli_abort("Please specify features via the {.arg feature} parameter")
    }
  }

  if (!is.character(feature) || length(feature) == 0) {
    cli::cli_abort("{.arg feature} must be a non-empty character vector.")
  }

  # Filter valid features
  valid_features <- feature[feature %in% colnames(data)]
  invalid_features <- setdiff(feature, colnames(data))

  if (length(invalid_features) > 0) {
    cli::cli_alert_warning(
      "Ignoring {length(invalid_features)} invalid feature{?s}:",
      " {.val {invalid_features}}"
    )
  }

  if (length(valid_features) == 0) {
    cli::cli_abort("No valid features found in data.")
  }

  # Apply feature manipulation if requested
  if (feature_manipulation) {
    valid_features <- feature_manipulation(
      data = data,
      feature = valid_features,
      print_result = FALSE
    )
  }

  data <- data[, c("group", valid_features), drop = FALSE]

  cli::cli_alert_info("Groups: {length(group_names)} ({.val {group_names}})")
  cli::cli_alert_info("Features: {length(valid_features)}")

  # Kruskal-Wallis test - vectorized
  results <- vapply(valid_features, function(feat) {
    test <- stats::kruskal.test(data[[feat]] ~ data$group)
    c(statistic = as.numeric(test$statistic), p.value = test$p.value)
  }, numeric(2))

  cc <- data.frame(
    sig_names = valid_features,
    p.value = results["p.value", ],
    statistic = results["statistic", ],
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  # Calculate group means (mean-centered)
  result_mean <- data %>%
    dplyr::group_by(.data$group) %>%
    dplyr::summarise(
      dplyr::across(
        dplyr::all_of(valid_features),
        function(.x) mean(.x, na.rm = TRUE)
      ),
      .groups = "drop"
    ) %>%
    tidyr::pivot_longer(
      cols = dplyr::all_of(valid_features),
      names_to = "sig_names",
      values_to = "value"
    ) %>%
    tidyr::pivot_wider(
      names_from = "group",
      values_from = "value"
    ) %>%
    as.data.frame()

  # Calculate mean-centered values
  group_cols <- intersect(group_names, colnames(result_mean))
  if (length(group_cols) > 0) {
    result_mean$overall_mean <- rowMeans(
      as.matrix(result_mean[, group_cols, drop = FALSE]),
      na.rm = TRUE
    )

    for (gn in group_cols) {
      result_mean[[gn]] <- result_mean[[gn]] - result_mean$overall_mean
    }
    result_mean$overall_mean <- NULL
  }

  # Merge results
  cc <- merge(cc, result_mean, by = "sig_names", all.x = TRUE)
  cc$p.adj <- stats::p.adjust(cc$p.value, method = "BH")
  cc$log10pvalue <- -log10(cc$p.value)
  cc$stars <- cut(cc$p.value,
    breaks = c(-Inf, 0.0001, 0.001, 0.01, 0.05, 0.5, Inf),
    labels = c("****", "***", "**", "*", "+", "")
  )

  cc <- cc[order(cc$p.value), , drop = FALSE]

  cli::cli_alert_success("Kruskal-Wallis test complete")

  tibble::as_tibble(cc)
}

#' Binomial Model Construction
#'
#' @description
#' Constructs and evaluates binomial logistic regression models using Lasso and Ridge
#' regularization. Processes input data, scales features if specified, splits data into
#' training/testing sets, and fits both Lasso and Ridge models. Optionally generates
#' AUC plots for model evaluation.
#'
#' @param x A data frame containing sample ID and features. First column must be sample ID.
#' @param y A data frame where first column is sample ID and second column is outcome
#'   (numeric or factor).
#' @param seed Integer for random seed. Default is `123456`.
#' @param scale Logical indicating whether to scale features. Default is `TRUE`.
#' @param train_ratio Numeric between 0 and 1 for training proportion. Default is `0.7`.
#' @param nfold Integer for cross-validation folds. Default is `10`.
#' @param plot Logical indicating whether to generate AUC plots. Default is `TRUE`.
#' @param cols Optional color vector for ROC curves. Default is `NULL`.
#' @param palette Character string for color palette. Default is `"jama"`.
#'
#' @return List containing:
#' \describe{
#'   \item{lasso_result}{Lasso model results}
#'   \item{ridge_result}{Ridge model results}
#'   \item{train.x}{Training data with IDs}
#' }
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' x <- data.frame(
#'   ID = paste0("Sample", 1:50),
#'   Feature1 = rnorm(50),
#'   Feature2 = rnorm(50),
#'   Feature3 = rnorm(50)
#' )
#' y <- data.frame(
#'   ID = x$ID,
#'   Outcome = factor(rbinom(50, 1, 0.5))
#' )
#' result <- BinomialModel(x = x, y = y, plot = FALSE, nfold = 5)
#' str(result, max.level = 1)
BinomialModel <- function(x, y, seed = 123456, scale = TRUE, train_ratio = 0.7,
                          nfold = 10, plot = TRUE, palette = "jama", cols = NULL) {
  rlang::check_installed("glmnet")

  x <- as.data.frame(x)
  y <- as.data.frame(y)

  cli::cli_alert_info("Processing data")
  processdat <- ProcessingData(x = x, y = y, scale = scale, type = "binomial")
  x_scale <- processdat$x_scale
  y <- processdat$y
  x_ID <- processdat$x_ID

  cli::cli_alert_info("Splitting data into training and test sets")
  train_test <- SplitTrainTest(
    x = x_scale, y = y, train_ratio = train_ratio,
    type = "binomial", seed = seed
  )
  train.x <- train_test$train.x
  train.y <- train_test$train.y
  test.x <- train_test$test.x
  test.y <- train_test$test.y
  train_sample <- train_test$train_sample
  return.x <- data.frame(ID = x_ID[train_sample], train.x)

  cli::cli_alert_info("Running LASSO")
  set.seed(seed)
  lasso_model <- glmnet::cv.glmnet(
    x = train.x, y = train.y, family = "binomial",
    type.measure = "class", alpha = 1, nfolds = nfold
  )
  lasso_result <- RegressionResult(
    train.x = train.x, train.y = train.y,
    test.x = test.x, test.y = test.y, model = lasso_model
  )

  if (plot) {
    p1 <- PlotAUC(
      train.x = train.x, train.y = train.y,
      test.x = test.x, test.y = test.y, model = lasso_model,
      cols = cols, palette = palette, modelname = "lasso_model"
    )
    if (interactive()) print(p1)
  }

  cli::cli_alert_info("Running RIDGE REGRESSION")
  set.seed(seed)
  ridge_model <- glmnet::cv.glmnet(
    x = train.x, y = train.y, family = "binomial",
    type.measure = "class", alpha = 0, nfolds = nfold
  )
  ridge_result <- RegressionResult(
    train.x = train.x, train.y = train.y,
    test.x = test.x, test.y = test.y, model = ridge_model
  )

  if (plot) {
    p2 <- PlotAUC(
      train.x = train.x, train.y = train.y,
      test.x = test.x, test.y = test.y, model = ridge_model,
      cols = cols, palette = palette, modelname = "ridge_model"
    )
    if (interactive()) print(p2)
  }

  cli::cli_alert_success("Model fitting complete")

  list(
    lasso_result = lasso_result, ridge_result = ridge_result, train.x = return.x,
    plots = if (plot) list(lasso = p1, ridge = p2) else NULL
  )
}

#' Process Data for Model Construction
#'
#' @description
#' Preprocesses data for binomial or survival analysis. Aligns and filters data
#' based on sample IDs, optionally scales data, and ensures appropriate data types.
#' Handles missing values by removing columns with NA values.
#'
#' @param x Data frame of predictors with first column as IDs.
#' @param y Data frame of outcomes with first column as IDs. For survival,
#'   expects two additional columns for time and status.
#' @param scale Logical indicating whether to scale predictors.
#' @param type Character string: `"binomial"` or `"survival"`.
#'
#' @return List containing:
#' \describe{
#'   \item{x_scale}{Processed predictor matrix}
#'   \item{y}{Processed outcome variable}
#'   \item{x_ID}{Sample IDs}
#' }
#'
#' @export
#'
#' @examples
#' x <- data.frame(ID = 1:10, predictor1 = rnorm(10), predictor2 = rnorm(10))
#' y <- data.frame(ID = 1:10, outcome = sample(c(0, 1), 10, replace = TRUE))
#' result <- ProcessingData(x, y, scale = TRUE, type = "binomial")
ProcessingData <- function(x, y, scale, type = c("binomial", "survival")) {
  type <- rlang::arg_match(type)

  if (!is.data.frame(x)) {
    cli::cli_abort("{.arg x} must be a data frame")
  }
  if (!is.data.frame(y)) {
    cli::cli_abort("{.arg y} must be a data frame")
  }
  if (ncol(x) < 2) {
    cli::cli_abort("{.arg x} must have at least 2 columns (ID + predictors)")
  }

  colnames(x)[1] <- "ID"
  colnames(y)[1] <- "ID"
  x$ID <- as.character(x$ID)
  y$ID <- as.character(y$ID)

  if (type == "survival") {
    if (ncol(y) < 3) {
      cli::cli_abort("{.arg y} must have at least 3 columns for survival analysis (ID, time, status)")
    }
    colnames(y) <- c("ID", "time", "status")
    y <- dplyr::filter(y, .data$time > 0)
  }

  samples <- intersect(x$ID, y$ID)
  if (length(samples) == 0) {
    cli::cli_abort("No matching sample IDs found between x and y")
  }

  x <- x[match(samples, x$ID), ]
  y <- y[match(samples, y$ID), ]

  if (type == "binomial") {
    colnames(y) <- c("ID", "Group")
    y <- dplyr::pull(y, .data$Group)
    if (!is.factor(y)) {
      cli::cli_alert_info("Converting outcome to factor")
      y <- as.factor(y)
    }
  } else if (type == "survival") {
    y <- y[, c("time", "status")]
  }

  x_scale <- if (scale) {
    scale(x[, -1, drop = FALSE], center = TRUE, scale = TRUE)
  } else {
    as.matrix(x[, -1, drop = FALSE])
  }

  x_ID <- x[, "ID"]
  na_cols <- which(colSums(is.na(x_scale)) > 0)
  if (length(na_cols) > 0) {
    cli::cli_warn("Removing {length(na_cols)} column(s) with NA values")
    x_scale <- x_scale[, -na_cols, drop = FALSE]
  }

  list(x_scale = x_scale, y = y, x_ID = x_ID)
}

#' Regression Result Computation
#'
#' @description
#' Computes regression results with coefficients at lambda.min and lambda.1se,
#' and evaluates AUC for binomial outcomes. Returns a comprehensive summary
#' of model performance on both training and testing datasets.
#'
#' @param train.x Training predictors matrix.
#' @param train.y Training outcomes (binary factor).
#' @param test.x Testing predictors matrix.
#' @param test.y Testing outcomes (binary factor).
#' @param model Fitted cv.glmnet model object.
#'
#' @return List containing:
#' \describe{
#'   \item{model}{The fitted cv.glmnet model}
#'   \item{coefs}{Data frame with feature names and coefficients at lambda.min and lambda.1se}
#'   \item{AUC}{Matrix of AUC values for train/test sets at both lambda values}
#' }
#'
#' @export
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   set.seed(123)
#'   train_data <- matrix(rnorm(100 * 10), ncol = 10)
#'   train_outcome <- rbinom(100, 1, 0.5)
#'   test_data <- matrix(rnorm(50 * 10), ncol = 10)
#'   test_outcome <- rbinom(50, 1, 0.5)
#'   fitted_model <- glmnet::cv.glmnet(train_data, train_outcome, family = "binomial", nfolds = 5)
#'   results <- RegressionResult(
#'     train.x = train_data, train.y = train_outcome,
#'     test.x = test_data, test.y = test_outcome, model = fitted_model
#'   )
#' }
RegressionResult <- function(train.x, train.y, test.x, test.y, model) {
  if (!is.matrix(train.x) || !is.matrix(test.x)) {
    cli::cli_abort("train.x and test.x must be matrices")
  }

  coefs <- cbind(
    stats::coef(model, s = model$lambda.min),
    stats::coef(model, s = model$lambda.1se)
  )
  coefs <- data.frame(
    feature = rownames(coefs),
    lambda.min = coefs[, 1],
    lambda.1se = coefs[, 2]
  )

  datasets <- list(
    list(data = train.x, outcome = train.y, lambda = model$lambda.min),
    list(data = train.x, outcome = train.y, lambda = model$lambda.1se),
    list(data = test.x, outcome = test.y, lambda = model$lambda.min),
    list(data = test.x, outcome = test.y, lambda = model$lambda.1se)
  )

  aucs <- vapply(datasets, function(d) {
    BinomialAUC(model, d$data, d$lambda, d$outcome)
  }, numeric(1))

  AUC <- matrix(aucs,
    ncol = 2, byrow = TRUE,
    dimnames = list(c("train", "test"), c("lambda.min", "lambda.1se"))
  )

  list(model = model, coefs = coefs, AUC = AUC)
}

#' Elastic Net Model Fitting
#'
#' @description
#' Fits elastic net model with cross-validation to find optimal alpha and lambda.
#' Searches across a grid of alpha values (0 to 1) and lambda values to minimize
#' cross-validation error.
#'
#' @param train.x Training predictors matrix.
#' @param train.y Training binary outcomes (0/1 or factor).
#' @param lambdamax Maximum lambda value for the grid search.
#' @param nfold Number of CV folds. Default is `10`.
#'
#' @return List containing:
#' \describe{
#'   \item{chose_alpha}{Optimal alpha value (0-1)}
#'   \item{chose_lambda}{Optimal lambda value}
#' }
#'
#' @export
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   set.seed(123)
#'   train_data <- matrix(rnorm(50 * 5), ncol = 5)
#'   train_outcome <- rbinom(50, 1, 0.5)
#'   result <- Enet(train.x = train_data, train.y = train_outcome, lambdamax = 1, nfold = 5)
#' }
Enet <- function(train.x, train.y, lambdamax, nfold = 10) {
  train.x <- as.matrix(train.x)
  train.y <- as.numeric(train.y)

  if (nrow(train.x) != length(train.y)) {
    cli::cli_abort("Number of rows in train.x ({nrow(train.x)}) must equal length of train.y ({length(train.y)})")
  }
  if (lambdamax <= 0) {
    cli::cli_abort("{.arg lambdamax} must be greater than 0, got {lambdamax}")
  }
  if (nfold < 2) {
    cli::cli_abort("{.arg nfold} must be at least 2, got {nfold}")
  }

  alpha.grid <- seq(0, 1, by = 0.2)
  lambda.grid <- seq(1e-4, lambdamax, length.out = 10)

  res <- lapply(alpha.grid, function(a) {
    fit <- glmnet::cv.glmnet(
      x = train.x, y = train.y, family = "binomial",
      alpha = a, lambda = lambda.grid, nfolds = nfold, type.measure = "class"
    )
    data.frame(alpha = a, lambda = fit$lambda.min, cv_error = min(fit$cvm))
  })

  res <- do.call(rbind, res)
  best <- res[which.min(res$cv_error), ]

  list(chose_alpha = best$alpha, chose_lambda = best$lambda)
}

#' Calculate AUC for Binomial Model
#'
#' @description
#' Computes Area Under the ROC Curve (AUC) for model predictions using the
#' ROCR package. Handles binary classification models from glmnet.
#'
#' @param model Fitted glmnet model object.
#' @param newx New data matrix for prediction.
#' @param s Lambda value for prediction (e.g., "lambda.min" or numeric).
#' @param acture.y Actual binary outcomes (numeric 0/1 or factor).
#'
#' @return Numeric AUC value between 0 and 1.
#'
#' @export
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE) &&
#'   requireNamespace("ROCR", quietly = TRUE)) {
#'   set.seed(123)
#'   train_data <- matrix(rnorm(100 * 5), ncol = 5)
#'   train_outcome <- rbinom(100, 1, 0.5)
#'   test_data <- matrix(rnorm(50 * 5), ncol = 5)
#'   test_outcome <- rbinom(50, 1, 0.5)
#'   fitted_model <- glmnet::cv.glmnet(train_data, train_outcome, family = "binomial", nfolds = 5)
#'   auc_value <- BinomialAUC(fitted_model, test_data, fitted_model$lambda.min, test_outcome)
#'   print(auc_value)
#' }
BinomialAUC <- function(model, newx, s, acture.y) {
  rlang::check_installed("ROCR")
  prob <- stats::predict(model, newx = newx, s = s, type = "response")
  pred <- ROCR::prediction(prob, acture.y)
  as.numeric(ROCR::performance(pred, "auc")@y.values)
}

#' Plot AUC ROC Curves
#'
#' @description
#' Generates ROC curves for model evaluation comparing training and testing
#' performance at both lambda.min and lambda.1se. Creates a ggplot visualization
#' with AUC values in the legend.
#'
#' @param train.x Training predictors matrix.
#' @param train.y Training outcomes (binary factor).
#' @param test.x Testing predictors matrix.
#' @param test.y Testing outcomes (binary factor).
#' @param model Fitted cv.glmnet model.
#' @param modelname Character string for plot title.
#' @param cols Optional color vector for ROC curves.
#' @param palette Color palette name from IOBR palettes. Default is `"jama"`.
#'
#' @return ggplot object of ROC curves.
#'
#' @export
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE)) {
#'   set.seed(123)
#'   train_data <- matrix(rnorm(100 * 5), ncol = 5)
#'   train_outcome <- rbinom(100, 1, 0.5)
#'   test_data <- matrix(rnorm(50 * 5), ncol = 5)
#'   test_outcome <- rbinom(50, 1, 0.5)
#'   fitted_model <- glmnet::cv.glmnet(train_data, train_outcome, family = "binomial", nfolds = 5)
#'   p <- PlotAUC(train_data, train_outcome, test_data, test_outcome, fitted_model, "MyModel")
#'   print(p)
#' }
PlotAUC <- function(train.x, train.y, test.x, test.y, model, modelname,
                    cols = NULL, palette = "jama") {
  cols <- cols %||% palettes(
    category = "box", palette = palette,
    show_message = FALSE, show_col = FALSE
  )

  datasets <- list(
    list(data = train.x, outcome = train.y, lambda = model$lambda.min),
    list(data = train.x, outcome = train.y, lambda = model$lambda.1se),
    list(data = test.x, outcome = test.y, lambda = model$lambda.min),
    list(data = test.x, outcome = test.y, lambda = model$lambda.1se)
  )

  pref <- lapply(datasets, function(d) {
    CalculatePref(model, d$data, d$lambda, d$outcome)
  })

  aucs <- round(vapply(datasets, function(d) {
    BinomialAUC(model, d$data, d$lambda, d$outcome)
  }, numeric(1)), 2)

  legend.name <- paste(
    c("train_lambda.min", "train_lambda.1se", "test_lambda.min", "test_lambda.1se"),
    "AUC", aucs,
    sep = " "
  )
  names(pref) <- c("train_lambda.min", "train_lambda.1se", "test_lambda.min", "test_lambda.1se")

  plotdat <- do.call(rbind, lapply(names(pref), function(nm) {
    data.frame(
      s = nm,
      x = pref[[nm]]@x.values[[1]],
      y = pref[[nm]]@y.values[[1]]
    )
  }))

  plotdat$s <- factor(plotdat$s, levels = names(pref))

  ggplot2::ggplot(plotdat, ggplot2::aes(x = .data$x, y = .data$y, color = .data$s)) +
    ggplot2::geom_path(linewidth = 1) +
    ggplot2::geom_abline(intercept = 0, slope = 1, linetype = "dashed") +
    ggplot2::xlab("False positive rate") +
    ggplot2::ylab("True positive rate") +
    ggplot2::theme_bw() +
    ggplot2::scale_color_manual(values = cols, labels = legend.name) +
    ggplot2::ggtitle(stringr::str_replace(modelname, "_", " ")) +
    ggplot2::theme(
      legend.title = ggplot2::element_blank(),
      plot.title = ggplot2::element_text(size = ggplot2::rel(2), hjust = 0.5),
      axis.text.x = ggplot2::element_text(face = "plain", angle = 0, hjust = 1, color = "black"),
      axis.text.y = ggplot2::element_text(face = "plain", angle = 0, hjust = 1, color = "black")
    )
}

#' Calculate Performance Metrics
#'
#' @description
#' Computes True Positive Rate (TPR) and False Positive Rate (FPR) for ROC
#' analysis using the ROCR package. Used internally for ROC curve generation.
#'
#' @param model Fitted glmnet model.
#' @param newx New data matrix for prediction.
#' @param s Lambda value for prediction.
#' @param acture.y Actual binary outcomes.
#'
#' @return ROCR performance object containing TPR and FPR values.
#'
#' @export
#'
#' @examples
#' if (requireNamespace("glmnet", quietly = TRUE) &&
#'   requireNamespace("ROCR", quietly = TRUE)) {
#'   fitted_model <- glmnet::cv.glmnet(matrix(rnorm(100), ncol = 2), rbinom(50, 1, 0.5), nfolds = 3)
#'   perf <- CalculatePref(fitted_model, matrix(rnorm(20), ncol = 2), "lambda.min", rbinom(10, 1, 0.5))
#' }
CalculatePref <- function(model, newx, s, acture.y) {
  rlang::check_installed("ROCR")
  prob <- stats::predict(model, newx = newx, s = s, type = "response")
  pred <- ROCR::prediction(prob, acture.y)
  ROCR::performance(pred, "tpr", "fpr")
}

#' Split Data into Training and Testing Sets
#'
#' @description
#' Divides dataset into training and testing sets using random sampling.
#' Maintains data integrity for both binomial and survival analysis types.
#'
#' @param x Predictor matrix or data frame.
#' @param y Outcome vector (binomial) or matrix with time/status (survival).
#' @param train_ratio Proportion for training (0-1). Default is `0.7`.
#' @param type Analysis type: `"binomial"` or `"survival"`.
#' @param seed Random seed for reproducibility.
#'
#' @return List containing:
#' \describe{
#'   \item{train.x}{Training predictors matrix}
#'   \item{train.y}{Training outcomes}
#'   \item{test.x}{Testing predictors matrix}
#'   \item{test.y}{Testing outcomes}
#'   \item{train_sample}{Indices of training samples}
#' }
#'
#' @export
#'
#' @examples
#' data_matrix <- matrix(rnorm(200), ncol = 2)
#' outcome_vector <- rbinom(100, 1, 0.5)
#' split_data <- SplitTrainTest(
#'   data_matrix, outcome_vector,
#'   train_ratio = 0.7,
#'   type = "binomial", seed = 123
#' )
SplitTrainTest <- function(x, y, train_ratio, type = c("binomial", "survival"), seed) {
  type <- rlang::arg_match(type)

  if (train_ratio <= 0 || train_ratio >= 1) {
    cli::cli_abort("{.arg train_ratio} must be between 0 and 1, got {train_ratio}")
  }

  sizes <- round(nrow(x) * train_ratio)
  set.seed(seed)
  train_sample <- sample(seq_len(nrow(x)), size = sizes, replace = FALSE)
  test_sample <- setdiff(seq_len(nrow(x)), train_sample)

  train.x <- x[train_sample, , drop = FALSE]
  test.x <- x[test_sample, , drop = FALSE]

  if (type == "binomial") {
    train.y <- y[train_sample]
    test.y <- y[test_sample]
  } else if (type == "survival") {
    train.y <- y[train_sample, , drop = FALSE]
    test.y <- y[test_sample, , drop = FALSE]
  }

  list(
    train.x = train.x, train.y = train.y, test.x = test.x, test.y = test.y,
    train_sample = train_sample
  )
}

#' Calculate Ligand-Receptor Interaction Scores
#'
#' @description
#' Quantifies ligand-receptor interactions in the tumor microenvironment from
#' bulk gene expression data using the easier package. This function processes
#' raw counts or TPM data and computes interaction scores for each sample.
#'
#' @param eset Gene expression matrix with genes as rows and samples as columns.
#' @param data_type Type of input data. Options are `"count"` or `"tpm"`.
#'   If `"count"`, data will be converted to TPM before analysis.
#' @param id_type Type of gene identifier. Default is `"ensembl"`.
#' @param cancer_type Character string specifying the cancer type for easier.
#'   Default is `"pancan"` for pan-cancer analysis.
#'
#' @return Data frame containing ligand-receptor interaction scores with
#'   sample IDs as row names.
#'
#' @references
#' Lapuente-Santana, van Genderen, M., Hilbers, P., Finotello, F., & Eduati, F. (2021).
#' Interpretable systems biomarkers predict response to immune-checkpoint inhibitors.
#' Patterns (New York, N.Y.), 2(8), 100293. https://doi.org/10.1016/j.patter.2021.100293
#'
#' @export
#'
#' @examples
#' # LR_cal requires HGNC gene symbols as rownames
#' # Create a simple example with gene symbols
#' example_genes <- c(
#'   "TGFB1", "EGFR", "VEGFA", "PDGFB", "FGF2", "CXCL12",
#'   "CXCR4", "IL6", "IL6R", "TNF", "TNFRSF1A", "IFNG"
#' )
#' sim_eset <- as.data.frame(matrix(
#'   rnorm(length(example_genes) * 10, mean = 5, sd = 2),
#'   nrow = length(example_genes), ncol = 10
#' ))
#' rownames(sim_eset) <- example_genes
#' colnames(sim_eset) <- paste0("Sample", 1:10)
#' \dontrun{
#' if (requireNamespace("easier", quietly = TRUE)) {
#'   tryCatch({
#'     lr <- LR_cal(eset = sim_eset, data_type = "tpm")
#'     head(lr)
#'   }, error = function(e) {
#'     message("Example skipped: could not download ExperimentHub data")
#'   })
#' }
#' }
LR_cal <- function(eset, data_type = c("count", "tpm"), id_type = "ensembl", cancer_type = "pancan") {
  rlang::check_installed("easier")
  data_type <- rlang::arg_match(data_type)

  if (!is.matrix(eset) && !is.data.frame(eset)) {
    cli::cli_abort("{.arg eset} must be a matrix or data frame")
  }
  if (nrow(eset) == 0 || ncol(eset) == 0) {
    cli::cli_abort("{.arg eset} must have at least one row and one column")
  }

  if (data_type == "count") {
    cli::cli_alert_info("Converting count data to TPM...")
    eset <- count2tpm(countMat = eset, idType = id_type, source = "local")
  }

  eset <- as.matrix(eset)
  feas <- feature_manipulation(data = eset, feature = rownames(eset), is_matrix = TRUE)
  eset <- eset[rownames(eset) %in% feas, , drop = FALSE]

  if (nrow(eset) == 0) {
    cli::cli_abort("No valid features remaining after filtering")
  }

  res <- easier::compute_LR_pairs(RNA_tpm = eset, cancer_type = cancer_type, verbose = TRUE)

  tibble::rownames_to_column(res, var = "ID")
}

# R/globalVariables.R
# Purpose: Suppress "no visible binding for global variable" NOTES from R CMD check
# Usage: Place in R/ directory; automatically sourced, no @export needed

#' Global Variables Declaration
#'
#' @description
#' This file declares global variables used in the IOBR package to suppress
#' R CMD check NOTES about "no visible binding for global variable". These
#' variables are primarily used in tidyverse/dplyr pipelines and data.table
#' operations where non-standard evaluation (NSE) makes static analysis difficult.
#'
#' @details
#' The variables listed here fall into several categories:
#' - tidyverse NSE variables: column names used in dplyr/data.table pipelines
#' - Internal data: package datasets referenced in functions
#' - Cross-package S3 methods: generic functions from suggested packages
#'
#' @keywords internal
#' @noRd
utils::globalVariables(c(
  # ---------- tidyverse / data.table internal variables ----------
  ".", "..p.format..", "value", "x", "y", "p.value", "p.adj", "padj",
  "label", "group1", "group2", "mean_group1", "mean_group2",
  "Estimate", "logFC", "adj.P.Val", "auc", "freq", "Freq",
  "percent_weight", "lab.ypos", "Prop", "P", "cell_type", "fraction",
  "ID", "ID1", "ID2", "Intensity", "Sample.Name", "Sample.Num",
  "Z.score", "title", "target_group", "sig_group", "variables",
  "idd", "goup3", "categorys", "point.alpha", "mutation",
  "entrez_gene", "gs_cat", "gs_name", "gs_subcat",
  "signature_collection",
  "mcp_genes", "mcp_probesets",
  "null_models",
  "my_palette", "my_palette2", "Group", "ENTREZID", "logfc", "p",
  "FPR", "TPR", "variable", "itor", "s",
  "y1", "y2", "avg_log2FC",

  # ---------- Cross-package functions / S3 methods ----------
  # Names called via :: or get() but still flagged by codetools

  # ---------- Variables added during code quality improvements ----------
  "main", "p.format", ".break_month", "draw", "coef", "Mode", "tb"
))

#' Identify High-Variance Features from Statistical Results
#'
#' @description
#' Selects top variable (up- and down-regulated) features based on adjusted
#' p-value and log fold-change thresholds.
#'
#' @param result Data frame or tibble. Statistical results containing feature,
#'   adjusted p-value, and logFC columns.
#' @param target Character. Column name of feature identifiers.
#' @param name_padj Character. Adjusted p-value column name. Default is `"padj"`.
#' @param padj_cutoff Numeric. Adjusted p-value threshold. Default is 1.
#' @param name_logfc Character. log2 fold-change column name.
#' @param logfc_cutoff Numeric. Absolute log2 fold-change threshold. Default is 0.
#' @param n Integer. Number of top up and top down features to select.
#'   Default is 10.
#' @param data_type Character or `NULL`. If `"survival"`, adjusts logFC
#'   interpretation. Default is `NULL`.
#'
#' @return Character vector of selected feature names (combined up and down sets).
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' result_data <- data.frame(
#'   gene = c("Gene1", "Gene2", "Gene3", "Gene4", "Gene5"),
#'   padj = c(0.01, 0.02, 0.05, 0.001, 0.03),
#'   logfc = c(-2, 1.5, -3, 2.5, 0.5)
#' )
#' high_var_fea(
#'   result = result_data,
#'   target = "gene",
#'   name_padj = "padj",
#'   name_logfc = "logfc",
#'   n = 2,
#'   padj_cutoff = 0.05,
#'   logfc_cutoff = 1.5
#' )
high_var_fea <- function(result, target, name_padj = "padj", padj_cutoff = 1,
                         name_logfc, logfc_cutoff = 0, n = 10, data_type = NULL) {
  if (!is.data.frame(result)) {
    cli::cli_abort("{.arg result} must be a data frame")
  }
  if (!target %in% colnames(result)) {
    cli::cli_abort("Column {.val {target}} not found in {.arg result}")
  }
  if (!name_padj %in% colnames(result)) {
    cli::cli_abort("Column {.val {name_padj}} not found in {.arg result}")
  }
  if (!name_logfc %in% colnames(result)) {
    cli::cli_abort("Column {.val {name_logfc}} not found in {.arg result}")
  }

  colnames(result)[colnames(result) == name_padj] <- "padj"
  colnames(result)[colnames(result) == name_logfc] <- "logfc"
  colnames(result)[colnames(result) == target] <- "target"

  if (!is.null(data_type) && data_type == "survival") {
    result[, "logfc"] <- result[, "logfc"] - 1
  }

  topVarFeature1 <- result %>%
    dplyr::filter(.data$padj < padj_cutoff) %>%
    dplyr::arrange(.data$padj) %>%
    dplyr::filter(.data$logfc < -abs(logfc_cutoff)) %>%
    dplyr::select(.data$target, .data$padj, .data$logfc)

  topVarFeature2 <- result %>%
    dplyr::filter(.data$padj < padj_cutoff) %>%
    dplyr::arrange(.data$padj) %>%
    dplyr::filter(.data$logfc > abs(logfc_cutoff)) %>%
    dplyr::select(.data$target, .data$padj, .data$logfc)

  if (nrow(topVarFeature1) < n) {
    cli::cli_alert_warning(
      "Cutoff too strict for down-regulated features, only {nrow(topVarFeature1)} found"
    )
  }
  if (nrow(topVarFeature2) < n) {
    cli::cli_alert_warning(
      "Cutoff too strict for up-regulated features, only {nrow(topVarFeature2)} found"
    )
  }
  if (nrow(topVarFeature1) == 0) {
    cli::cli_alert_warning("No down-regulated features found")
  }
  if (nrow(topVarFeature2) == 0) {
    cli::cli_alert_warning("No up-regulated features found")
  }

  topVarFeature <- c(
    as.character(topVarFeature1$target)[seq_len(min(n, nrow(topVarFeature1)))],
    as.character(topVarFeature2$target)[seq_len(min(n, nrow(topVarFeature2)))]
  )
  topVarFeature <- topVarFeature[!is.na(topVarFeature)]

  topVarFeature
}

#' Generate Heatmap for Signature Data
#'
#' @description
#' Creates a heatmap from signature data with grouping variables, offering
#' flexible options for colors, clustering, and output formats using
#' ComplexHeatmap.
#'
#' @param input Data frame with variables in columns.
#' @param feas Character vector. Feature names (columns) to include in heatmap.
#' @param group Character string. Column name for primary grouping variable.
#' @param group2 Character string or `NULL`. Optional secondary grouping variable.
#' @param group3 Character string or `NULL`. Optional tertiary grouping variable.
#' @param ID Character string. Column name for sample identifiers. Default is `"ID"`.
#' @param path Character string or `NULL`. Directory to save output files.
#'   If `NULL`, the heatmap is not saved. Default is `NULL`.
#' @param cols1 Character vector or `"random"` or `"normal"`. Colors for primary group.
#'   Default is `"random"`.
#' @param seed Integer. Random seed for color generation. Default is `54321`.
#' @param show_col Logical indicating whether to display colors. Default is `FALSE`.
#' @param cluster_cols Logical indicating whether to cluster columns. Default is `TRUE`.
#' @param palette_for_heatmape Integer. Palette number for heatmap. Default is `6`.
#' @param scale.matrix Logical indicating whether to scale the matrix. Default is `TRUE`.
#' @param cellwidth Numeric. Width of each cell in points. Default is `1`.
#' @param cellheight Numeric. Height of each cell in points. Default is `9`.
#' @param fig.type Character string. File format for saving. Default is `"pdf"`.
#' @param width Numeric. Width of saved figure in inches. Default is `6`.
#' @param height Numeric or `NULL`. Height of saved figure in inches. Calculated if `NULL`.
#' @param file_name_prefix Character or numeric. Prefix for saved file name. Default is `1`.
#' @param cols2 Character vector or `"random"` or `"normal"`. Colors for secondary group.
#'   Default is `"random"`.
#' @param cols3 Character vector or `"random"` or `"normal"`. Colors for tertiary group.
#'   Default is `"random"`.
#' @param palette1 Integer. Palette for primary group. Default is `1`.
#' @param palette2 Integer. Palette for secondary group. Default is `2`.
#' @param palette3 Integer. Palette for tertiary group. Default is `3`.
#' @param show_colnames Logical indicating whether to show column names. Default is `FALSE`.
#'
#' @return A list containing:
#' \describe{
#'   \item{p_anno}{Annotation data frame}
#'   \item{p_cols}{List of cluster colors}
#'   \item{plot}{ComplexHeatmap object}
#'   \item{eset}{Transformed expression matrix}
#' }
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' test_input <- data.frame(
#'   ID = paste0("Sample", 1:20),
#'   subtype = sample(c("TypeA", "TypeB"), 20, replace = TRUE),
#'   Sig1 = rnorm(20),
#'   Sig2 = rnorm(20)
#' )
#' sig_pheatmap(
#'   input = test_input,
#'   feas = c("Sig1", "Sig2"),
#'   group = "subtype"
#' )
sig_pheatmap <- function(input,
                         feas,
                         group,
                         group2 = NULL,
                         group3 = NULL,
                         ID = "ID",
                         path = NULL,
                         cols1 = "random",
                         cols2 = "random",
                         cols3 = "random",
                         seed = 54321,
                         show_col = FALSE,
                         palette1 = 1,
                         palette2 = 2,
                         palette3 = 3,
                         cluster_cols = TRUE,
                         palette_for_heatmape = 6,
                         scale.matrix = TRUE,
                         cellwidth = 1,
                         cellheight = 9,
                         show_colnames = FALSE,
                         fig.type = "pdf",
                         width = 6,
                         height = NULL,
                         file_name_prefix = 1) {
  if (is.null(input)) return(NULL)
  rlang::check_installed("ComplexHeatmap")
  rlang::check_installed("grid")

  # Create output directory
  save_results <- !is.null(path)
  if (save_results) {
    path_obj <- creat_folder(path)
  } else {
    path_obj <- NULL
  }

  # Prepare input data
  input <- as.data.frame(input)
  feas <- feas[feas %in% colnames(input)]

  if (length(feas) == 0) {
    cli::cli_abort("No valid features found in input")
  }

  if (!ID %in% colnames(input)) {
    cli::cli_abort("ID column {.val {ID}} not found in input")
  }

  if (!group %in% colnames(input)) {
    cli::cli_abort("Group column {.val {group}} not found in input")
  }

  # Create expression matrix
  eset <- input[, c(ID, feas), drop = FALSE]
  rownames(eset) <- NULL
  eset <- tibble::column_to_rownames(eset, var = ID)

  if (scale.matrix) {
    eset <- scale(eset, scale = TRUE, center = TRUE)
  }
  eset <- t(eset)

  # Create annotation data frame
  anno_cols <- c(ID, group)
  if (!is.null(group2)) anno_cols <- c(anno_cols, group2)
  if (!is.null(group3)) anno_cols <- c(anno_cols, group3)

  anno <- input[, anno_cols, drop = FALSE]
  rownames(anno) <- anno[[ID]]
  anno[[ID]] <- NULL
  anno[] <- lapply(anno, as.character)

  # Get heatmap palette
  mapal <- palettes(
    category = "heatmap",
    palette = palette_for_heatmape,
    counts = 200,
    show_col = show_col
  )

  # Calculate height
  if (is.null(height)) {
    height <- 2 + length(feas) * 0.25
  }

  # Generate group colors
  cluster_colors <- list()

  # Primary group colors
  mycols1 <- .get_group_colors(cols1, palette1, seed, show_col)
  lev1 <- unique(as.character(input[[group]]))
  lev1 <- lev1[!is.na(lev1)]
  cluster_colors[[group]] <- stats::setNames(mycols1[seq_along(lev1)], lev1)

  # Secondary group colors
  if (!is.null(group2)) {
    if (!group2 %in% colnames(input)) {
      cli::cli_abort("Group2 column {.val {group2}} not found in input")
    }
    mycols2 <- .get_group_colors(cols2, palette2, seed, show_col)
    lev2 <- unique(as.character(input[[group2]]))
    lev2 <- lev2[!is.na(lev2)]
    cluster_colors[[group2]] <- stats::setNames(mycols2[seq_along(lev2)], lev2)
  }

  # Tertiary group colors
  if (!is.null(group3)) {
    if (!group3 %in% colnames(input)) {
      cli::cli_abort("Group3 column {.val {group3}} not found in input")
    }
    mycols3 <- .get_group_colors(cols3, palette3, seed, show_col)
    lev3 <- unique(as.character(input[[group3]]))
    lev3 <- lev3[!is.na(lev3)]
    cluster_colors[[group3]] <- stats::setNames(mycols3[seq_along(lev3)], lev3)
  }

  if (interactive()) print(cluster_colors)

  # Create heatmap
  mat <- as.matrix(eset)
  col_fun <- grDevices::colorRampPalette(mapal)

  # Column annotation
  ha_top <- NULL
  if (!is.null(anno) && ncol(anno) > 0) {
    ha_top <- ComplexHeatmap::HeatmapAnnotation(
      df = anno,
      col = cluster_colors,
      annotation_name_gp = grid::gpar(fontsize = 6)
    )
  }

  # Draw heatmap
  ht <- ComplexHeatmap::Heatmap(
    mat,
    name = "value",
    col = col_fun(256),
    cluster_rows = TRUE,
    cluster_columns = cluster_cols,
    show_row_names = TRUE,
    show_column_names = show_colnames,
    row_names_gp = grid::gpar(fontsize = 6),
    column_names_gp = grid::gpar(fontsize = 6, angle = 45),
    top_annotation = ha_top,
    column_title = NULL,
    row_title = NULL
  )

  # Save output
  if (save_results) {
    outfile <- file.path(
      path_obj$folder_name,
      paste0(file_name_prefix, "-pheatmap-", group, ".", fig.type)
    )

    if (fig.type == "pdf") {
      grDevices::pdf(outfile, width = width, height = height)
    } else if (fig.type %in% c("png", "jpg", "jpeg", "tiff")) {
      # Handle other formats if needed
      cli::cli_alert_warning("Only PDF format is fully supported; saving as PDF")
      grDevices::pdf(outfile, width = width, height = height)
    }

    ComplexHeatmap::draw(ht)
    grDevices::dev.off()

    cli::cli_alert_success("Heatmap saved to: {.path {outfile}}")
  }

  list(
    p_anno = anno,
    p_cols = cluster_colors,
    plot = ht,
    eset = eset
  ) %>% invisible()
}

#' Get Group Colors
#' @keywords internal
#' @noRd
.get_group_colors <- function(cols, palette, seed, show_col) {
  if (length(cols) == 1 && cols %in% c("random", "normal")) {
    mycols <- palettes(
      category = "random",
      palette = palette,
      show_col = FALSE,
      show_message = FALSE
    )

    if (cols == "random") {
      cli::cli_alert_info("Using random seed: {seed}")
      set.seed(seed)
      mycols <- sample(mycols)
      if (show_col) {
        rlang::check_installed("scales")
        scales::show_col(mycols)
      }
    }
    mycols
  } else {
    if (show_col) {
      rlang::check_installed("scales")
      scales::show_col(cols)
    }
    cols
  }
}

#' Generate Reference Signature Matrix
#'
#' @description
#' Generates a reference signature matrix for cell types based on
#' differential expression analysis. Supports both limma for normalized data
#' and DESeq2 for raw count data.
#'
#' @param dds Matrix. Raw count data from RNA-seq. Required if
#'   `method = "DESeq2"`.
#' @param pheno Character vector. Cell type class of the samples.
#' @param FDR Numeric. Genes with BH adjusted p-value < FDR are considered
#'   significant. Default is 0.05.
#' @param dat Matrix or data frame. Normalized transcript quantification data
#'   (e.g., FPKM, TPM).
#' @param method Character. Method for differential expression: `"limma"` or
#'   `"DESeq2"`. Default is `"limma"`.
#'
#' @return List containing:
#'   - `reference_matrix`: Data frame of median expression for significant
#'     genes across cell types.
#'   - `G`: Optimal number of probes minimizing condition number.
#'   - `condition_number`: Minimum condition number.
#'   - `whole_matrix`: Full median expression matrix.
#'
#' @export
#'
#' @examples
#' expressionData <- matrix(runif(1000 * 4, min = 0, max = 10), ncol = 4)
#' rownames(expressionData) <- paste("Gene", 1:1000, sep = "_")
#' colnames(expressionData) <- paste("Sample", 1:4, sep = "_")
#'
#' phenotype <- c("celltype1", "celltype2", "celltype1", "celltype2")
#'
#' rawCountData <- matrix(sample(1:100, 1000 * 4, replace = TRUE), ncol = 4)
#' rownames(rawCountData) <- paste("Gene", 1:1000, sep = "_")
#' colnames(rawCountData) <- paste("Sample", 1:4, sep = "_")
#'
#' \donttest{
#' result <- generateRef(
#'   dds = rawCountData, pheno = phenotype,
#'   FDR = 0.05, dat = expressionData, method = "DESeq2"
#' )
#' }
generateRef <- function(dds, pheno, FDR = 0.05, dat, method = "limma") {
  method <- rlang::arg_match(method, c("limma", "DESeq2"))

  cli::cli_alert_info("Running differentially expressed genes using {method}")

  res <- switch(method,
    limma = generateRef_limma(dat, pheno, FDR),
    DESeq2 = generateRef_DEseq2(dat, pheno, FDR, dds)
  )

  ref <- res$reference_matrix
  res$reference_matrix <- ref[, -1, drop = FALSE]

  res
}

#' Select a Signature Scoring Method Subset
#'
#' @description
#' Filters an integrated signature score matrix to retain results from a
#' specified method (PCA, ssGSEA, or zscore) and strips method suffixes from
#' column names.
#'
#' @param data Data frame or matrix. Integrated signature score matrix.
#' @param method Character. One of "PCA", "ssGSEA", or "zscore" (case-insensitive).
#'   Default is "ssGSEA".
#'
#' @return Matrix or data frame containing only the selected method's scores.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' # Simulate data with multiple method columns
#' set.seed(123)
#' sim_res <- data.frame(
#'   ID = paste0("Sample", 1:10),
#'   Sig1_PCA = rnorm(10),
#'   Sig1_ssGSEA = rnorm(10),
#'   Sig1_zscore = rnorm(10),
#'   Sig2_PCA = rnorm(10),
#'   Sig2_ssGSEA = rnorm(10),
#'   Sig2_zscore = rnorm(10)
#' )
#' # Select PCA method columns only
#' pca_res <- select_method(sim_res, method = "PCA")
#' print(colnames(pca_res))
select_method <- function(data, method = c("ssGSEA", "PCA", "zscore")) {
  method <- tolower(rlang::arg_match(method))

  if (!is.data.frame(data) && !is.matrix(data)) {
    cli::cli_abort("{.arg data} must be a data frame or matrix.")
  }

  patterns_to_remove <- switch(method,
    "ssgsea" = c("_PCA", "_zscore"),
    "pca"    = c("_ssGSEA", "_zscore"),
    "zscore" = c("_ssGSEA", "_PCA")
  )

  suffix_to_remove <- switch(method,
    "ssgsea" = "_ssGSEA",
    "pca"    = "_PCA",
    "zscore" = "_zscore"
  )

  for (pattern in patterns_to_remove) {
    data <- data[, !grepl(colnames(data), pattern = pattern), drop = FALSE]
  }

  colnames(data) <- gsub(colnames(data), pattern = suffix_to_remove, replacement = "")

  data
}

#' Combine Phenotype Data and Expression Set
#'
#' @description
#' Merges phenotype data with an expression matrix by matching sample IDs.
#' Optionally filters features, applies feature manipulation, and scales
#' expression data before combining.
#'
#' @param eset Expression matrix with genes/features in rows and samples in
#'   columns.
#' @param pdata Data frame containing phenotype/clinical data.
#' @param id_pdata Character string specifying the column name in `pdata`
#'   containing sample identifiers. Default is `"ID"`.
#' @param feas Character vector specifying features to include from `eset`.
#'   If `NULL`, all features are used. Default is `NULL`.
#' @param feature_manipulation Logical indicating whether to apply feature
#'   manipulation to filter valid features. Default is `TRUE`.
#' @param scale Logical indicating whether to scale (standardize) expression
#'   data. Default is `TRUE`.
#' @param choose_who_when_duplicate Character string specifying which data to
#'   prefer when duplicate columns exist. Options are `"eset"` or `"pdata"`.
#'   Default is `"eset"`.
#'
#' @return Data frame combining phenotype data and (transposed) expression data,
#'   with samples in rows and features/phenotypes in columns.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
#' rownames(eset) <- paste0("Gene", 1:100)
#' colnames(eset) <- paste0("Sample", 1:10)
#' pdata <- data.frame(
#'   ID = colnames(eset),
#'   group = rep(c("A", "B"), each = 5),
#'   age = rnorm(10, 50, 10)
#' )
#' result <- combine_pd_eset(eset = eset, pdata = pdata, scale = FALSE)
#' dim(result)
combine_pd_eset <- function(eset,
                            pdata,
                            id_pdata = "ID",
                            feas = NULL,
                            feature_manipulation = TRUE,
                            scale = TRUE,
                            choose_who_when_duplicate = c("eset", "pdata")) {
  # Validate arguments
  choose_who_when_duplicate <- rlang::arg_match(choose_who_when_duplicate)

  # Filter features if specified
  if (!is.null(feas)) {
    feas <- feas[feas %in% rownames(eset)]
    if (length(feas) == 0) {
      cli::cli_abort("No specified features found in eset")
    }
    eset <- eset[feas, , drop = FALSE]
    cli::cli_alert_info("Filtered to {nrow(eset)} features")
  }

  # Apply feature manipulation
  if (feature_manipulation) {
    feas_filter <- IOBR::feature_manipulation(
      data = eset,
      feature = rownames(eset),
      is_matrix = TRUE,
      print_result = FALSE
    )
    eset <- eset[feas_filter, , drop = FALSE]
    cli::cli_alert_info("Retained {nrow(eset)} features after QC")
  }

  # Transpose and scale
  eset <- t(eset)
  if (scale) {
    eset <- scale(eset, center = TRUE, scale = TRUE)
  }

  # Convert to data frame with ID column
  eset_df <- as.data.frame(eset)
  eset_df$ID <- rownames(eset_df)

  # Check pdata ID column
  if (!id_pdata %in% colnames(pdata)) {
    cli::cli_abort("Column {.val {id_pdata}} not found in pdata")
  }
  pdata_df <- as.data.frame(pdata)
  colnames(pdata_df)[colnames(pdata_df) == id_pdata] <- "ID"

  # Merge data frames
  choose <- ifelse(choose_who_when_duplicate == "eset", "x", "y")

  pd_eset <- merge_duplicate(
    pdata_df,
    eset_df,
    by.x = "ID",
    by.y = "ID",
    choose = choose,
    all = FALSE
  )

  cli::cli_alert_success(
    "Combined data: {nrow(pd_eset)} samples x {ncol(pd_eset)} variables"
  )

  pd_eset
}

#' Set and View Color Palettes
#'
#' @description
#' Retrieves color palettes from the IOBR package with options for randomization
#' and visualization. Users can specify predefined palettes or provide custom
#' colors.
#'
#' @param cols Character vector of colors, or one of:
#'   - `"normal"`: Use standard palette
#'   - `"random"`: Randomly shuffle the palette
#'   Default is `"normal"`.
#' @param palette Numeric or character specifying the palette. Options are
#'   1, 2, 3, 4, or palette name. Default is 1.
#' @param show_col Logical indicating whether to display the color palette.
#'   Default is `TRUE`.
#' @param seed Integer seed for random number generator when `cols = "random"`.
#'   Default is 123.
#'
#' @return Character vector of colors.
#'
#' @export
#'
#' @examples
#' # Get default palette
#' mycols <- get_cols()
#'
#' # Get random palette
#' mycols <- get_cols(cols = "random", seed = 456)
#'
#' # Use custom colors
#' mycols <- get_cols(cols = c("red", "blue", "green"))
get_cols <- function(cols = "normal",
                     palette = 1,
                     show_col = TRUE,
                     seed = 123) {
  # Map numeric palette to name
  palette_map <- c(
    "1" = "nrc",
    "2" = "jama",
    "3" = "aaas",
    "4" = "jco",
    "5" = "paired1",
    "6" = "paired2",
    "7" = "paired3",
    "8" = "paired4",
    "9" = "accent",
    "10" = "set2"
  )

  # Convert numeric palette to name
  if (is.numeric(palette) || grepl("^[0-9]+$", as.character(palette))) {
    palette_key <- as.character(as.numeric(palette))
    if (palette_key %in% names(palette_map)) {
      palette <- palette_map[palette_key]
    } else {
      palette <- "nrc" # default fallback
    }
  }

  # Handle custom color vector
  if (length(cols) > 1) {
    mycols <- cols
    if (show_col) {
      rlang::check_installed("scales")
      scales::show_col(mycols)
    }
    return(mycols)
  }

  # Handle single character specification
  cols <- as.character(cols)

  if (cols == "random") {
    # Get palette and shuffle
    mycols <- palettes(
      category = "random",
      palette = palette,
      show_col = FALSE,
      show_message = FALSE
    )

    cli::cli_alert_info("Using random seed: {seed}")
    set.seed(seed)
    mycols <- sample(mycols)

    if (show_col) {
      rlang::check_installed("scales")
      scales::show_col(mycols)
    }
  } else if (cols == "normal") {
    mycols <- palettes(
      category = "box",
      palette = palette,
      show_col = show_col,
      show_message = FALSE
    )
  } else {
    # Treat as palette name
    mycols <- palettes(
      category = "box",
      palette = cols,
      show_col = show_col,
      show_message = FALSE
    )
  }

  mycols
}

#' Generate Reference Signature Matrix Using DESeq2
#'
#' @description
#' Uses DESeq2 to perform differential expression analysis across cell types,
#' identifies significantly expressed genes, and creates a reference signature
#' matrix from median expression levels.
#'
#' @param dds Matrix. Raw count data from RNA-seq.
#' @param pheno Character vector. Cell type classes for samples.
#' @param FDR Numeric. Threshold for adjusted p-values. Default is 0.05.
#' @param dat Matrix. Normalized expression data (e.g., FPKM, TPM) for
#'   calculating median expression.
#'
#' @return List containing:
#'   - `reference_matrix`: Data frame of median expression for significant
#'     genes across cell types.
#'   - `G`: Optimal number of probes minimizing condition number.
#'   - `condition_number`: Minimum condition number.
#'   - `whole_matrix`: Full median expression matrix.
#'
#' @export
#'
#' @examples
#' set.seed(123)
#' dds <- matrix(sample(0:1000, 2000, replace = TRUE), nrow = 100, ncol = 20)
#' colnames(dds) <- paste("Sample", 1:20, sep = "_")
#' rownames(dds) <- paste("Gene", 1:100, sep = "_")
#' pheno <- rep(c("Type1", "Type2"), each = 10)
#' dat <- matrix(runif(2000), nrow = 100, ncol = 20)
#' rownames(dat) <- rownames(dds)
#' colnames(dat) <- colnames(dds)
#' \donttest{
#' result <- generateRef_DEseq2(dds = dds, pheno = pheno, FDR = 0.05, dat = dat)
#' print(result$reference_matrix)
#' }
generateRef_DEseq2 <- function(dds, pheno, FDR = 0.05, dat) {
  rlang::check_installed("DESeq2")

  if (!is.matrix(dds) && !is.data.frame(dds)) {
    cli::cli_abort("{.arg dds} must be a matrix or data frame")
  }
  if (!all(colnames(dds) == colnames(dat))) {
    cli::cli_abort("Sample order of {.arg dds} and {.arg dat} must be identical")
  }
  if (length(pheno) != ncol(dds)) {
    cli::cli_abort("Length of {.arg pheno} must match number of columns in {.arg dds}")
  }

  dds <- round(dds, 0)
  keep <- rowSums(dds) >= 0.05 * ncol(dds)
  dds <- dds[keep, ]
  dds <- stats::na.omit(dds)

  samples <- data.frame(row.names = colnames(dds), cell = pheno)
  ncells <- unique(pheno)
  res <- vector("list", length(ncells))

  for (i in seq_along(ncells)) {
    cellA <- ncells[i]
    samples$group <- ifelse(samples$cell == cellA, cellA, "others")
    dds2 <- DESeq2::DESeqDataSetFromMatrix(
      dds,
      colData = samples,
      design = ~group
    )
    dds2 <- DESeq2::estimateSizeFactors(dds2, type = "poscounts")
    dds2 <- DESeq2::DESeq(dds2)
    res[[i]] <- DESeq2::results(dds2, contrast = c("group", cellA, "others"))
  }

  resData <- lapply(res, function(x) {
    tmp <- as.data.frame(x[order(x$padj), ])
    tmp <- data.frame(probe = rownames(tmp), tmp)
    tmp <- tmp[tmp$padj < FDR, ]
    tmp
  })

  median_value <- t(dat) %>%
    as.data.frame() %>%
    split(pheno) %>%
    purrr::map(function(x) apply(as.matrix(x), 2, stats::median, na.rm = TRUE))
  median_value <- do.call(cbind, median_value)
  rownames(median_value) <- rownames(dat)

  con_nums <- numeric(151)
  for (i in 50:200) {
    probes <- resData %>%
      purrr::map(function(x) Top_probe(dat = x, i = i)) %>%
      unlist() %>%
      unique()
    tmpdat <- median_value[probes, , drop = FALSE]
    con_nums[i - 49] <- kappa(tmpdat)
  }

  i <- 49 + which.min(con_nums)
  probes <- resData %>%
    purrr::map(function(x) Top_probe(dat = x, i = i)) %>%
    unlist() %>%
    unique()
  reference <- median_value[probes, , drop = FALSE]
  reference <- data.frame(NAME = rownames(reference), reference)
  G <- i
  condition_number <- min(con_nums)

  list(
    reference_matrix = reference,
    G = G,
    condition_number = condition_number,
    whole_matrix = median_value
  )
}


#' Top Probe Selector
#'
#' @description
#' Extracts the top `i` probes based on their ordering in the provided data
#' frame. If the number of rows is less than or equal to `i`, returns all
#' probes.
#'
#' @param dat Data frame containing a column named "probe".
#' @param i Integer. Number of top probes to return.
#'
#' @return Character vector containing the names of the top `i` probes.
#'
#' @export
#'
#' @examples
#' dat <- data.frame(
#'   probe = c("Probe1", "Probe2", "Probe3", "Probe4", "Probe5"),
#'   value = c(5, 3, 2, 4, 1)
#' )
#' top_probes <- Top_probe(dat, 3)
#' print(top_probes)
Top_probe <- function(dat, i) {
  if (nrow(dat) <= i) {
    probe <- as.character(dat[, "probe"])
  } else {
    probe <- as.character(dat[seq_len(i), "probe"])
  }
  probe
}

# ####################################'
#
# Package EPIC - developed by Julien Racle from David Gfeller's group of the
# University of Lausanne and Ludwig Institute for Cancer Research. Copyrights
# remain reserved as described in the included LICENSE file.
#
# ####################################'

#' Estimate the proportion of immune and cancer cells.
#'
#' \code{EPIC} takes as input bulk gene expression data (RNA-seq) and returns
#'  the proportion of mRNA and cells composing the various samples.
#'
#' This function uses a constrained least square minimization to estimate the
#' proportion of each cell type with a reference profile and another
#' uncharacterized cell type in bulk gene expression samples.
#'
#' The names of the genes in the bulk samples, the reference samples and in the
#' gene signature list need to be the same format (gene symbols are used in the
#' predefined reference profiles). The full list of gene names don't need to be
#' exactly the same between the reference and bulk samples: \emph{EPIC}
#' will use the intersection of the genes. In case of duplicate gene names,
#' \emph{EPIC} will use the median value per duplicate - if you want to consider
#' these cases differently, you can remove the duplicates before calling
#' \emph{EPIC}.
#'
#' @param bulk A matrix (\code{nGenes} x \code{nSamples}) of the genes
#'    expression from each bulk sample (the counts should be given in TPM,
#'    RPKM or FPKM when using the prebuilt reference profiles). This matrix
#'    needs to have rownames telling the gene names (corresponds to the gene
#'    symbol in the prebuilt reference profiles (e.g. CD8A, MS4A1) - no
#'    conversion of IDs is performed at the moment).
#'    It is advised to keep all genes in the bulk instead of a subset of
#'    signature genes (except if \code{scaleExprs = FALSE} in which case it
#'    doesn't make any difference).
#' @param reference (optional): A string or a list defining the reference cells.
#'    It can take multiple formats:
#'    - `NULL`: to use the default reference profiles and genes signature
#'    \code{TRef}
#'    - A character: either \code{"BRef"} or \code{"TRef"} to use the
#'    reference cells and genes signature of the corresponding datasets (see
#'    \code{BRef} and \code{TRef})
#'    - A list containing:
#'      - `$refProfiles`: a matrix (\code{nGenes} x \code{nCellTypes}) of the
#'      reference cells genes expression (don't include a column of the 'other
#'      cells' (representing usually the cancer cells for which such a profile
#'      is usually not conserved between samples); the rownames needs to be
#'      defined as well as the colnames giving the names of each gene and
#'      reference cell types respectively. It is advised to keep all genes in
#'      this \code{refProfiles} matrix instead of a subset of signature genes
#'      - `$sigGenes`: a character vector of the gene names to use as
#'      signature - sigGenes can also be given as a direct input to EPIC
#'      function
#'      - `$refProfiles.var` (optional): a matrix (\code{nGenes} x
#'      \code{nCellTypes}) of the variability of each gene expression for each
#'      cell type, which is used to define weights on each gene for the
#'      optimization (if this is absent, we assume an identical variability
#'      for all genes in all cells) - it needs to have the same dimnames than
#'      refProfiles
#' @param mRNA_cell (optional): A named numeric vector: tells (in arbitrary
#'    units) the amount of mRNA for each of the reference cells and of the
#'    other uncharacterized (cancer) cell. Two names are of special meaning:
#'    \emph{"otherCells"} - used for the mRNA/cell value of the "other cells"
#'    from the sample (i.e. the cell type that don't have any reference gene
#'    expression profile) ; and \emph{default} - used for the mRNA/cell of the
#'    cells from the reference profiles for which no specific value is given
#'    in mRNA_cell (i.e. if mRNA_cell=c(Bcells=2, NKcells=2.1, otherCells=3.5,
#'    default=1), then if the refProfiles described Bcells, NKcells and Tcells,
#'    we would use a value of 3.5 for the "otherCells" that didn't have any
#'    reference profile and a default value of 1 for the Tcells when computing
#'    the cell fractions).
#'    To note: if data is in tpm, this mRNA per cell would ideally correspond
#'    to some number of transcripts per cell.
#' @param mRNA_cell_sub (optional): This can be given instead of
#'   \code{mRNA_cell} (or in addition to it). It is also a named numeric vector,
#'   used to replace only the mRNA/cell values from some cell types (or to add
#'   values for new cell types). The values given in mRNA_cell_sub will overwrite
#'   the default values as well as those that might have been given by mRNA_cell.
#' @param sigGenes (optional): a character vector of the gene names to use as
#'    signature for the deconvolution. In principle this is given with the
#'    reference as the "reference$sigGenes" but if we give a value for this
#'    input variable, it is these signature genes that will be used instead of
#'    the ones given with the reference profile.
#' @param scaleExprs (optional, default is TRUE): boolean telling if the bulk
#'    samples and reference gene expression profiles should be rescaled based on
#'    the list of genes in common between the them (such a rescaling is
#'    recommanded).
#' @param withOtherCells (optional, default is TRUE): if EPIC should allow for
#'    an additional cell type for which no gene expression reference profile is
#'    available or if the bulk is assumed to be composed only of the cells with
#'    reference profiles.
#' @param constrainedSum (optional, default is TRUE): tells if the sum of all
#'    cell types should be constrained to be < 1. When
#'    \code{withOtherCells=FALSE}, there is additionally a constrain the the sum
#'    of all cell types with reference profiles must be > 0.99.
#' @param rangeBasedOptim (optional): when this is FALSE (the default), the
#'    least square optimization is performed as described in
#'    \href{https://elifesciences.org/articles/26476}{
#'     \cite{Racle et al., 2017, eLife}}, which is recommanded.
#'    When this variable is TRUE, EPIC uses the variability of each gene
#'    from the reference profiles in another way: instead of defining weights
#'    (based on the variability) for the fit of each gene, we define a range of
#'    values accessible for each gene (based on the gene expression value in
#'    the reference profile +/- the variability values). The
#'    error that the optimization tries to minimize is by how much
#'    the predicted gene expression is outside of this allowed range of values.
#'
#' @return A list of 3 matrices:
#' - `mRNAProportions`: (\code{nSamples} x (\code{nCellTypes+1})) the
#' proportion of mRNA coming from all cell types with a ref profile + the
#' uncharacterized other cell
#' - `cellFractions`: (\code{nSamples} x (\code{nCellTypes+1})) this gives the
#' proportion of cells from each cell type after accounting for the mRNA /
#' cell value
#' - `fit.gof`: (\code{nSamples} x 12) a matrix telling the quality for the
#' fit of the signature genes in each sample. It tells if the minimization
#' converged, and other info about this fit comparing the measured gene
#' expression in the sigGenes vs predicted gene expression in the sigGenes
#'
#' @examples
#' melanoma_counts <- matrix(abs(rnorm(80)), nrow = 20, ncol = 4)
#' rownames(melanoma_counts) <- paste0("Gene", 1:20)
#' colnames(melanoma_counts) <- paste0("Sample", 1:4)
#' mock_ref <- list(
#'   refProfiles = matrix(abs(rnorm(80)), nrow = 20, ncol = 4),
#'   sigGenes = paste0("Gene", 1:10)
#' )
#' rownames(mock_ref$refProfiles) <- paste0("Gene", 1:20)
#' colnames(mock_ref$refProfiles) <- c("Bcells", "CD4T", "CD8T", "NK")
#' res1 <- EPIC(melanoma_counts, reference = mock_ref)
#' if (!is.null(res1)) head(res1$cellFractions)
#'
#' @export
EPIC <- function(bulk, reference = NULL, mRNA_cell = NULL, mRNA_cell_sub = NULL,
                 sigGenes = NULL, scaleExprs = TRUE, withOtherCells = TRUE,
                 constrainedSum = TRUE, rangeBasedOptim = FALSE) {

  if (is.null(bulk)) return(NULL)

  # Checking the correct format of the bulk sample input
  if (!is.matrix(bulk) && !is.data.frame(bulk)) {
    stop("'bulk' needs to be given as a matrix or data.frame")
  }

  # First get the value of the reference profiles depending on the input
  # 'reference'.
  with_w <- TRUE
  if (is.null(reference)) {
    reference <- load_data("TRef")
    if (is.null(reference)) return(NULL)
  } else if (is.character(reference)) {
    if (reference %in% c("TRef", "BRef")) {
      reference <- load_data(reference)
      if (is.null(reference)) return(NULL)
      # Replace the char defining the reference name by the corresponding
      # pre-built reference values.
    } else {
      stop(
        "The reference, '", reference, "' is not part of the allowed ",
        "references:", paste(c("TRef", "BRef"), collapse = ", ")
      )
    }
  } else if (is.list(reference)) {
    refListNames <- names(reference)
    if ((!all(c("refProfiles", "sigGenes") %in% refListNames)) ||
      (("refProfiles" %in% refListNames) && !is.null(sigGenes))) {
      stop(
        "Reference, when given as a list needs to contain at least the ",
        "fields 'refProfiles' and 'sigGenes' (sigGenes could also be ",
        "given as input to EPIC instead)"
      )
    }
    if (!is.matrix(reference$refProfiles) &&
      !is.data.frame(reference$refProfiles)) {
      stop(
        "'reference$refProfiles' needs to be given as a matrix or data.frame"
      )
    }
    if (!("refProfiles.var" %in% refListNames)) {
      warning(
        "'refProfiles.var' not defined; using identical weights ",
        "for all genes"
      )
      with_w <- FALSE
    } else if (!is.matrix(reference$refProfiles.var) &&
      !is.data.frame(reference$refProfiles.var)) {
      stop(
        "'reference$refProfiles.var' needs to be given as a matrix or ",
        "data.frame when present."
      )
    } else if (!identical(
      dim(reference$refProfiles.var),
      dim(reference$refProfiles)
    ) ||
      !identical(
        dimnames(reference$refProfiles.var),
        dimnames(reference$refProfiles)
      )) {
      stop(
        "The dimensions and dimnames of 'reference$refProfiles' and ",
        "'reference$refProfiles.var' need to be the same"
      )
    }
  } else {
    stop("Unknown format for 'reference'")
  }

  bulk <- merge_duplicates(bulk, in_type = "bulk samples")
  refProfiles <- merge_duplicates(reference$refProfiles,
    in_type = "reference profiles"
  )
  if (with_w) {
    refProfiles.var <- merge_duplicates(reference$refProfiles.var, warn = FALSE)
    # Don't warn here as we're already warning for refProfile and they had same
    # dim names.
  } else {
    refProfiles.var <- 0
  }

  nSamples <- NCOL(bulk)
  samplesNames <- colnames(bulk)
  if (is.null(samplesNames)) {
    samplesNames <- 1:nSamples
    colnames(bulk) <- samplesNames
  }
  nRefCells <- NCOL(refProfiles)
  refCellsNames <- colnames(refProfiles)


  # Keeping only common genes and normalizing the counts based on these common
  # genes
  bulk_NA <- apply(is.na(bulk), MARGIN = 1, FUN = all)
  if (any(bulk_NA)) {
    warning(sum(bulk_NA), " genes are NA in all bulk samples, removing these.")
    bulk <- bulk[!bulk_NA, ]
  }
  bulkGenes <- rownames(bulk)
  refGenes <- rownames(refProfiles)
  commonGenes <- intersect(bulkGenes, refGenes)

  if (is.null(sigGenes)) {
    sigGenes <- unique(reference$sigGenes)
  } # Keep only once in case of duplicates
  sigGenes <- sigGenes[sigGenes %in% commonGenes]
  nSigGenes <- length(sigGenes)
  if (nSigGenes < nRefCells) {
    stop(
      "There are only ", nSigGenes, " signature genes",
      " matching common genes between bulk and reference profiles,",
      " but there should be more signature genes than reference cells"
    )
  }

  if (scaleExprs) {
    if (length(commonGenes) < 2e3) {
      warning(
        "there are few genes in common between the bulk samples and ",
        "reference cells:", length(commonGenes), ", so the data scaling ",
        "might be an issue"
      )
    }
    # The value of 2e3 is arbitrary, but should be a respectable number for the
    # data renormalization.
    bulk <- scaleCounts(bulk, sigGenes, commonGenes)$counts
    temp <- scaleCounts(refProfiles, sigGenes, commonGenes)
    refProfiles <- temp$counts
    if (with_w) {
      refProfiles.var <- scaleCounts(refProfiles.var, sigGenes,
        normFact = temp$normFact
      )$counts
    }
    # the refProfiles.var is normalized by the same factors as refProfiles.
  } else {
    bulk <- bulk[sigGenes, ]
    refProfiles <- refProfiles[sigGenes, ]
    if (with_w) {
      refProfiles.var <- refProfiles.var[sigGenes, ]
    }
  }

  if (is.null(mRNA_cell)) {
    mRNA_cell <- load_data("mRNA_cell_default")
    if (is.null(mRNA_cell)) return(NULL)
  }

  if (!is.null(mRNA_cell_sub)) {
    if (is.null(names(mRNA_cell_sub)) || !is.numeric(mRNA_cell_sub)) {
      stop("When mRNA_cell_sub is given, it needs to be a named numeric vector")
    }
    mRNA_cell[names(mRNA_cell_sub)] <- mRNA_cell_sub
  }

  minFun <- function(x, A, b, w) {
    # Basic minimization function used to minimize the squared sum of the error
    # between the fit and observed value (A*x - b). We also give a weight, w,
    # for each gene to give more or less importance to the fit of each (can use
    # a value 1 if don't want to give any weight).
    return(sum((w * (A %*% x - b)^2), na.rm = TRUE))
  }
  minFun.range <- function(x, A, b, A.var) {
    # Other minimization function where we don't use weights but instead give
    # also the variability on A as input and we'll compute for each gene the
    # min / max value of the pred based on this variability to keep the smallest
    # value. I.e. we compute a range of the values that y_pred can take for each
    # gene based on the ref profile variability and when the y_true is within
    # this range then we return an error of 0 for this gene and if y_true is
    # outside of the range, we keep the "smallest error" (i.e. value most
    # nearby of the possible range).
    val.max <- (A + A.var) %*% x - b
    val.min <- (A - A.var) %*% x - b
    cErr <- rep(0, length(b))
    outOfRange <- (sign(val.max) * sign(val.min) == 1)
    cErr[outOfRange] <- pmin(abs(val.max[outOfRange]), abs(val.min[outOfRange]))
    return(sum(cErr, na.rm = TRUE))
  }

  if (with_w && !rangeBasedOptim) {
    # Computing the weight to give to each gene
    w <- rowSums(refProfiles / (refProfiles.var + 1e-12), na.rm = TRUE)
    # 1e-12 to avoid divisions by 0: like this if refProfiles and
    # refProfiles.var both are 0 for a given element, it will result in a weight
    # of 0.
    med_w <- stats::median(w[w > 0], na.rm = TRUE)
    w[w > 100 * med_w] <- 100 * med_w
    # Set a limit for the big w to still not give too much weights to these
    # genes
  } else {
    w <- 1
  }

  # Defining the constraints for the fit of the proportions.
  if (withOtherCells) {
    cMin <- 0
  } else {
    cMin <- 0.99
    # So that when we assume no cells without known reference profile are
    # present, we ask the sum of the mRNA proportions of known cells to be at
    # least 0.99.
  }
  cMax <- 1
  ui <- diag(nRefCells)
  ci <- rep(0, nRefCells)
  if (constrainedSum) {
    ui <- rbind(ui, rep(1, nRefCells), rep(-1, nRefCells))
    ci <- c(ci, cMin, -cMax)
  }
  # ui and ci define the constraints, in the form "ui %*% x - ci >= 0".
  # The first constraints are that the proportion of each cell must be
  # positive; then if constrainedSum is true, we want the sum of all proportions
  # to be bigger than cMin and this sum must also be smaller or equal to cMax.
  cInitProp <- (min(1, cMax) - 1e-5) / nRefCells
  # used as an initial guess for the optimized - start with all cell fractions
  # equal. We added the -1e-5 in cInitProp because the optimizer needs to have
  # the initial guess inside the admissible region and not on its boundary

  # Estimating for each sample the proportion of the mRNA per cell type.
  tempPropPred <- lapply(1:nSamples, FUN = function(cSample) {
    b <- bulk[, cSample]
    if (!rangeBasedOptim) {
      fit <- stats::constrOptim(
        theta = rep(cInitProp, nRefCells), f = minFun,
        grad = NULL, ui = ui, ci = ci, A = refProfiles, b = b, w = w
      )
    } else {
      fit <- stats::constrOptim(
        theta = rep(cInitProp, nRefCells), f = minFun.range,
        grad = NULL, ui = ui, ci = ci, A = refProfiles,
        b = b, A.var = refProfiles.var
      )
    }
    fit$x <- fit$par
    if (!withOtherCells) {
      fit$x <- fit$x / sum(fit$x, na.rm = TRUE)
    }
    # So that the sum is really equal to 1 even if the best pred was giving
    # slightly lower values when we force the system to have only known cells.

    # Checking how well the estimated proportions predict the gene expression
    b_estimated <- refProfiles %*% fit$x

    if (nSigGenes > 2) {
      suppressWarnings(
        corSp.test <- stats::cor.test(b, b_estimated, method = "spearman")
      )
      # Use suppressWarnings to avoid the warnings that p-value isn't exact when
      # ties are present in the data.
      corPear.test <- stats::cor.test(b, b_estimated, method = "pearson")
    } else {
      # cannot compute correlations with less than 3 observations.
      corSp.test <- corPear.test <- list()
      corSp.test$estimate <- corSp.test$p.value <- corPear.test$estimate <-
        corPear.test$p.value <- NA
    }
    regLine <- stats::lm(b_estimated ~ b)
    regLine_through0 <- stats::lm(b_estimated ~ b + 0)
    if (!rangeBasedOptim) {
      rmse_pred <- sqrt(
        minFun(x = fit$x, A = refProfiles, b = b, w = w) / nSigGenes
      )
      rmse_0 <- sqrt(
        minFun(x = rep(0, nRefCells), A = refProfiles, b = b, w = w) / nSigGenes
      )
    } else {
      rmse_pred <- sqrt(minFun.range(
        x = fit$x, A = refProfiles, b = b,
        A.var = refProfiles.var
      ) / nSigGenes)
      rmse_0 <- sqrt(minFun.range(
        x = rep(0, nRefCells), A = refProfiles, b = b,
        A.var = refProfiles.var
      ) / nSigGenes)
    }
    gof <- data.frame(
      fit$convergence,
      ifelse(is.null(fit$message), "", fit$message),
      rmse_pred, rmse_0,
      # to only have sum((w*b)^2) or corresponding value based
      # on the minFun, in the worst case possible.
      corSp.test$estimate, corSp.test$p.value,
      corPear.test$estimate, corPear.test$p.value,
      regLine$coefficients[2], regLine$coefficients[1],
      regLine_through0$coefficients[1], sum(fit$x),
      stringsAsFactors = FALSE
    )

    return(list(mRNAProportions = fit$x, fit.gof = gof))
  })
  mRNAProportions <- do.call(rbind, lapply(
    tempPropPred,
    function(x) x$mRNAProportions
  ))
  dimnames(mRNAProportions) <- list(samplesNames, refCellsNames)

  fit.gof <- do.call(rbind, lapply(tempPropPred, function(x) x$fit.gof))
  dimnames(fit.gof) <- list(samplesNames, c(
    "convergeCode", "convergeMessage", "RMSE_weighted",
    "Root_mean_squared_geneExpr_weighted",
    "spearmanR", "spearmanP", "pearsonR", "pearsonP", "regline_a_x",
    "regline_b", "regline_a_x_through0", "sum_mRNAProportions"
  ))
  # Some matrix giving information on the goodness of the fit.

  if (any(fit.gof$convergeCode != 0)) {
    warning(
      "The optimization didn't fully converge for some samples:\n",
      paste(samplesNames[fit.gof$convergeCode != 0], collapse = "; "),
      "\n - check fit.gof for the convergeCode and convergeMessage"
    )
  }

  if (withOtherCells) {
    mRNAProportions <- cbind(
      mRNAProportions,
      otherCells = 1 - rowSums(mRNAProportions)
    )
  }
  # Adding a row to the proportion matrix corresponding to the other /
  # uncharacterized / cancer cells.

  tInds <- match(colnames(mRNAProportions), names(mRNA_cell))
  if (anyNA(tInds)) {
    defaultInd <- match("default", names(mRNA_cell))
    if (is.na(defaultInd)) {
      tStr <- paste(
        " and no default value is given for this mRNA per cell,",
        "so we cannot estimate the cellFractions, only",
        "the mRNA proportions"
      )
    } else {
      tStr <- paste(
        " - using the default value of", mRNA_cell[defaultInd],
        "for these but this might bias the true cell proportions from",
        "all cell types."
      )
    }
    warning(
      "mRNA_cell value unknown for some cell types: ",
      paste(colnames(mRNAProportions)[is.na(tInds)], collapse = ", "),
      tStr
    )
    tInds[is.na(tInds)] <- defaultInd
  }
  cellFractions <- t(t(mRNAProportions) / mRNA_cell[tInds])
  cellFractions <- cellFractions / rowSums(cellFractions, na.rm = FALSE)
  # So that the cell fractions sum up to 1. In case some values were NA (due
  # to unknown mRNA_cell value for some cell types without default value), then
  # the full cellFractions will be NA - if we used na.rm=T, then we'd have the
  # sum of the other than "NA" cells equal to 1 as if this "NA" cell was not
  # present in the sample.

  return(list(
    mRNAProportions = mRNAProportions, cellFractions = cellFractions,
    fit.gof = fit.gof
  ))
}

#' Scaling raw counts from each sample.
#'
#' Normalizing the sum of counts from each sample to 1e6.
#'
#' Function taking a matrix (\emph{genes} x \emph{samples}) of counts as input
#' and returning the scaled counts for the subset signature genes (or all genes
#' if it sigGenes is \code{NULL}), with the scaled counts computed based on all
#' the 'renormGenes' (or all genes if it is NULL). The renormalization is made
#' independently for each sample, to have the sum of each columns over the
#' renormGenes equal to 1e6.
#'
#' normFact, if not null, is used as the normalization factor instead of
#'  the colSums (used to renormalize the refProfiles.var by the same amount
#'  than the refProfiles).
#'
#' @keywords internal
scaleCounts <- function(counts, sigGenes = NULL, renormGenes = NULL,
                        normFact = NULL) {
  if (is.null(sigGenes)) {
    sigGenes <- 1:nrow(counts)
  }

  if (is.null(normFact)) {
    if (is.null(renormGenes)) {
      renormGenes <- 1:nrow(counts)
    }
    normFact <- colSums(counts[renormGenes, , drop = FALSE], na.rm = TRUE)
  }
  counts <- t(t(counts[sigGenes, , drop = FALSE]) / normFact) * 1e6
  # Need to take the transpose so that the division is made on the correct
  # elements
  return(list(counts = counts, normFact = normFact))
}

#' Merging the duplicates from the input matrix.
#'
#' In case there are some duplicate rownames in the input matrix (\emph{mat}),
#' this function will return a similar matrix with unique rownames and the
#' duplicate cases will be merged together based on the median values. When
#' \emph{warn} is true a warning message will be written in case there are
#' duplicates, this warning message will include the \emph{in_type} string to
#' indicate the current type of the input matrix.
#'
#' @param mat matrix
#' @param warn warning
#' @param in_type default is null
#'
#' @keywords internal
merge_duplicates <- function(mat, warn = TRUE, in_type = NULL) {
  dupl <- duplicated(rownames(mat))
  if (sum(dupl) > 0) {
    dupl_genes <- unique(rownames(mat)[dupl])
    if (warn) {
      warning(
        "There are ", length(dupl_genes), " duplicated gene names",
        ifelse(!is.null(in_type), paste(" in the", in_type), ""),
        ". We'll use the median value for each of these cases."
      )
    }
    mat_dupl <- mat[rownames(mat) %in% dupl_genes, , drop = FALSE]
    mat_dupl_names <- rownames(mat_dupl)
    mat <- mat[!dupl, , drop = FALSE]
    # First put the dupl cases in a separate matrix and keep only the unique
    # gene names in the mat matrix.
    mat[dupl_genes, ] <- t(sapply(dupl_genes, FUN = function(cgene) {
      apply(mat_dupl[mat_dupl_names == cgene, , drop = FALSE],
        MARGIN = 2, FUN = median
      )
    }))
  }
  return(mat)
}

#' Analyze Mutations Related to Signature Scores
#'
#' This function identifies mutations associated with a specific signature
#' score, performs statistical tests
#' for significance, and generates oncoprints and box plots to visualize
#' relationships.
#'
#' @param mutation_matrix A matrix of mutation data with samples in rows and genes in columns.
#' @param signature_matrix A data frame with sample identifiers and signature scores.
#' @param id_signature_matrix Column name in `signature_matrix` for sample identifiers.
#' @param signature Name of the target signature for analysis.
#' @param min_mut_freq Minimum mutation frequency required for gene inclusion. Default is 0.05.
#' @param plot Logical indicating whether to generate and save plots. Default is TRUE.
#' @param method Statistical test method: "multi" for both Cuzick and Wilcoxon, or "Wilcoxon" only. Default is "multi".
#' @param save_path Directory to save plots and results. If NULL, no files are saved.
#' @param palette Color palette for box plots(used when cols is NULL). Default is "jco".
#' @param cols Character vector. Custom colors for box plots. If NULL, uses palette. Default is NULL.
#' @param show_plot Logical indicating whether to display plots. Default is TRUE.
#' @param show_col Logical indicating whether to show color codes. Default is FALSE.
#' @param width Width of oncoprint plot. Default is 8.
#' @param height Height of oncoprint plot. Default is 4.
#' @param oncoprint_group_by Grouping method for oncoprint: "mean" or "quantile". Default is "mean".
#' @param oncoprint_col Color for mutations in oncoprint. Default is "#224444".
#' @param gene_counts Number of genes to display in oncoprint. Default is 10.
#' @param genes Optional vector of gene names; if NULL, selects based on frequency.
#' @param point_size Size of points in box plot. Default is 4.5.
#' @param point_alpha Transparency of points in box plot. Default is 0.1.
#' @param jitter Logical indicating whether to add jitter to box plot points. Default is FALSE.
#'
#' @return A list containing statistical test results, oncoprint plots, and box plots.
#' @export
#' @author Dongqiang Zeng
#' @examples
#' \dontrun{
#' # This example requires a MAF file from TCGA or maftools
#' # See maftools or TCGAbiolinks documentation for obtaining MAF files
#' mut_list <- make_mut_matrix(
#'   maf = "path_to_maf_file", isTCGA = TRUE,
#'   category = "multi"
#' )
#' mut <- mut_list$snp
#' results <- find_mutations(
#'   mutation_matrix = mut, signature_matrix = signature_data,
#'   id_signature_matrix = "ID", signature = "CD_8_T_effector",
#'   min_mut_freq = 0.01, plot = TRUE, method = "multi"
#' )
#' }
find_mutations <- function(mutation_matrix, signature_matrix, id_signature_matrix = "ID", signature,
                           min_mut_freq = 0.05, plot = TRUE, method = "multi", point_alpha = 0.1,
                           save_path = NULL, palette = "jco", cols = NULL, show_plot = TRUE,
                           show_col = FALSE, width = 8, height = 4, oncoprint_group_by = "mean",
                           oncoprint_col = "#224444", gene_counts = 10, jitter = FALSE, genes = NULL, point_size = 4.5) {
  rlang::check_installed("ggpubr")
  rlang::check_installed("reshape2")
  rlang::check_installed("PMCMRplus")
  if (!is.null(save_path)) {
    output_dir <- save_path
    if (!dir.exists(output_dir)) dir.create(output_dir, recursive = TRUE)
    abspath <- file.path(normalizePath(output_dir, winslash = "/", mustWork = FALSE), "")
  } else {
    abspath <- NULL
    output_dir <- NULL
  }
  #######################################################
  signature_matrix <- as.data.frame(signature_matrix)

  if (max(mutation_matrix) > 4) {
    mutation_matrix[mutation_matrix >= 3 & mutation_matrix <= 5] <- 3
    mutation_matrix[mutation_matrix > 5] <- 4
  }


  mut2 <- mutation_matrix
  mut2[mut2 >= 1] <- 1
  mut_onco <- mut2

  if (is.null(genes)) {
    mutfreq <- data.frame(head(sort(colSums(mut2), decreasing = TRUE), 500))
    colnames(mutfreq) <- "Freq"
    index <- which(mutfreq$Freq >= dim(mut2)[1] * min_mut_freq)
    index <- max(index)
    input_genes <- names(head(sort(colSums(mut2), decreasing = TRUE), index))
    input_genes <- unique(input_genes)
    input_genes <- input_genes[!is.na(input_genes)]
  } else {
    input_genes <- genes
  }

  mutation_matrix <- mutation_matrix[, colnames(mutation_matrix) %in% input_genes]
  genes <- genes[genes %in% colnames(mutation_matrix)]
  ########################################################

  colnames(signature_matrix)[which(colnames(signature_matrix) == id_signature_matrix)] <- "ID"
  mutation_matrix <- as.data.frame(mutation_matrix)
  mutation_matrix <- tibble::rownames_to_column(mutation_matrix, var = "ID")


  sig_mut <- merge(signature_matrix, mutation_matrix, by = "ID", all = FALSE)
  sig_mut <- tibble::column_to_rownames(sig_mut, var = "ID")


  mytheme <- ggplot2::theme_light() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(size = ggplot2::rel(2.3), hjust = 0.5, face = "italic"),
      axis.title.y = ggplot2::element_text(size = ggplot2::rel(1.8)),
      axis.title.x = ggplot2::element_blank(),
      axis.text.x = ggplot2::element_text(face = "plain", size = 18, angle = 0, color = "black"), # family="Times New Roman"
      axis.text.y = ggplot2::element_text(face = "plain", size = 15, angle = 90, color = "black"), # family="Times New Roman"
      axis.line = ggplot2::element_line(color = "black", size = 0.6)
    ) + ggplot2::theme(
      legend.key.size = grid::unit(.3, "inches"),
      legend.title = ggplot2::element_blank(),
      legend.position = "none",
      legend.direction = "horizontal",
      legend.justification = c(.5, .5),
      legend.box = "vertical",
      legend.box.just = "top",
      legend.text = ggplot2::element_text(colour = "black", size = 10, face = "plain")
    )

  fill_colors <- if (is.null(cols)) {
    palettes(category = "box", palette = palette, show_col = show_col)
  } else {
    cols
  }

  if (method == "multi") {
    if (!is.null(genes)) {
      if (length(genes) < 10) {
        message(paste(capture.output(genes), collapse = "\n"))
        stop(paste0("Please provide at least 10 genes with mutaion freq larger than ", min_mut_freq))
      }
      input_genes <- genes
    }

    input_genes <- input_genes[input_genes %in% colnames(sig_mut)]

    input <- sig_mut[, c(signature, input_genes)]
    ##############################
    aa <- lapply(input[, input_genes], function(x) PMCMRplus::cuzickTest(input[, 1] ~ x))
    res1 <- data.frame(
      p.value = sapply(aa, getElement, name = "p.value"),
      names = input_genes,
      statistic = sapply(aa, getElement, name = "statistic")
    )
    res1$adjust_pvalue <- stats::p.adjust(res1$p.value, method = "BH", n = length(res1$p.value))
    message(">>>> Result of Cuzick Test")
    res1 <- res1[order(res1$p.value, decreasing = FALSE), ]
    message(paste(capture.output(res1[1:10, ]), collapse = "\n"))

    # write.csv(res1, paste0(abspath, "1-cuzickTest-test-relevant-mutations.csv"))
    if (!is.null(save_path)) {
      write.csv(res1, paste0(abspath, "1-cuzickTest-test-relevant-mutations.csv"))
    }


    if (plot) {
      ####################################
      top10_genes <- res1$names[1:10]
      top10_genes <- top10_genes[!is.na(top10_genes)]
      top10_genes <- as.character(top10_genes[top10_genes %in% colnames(input)])
      input_long <- input[, c(signature, top10_genes)]
      input_long <- reshape2::melt(input_long,
        id.vars = 1,
        variable.name = "Gene",
        value.name = "mutation"
      )
      input_long[, signature] <- as.numeric(input_long[, signature])
      input_long$mutation <- as.factor(input_long$mutation)
      ####################################

      pl <- list()
      for (i in 1:length(top10_genes)) {
        gene <- top10_genes[i]
        dd <- input_long[input_long$Gene == gene, ]
        pl[[i]] <- ggplot2::ggplot(dd, ggplot2::aes(x = .data$mutation, y = !!rlang::sym(signature), fill = .data$mutation)) +
          ggplot2::geom_boxplot(outlier.shape = NA, outlier.size = -0.5) +
          # geom_jitter(width = 0.25,size= 5.9,alpha=0.75,color ="black")+
          ggplot2::scale_fill_manual(values = fill_colors) +
          mytheme +
          ggplot2::theme(legend.position = "none") +
          ggplot2::ggtitle(paste0(top10_genes[i])) +
          mytheme +
          ggpubr::stat_compare_means(comparisons = utils::combn(as.character(unique(dd[, "mutation"])), 2, simplify = FALSE), size = 6)

        if (jitter) {
          pl[[i]] <- pl[[i]] + ggplot2::geom_jitter(width = 0.25, size = point_size, alpha = point_alpha, color = "black")
        }
        if (!is.null(save_path)) {
          ggplot2::ggsave(
            filename = paste0(4 + i, "-1-", gene, "-continue.pdf"),
            plot = pl[[i]],
            width = 4, height = 5.8, path = output_dir
          )
        }
      }
      rlang::check_installed("patchwork")
      com_plot <- patchwork::wrap_plots(pl[1:10], #  等价拼图
        ncol = 5, nrow = 2,
        labels = "AUTO",
        label_size = 32
      )
      if (show_plot) print(com_plot)
      #####################################
      if (!is.null(save_path)) {
        ggplot2::ggsave(filename = "3-Relevant_mutations_Continue.pdf", plot = com_plot, width = 14, height = 10.5, path = output_dir)
      }
      #####################################
    }


    sig_mut2 <- tibble::rownames_to_column(sig_mut, var = "ID")
    patr1 <- sig_mut2[, c("ID", signature)]
    part2 <- sig_mut2[, input_genes]
    part2[part2 >= 1] <- 1
    sig_mut2 <- cbind(patr1, part2)
    sig_mut2 <- column_to_rownames(sig_mut2, var = "ID")

    input2 <- sig_mut2

    # print(input2[1:5,1:5])
    aa <- lapply(input2[, input_genes], function(x) stats::wilcox.test(input2[, 1] ~ x))

    res2 <- data.frame(
      p.value = sapply(aa, getElement, name = "p.value"),
      names = input_genes,
      statistic = sapply(aa, getElement, name = "statistic")
    )

    res2$adjust_pvalue <- stats::p.adjust(res2$p.value, method = "BH", n = length(res2$p.value))

    res2 <- res2[order(res2$p.value, decreasing = FALSE), ]
    message(">>> Result of Wilcoxon test (top 10)")
    message(paste(capture.output(res2[1:10, ]), collapse = "\n"))
    if (!is.null(save_path)) {
      write.csv(res2, paste0(abspath, "2-Wilcoxon-test-relevant-mutations.csv"))
    }

    result <- list(
      "cuzick_test" = res1, "wilcoxon_test" = res2,
      "sig_mut_data1" = input, "sig_mut_data2" = input2
    )


    if (plot) {
      ####################################
      top10_genes <- res2$names[1:10]
      top10_genes <- top10_genes[!is.na(top10_genes)]
      top10_genes <- as.character(top10_genes[top10_genes %in% colnames(input2)])

      input_long <- input2[, c(signature, top10_genes)]
      input_long <- reshape2::melt(input_long,
        id.vars = 1,
        variable.name = "Gene",
        value.name = "mutation"
      )
      input_long[, signature] <- as.numeric(input_long[, signature])
      input_long$mutation <- as.factor(input_long$mutation)

      input_long$mutation <- ifelse(input_long$mutation == 0, "WT", "Mutated")
      ####################################

      pl <- list()
      for (i in 1:length(top10_genes)) {
        gene <- top10_genes[i]
        dd <- input_long[input_long$Gene == gene, ]
        pl[[i]] <- ggplot2::ggplot(dd, ggplot2::aes(x = .data$mutation, y = !!rlang::sym(signature), fill = .data$mutation)) +
          ggplot2::geom_boxplot(outlier.shape = NA, outlier.size = -0.5) +
          # geom_jitter(width = 0.25,size=5.5,alpha=0.75,color ="black")+
          ggplot2::scale_fill_manual(values = fill_colors) +
          ggplot2::theme(legend.position = "none") +
          ggplot2::ggtitle(paste0(top10_genes[i])) +
          mytheme +
          ggpubr::stat_compare_means(comparisons = utils::combn(as.character(unique(dd[, "mutation"])), 2, simplify = FALSE), size = 6)
        if (jitter) {
          pl[[i]] <- pl[[i]] + ggplot2::geom_jitter(width = 0.25, size = point_size, alpha = point_alpha, color = "black")
        }

        if (!is.null(save_path)) {
          ggplot2::ggsave(
            filename = paste0(4 + i, "-2-", gene, "-binary.pdf"),
            plot = pl[[i]],
            width = 4, height = 5.8, path = output_dir
          )
        }
      }
      com_plot <- patchwork::wrap_plots(pl[1:10], #  等价拼图
        ncol = 5, nrow = 2,
        labels = "AUTO",
        label_size = 32
      )
      if (show_plot) print(com_plot)
      #####################################
      if (!is.null(save_path)) {
        ggplot2::ggsave(filename = "4-Relevant_mutations_binary.pdf", plot = com_plot, width = 14, height = 10.5, path = output_dir)
      }
      #####################################
    }
  } else if (tolower(method) == "wilcoxon") {
    if (!is.null(genes)) {
      if (length(genes) < 10) {
        message(paste(capture.output(genes), collapse = "\n"))
        stop(paste0("Please provide at least 10 genes with mutaion freq larger than ", min_mut_freq))
      }
      input_genes <- genes
    }

    sig_mut2 <- tibble::rownames_to_column(sig_mut, var = "ID")

    patr1 <- sig_mut2[, c("ID", signature)]

    input_genes <- input_genes[input_genes %in% colnames(sig_mut2)]

    part2 <- sig_mut2[, input_genes]
    part2[part2 >= 1] <- 1
    sig_mut2 <- cbind(patr1, part2)
    sig_mut2 <- column_to_rownames(sig_mut2, var = "ID")
    input2 <- sig_mut2

    # print(input2[1:5,1:5])
    ##############################
    aa <- lapply(input2[, input_genes], function(x) stats::wilcox.test(input2[, 1] ~ x))

    res <- data.frame(
      p.value = sapply(aa, getElement, name = "p.value"),
      names = input_genes,
      statistic = sapply(aa, getElement, name = "statistic")
    )

    res$adjust_pvalue <- stats::p.adjust(res$p.value, method = "BH", n = length(res$p.value))

    res <- res[order(res$p.value, decreasing = FALSE), ]

    message(">>> Result of Wilcoxon test (top 10): ")
    message(paste(capture.output(res[1:10, ]), collapse = "\n"))

    if (!is.null(save_path)) {
      write.csv(res, paste0(abspath, "0-Wilcoxon-test-relevant-mutations.csv"))
    }

    result <- list("wilcoxon_test" = res, "sig_mut_data" = input2)
    #################################

    if (plot) {
      ####################################
      top10_genes <- res$names[1:10]
      top10_genes <- top10_genes[!is.na(top10_genes)]
      top10_genes <- as.character(top10_genes[top10_genes %in% colnames(input2)])

      input_long <- input2[, c(signature, top10_genes)]
      input_long <- reshape2::melt(input_long,
        id.vars = 1,
        variable.name = "Gene",
        value.name = "mutation"
      )
      input_long[, signature] <- as.numeric(input_long[, signature])
      input_long$mutation <- as.factor(input_long$mutation)

      input_long$mutation <- ifelse(input_long$mutation == 0, "WT", "Mutated")
      ####################################
      pl <- list()
      for (i in 1:length(top10_genes)) {
        gene <- top10_genes[i]
        dd <- input_long[input_long$Gene == gene, ]
        pl[[i]] <- ggplot(dd, aes(x = mutation, y = !!sym(signature), fill = mutation)) +
          geom_boxplot(outlier.shape = NA, outlier.size = NA) +
          # geom_jitter(width = 0.25,size= 3.5,alpha=0.75,color ="black")+
          ggplot2::scale_fill_manual(values = fill_colors) +
          mytheme +
          ggplot2::theme(legend.position = "none") +
          ggplot2::ggtitle(paste0(top10_genes[i])) +
          mytheme +
          ggpubr::stat_compare_means(comparisons = utils::combn(as.character(unique(dd[, "mutation"])), 2, simplify = FALSE), size = 6)

        if (jitter) {
          pl[[i]] <- pl[[i]] + ggplot2::geom_jitter(width = 0.25, size = point_size, alpha = point_alpha, color = "black")
        }

        if (!is.null(save_path)) {
          ggplot2::ggsave(
            filename = paste0(i, "-1-", gene, "-binary.pdf"),
            plot = pl[[i]],
            width = 4, height = 5.8, path = output_dir
          )
        }
      }
      com_plot <- patchwork::wrap_plots(pl[1:10], #  等价拼图
        ncol = 5, nrow = 2,
        labels = "AUTO",
        label_size = 32
      )
      if (show_plot) print(com_plot)
      #####################################
      if (!is.null(save_path)) {
        ggplot2::ggsave(filename = "0-Relevant_mutations_binary.pdf", plot = com_plot, width = 14, height = 10.5, path = output_dir)
      }
      #####################################
    }
  }


  # if(!is.null(genes)){
  #   if(length(genes)<10) stop("Please provide at least 10 genes")
  #   genes_for_oncoprint<-genes
  #   genes_for_oncoprint<-genes_for_oncoprint[1:gene_counts]
  #   genes_for_oncoprint<-genes_for_oncoprint[!is.na(genes_for_oncoprint)]
  # }else{
  #   genes_for_oncoprint<-result$wilcoxon_test$names
  #   genes_for_oncoprint<-genes_for_oncoprint[1:gene_counts]
  #   genes_for_oncoprint<-genes_for_oncoprint[!is.na(genes_for_oncoprint)]
  # }
  #
  genes_for_oncoprint <- result$wilcoxon_test$names
  genes_for_oncoprint <- genes_for_oncoprint[1:gene_counts]
  genes_for_oncoprint <- genes_for_oncoprint[!is.na(genes_for_oncoprint)]


  signature_matrix <- signature_matrix[!duplicated(signature_matrix$ID), ]
  signature_matrix <- signature_matrix[signature_matrix$ID %in% rownames(mut_onco), ]

  mut_onco <- mut_onco[rownames(mut_onco) %in% signature_matrix$ID, ]
  ####################################


  if (oncoprint_group_by != "mean" & oncoprint_group_by != "quantile3") {
    pdata_group <- signature_matrix[, c("ID", signature, oncoprint_group_by)]
    pdata_group[, signature] <- as.numeric(pdata_group[, signature])
  } else {
    pdata_group <- signature_matrix[, c("ID", signature)]
    pdata_group[, signature] <- as.numeric(pdata_group[, signature])
  }

  max_sig <- max(pdata_group[, signature], na.rm = TRUE)
  min_sig <- min(pdata_group[, signature], na.rm = TRUE)
  #################################
  if (oncoprint_group_by == "mean") {
    if (!"group2" %in% colnames(pdata_group)) {
      pdata_group$group <- ifelse(pdata_group[, signature] >= mean(pdata_group[, signature]), "High", "Low")
    }
  } else if (oncoprint_group_by == "quantile3") {
    if (!"group3" %in% colnames(pdata_group)) {
      q1 <- stats::quantile(pdata_group[, signature], probs = 1 / 3)
      q2 <- quantile(pdata_group[, signature], probs = 2 / 3)
      pdata_group$group <- ifelse(pdata_group[, signature] <= q1, "Low", ifelse(pdata_group[, signature] >= q2, "High", "Middle"))
    }
  } else {
    if ("group" %in% colnames(pdata_group) & "group" != oncoprint_group_by) stop(" Variable:`group` already exist in colname of pdata!")

    if (!"group" %in% colnames(pdata_group)) {
      colnames(pdata_group)[which(colnames(pdata_group) == oncoprint_group_by)] <- "group"
    }
    # print(head(pdata_group))
    if (nlevels(as.factor(pdata_group$group)) > 2) stop("Levels of `oncoprint_group_by` must less than 3")

    if (!"High" %in% unique(pdata_group$group)) {
      print(summary(as.factor(pdata_group$group)))
      stop("Levels of `oncoprint_group_by` must be `High` or `Low`")
    }
  }


  # print(head(pdata_group))
  idh <- pdata_group[pdata_group$group == "High", "ID"]
  idl <- pdata_group[pdata_group$group == "Low", "ID"]
  pdata1 <- pdata_group[pdata_group$ID %in% idh, ]
  pdata2 <- pdata_group[pdata_group$ID %in% idl, ]

  rlang::check_installed("ComplexHeatmap")
  group_col <- palettes(category = "box", palette = palette, show_col = show_col)

  h1 <- ComplexHeatmap::HeatmapAnnotation(
    Signature_score = ComplexHeatmap::anno_barplot(as.numeric(pdata1[, signature]),
      border = FALSE,
      gp = grid::gpar(fill = "#2D004B"),
      axis = TRUE,
      ylim = c(min_sig, max_sig)
    ),
    Group = pdata1$group,
    annotation_height = grid::unit.c(rep(grid::unit(1.5, "cm"), 1), rep(grid::unit(0.5, "cm"), 1)), # unit.c(rep(unit(0.9, "cm"), 5))
    annotation_legend_param = list(
      labels_gp = grid::gpar(fontsize = 10),
      title_gp = grid::gpar(fontsize = 10, fontface = "bold"),
      ncol = 1
    ),
    gap = grid::unit(c(2, 2), "mm"),
    col = list(Group = c("High" = group_col[1], "Low" = group_col[2])),
    show_annotation_name = TRUE,
    # annotation_name_side="left",
    annotation_name_gp = grid::gpar(fontsize = 12)
  )

  h2 <- ComplexHeatmap::HeatmapAnnotation(
    Signature_score = ComplexHeatmap::anno_barplot(as.numeric(pdata2[, signature]),
      border = FALSE,
      gp = grid::gpar(fill = "#2D004B"),
      axis = TRUE,
      ylim = c(min_sig, max_sig)
    ),
    Group = pdata2$group,
    annotation_height = grid::unit.c(rep(grid::unit(1.5, "cm"), 1), rep(grid::unit(0.5, "cm"), 1)), # unit.c(rep(unit(0.9, "cm"), 5))
    annotation_legend_param = list(
      labels_gp = grid::gpar(fontsize = 10),
      title_gp = grid::gpar(fontsize = 10, fontface = "bold"),
      ncol = 1
    ),
    gap = grid::unit(c(2, 2), "mm"),
    col = list(Group = c("High" = group_col[1], "Low" = group_col[2])),
    show_annotation_name = TRUE,
    # annotation_name_side="left",
    annotation_name_gp = grid::gpar(fontsize = 12)
  )


  col <- c(mut = oncoprint_col)
  mut1 <- t(mut_onco[rownames(mut_onco) %in% idh, colnames(mut_onco) %in% genes_for_oncoprint])
  mut2 <- t(mut_onco[rownames(mut_onco) %in% idl, colnames(mut_onco) %in% genes_for_oncoprint])

  mut1 <- list(mut = mut1)
  mut2 <- list(mut = mut2)

  # width_h<- c(length(idh)/c(length(idh)+length(idl)))*width
  # width_l<- c(length(idl)/c(length(idh)+length(idl)))*width

  #########################################
  ho1 <- ComplexHeatmap::oncoPrint(mut1,
    alter_fun_is_vectorized = FALSE,
    alter_fun = list(mut = function(x, y, w, h) {
      grid::grid.rect(x, y, w * 0.9, h * 0.88,
        gp = grid::gpar(fill = oncoprint_col, col = oncoprint_col)
      )
    }),
    column_title = paste0(" High ", signature),
    show_heatmap_legend = FALSE,
    heatmap_legend_param = list(title = "", labels = ""),
    col = col,
    row_names_gp = grid::gpar(fontface = "italic"),
    # width = unit(width_h, "cm"),
    top_annotation = h1
  )


  ho2 <- ComplexHeatmap::oncoPrint(mut2,
    alter_fun_is_vectorized = FALSE,
    alter_fun = list(mut = function(x, y, w, h) {
      grid::grid.rect(x, y, w * 0.9, h * 0.88,
        gp = grid::gpar(fill = oncoprint_col, col = oncoprint_col)
      )
    }),
    column_title = paste0(" Low ", signature),
    show_heatmap_legend = FALSE,
    heatmap_legend_param = list(title = "", labels = "Mutation"),
    col = col,
    row_names_gp = grid::gpar(fontface = "italic"),
    # width = unit(width_l, "cm"),
    top_annotation = h2
  )


  p <- ho1 + ho2

  # fig.path<-paste0(getwd(),"/",save_path)
  # save to pdf
  if (!is.null(save_path)) {
    pdf(file.path(abspath, paste0("0-OncoPrint-", signature, ".pdf")), width = width, height = height)
    ComplexHeatmap::draw(p)
    invisible(dev.off())
  }
  # print to screen
  # draw(p)
  if (show_plot) {
    if (interactive()) print(p)
  }

  result$onco_plot <- p
  result$box_plot <- com_plot
  return(result)
}

#' Batch Survival Analysis
#'
#' @description
#' Performs Cox proportional hazards regression analysis on multiple variables.
#' Optionally determines optimal cutoffs to dichotomize continuous predictors before
#' modeling. Returns hazard ratios, confidence intervals, and p-values for each variable.
#'
#' @param pdata Data frame containing survival time, event status, and predictor variables.
#' @param variable Character vector specifying the names of predictor variables to analyze.
#' @param time Character string specifying the column name containing follow-up time.
#'   Default is `"time"`.
#' @param status Character string specifying the column name containing event status
#'   (1 = event occurred, 0 = censored). Default is `"status"`.
#' @param best_cutoff Logical indicating whether to compute optimal cutoffs for
#'   continuous variables and analyze dichotomized versions. Default is `FALSE`.
#'
#' @return Data frame containing hazard ratios (HR), 95% confidence intervals (CI),
#'   and p-values for each variable, sorted by p-value.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' # Create small example data
#' set.seed(123)
#' test_data <- data.frame(
#'   OS_time = runif(100, 0, 100),
#'   OS_status = sample(c(0, 1), 100, replace = TRUE),
#'   Signature1 = rnorm(100),
#'   Signature2 = rnorm(100)
#' )
#' batch_surv(
#'   pdata = test_data,
#'   variable = c("Signature1", "Signature2"),
#'   time = "OS_time",
#'   status = "OS_status"
#' )
batch_surv <- function(pdata, variable, time = "time", status = "status", best_cutoff = FALSE) {
  if (is.null(pdata)) return(NULL)
  if (!is.data.frame(pdata)) {
    cli::cli_abort("{.arg pdata} must be a data frame")
  }
  if (nrow(pdata) == 0) {
    cli::cli_abort("{.arg pdata} has no rows")
  }
  if (!time %in% colnames(pdata)) {
    cli::cli_abort("Time column {.val {time}} not found in pdata")
  }
  if (!status %in% colnames(pdata)) {
    cli::cli_abort("Status column {.val {status}} not found in pdata")
  }
  if (!is.character(variable) || length(variable) == 0) {
    cli::cli_abort("{.arg variable} must be a non-empty character vector")
  }

  pdata <- as.data.frame(pdata)
  colnames(pdata)[colnames(pdata) == time] <- "time_iobr"
  colnames(pdata)[colnames(pdata) == status] <- "status_iobr"

  pdata <- pdata[!is.na(pdata$time_iobr) & !is.na(pdata$status_iobr), , drop = FALSE]
  if (nrow(pdata) == 0) {
    cli::cli_abort("No valid data after removing NA values")
  }

  pdata$time_iobr <- as.numeric(pdata$time_iobr)
  pdata$status_iobr <- as.numeric(pdata$status_iobr)

  valid_vars <- variable[variable %in% colnames(pdata)]
  invalid_vars <- setdiff(variable, colnames(pdata))
  if (length(invalid_vars) > 0) {
    cli::cli_alert_warning("Ignoring {length(invalid_vars)} variable{?s} not found: {.val {invalid_vars}}")
  }
  if (length(valid_vars) == 0) {
    cli::cli_abort("No valid variables found in pdata")
  }

  if (best_cutoff) {
    cli::cli_alert_info("Computing optimal cutoffs for {length(valid_vars)} variable{?s}")
    for (var in valid_vars) {
      pdata <- best_cutoff(
        pdata = pdata,
        time = "time_iobr",
        status = "status_iobr",
        variable = var,
        print_result = FALSE
      )
      binary_col <- paste0(var, "_binary")
      pdata[[binary_col]] <- ifelse(pdata[[binary_col]] == "High", 1, 0)
    }
    valid_vars <- paste0(valid_vars, "_binary")
  }

  result_list <- lapply(valid_vars, function(var) {
    fit <- survival::coxph(
      survival::Surv(pdata$time_iobr, pdata$status_iobr) ~ pdata[[var]],
      data = pdata
    )
    getHRandCIfromCoxph(fit)
  })

  result <- dplyr::bind_rows(result_list)
  result$ID <- if (best_cutoff) gsub("_binary$", "", valid_vars) else valid_vars
  # 把 ID 调整到第一列
  result <- result[, c("ID", setdiff(colnames(result), "ID")), drop = FALSE]
  result <- result[order(result$P, decreasing = FALSE), , drop = FALSE]
  rownames(result) <- NULL

  tibble::as_tibble(result)
}

#' Identify Marker Features in Bulk Expression Data
#'
#' Identifies informative marker features across groups from bulk gene
#' expression or signature score matrices using Seurat workflows. Performs
#' feature selection, scaling, PCA, clustering, and marker discovery.
#'
#' @param pdata Data frame. Sample metadata.
#' @param eset Matrix. Gene expression or signature score matrix.
#' @param group Character. Column name in pdata specifying grouping variable.
#' @param id_pdata Character. Column name for sample IDs. Default is "ID".
#' @param nfeatures Integer. Number of top variable features to select. Default is 2000.
#' @param top_n Integer. Number of top markers to retain per cluster. Default is 20.
#' @param thresh.use Numeric. Threshold for marker selection. Default is 0.25.
#' @param only.pos Logical. Whether to retain only positive markers. Default is TRUE.
#' @param min.pct Numeric. Minimum expression percentage threshold. Default is 0.25.
#' @param npcs Integer. Number of principal components to use. Default is 30.
#'
#' @importFrom methods as
#' @importFrom tibble column_to_rownames
#' @return List with components: `sce` (Seurat object), `markers` (all markers), `top_markers` (top markers per group).
#' @export
#'
#' @examples
#' if (requireNamespace("Seurat", quietly = TRUE) && requireNamespace("Matrix", quietly = TRUE)) {
#'   # Simulate data
#'   set.seed(123)
#'   sim_eset <- matrix(abs(rnorm(100 * 30)), 100, 30)
#'   rownames(sim_eset) <- paste0("Gene", 1:100)
#'   colnames(sim_eset) <- paste0("Sample", 1:30)
#'   
#'   sim_pdata <- data.frame(
#'     ID = paste0("Sample", 1:30),
#'     TMEcluster = rep(c("A", "B", "C"), each = 10)
#'   )
#'   
#'   res <- find_markers_in_bulk(
#'     pdata = sim_pdata, eset = sim_eset,
#'     group = "TMEcluster", npcs = 5
#'   )
#'   if (!is.null(res)) head(res$top_markers)
#' }
find_markers_in_bulk <- function(pdata, eset, group, id_pdata = "ID", nfeatures = 2000, top_n = 20, thresh.use = 0.25, only.pos = TRUE, min.pct = 0.25, npcs = 30) {
  if (is.null(pdata) || is.null(eset)) return(NULL)
  # Check required packages
  rlang::check_installed(c("Seurat", "Matrix"))

  # 转换输入数据格式
  if (!inherits(eset, "dgCMatrix")) {
    eset <- methods::as(as.matrix(eset), "dgCMatrix")
  }

  # 处理元数据
  if (ncol(pdata) == 2) {
    pdata[, "newid"] <- pdata[, id_pdata]
  }
  pdata <- tibble::column_to_rownames(pdata, var = id_pdata)
  pdata <- pdata[rownames(pdata) %in% colnames(eset), ]

  # 创建Seurat对象
  sce <- Seurat::CreateSeuratObject(
    counts = eset,
    meta.data = pdata,
    min.cells = 0,
    min.features = 0,
    project = "sce"
  )

  # 获取版本号并进行规范比较
  seurat_version <- packageVersion("Seurat")
  v5_threshold <- package_version("5.0.0")

  # 版本适配处理
  if (seurat_version >= v5_threshold) {
    # Seurat v5+ 的处理逻辑
    message("Using Seurat v5+ workflow")
    sce <- Seurat::NormalizeData(sce)
    sce[["RNA"]]$data <- SeuratObject::LayerData(sce, assay = "RNA", layer = "counts") # 兼容v5的Layer访问
    sce <- Seurat::ScaleData(sce, layer = "data")
  } else {
    # Seurat v4及以下版本的处理逻辑
    message("Using Seurat v4 workflow")
    sce <- Seurat::NormalizeData(sce)
    sce <- Seurat::ScaleData(sce)
  }

  # 特征选择
  tryCatch(
    {
      sce <- Seurat::FindVariableFeatures(
        object = sce,
        selection.method = "vst",
        nfeatures = nfeatures,
        mean.cutoff = c(0.1, 8),
        dispersion.cutoff = c(1, Inf)
      )
    },
    error = function(e) {
      message("Error in FindVariableFeatures: ", e$message)
      counts <- Seurat::GetAssayData(sce, assay = "RNA", slot = "counts")
      gene_var <- apply(counts, 1, var)
      top_genes <- names(sort(gene_var, decreasing = TRUE))[1:nfeatures]
      Seurat::VariableFeatures(sce) <<- top_genes # Assign variable features
    }
  )

  # 降维和标记物识别
  if (ncol(sce) < npcs) {
    warning(
      "Number of samples (", ncol(sce), ") is less than requested PCs (", npcs, "). ",
      "Setting npcs to ", max(1, ncol(sce) - 1), "."
    )
    npcs <- max(1, ncol(sce) - 1) # 确保至少为1
  }

  sce <- Seurat::RunPCA(
    object = sce,
    features = Seurat::VariableFeatures(sce),
    npcs = npcs
  )
  Seurat::Idents(sce) <- as.character(sce[[group, drop = TRUE]])

  # 查找差异表达基因
  sce.markers <- Seurat::FindAllMarkers(
    object = sce,
    only.pos = only.pos,
    min.pct = min.pct,
    logfc.threshold = thresh.use,
    test.use = "wilcox"
  )

  # 提取top标记物
  topN <- sce.markers %>%
    dplyr::group_by(cluster) %>%
    dplyr::slice_max(order_by = avg_log2FC, n = top_n)

  return(list(sce = sce, markers = sce.markers, top_markers = topN))
}

#' Time-dependent ROC Curve for Survival Analysis
#'
#' @description
#' Generates a time-dependent Receiver Operating Characteristic (ROC) plot to evaluate
#' the predictive performance of one or more variables in survival analysis. Calculates
#' the Area Under the Curve (AUC) for each specified time point and variable, and
#' creates a multi-line ROC plot with annotated AUC values.
#'
#' @param input Data frame containing variables for analysis.
#' @param vars Character vector. Variable(s) to be evaluated.
#' @param time Character string. Name of the time variable. Default is `"time"`.
#' @param status Character string. Name of the status variable. Default is `"status"`.
#' @param time_point Integer or vector. Time point(s) for ROC analysis. Default is `12`.
#' @param time_type Character string. Time unit (`"day"` or `"month"`). Default is `"month"`.
#' @param palette Character string. Color palette for the plot. Default is `"jama"`.
#' @param cols Character vector or string. Color specification: `"normal"`, `"random"`,
#'   or custom color vector.
#' @param seed Integer. Random seed for reproducibility. Default is `1234`.
#' @param show_col Logical indicating whether to display the color palette.
#'   Default is `FALSE`.
#' @param path Character string or `NULL`. Path to save the plot. Default is `NULL`.
#' @param main Character string. Main title of the plot. Default is `"PFS"`.
#' @param index Integer. Index for plot filename. Default is `1`.
#' @param fig.type Character string. Output file type (e.g., `"pdf"`, `"png"`).
#'   Default is `"pdf"`.
#' @param width Numeric. Width of the plot. Default is `5`.
#' @param height Numeric. Height of the plot. Default is `5.2`.
#'
#' @return A ggplot object representing the time-dependent ROC plot.
#'
#' @export
#' @author Dongqiang Zeng
#' @import survival
#'
#' @examples
#' # Simulate data for offline testing
#' set.seed(123)
#' sim_input <- data.frame(
#'   ID = paste0("Sample", 1:100),
#'   time = runif(100, 1, 60),
#'   status = sample(0:1, 100, replace = TRUE),
#'   Marker1 = rnorm(100),
#'   Marker2 = rnorm(100),
#'   Marker3 = rnorm(100)
#' )
#' \dontrun{
#' roc_time(
#'   input = sim_input, vars = c("Marker1", "Marker2", "Marker3"),
#'   time = "time", status = "status", time_point = 12, path = NULL, main = "OS"
#' )
#' }
roc_time <- function(input, vars, time = "time", status = "status", time_point = 12,
                     time_type = "month", palette = "jama", cols = "normal",
                     seed = 1234, show_col = FALSE, path = NULL, main = "PFS",
                     index = 1, fig.type = "pdf", width = 5, height = 5.2) {
  rlang::check_installed("timeROC")
  rlang::check_installed("survival")

  # Ensure survival is attached for timeROC internal calls
  if (!"package:survival" %in% search()) {
    attachNamespace("survival")
    on.exit(detach("package:survival", character.only = TRUE, unload = FALSE), add = TRUE)
  }

  save_plot <- !is.null(path)

  if (save_plot) {
    folder_info <- creat_folder(path)
    save_path <- folder_info$folder_name
  }

  # Get colors
  if (is.null(cols)) cols <- "normal"
  cols <- get_cols(cols = cols, palette = palette, show_col = show_col, seed = seed)

  # Prepare data
  input <- as.data.frame(input)
  input <- input[, colnames(input) %in% c(time, status, vars)]
  colnames(input)[colnames(input) == time] <- "time"
  colnames(input)[colnames(input) == status] <- "status"

  # Validate and convert
  time_type <- tolower(time_type)
  unit_label <- match.arg(time_type, c("day", "month"))

  input$time <- as.numeric(input$time)
  input$status <- as.numeric(as.character(input$status))

  input <- input %>%
    dplyr::filter(!is.na(.data$time)) %>%
    dplyr::filter(!is.na(.data$status)) %>%
    dplyr::filter(.data$time > 0)

  if (nrow(input) == 0) {
    cli::cli_abort("No valid data after filtering")
  }

  cli::cli_alert_info("Time range: {paste(round(range(input$time, na.rm = TRUE), 2), collapse = ' to ')}")

  time_vec <- input$time
  status_vec <- input$status

  # Route to appropriate handler
  n_vars <- length(vars)
  if (n_vars == 1) {
    p <- .roc_single_var(input, vars, time_vec, status_vec, time_point, unit_label, cols, main)
  } else if (n_vars == 2) {
    p <- .roc_two_vars(input, vars, time_vec, status_vec, time_point, unit_label, cols, main)
  } else if (n_vars == 3) {
    p <- .roc_three_vars(input, vars, time_vec, status_vec, time_point, unit_label, cols, main)
  } else {
    cli::cli_abort("Only 1-3 variables are supported")
  }

  p <- p + design_mytheme(axis_angle = 0, hjust = 0.5, axis_title_size = 1.7)

  if (save_plot) {
    ggplot2::ggsave(
      filename = paste0(index, "-", main, "-ROC-time.", fig.type),
      plot = p,
      width = width, height = height, path = save_path
    )
  }

  p
}

#' ROC for single variable with 3 time points
#' @keywords internal
#' @noRd
.roc_single_var <- function(input, vars, time, status, time_point, unit_label, cols, main) {
  event_time <- time[status == 1]

  if (length(time_point) == 1) {
    if (length(event_time) >= 5) {
      time_point <- round(as.numeric(stats::quantile(event_time, probs = c(0.25, 0.5, 0.75), na.rm = TRUE)), 0)
    } else {
      cli::cli_warn("Too few events; using follow-up time quantiles")
      time_point <- round(as.numeric(stats::quantile(time, probs = c(0.25, 0.5, 0.75), na.rm = TRUE)), 0)
    }
  }

  if (length(time_point) != 3) {
    cli::cli_abort("For single variable, time_point must resolve to length 3")
  }

  time_point <- as.numeric(time_point)
  max_follow_up <- max(time, na.rm = TRUE)

  if (any(time_point > max_follow_up)) {
    cli::cli_abort("time_point exceeds maximum follow-up time ({max_follow_up})")
  }

  # Calculate ROCs
  rocs <- lapply(time_point, function(tp) {
    timeROC::timeROC(
      T = time, delta = status, marker = input[[vars]],
      cause = 1, weighting = "marginal", times = tp, iid = TRUE
    )
  })

  aucs <- vapply(rocs, function(r) round(as.numeric(r$AUC)[length(r$AUC)], 2), numeric(1))

  ggplot2::ggplot() +
    ggplot2::geom_line(ggplot2::aes(x = rocs[[1]]$FP[, 2], y = rocs[[1]]$TP[, 2]), color = cols[1]) +
    ggplot2::geom_line(ggplot2::aes(x = rocs[[2]]$FP[, 2], y = rocs[[2]]$TP[, 2]), color = cols[2]) +
    ggplot2::geom_line(ggplot2::aes(x = rocs[[3]]$FP[, 2], y = rocs[[3]]$TP[, 2]), color = cols[3]) +
    ggplot2::geom_line(ggplot2::aes(x = c(0, 1), y = c(0, 1)), color = "grey", linetype = "dashed") +
    ggplot2::theme_light() +
    ggplot2::annotate("text",
      x = 0.7, y = 0.35,
      label = paste("AUC of", time_point[1], unit_label, "=", aucs[1]), color = cols[1]
    ) +
    ggplot2::annotate("text",
      x = 0.7, y = 0.25,
      label = paste("AUC of", time_point[2], unit_label, "=", aucs[2]), color = cols[2]
    ) +
    ggplot2::annotate("text",
      x = 0.7, y = 0.15,
      label = paste("AUC of", time_point[3], unit_label, "=", aucs[3]), color = cols[3]
    ) +
    ggplot2::scale_x_continuous(name = "False Positive Rate") +
    ggplot2::scale_y_continuous(name = "True Positive Rate") +
    ggplot2::ggtitle(paste0(vars, ", ", main, " = ", paste0(time_point, collapse = ", "), " ", unit_label))
}

#' ROC for two variables
#' @keywords internal
#' @noRd
.roc_two_vars <- function(input, vars, time, status, time_point, unit_label, cols, main) {
  if (length(time_point) != 1) {
    cli::cli_abort("For 2+ variables, time_point must be a single value")
  }

  max_follow_up <- max(time, na.rm = TRUE)
  if (time_point > max_follow_up) {
    cli::cli_abort("time_point ({time_point}) exceeds maximum follow-up ({max_follow_up})")
  }

  var1 <- vars[1]
  var2 <- vars[2]

  cox_fit <- survival::coxph(survival::Surv(time, status) ~ input[[var1]] + input[[var2]], data = input)
  combined_score <- stats::predict(cox_fit, type = "lp", newdata = input)

  markers <- list(input[[var1]], input[[var2]], combined_score)
  rocs <- lapply(markers, function(m) {
    timeROC::timeROC(
      T = time, delta = status, marker = m, cause = 1,
      weighting = "marginal", times = time_point, iid = TRUE
    )
  })

  aucs <- vapply(rocs, function(r) round(as.numeric(r$AUC)[length(r$AUC)], 2), numeric(1))

  ggplot2::ggplot() +
    ggplot2::geom_line(ggplot2::aes(x = rocs[[1]]$FP[, 2], y = rocs[[1]]$TP[, 2]), color = cols[1]) +
    ggplot2::geom_line(ggplot2::aes(x = rocs[[2]]$FP[, 2], y = rocs[[2]]$TP[, 2]), color = cols[2]) +
    ggplot2::geom_line(ggplot2::aes(x = rocs[[3]]$FP[, 2], y = rocs[[3]]$TP[, 2]), color = cols[3]) +
    ggplot2::geom_line(ggplot2::aes(x = c(0, 1), y = c(0, 1)), color = "grey", linetype = "dashed") +
    ggplot2::theme_light() +
    ggplot2::annotate("text",
      x = 0.7, y = 0.35,
      label = paste("AUC of", var1, "=", aucs[1]), color = cols[1]
    ) +
    ggplot2::annotate("text",
      x = 0.7, y = 0.25,
      label = paste("AUC of", var2, "=", aucs[2]), color = cols[2]
    ) +
    ggplot2::annotate("text",
      x = 0.7, y = 0.15,
      label = paste("AUC of combined =", aucs[3]), color = cols[3]
    ) +
    ggplot2::scale_x_continuous(name = "False Positive Rate") +
    ggplot2::scale_y_continuous(name = "True Positive Rate") +
    ggplot2::ggtitle(paste0(main, " = ", time_point, " ", unit_label))
}

#' ROC for three variables
#' @keywords internal
#' @noRd
.roc_three_vars <- function(input, vars, time, status, time_point, unit_label, cols, main) {
  if (length(time_point) != 1) {
    cli::cli_abort("For 2+ variables, time_point must be a single value")
  }

  max_follow_up <- max(time, na.rm = TRUE)
  if (time_point > max_follow_up) {
    cli::cli_abort("time_point ({time_point}) exceeds maximum follow-up ({max_follow_up})")
  }

  var1 <- vars[1]
  var2 <- vars[2]
  var3 <- vars[3]

  cox_fit <- survival::coxph(survival::Surv(time, status) ~ input[[var1]] + input[[var2]] + input[[var3]], data = input)
  combined_score <- stats::predict(cox_fit, type = "lp", newdata = input)

  markers <- list(input[[var1]], input[[var2]], input[[var3]], combined_score)
  rocs <- lapply(markers, function(m) {
    timeROC::timeROC(
      T = time, delta = status, marker = m, cause = 1,
      weighting = "marginal", times = time_point, iid = TRUE
    )
  })

  aucs <- vapply(rocs, function(r) round(as.numeric(r$AUC)[length(r$AUC)], 2), numeric(1))

  ggplot2::ggplot() +
    ggplot2::geom_line(ggplot2::aes(x = rocs[[1]]$FP[, 2], y = rocs[[1]]$TP[, 2]), color = cols[1]) +
    ggplot2::geom_line(ggplot2::aes(x = rocs[[2]]$FP[, 2], y = rocs[[2]]$TP[, 2]), color = cols[2]) +
    ggplot2::geom_line(ggplot2::aes(x = rocs[[3]]$FP[, 2], y = rocs[[3]]$TP[, 2]), color = cols[3]) +
    ggplot2::geom_line(ggplot2::aes(x = rocs[[4]]$FP[, 2], y = rocs[[4]]$TP[, 2]), color = cols[4]) +
    ggplot2::geom_line(ggplot2::aes(x = c(0, 1), y = c(0, 1)), color = "grey", linetype = "dashed") +
    ggplot2::theme_light() +
    ggplot2::annotate("text",
      x = 0.7, y = 0.35,
      label = paste("AUC of", var1, "=", aucs[1]), color = cols[1]
    ) +
    ggplot2::annotate("text",
      x = 0.7, y = 0.25,
      label = paste("AUC of", var2, "=", aucs[2]), color = cols[2]
    ) +
    ggplot2::annotate("text",
      x = 0.7, y = 0.15,
      label = paste("AUC of", var3, "=", aucs[3]), color = cols[3]
    ) +
    ggplot2::annotate("text",
      x = 0.7, y = 0.05,
      label = paste("AUC of combined =", aucs[4]), color = cols[4]
    ) +
    ggplot2::scale_x_continuous(name = "False Positive Rate") +
    ggplot2::scale_y_continuous(name = "True Positive Rate") +
    ggplot2::ggtitle(paste0(main, " = ", time_point, " ", unit_label))
}

#' Integrative Correlation Analysis Between Phenotype and Features
#'
#' @description
#' Performs comprehensive correlation analysis between phenotype data and feature data,
#' supporting both continuous and categorical phenotypes. Filters features based on
#' statistical significance and generates publication-ready visualizations including
#' box plots, heatmaps, and correlation plots.
#'
#' @param pdata_group Data frame containing phenotype data with an identifier column.
#' @param id1 Character string specifying the column name in `pdata_group` serving
#'   as the sample identifier. Default is `"ID"`.
#' @param feature_data Data frame containing feature data with corresponding identifiers.
#' @param id2 Character string specifying the column name in `feature_data` serving
#'   as the sample identifier. Default is `"ID"`.
#' @param target Character string specifying the target variable column name for
#'   continuous analysis. Default is `NULL`.
#' @param group Character string specifying the grouping variable name for categorical
#'   analysis. Default is `"group3"`.
#' @param is_target_continuous Logical indicating whether the target variable is
#'   continuous, which affects grouping strategy. Default is `TRUE`.
#' @param padj_cutoff Numeric value specifying the adjusted p-value cutoff for filtering
#'   features. Default is `1`.
#' @param index Numeric index used for ordering output file names. Default is `1`.
#' @param signature_group List specifying the grouping variable for signatures.
#'   Options include `"sig_group"` for signature grouping or
#'   `"signature_collection"`/`"signature_tme"` for gene grouping.
#' @param category Character string specifying the data category: `"signature"`
#'   or `"gene"`.
#' @param ProjectID Character string specifying the project identifier for file naming.
#' @param feature_limit Integer specifying the maximum number of features to display.
#'   Default is `26`.
#' @param character_limit Integer specifying the maximum number of characters for
#'   variable labels. Default is `60`.
#' @param palette_box Character string or integer specifying the color palette for
#'   box plots. Default is `"nrc"`.
#' @param palette_corplot Character string or integer specifying the color palette for
#'   correlation plots. Default is `"pheatmap"`.
#' @param palette_heatmap Integer specifying the color palette index for heatmaps.
#'   Default is `2`.
#' @param show_heatmap_col_name Logical indicating whether to display column names on
#'   heatmaps. Default is `FALSE`.
#' @param show_col Logical indicating whether to display color codes for palettes.
#'   Default is `FALSE`.
#' @param show_plot Logical indicating whether to display plots. Default is `FALSE`.
#' @param path Character string specifying the directory path for saving output files.
#'   Default is `NULL`.
#' @param discrete_x Numeric threshold for character length beyond which labels are
#'   discretized. Default is `20`.
#' @param show_palettes Logical indicating whether to display color palettes. Default
#'   is `FALSE`.
#' @param discrete_width Numeric value specifying the width for label wrapping in plots.
#'   Default is `20`.
#' @param fig.type Character string specifying the format for saving figures (`"pdf"`,
#'   `"png"`, etc.). Default is `"pdf"`.
#' @param cols_box Character vector of specific colors for box plots. Default is
#'   `NULL`.
#'
#' @return Depending on configuration, returns ggplot2 objects (box plots, heatmaps,
#'   correlation plots) and/or a data frame containing statistical analysis results.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' set.seed(123)
#'
#' pdata_group <- data.frame(
#'   ID = 1:100,
#'   phenotype_score = rnorm(100)
#' )
#'
#' feature_data <- data.frame(
#'   ID = 1:100,
#'   Feature1 = rnorm(100),
#'   Feature2 = rnorm(100),
#'   Feature3 = rnorm(100)
#' )
#'
#' sig_group_example <- list(
#'   signature = c("Feature1", "Feature2", "Feature3")
#' )
#'
#' results <- iobr_cor_plot(
#'   pdata_group = pdata_group,
#'   feature_data = feature_data,
#'   id1 = "ID",
#'   id2 = "ID",
#'   target = "phenotype_score",
#'   is_target_continuous = TRUE,
#'   category = "signature",
#'   signature_group = sig_group_example,
#'   show_plot = FALSE,
#'   path = tempdir()
#' )
#'
#' print(results)
iobr_cor_plot <- function(pdata_group,
                          id1 = "ID",
                          feature_data,
                          id2 = "ID",
                          target = NULL,
                          group = "group3",
                          is_target_continuous = TRUE,
                          padj_cutoff = 1,
                          index = 1,
                          category = "signature",
                          signature_group = NULL,
                          ProjectID = "TCGA",
                          palette_box = "nrc",
                          cols_box = NULL,
                          palette_corplot = "pheatmap",
                          palette_heatmap = 2,
                          feature_limit = 26,
                          character_limit = 60,
                          show_heatmap_col_name = FALSE,
                          show_col = FALSE,
                          show_plot = FALSE,
                          path = NULL,
                          discrete_x = 20,
                          discrete_width = 20,
                          show_palettes = FALSE,
                          fig.type = "pdf") {
  rlang::check_installed("ggpubr")
  rlang::check_installed("tidyHeatmap")

  if (is.null(signature_group)) {
    cli::cli_abort("{.arg signature_group} must be provided")
  }

  # Create output directory
  save_results <- !is.null(path)
  if (save_results) {
    file_store <- path
    if (!dir.exists(file_store)) dir.create(file_store, recursive = TRUE)
    abspath <- file.path(normalizePath(file_store, winslash = "/", mustWork = FALSE), "")
  } else {
    abspath <- NULL
  }

  if (is.null(names(signature_group))) {
    signature_group <- list("signature" = signature_group)
  }

  # Prepare phenotype data
  pdata_group <- as.data.frame(pdata_group)
  colnames(pdata_group)[colnames(pdata_group) == id1] <- "ID"

  if (!is.null(target)) {
    if (!target %in% colnames(pdata_group)) {
      cli::cli_abort("Target {.val {target}} not found in pdata_group")
    }
    if (is_target_continuous) pdata_group[[target]] <- as.numeric(pdata_group[[target]])
  }

  # Create grouping variables
  if (is_target_continuous && !"group2" %in% colnames(pdata_group)) {
    mean_val <- mean(pdata_group[[target]], na.rm = TRUE)
    pdata_group$group2 <- ifelse(pdata_group[[target]] >= mean_val, "High", "Low")
  }

  if (is_target_continuous && !"group3" %in% colnames(pdata_group)) {
    q1 <- stats::quantile(pdata_group[[target]], probs = 1 / 3, na.rm = TRUE)
    q2 <- stats::quantile(pdata_group[[target]], probs = 2 / 3, na.rm = TRUE)
    pdata_group$group3 <- ifelse(
      pdata_group[[target]] <= q1, "Low",
      ifelse(pdata_group[[target]] >= q2, "High", "Middle")
    )
  }

  # Select relevant columns
  if (!is.null(target) && is_target_continuous) {
    pdata_group <- pdata_group[, c("ID", target, "group2", "group3")]
  } else {
    pdata_group <- pdata_group[, c("ID", group)]
  }

  # Prepare feature data
  if (category == "gene") {
    feature_data <- log2eset(feature_data)
    check_eset(feature_data)
    feature_data <- as.data.frame(t(feature_data))
    feature_data <- tibble::rownames_to_column(feature_data, var = "ID")
  }

  feature_data <- as.data.frame(feature_data)

  if (category == "signature") {
    if (!id2 %in% colnames(feature_data)) {
      cli::cli_abort("id2 {.val {id2}} not found in feature_data")
    }
    colnames(feature_data)[colnames(feature_data) == id2] <- "ID"
  }

  feature_selected <- feature_manipulation(
    data = feature_data,
    feature = setdiff(colnames(feature_data), "ID")
  )

  feature_data <- feature_data[, colnames(feature_data) %in% c("ID", feature_selected)]

  if (!is.null(target) && target %in% colnames(feature_data)) {
    feature_data <- feature_data[, colnames(feature_data) != target]
  }

  group_list <- signature_group
  panel <- names(signature_group)
  feature_data <- feature_data[, colnames(feature_data) %in% c("ID", unique(unlist(group_list)))]

  if (any(duplicated(colnames(feature_data)))) {
    cli::cli_abort("Duplicate column names found in feature_data")
  }

  # Merge data
  pf <- merge(pdata_group, feature_data, by = "ID", all = FALSE)
  scale_begin <- length(colnames(pdata_group)) + 1
  pf[, scale_begin:ncol(pf)] <- scale(pf[, scale_begin:ncol(pf)], center = TRUE, scale = TRUE)

  pf_stat <- pf

  # Validate signature_group
  if (!inherits(signature_group, "list")) {
    cli::cli_abort("{.arg signature_group} must be a list")
  }

  all_sig <- unique(unlist(group_list))

  if (length(intersect(colnames(pf), all_sig)) == 0) {
    cli::cli_abort(c(
      "No matching {category} found",
      i = "Check that signature_group matches the data"
    ))
  }

  # Set axis titles
  title.y <- if (category == "signature") "Signature score" else "Gene expression"
  title.x <- if (category == "signature") "Signatures" else "Signature genes"

  # Process each signature group
  for (x in seq_along(panel)) {
    group_name <- panel[x]
    features <- group_list[[x]]
    features <- features[features %in% colnames(pf)]

    if (length(features) < 2) {
      cli::cli_alert_warning("Panel {.val {group_name}} has fewer than 2 features; skipping")
      next
    }

    cli::cli_alert_info("Processing signature: {.val {group_name}}")

    # Select top features if too many
    if (length(features) > feature_limit) {
      if (!is_target_continuous) {
        eset <- pf[, colnames(pf) %in% c(group, features)]
        n_groups <- length(unique(eset[[group]]))
        if (n_groups == 2) {
          res <- batch_wilcoxon(data = eset, target = group, feature = setdiff(colnames(eset), group))
        } else {
          res <- batch_kruskal(data = eset, group = group, feature = setdiff(colnames(eset), group))
        }
        good_features <- high_var_fea(
          result = res, target = "sig_names", name_padj = "p.adj",
          padj_cutoff = padj_cutoff, name_logfc = "statistic",
          logfc_cutoff = 0, n = feature_limit / 2
        )
      } else {
        eset <- pf[, colnames(pf) %in% c(target, features)]
        res <- batch_cor(
          data = eset, target = target,
          feature = setdiff(colnames(eset), target),
          method = "spearman"
        )
        good_features <- high_var_fea(
          result = res, target = "sig_names", name_padj = "p.adj",
          padj_cutoff = padj_cutoff, name_logfc = "statistic",
          logfc_cutoff = 0, n = feature_limit / 2
        )
      }

      if (length(good_features) <= 2) {
        good_features <- res$sig_names[seq_len(min(6, nrow(res)))]
        cli::cli_alert_warning("Panel {.val {group_name}}: No statistically significant features")
      }

      features <- features[features %in% good_features]
    }

    # Prepare long format data
    if (!is.null(target)) {
      # Select ID, target, group columns, and feature columns
      pf_inter <- tibble::as_tibble(pf[, c("ID", target, group, features)])
      pf_long <- tidyr::pivot_longer(pf_inter, (3 + length(group)):ncol(pf_inter),
        names_to = "variables", values_to = "value"
      )
      pf_long$value <- as.numeric(pf_long$value)
    } else {
      pf_inter <- tibble::as_tibble(pf[, c("ID", group, features)])
      pf_long <- tidyr::pivot_longer(pf_inter, 3:ncol(pf_inter),
        names_to = "variables", values_to = "value",
        values_transform = list(value = as.numeric)
      )
    }

    pf_long$variables <- substring(pf_long$variables, 1, character_limit)

    # Determine target binary variable
    target_binary <- if (group == "group3") {
      pf_long <- pf_long[pf_long$group3 != "Middle", ]
      "group3"
    } else if (group == "group2") {
      "group2"
    } else {
      group
    }

    pf_long_group <- pf_long
    pf_long_group_box <- pf_long_group

    if (max(nchar(as.character(pf_long_group$variables))) > discrete_x) {
      long_idx <- nchar(as.character(pf_long_group_box$variables)) > discrete_x
      pf_long_group_box$variables[long_idx] <- gsub("_", " ", as.character(pf_long_group_box$variables[long_idx]))
    }

    # Get colors
    color_box <- cols_box %||% palettes(
      category = "box", palette = palette_box,
      show_col = show_col, show_message = show_palettes
    )

    # Create box plot
    axis_text_size <- max(8, 18 - max(nchar(as.character(pf_long$variables))) / 7)

    p <- ggpubr::ggboxplot(pf_long_group_box,
      x = "variables", y = "value",
      fill = target_binary
    ) +
      ggplot2::scale_fill_manual(values = color_box) +
      ggplot2::ylab(title.y) +
      ggplot2::ggtitle(group_name) +
      ggplot2::theme_light() +
      ggplot2::theme(
        plot.title = ggplot2::element_text(size = ggplot2::rel(2), hjust = 0.5),
        axis.title.y = ggplot2::element_text(size = ggplot2::rel(1.5)),
        axis.title.x = ggplot2::element_blank(),
        axis.text.x = ggplot2::element_text(
          face = "plain", size = axis_text_size,
          angle = 60, hjust = 1, color = "black"
        ),
        axis.text.y = ggplot2::element_text(
          face = "plain", size = 15, angle = 0,
          hjust = 1, color = "black"
        ),
        axis.line = ggplot2::element_line(color = "black", linewidth = 0.5),
        legend.key.size = ggplot2::unit(0.3, "inches"),
        legend.title = ggplot2::element_blank(),
        legend.position = "bottom",
        legend.direction = "horizontal",
        legend.justification = c(0.5, 0.5),
        legend.box = "horizontal",
        legend.box.just = "top",
        legend.text = ggplot2::element_text(colour = "black", size = 10, face = "plain")
      ) +
      ggplot2::scale_x_discrete(labels = function(x) stringr::str_wrap(x, width = discrete_width))

    max_variables <- max(pf_long_group_box$value, na.rm = TRUE)
    group_box <- rlang::sym(target_binary)

    pp1 <- p + ggpubr::stat_compare_means(
      ggplot2::aes(group = !!group_box, label = paste0("p = ", ggplot2::after_stat(p.format))),
      size = 2.6, label.y = max_variables - 0.3
    )
    pp2 <- p + ggpubr::stat_compare_means(
      ggplot2::aes(group = !!group_box),
      label = "p.signif",
      size = 6, label.y = max_variables - 0.6
    )

    if (show_plot && length(features) < 13) {
      if (interactive()) print(pp1)
    } else if (show_plot && length(features) > 13) {
      if (interactive()) print(pp2)
    }

    plot_width <- length(features) * 0.4 + 3
    plot_height <- 4 + max(nchar(as.character(pf_long_group_box$variables))) * 0.05

    fig_ext <- if (fig.type == "pdf") "pdf" else "png"

    prefix <- if (!is.null(target)) target else group
    if (save_results) {
      ggplot2::ggsave(
        filename = paste0(
          index, "-", x, "-1-", ProjectID, "-", prefix,
          "-", group_name, "-pvalue-box.", fig_ext
        ),
        plot = pp1,
        width = plot_width, height = plot_height, path = file_store
      )
      ggplot2::ggsave(
        filename = paste0(
          index, "-", x, "-2-", ProjectID, "-", prefix,
          "-", group_name, "-box.", fig_ext
        ),
        plot = pp2,
        width = plot_width, height = plot_height, path = file_store
      )
    }

    # Create heatmap
    colnames(pf_long_group)[colnames(pf_long_group) == group] <- "target_group"
    pf_long_group$value <- pmin(pmax(pf_long_group$value, -2.5), 2.5)

    height_heatmap <- length(features) * 0.2 + 3
    heatmap_col <- palettes(
      category = "tidyheatmap", palette = palette_heatmap,
      show_col = show_col, show_message = show_palettes
    )

    pp <- pf_long_group %>%
      dplyr::group_by(.data$target_group) %>%
      tidyHeatmap::heatmap(
        .column = ID, .row = variables, .value = value,
        palette_grouping = list(c(color_box)),
        palette_value = heatmap_col,
        show_column_names = show_heatmap_col_name
      )

    if (show_plot) print(pp)

    if (save_results) {
      pp %>% tidyHeatmap::save_pdf(
        paste0(
          abspath, index, "-", x, "-3-", ProjectID, "-", group, "-", group_name,
          "-tidyheatmap.pdf"
        ),
        width = 8, height = height_heatmap
      )
    }

    # Create correlation plot for continuous targets
    if (is_target_continuous && length(group_list[[x]]) <= 20) {
      pf_cor <- pf[, colnames(pf) %in% c(target, features)]

      rlang::check_installed("Hmisc")
      bbcor <- Hmisc::rcorr(as.matrix(pf_cor), type = "spearman")
      bbcor$P[is.na(bbcor$P)] <- 0

      col <- palettes(
        category = "heatmap3", palette = palette_corplot,
        show_col = show_col, show_message = show_palettes
      )

      width_heatmap <- length(group_list[[x]]) * 0.75 + 5
      height_heatmap <- length(group_list[[x]]) * 0.75 + 4

      if (save_results) {
        grDevices::pdf(
          file = paste0(
            abspath, index, "-", x, "-4-", ProjectID, "-", group_name,
            "-associated-", category, "-corplot.pdf"
          ),
          width = width_heatmap,
          height = height_heatmap
        )
        rlang::check_installed("corrplot")
        corrplot::corrplot(bbcor$r,
          type = "lower", order = "hclust", p.mat = bbcor$P,
          sig.level = 0.05, tl.srt = 45, tl.col = "black",
          tl.cex = 1.3, addrect = 2, rect.col = "black",
          rect.lwd = 3, col = grDevices::colorRampPalette(col)(50)
        )
        grDevices::dev.off()
      }

      corrplot::corrplot(bbcor$r,
        type = "lower", order = "hclust", p.mat = bbcor$P,
        sig.level = 0.05, tl.srt = 45, tl.col = "black",
        tl.cex = 1, addrect = 2, rect.col = "black",
        rect.lwd = 3, col = grDevices::colorRampPalette(col)(50)
      )

      # Create alternative correlation plot with ggplot2
      lab_size <- max(2, 13 - max(nchar(as.character(pf_long_group$variables))) / 4)
      tl_cex <- max(8, 20 - max(nchar(as.character(pf_long_group$variables))) / 9)

      corr <- bbcor$r
      p.mat <- bbcor$P
      corr[upper.tri(corr)] <- NA
      p.mat[upper.tri(p.mat)] <- NA

      hc <- stats::hclust(stats::dist(corr))
      corr <- corr[hc$order, hc$order]
      p.mat <- p.mat[hc$order, hc$order]

      df <- as.data.frame.table(corr, stringsAsFactors = FALSE)
      colnames(df) <- c("row", "col", "corr")
      df$row <- factor(df$row, levels = rev(unique(df$row)))
      df$col <- factor(df$col, levels = unique(df$col))

      # Create p-value data frame with same structure as corr
      df_p <- as.data.frame.table(p.mat, stringsAsFactors = FALSE)
      df$stars <- ifelse(df_p$Freq < 0.05 & !is.na(df_p$Freq), "*", "")
      col_fun <- grDevices::colorRampPalette(c("darkblue", "white", "darkred"))

      p <- ggplot2::ggplot(df, ggplot2::aes(
        x = .data$col, y = .data$row,
        fill = .data$corr,
        label = sprintf("%.2f", .data$corr)
      )) +
        ggplot2::geom_tile(color = "grey90") +
        ggplot2::geom_text(color = "black", size = lab_size) +
        ggplot2::geom_text(ggplot2::aes(label = .data$stars),
          color = "red",
          size = lab_size * 1.2, fontface = "bold"
        ) +
        ggplot2::scale_fill_gradientn(colours = col_fun(50), limits = c(-1, 1)) +
        ggplot2::theme_bw() +
        ggplot2::theme(
          axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = tl_cex * 0.3),
          axis.text.y = ggplot2::element_text(size = tl_cex * 0.3),
          plot.title = ggplot2::element_text(hjust = 0.5)
        ) +
        ggplot2::ggtitle(group_name)

      if (save_results) {
        ggplot2::ggsave(
          filename = paste0(
            index, "-", x, "-5-", ProjectID, "-", group_name,
            "-associated-", category, "-corplot.", fig_ext
          ),
          plot = p,
          width = 12, height = 12.8, path = file_store
        )
      }
    }
  }

  # Return statistical results
  if (!is_target_continuous) {
    n_groups <- length(unique(pf_stat[[group]]))
    cli::cli_alert_info("{n_groups}-group comparison: {table(pf_stat[[group]])}")
    eset <- pf_stat
    feas <- colnames(pf_stat)[scale_begin:ncol(pf_stat)]
    if (n_groups == 2) {
      res <- batch_wilcoxon(data = eset, target = group, feature = feas, feature_manipulation = TRUE)
    } else {
      res <- batch_kruskal(data = eset, group = group, feature = feas, feature_manipulation = TRUE)
    }
    res <- tibble::as_tibble(res)
  } else {
    if (is.null(target)) cli::cli_abort("target must be defined for continuous analysis")
    eset <- pf_stat
    feas <- colnames(pf_stat)[scale_begin:ncol(pf_stat)]
    res <- batch_cor(data = eset, target = target, feature = feas, method = "spearman")
    res <- tibble::as_tibble(res)
  }

  invisible(res)
}

#' Extract Data Frame from Seurat Object
#'
#' @description
#' Extracts and combines a data frame with cells as rows and features as columns
#' from Seurat assay data. Supports multiple assays and optional metadata
#' integration.
#'
#' @param sce Seurat object.
#' @param vars Character vector of feature names to extract. If `NULL`, all
#'   features are extracted.
#' @param assay Character vector specifying assay(s) to pull data from.
#' @param slot Character string specifying the assay data slot. Default is
#'   `"scale.data"`.
#' @param combine_meta_data Logical indicating whether to combine metadata
#'   with the extracted data frame. Default is `TRUE`.
#'
#' @return Data frame with cells as rows and features as columns.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' if (requireNamespace("Seurat", quietly = TRUE)) {
#'   pbmc <- SeuratObject::pbmc_small
#'   vars <- c("PPBP", "IGLL5", "VDAC3", "CD1C", "AKR1C3")
#'   eset <- extract_sc_data(sce = pbmc, vars = vars, assay = "RNA")
#' }
extract_sc_data <- function(sce, vars = NULL, assay, slot = "scale.data", combine_meta_data = TRUE) {
  rlang::check_installed(c("Seurat", "SeuratObject"))

  if (!inherits(sce, "Seurat")) {
    cli::cli_abort("{.arg sce} must be a Seurat object")
  }

  exist <- Seurat::Assays(sce)
  cli::cli_alert_info("Available assays: {.val {exist}}")

  assay <- assay[assay %in% exist]
  if (length(assay) == 0) {
    cli::cli_abort("No valid assay found in object")
  }

  eset_cbind <- data.frame(
    ID = rownames(sce@meta.data),
    index = seq_len(nrow(sce@meta.data))
  )

  for (i in seq_along(assay)) {
    method <- assay[i]
    Seurat::DefaultAssay(sce) <- method

    if (packageVersion("SeuratObject") < "5.0.0") {
      eset <- SeuratObject::GetAssayData(sce, assay = method, slot = slot)
    } else {
      eset <- SeuratObject::GetAssayData(sce, assay = method, layer = slot)
    }


    if (!is.null(vars)) {
      feas <- rownames(eset)[rownames(eset) %in% unique(vars)]
      if (length(feas) == 0) {
        cli::cli_abort("Required variables not found in expression matrix")
      }
      eset <- eset[feas, , drop = FALSE]
    }

    eset <- as.data.frame(t(as.matrix(eset)))

    if (!is.null(vars) && length(vars) == 1) {
      eset <- as.data.frame(eset)
      rownames(eset) <- vars
      eset <- t(eset)
      eset <- as.data.frame(eset)
    }

    eset <- tibble::rownames_to_column(eset, var = "ID")

    if (length(assay) > 1) {
      colnames(eset)[2:ncol(eset)] <- paste0(
        colnames(eset)[2:ncol(eset)], "_", method
      )
    }

    eset <- as.data.frame(eset)

    if (length(assay) == 1) {
      eset_cbind <- eset
    } else {
      eset_cbind <- dplyr::inner_join(eset_cbind, eset, by = "ID")
    }
  }

  if (combine_meta_data) {
    cli::cli_alert_info("Merging metadata...")
    meta.data <- tibble::rownames_to_column(sce@meta.data, var = "ID")
    eset_cbind <- dplyr::inner_join(meta.data, eset_cbind, by = "ID")
  }

  eset_cbind
}

#' Transform NA, Inf, or Zero Values in Data
#'
#' @description
#' Replaces NA, Inf, or zero values in specified columns of a data frame with a
#' user-defined value or the column mean.
#'
#' @param data Data frame. Input data to be transformed.
#' @param feature Character vector. Column names in `data` to apply
#'   transformation.
#' @param data_type Character. Type of value to replace: `"NA"`, `"Inf"`, or
#'   `"zero"`.
#' @param into Value to replace specified type with. Default is 0. If `"mean"`,
#'   replaces with column mean (excluding NA/Inf values as appropriate).
#'
#' @return Data frame with specified transformations applied to selected
#'   features.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' data_matrix <- data.frame(
#'   A = c(1, 2, NA, 4, Inf),
#'   B = c(Inf, 2, 3, 4, 5),
#'   C = c(0, 0, 0, 1, 2)
#' )
#'
#' # Replace NAs with 0
#' transform_data(data_matrix, feature = c("A", "B"), data_type = "NA")
#'
#' # Replace Inf values with the mean of the column
#' transform_data(data_matrix,
#'   feature = c("A", "B"),
#'   data_type = "Inf", into = "mean"
#' )
#'
#' # Replace zeros with -1 in column C
#' transform_data(data_matrix, feature = "C", data_type = "zero", into = -1)
transform_data <- function(data,
                           feature,
                           data_type = c("NA", "Inf", "zero"),
                           into = 0) {
  # Input validation
  if (!is.data.frame(data)) {
    cli::cli_abort(c(
      "Invalid input type.",
      "i" = "Expected a data frame, got {.cls {class(data)}}."
    ))
  }

  data_type <- rlang::arg_match(data_type)

  # Filter to valid features
  valid_features <- feature[feature %in% colnames(data)]
  invalid_features <- setdiff(feature, colnames(data))

  if (length(valid_features) == 0) {
    cli::cli_warn("No valid features found in data. Returning unchanged.")
    return(data)
  }

  if (length(invalid_features) > 0) {
    cli::cli_alert_warning(
      "Ignoring invalid feature{?s}: {.val {invalid_features}}"
    )
  }

  # Get column indices
  col_idx <- match(valid_features, colnames(data))

  # Define replacement function based on data_type
  replace_values <- switch(data_type,
    "NA"   = function(x) is.na(x),
    "Inf"  = function(x) is.infinite(x),
    "zero" = function(x) x == 0
  )

  # Apply transformation to each column
  for (i in seq_along(valid_features)) {
    col_name <- valid_features[i]
    j <- col_idx[i]

    mask <- replace_values(data[[j]])

    if (any(mask)) {
      if (identical(into, "mean")) {
        # Calculate mean excluding the values being replaced
        valid_vals <- data[[j]][!mask]
        replacement <- mean(valid_vals, na.rm = TRUE)
      } else {
        replacement <- into
      }
      data[mask, j] <- replacement
    }
  }

  data
}

#' Transform Signature Data into List Format
#'
#' @description
#' Converts signature data from a data frame (with signatures as columns and
#' genes as rows) into a list format suitable for IOBR functions. Handles
#' NA values appropriately.
#'
#' @param sig_data Data frame with signature names as columns and genes in
#'   rows. Use `NA` for missing values.
#' @param save_signature Logical. Whether to save the signature list as RData.
#'   Default is `FALSE`.
#' @param output_path Character. Full path (without extension) for output file.
#'   Required if `save_signature = TRUE`. Default is `NULL`.
#'
#' @return List of signatures.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' sig_data <- data.frame(
#'   Signature1 = c("Gene1", "Gene2", "Gene3", NA),
#'   Signature2 = c("Gene4", "Gene5", NA, NA)
#' )
#' format_signatures(sig_data)
format_signatures <- function(sig_data, save_signature = FALSE, output_path = NULL) {
  if (!is.data.frame(sig_data) && !is.matrix(sig_data)) {
    cli::cli_abort("{.arg sig_data} must be a data frame or matrix")
  }
  if (ncol(sig_data) == 0) {
    cli::cli_abort("{.arg sig_data} must have at least one column")
  }

  cli::cli_alert_info("There are {ncol(sig_data)} signatures")

  sig_data <- as.data.frame(sig_data)
  sig_data[sig_data == "NA"] <- NA

  bb <- lapply(seq_len(ncol(sig_data)), function(i) {
    aa <- as.character(sig_data[, i])
    aa <- aa[!is.na(aa)]
    aa <- aa[aa != ""]
    unique(aa)
  })
  names(bb) <- names(sig_data)

  names(bb) <- gsub(names(bb), pattern = "\\ ", replacement = "_")

  my_signatures <- bb

  if (save_signature) {
    if (is.null(output_path)) {
      cli::cli_abort("{.arg output_path} must be provided when save_signature = TRUE")
    }
    save(my_signatures, file = paste0(output_path, ".RData"))
  }

  my_signatures
}

#' Convert Mouse Gene Symbols to Human Gene Symbols
#'
#' @description
#' Converts mouse gene symbols to human gene symbols in an expression dataset.
#' Supports using either an online resource (Ensembl) or a local dataset for
#' conversion.
#'
#' @param eset Matrix or data frame. Expression matrix with genes in rows.
#' @param source Character. Data source for conversion: "ensembl" (online) or
#'   "local". Default is "ensembl". If Ensembl fails, use "local" which uses
#'   the internal `mus_human_gene_symbol` dataset.
#' @param is_matrix Logical. Whether `eset` is a matrix with gene symbols as
#'   row names. Default is `TRUE`. If `FALSE`, `column_of_symbol` must be
#'   specified.
#' @param column_of_symbol Character or `NULL`. Column name containing gene
#'   symbols if `eset` is not a matrix. Default is `NULL`.
#' @param verbose Logical. If `TRUE`, prints available Ensembl datasets.
#'   Default is `FALSE`.
#'
#' @return Expression set with human gene symbols.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' \dontrun{
#' set.seed(123)
#' data <- matrix(runif(50 * 3), nrow = 50, ncol = 3)
#' rownames(data) <- c("Tpt1", "Hmgb1", "Gapdh", paste0("Gene", 4:50))
#' colnames(data) <- paste0("Sample", 1:3)
#' human_data <- mouse2human_eset(data, source = "local", is_matrix = TRUE)
#' if (!is.null(human_data)) head(human_data)
#' }
mouse2human_eset <- function(eset,
                             source = c("local", "ensembl"),
                             is_matrix = TRUE,
                             column_of_symbol = NULL,
                             verbose = FALSE) {
  if (is.null(eset)) return(NULL)
  source <- rlang::arg_match(source)

  if (!is.matrix(eset) && !is.data.frame(eset)) {
    cli::cli_abort("{.arg eset} must be a matrix or data frame.")
  }

  if (is_matrix) {
    if (is.null(rownames(eset))) {
      cli::cli_abort("{.arg eset} must have row names when {.code is_matrix = TRUE}.")
    }
    genes <- rownames(eset)
  } else {
    if (is.null(column_of_symbol)) {
      cli::cli_abort("{.arg column_of_symbol} must be specified when {.code is_matrix = FALSE}.")
    }
    if (!column_of_symbol %in% colnames(eset)) {
      cli::cli_abort("Column {.val {column_of_symbol}} not found in {.arg eset}.")
    }
    eset <- remove_duplicate_genes(eset = eset, column_of_symbol = column_of_symbol)
    genes <- rownames(eset)
  }

  if (source == "ensembl") {
    rlang::check_installed("biomaRt")
    probe_data <- tryCatch(
      {
        ensembl <- biomaRt::useEnsembl(biomart = "ensembl")
        if (verbose) {
          datasets <- biomaRt::listDatasets(ensembl)
          message(paste(capture.output(head(datasets)), collapse = "\n"))
        }

        human <- biomaRt::useMart("ensembl", dataset = "hsapiens_gene_ensembl")
        mouse <- biomaRt::useMart("ensembl", dataset = "mmusculus_gene_ensembl")

        probe_data <- biomaRt::getLDS(
          attributes = c("mgi_symbol"),
          filters = "mgi_symbol",
          values = genes,
          mart = mouse,
          attributesL = c("hgnc_symbol"),
          martL = human,
          uniqueRows = TRUE
        )
        colnames(probe_data) <- c("gene_symbol_mus", "gene_symbol_human")
        probe_data
      },
      error = function(e) {
        cli::cli_warn("Failed to connect to Ensembl: {e$message}")
        cli::cli_alert_info("Falling back to local database.")
        load_data("mus_human_gene_symbol")
      }
    )
  } else {
    probe_data <- load_data("mus_human_gene_symbol")
  }

  if (is.null(probe_data)) return(NULL)

  if (nrow(probe_data) == 0) {
    cli::cli_abort("No gene mappings found for the provided genes.")
  }

  eset <- anno_eset(
    eset = eset,
    annotation = probe_data,
    symbol = "gene_symbol_human",
    probe = "gene_symbol_mus",
    method = "mean"
  )

  eset
}

#' Perform Gene Set Enrichment Analysis (GSEA)
#'
#' @description
#' Conducts Gene Set Enrichment Analysis to identify significantly enriched gene sets
#' from differential gene expression data. Supports MSigDB gene sets or custom gene
#' signatures, and generates comprehensive visualizations and statistical results.
#'
#' @param deg Data frame containing differential expression results with gene symbols
#'   and log fold changes.
#' @param genesets List of custom gene sets for enrichment analysis. If `NULL`,
#'   MSigDB gene sets are used based on `org` and `category`. Default is `NULL`.
#' @param path Character string specifying the directory path for saving results.
#'   Default is `NULL`.
#' @param gene_symbol Character string specifying the column name in `deg`
#'   containing gene symbols. Default is `"symbol"`.
#' @param logfc Character string specifying the column name in `deg` containing
#'   log fold change values. Default is `"log2FoldChange"`.
#' @param org Character string specifying the organism. Options are `"hsa"`
#'   (Homo sapiens) or `"mus"` (Mus musculus). Default is `"hsa"`.
#' @param msigdb Logical indicating whether to use MSigDB gene sets. Default is
#'   `TRUE`.
#' @param category Character string specifying the MSigDB category (e.g., `"H"`
#'   for Hallmark, `"C2"` for curated gene sets). Default is `"H"`.
#' @param subcategory Character string specifying the MSigDB subcategory to filter
#'   gene sets. Default is `NULL`.
#' @param palette_bar Character string or integer specifying the color palette for
#'   bar plots. Default is `"jama"`.
#' @param palette_gsea Integer specifying the color palette for GSEA plots. Default
#'   is `2`.
#' @param cols_gsea Character vector specifying custom colors for GSEA enrichment
#'   plots. If `NULL`, colors are automatically generated. Default is `NULL`.
#' @param cols_bar Character vector specifying custom colors for the enrichment
#'   bar plot. If `NULL`, colors are automatically generated. Default is `NULL`.
#' @param show_bar Integer specifying the number of top enriched gene sets to display
#'   in the bar plot. Default is `10`.
#' @param show_col Logical indicating whether to display color names in the bar plot.
#'   Default is `FALSE`.
#' @param show_plot Logical indicating whether to display GSEA enrichment plots.
#'   Default is `FALSE`.
#' @param show_gsea Integer specifying the number of top significant gene sets for
#'   which to generate GSEA plots. Default is `8`.
#' @param show_path_n Integer specifying the number of pathways to display in GSEA
#'   plots. Default is `20`.
#' @param plot_single_sig Logical indicating whether to generate separate plots for
#'   each significant gene set. Default is `TRUE`.
#' @param project Character string specifying the project name for output files.
#'   Default is `"custom_sig"`.
#' @param minGSSize Integer specifying the minimum gene set size for analysis.
#'   Default is `10`.
#' @param maxGSSize Integer specifying the maximum gene set size for analysis.
#'   Default is `500`.
#' @param verbose Logical indicating whether to display progress messages. Default
#'   is `TRUE`.
#' @param seed Logical indicating whether to set a random seed for reproducibility.
#'   Default is `FALSE`.
#' @param fig.type Character string specifying the file format for saving plots
#'   (e.g., `"pdf"`, `"png"`). Default is `"pdf"`.
#' @param print_bar Logical indicating whether to save and print the bar plot.
#'   Default is `TRUE`.
#'
#' @return List containing:
#' \describe{
#'   \item{up}{Data frame of up-regulated enriched gene sets}
#'   \item{down}{Data frame of down-regulated enriched gene sets}
#'   \item{all}{Complete GSEA results}
#'   \item{plot_top}{GSEA enrichment plot for top gene sets}
#' }
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' set.seed(123)
#' genes <- c(
#'   "TP53", "BRCA1", "EGFR", "MYC", "KRAS", "PTEN", "APC", "RB1",
#'   "CDKN2A", "VHL", "ATM", "ATR", "CHEK2", "PALB2", "RAD51", "MDM2",
#'   "CDK4", "CDK6", "CCND1", "CCNE1", "CDK2", "E2F1", "E2F2", "E2F3",
#'   "ARF1", "ARF3", "ARF4", "ARF5", "ARF6", "GSK3B", "AKT1", "AKT2",
#'   "PIK3CA", "PIK3CB", "PIK3CD", "PIK3CG", "PIK3R1", "PIK3R2", "PIK3R3"
#' )
#' deg <- data.frame(
#'   symbol = genes,
#'   log2FoldChange = rnorm(length(genes), mean = 0, sd = 2),
#'   padj = runif(length(genes), 0, 0.1)
#' )
#' signature <- list(
#'   DNA_Repair = c(
#'     "TP53", "BRCA1", "ATM",
#'     "ATR", "CHEK2", "PALB2", "RAD51"
#'   ),
#'   Cell_Cycle = c(
#'     "TP53", "MYC",
#'     "RB1", "CDKN2A", "CDK4",
#'     "CDK6", "CCND1", "CCNE1",
#'     "CDK2", "E2F1", "E2F2", "E2F3"
#'   ),
#'   PI3K_AKT = c(
#'     "AKT1", "AKT2",
#'     "PIK3CA", "PIK3CB", "PIK3CD",
#'     "PIK3CG", "PIK3R1", "PIK3R2", "PIK3R3"
#'   )
#' )
#' \donttest{
#' res <- sig_gsea(
#'   deg = deg,
#'   genesets = signature,
#'   path = tempdir(),
#'   show_plot = FALSE,
#'   print_bar = FALSE
#' )
#' print(names(res))
#' }
sig_gsea <- function(deg,
                     genesets = NULL,
                     path = NULL,
                     gene_symbol = "symbol",
                     logfc = "log2FoldChange",
                     org = c("hsa", "mus"),
                     msigdb = TRUE,
                     category = "H",
                     subcategory = NULL,
                     palette_bar = "jama",
                     palette_gsea = 2,
                     cols_gsea = NULL,
                     cols_bar = NULL,
                     show_bar = 10,
                     show_col = FALSE,
                     show_plot = FALSE,
                     show_gsea = 8,
                     show_path_n = 20,
                     plot_single_sig = FALSE,
                     project = "custom_sig",
                     minGSSize = 10,
                     maxGSSize = 500,
                     verbose = TRUE,
                     seed = FALSE,
                     fig.type = "pdf",
                     print_bar = TRUE) {
  org <- rlang::arg_match(org)

  # Input validation
  if (is.null(deg) || !is.data.frame(deg)) {
    cli::cli_abort("{.arg deg} must be a data frame with differential expression results")
  }
  if (!gene_symbol %in% colnames(deg)) {
    cli::cli_abort("Gene symbol column {.val {gene_symbol}} not found in deg")
  }
  if (!logfc %in% colnames(deg)) {
    cli::cli_abort("Log fold change column {.val {logfc}} not found in deg")
  }
  if (minGSSize < 1 || maxGSSize < minGSSize) {
    cli::cli_abort("Invalid gene set size parameters")
  }

  rlang::check_installed("clusterProfiler")

  # Setup output path
  save_results <- !is.null(path)
  abspath <- NULL
  file_store <- NULL
  if (save_results) {
    file_store <- path
    if (!dir.exists(file_store)) dir.create(file_store, recursive = TRUE)
    abspath <- file.path(normalizePath(file_store, winslash = "/", mustWork = FALSE), "")
  }

  # Safely rename columns
  if (gene_symbol != "symbol" && "symbol" %in% colnames(deg)) {
    colnames(deg)[colnames(deg) == "symbol"] <- "symbol_subs"
  }
  colnames(deg)[colnames(deg) == gene_symbol] <- "symbol"
  colnames(deg)[colnames(deg) == logfc] <- "logfc"

  # Map gene symbols to Entrez IDs
  cli::cli_alert_info("Mapping gene symbols to Entrez IDs for {.val {org}}")
  database <- switch(org,
    hsa = "org.Hs.eg.db",
    mus = "org.Mm.eg.db"
  )
  rlang::check_installed(database)

  entrizid <- clusterProfiler::bitr(deg$symbol,
    fromType = "SYMBOL",
    toType = c("GENENAME", "ENTREZID"),
    OrgDb = database
  )

  # Merge and prepare genelist
  deg <- merge(deg, entrizid, by.x = "symbol", by.y = "SYMBOL", all.x = TRUE, all.y = FALSE)
  gene_id_logfc <- deg %>%
    dplyr::select("ENTREZID", "logfc") %>%
    dplyr::distinct(ENTREZID, .keep_all = TRUE) %>%
    dplyr::filter(!is.na(ENTREZID), !is.na(logfc)) %>%
    dplyr::arrange(dplyr::desc(logfc))

  genelist <- gene_id_logfc$logfc
  names(genelist) <- gene_id_logfc$ENTREZID
  genelist <- genelist[order(genelist, decreasing = TRUE)]

  # Get gene sets
  term2genes <- .get_gene_sets(genesets, org, category, subcategory, project)

  # Run GSEA
  cli::cli_alert_info("Running GSEA analysis...")
  hall_gsea <- clusterProfiler::GSEA(
    genelist,
    exponent = 1,
    eps = 0,
    minGSSize = minGSSize,
    maxGSSize = maxGSSize,
    pvalueCutoff = 0.05,
    pAdjustMethod = "BH",
    TERM2GENE = term2genes,
    TERM2NAME = NA,
    verbose = verbose,
    seed = seed,
    by = "fgsea"
  )

  # Convert to readable format
  rlang::check_installed("clusterProfiler")
  hall_gsea <- clusterProfiler::setReadable(hall_gsea, OrgDb = database, keyType = "ENTREZID")

  # Save results
  if (save_results) {
    csv_file <- paste0(abspath, "1-", category, "_GSEA_significant_results.csv")
    utils::write.csv(as.data.frame(hall_gsea), file = csv_file, row.names = FALSE)
    cli::cli_alert_success("GSEA results written to: {.path {csv_file}}")
  }

  # Generate plots if results exist
  hall_gsea_result <- .process_gsea_results(
    hall_gsea, category, show_gsea, show_path_n, plot_single_sig,
    palette_gsea, cols_gsea, palette_bar, cols_bar, show_plot,
    print_bar, save_results, file_store, fig.type
  )

  if (save_results) {
    save(hall_gsea_result, file = paste0(abspath, "0-", category, "_combined_GSEA_result.RData"))
  }

  hall_gsea_result
}

#' Get gene sets for GSEA
#' @keywords internal
#' @noRd
.get_gene_sets <- function(genesets, org, category, subcategory, project) {
  if (is.null(genesets)) {
    rlang::check_installed("msigdbr")
    species <- switch(org,
      hsa = "Homo sapiens",
      mus = "Mus musculus"
    )

    m_df <- msigdbr::msigdbr(species = species)
    a <- m_df %>%
      dplyr::distinct(.data$gs_cat, .data$gs_subcat) %>%
      dplyr::arrange(.data$gs_cat, .data$gs_subcat)
    cli::cli_alert_info("Available MSigDB categories:")
    message(paste(capture.output(as.data.frame(a)), collapse = "\n"))

    category <- category %||% "H"
    term2genes <- msigdbr::msigdbr(species = species, category = category, subcategory = subcategory)
    term2genes %>%
      dplyr::select("gs_name", "entrez_gene") %>%
      as.data.frame()
  } else {
    # Process custom gene sets
    tmpbase <- tempfile()
    term2genes <- output_sig(genesets, file.name = tmpbase)
    file.remove(paste0(tmpbase, ".csv"))
    rlang::check_installed("reshape2")
    term2genes <- reshape2::melt(as.matrix(term2genes))
    term2genes <- term2genes[, -1]
    colnames(term2genes) <- c("gs_name", "symbol")

    database <- switch(org,
      hsa = "org.Hs.eg.db",
      mus = "org.Mm.eg.db"
    )
    rlang::check_installed(database)

    entrizid <- clusterProfiler::bitr(term2genes$symbol,
      fromType = "SYMBOL",
      toType = c("GENENAME", "ENTREZID"),
      OrgDb = database
    )

    term2genes <- merge(term2genes, entrizid, by.x = "symbol", by.y = "SYMBOL", all.x = TRUE, all.y = FALSE)
    term2genes <- term2genes[, c("gs_name", "ENTREZID")]
    colnames(term2genes) <- c("gs_name", "entrez_gene")
    term2genes
  }
}

#' Process GSEA results and generate plots
#' @keywords internal
#' @noRd
.process_gsea_results <- function(hall_gsea, category, show_gsea, show_path_n,
                                  plot_single_sig, palette_gsea, cols_gsea,
                                  palette_bar, cols_bar, show_plot, print_bar,
                                  save_results, file_store, fig.type) {
  if (nrow(hall_gsea) == 0) {
    cli::cli_alert_warning("No terms enriched under pvalue cutoff = 0.05")
    return(hall_gsea)
  }

  # Select top pathways
  n_paths <- min(show_gsea, nrow(hall_gsea))
  paths <- rownames(hall_gsea@result)[1:n_paths]
  cli::cli_alert_info("Most significant gene sets: {paste(paths, collapse = ', ')}")

  # Get colors
  gseacol <- cols_gsea %||% get_cols(palette = palette_gsea, show_col = FALSE)

  # Generate top GSEA plot
  rlang::check_installed("enrichplot")
  GSEAPLOT <- enrichplot::gseaplot2(
    x = hall_gsea,
    geneSetID = paths,
    subplots = c(1, 2),
    color = gseacol[seq_along(paths)],
    pvalue_table = TRUE
  )

  if (save_results) {
    ggplot2::ggsave(
      filename = paste0("2-", category, "_Top_", show_gsea, "_GSEA_plot.", fig.type),
      plot = GSEAPLOT, path = file_store,
      width = 11, height = 7, dpi = 300
    )
  }
  if (show_plot) print(GSEAPLOT)

  # Generate individual plots
  if (plot_single_sig) {
    paths_all <- rownames(hall_gsea@result)
    paths2 <- paths_all[1:min(show_path_n, length(paths_all))]
    paths2 <- paths2[!is.na(paths2)]

    for (i in seq_along(paths2)) {
      single_path <- paths2[i]
      cli::cli_alert_info("Processing: {.val {single_path}}")

      gseaplot <- enrichplot::gseaplot2(
        x = hall_gsea,
        geneSetID = single_path,
        subplots = c(1, 2),
        color = gseacol[i],
        pvalue_table = TRUE
      )

      if (show_plot) print(gseaplot)
      if (save_results) {
        ggplot2::ggsave(
          filename = paste0(i + 4, "-GSEA_plot-", single_path, ".", fig.type),
          plot = gseaplot, path = file_store,
          width = 11, height = 7.5, dpi = 300
        )
      }
    }
  }

  # Split into up and down
  down_gogo <- hall_gsea[hall_gsea$pvalue < 0.05 & hall_gsea$enrichmentScore < 0, ]
  if (nrow(down_gogo) > 0) down_gogo$group <- -1
  down_gogo <- down_gogo[1:min(show_path_n, nrow(down_gogo)), ]
  down_gogo <- down_gogo[!is.na(down_gogo$p.adjust), ]

  up_gogo <- hall_gsea[hall_gsea$pvalue < 0.05 & hall_gsea$enrichmentScore > 0, ]
  if (nrow(up_gogo) > 0) up_gogo$group <- 1
  up_gogo <- up_gogo[1:min(show_path_n, nrow(up_gogo)), ]
  up_gogo <- up_gogo[!is.na(up_gogo$p.adjust), ]

  # Generate bar plot
  if (print_bar) {
    gsea_bar <- enrichment_barplot(
      up_terms = up_gogo,
      down_terms = down_gogo,
      palette = palette_bar,
      cols = cols_bar,
      title = "GSEA-Enrichment",
      width_wrap = 30
    )

    n_bar <- nrow(up_gogo) + nrow(down_gogo)
    height_bar <- 0.5 * n_bar + 3

    if (save_results) {
      ggplot2::ggsave(
        filename = paste0("3-", category, "_GSEA_barplot.", fig.type),
        plot = gsea_bar, path = file_store,
        width = 6, height = height_bar
      )
    }
    if (show_plot) print(gsea_bar)
  }

  list(
    up = hall_gsea[hall_gsea$pvalue < 0.05 & hall_gsea$enrichmentScore > 0, ],
    down = hall_gsea[hall_gsea$pvalue < 0.05 & hall_gsea$enrichmentScore < 0, ],
    all = tibble::as_tibble(hall_gsea),
    plot_top = GSEAPLOT
  )
}

#' MCP-counter Cell Population Abundance Estimation
#'
#' @description
#' Estimates the abundance of different immune and stromal cell populations
#' using the MCP-counter method. Works with various gene identifiers including
#' Affymetrix probesets, HUGO gene symbols, Entrez IDs, and Ensembl IDs.
#'
#' @param expression Matrix or data.frame with features in rows and samples
#'   in columns.
#' @param featuresType Type of identifiers for expression features.
#'   Options: "affy133P2_probesets", "HUGO_symbols", "ENTREZ_ID", "ENSEMBL_ID".
#'   Default is "affy133P2_probesets".
#' @param probesets Probesets data table. Default loads from GitHub.
#' @param genes Genes data table. Default loads from GitHub.
#'
#' @return Matrix with cell populations in rows and samples in columns.
#'
#' @author Etienne Becht
#' @export
#'
#' @examples
#' expr <- matrix(runif(1000), nrow = 100, ncol = 10)
#' rownames(expr) <- paste0("Gene", 1:100)
#' estimates <- MCPcounter.estimate(expr, featuresType = "HUGO_symbols")
#'
MCPcounter.estimate <- function(
  expression,
  featuresType = c(
    "affy133P2_probesets", "HUGO_symbols", "ENTREZ_ID",
    "ENSEMBL_ID"
  ),
  probesets = read.table(
    system.file("extdata/probesets.txt", package = "IOBR"),
    sep = "\t", stringsAsFactors = FALSE, colClasses = "character"
  ),
  genes = read.table(
    system.file("extdata/genes.txt", package = "IOBR"),
    sep = "\t", stringsAsFactors = FALSE, header = TRUE,
    colClasses = "character", check.names = FALSE
  )
) {

  if (is.null(expression)) return(NULL)

  featuresType <- rlang::arg_match(featuresType)

  # Get features based on type
  result <- switch(featuresType,
    affy133P2_probesets = .get_probeset_features(expression, probesets),
    HUGO_symbols = .get_hugo_features(expression, genes),
    ENTREZ_ID = .get_entrez_features(expression, genes),
    ENSEMBL_ID = .get_ensembl_features(expression, genes)
  )

  features <- result$features
  missing.populations <- result$missing

  if (length(missing.populations) > 0) {
    cli::cli_warn("No markers for population(s): {.val {missing.populations}}")
  }

  t(.appendSignatures(expression, features))
}

#' @keywords internal
.get_probeset_features <- function(expression, probesets) {
  markers.names <- unique(probesets[, 2])
  features <- split(probesets[, 1], probesets[, 2])
  features <- lapply(features, intersect, x = rownames(expression))
  features <- features[vapply(features, length, integer(1)) > 0]
  missing.populations <- setdiff(markers.names, names(features))
  features <- features[intersect(markers.names, names(features))]

  list(features = features, missing = missing.populations)
}

#' @keywords internal
.get_hugo_features <- function(expression, genes) {
  markersG <- genes
  features <- subset(markersG, get("HUGO symbols") %in% rownames(expression))
  markers.names <- unique(features[, "Cell population"])
  features <- split(features[, "HUGO symbols"], features[, "Cell population"])
  missing.populations <- setdiff(markers.names, names(features))
  features <- features[intersect(markers.names, names(features))]

  list(features = features, missing = missing.populations)
}

#' @keywords internal
.get_entrez_features <- function(expression, genes) {
  markersG <- genes
  features <- subset(markersG, ENTREZID %in% rownames(expression))
  markers.names <- unique(features[, "Cell population"])
  features <- split(features[, "ENTREZID"], features[, "Cell population"])
  missing.populations <- setdiff(markers.names, names(features))
  features <- features[intersect(markers.names, names(features))]

  list(features = features, missing = missing.populations)
}

#' @keywords internal
.get_ensembl_features <- function(expression, genes) {
  markersG <- genes
  features <- subset(markersG, get("ENSEMBL ID") %in% rownames(expression))
  markers.names <- unique(features[, "Cell population"])
  features <- split(features[, "ENSEMBL ID"], features[, "Cell population"])
  missing.populations <- setdiff(markers.names, names(features))
  features <- features[intersect(markers.names, names(features))]

  list(features = features, missing = missing.populations)
}

#' Append Signatures to Expression Matrix
#'
#' @description
#' Calculates mean expression for each marker feature set.
#'
#' @param xp Expression matrix with features in rows and samples in columns.
#' @param markers List of marker gene vectors for each cell population.
#'
#' @return Matrix with summarized expression values.
#'
#' @keywords internal
.appendSignatures <- function(xp, markers) {
  res <- as.data.frame(do.call(
    cbind,
    lapply(markers, function(x) {
      common_genes <- intersect(row.names(xp), x)
      if (length(common_genes) == 0) {
        return(rep(NA, ncol(xp)))
      }
      apply(xp[common_genes, , drop = FALSE], 2, mean, na.rm = TRUE)
    })
  ))
  res
}

#' Test for Cell Population Infiltration
#'
#' @description
#' Returns p-values for the null hypothesis that samples are not infiltrated
#' by the corresponding cell population.
#'
#' @param MCPcounterMatrix Matrix, usually output from MCPcounter.estimate.
#' @param platform Expression platform: "133P2", "133A", or "HG1".
#'   Default is "133P2".
#'
#' @return Matrix with samples in rows and cell populations in columns.
#'   Elements are p-values.
#'
#' @author Etienne Becht
#' @export
#'
#' @examples
#' # This function requires null_models data which is loaded internally
#' # Create example data
#' scores <- matrix(runif(30), nrow = 3, ncol = 10)
#' rownames(scores) <- c("T cells", "B cells", "NK cells")
#' pvals <- test_for_infiltration(scores, platform = "133P2")
test_for_infiltration <- function(MCPcounterMatrix,
                                  platform = c("133P2", "133A", "HG1")) {

  if (is.null(MCPcounterMatrix)) return(NULL)

  platform <- rlang::arg_match(platform)

  # Load null_models from internal data
  null_models <- load_data("null_models")
  if (is.null(null_models)) return(NULL)

  MCPcounterMatrix <- t(MCPcounterMatrix)
  params <- null_models[grep(platform, colnames(null_models))]
  rownames(params) <- null_models[, "Cell.population"]
  colnames(params) <- sub(platform, "", colnames(params), fixed = TRUE)

  res <- vapply(colnames(MCPcounterMatrix), function(x) {
    stats::pnorm(
      MCPcounterMatrix[, x],
      mean = params[x, "mu."],
      sd = params[x, "sigma."],
      lower.tail = FALSE
    )
  }, numeric(nrow(MCPcounterMatrix)))

  rownames(res) <- rownames(MCPcounterMatrix)
  res
}

#' Generate Reference Matrix from Seurat Object
#'
#' @description
#' Generates reference gene expression data from a Seurat object by
#' identifying marker genes for each cell type and aggregating expression
#' data.
#'
#' @param sce Seurat object containing single-cell RNA-seq data.
#' @param celltype Character. Cell type column name in metadata.
#'   Default is `NULL` (uses default identity).
#' @param proportion Numeric. Proportion of cells to randomly select for
#'   analysis. Default is `NULL` (use all cells).
#' @param assay_deg Character. Assay for finding markers. Default is `"RNA"`.
#' @param slot_deg Character. Slot for finding markers. Default is `"data"`.
#' @param adjust_assay Logical. Whether to adjust assay for SCT. Default is
#'   `FALSE`.
#' @param assay_out Character. Assay for output. Default is `"RNA"`.
#' @param slot_out Character. Slot for output. Default is `"data"`.
#' @param verbose Logical. Print verbose messages. Default is `FALSE`.
#' @param only.pos Logical. Return only positive markers. Default is `TRUE`.
#' @param n_ref_genes Integer. Number of reference genes per cell type.
#'   Default is 50.
#' @param logfc.threshold Numeric. Log fold change threshold. Default is 0.15.
#' @param test.use Character. Statistical test for marker identification.
#'   Default is `"wilcox"`.
#'
#' @return Matrix containing aggregated expression data for reference genes.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' \dontrun{
#' if (requireNamespace("Seurat", quietly = TRUE)) {
#'   # Requires a Seurat object with sufficient cells and markers
#'   sm <- generateRef_seurat(sce = seurat_obj, celltype = "cell_type", slot_out = "data")
#' }
#' }
generateRef_seurat <- function(sce, celltype = NULL, proportion = NULL,
                               assay_deg = "RNA", slot_deg = "data",
                               adjust_assay = FALSE, assay_out = "RNA",
                               slot_out = "data", verbose = FALSE,
                               only.pos = TRUE, n_ref_genes = 50,
                               logfc.threshold = 0.15, test.use = "wilcox") {
  rlang::check_installed("Seurat")

  # TODO
  #   Warning: No DE genes identified
  # data frame with 0 columns and 0 rows
  # Error in `dplyr::group_by()`:

  if (!inherits(sce, "Seurat")) {
    cli::cli_abort("{.arg sce} must be a Seurat object")
  }

  cli::cli_alert_info("Assay used to find markers: {assay_deg}")

  if (!is.null(assay_deg)) {
    Seurat::DefaultAssay(sce) <- assay_deg
  }

  if (!is.null(celltype)) {
    cli::cli_alert_info("Idents of Seurat object is: {celltype}")
    Seurat::Idents(sce) <- celltype
    print(table(as.character(Seurat::Idents(sce))))
  } else {
    cli::cli_alert_info("Idents of Seurat object is: Default...")
    print(table(as.character(Seurat::Idents(sce))))
  }

  if (tolower(assay_deg) == "sct" && adjust_assay) {
    sce <- Seurat::PrepSCTFindMarkers(sce)
  }

  if (!is.null(proportion)) {
    input <- random_strata_cells(
      input = sce@meta.data,
      group = celltype,
      proportion = proportion
    )
    cell_input <- rownames(input)

    cli::cli_alert_info(
      "Subsetting cells in each cluster randomly: {proportion * 100}%..."
    )
    cli::cli_alert_info("Final training data:")
    print(table(as.factor(input[, celltype])))

    sce <- subset(sce, cells = as.character(cell_input))
  }

  cli::cli_alert_info("Find markers of each celltype...")

  sce.markers <- Seurat::FindAllMarkers(
    object = sce,
    slot = slot_deg,
    assay = assay_deg,
    features = NULL,
    only.pos = only.pos,
    min.pct = 0.25,
    thresh.use = 0.25,
    verbose = verbose,
    logfc.threshold = logfc.threshold,
    test.use = test.use,
    fc.name = "avg_log2FC"
  )

  if (interactive()) print(utils::head(sce.markers))

  refgene <- sce.markers %>%
    dplyr::group_by(.data$cluster) %>%
    dplyr::top_n(n_ref_genes, .data$avg_log2FC)

  if (interactive()) print(refgene)

  cli::cli_alert_info("Aggregating scRNAseq data...")

  if (is.null(slot_out)) slot_out <- "counts"

  cli::cli_alert_info(
    "orig.ident was set as group. User can define through parameter celltype..."
  )

  bulk <- Seurat::AggregateExpression(
    object = sce,
    assays = assay_out,
    slot = slot_out,
    group.by = celltype
  )
  bulk <- bulk[[1]]

  bulk <- bulk[rownames(bulk) %in% refgene$gene, , drop = FALSE]

  bulk
}

#' Differential Expression Analysis
#'
#' @description
#' Performs differential expression analysis on gene expression data using either
#' DESeq2 or limma. Includes pre-processing steps like filtering low count data,
#' and calculates fold changes and adjusted p-values. Optionally generates
#' volcano plots and heatmaps.
#'
#' @param eset A matrix of gene expression data where rows represent genes and
#'   columns represent samples.
#' @param annotation Optional data frame for mapping gene IDs to gene names.
#'   Default is `NULL`.
#' @param pdata A data frame containing sample information and grouping labels.
#' @param group_id Character string specifying the column name in `pdata` containing
#'   grouping labels. Default is `"group"`.
#' @param pdata_id Character string specifying the column name in `pdata` for sample IDs.
#'   Default is `"ID"`.
#' @param array Logical indicating whether to perform quantile normalization.
#'   Default is `FALSE`.
#' @param method Character string specifying the method: `"DESeq2"` or `"limma"`.
#'   Default is `"DESeq2"`.
#' @param contrast Character vector of length 2 specifying contrast groups.
#'   Default is `c("High", "Low")`.
#' @param padj_cutoff Numeric cutoff for adjusted p-values. Default is `0.01`.
#' @param logfc_cutoff Numeric log2 fold change cutoff. Default is `0.5`.
#' @param volcano_plot Logical indicating whether to generate a volcano plot.
#'   Default is `FALSE`.
#' @param col_volcano Integer specifying color index for volcano plot. Default is `1`.
#' @param heatmap Logical indicating whether to generate a heatmap. Default is `TRUE`.
#' @param col_heatmap Integer specifying color index for heatmap. Default is `1`.
#' @param path Character string for output directory. Default is `NULL`.
#' @param parallel Logical indicating whether to run in parallel. Default is `FALSE`.
#' @param id_anno Character string specifying the identifier column in annotation.
#'   Default is `NULL`.
#'
#' @return Data frame containing differentially expressed genes with statistics
#'   including log2 fold changes and adjusted p-values.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_eset <- matrix(abs(rnorm(100 * 20)), 100, 20)
#' rownames(sim_eset) <- paste0("Gene", 1:100)
#' colnames(sim_eset) <- paste0("Sample", 1:20)
#'
#' sim_pdata <- data.frame(
#'   ID = paste0("Sample", 1:20),
#'   group = rep(c("High", "Low"), each = 10)
#' )
#'
#' # Run DEG analysis
#' deg <- iobr_deg(
#'   eset = sim_eset, pdata = sim_pdata,
#'   group_id = "group", pdata_id = "ID",
#'   method = "limma", contrast = c("High", "Low"),
#'   heatmap = FALSE
#' )
#' if (!is.null(deg)) head(deg)
iobr_deg <- function(eset,
                     annotation = NULL,
                     id_anno = NULL,
                     pdata,
                     group_id = "group",
                     pdata_id = "ID",
                     array = FALSE,
                     method = c("DESeq2", "limma"),
                     contrast = c("High", "Low"),
                     path = NULL,
                     padj_cutoff = 0.01,
                     logfc_cutoff = 0.5,
                     volcano_plot = FALSE,
                     col_volcano = 1,
                     heatmap = TRUE,
                     col_heatmap = 1,
                     parallel = FALSE) {
  if (is.null(eset)) return(NULL)
  method <- rlang::arg_match(method)
  # Input validation
  if (is.null(eset)) {
    cli::cli_abort("{.arg eset} cannot be NULL")
  }
  if (!is.matrix(eset) && !is.data.frame(eset)) {
    cli::cli_abort("{.arg eset} must be a matrix or data frame")
  }
  if (nrow(eset) == 0 || ncol(eset) == 0) {
    cli::cli_abort("{.arg eset} is empty")
  }
  if (is.null(pdata)) {
    cli::cli_abort("{.arg pdata} cannot be NULL")
  }
  pdata <- as.data.frame(pdata)
  if (!pdata_id %in% colnames(pdata)) {
    cli::cli_abort("Column {.val {pdata_id}} not found in pdata")
  }
  if (!group_id %in% colnames(pdata)) {
    cli::cli_abort("Column {.val {group_id}} not found in pdata")
  }
  if (length(contrast) != 2) {
    cli::cli_abort("{.arg contrast} must be a character vector of length 2")
  }

  # Setup output path
  abspath <- NULL
  if (!is.null(path)) {
    folder_info <- creat_folder(path)
    abspath <- folder_info$abspath
  }

  # Safely rename columns
  if (pdata_id != "ID") {
    colnames(pdata)[colnames(pdata) == pdata_id] <- "ID"
  }
  if (group_id != "deg_group") {
    colnames(pdata)[colnames(pdata) == group_id] <- "deg_group"
  }

  cli::cli_alert_info("Matching grouping information and expression matrix")

  if (group_id == "group3") {
    pdata <- pdata[pdata$deg_group != "Middle", ]
  }

  pdata <- pdata[!is.na(pdata$deg_group) & pdata$deg_group != "NA", ]
  pdata <- pdata[pdata$ID %in% colnames(eset), ]
  eset <- eset[, colnames(eset) %in% pdata$ID]
  pdata <- pdata[match(colnames(eset), pdata$ID), ]

  if (array) {
    rlang::check_installed("preprocessCore")
    eset <- preprocessCore::normalize.quantiles(as.matrix(eset), keep.names = TRUE)
  }

  if (method == "DESeq2") {
    DEG <- .run_deseq2(
      eset, pdata, contrast, annotation, id_anno,
      padj_cutoff, logfc_cutoff, parallel
    )
  } else {
    DEG <- .run_limma(eset, pdata, contrast, padj_cutoff, logfc_cutoff)
  }

  # Save results
  if (!is.null(abspath)) {
    save(DEG, file = paste0(abspath, "1-DEGs.RData"))
    csv_file <- paste0(abspath, "2-DEGs.csv")
    utils::write.csv(DEG, file = csv_file, row.names = FALSE)
    cli::cli_alert_success("DEG results written to: {.path {csv_file}}")
  }

  DEG
}

#' Run DESeq2 analysis
#' @keywords internal
#' @noRd
.run_deseq2 <- function(eset, pdata, contrast, annotation, id_anno,
                        padj_cutoff, logfc_cutoff, parallel) {
  rlang::check_installed("DESeq2")

  cli::cli_alert_info("Using DESeq2 for RNA-seq differential analysis")
  cli::cli_alert_warning("Ensure {.arg eset} is a count expression matrix")

  eset <- round(eset, 0)
  dds <- DESeq2::DESeqDataSetFromMatrix(
    countData = eset,
    colData = pdata,
    design = ~deg_group
  )

  # Filter low count genes
  dds <- dds[rowSums(DESeq2::counts(dds)) > ncol(eset) / 5, ]
  dds <- DESeq2::DESeq(dds, parallel = parallel)

  contrast <- c("deg_group", contrast)
  res_tidy <- DESeq2::results(dds, tidy = TRUE, contrast = contrast)
  res_tidy <- as.data.frame(res_tidy)

  cli::cli_alert_info("Differential analysis summary:")
  cli::cli_alert_info("  Adj.p < 0.001: {sum(res_tidy$padj < 0.001, na.rm = TRUE)}")
  cli::cli_alert_info("  Adj.p < 0.05: {sum(res_tidy$padj < 0.05, na.rm = TRUE)}")
  cli::cli_alert_info("  Adj.p < 0.1: {sum(res_tidy$padj < 0.1, na.rm = TRUE)}")
  cli::cli_alert_info("  Adj.p < 0.25: {sum(res_tidy$padj < 0.25, na.rm = TRUE)}")

  DEG <- res_tidy[order(res_tidy$padj, decreasing = FALSE), ]

  # Add annotation
  if (!is.null(annotation)) {
    colnames(annotation)[colnames(annotation) == id_anno] <- "id"
    DEG <- merge(DEG, annotation, by.x = "row", by.y = "id", all = FALSE)
  } else {
    cli::cli_alert_info("Using built-in anno_grch38 for annotation")
    anno_grch38 <- load_data("anno_grch38")
    DEG <- merge(DEG, anno_grch38, by.x = "row", by.y = "id", all = FALSE)
  }

  DEG <- .classify_degs(DEG, padj_cutoff, logfc_cutoff)

  # Get group IDs
  level1 <- as.character(contrast[2])
  level2 <- as.character(contrast[3])
  aa <- as.character(pdata[pdata$deg_group == level1, "ID"])
  aa <- aa[aa %in% colnames(eset)]
  bb <- as.character(pdata[pdata$deg_group == level2, "ID"])
  bb <- bb[bb %in% colnames(eset)]

  cli::cli_alert_info("Group 1 = {level1}: {length(aa)} samples")
  cli::cli_alert_info("Group 2 = {level2}: {length(bb)} samples")

  if (length(aa) <= 20 && length(bb) <= 20) {
    cli::cli_alert_info("Group 1 samples: {paste(aa, collapse = ', ')}")
    cli::cli_alert_info("Group 2 samples: {paste(bb, collapse = ', ')}")
  }

  # Calculate mean counts
  rlang::check_installed("preprocessCore")
  eset2 <- preprocessCore::normalize.quantiles(as.matrix(eset), keep.names = TRUE)
  eset2 <- tibble::rownames_to_column(as.data.frame(eset2), var = "ID")

  meancounts <- eset2 %>%
    dplyr::mutate(mean_group1 = rowSums(.[, aa, drop = FALSE]) / length(aa)) %>%
    dplyr::mutate(mean_group2 = rowSums(.[, bb, drop = FALSE]) / length(bb)) %>%
    dplyr::select("ID", "mean_group1", "mean_group2")

  DEG <- merge(DEG, meancounts, by.x = "row", by.y = "ID", all = FALSE)
  colnames(DEG)[colnames(DEG) == "mean_group1"] <- level1
  colnames(DEG)[colnames(DEG) == "mean_group2"] <- level2

  DEG <- tibble::as_tibble(DEG)
  DEG[order(DEG$padj, decreasing = FALSE), ]
}

#' Run limma analysis
#' @keywords internal
#' @noRd
.run_limma <- function(eset, pdata, contrast, padj_cutoff, logfc_cutoff) {
  rlang::check_installed("limma")

  cli::cli_alert_info("Using limma for array differential analysis")

  contrast_full <- c("deg_group", contrast)
  keep_groups <- contrast[1:2]
  pdata <- pdata[pdata$deg_group %in% keep_groups, , drop = FALSE]
  pdata <- pdata[!is.na(pdata$deg_group), , drop = FALSE]
  eset <- eset[, colnames(eset) %in% pdata$ID, drop = FALSE]
  pdata <- pdata[match(colnames(eset), pdata$ID), , drop = FALSE]

  if (!all(keep_groups %in% unique(as.character(pdata$deg_group)))) {
    available <- paste(unique(as.character(pdata$deg_group)), collapse = ", ")
    cli::cli_abort("Selected groups not found. Available: {available}")
  }

  pdata$deg_group <- ifelse(pdata$deg_group == contrast[1], "group1", "group2")

  cli::cli_alert_info("Group 1 = {contrast[1]}")
  cli::cli_alert_info("Group 2 = {contrast[2]}")

  design <- stats::model.matrix(~ 0 + factor(pdata$deg_group))
  colnames(design) <- levels(factor(pdata$deg_group))
  rownames(design) <- colnames(eset)

  contrast.matrix <- limma::makeContrasts(group1 - group2, levels = design)
  fit <- limma::lmFit(eset, design)
  fit2 <- limma::contrasts.fit(fit, contrast.matrix)
  fit2 <- limma::eBayes(fit2)
  DEG <- limma::topTable(fit2, coef = 1, n = Inf, adjust.method = "BH", sort.by = "P")

  DEG <- tibble::rownames_to_column(DEG, var = "symbol")
  colnames(DEG)[colnames(DEG) == "logFC"] <- "log2FoldChange"
  colnames(DEG)[colnames(DEG) == "adj.P.Val"] <- "padj"
  colnames(DEG)[colnames(DEG) == "P.Value"] <- "pvalue"

  DEG <- .classify_degs(DEG, padj_cutoff, logfc_cutoff)

  # Get mean counts
  aa <- as.character(pdata[pdata$deg_group == "group1", "ID"])
  aa <- aa[aa %in% colnames(eset)]
  bb <- as.character(pdata[pdata$deg_group == "group2", "ID"])
  bb <- bb[bb %in% colnames(eset)]

  eset2 <- tibble::rownames_to_column(as.data.frame(eset), var = "ID")
  meancounts <- eset2 %>%
    dplyr::mutate(mean_group1 = rowSums(.[, aa, drop = FALSE]) / length(aa)) %>%
    dplyr::mutate(mean_group2 = rowSums(.[, bb, drop = FALSE]) / length(bb)) %>%
    dplyr::select("ID", "mean_group1", "mean_group2")

  DEG <- merge(DEG, meancounts, by.x = "symbol", by.y = "ID", all = FALSE)
  colnames(DEG)[colnames(DEG) == "mean_group1"] <- contrast[1]
  colnames(DEG)[colnames(DEG) == "mean_group2"] <- contrast[2]

  tibble::as_tibble(DEG[order(DEG$padj, decreasing = FALSE), ])
}

#' Classify DEGs
#' @keywords internal
#' @noRd
.classify_degs <- function(DEG, padj_cutoff, logfc_cutoff) {
  DEG$sigORnot <- ifelse(
    DEG$log2FoldChange > logfc_cutoff & DEG$padj < padj_cutoff, "Up_regulated",
    ifelse(DEG$log2FoldChange < -logfc_cutoff & DEG$padj < padj_cutoff, "Down_regulated", "NOT")
  )
  DEG$label <- ifelse(
    abs(DEG$log2FoldChange) > logfc_cutoff & DEG$padj < padj_cutoff, "Both",
    ifelse(DEG$padj < padj_cutoff, "Significant",
      ifelse(abs(DEG$log2FoldChange) >= logfc_cutoff, paste0("log2FC >= ", logfc_cutoff), "NOT")
    )
  )
  DEG
}

#' Use quanTIseq to Deconvolute a Gene Expression Matrix
#'
#' @description
#' Deconvolutes gene expression data to estimate immune cell fractions using
#' the quanTIseq method. Source code from https://github.com/FFinotello/quanTIseq.
#'
#' @references
#' F. Finotello, C. Mayer, C. Plattner, G. Laschober, D. Rieder,
#' H. Hackl, A. Krogsdam, W. Posch, D. Wilflingseder, S. Sopper, M. Jsselsteijn,
#' D. Johnsons, Y. Xu, Y. Wang, M. E. Sanders, M. V. Estrada, P.
#' Ericsson-Gonzalez, J. Balko, N. F. de Miranda, Z. Trajanoski.
#' "quanTIseq: quantifying immune contexture of human tumors".
#' bioRxiv 223180. https://doi.org/10.1101/223180.
#'
#' @param mix.mat Data frame or matrix. Gene expression matrix with gene symbols
#'   on the first column and sample IDs on the first row. Expression data must
#'   be on non-log scale (TPM for RNA-seq or expression values for microarrays).
#' @param arrays Logical. Whether expression data are from microarrays.
#'   Default is FALSE. If TRUE, the rmgenes parameter is set to "none".
#' @param signame Character. Name of the signature matrix. Currently only
#'   "TIL10" is available. Default is "TIL10".
#' @param tumor Logical. Whether expression data are from tumor samples.
#'   If TRUE, signature genes with high expression in tumor samples are removed.
#'   Default is FALSE.
#' @param mRNAscale Logical. Whether cell fractions must be scaled to account
#'   for cell-type-specific mRNA content. Default is TRUE.
#' @param method Character. Deconvolution method: "hampel", "huber", "bisquare"
#'   for robust regression, or "lsei" for constrained least squares.
#'   Default is "lsei".
#' @param rmgenes Character. Genes to remove: "unassigned" (default), "default",
#'   "none", or "path".
#'
#' @return Data frame with cell fractions for each sample.
#'
#' @export
#' @author Finotello F, et al. (adapted for IOBR)
#'
#' @examples
#' \dontrun{
#' quantiseq_data <- load_data("quantiseq_data")
#' if (!is.null(quantiseq_data)) {
#'   common_genes <- rownames(quantiseq_data$TIL10_signature)
#'   tpm_matrix <- as.data.frame(matrix(
#'     abs(rnorm(length(common_genes) * 2, mean = 5, sd = 2)),
#'     nrow = length(common_genes), ncol = 2
#'   ))
#'   rownames(tpm_matrix) <- common_genes
#'   colnames(tpm_matrix) <- paste0("Sample", 1:2)
#'   results <- deconvolute_quantiseq.default(mix.mat = tpm_matrix)
#'   if (!is.null(results)) head(results)
#' }
#' }
deconvolute_quantiseq.default <- function(mix.mat,
                                          arrays = FALSE,
                                          signame = "TIL10",
                                          tumor = FALSE,
                                          mRNAscale = TRUE,
                                          method = c("lsei", "hampel", "huber", "bisquare"),
                                          rmgenes = "unassigned") {

  if (is.null(mix.mat)) return(NULL)

  method <- rlang::arg_match(method)

  if (nrow(mix.mat) == 0) {
    cli::cli_abort("{.arg mix.mat} cannot be empty.")
  }

  cli::cli_alert_info("Running quanTIseq deconvolution module")

  if (rmgenes == "unassigned" && arrays) {
    rmgenes <- "none"
  } else if (rmgenes == "unassigned" && !arrays) {
    rmgenes <- "default"
  }

  listsig <- c("TIL10")

  quantiseq_ref <- load_data("quantiseq_data")
  if (is.null(quantiseq_ref)) return(NULL)

  if (signame %in% listsig) {
    sig.mat <- quantiseq_ref$TIL10_signature
    mRNA <- quantiseq_ref$TIL10_mRNA_scaling
    lrmgenes <- quantiseq_ref$TIL10_rmgenes
  } else {
    sig.mat.file <- paste0(signame, "_signature.txt")
    sig.mat <- read.table(sig.mat.file, header = TRUE, sep = "\t", row.names = 1)

    mRNA.file <- paste0(signame, "_mRNA_scaling.txt")
    mRNA <- read.table(mRNA.file, sep = "\t", header = FALSE, stringsAsFactors = FALSE)
  }

  if (!is.numeric(mix.mat[[1, 1]])) {
    cli::cli_abort(c(
      "Wrong input format for the mixture matrix!",
      "i" = "Please follow the instructions in the documentation."
    ))
  }

  if (mRNAscale) {
    colnames(mRNA) <- c("celltype", "scaling")
    mRNA <- as.vector(as.matrix(mRNA$scaling[match(colnames(sig.mat), mRNA$celltype)]))
  } else {
    mRNA <- rep(1, ncol(sig.mat))
  }

  cli::cli_alert_info("Gene expression normalization and re-annotation (arrays: {arrays})")
  mix.mat <- fixMixture(mix.mat, arrays = arrays)

  if (rmgenes != "none") {
    if (signame %in% listsig) {
      n1 <- nrow(sig.mat)
      sig.mat <- sig.mat[!rownames(sig.mat) %in% lrmgenes, , drop = FALSE]
      n2 <- nrow(sig.mat)
      cli::cli_alert_info("Removing {n1 - n2} noisy genes")
    }
  }

  if (tumor) {
    if (signame %in% listsig) {
      abgenes <- quantiseq_ref$TIL10_TCGA_aberrant_immune_genes
      n1 <- nrow(sig.mat)
      sig.mat <- sig.mat[!rownames(sig.mat) %in% abgenes, , drop = FALSE]
      n2 <- nrow(sig.mat)
      cli::cli_alert_info("Removing {n1 - n2} genes with high expression in tumors")
    }
  }

  ns <- nrow(sig.mat)
  us <- length(intersect(rownames(sig.mat), rownames(mix.mat)))
  perc <- round(us * 100 / ns, digits = 2)
  cli::cli_alert_info("Signature genes found in data set: {us}/{ns} ({perc}%)")

  cli::cli_alert_info("Mixture deconvolution (method: {method})")
  results1 <- quanTIseq(sig.mat, mix.mat, scaling = mRNA, method = method)

  if ("Tregs" %in% colnames(sig.mat) && "T.cells.CD4" %in% colnames(sig.mat) &&
    method == "lsei") {
    minTregs <- 0.02
    i <- which(colnames(sig.mat) == "T.cells.CD4")
    results2 <- quanTIseq(sig.mat[, -i], mix.mat, scaling = mRNA[-i], method = method)

    ind <- which(results1[, "Tregs"] < minTregs)
    if (length(ind) > 0) {
      results1[ind, "Tregs"] <- (results2[ind, "Tregs"] + results1[ind, "Tregs"]) / 2
      results1[ind, "T.cells.CD4"] <- pmax(
        0, results1[ind, "T.cells.CD4"] - (results2[ind, "Tregs"] + results1[ind, "Tregs"]) / 2
      )
    }
  }

  results <- results1
  results <- results / apply(results, 1, sum)

  cli::cli_alert_success("Deconvolution successful!")

  results <- data.frame(results)
  results <- cbind(rownames(results), results)
  colnames(results)[1] <- "Sample"

  results
}

#' Source code for the TIMER deconvolution method.
#'
#' This code is adapted from https://github.com/hanfeisun/TIMER, which
#' again is an adapted version of the original TIMER source code
#' from http://cistrome.org/TIMER/download.html.
#'
#' The method is described in Li et al. Genome Biology 2016;17(1):174., PMID
#' 27549193.


#' Display Timer Information Messages
#'
#' @description
#' Formats and displays informational messages for timing or logging purposes.
#' Useful for tracking progress or stages of execution within scripts.
#'
#' @param string Character. Message to be displayed.
#'
#' @return None; used for its side effect of printing a message.
#'
#' @export
#' @author Bo Li
#'
#' @examples
#' timer_info("Data processing started.")
timer_info <- function(string) {
  if (is.null(string)) return(NULL)
  cli::cli_alert_info(string)
}


#' TIMER Available Cancer Types
#'
#' Character vector of cancer types supported by TIMER deconvolution.
#' TIMER signatures are cancer-specific.
#'
#' @return Character vector of available cancer type abbreviations.
#'
#' @export
#'
#' @examples
#' # List all available cancer types for TIMER
#' timer_available_cancers
#'
#' # Check if a cancer type is supported
#' "brca" %in% timer_available_cancers
timer_available_cancers <- c(
  "kich", "blca", "brca", "cesc", "gbm", "hnsc", "kirp", "lgg",
  "lihc", "luad", "lusc", "prad", "sarc", "pcpg", "paad", "tgct",
  "ucec", "ov", "skcm", "dlbc", "kirc", "acc", "meso", "thca",
  "uvm", "ucs", "thym", "esca", "stad", "read", "coad", "chol"
)


#' Remove Batch Effect of Expression Set
#'
#' @description
#' Removes batch effects between two gene expression datasets, typically
#' representing different sample types such as cancer cells and immune cells.
#' Uses ComBat from the sva package for batch correction.
#'
#' @param cancer.exp Matrix or data frame. Cancer cell expression data with
#'   genes as rows and samples as columns.
#' @param immune.exp Matrix or data frame. Immune cell expression data with
#'   genes as rows and samples as columns.
#' @param immune.cellType Vector. Cell type for each column in `immune.exp`.
#'
#' @return A list containing:
#' \describe{
#'   \item{1}{Batch effect corrected cancer expression data}
#'   \item{2}{Batch effect corrected immune expression data}
#'   \item{3}{Aggregated immune expression data (median per cell type)}
#' }
#'
#' @export
#' @author Bo Li
#'
#' @examples
#' set.seed(123)
#' gene_names <- paste0("Gene", 1:100)
#' sample_names_cancer <- paste0("CancerSample", 1:10)
#' cancer.exp <- matrix(runif(1000, 1, 1000),
#'   nrow = 100, ncol = 10,
#'   dimnames = list(gene_names, sample_names_cancer)
#' )
#'
#' sample_names_immune <- paste0("ImmuneSample", 1:5)
#' immune.exp <- matrix(runif(500, 1, 1000),
#'   nrow = 100, ncol = 5,
#'   dimnames = list(gene_names, sample_names_immune)
#' )
#'
#' immune.cellType <- c("T-cell", "B-cell", "T-cell", "NK-cell", "B-cell")
#' names(immune.cellType) <- sample_names_immune
#'
#' result <- RemoveBatchEffect(cancer.exp, immune.exp, immune.cellType)
#' if (!is.null(result)) str(result)
RemoveBatchEffect <- function(cancer.exp, immune.exp, immune.cellType) {
  if (is.null(cancer.exp)) return(NULL)
  rlang::check_installed("sva")

  tmp.dd <- as.matrix(cancer.exp)
  tmp.ss <- intersect(rownames(tmp.dd), rownames(immune.exp))

  N1 <- ncol(tmp.dd)
  tmp.dd <- cbind(tmp.dd[tmp.ss, ], immune.exp[tmp.ss, ])
  tmp.dd <- as.matrix(tmp.dd)
  mode(tmp.dd) <- "numeric"

  N2 <- ncol(immune.exp)
  tmp.batch <- c(rep(1, N1), rep(2, N2))

  rlang::check_installed(c("sva", "BiocParallel"))
  suppressMessages(
    tmp.dd0 <- sva::ComBat(tmp.dd, tmp.batch, c(),
      BPPARAM = BiocParallel::bpparam("SerialParam")
    )
  )

  dd.br <- tmp.dd0[, 1:N1]
  immune.exp.br <- tmp.dd0[, (N1 + 1):(N1 + N2)]

  tmp0 <- NULL
  for (kk in unique(names(immune.cellType))) {
    tmp.vv <- which(names(immune.cellType) == kk)

    if (length(tmp.vv) == 1) {
      median_expression <- apply(immune.exp.br[, tmp.vv, drop = FALSE], 1, median, na.rm = TRUE)
    } else {
      median_expression <- apply(immune.exp.br[, tmp.vv], 1, median, na.rm = TRUE)
    }

    if (is.null(tmp0)) {
      tmp0 <- median_expression
    } else {
      tmp0 <- cbind(tmp0, median_expression)
    }
  }

  immune.exp.agg.br <- tmp0
  colnames(immune.exp.agg.br) <- unique(names(immune.cellType))

  list(as.matrix(dd.br), immune.exp.br, immune.exp.agg.br)
}


#' Process Batch Table and Validate Cancer Types
#'
#' @description
#' Processes input data containing cancer types and validates each category
#' against a predefined list of supported cancer types (`timer_available_cancers`).
#'
#' @param args A list containing input parameters:
#' \describe{
#'   \item{batch}{Character. Path to a CSV file (optional).}
#'   \item{expression}{Character vector of expression identifiers (used if
#'     `batch` is NULL).}
#'   \item{category}{Character vector of cancer types (used if `batch` is NULL).}
#' }
#'
#' @return A character matrix with two columns:
#' \describe{
#'   \item{Column 1}{Expression identifiers}
#'   \item{Column 2}{Cancer categories}
#' }
#'
#' @export
#'
#' @examples
#' args <- list(
#'   expression = c("exp1", "exp2"),
#'   category = c("luad", "brca"),
#'   batch = NULL
#' )
#' result <- check_cancer_types(args)
check_cancer_types <- function(args) {
  if (is.null(args)) return(NULL)
  if (!is.null(args$batch)) {
    timer_info("Enter batch mode")
    cancers <- as.matrix(read.table(args$batch, sep = ",", stringsAsFactors = FALSE))
  } else {
    if (length(args$expression) != length(args$category)) {
      cli::cli_abort("expression and category must have the same length")
    }
    cancers <- cbind(args$expression, args$category)
  }

  invalid_cancers <- setdiff(cancers[, 2], timer_available_cancers)
  if (length(invalid_cancers) > 0) {
    cli::cli_abort("Unknown cancer type(s): {.val {invalid_cancers}}")
  }

  cancers
}


#' Constrained Regression Method (Abbas et al., 2009)
#'
#' @description
#' Implements a constrained regression approach described by Abbas et al. (2009).
#' Estimates proportions of immune cell types within mixed cancer tissue samples
#' based on gene expression data. Iteratively adjusts regression coefficients
#' to ensure non-negative values.
#'
#' @param XX Matrix. Immune expression data with genes as rows and cell types
#'   as columns.
#' @param YY Vector. Cancer expression data with gene expression levels.
#' @param w Vector or NA. Weights for regression. Default is NA (no weights).
#'
#' @return Vector with non-negative coefficients representing proportions of
#'   each cell type.
#'
#' @export
#'
#' @examples
#' XX <- matrix(runif(100), nrow = 10, ncol = 10)
#' colnames(XX) <- paste("CellType", 1:10, sep = "")
#' YY <- runif(10)
#' results <- GetFractions.Abbas(XX, YY)
#' print(results)
GetFractions.Abbas <- function(XX, YY, w = NA) {
  if (is.null(XX)) return(NULL)
  ss.remove <- c()
  ss.names <- colnames(XX)

  while (TRUE) {
    if (length(ss.remove) == 0) {
      tmp.XX <- XX
    } else {
      if (is.null(ncol(tmp.XX))) {
        return(rep(0, ncol(XX)))
      }
      tmp.XX <- tmp.XX[, -ss.remove]
    }

    if (length(ss.remove) > 0) {
      ss.names <- ss.names[-ss.remove]
      if (length(ss.names) == 0) {
        return(rep(0, ncol(XX)))
      }
    }

    if (is.na(w[1])) {
      tmp <- lsfit(tmp.XX, YY, intercept = FALSE)
    } else {
      tmp <- lsfit(tmp.XX, YY, w, intercept = FALSE)
    }

    if (is.null(ncol(tmp.XX))) {
      tmp.beta <- tmp$coefficients[1]
    } else {
      tmp.beta <- tmp$coefficients[seq_len(ncol(tmp.XX))]
    }

    if (min(tmp.beta) > 0) break
    ss.remove <- which.min(tmp.beta)
  }

  tmp.F <- rep(0, ncol(XX))
  names(tmp.F) <- colnames(XX)
  tmp.F[ss.names] <- tmp.beta
  tmp.F
}

#' Convert Rowname To Loci
#'
#' @description
#' Processes a gene expression data matrix by modifying its row names.
#' Extracts the gene identifier from row names formatted as 'GENE|ID',
#' simplifying them to 'GENE'.
#'
#' @param cancerGeneExpression Matrix or data frame. Gene expression data with
#'   row names in the format 'GENE|ID'.
#'
#' @return Matrix with modified gene expression data with updated row names.
#'   Rows without a valid identifier are removed.
#'
#' @export
#'
#' @examples
#' example_data <- matrix(runif(20), ncol = 5)
#' rownames(example_data) <- c("LOC101", "LOC102", "LOC103", "LOC104")
#' processed_data <- ConvertRownameToLoci(example_data)
#' print(processed_data)
ConvertRownameToLoci <- function(cancerGeneExpression) {
  if (is.null(cancerGeneExpression)) return(NULL)
  tmp <- strsplit(rownames(cancerGeneExpression), "\\|")
  tmp <- sapply(tmp, function(x) x[[1]])
  tmp.vv <- which(nchar(tmp) > 1)
  rownames(cancerGeneExpression) <- tmp
  extracted <- as.matrix(cancerGeneExpression[tmp.vv, ])
  colnames(extracted) <- colnames(cancerGeneExpression)
  extracted
}


#' Parse Input Gene Expression Data
#'
#' @description
#' Reads gene expression data from a tab-delimited text file, using the first
#' column as row names. Converts data into a numeric matrix for analysis.
#'
#' @param path Character. Path to a tab-delimited gene expression file.
#'   First column should contain gene identifiers.
#'
#' @return Numeric matrix of gene expression values with genes as rows and
#'   samples as columns.
#'
#' @export
#'
#' @examples
#' tf <- tempfile(fileext = ".tsv")
#' expr <- data.frame(
#'   gene = c("GeneA", "GeneB", "GeneC"),
#'   Sample1 = c(10, 20, 30),
#'   Sample2 = c(15, 25, 35)
#' )
#' write.table(expr, tf, sep = "\t", row.names = FALSE, quote = FALSE)
#' gene_expression_data <- ParseInputExpression(tf)
#' print(gene_expression_data)
ParseInputExpression <- function(path) {
  if (is.null(path)) return(NULL)
  ret <- read.delim(path, row.names = 1, check.names = FALSE)
  ret <- as.matrix(ret)
  mode(ret) <- "numeric"
  ret
}


#' Draw QQ Plot Comparing Cancer and Immune Expression
#'
#' @description
#' Creates a quantile-quantile (QQ) plot to compare gene expression
#' distributions between cancer and immune samples. Points along the diagonal
#' indicate similar distributions.
#'
#' @param cancer.exp Vector. Gene expression data for cancer samples.
#' @param immune.exp Vector. Gene expression data for immune samples.
#' @param name Character. Optional subtitle with additional information.
#'
#' @return Generates a QQ plot.
#'
#' @export
#'
#' @examples
#' cancer_exp <- rnorm(100, mean = 5, sd = 1.5)
#' immune_exp <- rnorm(100, mean = 5, sd = 1.5)
#' DrawQQPlot(
#'   cancer.exp = cancer_exp,
#'   immune.exp = immune_exp,
#'   name = "Comparison of Gene Expression"
#' )
DrawQQPlot <- function(cancer.exp, immune.exp, name = "") {
  if (is.null(cancer.exp)) return(NULL)
  qq <- qqplot(cancer.exp, immune.exp,
    xlab = "Tumor Expression", ylab = "Ref Expression",
    main = "Sample-Sample Q-Q plot"
  )
  mtext(name, col = "gray11")

  start <- ceiling(0.4 * length(qq$x))
  end <- floor(0.9 * length(qq$x))
  qq.sub <- list(x = qq$x[start:end], y = qq$y[start:end])
  fit <- lm(y ~ x, data = qq.sub)
  abline(fit, col = "blue")
  invisible(qq)
}

#' Get Outlier Genes
#'
#' @description
#' Identifies outlier genes from multiple cancer datasets. Treats the top 5
#' expressed genes in each sample as outliers and returns unique outlier genes.
#'
#' @param cancers Data frame. One column containing paths to gene expression files.
#'
#' @return Vector of unique gene names identified as outliers.
#'
#' @export
#'
#' @examples
#' tf1 <- tempfile(fileext = ".tsv")
#' tf2 <- tempfile(fileext = ".tsv")
#'
#' expr1 <- data.frame(
#'   gene = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE", "GeneF"),
#'   Sample1 = c(10, 50, 30, 80, 60, 20),
#'   Sample2 = c(15, 40, 25, 90, 55, 10)
#' )
#'
#' expr2 <- data.frame(
#'   gene = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE", "GeneF"),
#'   Sample3 = c(100, 20, 10, 60, 30, 80),
#'   Sample4 = c(95, 25, 15, 70, 35, 85)
#' )
#'
#' write.table(expr1, tf1, sep = "\t", row.names = FALSE, quote = FALSE)
#' write.table(expr2, tf2, sep = "\t", row.names = FALSE, quote = FALSE)
#'
#' cancers <- data.frame(ExpressionFiles = c(tf1, tf2))
#' outlier_genes <- GetOutlierGenes(cancers)
GetOutlierGenes <- function(cancers) {
  if (is.null(cancers)) return(NULL)
  outlier.total <- c()

  for (i in seq_len(nrow(cancers))) {
    cancer.expFile <- as.character(cancers[i, 1])
    cancer.expression <- ParseInputExpression(cancer.expFile)

    for (j in seq_len(ncol(cancer.expression))) {
      outlier <- rownames(cancer.expression)[tail(order(cancer.expression[, j]), 5)]
      outlier.total <- c(outlier.total, outlier)
    }
  }

  unique(outlier.total)
}


#' Deconvolute Tumor Microenvironment Using TIMER
#'
#' @description
#' Performs deconvolution of the tumor microenvironment using the TIMER
#' algorithm. Processes multiple cancer datasets, removes batch effects,
#' and estimates immune cell type abundances.
#'
#' @param args List or environment containing parameters:
#' \describe{
#'   \item{outdir}{Character. Output directory path.}
#'   \item{batch}{Character. File containing paths to expression data and
#'     cancer types.}
#' }
#'
#' @return Matrix of abundance scores for different immune cell types across
#'   multiple cancer samples.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' set.seed(123)
#' immune <- load_data("immuneCuratedData")
#' cancer_genes <- load_data("cancer_type_genes")
#' if (!is.null(immune) && !is.null(cancer_genes)) {
#'   gene_names <- unique(c(head(rownames(immune$genes), 30), cancer_genes[["stad"]]))
#'   sample_names <- paste0("Sample", 1:2)
#'   n_genes <- length(gene_names)
#'   expr <- matrix(runif(n_genes * 2, 1, 100), n_genes, 2,
#'                  dimnames = list(gene_names, sample_names))
#'   tf <- tempfile(fileext = ".tsv")
#'   write.table(as.data.frame(expr), tf, sep = "\t", quote = FALSE)
#'   batch_tf <- tempfile(fileext = ".csv")
#'   write.table(data.frame(path = tf, type = "stad"), batch_tf,
#'               sep = ",", col.names = FALSE, row.names = FALSE, quote = FALSE)
#'   args <- list(outdir = tempdir(), batch = batch_tf)
#'   result <- deconvolute_timer.default(args)
#'   if (!is.null(result)) head(result)
#' }
#' }
deconvolute_timer.default <- function(args) {
  if (is.null(args)) return(NULL)
  cancers <- check_cancer_types(args)

  timer_info("Loading immune gene expression")

  immune <- load_data("immuneCuratedData")
  if (is.null(immune)) return(NULL)
  immune.geneExpression <- immune$genes
  immune.cellTypes <- immune$celltypes

  outlier.genes <- sort(GetOutlierGenes(cancers))
  if (is.null(outlier.genes)) return(NULL)
  cli::cli_alert_info("Outlier genes: {paste(outlier.genes, collapse = ' ')}")

  dir.create(args$outdir, showWarnings = FALSE, recursive = TRUE)
  results_dir <- file.path(args$outdir, "results")
  if (!dir.exists(results_dir)) {
    dir.create(results_dir)
  }

  abundance.score.matrix <- c()
  pdf(file.path(results_dir, "output.pdf"))

  for (i in seq_len(nrow(cancers))) {
    cancer.expFile <- cancers[i, 1]
    cancer.category <- cancers[i, 2]

    cancer.expression <- ParseInputExpression(cancer.expFile)
    index <- !(row.names(cancer.expression) %in% outlier.genes)
    cancer.expression <- cancer.expression[index, , drop = FALSE]
    cancer.colnames <- colnames(cancer.expression)

    timer_info(paste("Removing batch effects for", cancer.category))

    for (j in seq_along(cancer.colnames)) {
      DrawQQPlot(cancer.expression[, j], immune.geneExpression[, 1], name = cancer.colnames[j])
    }

    tmp <- RemoveBatchEffect(cancer.expression, immune.geneExpression, immune.cellTypes)
    if (is.null(tmp)) next
    cancer.expNorm <- tmp[[1]]
    immune.expNormMedian <- tmp[[3]]

    for (j in seq_along(cancer.colnames)) {
      DrawQQPlot(cancer.expNorm[, j], immune.expNormMedian[, 1],
        name = paste("After batch removing and aggregating for", cancer.colnames[j])
      )
    }

    cancer_type_genes_data <- load_data("cancer_type_genes")
    if (is.null(cancer_type_genes_data)) return(NULL)
    gene.selected.marker <- cancer_type_genes_data[[which(names(cancer_type_genes_data) == cancer.category)]]
    gene.selected.marker <- intersect(gene.selected.marker, row.names(cancer.expNorm))

    if (length(gene.selected.marker) < 6) {
      cli::cli_abort(c(
        "Insufficient marker genes for TIMER deconvolution.",
        "i" = "Cancer type '{cancer.category}' requires at least 6 marker genes.",
        "i" = "Found only {length(gene.selected.marker)} marker genes in the input data.",
        "*" = "Use the full expression matrix instead of a subset (e.g., eset[1:500, ]).",
        "*" = "TIMER requires cancer-specific gene markers that may not be in top-expressed genes."
      ))
    }

    XX <- immune.expNormMedian[gene.selected.marker, -4]
    YY <- cancer.expNorm[gene.selected.marker, , drop = FALSE]

    for (j in seq_along(cancer.colnames)) {
      fractions <- GetFractions.Abbas(XX, YY[, j])
      barplot(fractions,
        cex.names = 0.8, names.arg = names(fractions),
        xlab = "cell type", ylab = "abundance",
        main = paste("Abundance estimation for", cancer.colnames[j])
      )
      box()

      abundance.score.matrix <- cbind(abundance.score.matrix, fractions)
      colnames(abundance.score.matrix)[ncol(abundance.score.matrix)] <- cancer.colnames[j]
    }
  }

  dev.off()

  write.table(abundance.score.matrix,
    file.path(results_dir, "score_matrix.txt"),
    sep = "\t", quote = FALSE, row.names = TRUE, col.names = NA
  )

  abundance.score.matrix
}

#' Removing Batch Effect from Expression Sets
#'
#' @description
#' Removes batch effects from expression datasets using sva::ComBat (for
#' microarray/TPM data) or sva::ComBat_seq (for RNA-seq count data). Generates
#' PCA plots to visualize data before and after correction.
#'
#' @param eset1 First expression set (matrix or data frame with genes as rows).
#' @param eset2 Second expression set.
#' @param eset3 Optional third expression set. Use `NULL` if not available.
#' @param id_type Type of gene ID in expression sets (e.g., `"ensembl"`,
#'   `"symbol"`). Required for count data normalization.
#' @param data_type Type of data: `"array"`, `"count"`, or `"tpm"`.
#'   Default is `"array"`.
#' @param cols Color scale for PCA plot. Default is `"normal"`.
#' @param palette Color palette for PCA plot. Default is `"jama"`.
#' @param log2 Whether to perform log2 transformation. Default is `TRUE`.
#'   Ignored for count data.
#' @param check_eset Whether to check expression sets for errors.
#'   Default is `TRUE`.
#' @param adjust_eset Whether to adjust expression sets by removing problematic
#'   features. Default is `TRUE`.
#' @param repel Whether to add repelling labels to PCA plot. Default is `FALSE`.
#' @param path Directory where results should be saved. Default is `NULL`
#'   (display only).
#'
#' @return Expression matrix after batch correction.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @references
#' Zhang Y, et al. ComBat-seq: batch effect adjustment for RNA-seq count data.
#' NAR Genomics and Bioinformatics. 2020;2(3):lqaa078.
#' doi:10.1093/nargab/lqaa078
#'
#' Leek JT, et al. The sva package for removing batch effects and other
#' unwanted variation in high-throughput experiments. Bioinformatics.
#' 2012;28(6):882-883.
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_eset1 <- matrix(rnorm(100 * 5, mean = 10, sd = 2), 100, 5)
#' sim_eset2 <- matrix(rnorm(100 * 5, mean = 12, sd = 2), 100, 5)
#' rownames(sim_eset1) <- rownames(sim_eset2) <- paste0("Gene", 1:100)
#' colnames(sim_eset1) <- paste0("S1_", 1:5)
#' colnames(sim_eset2) <- paste0("S2_", 1:5)
#'
#' # Run batch correction
#' if (requireNamespace("sva", quietly = TRUE) && requireNamespace("BiocParallel", quietly = TRUE)) {
#'   eset_corrected <- remove_batcheffect(sim_eset1, sim_eset2, data_type = "tpm")
#'   if (!is.null(eset_corrected)) head(eset_corrected)
#' }
remove_batcheffect <- function(eset1,
                               eset2,
                               eset3 = NULL,
                               id_type = "ensembl",
                               data_type = c("array", "count", "tpm"),
                               cols = "normal",
                               palette = "jama",
                               log2 = TRUE,
                               check_eset = TRUE,
                               adjust_eset = TRUE,
                               repel = FALSE,
                               path = NULL) {
  if (is.null(eset1) || is.null(eset2)) return(NULL)
  data_type <- rlang::arg_match(data_type)
  # linetype : "batch"

  # Validate data_type
  data_type <- rlang::arg_match(data_type)

  # Create output folder if path provided
  if (!is.null(path)) {
    path <- creat_folder(path)
  }

  # Log2 transformation for non-count data
  if (data_type != "count" && log2) {
    eset1 <- log2eset(eset1)
    eset2 <- log2eset(eset2)
    if (!is.null(eset3)) {
      eset3 <- log2eset(eset3)
    }
  }

  # Check expression sets
  if (check_eset) {
    check_eset(eset1)
    check_eset(eset2)
    if (!is.null(eset3)) check_eset(eset3)
  }

  # Adjust expression sets
  if (adjust_eset) {
    eset1 <- eset1[rownames(eset1) %in%
      feature_manipulation(eset1, is_matrix = TRUE), ]
    eset2 <- eset2[rownames(eset2) %in%
      feature_manipulation(eset2, is_matrix = TRUE), ]
    if (!is.null(eset3)) {
      eset3 <- eset3[rownames(eset3) %in%
        feature_manipulation(eset3, is_matrix = TRUE), ]
    }
  }

  # Find common genes
  if (is.null(eset3)) {
    comgene <- intersect(rownames(eset1), rownames(eset2))
    comgene <- comgene[nzchar(comgene) & !is.na(comgene)]

    cli::cli_alert_info(
      "The two expression matrices share {length(comgene)} features"
    )

    combined.expr <- cbind(
      eset1[comgene, , drop = FALSE],
      eset2[comgene, , drop = FALSE]
    )
    batch <- data.frame(
      ID = colnames(combined.expr),
      batch = rep(c("eset1", "eset2"), times = c(ncol(eset1), ncol(eset2)))
    )
  } else {
    comgene <- intersect(
      intersect(rownames(eset1), rownames(eset2)),
      rownames(eset3)
    )
    comgene <- comgene[nzchar(comgene) & !is.na(comgene)]

    cli::cli_alert_info(
      "The three expression matrices share {length(comgene)} features"
    )

    combined.expr <- cbind(
      eset1[comgene, , drop = FALSE],
      eset2[comgene, , drop = FALSE],
      eset3[comgene, , drop = FALSE]
    )
    batch <- data.frame(
      ID = colnames(combined.expr),
      batch = rep(c("eset1", "eset2", "eset3"),
        times = c(ncol(eset1), ncol(eset2), ncol(eset3))
      )
    )
  }

  # Batch correction
  rlang::check_installed("sva")

  if (data_type != "count") {
    cli::cli_alert_info("Processing method: sva::ComBat")
    modcombat <- stats::model.matrix(~1, data = batch)
    combined.expr.combat <- sva::ComBat(
      dat = as.matrix(combined.expr),
      batch = batch$batch,
      mod = modcombat
    )
    rlang::check_installed("preprocessCore")
    combined.expr.combat <- preprocessCore::normalize.quantiles(
      as.matrix(combined.expr.combat),
      keep.names = TRUE
    )
    prefix <- data_type
  } else {
    cli::cli_alert_info("Processing method: sva::ComBat_seq")
    combined.expr.combat <- sva::ComBat_seq(
      as.matrix(combined.expr),
      batch = batch$batch
    )

    # Convert corrected counts to TPM
    eset2_tpm <- count2tpm(
      countMat = combined.expr.combat,
      idType = id_type,
      source = "local"
    )
    eset2_tpm <- log2eset(eset2_tpm)
    cli::cli_alert_info("Count data processed with ComBat_seq")
    prefix <- "count"
  }

  # Generate PCA plots if FactoMineR is available
  if (requireNamespace("FactoMineR", quietly = TRUE) &&
    requireNamespace("factoextra", quietly = TRUE)) {
    p1 <- iobr_pca(
      data = combined.expr,
      is.matrix = TRUE,
      scale = TRUE,
      is.log = TRUE,
      pdata = batch,
      id_pdata = "ID",
      group = "batch",
      cols = cols,
      palette = palette,
      repel = repel,
      ncp = 3,
      axes = c(1, 2),
      addEllipses = TRUE
    ) + ggplot2::ggtitle(paste("Before correction:", prefix))

    p2 <- iobr_pca(
      data = combined.expr.combat,
      is.matrix = TRUE,
      scale = TRUE,
      is.log = TRUE,
      pdata = batch,
      id_pdata = "ID",
      group = "batch",
      cols = cols,
      palette = palette,
      repel = repel,
      ncp = 3,
      axes = c(1, 2),
      addEllipses = TRUE
    ) + ggplot2::ggtitle(paste("After correction:", prefix))

    if (data_type == "count") {
      p3 <- iobr_pca(
        data = eset2_tpm,
        is.matrix = TRUE,
        scale = TRUE,
        is.log = TRUE,
        pdata = batch,
        id_pdata = "ID",
        group = "batch",
        cols = cols,
        palette = palette,
        repel = repel,
        ncp = 3,
        axes = c(1, 2),
        addEllipses = TRUE
      ) + ggplot2::ggtitle("After correction: count2TPM")

      p <- patchwork::wrap_plots(p1, p2, p3, nrow = 1)
    } else {
      p <- patchwork::wrap_plots(p1, p2, nrow = 1)
    }

    if (interactive()) print(p)

    # Save plot if path provided
    if (!is.null(path)) {
      num_plots <- if (data_type == "count") 3 else 2
      width <- num_plots * 5

      ggplot2::ggsave(
        filename = paste0("0-PCA-of-", num_plots, "-eset.pdf"),
        plot = p,
        width = width,
        height = 5,
        path = path$folder_name
      )
    }
  }

  invisible(combined.expr.combat)
}

#' Check Integrity and Outliers of Expression Set
#'
#' @description
#' Performs quality checks on an expression matrix to identify missing values,
#' infinite values, and features with zero variance. Issues warnings when
#' potential problems are detected that may affect downstream analyses.
#'
#' @param eset Expression matrix or data frame with genes/features in rows and
#'   samples in columns.
#' @param print_result Logical indicating whether to print detailed check
#'   results to the console. Default is `FALSE`.
#' @param estimate_sd Logical indicating whether to check for features with
#'   zero standard deviation. Default is `FALSE`.
#'
#' @return Invisibly returns `NULL`. Function is called for its side effects
#'   (printing messages and issuing warnings).
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' set.seed(123)
#' eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
#' rownames(eset) <- paste0("Gene", 1:100)
#' colnames(eset) <- paste0("Sample", 1:10)
#' check_eset(eset, print_result = TRUE)
check_eset <- function(eset, print_result = FALSE, estimate_sd = FALSE) {
  # Input validation
  if (!is.matrix(eset) && !is.data.frame(eset)) {
    cli::cli_abort(c(
      "Invalid input type for {.arg eset}.",
      "i" = "Expected a matrix or data frame, got {.cls {class(eset)}}."
    ))
  }

  # Check for NA values
  na_count <- sum(is.na(eset))

  if (print_result) {
    cli::cli_alert_info("Checking for NA values: {na_count} found")
  }

  if (na_count > 0) {
    cli::cli_warn(c(
      paste("There are", na_count, "missing values in the matrix."),
      "i" = "This may affect score calculation.",
      "*" = paste(
        "Set adjust_eset = TRUE to handle missing values",
        "automatically."
      )
    ))
  }

  # Check for infinite values (vectorized)
  # Convert to matrix for consistent handling
  eset_mat <- as.matrix(eset)
  eset_range <- range(eset_mat, na.rm = TRUE)
  has_neg_inf <- !is.finite(eset_range[1]) && eset_range[1] < 0
  has_pos_inf <- !is.finite(eset_range[2]) && eset_range[2] > 0

  if (print_result) {
    cli::cli_alert_info(paste(
      "Checking for -Inf values:",
      sum(is.infinite(eset_mat) & eset_mat < 0, na.rm = TRUE),
      "found"
    ))
    cli::cli_alert_info(paste(
      "Checking for +Inf values:",
      sum(is.infinite(eset_mat) & eset_mat > 0, na.rm = TRUE),
      "found"
    ))
  }

  if (has_neg_inf || has_pos_inf) {
    cli::cli_warn(c(
      "Infinite values detected in the matrix.",
      "i" = "This may affect score calculation.",
      "*" = paste(
        "Set adjust_eset = TRUE to handle infinite values",
        "automatically."
      )
    ))
  }

  # Check for zero variance features
  if (estimate_sd) {
    row_sds <- apply(eset_mat, 1, stats::sd, na.rm = TRUE)
    zero_sd_count <- sum(row_sds == 0, na.rm = TRUE)

    if (print_result) {
      cli::cli_alert_info("Features with zero variance: {zero_sd_count}")
    }

    if (zero_sd_count > 0) {
      cli::cli_warn(c(
        "{zero_sd_count} feature{?s} {?has/have} zero variance.",
        "i" = "Zero-variance features may affect score calculation.",
        "*" = "Set {.code adjust_eset = TRUE} to remove them automatically."
      ))
    }
  }

  invisible(NULL)
}

#' GSVA API Version Detection
#'
#' @description
#' Detects whether the installed GSVA package supports the new
#' parameter-based API (gsvaParam/ssgseaParam) or the old direct argument API.
#' This function is used internally to ensure compatibility across different
#' GSVA package versions (1.50.0+ vs older versions).
#'
#' @return A list with two elements:
#'   - `use_new_api`: Logical indicating whether to use the new API (`TRUE`)
#'     or old API (`FALSE`)
#'   - `gsva_version`: Character string of the installed GSVA version,
#'     or `"not installed"` if not available
#'
#' @keywords internal
gsva_use_new_api <- function() {
  if (!requireNamespace("GSVA", quietly = TRUE)) {
    return(list(use_new_api = FALSE, gsva_version = "not installed"))
  }

  gsva_version <- as.character(utils::packageVersion("GSVA"))

  # Check for new API functions (introduced in GSVA 1.50.0)
  use_new_api <- exists("gsvaParam", where = asNamespace("GSVA"), inherits = FALSE) &&
    exists("ssgseaParam", where = asNamespace("GSVA"), inherits = FALSE)

  list(use_new_api = use_new_api, gsva_version = gsva_version)
}

#' Load IOBR Datasets
#'
#' @description
#' Loads internal datasets from the IOBR package. Supports both sysdata (internal)
#' and exported data files included in the package.
#'
#' @param name Character string. Name of the dataset to load. Must be a single value.
#'   Available datasets include:
#'   - Expression data: `"eset_stad"`, `"imvigor210_eset"`, `"melanoma_data"`
#'   - Signatures: `"signature_tme"`, `"signature_metabolism"`, `"signature_collection"`
#'   - Gene sets: `"hallmark"`, `"kegg"`, `"go_bp"`, `"go_cc"`, `"go_mf"`
#'   - Cell markers: `"cellmarkers"`, `"mcp_genes"`
#'   - Phenotype data: `"pdata_stad"`, `"pdata_sig_tme"`, `"pdata_acrg"`
#'   - Reference data: `"xCell.data"`, `"quantiseq_data"`, `"TRef"`, `"BRef"`
#'   - Color palettes: `"palette1"`, `"palette2"`, `"palette3"`, `"palette4"`
#'
#' @returns Dataset object, typically a `list`, `data.frame`, or `matrix`.
#'   The exact type depends on the requested dataset.
#'
#' @export
#'
#' @examples
#' # Load signature collection (stored in sysdata, no download)
#' sig_tme <- load_data("signature_tme")
#'
#' # Load color palette (stored in sysdata, no download)
#' colors <- load_data("palette1")
#'
#' # Error handling with suggestions for similar names
#' try(load_data("sign_tme")) # Will suggest "signature_tme"
#'
#' \dontrun{
#' # Load expression data (triggers download from GitHub)
#' eset <- load_data("eset_stad")
#' }
load_data <- function(name) {
  # Input validation
  if (!is.character(name) || length(name) != 1 || is.na(name) || nchar(name) == 0) {
    stop("'name' must be a single non-empty character string")
  }

  # Define sysdata names (internal package data - minimal essential data)
  # Large datasets moved to GitHub to meet CRAN size requirements
  sysdata_names <- c(
    # Core signatures (small, frequently used) - stored in sysdata.rda
    "signature_tme",

    # MCPCounter (required for deconvolution) - stored in sysdata.rda
    "mcp_genes", "mcp_probesets",

    # Color palettes and configs (small) - stored in sysdata.rda
    "palette1", "palette2", "palette3", "palette4",
    "panel_for_gene", "panel_for_signature",
    "patterns_to_na", "sig_excel"
  )

  # Get available data names from package
  pkg_data <- utils::data(package = "IOBR")$results
  data_names <- if (nrow(pkg_data) > 0) pkg_data[, "Item"] else character(0)

  # Load data based on type
  if (name %in% sysdata_names) {
    # Internal sysdata - use eval for lazy loading
    result <- eval(parse(text = name))
    return(result)
  }

  if (name %in% data_names) {
    # External data file
    data(list = name, package = "IOBR", envir = environment())
    return(get(name, envir = environment()))
  }

  # Check if it's a GitHub-hosted dataset (large datasets moved from sysdata and data/)
  github_datasets <- list_github_datasets()

  if (name %in% github_datasets) {
    # Try to download from GitHub
    return(download_iobr_data(name))
  }

  # Dataset not found - provide helpful error message with suggestions
  available <- sort(c(sysdata_names, data_names, github_datasets))

  # Find similar names for suggestions
  distances <- utils::adist(name, available, ignore.case = TRUE)
  suggestions <- available[which(distances <= 3)]

  error_msg <- c(
    "Dataset {.val {name}} not found in IOBR package.",
    "i" = paste(
      "Available datasets:",
      paste(utils::head(available, 100), collapse = ", "),
      ifelse(length(available) > 100, "...", "")
    )
  )

  if (length(suggestions) > 0) {
    error_msg <- c(
      error_msg,
      "!" = "Did you mean: {.val {suggestions}}?"
    )
  }

  cli::cli_abort(error_msg)
}

#' Principal Component Analysis (PCA) Visualization
#'
#' This function performs Principal Component Analysis (PCA) on gene
#' expression data,
#' reduces dimensionality while preserving variance, and generates a scatter
#' plot visualization.
#'
#' @param data Input data for PCA: matrix or data frame.
#' @param is.matrix Logical indicating if input is a matrix. Default is TRUE.
#' @param scale Logical indicating whether to scale the data. Default is TRUE.
#' @param is.log Logical indicating whether to log-transform the data. Default is FALSE.
#' @param pdata Data frame with sample IDs and grouping information.
#' @param id_pdata Column name in `pdata` for sample IDs. Default is "ID".
#' @param group Column name in `pdata` for grouping variable. Default is NULL.
#' @param cols Color scheme for groups. Default is "normal".
#' @param palette Color palette for groups. Default is "jama".
#' @param repel Logical indicating whether to repel overlapping points. Default is FALSE.
#' @param ncp Number of principal components to retain. Default is 5.
#' @param axes Principal components to plot (e.g., c(1, 2)). Default is c(1, 2).
#' @param addEllipses Logical indicating whether to add concentration ellipses. Default is TRUE.
#' @param geom.ind Type of geometric representation for points. Default is "point".
#'
#' @return A ggplot object of the PCA plot.
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' if (requireNamespace("FactoMineR", quietly = TRUE) &&
#'   requireNamespace("factoextra", quietly = TRUE)) {
#'   set.seed(123)
#'   eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
#'   rownames(eset) <- paste0("Gene", 1:100)
#'   colnames(eset) <- paste0("Sample", 1:10)
#'   pdata <- data.frame(
#'     ID = colnames(eset),
#'     group = rep(c("A", "B"), each = 5)
#'   )
#'   iobr_pca(eset, pdata = pdata, id_pdata = "ID", group = "group", addEllipses = FALSE)
#' }
#'
iobr_pca <- function(data, is.matrix = TRUE, scale = TRUE, is.log = FALSE, pdata, id_pdata = "ID", group = NULL,
                     geom.ind = "point", cols = "normal", palette = "jama", repel = FALSE, ncp = 5, axes = c(1, 2), addEllipses = TRUE) {
  if (is.log) data <- log2eset(data + 1)
  feas <- feature_manipulation(data = data, feature = rownames(data), is_matrix = TRUE)
  data <- data[rownames(data) %in% feas, ]
  #######################################
  if (is.matrix) data <- t(data)
  if (scale) data <- base::scale(data)

  rlang::check_installed("FactoMineR")

  res.pca <- FactoMineR::PCA(data, ncp = ncp, graph = FALSE)

  pdata <- as.data.frame(pdata)
  colnames(pdata)[which(colnames(pdata) == id_pdata)] <- "id"

  pdata <- pdata[pdata$id %in% rownames(data), ]
  pdata <- pdata[match(rownames(data), pdata$id), ]


  message(paste(capture.output(table(pdata[, group])), collapse = "\n"))
  ##########################################

  cols <- get_cols(cols = cols, palette = palette, show_col = FALSE, seed = 123)

  # print(cols)
  # print(pdata[, group])
  cols <- cols[1:length(unique(pdata[[group]]))]

  message(paste0(">>== colors for group: "))
  message(paste0(">>== ", cols))
  #########################################

  rlang::check_installed("factoextra")

  p <- factoextra::fviz_pca_ind(res.pca,
    axes = axes,
    geom.ind = geom.ind, # show points only (nbut not "text")
    col.ind = as.character(pdata[, group]), # color by groups
    palette = cols,
    repel = repel,
    addEllipses = addEllipses, # Concentration ellipses
    legend.title = group
  )
  return(p)
}

#' Batch Calculation of Partial Correlation Coefficients
#'
#' @description
#' Computes partial correlation coefficients between multiple features and a
#' target variable while controlling for an interference (confounding) variable.
#' Adjusts p-values for multiple testing using the Benjamini-Hochberg method.
#'
#' @param input Data frame containing feature variables, target variable, and
#'   interference variable.
#' @param interferenceid Character string specifying the column name of the
#'   interference (confounding) variable to control for.
#' @param target Character string specifying the column name of the target
#'   variable.
#' @param features Character vector specifying the column names of feature
#'   variables to correlate with the target.
#' @param method Character string specifying the correlation method. Options are
#'   `"pearson"`, `"spearman"`, or `"kendall"`. Default is `"pearson"`.
#'
#' @return Tibble containing the following columns for each feature:
#' \describe{
#'   \item{sig_names}{Feature name}
#'   \item{p.value}{Raw p-value}
#'   \item{statistic}{Partial correlation coefficient}
#'   \item{p.adj}{Adjusted p-value (Benjamini-Hochberg method)}
#'   \item{log10pvalue}{Negative log10-transformed p-value}
#'   \item{stars}{Significance stars: **** p.adj<0.0001, *** p.adj<0.001,
#'     ** p.adj<0.01, * p.adj<0.05, + p.adj<0.5}
#' }
#'
#' @author Rongfang Shen
#' @export
#'
#' @examples
#' # Create small example data
#' set.seed(123)
#' test_data <- data.frame(
#'   TumorPurity = runif(100),
#'   TargetVar = rnorm(100),
#'   Signature1 = rnorm(100),
#'   Signature2 = rnorm(100)
#' )
#' # Calculate partial correlations controlling for tumor purity
#' res <- batch_pcc(
#'   input = test_data,
#'   interferenceid = "TumorPurity",
#'   target = "TargetVar",
#'   method = "pearson",
#'   features = c("Signature1", "Signature2")
#' )
#' head(res)
batch_pcc <- function(input,
                      interferenceid,
                      target,
                      features,
                      method = c("pearson", "spearman", "kendall")) {
  if (is.null(input)) return(NULL)
  # Input validation
  if (!is.data.frame(input)) {
    cli::cli_abort("{.arg input} must be a non-null data frame.")
  }
  if (nrow(input) == 0) {
    cli::cli_abort("{.arg input} has no rows.")
  }
  if (!interferenceid %in% colnames(input)) {
    cli::cli_abort(
      "Interference variable {.val {interferenceid}} not found in data."
    )
  }
  if (!target %in% colnames(input)) {
    cli::cli_abort("Target variable {.val {target}} not found in data.")
  }
  if (!is.character(features) || length(features) == 0) {
    cli::cli_abort("{.arg features} must be a non-empty character vector.")
  }

  method <- rlang::arg_match(method)

  # Filter valid features
  features <- setdiff(features, c(interferenceid, target))
  valid_features <- features[features %in% colnames(input)]
  invalid_features <- setdiff(features, colnames(input))

  if (length(invalid_features) > 0) {
    cli::cli_alert_warning(
      "Ignoring {length(invalid_features)} invalid feature{?s}:",
      " {.val {invalid_features}}}"
    )
  }

  if (length(valid_features) == 0) {
    cli::cli_abort("No valid features found in data.")
  }

  cli::cli_alert_info(
    "Computing {method} partial correlation for",
    " {length(valid_features)} feature{?s}"
  )

  # Partial correlation test function
  pcor_test <- function(x, y, z, method) {
    dat <- stats::na.omit(cbind(x, y, z))
    if (nrow(dat) < 4) {
      return(c(estimate = NA_real_, p.value = NA_real_))
    }

    r_mat <- stats::cor(dat, method = method)
    rxy <- r_mat[1, 2]
    rxz <- r_mat[1, 3]
    ryz <- r_mat[2, 3]

    # Partial correlation formula
    rho <- (rxy - rxz * ryz) / sqrt((1 - rxz^2) * (1 - ryz^2))

    n <- nrow(dat)
    if (abs(rho) >= 1) {
      return(c(estimate = rho, p.value = 0))
    }

    tstat <- rho * sqrt((n - 3) / (1 - rho^2))
    pval <- 2 * stats::pt(abs(tstat), df = n - 3, lower.tail = FALSE)

    c(estimate = rho, p.value = pval)
  }

  # Vectorized partial correlation calculation
  results <- vapply(valid_features, function(feat) {
    pcor_test(
      input[[feat]],
      input[[target]],
      input[[interferenceid]],
      method = method
    )
  }, numeric(2))

  # Build results data frame
  cc <- data.frame(
    sig_names = valid_features,
    p.value = results["p.value", ],
    statistic = results["estimate", ],
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  cc$p.adj <- stats::p.adjust(cc$p.value, method = "BH")
  cc$log10pvalue <- -log10(cc$p.value)
  cc$stars <- cut(cc$p.adj,
    breaks = c(-Inf, 0.0001, 0.001, 0.01, 0.05, 0.5, Inf),
    labels = c("****", "***", "**", "*", "+", "")
  )

  cc <- cc[order(cc$p.value), , drop = FALSE]

  cli::cli_alert_success("Partial correlation analysis complete")

  tibble::as_tibble(cc)
}

#' Merge Data Frames with Duplicated Column Names
#'
#' @description
#' Merges two data frames, resolving duplicated column names according to user
#' preference. Allows selection of which data frame's duplicated columns to
#' retain, ensuring data integrity during merging.
#'
#' @param x Data frame. First data frame to merge.
#' @param y Data frame. Second data frame to merge.
#' @param by.x Character. Column name(s) in `x` used for merging.
#' @param by.y Character. Column name(s) in `y` used for merging.
#' @param all.x Logical. Include all rows from `x` in output. Default is `FALSE`.
#' @param all.y Logical. Include all rows from `y` in output. Default is `FALSE`.
#' @param all Logical or `NULL`. If not `NULL`, include all rows from both `x`
#'   and `y`, overriding `all.x` and `all.y`.
#' @param choose Character. Which data frame's duplicated non-joining columns to
#'   retain: `"x"` or `"y"`. Default is `"x"`.
#'
#' @return Data frame resulting from merging `x` and `y` according to specified
#'   parameters.
#'
#' @export
#'
#' @examples
#' df1 <- data.frame(ID = 1:3, Name = c("A", "B", "C"), Value = 1:3)
#' df2 <- data.frame(ID = 1:3, Name = c("X", "Y", "Z"), Score = 4:6)
#'
#' # Merge keeping duplicated columns from x
#' merged_df <- merge_duplicate(df1, df2,
#'   by.x = "ID", by.y = "ID",
#'   all.x = TRUE, choose = "x"
#' )
#' print(merged_df)
#'
#' # Merge keeping duplicated columns from y
#' merged_df2 <- merge_duplicate(df1, df2,
#'   by.x = "ID", by.y = "ID",
#'   all = TRUE, choose = "y"
#' )
merge_duplicate <- function(x, y, by.x, by.y,
                            all.x = FALSE, all.y = FALSE,
                            all = NULL,
                            choose = c("x", "y")) {
  # Input validation
  if (!is.data.frame(x)) {
    cli::cli_abort("{.arg x} must be a data frame.")
  }
  if (!is.data.frame(y)) {
    cli::cli_abort("{.arg y} must be a data frame.")
  }

  choose <- rlang::arg_match(choose)

  if (!by.x %in% colnames(x)) {
    cli::cli_abort("Column {.val {by.x}} not found in {.arg x}.")
  }
  if (!by.y %in% colnames(y)) {
    cli::cli_abort("Column {.val {by.y}} not found in {.arg y}.")
  }

  # Find duplicate column names (excluding merge keys)
  duplicate_names <- intersect(colnames(x), colnames(y))

  if (by.x == by.y) {
    duplicate_names <- setdiff(duplicate_names, by.x)
  } else {
    duplicate_names <- setdiff(duplicate_names, c(by.x, by.y))
  }

  # Remove duplicates from the non-chosen data frame
  if (length(duplicate_names) > 0) {
    cli::cli_alert_info(
      "Removing {length(duplicate_names)} duplicate column{?s} from {.arg {ifelse(choose == 'x', 'y', 'x')}}"
    )

    if (choose == "x") {
      y <- y[, !colnames(y) %in% duplicate_names, drop = FALSE]
    } else {
      x <- x[, !colnames(x) %in% duplicate_names, drop = FALSE]
    }
  }

  # Perform merge
  if (!is.null(all)) {
    res <- merge(x = x, y = y, by.x = by.x, by.y = by.y, all = all)
  } else {
    res <- merge(x = x, y = y, by.x = by.x, by.y = by.y, all.x = all.x, all.y = all.y)
  }

  res
}

#' Map Score to Immunophenoscore
#'
#' @description
#' Maps input score to Immunophenoscore (IPS) on a 0-10 scale. Scores \eqn{\le}0 map to 0,
#' scores \eqn{\ge}3 map to 10, and intermediate scores are linearly scaled.
#'
#' @param x Numeric value representing the aggregate z-score.
#'
#' @return Integer value between 0 and 10 representing the Immunophenoscore.
#'
#' @export
#'
#' @examples
#' ips <- ipsmap(2.5)
#' ips <- ipsmap(-1)
#' ips <- ipsmap(5)
ipsmap <- function(x) {
  if (is.na(x) || x <= 0) {
    0L
  } else if (x >= 3) {
    10L
  } else {
    as.integer(round(x * 10 / 3, digits = 0))
  }
}


#' Map Score to Color
#'
#' @description
#' Maps a numeric input value to a color from a blue-white-red gradient palette.
#' Values are mapped to a 1001-color palette where -3 maps to blue, 0 maps to white,
#' and +3 maps to red.
#'
#' @param x Numeric value to be mapped to a color (typically between -3 and 3).
#' @param my_palette Color palette vector (should have 1001 colors).
#'   Default uses blue-white-red gradient.
#'
#' @return A color from the palette as a hex code.
#'
#' @export
#'
#' @examples
#' my_palette <- grDevices::colorRampPalette(c("blue", "white", "red"))(1001)
#' color <- mapcolors(2, my_palette)
#' color <- mapcolors(-2, my_palette)
mapcolors <- function(x, my_palette = NULL) {
  if (is.null(my_palette)) {
    my_palette <- grDevices::colorRampPalette(c("blue", "white", "red"))(1001)
  }

  za <- if (x >= 3) {
    1000L
  } else if (x <= -3) {
    1L
  } else {
    as.integer(round(166.5 * x + 500.5, digits = 0))
  }

  my_palette[za]
}


#' Map Score to Black and White Color
#'
#' @description
#' Maps a numeric input value to a color from a black-white gradient palette.
#' Values are mapped to a 1001-color palette where -2 maps to black and +2 maps to white.
#'
#' @param x Numeric value to be mapped to a color (typically between -2 and 2).
#' @param my_palette2 Color palette vector (should have 1001 colors).
#'   Default uses black-white gradient.
#'
#' @return A color from the black-white palette as a hex code.
#'
#' @export
#'
#' @examples
#' my_palette2 <- grDevices::colorRampPalette(c("black", "white"))(1001)
#' color <- mapbw(1.5, my_palette2)
#' color <- mapbw(-1, my_palette2)
mapbw <- function(x, my_palette2 = NULL) {
  if (is.null(my_palette2)) {
    my_palette2 <- grDevices::colorRampPalette(c("black", "white"))(1001)
  }

  za2 <- if (x >= 2) {
    1000L
  } else if (x <= -2) {
    1L
  } else {
    as.integer(round(249.75 * x + 500.5, digits = 0))
  }

  my_palette2[za2]
}

#' TME Deconvolution Methods
#'
#' @description
#' A named vector of supported tumor microenvironment (TME) deconvolution
#' methods in the IOBR package.
#'
#' @details
#' The methods currently supported are:
#' \itemize{
#'   \item `mcpcounter`: MCP-counter for immune and stromal cell populations
#'   \item `epic`: EPIC for immune, stromal, and cancer cell fractions
#'   \item `xcell`: xCell for 64 immune and stromal cell types
#'   \item `cibersort`: CIBERSORT for 22 immune cell types
#'   \item `cibersort_abs`: CIBERSORT in absolute mode
#'   \item `ips`: Immunophenoscore calculation
#'   \item `estimate`: ESTIMATE for stromal/immune/estimate scores
#'   \item `svr`: Support Vector Regression (custom reference)
#'   \item `lsei`: Least Squares with Equality/Inequality constraints
#'   \item `timer`: TIMER for cancer-specific immune estimation
#'   \item `quantiseq`: quanTIseq for RNA-seq immune deconvolution
#' }
#'
#' @format A named character vector where names are display names and values
#'   are internal method names.
#'
#' @return Named character vector of available deconvolution methods.
#'
#' @export
#'
#' @examples
#' # List all available TME deconvolution methods
#' tme_deconvolution_methods
#'
#' # Get internal method name for a specific method
#' tme_deconvolution_methods["MCPcounter"]
tme_deconvolution_methods <- c(
  "MCPcounter"         = "mcpcounter",
  "EPIC"               = "epic",
  "xCell"              = "xcell",
  "CIBERSORT"          = "cibersort",
  "CIBERSORT Absolute" = "cibersort_abs",
  "IPS"                = "ips",
  "ESTIMATE"           = "estimate",
  "SVR"                = "svr",
  "lsei"               = "lsei",
  "TIMER"              = "timer",
  "quanTIseq"          = "quantiseq"
)

# Helper: Add project ID and format result
.format_deconv_result <- function(res, project, suffix) {
  if (is.null(res) || ncol(res) == 0 || nrow(res) == 0) return(NULL)

  colnames(res) <- gsub("\\.", "_", colnames(res))
  colnames(res) <- gsub(" ", "_", colnames(res))
  colnames(res) <- paste0(colnames(res), "_", suffix)

  if (!is.null(project)) {
    res$ProjectID <- project
    res <- res[, c(ncol(res), seq_len(ncol(res) - 1)), drop = FALSE]
  }

  tibble::rownames_to_column(as.data.frame(res), var = "ID")
}

#' Deconvolve Immune Microenvironment Using xCell
#'
#' @description
#' Estimates immune cell fractions using the xCell algorithm. xCell provides
#' cell type enrichment scores for 64 immune and stromal cell types from gene
#' expression data.
#'
#' @param eset Gene expression matrix with HGNC gene symbols as row names and
#'   samples as columns.
#' @param project Optional project name to add as `ProjectID` column.
#'   Default is `NULL`.
#' @param arrays Logical indicating microarray data (`TRUE`) or RNA-seq
#'   (`FALSE`). Default is `FALSE`.
#'
#' @return Data frame with xCell enrichment scores. Cell type columns are
#'   suffixed with `_xCell`.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' \dontrun{
#' xcell <- load_data("xCell.data")
#' if (!is.null(xcell)) {
#'   set.seed(123)
#'   sim_eset <- matrix(rnorm(length(xcell$genes) * 2), length(xcell$genes), 2)
#'   rownames(sim_eset) <- xcell$genes
#'   colnames(sim_eset) <- paste0("Sample", 1:2)
#'   result <- deconvo_xcell(eset = sim_eset, project = "TCGA-STAD")
#'   if (!is.null(result)) head(result)
#' }
#' }
deconvo_xcell <- function(eset, project = NULL, arrays = FALSE) {
  if (is.null(eset)) return(NULL)
  cli::cli_alert_info("Running xCell deconvolution")

  # Validate row names
  rn <- rownames(eset)
  if (is.null(rn) || length(rn) == 0) {
    cli::cli_abort(c(
      "xCell requires gene symbols as row names.",
      "i" = "Your input has no row names.",
      "*" = "For RNA-seq: use HGNC gene symbols as row names with arrays=FALSE",
      "*" = "For microarray: set arrays=TRUE or map probes to gene symbols"
    ))
  }

  # Check identifier format
  ensg_frac <- mean(grepl("^ENSG", rn))
  ensver_frac <- mean(grepl("\\.\\d+$", rn))
  probe_frac <- mean(grepl("(_at$|_s_at$|_x_at$|^AFFX)", rn,
    ignore.case = TRUE
  ))

  if (ensg_frac > 0.5 || probe_frac > 0.5) {
    cli::cli_abort(c(
      "Gene identifier format appears incompatible with xCell.",
      "i" = "Detected: {ensg_frac*100:.1f}% Ensembl-like, \
              {probe_frac*100:.1f}% probe-like",
      "*" = "xCell requires HGNC gene symbols for RNA-seq mode",
      "*" = "Use anno_eset() to convert identifiers to gene symbols"
    ))
  }

  if (any(grepl("\\s", rn))) {
    cli::cli_warn("Row names contain whitespace; consider trimws()")
  }
  if (all(rn == tolower(rn))) {
    cli::cli_warn("Row names are lowercase; xCell uses uppercase HGNC symbols")
  }

  # Run xCell
  rnaseq <- !arrays
  res <- xCellAnalysis(eset, rnaseq = rnaseq)
  if (is.null(res)) return(NULL)
  res <- as.data.frame(t(res))

  .format_deconv_result(res, project, "xCell")
}

#' Deconvolve Immune Microenvironment Using MCP-Counter
#'
#' @description
#' Estimates immune cell abundances using MCP-counter.
#'
#' @param eset Gene expression matrix with HGNC symbols as row names.
#' @param project Optional project name. Default is `NULL`.
#'
#' @return Data frame with MCP-counter scores. Columns suffixed with
#'   `_MCPcounter`.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' mcp_genes <- load_data("mcp_genes")
#' if (!is.null(mcp_genes)) {
#'   set.seed(123)
#'   sim_eset <- matrix(rnorm(nrow(mcp_genes) * 3), nrow(mcp_genes), 3)
#'   rownames(sim_eset) <- mcp_genes$`HUGO symbols`
#'   colnames(sim_eset) <- paste0("Sample", 1:3)
#'   
#'   # Run deconvolution
#'   result <- deconvo_mcpcounter(eset = sim_eset, project = "TCGA-STAD")
#'   if (!is.null(result)) head(result)
#' }
deconvo_mcpcounter <- function(eset, project = NULL) {
  if (is.null(eset)) return(NULL)
  cli::cli_alert_info("Running MCP-counter deconvolution")

  mcp_genes <- load_data("mcp_genes")
  if (is.null(mcp_genes)) return(NULL)
  mcp_probesets <- load_data("mcp_probesets")
  if (is.null(mcp_probesets)) return(NULL)

  res <- MCPcounter.estimate(
    eset,
    featuresType = "HUGO_symbols",
    probesets = mcp_probesets,
    genes = mcp_genes
  )
  res <- as.data.frame(t(res))

  .format_deconv_result(res, project, "MCPcounter")
}

#' Deconvolve Immune Microenvironment Using EPIC
#'
#' @description
#' Estimates immune cell fractions using EPIC algorithm.
#'
#' @param eset Gene expression matrix with genes as row names.
#' @param project Optional project name. Default is `NULL`.
#' @param tumor Logical indicating tumor (`TRUE`) or normal (`FALSE`) samples.
#'
#' @return Data frame with EPIC cell fraction estimates. Columns suffixed with
#'   `_EPIC`.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' \dontrun{
#' TRef <- load_data("TRef")
#' if (!is.null(TRef)) {
#'   set.seed(123)
#'   sim_eset <- matrix(rnorm(nrow(TRef$refProfiles) * 2), nrow(TRef$refProfiles), 2)
#'   rownames(sim_eset) <- rownames(TRef$refProfiles)
#'   colnames(sim_eset) <- paste0("Sample", 1:2)
#'   result <- deconvo_epic(eset = sim_eset, project = "Example", tumor = TRUE)
#'   if (!is.null(result)) head(result)
#' }
#' }
deconvo_epic <- function(eset, project = NULL, tumor = TRUE) {
  if (is.null(eset)) return(NULL)
  cli::cli_alert_info("Running EPIC deconvolution")

  ref <- if (tumor) "TRef" else "BRef"

  out <- EPIC(
    bulk = eset,
    reference = ref,
    mRNA_cell = NULL,
    scaleExprs = TRUE
  )
  if (is.null(out)) return(NULL)
  res <- as.data.frame(out$cellFractions)

  .format_deconv_result(res, project, "EPIC")
}

#' Deconvolve Using CIBERSORT
#'
#' @description
#' CIBERSORT is freely available to academic users. License and binary can be
#' obtained from https://cibersortx.stanford.edu.
#'
#' @param eset Expression matrix with gene symbols as row names.
#' @param project Optional project name. Default is `NULL`.
#' @param arrays Logical: optimized for microarray data. Default is `FALSE`.
#' @param perm Permutations for statistical analysis. Default is 1000.
#' @param absolute Logical: run in absolute mode. Default is `FALSE`.
#' @param abs_method Method for absolute mode: `"sig.score"` or `"no.sumto1"`.
#'   Default is `"sig.score"`.
#' @param parallel Enable parallel execution. Default is `FALSE`.
#' @param num_cores Number of cores for parallel mode. Default is 2.
#' @param seed Random seed for reproducibility. Default is `NULL`.
#'
#' @return Data frame with CIBERSORT cell fractions. Columns suffixed with `_CIBERSORT`.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' \dontrun{
#' lm22 <- load_data("lm22")
#' if (!is.null(lm22)) {
#'   set.seed(123)
#'   sim_eset <- matrix(rnorm(nrow(lm22) * 2), nrow(lm22), 2)
#'   rownames(sim_eset) <- rownames(lm22)
#'   colnames(sim_eset) <- paste0("Sample", 1:2)
#'   result <- deconvo_cibersort(eset = sim_eset, project = "TCGA-STAD", perm = 10)
#'   if (!is.null(result)) head(result)
#' }
#' }
deconvo_cibersort <- function(eset,
                              project = NULL,
                              arrays = FALSE,
                              perm = 1000,
                              absolute = FALSE,
                              abs_method = "sig.score",
                              parallel = FALSE,
                              num_cores = 2,
                              seed = NULL) {
  if (is.null(eset)) return(NULL)
  mode_label <- if (absolute) "CIBERSORT (absolute mode)" else "CIBERSORT"
  cli::cli_alert_info("Running {mode_label}")

  eset <- as.data.frame(eset)
  quantile_norm <- arrays # Disable for RNA-seq

  sig_matrix <- load_data("lm22")
  if (is.null(sig_matrix)) return(NULL)

  res <- CIBERSORT(
    sig_matrix = sig_matrix,
    mixture_file = eset,
    perm = perm,
    QN = quantile_norm,
    absolute = absolute,
    abs_method = abs_method,
    parallel = parallel,
    num_cores = num_cores,
    seed = seed
  )
  if (is.null(res)) return(NULL)

  .format_deconv_result(as.data.frame(res), project, "CIBERSORT")
}

#' Calculate Immunophenoscore (IPS)
#'
#' @description
#' Calculates immune phenotype scores from gene expression data.
#'
#' @param eset Gene expression matrix.
#' @param project Optional project name. Default is `NULL`.
#' @param plot Logical: generate visualization. Default is `FALSE`.
#'
#' @return Data frame with IPS scores. Columns suffixed with `_IPS`.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' \dontrun{
#' ips_genes <- load_data("ips_gene_set")
#' if (!is.null(ips_genes)) {
#'   set.seed(123)
#'   sim_eset <- matrix(rnorm(nrow(ips_genes) * 2), nrow(ips_genes), 2)
#'   rownames(sim_eset) <- ips_genes$GENE
#'   colnames(sim_eset) <- paste0("Sample", 1:2)
#'   result <- deconvo_ips(eset = sim_eset, project = "Example")
#'   if (!is.null(result)) head(result)
#' }
#' }
deconvo_ips <- function(eset, project = NULL, plot = FALSE) {
  if (is.null(eset)) return(NULL)
  cli::cli_alert_info("Running IPS calculation")

  res <- IPS_calculation(project = project, eset = eset, plot = plot)
  if (is.null(res)) return(NULL)

  .format_deconv_result(res, project, "IPS")
}

#' Calculate ESTIMATE Scores
#'
#' @description
#' Calculates stromal, immune, and ESTIMATE scores from gene expression.
#'
#' @param eset Gene expression matrix with gene symbols.
#' @param project Optional project name. Default is `NULL`.
#' @param platform Platform type: `"affymetrix"` or `"illumina"`.
#'   Default is `"affymetrix"`.
#'
#' @return Data frame with ESTIMATE scores. Columns suffixed with `_estimate`.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' \dontrun{
#' common_genes <- load_data("common_genes")
#' if (!is.null(common_genes)) {
#'   set.seed(123)
#'   sim_eset <- matrix(rnorm(nrow(common_genes) * 2), nrow(common_genes), 2)
#'   rownames(sim_eset) <- common_genes$GeneSymbol
#'   colnames(sim_eset) <- paste0("Sample", 1:2)
#'   result <- deconvo_estimate(sim_eset, project = "TCGA-STAD")
#'   if (!is.null(result)) head(result)
#' }
#' }
deconvo_estimate <- function(eset, project = NULL, platform = "affymetrix") {
  if (is.null(eset)) return(NULL)
  cli::cli_alert_info("Running ESTIMATE")

  # Save original sample IDs for later restoration
  original_sample_ids <- colnames(eset)

  eset <- as.data.frame(eset)
  eset <- tibble::rownames_to_column(eset, var = "symbol")

  sampleData <- tempfile(pattern = "estimate_", fileext = ".txt")
  on.exit(unlink(sampleData), add = TRUE)

  utils::write.table(eset, sampleData,
    sep = "\t",
    row.names = FALSE, quote = FALSE
  )

  gct_file <- tempfile(pattern = "estimate_", fileext = ".gct")
  score_file <- tempfile(pattern = "estimate_", fileext = ".gct")
  on.exit(unlink(c(gct_file, score_file)), add = TRUE)

  filter_res <- filterCommonGenes(
    input.f = sampleData,
    output.f = gct_file,
    id = "GeneSymbol"
  )
  if (isFALSE(filter_res)) return(NULL)

  score_res <- estimateScore(
    input.ds = gct_file,
    output.ds = score_file,
    platform = platform
  )
  if (isFALSE(score_res)) return(NULL)

  if (!file.exists(score_file)) return(NULL)
  scores <- tryCatch(utils::read.table(score_file,
    skip = 2, header = TRUE,
    stringsAsFactors = FALSE
  ), error = function(e) NULL)
  if (is.null(scores)) return(NULL)
  
  rownames(scores) <- scores[, 1]
  scores <- t(scores[, 3:ncol(scores), drop = FALSE])

  # Restore original sample IDs (R's read.table converts - to . in column names)
  rownames(scores) <- original_sample_ids

  .format_deconv_result(as.data.frame(scores), project, "estimate")
}

#' Deconvolve Using Custom Reference
#'
#' @description
#' Cell fraction estimation using SVR or lsei methods with custom reference.
#'
#' @param eset Gene expression matrix.
#' @param project Optional project name. Default is `NULL`.
#' @param arrays Logical: use quantile normalization. Default is `TRUE`.
#' @param method Method: `"svr"` or `"lsei"`. Default is `"svr"`.
#' @param perm Permutations for SVR. Default is 100.
#' @param reference Custom reference matrix (e.g., lm22, lm6).
#' @param scale_reference Logical: scale reference. Default is `TRUE`.
#' @param absolute.mode Logical: absolute mode for SVR. Default is `FALSE`.
#' @param abs.method Method for absolute mode. Default is `"sig.score"`.
#'
#' @return Data frame with cell fractions. Columns suffixed with `_CIBERSORT`.
#'
#' @author Dongqiang Zeng, Rongfang Shen
#' @export
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_ref <- matrix(rnorm(100 * 5), 100, 5)
#' rownames(sim_ref) <- paste0("Gene", 1:100)
#' colnames(sim_ref) <- paste0("CellType", 1:5)
#'
#' sim_eset <- matrix(rnorm(100 * 3), 100, 3)
#' rownames(sim_eset) <- paste0("Gene", 1:100)
#' colnames(sim_eset) <- paste0("Sample", 1:3)
#'
#' # Run deconvolution
#' result <- deconvo_ref(eset = sim_eset, reference = sim_ref, method = "lsei")
#' if (!is.null(result)) head(result)
deconvo_ref <- function(eset,
                        project = NULL,
                        arrays = TRUE,
                        method = c("svr", "lsei"),
                        perm = 100,
                        reference,
                        scale_reference = TRUE,
                        absolute.mode = FALSE,
                        abs.method = "sig.score") {
  if (is.null(eset)) return(NULL)
  method <- rlang::arg_match(method)
  rlang::check_installed("limSolve")

  # Check gene overlap
  common_genes <- intersect(rownames(eset), rownames(reference))
  if (length(common_genes) == 0) {
    cli::cli_abort(c(
      "No matching genes between eset and reference.",
      "i" = "Check gene identifier formats match."
    ))
  }
  cli::cli_alert_info("Found {length(common_genes)} common genes")

  if (method == "svr") {
    cli::cli_alert_info("Running SVR deconvolution")

    res <- CIBERSORT(
      sig_matrix = reference,
      mixture_file = as.data.frame(eset),
      perm = perm,
      QN = arrays,
      absolute = absolute.mode,
      abs_method = abs.method
    )
    res <- as.data.frame(res)
  } else if (method == "lsei") {
    cli::cli_alert_info("Running lsei deconvolution")

    eset <- as.matrix(eset)
    reference <- as.matrix(reference)

    # Quantile normalization
    if (arrays) {
      rlang::check_installed("preprocessCore")
      eset <- preprocessCore::normalize.quantiles(eset)
    }

    # Scale reference (global scaling for backward compatibility)
    if (scale_reference) {
      reference <- (reference - mean(reference)) / sd(as.vector(reference))
    }

    # Find common genes
    common <- intersect(rownames(eset), rownames(reference))
    eset <- eset[match(common, rownames(eset)), , drop = FALSE]
    reference <- reference[match(common, rownames(reference)), , drop = FALSE]

    # Setup lsei constraints
    Numofx <- ncol(reference)
    AA <- reference
    EE <- rep(1, Numofx)
    FF <- 1
    GG <- diag(nrow = Numofx)
    HH <- rep(0, Numofx)

    # Run deconvolution
    out.all <- vapply(seq_len(ncol(eset)), function(i) {
      BB <- eset[, i]
      BB <- (BB - mean(BB)) / stats::sd(BB)
      out <- limSolve::lsei(AA, BB, EE, FF, GG, HH)
      out$X
    }, numeric(Numofx))

    res <- as.data.frame(t(out.all))
    colnames(res) <- colnames(reference)
    rownames(res) <- colnames(eset)
  }

  .format_deconv_result(res, project, "CIBERSORT")
}

#' Deconvolve Using TIMER
#'
#' @description
#' TIMER deconvolution for cancer-specific immune estimation.
#'
#' @param eset Gene expression matrix.
#' @param project Optional project name. Default is `NULL`.
#' @param indications Cancer type for each sample (e.g., `"brca"`, `"stad"`).
#'   Must match number of columns in `eset`.
#'
#' @return Data frame with TIMER cell fractions. Columns suffixed with `_TIMER`.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' \dontrun{
#' immune <- load_data("immuneCuratedData")
#' cancer_genes <- load_data("cancer_type_genes")
#' if (!is.null(immune) && !is.null(cancer_genes)) {
#'   set.seed(123)
#'   genes <- unique(c(head(rownames(immune$genes), 100), cancer_genes[["stad"]]))
#'   sim_eset <- matrix(rnorm(length(genes) * 2), length(genes), 2)
#'   rownames(sim_eset) <- genes
#'   colnames(sim_eset) <- paste0("Sample", 1:2)
#'   result <- deconvo_timer(eset = sim_eset, project = "TCGA-STAD",
#'                           indications = rep("stad", 2))
#'   if (!is.null(result)) head(result)
#' }
#' }
deconvo_timer <- function(eset, project = NULL, indications = NULL) {
  if (is.null(eset)) return(NULL)
  cli::cli_alert_info("Running TIMER deconvolution")

  if (!is.null(indications)) {
    indications <- tolower(indications)

    if (length(indications) != ncol(eset)) {
      cli::cli_abort(c(
        "Length of 'indications' must match number of samples.",
        "i" = "Got {length(indications)} indications for {ncol(eset)} samples"
      ))
    }
  }

  # Prepare temp files
  args <- new.env()
  args$outdir <- tempdir()
  args$batch <- tempfile()

  # Write data for each cancer type
  unique_inds <- unique(indications)
  for (ind in unique_inds) {
    tmp_file <- tempfile()
    tmp_mat <- eset[, indications == ind, drop = FALSE]
    tmp_mat <- tibble::as_tibble(tmp_mat, rownames = "gene_symbol")
    utils::write.table(tmp_mat, tmp_file,
      sep = "\t",
      quote = FALSE, row.names = FALSE
    )
    cat(paste0(tmp_file, ",", ind, "\n"), file = args$batch, append = TRUE)
  }

  # Run TIMER
  results <- deconvolute_timer.default(args)
  if (is.null(results)) return(NULL)
  results <- results[, colnames(eset), drop = FALSE]
  colnames(results) <- colnames(eset)
  results <- as.data.frame(t(results))

  .format_deconv_result(results, project, "TIMER")
}

#' Deconvolve Using quanTIseq
#'
#' @description
#' quanTIseq deconvolution for RNA-seq immune cell fractions.
#'
#' @param eset Gene expression matrix.
#' @param project Optional project name. Default is `NULL`.
#' @param tumor Logical: tumor samples. Must be specified.
#' @param arrays Logical: microarray data. Must be specified.
#' @param scale_mrna Logical: correct for mRNA content. Must be specified.
#'
#' @return Data frame with quanTIseq cell fractions. Columns suffixed with
#'   `_quantiseq`.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' \dontrun{
#' quantiseq_data <- load_data("quantiseq_data")
#' if (!is.null(quantiseq_data)) {
#'   set.seed(123)
#'   n_sig <- nrow(quantiseq_data$TIL10_signature)
#'   sim_eset <- matrix(rnorm(n_sig * 2), n_sig, 2)
#'   rownames(sim_eset) <- rownames(quantiseq_data$TIL10_signature)
#'   colnames(sim_eset) <- paste0("Sample", 1:2)
#'   result <- deconvo_quantiseq(eset = sim_eset, project = "Example", tumor = TRUE,
#'                               arrays = FALSE, scale_mrna = FALSE)
#'   if (!is.null(result)) head(result)
#' }
#' }
deconvo_quantiseq <- function(eset, project = NULL, tumor, arrays, scale_mrna) {
  if (is.null(eset)) return(NULL)
  cli::cli_alert_info("Running quanTIseq deconvolution")

  res <- deconvolute_quantiseq.default(
    mix.mat = eset,
    tumor = tumor,
    arrays = arrays,
    mRNAscale = scale_mrna
  )
  if (is.null(res)) return(NULL)

  res <- tibble::column_to_rownames(tibble::as_tibble(res), var = "Sample")

  .format_deconv_result(res, project, "quantiseq")
}

#' Main TME Deconvolution Function
#'
#' @description
#' Unified interface for multiple TME deconvolution methods.
#'
#' @param eset Gene expression matrix with HGNC symbols as row names.
#' @param project Optional project name. Default is `NULL`.
#' @param method Deconvolution method. See [tme_deconvolution_methods].
#' @param arrays Logical: microarray-optimized mode. Default is `FALSE`.
#' @param tumor Logical: tumor-optimized mode (EPIC). Default is `TRUE`.
#' @param perm Permutations (CIBERSORT/SVR). Default is 1000.
#' @param reference Custom reference matrix (SVR/lsei).
#' @param scale_reference Logical: scale reference (SVR/lsei).
#' @param plot Logical: generate plots (IPS). Default is `FALSE`.
#' @param scale_mrna Logical: mRNA correction (quanTIseq/EPIC).
#' @param group_list Cancer types for TIMER (vector).
#' @param platform Platform for ESTIMATE. Default is `"affymetrix"`.
#' @param absolute.mode Logical: absolute mode (CIBERSORT/SVR).
#'   Default is `FALSE`.
#' @param abs.method Absolute mode method. Default is `"sig.score"`.
#' @param ... Additional arguments passed to method.
#'
#' @return Tibble with cell fractions and `ID` column.
#'
#' @author Dongqiang Zeng, Rongfang Shen
#' @export
#'
#' @references
#' \enumerate{
#'   \item Newman et al. (2015). Robust enumeration of cell subsets from tissue
#'     expression profiles. Nature Methods.
#'   \item Vegesna et al. (2013). Inferring tumour purity and stromal/immune
#'     cell admixture. Nature Communications.
#'   \item Finotello et al. (2019). Molecular and pharmacological modulators of
#'     the tumor immune contexture. Genome Medicine.
#'   \item Li et al. (2016). Comprehensive analyses of tumor immunity.
#'     Genome Biology.
#'   \item Charoentong et al. (2017). Pan-cancer Immunogenomic Analyses.
#'     Cell Reports.
#'   \item Becht et al. (2016). Estimating population abundance of tissue-infiltrating
#'     immune cells. Genome Biology.
#'   \item Aran et al. (2017). xCell: digitally portraying tissue cellular
#'     heterogeneity. Genome Biology.
#'   \item Racle et al. (2017). Simultaneous enumeration of cancer and immune
#'     cell types. ELife.
#' }
#'
#' @examples
#' mcp_genes <- load_data("mcp_genes")
#' if (!is.null(mcp_genes)) {
#'   set.seed(123)
#'   sim_eset <- matrix(rnorm(nrow(mcp_genes) * 3), nrow(mcp_genes), 3)
#'   rownames(sim_eset) <- mcp_genes$`HUGO symbols`
#'   colnames(sim_eset) <- paste0("Sample", 1:3)
#'   
#'   # Run deconvolution
#'   result <- deconvo_tme(eset = sim_eset, method = "mcpcounter")
#'   if (!is.null(result)) head(result)
#' }
deconvo_tme <- function(eset,
                        project = NULL,
                        method = tme_deconvolution_methods,
                        arrays = FALSE,
                        tumor = TRUE,
                        perm = 1000,
                        reference,
                        scale_reference = TRUE,
                        plot = FALSE,
                        scale_mrna = TRUE,
                        group_list = NULL,
                        platform = "affymetrix",
                        absolute.mode = FALSE,
                        abs.method = "sig.score",
                        ...) {
  if (is.null(eset)) return(NULL)
  method <- rlang::arg_match(method, tme_deconvolution_methods)

  # Validate input
  if (any(grepl("^ENSG000", rownames(eset)))) {
    cli::cli_abort(c(
      "Ensembl IDs detected in row names.",
      "i" = "Most deconvolution methods require HGNC gene symbols.",
      "*" = "Use anno_eset() to convert Ensembl IDs to gene symbols"
    ))
  }

  if (max(eset, na.rm = TRUE) < 50) {
    cli::cli_warn(c(
      "Data values appear small (< 50).",
      "i" = "Input should be in TPM/FPKM scale, not log-transformed"
    ))
  }

  # Dispatch to method
  res <- switch(method,
    xcell = deconvo_xcell(eset, project, arrays = arrays, ...),
    mcpcounter = deconvo_mcpcounter(eset, project, ...),
    epic = deconvo_epic(eset, project, tumor = tumor, ...),
    cibersort = deconvo_cibersort(eset, project,
      absolute = absolute.mode,
      arrays = arrays, perm = perm, ...
    ),
    cibersort_abs = deconvo_cibersort(eset, project,
      absolute = TRUE,
      abs_method = abs.method,
      arrays = arrays, perm = perm, ...
    ),
    ips = deconvo_ips(eset, project, plot = plot, ...),
    quantiseq = deconvo_quantiseq(eset, project,
      tumor = tumor,
      arrays = arrays, scale_mrna = scale_mrna, ...
    ),
    estimate = deconvo_estimate(eset, project, platform, ...),
    timer = deconvo_timer(eset, project, indications = group_list, ...),
    svr = deconvo_ref(eset, project,
      reference = reference,
      arrays = arrays, method = "svr",
      absolute.mode = absolute.mode, ...
    ),
    lsei = deconvo_ref(eset, project,
      reference = reference,
      arrays = arrays, method = "lsei",
      scale_reference = scale_reference, ...
    )
  )

  if (is.null(res)) return(NULL)
  tibble::as_tibble(res)
}

#' Calculate Immunophenoscore (IPS)
#'
#' @description
#' Calculates Immunophenoscore (IPS) from gene expression data. IPS is a
#' composite score measuring immunophenotype based on four major categories:
#' MHC molecules, immunomodulators, effector cells, and suppressor cells.
#'
#' @param eset Gene expression matrix with official human gene symbols (HGNC)
#'   as rownames. Expression values should be log2(TPM+1) or will be transformed
#'   if max value > 100.
#' @param plot Logical. Whether to generate immunophenogram plots. Default is FALSE.
#' @param project Character string for project identifier. Default is NULL.
#'
#' @return Data frame containing:
#' \describe{
#'   \item{MHC}{MHC molecules score}
#'   \item{EC}{Effector cells score}
#'   \item{SC}{Suppressor cells score}
#'   \item{CP}{Checkpoints/Immunomodulators score}
#'   \item{AZ}{Aggregate score (sum of MHC, CP, EC, SC)}
#'   \item{IPS}{Immunophenoscore (0-10 scale)}
#' }
#'
#' @export
#'
#' @import dplyr
#' @import tibble
#' @import tidyr
#' @import purrr
#' @import stringr
#' @import ggplot2
#' @import grid
#' @examples
#' \dontrun{
#' example_genes <- c(
#'   "HLA-A", "HLA-B", "HLA-C", "CD274", "PDCD1", "CTLA4",
#'   "CD8A", "CD8B", "GZMB", "PRF1", "FOXP3", "IL10"
#' )
#' sim_eset <- as.data.frame(matrix(
#'   rnorm(length(example_genes) * 5, mean = 5, sd = 2),
#'   nrow = length(example_genes), ncol = 5
#' ))
#' rownames(sim_eset) <- example_genes
#' colnames(sim_eset) <- paste0("Sample", 1:5)
#' ips_result <- IPS_calculation(eset = sim_eset, project = "Example", plot = FALSE)
#' if (!is.null(ips_result)) head(ips_result)
#' }
IPS_calculation <- function(eset, project = NULL, plot = FALSE) {
  if (is.null(eset)) return(NULL)

  if (!is.matrix(eset) && !is.data.frame(eset)) {
    cli::cli_abort("{.arg eset} must be a matrix or data frame")
  }
  if (ncol(eset) == 0) {
    cli::cli_abort("{.arg eset} must have at least one sample")
  }

  if (max(eset, na.rm = TRUE) > 100) {
    eset <- log2(eset + 1)
  }

  gene_expression <- eset
  sample_names <- colnames(gene_expression)
  if (is.null(sample_names)) {
    sample_names <- paste0("Sample", seq_len(ncol(gene_expression)))
    colnames(gene_expression) <- sample_names
  }

  IPSG <- load_data("ips_gene_set")
  if (is.null(IPSG)) return(NULL)

  IPSG <- IPSG[IPSG$GENE %in% rownames(gene_expression), ]

  if (nrow(IPSG) == 0) {
    cli::cli_abort("No IPS genes found in the expression data")
  }

  unique_ips_genes <- unique(IPSG$NAME)

  GVEC <- row.names(gene_expression)
  VEC <- as.vector(IPSG$GENE)
  ind <- which(is.na(match(VEC, GVEC)))
  MISSING_GENES <- VEC[ind]

  if (length(MISSING_GENES) > 0) {
    cli::cli_alert_warning("Differently named or missing genes: {paste(MISSING_GENES, collapse = ', ')}")
  }

  n_samples <- ncol(eset)
  MHC <- numeric(n_samples)
  CP <- numeric(n_samples)
  EC <- numeric(n_samples)
  SC <- numeric(n_samples)
  AZ <- numeric(n_samples)
  IPS <- numeric(n_samples)

  for (i in seq_len(n_samples)) {
    GE <- gene_expression[, i]
    mGE <- mean(GE, na.rm = TRUE)
    sGE <- sd(GE, na.rm = TRUE)
    Z1 <- (gene_expression[as.vector(IPSG$GENE), i] - mGE) / sGE
    W1 <- IPSG$WEIGHT

    MIG <- numeric(length(unique_ips_genes))
    WEIGHT <- numeric(length(unique_ips_genes))

    for (k in seq_along(unique_ips_genes)) {
      gen <- unique_ips_genes[k]
      MIG[k] <- mean(Z1[which(as.vector(IPSG$NAME) == gen)], na.rm = TRUE)
      WEIGHT[k] <- mean(W1[which(as.vector(IPSG$NAME) == gen)], na.rm = TRUE)
    }

    WG <- MIG * WEIGHT
    MHC[i] <- mean(WG[seq_len(10)], na.rm = TRUE)
    CP[i] <- mean(WG[11:20], na.rm = TRUE)
    EC[i] <- mean(WG[21:24], na.rm = TRUE)
    SC[i] <- mean(WG[25:26], na.rm = TRUE)
    AZ[i] <- sum(MHC[i], CP[i], EC[i], SC[i], na.rm = TRUE)
    IPS[i] <- ipsmap(AZ[i])

    if (plot) {
      file_name <- file.path(tempdir(), "IPS-Results")
      if (!dir.exists(file_name)) dir.create(file_name, recursive = TRUE)

      plotpath <- file.path(file_name, "IPS_plot_results")
      if (!dir.exists(plotpath)) dir.create(plotpath, recursive = TRUE)

      data_a <- data.frame(
        start = c(0, 2.5, 5, 7.5, 10, 15, seq(20, 39), 0, 10, 20, 30),
        end = c(2.5, 5, 7.5, 10, 15, seq(20, 40), 10, 20, 30, 40),
        y1 = c(rep(2.6, 26), rep(0.4, 4)),
        y2 = c(rep(5.6, 26), rep(2.2, 4)),
        z = c(MIG[c(21:26, 11:20, 1:10)], EC[i], SC[i], CP[i], MHC[i]),
        vcol = c(
          unlist(lapply(MIG[c(21:26, 11:20, 1:10)], mapcolors)),
          unlist(lapply(c(EC[i], SC[i], CP[i], MHC[i]), mapbw))
        ),
        label = c(unique_ips_genes[c(21:26, 11:20, 1:10)], "EC", "SC", "CP", "MHC")
      )
      data_a$label <- factor(data_a$label, levels = unique(data_a$label))

      plot_a1 <- ggplot2::ggplot() +
        ggplot2::geom_rect(
          data = data_a, mapping = ggplot2::aes(xmin = start, xmax = end, ymin = y1, ymax = y2, fill = label),
          size = 0.5, color = "black", alpha = 1
        ) +
        ggplot2::coord_polar() +
        ggplot2::scale_y_continuous(limits = c(0, 6)) +
        ggplot2::scale_fill_manual(values = as.vector(data_a$vcol), guide = FALSE) +
        ggplot2::theme_bw() +
        ggplot2::theme(
          panel.margin = grid::unit(0, "mm"), panel.grid.major = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank(), panel.border = ggplot2::element_blank(),
          panel.background = ggplot2::element_blank(), axis.line = ggplot2::element_line(colour = "white"),
          axis.text = ggplot2::element_blank(), axis.ticks = ggplot2::element_blank()
        ) +
        ggplot2::geom_text(ggplot2::aes(x = 5, y = 1.3, label = "EC"), size = 4) +
        ggplot2::geom_text(ggplot2::aes(x = 15, y = 1.3, label = "SC"), size = 4) +
        ggplot2::geom_text(ggplot2::aes(x = 25, y = 1.3, label = "CP"), size = 4) +
        ggplot2::geom_text(ggplot2::aes(x = 35, y = 1.3, label = "MHC"), size = 4)

      plot_a2 <- plot_a1 +
        geom_text(aes(x = 1.25, y = 4.1, label = "+ Act CD4"), angle = 78.75, size = 4) +
        geom_text(aes(x = 3.75, y = 4.1, label = "+ Act CD8"), angle = 56.25, size = 4) +
        geom_text(aes(x = 6.25, y = 4.1, label = "+ Tem CD4"), angle = 33.75, size = 4) +
        geom_text(aes(x = 8.75, y = 4.1, label = "+ Tem CD8"), angle = 11.25, size = 4) +
        geom_text(aes(x = 17.5, y = 4.1, label = "- MDSC"), angle = -67.5, size = 4) +
        geom_text(aes(x = 12.5, y = 4.1, label = "- Treg"), angle = -22.5, size = 4)

      plot_a3 <- plot_a2 +
        geom_text(aes(x = 20.5, y = 4.1, label = "PD-1 -"), angle = 85.5, size = 4) +
        geom_text(aes(x = 21.5, y = 4.1, label = "CTLA4 -"), angle = 76.5, size = 4) +
        geom_text(aes(x = 22.5, y = 4.1, label = "LAG3 -"), angle = 67.5, size = 4) +
        geom_text(aes(x = 23.5, y = 4.1, label = "TIGIT -"), angle = 58.5, size = 4) +
        geom_text(aes(x = 24.5, y = 4.1, label = "TIM3 -"), angle = 49.5, size = 4) +
        geom_text(aes(x = 25.5, y = 4.1, label = "PD-L1 -"), angle = 40.5, size = 4) +
        geom_text(aes(x = 26.5, y = 4.1, label = "PD-L2 -"), angle = 31.5, size = 4) +
        geom_text(aes(x = 27.5, y = 4.1, label = "CD27 +"), angle = 22.5, size = 4) +
        geom_text(aes(x = 28.5, y = 4.1, label = "ICOS +"), angle = 13.5, size = 4) +
        geom_text(aes(x = 29.5, y = 4.1, label = "IDO1 -"), angle = 4.5, size = 4)

      plot_a4 <- plot_a3 +
        geom_text(aes(x = 30.5, y = 4.1, label = "B2M +"), angle = -4.5, size = 4) +
        geom_text(aes(x = 31.5, y = 4.1, label = "TAP1 +"), angle = -13.5, size = 4) +
        geom_text(aes(x = 32.5, y = 4.1, label = "TAP2 +"), angle = -22.5, size = 4) +
        geom_text(aes(x = 33.5, y = 4.1, label = "HLA-A +"), angle = -31.5, size = 4) +
        geom_text(aes(x = 34.5, y = 4.1, label = "HLA-B +"), angle = -40.5, size = 4) +
        geom_text(aes(x = 35.5, y = 4.1, label = "HLA-C +"), angle = -49.5, size = 4) +
        geom_text(aes(x = 36.5, y = 4.1, label = "HLA-DPA1 +"), angle = -58.5, size = 4) +
        geom_text(aes(x = 37.5, y = 4.1, label = "HLA-DPB1 +"), angle = -67.5, size = 4) +
        geom_text(aes(x = 38.5, y = 4.1, label = "HLA-E +"), angle = -76.5, size = 4) +
        geom_text(aes(x = 39.5, y = 4.1, label = "HLA-F +"), angle = -85.5, size = 4)

      plot_a5 <- plot_a4 +
        ggplot2::geom_text(ggplot2::aes(x = 0, y = 6, label = paste("Immunophenoscore: ", IPS[i], sep = "")), angle = 0, size = 6, vjust = -0.5) +
        ggplot2::theme(axis.title = ggplot2::element_blank())

      plot_a <- plot_a5 +
        ggplot2::theme(plot.margin = grid::unit(c(0, 0, 0, 0), "mm")) +
        ggplot2::geom_text(vjust = 1.15, hjust = 0, ggplot2::aes(x = 25.5, y = 6, label = "\n\n\n\n   MHC: Antigen Processing                                 EC: Effector Cells\n   CP: Checkpoints | Immunomodulators              SC: Suppressor Cells\n\n", hjust = 0), size = 4)

      data_b <- data.frame(
        start = rep(0, 23), end = rep(0.7, 23), y1 = seq(0, 22, by = 1), y2 = seq(1, 23, by = 1),
        z = seq(-3, 3, by = 6 / 22), vcol = c(unlist(lapply(seq(-3, 3, by = 6 / 22), mapcolors))),
        label = LETTERS[1:23]
      )
      data_b_ticks <- data.frame(x = rep(1.2, 7), value = seq(-3, 3, by = 1), y = seq(0, 6, by = 1) * (22 / 6) + 0.5)

      legendtheme <- ggplot2::theme(
        plot.margin = grid::unit(c(2, 0, 2, 0), "inch"), panel.margin = grid::unit(0, "null"),
        panel.grid.major = ggplot2::element_blank(), panel.grid.minor = ggplot2::element_blank(),
        panel.border = ggplot2::element_blank(), panel.background = ggplot2::element_blank(),
        axis.line = ggplot2::element_line(colour = "white"), axis.text = ggplot2::element_blank(),
        axis.ticks = ggplot2::element_blank(), axis.title.x = ggplot2::element_blank()
      )

      plot_b <- ggplot2::ggplot(hjust = 0) +
        ggplot2::geom_rect(data = data_b, mapping = ggplot2::aes(xmin = start, xmax = end, ymin = y1, ymax = y2, fill = label), size = 0.5, color = "black", alpha = 1) +
        ggplot2::scale_x_continuous(limits = c(0, 1.5), expand = c(0, 0)) +
        ggplot2::scale_fill_manual(values = as.vector(data_b$vcol), guide = FALSE) +
        ggplot2::geom_text(data = data_b_ticks, ggplot2::aes(x = x, y = y, label = value), hjust = "inward", size = 4) +
        ggplot2::theme_bw() +
        legendtheme +
        ggplot2::ylab("Sample-wise (averaged) z-score")

      data_c <- data.frame(
        start = rep(0, 23), end = rep(0.7, 23), y1 = seq(0, 22, by = 1), y2 = seq(1, 23, by = 1),
        z = seq(-2, 2, by = 4 / 22), vcol = c(unlist(lapply(seq(-2, 2, by = 4 / 22), mapbw))),
        label = LETTERS[1:23]
      )
      data_c_ticks <- data.frame(x = rep(1.2, 5), value = seq(-2, 2, by = 1), y = seq(0, 4, by = 1) * (22 / 4) + 0.5)

      plot_c <- ggplot2::ggplot() +
        ggplot2::geom_rect(
          data = data_c, mapping = ggplot2::aes(xmin = start, xmax = end, ymin = y1, ymax = y2, fill = label),
          size = 0.5, color = "black", alpha = 1
        ) +
        ggplot2::scale_x_continuous(limits = c(0, 1.5), expand = c(0, 0)) +
        ggplot2::scale_fill_manual(values = as.vector(data_c$vcol), guide = FALSE) +
        ggplot2::geom_text(data = data_c_ticks, ggplot2::aes(x = x, y = y, label = value), hjust = "inward", size = 4) +
        ggplot2::theme_bw() +
        legendtheme +
        ggplot2::ylab("Weighted z-score")

      filename <- file.path(plotpath, paste0("IPS_", sample_names[i], ".pdf"))
      pdf(filename, width = 10, height = 8)
      gridExtra::grid.arrange(plot_a, plot_b, plot_c, ncol = 3, widths = c(0.8, 0.1, 0.1))
      dev.off()
    }
  }

  res <- data.frame(ID = sample_names, MHC = MHC, EC = EC, SC = SC, CP = CP, AZ = AZ, IPS = IPS)

  if (!is.null(project)) {
    res$ProjectID <- project
    res <- res[, c(ncol(res), seq_len(ncol(res) - 1))]
  }

  res <- tibble::column_to_rownames(res, var = "ID")

  return(res)
}
######################################
######################################
######################################

#' Tumor Microenvironment (TME) Deconvolution Pipeline
#'
#' Executes an integrated TME analysis on a gene expression matrix: performs
#' immune/stromal cell deconvolution using multiple algorithms, computes
#' signature scores, and aggregates results. Designed for exploratory
#' immunogenomic profiling.
#'
#' @param eset Numeric matrix. Gene expression (TPM/log scale) with genes in rows.
#' @param project Character. Project name (used in output naming).
#' @param array Logical. Whether data originated from an array platform. Affects deconvolution choices.
#' @param tumor_type Character. Tumor type code (e.g., "stad") used by certain methods.
#' @param path Character. Output directory. Default is NULL (uses tempdir()).
#' @param permutation Integer. Number of permutations for CIBERSORT (and similar). Default is 1000.
#'
#' @return Data frame integrating cell fractions and signature scores (also writes intermediate outputs to disk).
#' @author Dongqiang Zeng
#' @export
#' @examples
#' \dontrun{
#' lm22 <- load_data("lm22")
#' cancer_genes <- load_data("cancer_type_genes")
#' if (!is.null(lm22) && !is.null(cancer_genes)) {
#'   set.seed(123)
#'   genes <- rownames(lm22)
#'   xcell <- load_data("xCell.data")
#'   if (!is.null(xcell)) genes <- unique(c(genes, xcell$genes))
#'   genes <- unique(c(genes, cancer_genes[["stad"]]))
#'   eset <- matrix(runif(length(genes) * 2), nrow = length(genes), ncol = 2)
#'   rownames(eset) <- genes
#'   colnames(eset) <- paste0("Sample", 1:2)
#'   res <- iobr_deconvo_pipeline(
#'     eset = eset, project = "TEST",
#'     array = FALSE, tumor_type = "stad",
#'     path = tempdir(), permutation = 2
#'   )
#'   if (!is.null(res)) head(res)
#' }
#' }
#'
iobr_deconvo_pipeline <- function(eset, project, array, tumor_type, path = NULL, permutation = 1000) {

  if (is.null(eset)) return(NULL)

  if (is.null(path)) {
    path <- tempdir()
  }
  #######################################
  path <- creat_folder(paste0(path))
  #######################################
  eset <- log2eset(eset)
  #######################################

  # tumor_type<-"stad"
  #########################################
  cibersort <- deconvo_tme(eset = eset, method = "cibersort", arrays = array, perm = permutation)
  epic <- deconvo_tme(eset = eset, method = "epic", arrays = array)
  mcp <- deconvo_tme(eset = eset, method = "mcpcounter")
  xcell <- deconvo_tme(eset = eset, method = "xcell", arrays = array)
  estimate <- deconvo_tme(eset = eset, method = "estimate")
  timer <- deconvo_tme(eset = eset, method = "timer", group_list = rep(tumor_type, dim(eset)[2]))
  quantiseq <- deconvo_tme(eset = eset, method = "quantiseq", tumor = TRUE, arrays = array, scale_mrna = TRUE)
  ips <- deconvo_tme(eset = eset, method = "ips", plot = FALSE)

  res_list <- list(cibersort, mcp, xcell, epic, estimate, quantiseq, timer, ips)
  res_list <- res_list[!sapply(res_list, is.null)]

  if (length(res_list) == 0) {
    cli::cli_alert_warning("All deconvolution methods returned NULL (likely due to no internet connection).")
    return(NULL)
  }

  tme_combine <- res_list[[1]]
  if (length(res_list) > 1) {
    for (i in 2:length(res_list)) {
      tme_combine <- dplyr::inner_join(tme_combine, res_list[[i]], by = "ID")
    }
  }

  # tme_combine<-tme_combine[,-c(grep(colnames(tme_combine),pattern = "Index"))]
  ############################################
  save(tme_combine, file = paste0(path$abspath, "1-", project, "-TME-Cell-fration.RData"))
  print(paste0(">>>>> TME cell deconvolution was completed: ", project))
  #######################################

  signature_collection <- load_data("signature_collection")
  if (is.null(signature_collection)) return(NULL)

  sig_res <- calculate_sig_score(
    pdata = NULL,
    eset = eset,
    signature = signature_collection,
    adjust_eset = TRUE,
    method = "integration",
    mini_gene_count = 2
  )
  print(paste0(">>>>> Signature esitmation was completed: ", project))
  save(sig_res, file = paste0(path$abspath, "2-", project, "-Signature-score-mycollection.RData"))
  ########################################
  hallmark_data <- load_data("hallmark")
  if (is.null(hallmark_data)) return(NULL)

  go_bp_data <- load_data("go_bp")
  if (is.null(go_bp_data)) return(NULL)

  go_cc_data <- load_data("go_cc")
  if (is.null(go_cc_data)) return(NULL)

  go_mf_data <- load_data("go_mf")
  if (is.null(go_mf_data)) return(NULL)

  kegg_data <- load_data("kegg")
  if (is.null(kegg_data)) return(NULL)

  reactome_data <- load_data("reactome")
  if (is.null(reactome_data)) return(NULL)

  sig_go_kegg <- calculate_sig_score(
    pdata = NULL,
    eset = eset,
    adjust_eset = TRUE,
    signature = c(hallmark_data, go_bp_data, go_cc_data, go_mf_data, kegg_data, reactome_data),
    method = "ssgsea",
    mini_gene_count = 2
  )


  save(sig_go_kegg, file = paste0(path$abspath, "3-", project, "-Signature-score-Hallmark-GO-KEGG.RData"))
  print(paste0(">>>>> HALLMARK GO KEGG REACTOME esitmation was completed: ", project))
  #########################################
  tme_sig_combin <- tme_combine
  if (!is.null(sig_res)) {
    tme_sig_combin <- dplyr::inner_join(tme_sig_combin, sig_res, by = "ID")
  }
  if (!is.null(sig_go_kegg)) {
    tme_sig_combin <- dplyr::inner_join(tme_sig_combin, sig_go_kegg, by = "ID")
  }
  save(tme_sig_combin, file = paste0(path$abspath, "0-", project, "-Merge-TME-Signature-and-Hallmark-GO-KEGG.RData"))
  #########################################
  return(tme_sig_combin)
}

#' Row Bind Multiple Data Sets
#'
#' @description
#' Combines two or three data frames or matrices vertically using `rbind`.
#' Ensures compatibility of input data before binding by aligning columns.
#'
#' @param data1 Data frame or matrix. First dataset to combine.
#' @param data2 Data frame or matrix. Second dataset to combine.
#' @param data3 Data frame or matrix or `NULL`. Optional third dataset.
#'   Default is `NULL`.
#'
#' @return Combined data frame resulting from row binding the input datasets.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' data1 <- data.frame(A = 1:5, B = letters[1:5])
#' data2 <- data.frame(A = 6:10, B = letters[6:10])
#' combined_data <- rbind_iobr(data1, data2)
#'
#' # With three datasets
#' data3 <- data.frame(A = 11:15, B = letters[11:15])
#' combined_data <- rbind_iobr(data1, data2, data3)
rbind_iobr <- function(data1, data2, data3 = NULL) {
  if (is.null(data1) || is.null(data2)) {
    cli::cli_abort("{.arg data1} and {.arg data2} must not be NULL.")
  }

  if (!is.data.frame(data1) && !is.matrix(data1)) {
    cli::cli_abort("{.arg data1} must be a data frame or matrix.")
  }
  if (!is.data.frame(data2) && !is.matrix(data2)) {
    cli::cli_abort("{.arg data2} must be a data frame or matrix.")
  }

  data2 <- assimilate_data(data_a = data1, data_b = data2)
  data.com <- rbind(data1, data2)

  if (!is.null(data3)) {
    if (!is.data.frame(data3) && !is.matrix(data3)) {
      cli::cli_abort("{.arg data3} must be a data frame, matrix, or NULL.")
    }
    data3 <- assimilate_data(data_a = data.com, data_b = data3)
    data.com <- rbind(data.com, data3)
  }

  data.com
}

#' Design Custom Theme for ggplot2 Plots
#'
#' @description
#' Creates a customized ggplot2 theme based on user-specified parameters for
#' plot elements such as title size, axis sizes, legend settings, and theme style.
#' Supports various base themes and allows fine-tuning of visual aspects.
#'
#' @param theme Base theme: "light", "bw", "classic", "classic2".
#'   Default is "light".
#' @param plot_title_size Relative size of plot title. Default is 2.
#' @param axis_title_size Relative size of axis titles. Default is 2.
#' @param axis_text_size Size of axis tick labels. Default is 12.
#' @param axis_angle Angle of x-axis tick labels. Default is 60.
#' @param hjust Horizontal justification for x-axis text. Default is 1.
#' @param legend.position Legend position: "none", "left", "right", "bottom",
#'   "top". Default is "bottom".
#' @param legend.direction Direction of legend items: "horizontal" or "vertical".
#'   Default is "horizontal".
#' @param legend.size Size of legend key. Default is 0.25.
#' @param legend.key.height Height of legend key in cm. Default is 0.5.
#' @param legend.key.width Width of legend key in cm. Default is 0.5.
#' @param legend.size.text Size of legend text labels. Default is 10.
#' @param legend.box Orientation of legend box: "horizontal" or "vertical".
#'   Default is "horizontal".
#'
#' @return A ggplot2 theme object.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' library(ggplot2)
#' p <- ggplot(mtcars, aes(wt, mpg)) +
#'   geom_point()
#' mytheme <- design_mytheme(theme = "bw", plot_title_size = 1.5, axis_text_size = 14)
#' p + mytheme + ggtitle("Example Plot")
design_mytheme <- function(theme = c("light", "bw", "classic", "classic2"),
                           plot_title_size = 2,
                           axis_title_size = 2,
                           axis_text_size = 12,
                           axis_angle = 60,
                           hjust = 1,
                           legend.position = "bottom",
                           legend.direction = "horizontal",
                           legend.size = 0.25,
                           legend.key.height = 0.5,
                           legend.key.width = 0.5,
                           legend.size.text = 10,
                           legend.box = "horizontal") {
  theme <- rlang::arg_match(theme)

  # Base theme
  base_theme <- switch(theme,
    light = ggplot2::theme_light(),
    bw = ggplot2::theme_bw(),
    classic = ggplot2::theme_classic(),
    classic2 = ggplot2::theme_classic() + ggplot2::theme(text = ggplot2::element_text(size = 14))
  )

  # Customize theme
  mytheme <- base_theme + ggplot2::theme(
    plot.title = ggplot2::element_text(size = ggplot2::rel(plot_title_size), hjust = 0.5),
    axis.title = ggplot2::element_text(size = ggplot2::rel(axis_title_size)),
    axis.text.x = ggplot2::element_text(
      face = "plain", size = axis_text_size,
      angle = axis_angle, hjust = hjust, color = "black"
    ),
    axis.text.y = ggplot2::element_text(
      face = "plain", size = axis_text_size, color = "black"
    ),
    axis.line = ggplot2::element_line(color = "grey", linewidth = 0.2)
  )

  # Legend settings
  mytheme <- mytheme + ggplot2::theme(
    legend.key.height = ggplot2::unit(legend.key.height, "cm"),
    legend.key.width = ggplot2::unit(legend.key.width, "cm"),
    legend.position = legend.position,
    legend.direction = legend.direction,
    legend.justification = c(0.5, 0.5),
    legend.box = legend.box,
    legend.box.just = "top",
    legend.text = ggplot2::element_text(
      colour = "black", size = legend.size.text, face = "plain"
    )
  )

  mytheme
}

#' Signature Heatmap with Optional Annotations
#'
#' @description
#' Generates a heatmap of selected features grouped by a categorical variable,
#' with optional conditional (annotation) bars. Supports palette customization,
#' scaling, size controls, and output saving.
#'
#' @param input Data frame containing ID, grouping variable, and feature columns.
#' @param id Character string. Column name for sample identifier. Default is `"ID"`.
#' @param features Character vector. Feature (column) names to include in the heatmap.
#' @param group Character string. Grouping variable column name.
#' @param condition Data frame or `NULL`. Optional annotation table with
#'   variable-condition mapping. Default is `NULL`.
#' @param id_condition Character string. Column name in `condition` for feature IDs.
#'   Default is `"vars"`.
#' @param col_condition Character string. Column name in `condition` for condition
#'   labels. Default is `"condition"`.
#' @param cols_condition Character vector. Colors for conditions.
#' @param scale Logical indicating whether to scale values by row. Default is `FALSE`.
#' @param palette Integer or character. Palette index/name for heatmap colors.
#'   Default is `2`.
#' @param cols_heatmap Character vector. Custom colors for heatmap gradient.
#' @param palette_group Character string. Palette name for group colors.
#'   Default is `"jama"`.
#' @param show_col Logical indicating whether to display the color vector.
#'   Default is `FALSE`.
#' @param show_palettes Logical indicating whether to print palette options.
#'   Default is `FALSE`.
#' @param cols_group Character vector. Custom colors for groups.
#' @param show_plot Logical indicating whether to print the heatmap. Default is `TRUE`.
#' @param width Numeric. Plot width in inches. Default is `8`.
#' @param height Numeric or `NULL`. Plot height in inches. Auto-calculated if `NULL`.
#' @param size_col Numeric. Font size for column labels. Default is `10`.
#' @param size_row Numeric. Font size for row labels. Default is `8`.
#' @param angle_col Numeric. Rotation angle for column labels in degrees.
#'   Default is `90`.
#' @param column_title Character string or `NULL`. Title for column annotation.
#' @param row_title Character string or `NULL`. Title for row annotation.
#' @param show_heatmap_col_name Logical indicating whether to show column names.
#'   Default is `FALSE`.
#' @param path Character string or `NULL`. Output directory for saving the heatmap.
#' @param index Integer or `NULL`. Index appended to filename. Default is `NULL`.
#'
#' @return A tidyHeatmap object. Saves PDF only when `path` is provided.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' test_input <- data.frame(
#'   ID = paste0("Sample", 1:20),
#'   subtype = sample(c("TypeA", "TypeB"), 20, replace = TRUE),
#'   Sig1 = rnorm(20),
#'   Sig2 = rnorm(20)
#' )
#' sig_heatmap(input = test_input, features = c("Sig1", "Sig2"), group = "subtype", scale = TRUE)
sig_heatmap <- function(input,
                        id = "ID",
                        features,
                        group,
                        condition = NULL,
                        id_condition = "vars",
                        col_condition = "condition",
                        cols_condition = NULL,
                        scale = FALSE,
                        palette = 2,
                        cols_heatmap = NULL,
                        palette_group = "jama",
                        show_col = FALSE,
                        show_palettes = FALSE,
                        cols_group = NULL,
                        show_plot = TRUE,
                        width = 8,
                        height = NULL,
                        size_col = 10,
                        size_row = 8,
                        angle_col = 90,
                        column_title = NULL,
                        row_title = NULL,
                        show_heatmap_col_name = FALSE,
                        path = NULL,
                        index = NULL) {
  if (is.null(input)) return(NULL)
  rlang::check_installed("tidyHeatmap")
  rlang::check_installed("circlize")

  input <- as.data.frame(input)

  # Input validation
  if (!id %in% colnames(input)) {
    cli::cli_abort("ID column {.val {id}} not found in input")
  }

  if (!group %in% colnames(input)) {
    cli::cli_abort("Group column {.val {group}} not found in input")
  }

  features <- unique(features)
  features <- features[features %in% colnames(input)]

  if (length(features) == 0) {
    cli::cli_abort("No valid features found in input")
  }

  cli::cli_alert_info("Creating heatmap with {length(features)} features")

  # Create output directory if needed
  out_dir <- NULL
  if (!is.null(path)) {
    if (!dir.exists(path)) dir.create(path, recursive = TRUE)
    out_dir <- normalizePath(path, winslash = "/", mustWork = FALSE)
  }

  # Standardize column names temporarily
  input_copy <- input
  colnames(input_copy)[colnames(input_copy) == id] <- "idd"
  colnames(input_copy)[colnames(input_copy) == group] <- "target_group"

  input_copy <- input_copy[, c("idd", "target_group", features), drop = FALSE]
  input_copy <- input_copy[!is.na(input_copy$target_group), , drop = FALSE]

  # Convert to long format
  pf_long_group <- tidyr::pivot_longer(
    input_copy,
    cols = dplyr::all_of(features),
    names_to = "variables",
    values_to = "value"
  )

  # Optional row annotation
  if (!is.null(condition)) {
    condition <- as.data.frame(condition)

    if (!all(c(id_condition, col_condition) %in% colnames(condition))) {
      cli::cli_abort(
        "Columns {.val {id_condition}} or {.val {col_condition}} not found in condition"
      )
    }

    condition <- condition[, c(id_condition, col_condition), drop = FALSE]
    colnames(condition) <- c("vars", "condition")

    pf_long_group <- merge(
      pf_long_group,
      condition,
      by.x = "variables",
      by.y = "vars",
      all.x = TRUE,
      all.y = FALSE
    )

    pf_long_group$condition[is.na(pf_long_group$condition)] <- "NE"
  }

  # Calculate plot height
  height_heatmap <- height %||% (length(features) * 0.1 + 3)

  # Build heatmap colors
  heatmap_col <- .build_heatmap_colors(
    cols_heatmap = cols_heatmap,
    palette = palette,
    show_col = show_col,
    show_palettes = show_palettes
  )

  # Get group colors
  color_box <- cols_group %||% palettes(
    category = "box",
    palette = palette_group,
    show_col = show_col,
    show_message = show_palettes
  )

  # Annotation colors for condition
  color_box1 <- NULL
  if (!is.null(condition)) {
    target_level1 <- unique(as.character(pf_long_group$condition))
    target_level1 <- target_level1[!is.na(target_level1)]
    n1 <- length(target_level1)
    color_box1 <- rep(cols_condition %||% color_box, length.out = n1)
  }

  # Group colors for target_group
  target_level2 <- unique(as.character(pf_long_group$target_group))
  target_level2 <- target_level2[!is.na(target_level2)]
  n2 <- length(target_level2)
  color_box2 <- rep(color_box, length.out = n2)

  # Scale or clip values
  if (isTRUE(scale)) {
    scale_mode <- "row"
  } else {
    pf_long_group$value <- pmin(pmax(pf_long_group$value, -3), 3)
    scale_mode <- "none"
  }

  pf_long_group <- as.data.frame(pf_long_group)

  # Build heatmap
  if (is.null(condition)) {
    pp <- pf_long_group %>%
      dplyr::group_by(.data$target_group) %>%
      tidyHeatmap::heatmap(
        .column = idd,
        .row = variables,
        .value = value,
        palette_grouping = list(color_box2),
        scale = scale_mode,
        column_title = column_title,
        row_title = row_title,
        palette_value = heatmap_col,
        show_column_names = show_heatmap_col_name,
        column_names_gp = grid::gpar(fontsize = size_col),
        row_names_gp = grid::gpar(fontsize = size_row),
        column_names_rot = angle_col
      )
  } else {
    pp <- pf_long_group %>%
      dplyr::group_by(.data$condition, .data$target_group) %>%
      tidyHeatmap::heatmap(
        .column = idd,
        .row = variables,
        .value = value,
        palette_grouping = list(color_box1, color_box2),
        scale = scale_mode,
        column_title = column_title,
        row_title = row_title,
        palette_value = heatmap_col,
        show_column_names = show_heatmap_col_name,
        column_names_gp = grid::gpar(fontsize = size_col),
        row_names_gp = grid::gpar(fontsize = size_row),
        column_names_rot = angle_col
      )
  }

  if (isTRUE(show_plot) && interactive()) {
    print(pp)
  }

  # Save to file if path provided
  if (!is.null(out_dir)) {
    idx <- index %||% 1
    outfile <- file.path(out_dir, paste0(idx, "-", group, "-tidyheatmap.pdf"))

    pp %>%
      tidyHeatmap::save_pdf(
        filename = outfile,
        width = width,
        height = height_heatmap
      )

    cli::cli_alert_success("Heatmap saved to: {.path {outfile}}")
  }

  invisible(pp)
}

#' Build Heatmap Colors
#' @keywords internal
#' @noRd
.build_heatmap_colors <- function(cols_heatmap, palette, show_col, show_palettes) {
  if (!is.null(cols_heatmap)) {
    n_colors <- length(cols_heatmap)

    if (n_colors >= 5) {
      return(circlize::colorRamp2(c(-3, -1.5, 0, 1.5, 3), cols_heatmap[1:5]))
    } else if (n_colors >= 3) {
      return(circlize::colorRamp2(c(-3, 0, 3), cols_heatmap[1:3]))
    } else if (n_colors == 2) {
      cli::cli_alert_info("Only 2 heatmap colors provided, using white as midpoint")
      return(circlize::colorRamp2(c(-3, 0, 3), c(cols_heatmap[1], "white", cols_heatmap[2])))
    } else {
      cli::cli_warn("Invalid cols_heatmap, using default colors")
      return(circlize::colorRamp2(c(-3, 0, 3), c("blue", "white", "red")))
    }
  }

  heatmap_col_raw <- palettes(
    category = "tidyheatmap",
    palette = palette,
    show_col = show_col,
    show_message = show_palettes
  )

  if (is.function(heatmap_col_raw)) {
    return(heatmap_col_raw)
  }

  # Convert to color vector
  color_vector <- .convert_to_colors(heatmap_col_raw)

  if (length(color_vector) >= 5) {
    circlize::colorRamp2(c(-3, -1.5, 0, 1.5, 3), color_vector[1:5])
  } else if (length(color_vector) >= 3) {
    circlize::colorRamp2(c(-3, 0, 3), color_vector[1:3])
  } else if (length(color_vector) == 2) {
    circlize::colorRamp2(c(-3, 0, 3), c(color_vector[1], "white", color_vector[2]))
  } else {
    circlize::colorRamp2(c(-3, 0, 3), c("blue", "white", "red"))
  }
}

#' Convert various color formats to color vector
#' @keywords internal
#' @noRd
.convert_to_colors <- function(col_raw) {
  if (is.matrix(col_raw) && ncol(col_raw) == 3) {
    max_val <- max(col_raw, na.rm = TRUE)
    grDevices::rgb(
      col_raw[, 1],
      col_raw[, 2],
      col_raw[, 3],
      maxColorValue = ifelse(max_val > 1, 255, 1)
    )
  } else if (is.character(col_raw)) {
    col_raw
  } else {
    c("blue", "white", "red")
  }
}

#' Feature Quality Control and Filtering
#'
#' @description
#' Filters features (variables) in a matrix or data frame by removing those with
#' missing values, non-numeric types, infinite values, or zero variance.
#' This is useful for preparing data for downstream statistical analyses.
#'
#' @param data A matrix or data frame containing features to filter.
#' @param feature Character vector of feature names to check. If `is_matrix = TRUE`,
#'   features are extracted from row names of the matrix.
#' @param is_matrix Logical indicating whether `data` is a gene expression matrix
#'   (features as rows, samples as columns). If `TRUE`, the matrix is transposed
#'   for processing. Default is `FALSE`.
#' @param print_result Logical indicating whether to print filtering statistics.
#'   Default is `FALSE`.
#'
#' @return Character vector of feature names that pass all quality checks.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' test_data <- data.frame(
#'   A = c(1, 2, 3),
#'   B = c(1, 1, 1), # zero variance
#'   C = c(1, NA, 3), # missing value
#'   D = c("a", "b", "c") # non-numeric
#' )
#' feas <- feature_manipulation(data = test_data, feature = colnames(test_data), print_result = TRUE)
#' print(feas)
feature_manipulation <- function(data,
                                 feature = NULL,
                                 is_matrix = FALSE,
                                 print_result = FALSE) {
  if (is.null(data)) return(NULL)
  # Handle matrix input (features as rows)
  if (is_matrix) {
    if (!is.matrix(data) && !is.data.frame(data)) {
      cli::cli_abort("{.arg data} must be a matrix or data frame when {.code is_matrix = TRUE}")
    }
    data <- as.data.frame(t(data))
    feature <- colnames(data)
  }

  if (is.null(feature)) {
    cli::cli_abort("{.arg feature} must be specified when {.code is_matrix = FALSE}")
  }

  data <- as.data.frame(data)

  # Check which features exist in data
  valid_features <- feature[feature %in% colnames(data)]
  missing_features <- setdiff(feature, colnames(data))

  if (length(missing_features) > 0 && print_result) {
    cli::cli_alert_warning(
      "{length(missing_features)} feature{?s} not found in data: {.val {missing_features}}"
    )
  }

  if (length(valid_features) == 0) {
    cli::cli_abort("No valid features found in data")
  }

  original_count <- length(valid_features)

  # Step 1: Remove features with missing values
  if (any(is.na(data[, valid_features, drop = FALSE]))) {
    na_counts <- colSums(is.na(data[, valid_features, drop = FALSE]))
    features_with_na <- names(na_counts)[na_counts > 0]

    if (print_result) {
      cli::cli_alert_info(
        "Removing {length(features_with_na)} feature{?s} with NA values"
      )
    }

    valid_features <- setdiff(valid_features, features_with_na)
  }

  # Step 2: Remove non-numeric features
  is_numeric <- vapply(
    data[, valid_features, drop = FALSE],
    is.numeric,
    logical(1)
  )

  if (any(!is_numeric)) {
    if (print_result) {
      cli::cli_alert_info(
        "Removing {sum(!is_numeric)} non-numeric feature{?s}"
      )
    }
    valid_features <- valid_features[is_numeric]
  }

  # Step 3: Remove features with infinite values
  if (length(valid_features) > 0) {
    col_ranges <- vapply(
      data[, valid_features, drop = FALSE],
      function(x) range(x, na.rm = TRUE),
      numeric(2)
    )

    has_inf <- !is.finite(col_ranges[1, ]) | !is.finite(col_ranges[2, ])

    if (any(has_inf)) {
      if (print_result) {
        cli::cli_alert_info(
          "Removing {sum(has_inf)} feature{?s} with infinite values"
        )
      }
      valid_features <- valid_features[!has_inf]
    }
  }

  # Step 4: Remove features with zero variance
  if (length(valid_features) > 0) {
    sds <- vapply(
      data[, valid_features, drop = FALSE],
      sd,
      numeric(1),
      na.rm = TRUE
    )

    zero_sd <- sds == 0 | is.na(sds)

    if (any(zero_sd)) {
      if (print_result) {
        cli::cli_alert_info(
          "Removing {sum(zero_sd)} feature{?s} with zero variance"
        )
      }
      valid_features <- valid_features[!zero_sd]
    }
  }

  # Print summary
  if (print_result) {
    cli::cli_alert_success(
      "Retained {length(valid_features)} of {original_count} feature{?s}"
    )
  }

  valid_features
}

#' Merge Expression Sets by Row Names
#'
#' @description
#' Merges two or three expression sets (matrices or data frames) by row names
#' (gene symbols), removing duplicates. The function ensures common genes
#' across all input expression sets are retained.
#'
#' @param eset1 First expression set (matrix or data frame with row names).
#' @param eset2 Second expression set (matrix or data frame with row names).
#' @param eset3 Optional third expression set. Default is `NULL`.
#'
#' @return Merged expression set (data frame) with duplicates removed.
#'   Row names correspond to gene symbols.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' # Create mock expression sets with common genes
#' set.seed(123)
#' common_genes <- c("TP53", "BRCA1", "EGFR", "MYC")
#' eset1 <- matrix(rnorm(12),
#'   nrow = 4,
#'   dimnames = list(common_genes, paste0("S", 1:3))
#' )
#' eset2 <- matrix(rnorm(16),
#'   nrow = 4,
#'   dimnames = list(common_genes, paste0("S", 4:7))
#' )
#' # Merge two expression sets
#' merged_eset <- merge_eset(eset1, eset2)
#' print(dim(merged_eset))
merge_eset <- function(eset1, eset2, eset3 = NULL) {
  # Input validation
  if (!is.matrix(eset1) && !is.data.frame(eset1)) {
    cli::cli_abort(c(
      "Invalid type for {.arg eset1}.",
      "i" = "Expected a matrix or data frame, got {.cls {class(eset1)}}."
    ))
  }
  if (!is.matrix(eset2) && !is.data.frame(eset2)) {
    cli::cli_abort(c(
      "Invalid type for {.arg eset2}.",
      "i" = "Expected a matrix or data frame, got {.cls {class(eset2)}}."
    ))
  }

  # Convert to data frames and ensure row names
  eset1 <- as.data.frame(eset1)
  eset2 <- as.data.frame(eset2)

  if (is.null(rownames(eset1)) || is.null(rownames(eset2))) {
    cli::cli_abort("Input expression sets must have row names (gene symbols).")
  }

  # First merge
  common_genes_12 <- intersect(rownames(eset1), rownames(eset2))
  cli::cli_alert_info(
    "Common genes between eset1 and eset2: {length(common_genes_12)}"
  )

  if (length(common_genes_12) == 0) {
    cli::cli_abort("No common genes found between eset1 and eset2.")
  }

  eset <- merge(eset1, eset2, by = "row.names", all = FALSE)
  eset <- remove_duplicate_genes(eset = eset, column_of_symbol = "Row.names")

  # Optional third merge
  if (!is.null(eset3)) {
    if (!is.matrix(eset3) && !is.data.frame(eset3)) {
      cli::cli_abort(c(
        "Invalid type for {.arg eset3}.",
        "i" = "Expected a matrix or data frame, got {.cls {class(eset3)}}."
      ))
    }
    eset3 <- as.data.frame(eset3)

    if (is.null(rownames(eset3))) {
      cli::cli_abort("eset3 must have row names (gene symbols).")
    }

    common_genes_3 <- intersect(rownames(eset), rownames(eset3))
    cli::cli_alert_info(
      "Common genes after merging with eset3: {length(common_genes_3)}"
    )

    if (length(common_genes_3) == 0) {
      cli::cli_abort("No common genes found with eset3.")
    }

    eset <- merge(eset, eset3, by = "row.names", all = FALSE)
    eset <- remove_duplicate_genes(eset = eset, column_of_symbol = "Row.names")
  }

  cli::cli_alert_success(
    "Final merged expression set: {nrow(eset)} genes x {ncol(eset)} samples"
  )

  eset
}

#' Extract Hazard Ratio and Confidence Intervals from Cox Model
#'
#' @description
#' Extracts hazard ratio (HR) and 95% confidence intervals from a fitted
#' Cox proportional hazards model.
#'
#' @param coxphData Fitted Cox model object from `survival::coxph()`.
#'
#' @return Data frame with p-values, HR, and confidence intervals.
#'
#' @export
#' @author Dorothee Nickles
#' @author Dongqiang Zeng
#'
#' @examples
#' library(survival)
#' set.seed(123)
#' df <- data.frame(
#'   TTE = rexp(200, rate = 0.1),
#'   Cens = rbinom(200, size = 1, prob = 0.7),
#'   group = sample(c("Treatment", "Control"), 200, replace = TRUE)
#' )
#' coxphData <- survival::coxph(survival::Surv(TTE, Cens) ~ group, data = df)
#' results <- getHRandCIfromCoxph(coxphData)
#' print(results)
getHRandCIfromCoxph <- function(coxphData) {
  if (!inherits(coxphData, "coxph")) {
    cli::cli_abort("{.arg coxphData} must be a coxph object")
  }

  tmp <- cbind(
    summary(coxphData)$coef[, c("Pr(>|z|)", "exp(coef)"), drop = FALSE],
    summary(coxphData)$conf[, c("lower .95", "upper .95"), drop = FALSE]
  )
  colnames(tmp) <- c("P", "HR", "CI_low_0.95", "CI_up_0.95")
  tmp[, 2:4] <- round(tmp[, 2:4], digits = 4)

  as.data.frame(tmp)
}

#' Annotate Gene Expression Matrix and Remove Duplicated Genes
#'
#' @description
#' Annotates an expression matrix with gene symbols using provided annotation
#' data,
#' filters out missing or invalid symbols, handles duplicate gene entries, and
#' removes uninformative rows. The function supports multiple aggregation
#' methods
#' for resolving duplicate gene symbols.
#'
#' @details
#' The function performs the following operations:
#' \enumerate{
#'   \item Filters probes with missing symbols or labeled as `"NA_NA"`
#'   \item Matches probes between expression set and annotation data
#'   \item Merges annotation with expression data
#'   \item Handles duplicate gene symbols using specified aggregation method
#'   \item Removes rows with all zeros, all NAs, or missing values in the first
#'     column
#' }
#'
#' @param eset Expression matrix or ExpressionSet object containing gene
#'   expression data.
#' @param annotation Data frame containing annotation information for probes.
#'   Built-in options include `anno_hug133plus2`, `anno_rnaseq`, and
#'   `anno_illumina`.
#' @param symbol Character string specifying the column name in `annotation`
#'   that represents gene symbols. Default is `"symbol"`.
#' @param probe Character string specifying the column name in `annotation`
#'   that represents probe identifiers. Default is `"probe_id"`.
#' @param method Character string specifying the aggregation method for
#'   duplicate gene symbols. Options are `"mean"`, `"sum"`, or `"sd"`.
#'   Default is `"mean"`.
#'
#' @return Annotated and cleaned gene expression matrix with gene symbols as
#'   row names.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' # Create a small example expression matrix
#' eset_mat <- matrix(runif(100), nrow = 10, ncol = 10)
#' rownames(eset_mat) <- paste0("Probe", 1:10)
#' colnames(eset_mat) <- paste0("Sample", 1:10)
#'
#' # Create a matching annotation data frame
#' anno_df <- data.frame(
#'   probe_id = paste0("Probe", 1:10),
#'   symbol = c("Gene1", "Gene1", "Gene2", "Gene3", "Gene4",
#'              "Gene5", "Gene6", "Gene7", "Gene8", "Gene9")
#' )
#'
#' # Annotate
#' result <- anno_eset(eset = eset_mat, annotation = anno_df)
#' head(result)
anno_eset <- function(eset,
                      annotation,
                      symbol = "symbol",
                      probe = "probe_id",
                      method = "mean") {
  # Input validation
  if (is.null(eset)) return(NULL)
  if (is.null(annotation)) return(eset)
  if (is.null(eset)) {
    cli::cli_abort("{.arg eset} cannot be NULL.")
  }
  if (!is.matrix(eset) && !is.data.frame(eset)) {
    cli::cli_abort(c(
      "Invalid type for {.arg eset}.",
      "i" = "Expected a matrix or data frame, got {.cls {class(eset)}}."
    ))
  }
  if (nrow(eset) == 0) {
    cli::cli_abort("{.arg eset} has no rows.")
  }
  if (is.null(annotation) || !is.data.frame(annotation)) {
    cli::cli_abort(c(
      "Invalid type for {.arg annotation}.",
      "i" = "Expected a data frame, got {.cls {class(annotation)}}."
    ))
  }
  if (!symbol %in% colnames(annotation)) {
    cli::cli_abort("Symbol column {.val {symbol}} not found in annotation.")
  }
  if (!probe %in% colnames(annotation)) {
    cli::cli_abort("Probe column {.val {probe}} not found in annotation.")
  }
  method <- rlang::arg_match(method, c("mean", "sum", "sd"))

  # Prepare annotation
  annotation <- as.data.frame(annotation)
  colnames(annotation)[colnames(annotation) == symbol] <- "symbol"
  colnames(annotation)[colnames(annotation) == probe] <- "probe_id"
  annotation <- annotation[, c("probe_id", "symbol")]

  # Filter out NA and invalid symbols
  valid_symbols <- !is.na(annotation$symbol) & annotation$symbol != "NA_NA"
  annotation <- annotation[valid_symbols, , drop = FALSE]

  cli::cli_alert_info("Row number of original eset: {nrow(eset)}")

  # Match probes
  matched_probes <- rownames(eset) %in% annotation$probe_id
  anno_count <- sum(matched_probes) / length(matched_probes)
  cli::cli_alert_success(paste0(
    format(100 * anno_count, digits = 2),
    "% of probes in expression set were annotated"
  ))

  if (sum(matched_probes) == 0) {
    cli::cli_abort(c(
      "No probes matched between eset and annotation.",
      "i" = paste(
        "Check that probe ID formats match between eset rownames and",
        "annotation."
      ),
      "*" = "Example eset rowname: {.val {rownames(eset)[1]}}",
      "*" = "Example annotation probe: {.val {annotation$probe_id[1]}}"
    ))
  }

  # Filter eset to matched probes
  eset <- eset[matched_probes, , drop = FALSE]
  annotation <- annotation[annotation$probe_id %in% rownames(eset), ,
    drop = FALSE
  ]

  # Merge
  eset <- as.data.frame(eset)
  eset <- tibble::rownames_to_column(eset, var = "id")
  eset <- merge(annotation, eset, by.x = "probe_id", by.y = "id", all = FALSE)
  eset$probe_id <- NULL

  # Handle duplicates
  n_dups <- nrow(eset) - length(unique(eset$symbol))
  if (n_dups > 0) {
    cli::cli_alert_info(
      "Found {n_dups} duplicate symbol{?s}, using {.val {method}} method"
    )

    agg_col <- setdiff(colnames(eset), "symbol")

    order_index <- switch(method,
      "mean" = rowMeans(eset[, agg_col, drop = FALSE], na.rm = TRUE),
      "sum"  = rowSums(eset[, agg_col, drop = FALSE], na.rm = TRUE),
      "sd"   = apply(eset[, agg_col, drop = FALSE], 1, stats::sd, na.rm = TRUE)
    )

    eset <- eset[order(order_index, decreasing = TRUE), , drop = FALSE]
    eset <- dplyr::distinct(eset, symbol, .keep_all = TRUE)
  }

  eset <- tibble::column_to_rownames(eset, var = "symbol")

  # Remove uninformative rows
  eset <- eset[rowSums(eset == 0) < ncol(eset), , drop = FALSE]
  eset <- eset[rowSums(is.na(eset)) < ncol(eset), , drop = FALSE]
  eset <- eset[!is.na(eset[, 1]), , drop = FALSE]

  cli::cli_alert_info(
    "Row number after filtering duplicated gene symbol: {nrow(eset)}"
  )

  as.matrix(eset)
}

#' Signature Score Calculation Methods
#'
#' @description
#' A named vector of supported methods for calculating signature scores.
#'
#' @format Named character vector:
#' \describe{
#'   \item{PCA}{Principal Component Analysis method ("pca")}
#'   \item{ssGSEA}{Single-sample Gene Set Enrichment Analysis ("ssgsea")}
#'   \item{z-score}{Z-score transformation method ("zscore")}
#'   \item{Integration}{Integration of multiple methods ("integration")}
#' }
#'
#' @export
#'
#' @examples
#' signature_score_calculation_methods
#' signature_score_calculation_methods["PCA"]
signature_score_calculation_methods <- c(
  "PCA"         = "pca",
  "ssGSEA"      = "ssgsea",
  "z-score"     = "zscore",
  "Integration" = "integration"
)

# Helper: Prepare pdata and match with eset
.prepare_pdata <- function(pdata, eset, column_of_sample) {
  if (is.null(pdata)) {
    pdata <- data.frame(
      Index = seq_len(ncol(eset)),
      ID = colnames(eset),
      stringsAsFactors = FALSE
    )
  } else {
    pdata <- as.data.frame(pdata)

    # Rename existing ID column to avoid conflict
    if ("ID" %in% colnames(pdata) && column_of_sample != "ID") {
      colnames(pdata)[colnames(pdata) == "ID"] <- "ID2"
      cli::cli_alert_warning(
        "Renamed existing 'ID' column to 'ID2' to avoid conflicts"
      )
    }

    if (column_of_sample %in% colnames(pdata)) {
      colnames(pdata)[colnames(pdata) == column_of_sample] <- "ID"
    }
  }

  # Match pdata with eset
  pdata <- pdata[pdata$ID %in% colnames(eset), , drop = FALSE]
  eset <- eset[, colnames(eset) %in% pdata$ID, drop = FALSE]
  eset <- eset[, match(pdata$ID, colnames(eset)), drop = FALSE]

  list(pdata = pdata, eset = eset)
}

# Helper: Filter signatures by gene count
.filter_signatures <- function(signature, eset, mini_gene_count) {
  gene_counts <- vapply(
    signature,
    function(x) sum(x %in% rownames(eset)),
    integer(1)
  )
  signature[gene_counts >= mini_gene_count]
}

# Helper: Add TME combined scores
.add_tme_scores <- function(pdata, suffix = "") {
  pca_suffix <- ifelse(nzchar(suffix), paste0("_", suffix), "")

  if (all(c(
    paste0("TMEscoreA_CIR", pca_suffix),
    paste0("TMEscoreB_CIR", pca_suffix)
  ) %in% colnames(pdata))) {
    pdata[[paste0("TMEscore_CIR", pca_suffix)]] <-
      pdata[[paste0("TMEscoreA_CIR", pca_suffix)]] -
      pdata[[paste0("TMEscoreB_CIR", pca_suffix)]]
  }

  if (all(c(
    paste0("TMEscoreA_plus", pca_suffix),
    paste0("TMEscoreB_plus", pca_suffix)
  ) %in% colnames(pdata))) {
    pdata[[paste0("TMEscore_plus", pca_suffix)]] <-
      pdata[[paste0("TMEscoreA_plus", pca_suffix)]] -
      pdata[[paste0("TMEscoreB_plus", pca_suffix)]]
  }

  pdata
}

#' Calculate Signature Score Using PCA Method
#'
#' @description
#' Computes signature scores using Principal Component Analysis.
#' The first principal component is used as the signature score.
#'
#' @param pdata Data frame with phenotype data. If `NULL`, created from
#'   `eset` column names.
#' @param eset Expression matrix (genes as rows, samples as columns).
#' @param signature List of gene signatures.
#' @param mini_gene_count Minimum genes required per signature. Default is 3.
#' @param column_of_sample Column in `pdata` with sample IDs. Default is `"ID"`.
#' @param adjust_eset Logical: remove problematic features. Default is `FALSE`.
#'
#' @return Tibble with signature scores.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
#' rownames(eset) <- paste0("Gene", 1:100)
#' colnames(eset) <- paste0("Sample", 1:10)
#' signature <- list(
#'   Signature1 = paste0("Gene", 1:10),
#'   Signature2 = paste0("Gene", 11:20)
#' )
#' \donttest{
#' result <- calculate_sig_score_pca(eset = eset, signature = signature)
#' }
calculate_sig_score_pca <- function(pdata = NULL,
                                    eset,
                                    signature,
                                    mini_gene_count = 3,
                                    column_of_sample = "ID",
                                    adjust_eset = FALSE) {
  cli::cli_alert_info("Calculating signature scores using PCA method")

  # Prepare data
  prepared <- .prepare_pdata(pdata, eset, column_of_sample)
  pdata <- prepared$pdata
  eset <- prepared$eset

  if (ncol(eset) == 0) {
    cli::cli_abort("No matching samples between pdata and eset")
  }

  # Preprocessing for small datasets
  if (ncol(eset) < 5000) {
    eset <- log2eset(eset)
    check_eset(eset)
  }

  if (adjust_eset) {
    feas <- feature_manipulation(data = eset, is_matrix = TRUE)
    eset <- eset[rownames(eset) %in% feas, , drop = FALSE]
  }

  # Filter signatures
  signature <- .filter_signatures(signature, eset, max(mini_gene_count, 2))

  if (length(signature) == 0) {
    cli::cli_warn("No signatures have enough genes. Returning pdata unchanged.")
    return(tibble::as_tibble(pdata))
  }

  cli::cli_alert_info("Calculating scores for {length(signature)} signature(s)")

  # Calculate scores
  sig_scores <- vapply(names(signature), function(sig) {
    genes <- signature[[sig]]
    genes <- genes[genes %in% rownames(eset)]
    if (length(genes) == 0) {
      return(rep(NA_real_, ncol(eset)))
    }
    sigScore(eset[genes, , drop = FALSE], methods = "PCA")
  }, numeric(ncol(eset)))

  # Add to pdata
  pdata <- cbind(pdata, as.data.frame(sig_scores))
  pdata <- .add_tme_scores(pdata)

  if ("Index" %in% colnames(pdata)) {
    pdata <- pdata[, colnames(pdata) != "Index", drop = FALSE]
  }

  tibble::as_tibble(pdata)
}

#' Calculate Signature Score Using Z-Score Method
#'
#' @description
#' Computes signature scores using z-score transformation.
#'
#' @inheritParams calculate_sig_score_pca
#'
#' @return Tibble with signature scores.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
#' rownames(eset) <- paste0("Gene", 1:100)
#' colnames(eset) <- paste0("Sample", 1:10)
#' signature <- list(
#'   Signature1 = paste0("Gene", 1:10),
#'   Signature2 = paste0("Gene", 11:20)
#' )
#' \donttest{
#' result <- calculate_sig_score_zscore(eset = eset, signature = signature)
#' }
calculate_sig_score_zscore <- function(pdata = NULL,
                                       eset,
                                       signature,
                                       mini_gene_count = 3,
                                       column_of_sample = "ID",
                                       adjust_eset = FALSE) {
  cli::cli_alert_info("Calculating signature scores using z-score method")

  # Prepare data
  prepared <- .prepare_pdata(pdata, eset, column_of_sample)
  pdata <- prepared$pdata
  eset <- prepared$eset

  if (ncol(eset) < 5000) {
    eset <- log2eset(eset)
    check_eset(eset)
  }

  if (adjust_eset) {
    feas <- feature_manipulation(data = eset, is_matrix = TRUE)
    eset <- eset[rownames(eset) %in% feas, , drop = FALSE]
  }

  # Filter signatures
  signature <- .filter_signatures(signature, eset, max(mini_gene_count, 2))

  if (length(signature) == 0) {
    cli::cli_warn("No signatures have enough genes. Returning pdata unchanged.")
    return(tibble::as_tibble(pdata))
  }

  cli::cli_alert_info("Calculating scores for {length(signature)} signature(s)")

  # Calculate scores
  sig_scores <- vapply(names(signature), function(sig) {
    genes <- signature[[sig]]
    genes <- genes[genes %in% rownames(eset)]
    if (length(genes) == 0) {
      return(rep(NA_real_, ncol(eset)))
    }
    sigScore(eset[genes, , drop = FALSE], methods = "zscore")
  }, numeric(ncol(eset)))

  pdata <- cbind(pdata, as.data.frame(sig_scores))
  pdata <- .add_tme_scores(pdata)

  if ("Index" %in% colnames(pdata)) {
    pdata <- pdata[, colnames(pdata) != "Index", drop = FALSE]
  }

  tibble::as_tibble(pdata)
}

#' Calculate Signature Score Using ssGSEA Method
#'
#' @description
#' Computes signature scores using single-sample Gene Set Enrichment Analysis.
#'
#' @inheritParams calculate_sig_score_pca
#' @param parallel.size Number of parallel workers. Default is 1.
#'
#' @return Tibble with signature scores.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
#' rownames(eset) <- paste0("Gene", 1:100)
#' colnames(eset) <- paste0("Sample", 1:10)
#' signature <- list(
#'   Signature1 = paste0("Gene", 1:15),
#'   Signature2 = paste0("Gene", 16:30)
#' )
#' \donttest{
#' result <- calculate_sig_score_ssgsea(eset = eset, signature = signature)
#' }
calculate_sig_score_ssgsea <- function(pdata = NULL,
                                       eset,
                                       signature,
                                       mini_gene_count = 5,
                                       column_of_sample = "ID",
                                       adjust_eset = FALSE,
                                       parallel.size = 1L) {
  cli::cli_alert_info("Calculating signature scores using ssGSEA method")

  # ssGSEA needs more genes for robustness
  min_genes <- max(mini_gene_count, 5)

  # Filter signatures early
  signature <- .filter_signatures(signature, eset, min_genes)

  if (length(signature) == 0) {
    cli::cli_warn(paste0(
      "No signatures have enough genes (min: ",
      min_genes, "). Returning pdata unchanged."
    ))
    return(tibble::as_tibble(pdata))
  }

  # Prepare data
  prepared <- .prepare_pdata(pdata, eset, column_of_sample)
  pdata <- prepared$pdata
  eset <- prepared$eset

  if (ncol(eset) < 5000) {
    eset <- log2eset(eset)
    check_eset(eset)
  }

  if (adjust_eset) {
    feas <- feature_manipulation(data = eset, is_matrix = TRUE)
    eset <- eset[rownames(eset) %in% feas, , drop = FALSE]
  }

  cli::cli_alert_info("Calculating scores for {length(signature)} signature(s)")

  # Run ssGSEA with appropriate API
  rlang::check_installed("GSVA")
  gsva_info <- gsva_use_new_api()

  if (gsva_info$use_new_api) {
    rlang::check_installed("BiocParallel")

    bp <- if (parallel.size > 1) {
      if (.Platform$OS.type == "windows") {
        BiocParallel::SnowParam(workers = parallel.size, progressbar = TRUE)
      } else {
        BiocParallel::MulticoreParam(
          workers = parallel.size,
          progressbar = TRUE
        )
      }
    } else {
      BiocParallel::SerialParam(progressbar = TRUE)
    }

    param <- GSVA::ssgseaParam(
      exprData = as.matrix(eset),
      geneSets = signature,
      minSize = min_genes,
      maxSize = Inf,
      normalize = TRUE
    )
    res <- GSVA::gsva(param, verbose = TRUE, BPPARAM = bp)
  } else {
    res <- GSVA::gsva(
      as.matrix(eset),
      signature,
      method = "ssgsea",
      kcdf = "Gaussian",
      min.sz = min_genes,
      ssgsea.norm = TRUE,
      parallel.sz = parallel.size
    )
  }

  # Process results
  res <- as.data.frame(t(res))
  res <- .add_tme_scores(res)
  res <- tibble::rownames_to_column(res, var = "ID")

  pdata <- merge(pdata, res, by = "ID", all = FALSE)

  if ("Index" %in% colnames(pdata)) {
    pdata <- pdata[, colnames(pdata) != "Index", drop = FALSE]
  }

  tibble::as_tibble(pdata)
}

#' Calculate Signature Score Using Integration Method
#'
#' @description
#' Computes signature scores using PCA, z-score, and ssGSEA methods combined.
#'
#' @inheritParams calculate_sig_score_ssgsea
#'
#' @return Tibble with signature scores from all three methods.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
#' rownames(eset) <- paste0("Gene", 1:100)
#' colnames(eset) <- paste0("Sample", 1:10)
#' signature <- list(
#'   Signature1 = paste0("Gene", 1:15),
#'   Signature2 = paste0("Gene", 16:30)
#' )
#' \donttest{
#' result <- calculate_sig_score_integration(eset = eset, signature = signature)
#' }
calculate_sig_score_integration <- function(pdata = NULL,
                                            eset,
                                            signature,
                                            mini_gene_count = 2,
                                            column_of_sample = "ID",
                                            adjust_eset = FALSE,
                                            parallel.size = 1L) {
  cli::cli_alert_info("Calculating signature scores using PCA, z-score, and ssGSEA methods")

  # Filter signatures
  signature <- .filter_signatures(signature, eset, mini_gene_count)

  if (length(signature) == 0) {
    cli::cli_abort("No signatures have enough genes (min: {mini_gene_count})")
  }

  # Prepare data
  prepared <- .prepare_pdata(pdata, eset, column_of_sample)
  pdata <- prepared$pdata
  eset <- prepared$eset

  if (ncol(eset) < 5000) {
    eset <- log2eset(eset)
    check_eset(eset)
  }

  if (adjust_eset) {
    feas <- feature_manipulation(data = eset, is_matrix = TRUE)
    eset <- eset[rownames(eset) %in% feas, , drop = FALSE]
  }

  goi <- names(signature)

  # Step 1: PCA
  cli::cli_alert_info("Step 1/3: PCA method")
  for (sig in goi) {
    genes <- signature[[sig]]
    genes <- genes[genes %in% rownames(eset)]
    pdata[[paste0(sig, "_PCA")]] <- sigScore(eset[genes, , drop = FALSE], methods = "PCA")
  }
  pdata <- .add_tme_scores(pdata, "PCA")

  # Step 2: z-score
  cli::cli_alert_info("Step 2/3: z-score method")
  for (sig in goi) {
    genes <- signature[[sig]]
    genes <- genes[genes %in% rownames(eset)]
    pdata[[paste0(sig, "_zscore")]] <- sigScore(eset[genes, , drop = FALSE],
      methods = "zscore"
    )
  }
  pdata <- .add_tme_scores(pdata, "zscore")

  # Step 3: ssGSEA
  cli::cli_alert_info("Step 3/3: ssGSEA method")
  ssgsea_min <- max(mini_gene_count, 5)
  sig_ssgsea <- signature[vapply(
    signature,
    function(x) sum(x %in% rownames(eset)),
    integer(1)
  ) >= ssgsea_min]

  if (length(sig_ssgsea) > 0) {
    rlang::check_installed("GSVA")
    gsva_info <- gsva_use_new_api()

    if (gsva_info$use_new_api) {
      rlang::check_installed("BiocParallel")
      param <- GSVA::ssgseaParam(
        exprData = as.matrix(eset),
        geneSets = sig_ssgsea,
        minSize = ssgsea_min,
        maxSize = Inf,
        normalize = TRUE
      )
      res <- GSVA::gsva(param,
        verbose = TRUE,
        BPPARAM = BiocParallel::SerialParam(progressbar = TRUE)
      )
    } else {
      res <- GSVA::gsva(
        as.matrix(eset),
        sig_ssgsea,
        method = "ssgsea",
        kcdf = "Gaussian",
        min.sz = ssgsea_min,
        ssgsea.norm = TRUE
      )
    }

    res <- as.data.frame(t(res))
    res <- .add_tme_scores(res)
    colnames(res) <- paste0(colnames(res), "_ssGSEA")
    res <- tibble::rownames_to_column(res, var = "ID")
    pdata <- merge(pdata, res, by = "ID", all = FALSE)
  }

  tibble::as_tibble(pdata)
}

#' Calculate Signature Score
#'
#' @description
#' Main interface for calculating signature scores from gene expression data.
#' Supports PCA, z-score, ssGSEA, and integration methods.
#'
#' @param pdata Phenotype data. If `NULL`, created from `eset` column names.
#' @param eset Expression matrix (CPM, TPM, RPKM, FPKM, etc.).
#' @param signature List of gene signatures. Can also be a character string
#'   naming a built-in signature collection (e.g., `"signature_collection"`,
#'   `"signature_tme"`, `"go_bp"`, `"kegg"`, `"hallmark"`).
#' @param method Scoring method: `"pca"`, `"ssgsea"`, `"zscore"`,
#'   or `"integration"`. Default is `"pca"`.
#' @param mini_gene_count Minimum genes required per signature. Default is 3
#'   (or 5 for ssGSEA).
#' @param column_of_sample Column with sample IDs in `pdata`. Default is `"ID"`.
#' @param print_gene_proportion Logical: print gene coverage. Default is
#'   `FALSE`.
#' @param print_filtered_signatures Logical: print filtered signatures.
#'   Default is `FALSE`.
#' @param adjust_eset Logical: clean problematic features. Default is `FALSE`.
#' @param parallel.size Parallel workers for ssGSEA. Default is 1.
#' @param ... Additional arguments passed to specific methods.
#'
#' @return Tibble with phenotype data and signature scores.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @references
#' \enumerate{
#'   \item Hänzelmann S, Castelo R, Guinney J. GSVA: gene set variation
#'     analysis. BMC Bioinformatics. 2013;14:7.
#'   \item Mariathasan S, et al. TGF\eqn{\beta} attenuates tumour response to PD-L1 blockade.
#'     Nature. 2018;554:544-548.
#' }
#'
#' @examples
#' set.seed(123)
#' eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
#' rownames(eset) <- paste0("Gene", 1:100)
#' colnames(eset) <- paste0("Sample", 1:10)
#' signature <- list(
#'   Signature1 = paste0("Gene", 1:10),
#'   Signature2 = paste0("Gene", 11:20)
#' )
#' \donttest{
#' result <- calculate_sig_score(eset = eset, signature = signature, method = "pca")
#' }
calculate_sig_score <- function(pdata = NULL,
                                eset,
                                signature = NULL,
                                method = c(
                                  "pca", "ssgsea", "zscore",
                                  "integration"
                                ),
                                mini_gene_count = 3,
                                column_of_sample = "ID",
                                print_gene_proportion = FALSE,
                                print_filtered_signatures = FALSE,
                                adjust_eset = FALSE,
                                parallel.size = 1L,
                                ...) {
  # Validate signature
  if (is.null(signature)) {
    cli::cli_abort(c(
      "Please provide a signature list.",
      "i" = "Available: signature_tme, signature_collection, go_bp, \
              kegg, hallmark"
    ))
  }

  # Load signature if character
  if (is.character(signature) && length(signature) == 1) {
    builtin_sigs <- c(
      "signature_collection", "signature_tme", "go_bp",
      "kegg", "hallmark"
    )
    if (signature %in% builtin_sigs) {
      signature <- load_data(signature)
    }
  }

  # Clean signature list
  if (is.list(signature)) {
    signature <- lapply(signature, function(x) {
      x <- stats::na.omit(as.character(x))
      unique(x[nzchar(x)])
    })
  }

  # Print gene proportion if requested
  if (print_gene_proportion) {
    coverage <- lapply(signature, function(x) {
      prop <- mean(x %in% rownames(eset))
      cli::cli_alert_info("{length(x)} genes, {prop*100:.1f}% in expression data")
    })
  }

  # Print filtered signatures if requested
  if (print_filtered_signatures) {
    gene_counts <- vapply(signature, function(x) sum(x %in% rownames(eset)), integer(1))
    filtered <- signature[gene_counts <= mini_gene_count]

    cli::cli_alert_info("{length(filtered)} signature(s) filtered (<{mini_gene_count} genes)")
    if (length(filtered) > 0 && length(filtered) <= 10) {
      cli::cli_text("Filtered: {.val {names(filtered)}}")
    } else if (length(filtered) > 10) {
      cli::cli_text("First 10 filtered: {.val {names(filtered)[1:10]}}")
    }
  }

  # Validate and dispatch
  method <- rlang::arg_match(method)

  switch(method,
    pca = calculate_sig_score_pca(
      pdata, eset,
      signature = signature,
      mini_gene_count = mini_gene_count,
      column_of_sample = column_of_sample,
      adjust_eset = adjust_eset, ...
    ),
    ssgsea = calculate_sig_score_ssgsea(
      pdata, eset,
      signature = signature,
      mini_gene_count = mini_gene_count,
      column_of_sample = column_of_sample,
      adjust_eset = adjust_eset,
      parallel.size = parallel.size, ...
    ),
    zscore = calculate_sig_score_zscore(
      pdata, eset,
      signature = signature,
      mini_gene_count = mini_gene_count,
      column_of_sample = column_of_sample,
      adjust_eset = adjust_eset, ...
    ),
    integration = calculate_sig_score_integration(
      pdata, eset,
      signature = signature,
      mini_gene_count = mini_gene_count,
      column_of_sample = column_of_sample,
      adjust_eset = adjust_eset,
      parallel.size = parallel.size, ...
    )
  )
}

#' The xCell Analysis Pipeline
#'
#' @description
#' Returns the xCell cell types enrichment scores for tumor microenvironment
#' deconvolution. Uses ssGSEA-based enrichment analysis with spillover
#' compensation to estimate cell type proportions from gene expression data.
#'
#' @param expr Gene expression data matrix with row names as gene symbols and
#'   columns as samples.
#' @param signatures GMT object of signatures. If `NULL`, uses xCell defaults.
#' @param genes Character vector of genes to use in the analysis. If `NULL`,
#'   uses xCell defaults.
#' @param spill Spillover object for adjusting scores. If `NULL`, uses xCell defaults.
#' @param rnaseq Logical indicating whether to use RNA-seq (TRUE) or array
#'   (FALSE) spillover parameters. Default is `TRUE`.
#' @param file.name Character string for saving scores. Default is `NULL`.
#' @param scale Logical indicating whether to use scaling. Default is `TRUE`.
#' @param alpha Numeric value to override spillover alpha parameter. Default is `0.5`.
#' @param save.raw Logical indicating whether to save raw scores. Default is `FALSE`.
#' @param cell.types.use Character vector of cell types to use. If `NULL`, uses
#'   all available cell types. Default is `NULL`.
#'
#' @return A matrix of adjusted xCell scores.
#'
#' @keywords internal
xCellAnalysis <- function(expr, signatures = NULL, genes = NULL,
                          spill = NULL, rnaseq = TRUE, file.name = NULL, scale = TRUE,
                          alpha = 0.5, save.raw = FALSE,
                          cell.types.use = NULL) {
  # Input validation
  if (is.null(expr)) {
    cli::cli_abort("{.arg expr} cannot be NULL")
  }
  if (!is.matrix(expr) && !is.data.frame(expr)) {
    cli::cli_abort("{.arg expr} must be a matrix or data frame")
  }
  if (nrow(expr) == 0 || ncol(expr) == 0) {
    cli::cli_abort("{.arg expr} is empty")
  }
  if (is.null(rownames(expr))) {
    cli::cli_abort("{.arg expr} must have row names (gene symbols)")
  }
  if (!is.logical(rnaseq) || length(rnaseq) != 1) {
    cli::cli_abort("{.arg rnaseq} must be a single logical value")
  }
  if (!is.numeric(alpha) || length(alpha) != 1 || alpha < 0 || alpha > 1) {
    cli::cli_abort("{.arg alpha} must be a numeric value between 0 and 1")
  }

  # Load default data if not provided
  xcell_ref <- load_data("xCell.data")
  if (is.null(xcell_ref) && (is.null(signatures) || is.null(genes) || is.null(spill))) {
    return(NULL)
  }
  signatures <- signatures %||% xcell_ref$signatures
  genes <- genes %||% xcell_ref$genes
  spill <- spill %||% if (rnaseq) {
    xcell_ref$spill
  } else {
    xcell_ref$spill.array
  }

  # Validate cell types if specified
  if (!is.null(cell.types.use)) {
    if (!is.character(cell.types.use)) {
      cli::cli_abort("{.arg cell.types.use} must be a character vector")
    }
    available_types <- rownames(spill$K)
    invalid_types <- setdiff(cell.types.use, available_types)
    if (length(invalid_types) > 0) {
      cli::cli_abort("Invalid cell types: {.val {head(invalid_types, 5)}}.
                     Available: {.val {head(available_types, 10)}}")
    }
  }

  fn <- if (is.null(file.name) || !save.raw) {
    NULL
  } else {
    paste0(file.name, "_RAW.txt")
  }

  scores <- rawEnrichmentAnalysis(expr, signatures, genes, fn)
  scores.transformed <- transformScores(scores, spill$fv, scale)

  if (is.null(cell.types.use)) {
    scores.adjusted <- spillOver(scores.transformed, spill$K, alpha, file.name)
    scores.adjusted <- microenvironmentScores(scores.adjusted)
  } else {
    scores.adjusted <- spillOver(
      scores.transformed[cell.types.use, ],
      spill$K, alpha, file.name
    )
  }

  scores.adjusted
}

#' Calculate Raw xCell Enrichment Scores
#'
#' @description
#' Returns the raw xCell cell types enrichment scores using ssGSEA.
#'
#' @param expr Gene expression data matrix with row names as gene symbols.
#' @param signatures GMT object of signatures.
#' @param genes Character vector of genes to use.
#' @param file.name Character string for saving scores. Default is `NULL`.
#'
#' @return Matrix of raw xCell scores.
#'
#' @keywords internal
rawEnrichmentAnalysis <- function(expr, signatures, genes, file.name = NULL) {
  # Reduce expression dataset to required genes
  shared.genes <- intersect(rownames(expr), genes)
  cli::cli_alert_info("Number of genes: {length(shared.genes)}")

  if (length(shared.genes) < 5000) {
    cli::cli_abort("Not enough genes. Need at least 5000, found {length(shared.genes)}")
  }

  expr <- expr[shared.genes, ]
  expr <- apply(expr, 2, rank)

  # Check the formal argument of GSVA::gsva
  FA <- formals(GSVA::gsva)

  # Run ssGSEA analysis for the ranked gene expression dataset
  if (is.null(FA[["method"]])) {
    params <- GSVA::gsvaParam(
      exprData = as.matrix(expr),
      geneSets = signatures,
      minSize = 1,
      maxSize = Inf,
      tau = 1,
      maxDiff = TRUE,
      absRanking = FALSE
    )

    rlang::check_installed("BiocParallel")
    scores <- GSVA::gsva(
      params,
      verbose = TRUE,
      BPPARAM = BiocParallel::SerialParam(progressbar = TRUE)
    )
  } else {
    scores <- GSVA::gsva(
      expr,
      signatures,
      method = "ssgsea",
      ssgsea.norm = FALSE
    )
  }

  scores <- scores - apply(scores, 1, min)

  # Combine signatures for same cell types
  cell_types <- unlist(strsplit(rownames(scores), "%"))
  cell_types <- cell_types[seq(1, length(cell_types), 3)]
  agg <- stats::aggregate(scores ~ cell_types, FUN = mean)
  rownames(agg) <- agg[, 1]
  scores <- agg[, -1]

  if (!is.null(file.name)) {
    utils::write.table(scores,
      file = file.name, sep = "\t",
      col.names = NA, quote = FALSE
    )
  }

  scores
}

#' Transform Raw Scores to Fractions
#'
#' @description
#' Transforms raw xCell scores to estimated cell fractions.
#'
#' @param scores Raw scores from rawEnrichmentAnalysis.
#' @param fit.vals Calibration values from spill object.
#' @param scale Logical indicating whether to use scaling. Default is `TRUE`.
#' @param fn Character string for saving scores. Default is `NULL`.
#'
#' @return Transformed xCell scores matrix.
#'
#' @keywords internal
transformScores <- function(scores, fit.vals, scale = TRUE, fn = NULL) {
  rows <- rownames(scores)[rownames(scores) %in% rownames(fit.vals)]
  tscores <- scores[rows, ]
  minX <- apply(tscores, 1, min)
  A <- rownames(tscores)
  tscores <- (as.matrix(tscores) - minX) / 5000
  tscores[tscores < 0] <- 0

  if (!scale) {
    fit.vals[A, 3] <- 1
  }

  tscores <- (tscores^fit.vals[A, 2]) / (fit.vals[A, 3] * 2)

  if (!is.null(fn)) {
    utils::write.table(format(tscores, digits = 4),
      file = fn, sep = "\t",
      col.names = NA, quote = FALSE
    )
  }

  tscores
}

#' Adjust Scores Using Spillover Compensation
#'
#' @description
#' Adjusts xCell scores using spillover compensation matrix.
#'
#' @param transformedScores Transformed scores from transformScores.
#' @param K Spillover matrix.
#' @param alpha Spillover alpha parameter. Default is `0.5`.
#' @param file.name Character string for saving scores. Default is `NULL`.
#'
#' @return Adjusted xCell scores matrix.
#'
#' @keywords internal
spillOver <- function(transformedScores, K, alpha = 0.5, file.name = NULL) {
  rlang::check_installed("pracma")

  K <- K * alpha
  diag(K) <- 1
  rows <- rownames(transformedScores)[rownames(transformedScores) %in%
    rownames(K)]
  scores <- apply(transformedScores[rows, ], 2, function(x) {
    pracma::lsqlincon(K[
      rows,
      rows
    ], x, lb = 0)
  })
  scores[scores < 0] <- 0
  rownames(scores) <- rows
  if (!is.null(file.name)) {
    scores <- round(scores * 10000)
    scores <- scores / 10000
    write.table(scores,
      file = file.name, sep = "\t", col.names = NA,
      quote = FALSE
    )
  }
  return(scores)

  # K <- K * alpha
  # diag(K) <- 1
  # rows <- rownames(transformedScores)[rownames(transformedScores) %in% rownames(K)]

  # rlang::check_installed("limSolve")

  # scores <- apply(transformedScores[rows, , drop = FALSE], 2, function(x) {
  #   G <- diag(nrow(K[rows, rows]))
  #   H <- rep(0, nrow(G))
  #   res <- limSolve::lsei(
  #     A = K[rows, rows],
  #     B = x,
  #     G = G,
  #     H = H,
  #     verbose = FALSE
  #   )
  #   pmax(res$X, 0)
  # })

  # scores[scores < 0] <- 0
  # rownames(scores) <- rows

  # if (!is.null(file.name)) {
  #   scores <- round(scores * 10000) / 10000
  #   utils::write.table(scores,
  #     file = file.name, sep = "\t",
  #     col.names = NA, quote = FALSE
  #   )
  # }

  # scores
}

#' Calculate Microenvironment Scores
#'
#' @description
#' Calculates combined immune and stroma scores from adjusted xCell scores.
#'
#' @param adjustedScores Adjusted xCell scores matrix.
#'
#' @return Matrix with additional microenvironment score rows.
#'
#' @keywords internal
microenvironmentScores <- function(adjustedScores) {
  immune_cells <- c(
    "B-cells", "CD4+ T-cells", "CD8+ T-cells", "DC",
    "Eosinophils", "Macrophages", "Monocytes", "Mast cells",
    "Neutrophils", "NK cells"
  )
  stroma_cells <- c("Adipocytes", "Endothelial cells", "Fibroblasts")

  ImmuneScore <- apply(adjustedScores[immune_cells, ], 2, sum) / 1.5
  StromaScore <- apply(adjustedScores[stroma_cells, ], 2, sum) / 2
  MicroenvironmentScore <- ImmuneScore + StromaScore

  rbind(adjustedScores,
    ImmuneScore = ImmuneScore,
    StromaScore = StromaScore, MicroenvironmentScore = MicroenvironmentScore
  )
}

#' Calculate Significance P-values Using Beta Distribution
#'
#' @description
#' Calculates FDR-adjusted p-values for the null hypothesis that a cell type
#' is not present in the mixture.
#'
#' @param scores xCell scores matrix.
#' @param beta_params Pre-calculated beta distribution parameters.
#' @param rnaseq Logical for RNA-seq vs array parameters. Default is `TRUE`.
#' @param file.name Character string for saving p-values. Default is `NULL`.
#'
#' @return Matrix of p-values.
#'
#' @keywords internal
xCellSignifcanceBetaDist <- function(scores, beta_params = NULL, rnaseq = TRUE,
                                     file.name = NULL) {
  xcell_ref <- load_data("xCell.data")
  beta_params <- beta_params %||% if (rnaseq) {
    xcell_ref$spill$beta_params
  } else {
    xcell_ref$spill.array$beta_params
  }

  scores <- scores[rownames(scores) %in%
    colnames(xcell_ref$spill$beta_params[[1]]), ]
  pvals <- matrix(0, nrow(scores), ncol(scores))
  rownames(pvals) <- rownames(scores)
  eps <- 1e-3

  for (i in seq_len(nrow(scores))) {
    ct <- rownames(scores)[i]
    beta_dist <- c()

    for (bp in beta_params) {
      if (sum(bp[, i] == 0)) {
        bd <- matrix(eps, 1, 100000)
      } else {
        bd <- stats::rbeta(100000, bp[1, ct], bp[2, ct])
        bd <- ((1 + eps) * (bp[3, ct])) * bd
      }
      beta_dist <- c(beta_dist, bd)
    }

    pvals[i, ] <- 1 - mapply(scores[i, ], FUN = function(x) mean(x > beta_dist))
  }

  if (!is.null(file.name)) {
    utils::write.table(pvals,
      file = file.name, quote = FALSE,
      row.names = TRUE, sep = "\t", col.names = NA
    )
  }

  pvals
}

#' Calculate Significance Using Random Matrix
#'
#' @description
#' Calculates FDR-adjusted p-values using a random shuffled matrix.
#'
#' @param scores xCell scores matrix.
#' @param expr Input expression matrix.
#' @param spill Spillover object.
#' @param alpha Spillover alpha parameter. Default is `0.5`.
#' @param nperm Number of permutations. Default is `250`.
#' @param file.name Character string for saving p-values. Default is `NULL`.
#'
#' @return List containing p-values, shuffled xCell scores, shuffled expression,
#'   and beta distributions.
#'
#' @keywords internal
xCellSignifcanceRandomMatrix <- function(scores, expr, spill, alpha = 0.5,
                                         nperm = 250, file.name = NULL) {
  shuff_expr <- mapply(seq_len(nperm),
    FUN = function(x) sample(nrow(expr), nrow(expr))
  )
  rownames(shuff_expr) <- sample(rownames(expr))
  shuff_xcell <- xCellAnalysis(shuff_expr, spill = spill, alpha = alpha)
  shuff_xcell <- shuff_xcell[rownames(scores), ]

  pvals <- matrix(0, nrow(scores), ncol(scores))
  beta_dist <- matrix(0, nrow(scores), 100000)
  eps <- 1e-3

  for (i in seq_len(nrow(scores))) {
    x <- shuff_xcell[i, ]
    if (stats::sd(x) < eps) {
      beta_dist[i, ] <- rep(eps, 100000)
    } else {
      x <- x + eps
      rlang::check_installed("MASS")
      beta_params <- MASS::fitdistr(x / ((1 + 2 * eps) * (max(x))) + eps,
        "beta", list(shape1 = 1, shape2 = 1),
        lower = eps
      )
      beta_dist[i, ] <- stats::rbeta(
        100000, beta_params$estimate[1],
        beta_params$estimate[2]
      )
      beta_dist[i, ] <- ((1 + 2 * eps) * (max(x))) * beta_dist[i, ]
    }

    pvals[i, ] <- 1 - unlist(lapply(scores[i, ],
      FUN = function(x) mean(x > beta_dist[i, ])
    ))
  }

  rownames(pvals) <- rownames(scores)
  colnames(pvals) <- colnames(scores)
  rownames(shuff_xcell) <- rownames(scores)
  rownames(beta_dist) <- rownames(scores)

  if (!is.null(file.name)) {
    utils::write.table(pvals,
      file = file.name, quote = FALSE,
      row.names = TRUE, sep = "\t", col.names = NA
    )
  }

  list(
    pvals = pvals, shuff_xcell = shuff_xcell,
    shuff_expr = shuff_expr, beta_dist = beta_dist
  )
}

#' Subgroup Survival Analysis Using Cox Proportional Hazards Models
#'
#' @description
#' Extracts hazard ratios (HR) and 95% confidence intervals from Cox
#' proportional hazards models across specified subgroups.
#'
#' @param pdata Data frame containing variables, follow-up time, and outcome.
#' @param variables Character vector. Subgrouping variables (each processed
#'   independently).
#' @param time_name Character. Column name of follow-up time. Default is `"time"`.
#' @param status_name Character. Column name of event status (1/0).
#'   Default is `"status"`.
#' @param object Character. Variable of interest used in Cox model. If it has
#'   levels "High" and "Low", recode "High" to 0 and "Low" to 1 before calling.
#'
#' @return Data frame summarizing subgroup Cox results (HR, CI, p-value).
#'
#' @export
#' @author Dongqiang Zeng
#' @import survival
#'
#' @examples
#' set.seed(123)
#' test_pdata <- data.frame(
#'   time = runif(100, 1, 100),
#'   status = sample(c(0, 1), 100, replace = TRUE),
#'   subgroup = sample(c("A", "B"), 100, replace = TRUE),
#'   score = rnorm(100)
#' )
#' res <- subgroup_survival(
#'   pdata = test_pdata,
#'   time_name = "time",
#'   status_name = "status",
#'   variables = "subgroup",
#'   object = "score"
#' )
#' print(res)
subgroup_survival <- function(pdata,
                              time_name = "time",
                              status_name = "status",
                              variables,
                              object) {
  if (is.null(pdata)) return(NULL)
  if (!is.data.frame(pdata)) {
    cli::cli_abort("{.arg pdata} must be a data frame.")
  }

  required_cols <- c(time_name, status_name, object, variables)
  missing_cols <- setdiff(required_cols, colnames(pdata))
  if (length(missing_cols) > 0) {
    cli::cli_abort("Missing required column(s): {.val {missing_cols}}")
  }

  pdata <- as.data.frame(pdata)

  result <- data.frame(
    P = 1, HR = 1, CI_low_0.95 = 1, CI_up_0.95 = 1,
    row.names = "default"
  )

  for (sig in variables) {
    tmp <- pdata[!is.na(pdata[[sig]]), ]

    if (nrow(tmp) == 0) {
      cli::cli_warn("No valid data for variable {.val {sig}}, skipping.")
      next
    }

    tmp[[sig]] <- as.factor(as.character(tmp[[sig]]))

    result2 <- data.frame(
      P = 1, HR = 1, CI_low_0.95 = 1, CI_up_0.95 = 1,
      row.names = "default"
    )

    nl <- nlevels(tmp[[sig]])

    for (i in seq_len(nl)) {
      level_name <- levels(tmp[[sig]])[i]
      tmp1 <- tmp[as.character(tmp[[sig]]) == level_name, ]

      if (nrow(tmp1) == 0) {
        next
      }

      fit <- survival::coxph(
        survival::Surv(tmp1[[time_name]], tmp1[[status_name]]) ~ tmp1[[object]],
        data = tmp1
      )
      result1 <- getHRandCIfromCoxph(fit)
      rownames(result1) <- paste(sig, level_name, sep = "_")
      result2 <- rbind(result2, result1)
    }

    result <- rbind(result, result2)
  }

  result[result > 100] <- Inf
  result <- result[!grepl("default", rownames(result)), , drop = FALSE]

  result
}

#' Plot ROC Curves and Compare Them
#'
#' @description
#' Generates Receiver Operating Characteristic (ROC) curves for multiple
#' predictors and optionally performs statistical comparisons between them.
#'
#' @param data Data frame containing the predictor variables and binary outcome.
#' @param response Character. Name of the binary outcome variable in `data`.
#' @param variables Character vector. Names of predictor variables for ROC curves.
#' @param fig.path Character or `NULL`. Directory path to save output PDF.
#'   Default is `NULL`.
#' @param main Character or `NULL`. Main title for the ROC plot. Default is `NULL`.
#' @param file.name Character or `NULL`. Output PDF filename without extension.
#'   Default is `"0-ROC of multiple variables"`.
#' @param palette Character. Color palette for ROC curves. Default is `"jama"`.
#' @param cols Character vector or `NULL`. Custom colors for ROC curves.
#'   Default is `NULL`.
#' @param alpha Numeric. Transparency level (1 = opaque, 0 = transparent).
#'   Default is `1`.
#' @param compare Logical. Whether to perform statistical comparison of AUCs.
#'   Default is `FALSE`.
#' @param smooth Logical. Whether to smooth ROC curves. Default is `TRUE`.
#' @param compare_method Character. Method for comparing ROC curves.
#'   Default is `"bootstrap"`.
#' @param boot.n Integer. Number of bootstrap replications. Default is `100`.
#'
#' @return A list containing:
#' \describe{
#'   \item{auc.out}{Data frame with AUC values and confidence intervals}
#'   \item{legend.name}{Vector of legend entries for the plot}
#'   \item{p.out}{If `compare = TRUE`, data frame with p-values from comparisons}
#' }
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' test_data <- data.frame(
#'   OS_status = sample(c(0, 1), 20, replace = TRUE),
#'   Marker1 = rnorm(20),
#'   Marker2 = rnorm(20)
#' )
#' result <- sig_roc(data = test_data, response = "OS_status",
#'                   variables = c("Marker1", "Marker2"),
#'                   smooth = FALSE)
#' if (!is.null(result)) print(result$auc.out)
sig_roc <- function(data,
                    response,
                    variables,
                    fig.path = NULL,
                    main = NULL,
                    file.name = NULL,
                    palette = "jama",
                    cols = NULL,
                    alpha = 1,
                    compare = FALSE,
                    smooth = TRUE,
                    compare_method = "bootstrap",
                    boot.n = 100) {
  if (is.null(data)) return(NULL)
  options(pROCProgress = list(name = "none"))

  if (!is.data.frame(data)) {
    cli::cli_abort("{.arg data} must be a data frame.")
  }

  if (!response %in% colnames(data)) {
    cli::cli_abort("Response column {.val {response}} not found in data.")
  }

  data <- as.data.frame(data)
  data[[response]] <- as.factor(data[[response]])

  if (length(levels(data[[response]])) != 2) {
    cli::cli_abort("Response variable must have exactly two levels.")
  }

  data <- data[!is.na(data[[response]]), ]
  variables <- variables[variables %in% colnames(data)]

  if (length(variables) == 0) {
    cli::cli_abort("No valid variables found in data.")
  }

  input <- as.data.frame(data[, c(response, variables)])

  cli::cli_alert_info("Input data preview:")
  if (interactive()) print(head(input))

  var_counts <- length(variables)

  if (is.null(cols)) {
    cols <- palettes(palette = palette, alpha = alpha, show_col = FALSE, show_message = FALSE)
    if (var_counts > length(cols)) {
      cols <- palettes(category = "random", alpha = alpha, show_col = FALSE, show_message = FALSE)
    }
  }

  auc.out <- c()

  if (!is.null(fig.path)) {
    if (is.null(file.name)) file.name <- "0-ROC of multiple variables"
    outfile <- file.path(fig.path, paste0(file.name, ".pdf"))
    if (!dir.exists(fig.path)) {
      dir.create(fig.path, recursive = TRUE, showWarnings = FALSE)
    }
    pdf(file = outfile, width = 5, height = 5)
    on.exit(dev.off(), add = FALSE)
  }

  x <- pROC::plot.roc(input[, 1], input[, 2],
    #ylim = c(0, 1),
    #xlim = c(1, 0),
    smooth = smooth,
    ci = TRUE,
    main = main,
    col = cols[2],
    lwd = 1.5,
    legacy.axes = TRUE,
    xlab = "False Positive Rate",
    ylab = "True Positive Rate"
  )

  ci.lower <- round(as.numeric(x$ci[1]), 3)
  ci.upper <- round(as.numeric(x$ci[3]), 3)

  auc.ci <- c(colnames(input)[2], round(as.numeric(x$auc), 3), paste(ci.lower, ci.upper, sep = "-"))
  auc.out <- rbind(auc.out, auc.ci)

  for (i in seq(3, ncol(input))) {
    x <- pROC::plot.roc(input[, 1], input[, i],
      add = TRUE,
      smooth = smooth,
      ci = TRUE,
      col = cols[i],
      lwd = 2,
      legacy.axes = TRUE,
      xlab = "False Positive Rate",
      ylab = "True Positive Rate"
    )

    ci.lower <- round(as.numeric(x$ci[1]), 3)
    ci.upper <- round(as.numeric(x$ci[3]), 3)
    auc.ci <- c(colnames(input)[i], round(as.numeric(x$auc), 3), paste(ci.lower, ci.upper, sep = "-"))
    auc.out <- rbind(auc.out, auc.ci)
  }

  auc.out <- as.data.frame(auc.out)
  colnames(auc.out) <- c("Name", "AUC", "AUC CI")

  legend.name <- paste(colnames(input)[seq(2, ncol(input))], " AUC = ", auc.out$AUC, sep = " ")
  #oldpar <- par(xpd = TRUE)
  #on.exit(par(oldpar), add = TRUE)
  legend("bottomright",
    legend = legend.name,
    col = cols[seq(2, length(variables) + 1)],
    lwd = 2,
    bty = "n"
  )

  if (compare) {
    p.out <- c()
    for (i in seq(2, ncol(input) - 1)) {
      for (j in seq(i + 1, ncol(input))) {
        p <- pROC::roc.test(input[, 1], input[, i], input[, j],
          method = compare_method, boot.n = boot.n, progress = "none"
        )
        p.tmp <- c(colnames(input)[i], colnames(input)[j], p$p.value)
        p.out <- rbind(p.out, p.tmp)
      }
    }
    p.out <- as.data.frame(p.out)
    colnames(p.out) <- c("ROC1", "ROC2", "p.value")
    p.out$p.value <- round(as.numeric(p.out$p.value), 5)
    return(list(auc.out = auc.out, legend.name = legend.name, p.out = p.out))
  } else {
    return(list(auc.out = auc.out, legend.name = legend.name))
  }
}

#' Remove Duplicate Gene Symbols in Gene Expression Data
#'
#' @description
#' This function addresses duplicate gene symbols in a gene expression dataset
#' by selecting the highest-expressing instance among duplicates. Users can
#' choose between mean, standard deviation, or sum as the ranking criterion
#' for selection. This is useful for preparing data where duplicates can lead
#' to issues in downstream analyses.
#'
#' @param eset A data frame or matrix representing gene expression data, with
#'   gene symbols as one of the columns.
#' @param column_of_symbol The name of the column containing gene symbols in
#'   `eset`.
#' @param method The ranking method to use for selecting among duplicate gene
#'   symbols: `"mean"` for mean expression, `"sd"` for standard deviation,
#'   or `"sum"` for sum of expression values. Default is `"mean"`.
#'
#' @return A modified version of `eset` where duplicate gene symbols have been
#'   reduced to a single entry (the highest-ranking one). The gene symbols are
#'   set as row names in the returned data frame.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @note
#' Important: This function performs selection, not aggregation. For duplicate
#' genes, it retains only the highest-ranking instance (based on the specified
#' method) and discards others.
#'
#' @examples
#' set.seed(123)
#' test_eset <- data.frame(
#'   symbol = c("GeneA", "GeneA", "GeneB", "GeneC"),
#'   S1 = c(10, 5, 20, 15),
#'   S2 = c(12, 7, 22, 17)
#' )
#' # Remove duplicates using mean expression
#' test_eset_unique <- remove_duplicate_genes(
#'   eset = test_eset,
#'   column_of_symbol = "symbol",
#'   method = "mean"
#' )
#' print(test_eset_unique)
remove_duplicate_genes <- function(eset,
                                   column_of_symbol,
                                   method = c("mean", "sd", "sum")) {
  if (is.null(eset)) return(NULL)
  # Input validation
  if (!is.data.frame(eset) && !is.matrix(eset)) {
    cli::cli_abort(c(
      "Invalid type for {.arg eset}.",
      "i" = "Expected a data frame or matrix, got {.cls {class(eset)}}."
    ))
  }

  eset <- as.data.frame(eset)
  rownames(eset) <- NULL

  if (!column_of_symbol %in% colnames(eset)) {
    cli::cli_abort("Column {.val {column_of_symbol}} not found in eset.")
  }

  method <- rlang::arg_match(method)

  # Check for duplicates
  n_dups <- nrow(eset) - length(unique(eset[[column_of_symbol]]))

  if (n_dups == 0) {
    cli::cli_alert_info("No duplicate gene symbols found.")
    return(tibble::column_to_rownames(eset, var = column_of_symbol))
  }

  cli::cli_alert_info(
    "Found {n_dups} duplicate symbol{?s}. Using {.val {method}} for ranking."
  )

  # Get expression columns (excluding symbol column)
  expr_cols <- setdiff(colnames(eset), column_of_symbol)

  # Calculate ranking index based on method (per row)
  order_index <- switch(method,
    "mean" = rowMeans(eset[, expr_cols, drop = FALSE], na.rm = TRUE),
    "sd" = apply(eset[, expr_cols, drop = FALSE], 1, stats::sd, na.rm = TRUE),
    "sum" = rowSums(eset[, expr_cols, drop = FALSE], na.rm = TRUE)
  )

  # Sort by ranking (descending) and keep first occurrence
  eset <- eset[order(order_index, decreasing = TRUE), , drop = FALSE]
  eset <- dplyr::distinct(eset, !!rlang::sym(column_of_symbol),
    .keep_all = TRUE
  )

  cli::cli_alert_success(
    "Reduced to {nrow(eset)} unique gene{?s}"
  )

  tibble::column_to_rownames(eset, var = column_of_symbol)
}

#' Visualize Expression Set Distribution
#'
#' @description
#' Generates boxplots and density plots to analyze the distribution of
#' expression values in an expression set. Useful for quality control and
#' assessing data normalization.
#'
#' @param eset Expression matrix or data frame with genes in rows and samples
#'   in columns.
#' @param quantile Integer specifying the divisor for sampling columns.
#'   Default is 3 (samples 1/3 of columns).
#' @param log Logical indicating whether to perform log2 transformation.
#'   Default is `TRUE`.
#' @param project Optional output directory path for saving files. If `NULL`,
#'   no files are saved. Default is `NULL`.
#'
#' @return Invisibly returns `NULL`. If `project` is provided, saves PNG files to disk.
#'
#' @export
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_eset <- matrix(rnorm(1000 * 10, mean = 5, sd = 2), 1000, 10)
#' rownames(sim_eset) <- paste0("Gene", 1:1000)
#' colnames(sim_eset) <- paste0("Sample", 1:10)
#'
#' # Run distribution plot
#' result <- eset_distribution(sim_eset)
eset_distribution <- function(eset, quantile = 3, log = TRUE, project = NULL) {
  if (is.null(eset)) return(NULL)
  if (!is.matrix(eset) && !is.data.frame(eset)) {
    cli::cli_abort("{.arg eset} must be a matrix or data frame")
  }
  if (nrow(eset) == 0 || ncol(eset) == 0) {
    cli::cli_abort("{.arg eset} must have at least one row and one column")
  }
  if (!is.numeric(quantile) || quantile <= 0) {
    cli::cli_abort("{.arg quantile} must be a positive number")
  }

  feas <- feature_manipulation(data = eset, feature = rownames(eset), is_matrix = TRUE)
  eset <- eset[rownames(eset) %in% feas, , drop = FALSE]

  n_samples <- ncol(eset)
  n_select <- max(1, round(n_samples / quantile))
  index <- sample(seq_len(n_samples), min(n_select, n_samples))
  eset1 <- eset[, index, drop = FALSE]

  if (log) {
    eset1 <- log2eset(eset1)
  }

  eset1 <- t(eset1)
  eset1 <- as.data.frame(eset1)
  eset1$Sample.Name <- rownames(eset1)

  rlang::check_installed("reshape2")
  eset.melt <- reshape2::melt(eset1, id.vars = "Sample.Name")
  colnames(eset.melt)[2:3] <- c("Symbol", "Intensity")

  if (!is.null(project)) {
    path <- creat_folder(project)
  }

  p <- ggplot2::ggplot(eset.melt, ggplot2::aes(x = .data$Sample.Name, y = .data$Intensity)) +
    ggplot2::geom_boxplot() +
    ggplot2::theme_bw() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(size = 23),
      axis.text.x = ggplot2::element_text(angle = 90, vjust = 0.5, hjust = 1.0)
    ) +
    ggplot2::ggtitle(
      "Normalized signal intensity",
      paste0(
        "Patient No. is: ", n_samples, "; ",
        "Number of features is: ", nrow(eset), ";  ",
        "Maximum is: ", round(max(eset, na.rm = TRUE), 2)
      )
    ) +
    ggplot2::ylab("Intensity") +
    ggplot2::xlab("Sample") +
    ggplot2::theme(
      legend.position = "none",
      panel.grid.major = ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank(),
      plot.subtitle = ggplot2::element_text(size = 18, hjust = 0.1, face = "italic", color = "black")
    )

  p2 <- ggplot2::ggplot(eset.melt, ggplot2::aes(.data$Intensity, group = .data$Sample.Name)) +
    ggplot2::geom_density() +
    ggplot2::theme_bw() +
    ggplot2::theme(
      panel.grid.major = ggplot2::element_blank(),
      panel.grid.minor = ggplot2::element_blank()
    ) +
    ggplot2::ggtitle("Histogram of Log2 Expression") +
    ggplot2::ylab("Density") +
    ggplot2::xlab("Log2 Expression")

  if (!is.null(project)) {
    ggplot2::ggsave(
      filename = paste0("1-", project, "-boxplot.png"),
      plot = p,
      width = 15, height = 8,
      path = path$folder_name
    )

    ggplot2::ggsave(
      filename = paste0("2-", project, "-Densityplot.png"),
      plot = p2,
      width = 9, height = 6,
      path = path$folder_name
    )
  }

  invisible(NULL)
}

#' Create Nested Output Folders
#'
#' @description
#' Creates one to three nested folders (if not existing) under the current
#' working directory and returns their names and absolute paths.
#'
#' @param f1 Character. First-level folder name.
#' @param f2 Character or `NULL`. Second-level folder name. Default is `NULL`.
#' @param f3 Character or `NULL`. Third-level folder name. Default is `NULL`.
#' @param return Deprecated (not used). Kept for backward compatibility.
#'
#' @return List with elements:
#' \describe{
#'   \item{folder_name}{Relative path to the created folder}
#'   \item{abspath}{Absolute path ending with '/'}
#' }
#'
#' @export
#'
#' @examples
#' creat_folder(file.path(tempdir(), "1-result"))
#' creat_folder(file.path(tempdir(), "1-result"), "figures", "correlation")
creat_folder <- function(f1, f2 = NULL, f3 = NULL, return = NULL) {
  if (!is.character(f1) || length(f1) != 1 || nchar(f1) == 0) {
    cli::cli_abort("{.arg f1} must be a non-empty character string")
  }

  components <- c(f1, f2, f3)
  components <- components[!vapply(components, is.null, logical(1))]

  if (startsWith(f1, "/") || startsWith(f1, "~") ||
    (nchar(f1) > 1 && substr(f1, 2, 2) == ":")) {
    path <- do.call(file.path, as.list(components))
  } else {
    path <- do.call(file.path, c(list(tempdir()), components))
  }

  if (!dir.exists(path)) {
    dir.create(path, recursive = TRUE, showWarnings = FALSE)
  }

  folder_name <- paste(components, collapse = "/")
  abspath <- paste0(normalizePath(path, mustWork = FALSE), "/")

  if (!is.null(return)) {
    cli::cli_warn("{.arg return} parameter is deprecated and will be ignored")
  }

  list(
    folder_name = folder_name,
    abspath = abspath
  )
}

#' Select Color Palettes for Visualization
#'
#' @description
#' Provides curated qualitative, sequential, and diverging palettes for multiple
#' plot types. Supports intensity adjustment and preview.
#'
#' @param category Character. Plot/palette category: one of `box`, `continue2`,
#'   `continue`, `random`, `heatmap`, `heatmap3`, `tidyheatmap`.
#' @param palette Character or numeric. Palette name or index (varies by category).
#' @param alpha Numeric. Alpha (transparency) scaling factor. Default is 1.
#' @param counts Integer. Number of colors (for continuous palettes).
#'   Default is 50.
#' @param show_col Logical. If TRUE, prints the palette. Default is TRUE.
#' @param show_message Logical. If TRUE, prints available options.
#'   Default is FALSE.
#'
#' @return Character vector of hex color codes.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' colors <- palettes(category = "box", palette = "nrc", show_col = FALSE)
#' heatmap_colors <- palettes(
#'   category = "heatmap", palette = 1, counts = 10, show_col = FALSE
#' )
palettes <- function(category = "box", palette = "nrc", alpha = 1,
                     counts = 50, show_col = TRUE, show_message = FALSE) {
  rlang::check_installed("ggsci")
  rlang::check_installed("RColorBrewer")
  rlang::check_installed("scales")

  valid_categories <- c(
    "box", "continue2", "continue", "random", "heatmap",
    "heatmap3", "tidyheatmap"
  )
  category <- rlang::arg_match(category, valid_categories)

  if (show_message) {
    cli::cli_alert_info(
      "Available categories: {paste(valid_categories, collapse = ', ')}"
    )
  }

  mypal <- switch(category,
    box = .palette_box(palette, alpha, show_message),
    continue2 = .palette_continue2(palette, alpha, show_message),
    random = .palette_random(palette),
    continue = .palette_continue(palette, show_message),
    heatmap = .palette_heatmap(palette, counts),
    heatmap3 = .palette_heatmap3(palette),
    tidyheatmap = .palette_tidyheatmap(palette, show_message)
  )

  if (show_col && !is.null(mypal)) {
    message(paste0("'", mypal, "'", collapse = ", "))
    scales::show_col(mypal)
  }

  mypal
}

#' @keywords internal
.palette_box <- function(palette, alpha, show_message) {
  valid_palettes <- c(
    "nrc", "jama", "aaas", "jco", "paired1", "paired2",
    "paired3", "paired4", "accent", "set2"
  )

  if (show_message) {
    cli::cli_alert_info(
      "Box palettes: {paste(valid_palettes, collapse = ', ')}"
    )
  }

  if (palette == "nrc") {
    return(ggsci::pal_npg("nrc", alpha = alpha)(9))
  }
  if (palette == "jama") {
    return(ggsci::pal_jama(palette = "default", alpha = alpha)(7))
  }
  if (palette == "aaas") {
    return(ggsci::pal_aaas(palette = "default", alpha = alpha)(9))
  }
  if (palette == "jco") {
    return(ggsci::pal_jco(palette = "default", alpha = alpha)(9))
  }
  if (palette == "paired1") {
    cols <- RColorBrewer::brewer.pal(11, "Paired")
    return(cols[1:8])
  }
  if (palette == "paired2") {
    cols <- RColorBrewer::brewer.pal(11, "Paired")
    return(cols[3:10])
  }
  if (palette == "paired3") {
    cols <- RColorBrewer::brewer.pal(11, "Paired")
    return(cols[5:11])
  }
  if (palette == "paired4") {
    cols <- RColorBrewer::brewer.pal(11, "Paired")
    return(cols[7:11])
  }
  if (palette == "accent") {
    return(RColorBrewer::brewer.pal(8, "Accent"))
  }
  if (palette == "set2") {
    return(RColorBrewer::brewer.pal(7, "Set2"))
  }

  cli::cli_abort("Palette {.val {palette}} not found. Use: {valid_palettes}")
}

#' @keywords internal
.palette_continue2 <- function(palette, alpha, show_message) {
  valid_palettes <- c("nrc", "jama", "aaas", "jco", "rdbu")

  if (show_message) {
    cli::cli_alert_info(
      "Continue2 palettes: {paste(valid_palettes, collapse = ', ')}"
    )
  }

  if (palette == "nrc") {
    cols <- ggsci::pal_npg("nrc", alpha = alpha)(9)
    return(cols[c(4, 1)])
  }
  if (palette == "jama") {
    cols <- ggsci::pal_jama(palette = "default", alpha = alpha)(9)
    return(cols[c(1, 4)])
  }
  if (palette == "aaas") {
    cols <- ggsci::pal_aaas(palette = "default", alpha = alpha)(9)
    return(cols[c(1, 6)])
  }
  if (palette == "jco") {
    cols <- ggsci::pal_jco(palette = "default", alpha = alpha)(9)
    return(cols[c(1, 2)])
  }
  if (palette == "rdbu") {
    cols <- RColorBrewer::brewer.pal(11, "RdBu")
    return(cols[c(10, 2)])
  }

  cli::cli_abort("Palette {.val {palette}} not found. Use: {valid_palettes}")
}

#' @keywords internal
.palette_random <- function(palette) {
  cli::cli_alert_info(
    "Random palettes: 1 (palette1), 2 (palette2), 3 (palette3), 4 (palette4)"
  )
  palette_num <- ifelse(palette %in% 1:4, palette, 4)
  load_data(paste0("palette", palette_num))
}

#' @keywords internal
.palette_continue <- function(palette, show_message) {
  valid_palettes <- c("rdbu", "puor", "blues", "reds")

  if (show_message) {
    cli::cli_alert_info(
      "Continue palettes: {paste(valid_palettes, collapse = ', ')}"
    )
  }

  palette_upper <- toupper(palette)

  if (palette == "rdbu") {
    return(RColorBrewer::brewer.pal(11, "RdBu"))
  }
  if (palette == "puor") {
    return(RColorBrewer::brewer.pal(11, "PuOr"))
  }
  if (palette == "blues") {
    return(RColorBrewer::brewer.pal(11, "Blues"))
  }
  if (palette == "reds") {
    return(RColorBrewer::brewer.pal(11, "Reds"))
  }

  cli::cli_abort("Palette {.val {palette}} not found. Use: {valid_palettes}")
}

#' @keywords internal
.palette_heatmap <- function(palette, counts) {
  cli::cli_alert_info(paste0(
    "Heatmap palettes: 1 (pheatmap), 2 (peach), 3 (blues), ",
    "4 (virids), 5 (reds), 6 (RdBu), 7 (navy_firebrick), 8 (magma)"
  ))

  if (palette == 1) {
    return(rev(colorRampPalette(
      RColorBrewer::brewer.pal(n = 7, name = "RdYlBu")
    )(counts)))
  }
  if (palette == 2) {
    return(colorRampPalette(c("#3182bd", "white", "#dd1c77"))(counts))
  }
  if (palette == 3) {
    return(rev(colorRampPalette(RColorBrewer::brewer.pal(8, "Blues"))(counts)))
  }
  if (palette == 4) {
    return(grDevices::colorRampPalette(c(
      "#000004FF", "#51127CFF", "#B63679FF",
      "#FCA50AFF", "#F7F419FF"
    ))(counts))
  }
  if (palette == 5) {
    return(rev(colorRampPalette(RColorBrewer::brewer.pal(8, "Reds"))(counts)))
  }
  if (palette == 6) {
    return(rev(colorRampPalette(RColorBrewer::brewer.pal(11, "RdBu"))(counts)))
  }
  if (palette == 7) {
    return(colorRampPalette(c("navy", "white", "firebrick"))(counts))
  }
  if (palette == 8) {
    return(grDevices::colorRampPalette(c(
      "#000004FF", "#6B0A7DFF", "#B63679FF",
      "#FCA50AFF", "#F7F419FF"
    ))(counts))
  }

  cli::cli_abort("Heatmap palette {.val {palette}} must be 1-8")
}

#' @keywords internal
.palette_heatmap3 <- function(palette) {
  cli::cli_alert_info(paste0(
    "Heatmap3 palettes: pheatmap, peach, blues, virids, reds, normal"
  ))

  if (palette == "pheatmap") {
    return(c("#4575B4", "#FEF9B6", "#D73027"))
  }
  if (palette == "peach") {
    return(c("#3182bd", "white", "#dd1c77"))
  }
  if (palette == "blues") {
    return(c("#F7FBFF", "#88BEDC", "#084594"))
  }
  if (palette == "virids") {
    return(c("#000004FF", "#CD4347FF", "#FCFFA4FF"))
  }
  if (palette == "reds") {
    return(c("#FFF5F0", "#FB7555", "#99000D"))
  }
  if (palette == "normal") {
    return(c("navy", "white", "firebrick"))
  }

  cli::cli_abort(paste0(
    "Heatmap3 palette {.val {palette}} not found. ",
    "Use: pheatmap, peach, blues, virids, reds, normal"
  ))
}

#' @keywords internal
.palette_tidyheatmap <- function(palette, show_message) {
  if (show_message) {
    cli::cli_alert_info("Tidyheatmap palettes: 1-6")
  }

  if (palette == 1) {
    return(c("#000004FF", "#B63679FF", "#F7F419FF"))
  }
  if (palette == 2) {
    return(rev(RColorBrewer::brewer.pal(3, "RdYlBu")))
  }
  if (palette == 3) {
    return(rev(RColorBrewer::brewer.pal(3, "RdYlGn")))
  }
  if (palette == 4) {
    return(rev(RColorBrewer::brewer.pal(3, "Spectral")))
  }
  if (palette == 5) {
    return(rev(RColorBrewer::brewer.pal(3, "PiYG")))
  }
  if (palette == 6) {
    return(c("navy", "white", "firebrick"))
  }

  cli::cli_abort("Tidyheatmap palette {.val {palette}} must be 1-6")
}

#' Batch Signature Box Plots for Group Comparisons
#'
#' @description
#' Generates multiple box plots for specified features (signatures) across groups
#' in the input data. Supports customization of plot appearance, output path,
#' statistical annotation, and compatibility with Seurat objects. Plots are saved
#' to the specified directory or a default folder.
#'
#' @param input Data frame or Seurat object containing the data for analysis.
#' @param vars Character vector. Features or variables to analyze. When
#'   `pattern_vars = TRUE`, these are treated as regular expression patterns.
#' @param groups Character vector. Grouping variable(s) for comparison.
#' @param pattern_vars Logical indicating whether to treat `vars` as regular
#'   expression patterns for matching column names. Default is `FALSE`.
#' @param path Character string or `NULL`. Directory to save plots. If `NULL`,
#'   plots are not saved. Default is `NULL`.
#' @param index Integer. Starting index for plot filenames. Default is `0`.
#' @param angle_x_text Numeric. Angle of x-axis labels in degrees. Default is `0`.
#' @param hjust Numeric. Horizontal justification of x-axis labels. Default is `0.5`.
#' @param palette Character. Color palette for plots. Default is `"jama"`.
#' @param cols Character vector or `NULL`. Custom colors for plot elements.
#' @param jitter Logical indicating whether to add jittered points. Default is `FALSE`.
#' @param point_size Numeric. Size of points. Default is `5`.
#' @param size_of_font Numeric. Base font size. Default is `8`.
#' @param size_of_pvalue Numeric. Size of p-value text. Default is `4.5`.
#' @param show_pvalue Logical indicating whether to display p-values.
#'   Default is `TRUE`.
#' @param return_stat_res Logical indicating whether to return statistical
#'   results instead of saving plots. Default is `FALSE`.
#' @param assay Character string or `NULL`. Assay type for Seurat objects.
#' @param slot Character. Data slot for Seurat objects. Default is `"scale.data"`.
#' @param scale Logical indicating whether to scale data before analysis.
#'   Default is `FALSE`.
#' @param height Numeric. Height of plots in inches. Default is `5`.
#' @param width Numeric. Width of plots in inches. Default is `3.5`.
#' @param fig_type Character. File format for plots (e.g., `"pdf"`, `"png"`).
#'   Default is `"pdf"`.
#' @param max_count_feas Integer. Maximum number of features to analyze when
#'   `pattern_vars = TRUE`. If matched variables exceed this limit, only the
#'   first `max_count_feas` features are used. Default is `30`.
#'
#' @return If `return_stat_res = TRUE`, returns a data frame of statistical
#'   results; otherwise, invisibly returns the path to saved plots.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_pdata <- data.frame(
#'   ID = paste0("Sample", 1:50),
#'   subtype = sample(c("TypeA", "TypeB", "TypeC"), 50, replace = TRUE),
#'   TMEscore_plus = rnorm(50),
#'   GZMB = rnorm(50)
#' )
#' sig_box_batch(
#'   input = sim_pdata,
#'   vars = c("TMEscore_plus", "GZMB"),
#'   groups = "subtype",
#'   jitter = TRUE,
#'   palette = "jco",
#'   path = tempdir()
#' )
sig_box_batch <- function(input,
                          vars,
                          groups,
                          pattern_vars = FALSE,
                          path = NULL,
                          index = 0,
                          angle_x_text = 0,
                          hjust = 0.5,
                          palette = "jama",
                          cols = NULL,
                          jitter = FALSE,
                          point_size = 5,
                          size_of_font = 8,
                          size_of_pvalue = 4.5,
                          show_pvalue = TRUE,
                          return_stat_res = FALSE,
                          assay = NULL,
                          slot = "scale.data",
                          scale = FALSE,
                          height = 5,
                          width = 3.5,
                          fig_type = "pdf",
                          max_count_feas = 30) {
  # Input validation
  if (!is.data.frame(input) && !inherits(input, "Seurat")) {
    cli::cli_abort("{.arg input} must be a data frame or Seurat object")
  }

  if (!is.character(vars)) {
    cli::cli_abort("{.arg vars} must be a character vector")
  }

  if (!is.character(groups)) {
    cli::cli_abort("{.arg groups} must be a character vector")
  }

  # Handle pattern matching for vars
  if (pattern_vars) {
    rlang::check_installed("stringr")

    vars_matched <- unique(unlist(lapply(vars, function(pattern) {
      colnames(input)[stringr::str_detect(colnames(input), pattern = pattern)]
    })))

    vars_matched <- vars_matched[!is.na(vars_matched)]

    if (length(vars_matched) == 0) {
      cli::cli_abort("No variables matched the specified patterns")
    }

    if (length(vars_matched) > max_count_feas) {
      cli::cli_alert_warning(
        "Matched {length(vars_matched)} variables, using first {max_count_feas}"
      )
      vars <- vars_matched[seq_len(max_count_feas)]
    } else {
      vars <- vars_matched
    }

    cli::cli_alert_info("Variables to analyze: {.val {paste(vars, collapse = ', ')}}")
  }

  # Validate that sig_box_batch is appropriate (needs multiple vars or groups)
  if (length(vars) == 1 && length(groups) == 1) {
    cli::cli_abort(c(
      "{.fn sig_box_batch} requires multiple variables or groups",
      i = "Use {.fn sig_box} for single variable and single group comparisons"
    ))
  }

  # Create output directory
  save_results <- !is.null(path)
  if (save_results) {
    path <- creat_folder(path)
  } else {
    path <- NULL
  }

  # Process multiple variables
  if (length(vars) > 1) {
    for (i in seq_along(vars)) {
      cli::cli_alert_info("Processing feature: {.val {vars[i]}}")

      p <- sig_box(
        data            = input,
        signature       = vars[i],
        variable        = groups,
        angle_x_text    = angle_x_text,
        hjust           = hjust,
        palette         = palette,
        cols            = cols,
        jitter          = jitter,
        point_size      = point_size,
        size_of_font    = size_of_font,
        size_of_pvalue  = size_of_pvalue,
        show_pairwise_p = show_pvalue,
        show_overall_p  = FALSE,
        return_stat_res = return_stat_res,
        assay           = assay,
        slot            = slot,
        scale           = scale
      )

      if (!return_stat_res && save_results) {
        ggplot2::ggsave(
          filename = paste0(index + i, "-", vars[i], "-and-", groups, ".", fig_type),
          plot = p,
          height = height,
          width = width,
          path = path$folder_name
        )
      }
    }
  }

  # Process multiple groups
  if (length(groups) > 1) {
    base_height <- height

    for (i in seq_along(groups)) {
      cli::cli_alert_info("Processing group: {.val {groups[i]}}")

      p <- sig_box(
        data            = input,
        signature       = vars,
        variable        = groups[i],
        angle_x_text    = angle_x_text,
        hjust           = hjust,
        palette         = palette,
        cols            = cols,
        jitter          = jitter,
        point_size      = point_size,
        size_of_font    = size_of_font,
        size_of_pvalue  = size_of_pvalue,
        show_pairwise_p = show_pvalue,
        show_overall_p  = FALSE,
        return_stat_res = return_stat_res,
        assay           = assay,
        slot            = slot,
        scale           = scale
      )

      if (!return_stat_res && save_results) {
        levs <- unique(input[[groups[i]]])
        levs <- length(levs[!is.na(levs)])
        plot_height <- 2 + base_height * levs

        ggplot2::ggsave(
          filename = paste0(index + i, "-", vars, "-and-", groups[i], ".", fig_type),
          plot = p,
          height = plot_height,
          width = width,
          path = path$folder_name
        )
      }
    }
  }

  if (save_results) {
    cli::cli_alert_success("Batch processing complete. Plots saved to: {.path {path$folder_name}}")
    invisible(path$folder_name)
  } else {
    invisible(NULL)
  }
}

#' Visualize Cell Fractions as Stacked Bar Chart
#'
#' @description
#' Creates stacked bar charts to visualize tumor microenvironment (TME) cell
#' fractions. Supports batch visualization of deconvolution results from
#' methods such as CIBERSORT, EPIC, and quanTIseq.
#'
#' @param input Data frame containing deconvolution results.
#' @param id Character string specifying the column name containing sample
#'   identifiers. Default is "ID".
#' @param title Character string specifying the plot title.
#'   Default is "Cell Fraction".
#' @param features Character vector specifying column names representing cell
#'   types to plot. If NULL, columns are selected based on `pattern`.
#'   Default is NULL.
#' @param pattern Character string or regular expression to match column names
#'   for automatic feature selection. Used when `features` is NULL.
#'   Default is NULL.
#' @param legend.position Character string specifying legend position
#'   ("bottom", "top", "left", "right"). Default is "bottom".
#' @param coord_flip Logical indicating whether to flip plot coordinates using
#'   `coord_flip()`. Default is TRUE.
#' @param palette Integer specifying the color palette to use. Default is 3.
#' @param show_col Logical indicating whether to display color information.
#'   Default is FALSE.
#' @param cols Character vector of custom colors. If NULL, palette is used.
#'   Default is NULL.
#'
#' @return A ggplot2 object representing the stacked bar chart.
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' set.seed(123)
#' input_data <- data.frame(
#'   ID = paste0("Sample", 1:10),
#'   Cell_A = runif(10, 0, 0.4),
#'   Cell_B = runif(10, 0, 0.3),
#'   Cell_C = runif(10, 0, 0.3)
#' )
#' cell_bar_plot(input = input_data, id = "ID", features = c("Cell_A", "Cell_B", "Cell_C"))
cell_bar_plot <- function(input, id = "ID", title = "Cell Fraction",
                          features = NULL, pattern = NULL,
                          legend.position = "bottom", coord_flip = TRUE,
                          palette = 3, show_col = FALSE, cols = NULL) {
  if (!is.data.frame(input)) {
    cli::cli_abort("{.arg input} must be a data frame")
  }
  if (nrow(input) == 0) {
    cli::cli_abort("{.arg input} has no rows")
  }
  if (!id %in% colnames(input)) {
    cli::cli_abort("ID column {.val {id}} not found in input")
  }

  input <- as.data.frame(input)
  colnames(input)[colnames(input) == id] <- "ID"

  if (is.null(features)) {
    if (is.null(pattern)) {
      cli::cli_abort("{.arg pattern} must be provided when {.arg features} is NULL")
    }
    feas <- colnames(input)[stringr::str_detect(colnames(input), pattern)]
    if (length(feas) == 0) {
      cli::cli_abort("No columns match pattern {.val {pattern}}")
    }
  } else {
    feas <- features[features %in% colnames(input)]
    if (length(feas) == 0) {
      cli::cli_abort("None of the specified features found in input")
    }
  }

  input <- input[, c("ID", feas), drop = FALSE]

  legend.direction <- if (legend.position %in% c("top", "bottom")) {
    "horizontal"
  } else {
    "vertical"
  }

  if (is.null(cols)) {
    cols <- palettes(
      category = "random",
      palette = palette %||% 4,
      show_col = show_col,
      show_message = TRUE
    )
  }

  plot_data <- tidyr::gather(input, "cell_type", "fraction", -.data$ID)

  lms <- rev(levels(input$ID))
  if (!all(lms %in% plot_data$ID)) {
    cli::cli_alert_warning("ID column is not a factor or has levels not present in data. Using unique IDs from data.")
    lms <- sort(unique(plot_data$ID))
  }

  p <- ggplot2::ggplot(plot_data, ggplot2::aes(
    x = .data$ID, y = .data$fraction, fill = .data$cell_type
  )) +
    ggplot2::geom_bar(stat = "identity") +
    ggplot2::theme_light() +
    ggplot2::scale_fill_manual(values = cols) +
    ggplot2::scale_x_discrete(limits = lms) +
    ggplot2::ggtitle(title) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(size = ggplot2::rel(2), hjust = 0.5),
      axis.text.x = ggplot2::element_text(
        face = "plain", angle = 0, hjust = 1, color = "black"
      ),
      axis.text.y = ggplot2::element_text(
        face = "plain", angle = 30, hjust = 1, color = "black"
      ),
      legend.title = ggplot2::element_blank(),
      legend.position = legend.position,
      legend.direction = legend.direction,
      legend.justification = c(0.5, 0.5),
      legend.box = "horizontal",
      legend.box.just = "top"
    )

  if (coord_flip) {
    p <- p + ggplot2::coord_flip()
  }

  p
}

#' Extract Best Cutoff and Add Binary Variable to Data Frame
#'
#' @description
#' Finds the best cutoff point for a continuous variable in survival analysis.
#' Takes input data containing a continuous variable and survival information
#' (time and status). Returns the modified input data with a new binary variable
#' created based on the best cutoff point.
#'
#' @param pdata Data frame containing survival information and features.
#' @param variable Character string specifying the continuous variable name.
#' @param time Character string specifying the time-to-event column name.
#'   Default is `"time"`.
#' @param status Character string specifying the event status column name.
#'   Default is `"status"`.
#' @param print_result Logical indicating whether to print results.
#'   Default is `TRUE`.
#'
#' @return List containing:
#' \describe{
#'   \item{pdata}{Data frame with binary variable added}
#'   \item{best_cutoff}{Numeric cutoff value}
#'   \item{cox_continuous_object}{Cox model summary for continuous variable}
#'   \item{summary_binary_variable}{Summary of binary variable}
#'   \item{cox_binary_object}{Cox model summary for binary variable}
#' }
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' pdata <- data.frame(
#'   time = rexp(100),
#'   status = rbinom(100, 1, 0.5),
#'   score = rnorm(100, mean = 50, sd = 10)
#' )
#' result <- best_cutoff2(pdata, variable = "score", print_result = FALSE)
#' result$best_cutoff
best_cutoff2 <- function(pdata, variable, time = "time", status = "status",
                         print_result = TRUE) {
  if (!is.data.frame(pdata)) {
    cli::cli_abort("{.arg pdata} must be a data frame")
  }
  if (nrow(pdata) == 0) {
    cli::cli_abort("{.arg pdata} has no rows")
  }
  if (!variable %in% colnames(pdata)) {
    cli::cli_abort("Variable {.val {variable}} not found in pdata")
  }
  if (!time %in% colnames(pdata)) {
    cli::cli_abort("Time column {.val {time}} not found in pdata")
  }
  if (!status %in% colnames(pdata)) {
    cli::cli_abort("Status column {.val {status}} not found in pdata")
  }

  pdata <- as.data.frame(pdata)
  colnames(pdata)[colnames(pdata) == time] <- "time"
  colnames(pdata)[colnames(pdata) == status] <- "status"

  pdata <- pdata[!is.na(pdata$time) & !is.na(pdata$status), , drop = FALSE]
  if (nrow(pdata) == 0) {
    cli::cli_abort("No valid data after removing NA values")
  }

  pdata$time <- as.numeric(pdata$time)
  pdata$status <- as.numeric(pdata$status)

  surv_obj <- survival::Surv(pdata$time, pdata$status)
  iscutoff <- survminer::surv_cutpoint(
    pdata,
    time = "time",
    event = "status",
    variables = variable
  )

  cutoff_value <- iscutoff$cutpoint$cutpoint
  cli::cli_alert_success("Best cutoff for {.val {variable}}: {round(cutoff_value, 3)}")
  plot(iscutoff, variable, palette = "npg")

  cox_cont <- summary(survival::coxph(
    surv_obj ~ pdata[[variable]],
    data = pdata
  ))

  variable2 <- paste0(variable, "_binary")
  pdata[[variable2]] <- ifelse(
    pdata[[variable]] < cutoff_value,
    "Low",
    "High"
  )
  pdata[[variable2]] <- factor(pdata[[variable2]], levels = c("Low", "High"))

  colnames(pdata)[colnames(pdata) == "time"] <- time
  colnames(pdata)[colnames(pdata) == "status"] <- status

  binary_summary <- summary(pdata[[variable2]])
  cox_binary <- summary(survival::coxph(
    surv_obj ~ pdata[[variable2]],
    data = pdata
  ))

  if (print_result) {
    print(list(
      best_cutoff = paste0("best cutoff = ", cutoff_value),
      cox_continuous_object = cox_cont,
      summary_binary_variable = binary_summary,
      cox_binary_object = cox_binary
    ))
  }

  list(
    pdata = pdata,
    best_cutoff = cutoff_value,
    cox_continuous_object = cox_cont,
    summary_binary_variable = binary_summary,
    cox_binary_object = cox_binary
  )
}

#' Batch Wilcoxon Rank-Sum Test Between Two Groups
#'
#' @description
#' Performs Wilcoxon rank-sum tests (Mann-Whitney U tests) to compare the
#' distribution of specified features between two groups. Computes p-values,
#' adjusts for multiple testing, and ranks features by significance.
#'
#' @param data Data frame containing the dataset for analysis.
#' @param target Character string specifying the column name of the grouping
#'   variable. Default is `"group"`.
#' @param feature Character vector specifying feature names to analyze. If
#'   `NULL`, prompts for selection (interactive mode only). Default is `NULL`.
#' @param feature_manipulation Logical indicating whether to apply feature
#'   manipulation filtering. Default is `FALSE`.
#'
#' @return Tibble with columns:
#' \describe{
#'   \item{sig_names}{Feature name}
#'   \item{p.value}{Raw p-value}
#'   \item{statistic}{Difference in means between groups}
#'   \item{p.adj}{Adjusted p-value (Benjamini-Hochberg)}
#'   \item{log10pvalue}{Negative log10-transformed p-value}
#'   \item{stars}{Significance stars: **** p<0.0001, *** p<0.001,
#'     ** p<0.01, * p<0.05, + p<0.5}
#'   \item{group1, group2}{Mean values for each group}
#' }
#'
#' @author Dongqiang Zeng
#' @export
#'
#' @examples
#' # Create small example data
#' set.seed(123)
#' test_data <- data.frame(
#'   Gender = rep(c("Male", "Female"), each = 50),
#'   Signature1 = rnorm(100),
#'   Signature2 = rnorm(100)
#' )
#' # Compare features by gender
#' res <- batch_wilcoxon(
#'   data = test_data,
#'   target = "Gender",
#'   feature = c("Signature1", "Signature2")
#' )
#' head(res)
batch_wilcoxon <- function(data,
                           target = "group",
                           feature = NULL,
                           feature_manipulation = FALSE) {
  if (is.null(data)) return(NULL)
  # Input validation
  if (!is.data.frame(data)) {
    cli::cli_abort("{.arg data} must be a non-null data frame.")
  }
  if (!target %in% colnames(data)) {
    cli::cli_abort("Target variable {.val {target}} not found in data.")
  }

  data <- as.data.frame(data)

  # Rename target column
  colnames(data)[colnames(data) == target] <- "group"

  # Clean group data
  data <- data[!is.na(data$group) & data$group != "", , drop = FALSE]
  data$group <- as.character(data$group)

  group_names <- unique(data$group)
  if (length(group_names) != 2) {
    cli::cli_abort(c(
      "Wilcoxon test requires exactly 2 groups.",
      "i" = "Found {length(group_names)} group{?s}: {.val {group_names}}"
    ))
  }

  # Feature selection
  if (is.null(feature)) {
    if (!interactive()) {
      cli::cli_abort(paste(
        "{.arg feature} must be specified in",
        "non-interactive mode."
      ))
    }
    cli::cli_alert_info("Select features for analysis")
    index <- utils::menu(
      c("All continuous features", "Specify manually"),
      title = "Feature selection:"
    )
    if (index == 1) {
      feature <- colnames(data)[vapply(data, is.numeric, logical(1))]
    } else {
      cli::cli_abort("Please specify features via the {.arg feature} parameter")
    }
  }

  if (!is.character(feature) || length(feature) == 0) {
    cli::cli_abort("{.arg feature} must be a non-empty character vector.")
  }

  # Filter valid features
  valid_features <- feature[feature %in% colnames(data)]
  invalid_features <- setdiff(feature, colnames(data))

  if (length(invalid_features) > 0) {
    cli::cli_alert_warning(paste(
      "Ignoring {length(invalid_features)} invalid feature{?s}:",
      "{.val {invalid_features}}"
    ))
  }

  if (length(valid_features) == 0) {
    cli::cli_abort("No valid features found in data.")
  }

  # Apply feature manipulation if requested
  if (feature_manipulation) {
    valid_features <- IOBR::feature_manipulation(
      data = data,
      feature = valid_features,
      print_result = FALSE
    )
  }

  cli::cli_alert_info("Groups: {.val {group_names}}")
  cli::cli_alert_info("Features: {length(valid_features)}")

  # Subset data
  data <- data[, c("group", valid_features), drop = FALSE]

  # Perform Wilcoxon tests
  test_results <- lapply(valid_features, function(feat) {
    stats::wilcox.test(data[[feat]] ~ data$group, exact = FALSE)
  })
  names(test_results) <- valid_features

  # Calculate group means - vectorized approach
  result_mean <- data %>%
    dplyr::group_by(.data$group) %>%
    dplyr::summarise(
      dplyr::across(
        dplyr::all_of(valid_features), function(.x) mean(.x, na.rm = TRUE)
      ),
      .groups = "drop"
    )

  # Convert to wide format with groups as columns
  result_mean <- tidyr::pivot_longer(result_mean,
    cols = valid_features
  ) %>% tidyr::pivot_wider(
    id_cols = "name", names_from = "group"
  )

  # Calculate statistic (difference between groups)
  if (ncol(result_mean) >= 3) { # name + 2 groups column
    result_mean$statistic <- result_mean[[2]] - result_mean[[3]]
  } else {
    result_mean$statistic <- NA_real_
  }

  # Build results
  cc <- data.frame(
    name = valid_features,
    p.value = vapply(test_results, function(x) x$p.value, numeric(1)),
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  cc <- dplyr::left_join(cc, result_mean, by = "name") %>%
    dplyr::arrange(.data$p.value) %>%
    dplyr::mutate(
      p.adj = stats::p.adjust(.data$p.value, method = "BH"),
      log10pvalue = -log10(.data$p.value),
      stars = cut(.data$p.value,
        breaks = c(-Inf, 0.0001, 0.001, 0.01, 0.05, 0.5, Inf),
        labels = c("****", "***", "**", "*", "+", "")
      )
    )
  colnames(cc)[1] <- "sig_names"

  cli::cli_alert_success("Wilcoxon test complete")

  tibble::as_tibble(cc)
}

#' Scale and Manipulate a Matrix
#'
#' @description
#' Scales a gene expression matrix, optionally applies logarithmic
#' transformation, and performs feature manipulation to remove problematic
#' variables (e.g., genes with zero variance or missing values).
#'
#' @param matrix Numeric matrix with genes as rows and samples as columns.
#' @param log2matrix Logical indicating whether to apply log2 transformation
#'   using [log2eset()]. Default is `TRUE`.
#' @param manipulate Logical indicating whether to perform feature manipulation
#'   to remove problematic features. Default is `TRUE`.
#'
#' @return A scaled matrix (genes as rows, samples as columns).
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' set.seed(123)
#' test_matrix <- matrix(
#'   rlnorm(100),
#'   ncol = 5,
#'   dimnames = list(paste0("Gene", 1:20), paste0("Sample", 1:5))
#' )
#' eset2 <- scale_matrix(test_matrix, log2matrix = FALSE, manipulate = TRUE)
#' head(eset2)
scale_matrix <- function(matrix, log2matrix = TRUE, manipulate = TRUE) {
  if (is.null(matrix)) return(NULL)
  # Input validation
  if (!is.matrix(matrix) && !is.data.frame(matrix)) {
    cli::cli_abort(c(
      "Invalid input type for {.arg matrix}.",
      "i" = "Expected a matrix or data frame, got {.cls {class(matrix)}}."
    ))
  }

  # Ensure numeric matrix
  if (!is.matrix(matrix)) {
    matrix <- as.matrix(matrix)
  }

  if (!is.numeric(matrix)) {
    cli::cli_abort("Matrix must contain numeric values.")
  }

  # Apply log2 transformation if requested
  if (log2matrix) {
    matrix <- log2eset(matrix)
  }

  # Scale: center and scale by column (sample), then transpose back
  # Transpose so samples become rows for scale()
  matrix <- as.data.frame(t(matrix))
  matrix <- scale(matrix, center = TRUE, scale = TRUE)
  matrix <- as.data.frame(t(matrix))

  # Feature manipulation to remove problematic features
  if (manipulate) {
    feas <- feature_manipulation(
      data = matrix,
      feature = rownames(matrix),
      is_matrix = TRUE,
      print_result = TRUE
    )
    matrix <- matrix[rownames(matrix) %in% feas, , drop = FALSE]
    cli::cli_alert_info("Retained {length(feas)} features after manipulation")
  }

  as.matrix(matrix)
}

#' Calculate and Visualize Correlation Between Two Variables
#'
#' @description
#' Calculates and visualizes the correlation between two variables with options
#' for scaling, handling missing values, and incorporating grouping data.
#'
#' @param eset Dataset containing the variables (data frame or matrix).
#' @param pdata Optional phenotype data frame. Default is `NULL`.
#' @param var1 Name of the first variable.
#' @param var2 Name of the second variable.
#' @param is.matrix Logical indicating if `eset` is a matrix with features as rows.
#'   Default is `FALSE`.
#' @param id_eset ID column in `eset`. Default is `"ID"`.
#' @param id_pdata ID column in `pdata`. Default is `"ID"`.
#' @param scale Logical indicating whether to scale data. Default is `TRUE`.
#' @param subtype Optional grouping variable for coloring points. Default is `NULL`.
#' @param na.subtype.rm Logical indicating whether to remove NA in subtype.
#'   Default is `FALSE`.
#' @param color_subtype Colors for subtypes. Default is `NULL`.
#' @param palette Color palette name. Default is `"jama"`.
#' @param index Plot index for filename. Default is `NULL` (uses 1).
#' @param method Correlation method: `"spearman"`, `"pearson"`, or `"kendall"`.
#'   Default is `"spearman"`.
#' @param show_cor_result Logical indicating whether to print correlation result.
#'   Default is `TRUE`.
#' @param col_line Color of regression line. Default is `NULL` (auto-determine).
#' @param id Column for point labels. Default is `NULL`.
#' @param show_label Logical indicating whether to show labels. Default is `FALSE`.
#' @param point_size Size of points. Default is 4.
#' @param title Plot title. Default is `NULL`.
#' @param alpha Transparency of points. Default is 0.5.
#' @param title_size Title font size. Default is 1.5.
#' @param text_size Text font size. Default is 10.
#' @param axis_angle Axis label angle. Default is 0.
#' @param hjust Horizontal justification. Default is 0.
#' @param show_plot Logical indicating whether to display plot. Default is `TRUE`.
#' @param save_plot Logical indicating whether to save plot. Default is `FALSE`.
#' @param path Save path. Default is `NULL`.
#' @param fig.format Figure format: `"png"` or `"pdf"`. Default is `"png"`.
#' @param fig.width Figure width in inches. Default is 7.
#' @param fig.height Figure height in inches. Default is 7.3.
#' @param add.hdr.line Logical for adding HDR (high density region) lines.
#'   Default is `FALSE`.
#'
#' @return A ggplot object of the correlation plot.
#'
#' @export
#' @author Dongqiang Zeng
#'
#' @examples
#' # Simulate data
#' set.seed(123)
#' sim_eset <- matrix(rnorm(100 * 20), 100, 20)
#' rownames(sim_eset) <- c("GZMB", "CD274", paste0("Gene", 3:100))
#' colnames(sim_eset) <- paste0("Sample", 1:20)
#' 
#' # Calculate and plot correlation
#' p <- get_cor(eset = sim_eset, is.matrix = TRUE, var1 = "GZMB", var2 = "CD274", show_plot = FALSE)
#' if (!is.null(p)) print(p)
get_cor <- function(eset,
                    pdata = NULL,
                    var1,
                    var2,
                    is.matrix = FALSE,
                    id_eset = "ID",
                    id_pdata = "ID",
                    scale = TRUE,
                    subtype = NULL,
                    na.subtype.rm = FALSE,
                    color_subtype = NULL,
                    palette = "jama",
                    index = NULL,
                    method = c("spearman", "pearson", "kendall"),
                    show_cor_result = TRUE,
                    col_line = NULL,
                    id = NULL,
                    show_label = FALSE,
                    point_size = 4,
                    title = NULL,
                    alpha = 0.5,
                    title_size = 1.5,
                    text_size = 10,
                    axis_angle = 0,
                    hjust = 0,
                    show_plot = TRUE,
                    save_plot = FALSE,
                    path = NULL,
                    fig.format = "png",
                    fig.width = 7,
                    fig.height = 7.3,
                    add.hdr.line = FALSE) {
  if (is.null(eset)) return(NULL)
  # Validate arguments
  method <- rlang::arg_match(method)
  if (is.null(index)) index <- 1

  # Validate variables
  if (missing(var1) || missing(var2)) {
    cli::cli_abort("Both {.arg var1} and {.arg var2} must be specified.")
  }

  # Prepare data
  data <- .prepare_cor_data(
    eset = eset,
    pdata = pdata,
    var1 = var1,
    var2 = var2,
    is.matrix = is.matrix,
    id_eset = id_eset,
    id_pdata = id_pdata,
    scale = scale
  )

  # Check if variables exist
  if (!var1 %in% colnames(data)) {
    cli::cli_abort("Variable {.val {var1}} not found in data.")
  }
  if (!var2 %in% colnames(data)) {
    cli::cli_abort("Variable {.val {var2}} not found in data.")
  }

  # Remove NA values
  data <- data[!is.na(data[[var1]]) & !is.na(data[[var2]]), , drop = FALSE]

  if (nrow(data) < 3) {
    cli::cli_abort("Insufficient data after removing NA values (need >= 3).")
  }

  cli::cli_alert_info("Calculating {method} correlation (n = {nrow(data)})")

  # Calculate correlation
  cor_result <- stats::cor.test(data[[var1]], data[[var2]], method = method)
  if (show_cor_result) print(cor_result)

  pvalue <- exact_pvalue(data[[var1]], data[[var2]], method = method)
  cli::cli_alert_info("Exact p-value: {format(pvalue, digits = 2, scientific = TRUE)}")

  # Determine line color
  if (is.null(col_line)) {
    col_line <- if (cor_result$estimate > 0) "darkred" else "steelblue"
  }

  # Get color palette
  if (is.null(color_subtype) && !is.null(subtype)) {
    color_subtype <- palettes(
      category = "box",
      palette = palette,
      show_col = FALSE,
      show_message = FALSE
    )
  }

  # Build plot
  p <- .build_cor_plot(
    data = data,
    var1 = var1,
    var2 = var2,
    subtype = subtype,
    na.subtype.rm = na.subtype.rm,
    color_subtype = color_subtype,
    col_line = col_line,
    point_size = point_size,
    alpha = alpha,
    title = title,
    cor_result = cor_result,
    pvalue = pvalue,
    id = id,
    show_label = show_label
  )

  # Apply theme
  theme <- design_mytheme(
    axis_title_size = title_size,
    axis_text_size = text_size,
    hjust = hjust,
    axis_angle = axis_angle
  )

  p <- p + theme +
    ggplot2::theme(
      plot.subtitle = ggplot2::element_text(
        size = 15,
        hjust = 0,
        face = "italic",
        color = "black"
      )
    )

  # Add HDR lines if requested
  if (add.hdr.line) {
    rlang::check_installed("ggdensity")
    p <- p + ggdensity::geom_hdr_lines()
  }

  if (show_plot) print(p)

  # Save plot if requested
  if (save_plot) {
    .save_cor_plot(
      p = p,
      path = path,
      var1 = var1,
      var2 = var2,
      index = index,
      fig.format = fig.format,
      fig.width = fig.width,
      fig.height = fig.height,
      data = data
    )
  }

  invisible(p)
}

# Helper: Prepare data for correlation
.prepare_cor_data <- function(eset, pdata, var1, var2, is.matrix, id_eset,
                              id_pdata, scale) {
  if (is.null(pdata)) {
    if (is.matrix) {
      if (scale) {
        eset <- scale_matrix(
          matrix = eset,
          log2matrix = FALSE,
          manipulate = FALSE
        )
      }
      data <- as.data.frame(t(eset))
      data$ID <- rownames(data)
    } else {
      data <- as.data.frame(eset)
      if (id_eset %in% colnames(data)) {
        colnames(data)[colnames(data) == id_eset] <- "ID"
      }
      if (scale) {
        numeric_cols <- sapply(data, is.numeric)
        data[, numeric_cols] <- scale(data[, numeric_cols])
      }
    }
  } else {
    # Merge pdata with eset
    pdata <- as.data.frame(pdata)
    colnames(pdata)[colnames(pdata) == id_pdata] <- "ID"

    if (is.matrix) {
      feas <- c(var1, var2)
      feas <- feas[feas %in% rownames(eset)]
      if (length(feas) == 0) {
        cli::cli_abort("No specified variables found in eset.")
      }
      data <- combine_pd_eset(
        eset = eset,
        pdata = pdata,
        id_pdata = "ID",
        feas = feas,
        scale = scale
      )
    } else {
      eset <- as.data.frame(eset)
      if (id_eset %in% colnames(eset)) {
        colnames(eset)[colnames(eset) == id_eset] <- "ID"
      }
      data <- merge(pdata, eset, by = "ID", all = FALSE)
    }
  }

  data
}

# Helper: Build correlation plot
.build_cor_plot <- function(data, var1, var2, subtype, na.subtype.rm,
                            color_subtype, col_line, point_size, alpha,
                            title, cor_result, pvalue, id, show_label) {
  # Base aesthetics
  if (!is.null(subtype)) {
    if (!subtype %in% colnames(data)) {
      cli::cli_warn("Subtype {.val {subtype}} not found. Ignoring.")
      subtype <- NULL
    }
  }

  if (!is.null(subtype)) {
    colnames(data)[colnames(data) == subtype] <- "categorys"

    if (na.subtype.rm) {
      data <- data[!is.na(data$categorys), , drop = FALSE]
    }

    data$categorys <- as.character(data$categorys)
    data$categorys[is.na(data$categorys)] <- "Not_available"
    data$categorys <- as.factor(data$categorys)

    cli::cli_alert_info("Groups: {.val {levels(data$categorys)}}")

    p <- ggplot2::ggplot(
      data,
      ggplot2::aes(
        x = .data[[var1]],
        y = .data[[var2]],
        colour = .data$categorys
      )
    ) +
      ggplot2::scale_color_manual(values = color_subtype) +
      ggplot2::geom_point(size = point_size, alpha = alpha)
  } else {
    p <- ggplot2::ggplot(
      data,
      ggplot2::aes(
        x = .data[[var1]],
        y = .data[[var2]]
      )
    ) +
      ggplot2::geom_point(
        size = point_size,
        alpha = alpha,
        colour = "black"
      )
  }

  # Add regression line and labels
  p <- p +
    ggplot2::geom_smooth(method = "lm", se = TRUE, color = col_line) +
    ggplot2::labs(
      x = var1,
      y = var2,
      title = title,
      subtitle = paste0(
        "r = ", round(as.numeric(cor_result$estimate), 3),
        ", P = ", format(pvalue, digits = 1, scientific = TRUE)
      )
    )

  # Add labels if requested
  if (show_label) {
    if (is.null(id) || !id %in% colnames(data)) {
      cli::cli_warn("Label column not found. Set {.arg id} to show labels.")
    } else {
      p <- p + ggplot2::geom_text(
        ggplot2::aes(label = .data[[id]]),
        nudge_x = 0.25,
        nudge_y = 0.25,
        check_overlap = TRUE,
        size = 3
      )
    }
  }

  p
}

# Helper: Save correlation plot
.save_cor_plot <- function(p, path, var1, var2, index, fig.format,
                           fig.width, fig.height, data) {
  if (is.null(path)) {
    cli::cli_warn("{.arg path} is NULL; plot will not be saved.")
    return(invisible(NULL))
  }

  ff <- creat_folder(path)

  filename <- paste0(index, "-", var2, "-", var1, "-correlation.", fig.format)

  ggplot2::ggsave(
    filename = filename,
    plot = p,
    width = fig.width,
    height = fig.height,
    path = ff$folder_name
  )

  save(data, file = paste0(ff$abspath, "0-input-data-", var1, "-", var2, ".RData"))

  cli::cli_alert_success("Plot saved to {.path {ff$folder_name}}")
}

#' Helper Functions for quanTIseq
#'
#' @description
#' Helper functions for quanTIseq deconvolution method.
#' Source code adapted from https://github.com/FFinotello/quanTIseq.
#'
#' @name quantiseq_helper
#'
#' @return Internal functions return processed data for quanTIseq deconvolution.
#'
#' @keywords internal
NULL

#' Fix Expression Mixture Matrix
#'
#' @description
#' Processes expression matrix by mapping genes, converting log values,
#' quantile normalization (if arrays), and TPM normalization.
#'
#' @param mix.mat Expression matrix with genes as rows.
#' @param arrays Logical indicating if data is from arrays. Default is FALSE.
#'
#' @return Processed expression matrix.
#'
#' @keywords internal
fixMixture <- function(mix.mat, arrays = FALSE) {
  # Map gene names
  mix.mat <- mapGenes(mix.mat)
  if (is.null(mix.mat)) return(NULL)

  # Un-log data in log2 base
  if (max(mix.mat) < 50) {
    mix.mat <- 2^mix.mat
  }

  # Quantile normalization
  if (arrays) mix.mat <- makeQN(mix.mat)

  # TPM normalization
  mix.mat <- t(t(mix.mat) * 1e6 / apply(mix.mat, 2, sum))

  mix.mat
}

#' Quantile Normalization
#'
#' @param mix.mat Expression matrix.
#'
#' @return Quantile normalized matrix.
#'
#' @keywords internal
makeQN <- function(mix.mat) {
  cnames <- colnames(mix.mat)
  rnames <- rownames(mix.mat)
  rlang::check_installed("preprocessCore")
  mix.mat <- preprocessCore::normalize.quantiles(as.matrix(mix.mat))
  colnames(mix.mat) <- cnames
  rownames(mix.mat) <- rnames
  mix.mat
}

#' Map Gene Names to Approved Symbols
#'
#' @description
#' Maps gene symbols to approved HGNC symbols, handling withdrawn and
#' previous symbols.
#'
#' @param mydata Expression matrix with gene symbols as rownames.
#'
#' @return Matrix with mapped gene symbols.
#'
#' @keywords internal
mapGenes <- function(mydata) {
  quantiseq_ref <- load_data("quantiseq_data")
  if (is.null(quantiseq_ref)) return(NULL)
  HGNC <- quantiseq_ref$HGNC_genenames_20170418

  curgenes <- rownames(mydata)
  newgenes <- rep(NA, length(curgenes))
  newgenes2 <- rep(NA, length(curgenes))
  ind <- match(curgenes, HGNC$ApprovedSymbol)

  # Current symbols and withdrawn ones
  genes.ind.notNA <- which(!is.na(ind))
  for (i in genes.ind.notNA) {
    if (HGNC$Status[ind[i]] == "Approved") {
      newgenes[i] <- curgenes[i]
    } else if (HGNC$Status[ind[i]] == "EntryWithdrawn") {
      next
    } else {
      Wstring <- "symbolwithdrawn,see"
      newsymbol <- gsub(Wstring, "", HGNC$ApprovedName[ind[i]])
      newgenes2[i] <- newsymbol
    }
  }

  # Not found as symbols - check previous symbols and synonyms
  genes.ind.NA <- which(is.na(ind))
  for (i in genes.ind.NA) {
    genei <- curgenes[i]

    # Previous symbol?
    ind1 <- grep(genei, HGNC$PreviousSymbols)
    for (i1 in ind1) {
      array1 <- unlist(strsplit(as.character(HGNC$PreviousSymbols[i1]), ","))
      if (any(array1 == genei)) {
        newgenes2[i] <- as.character(HGNC$ApprovedSymbol[i1])
      }
    }
    # Synonym?
    ind2 <- grep(genei, HGNC$Synonyms)
    for (i2 in ind2) {
      array2 <- unlist(strsplit(as.character(HGNC$Synonyms[i2]), ","))
      if (any(array2 == genei)) {
        newgenes2[i] <- as.character(HGNC$ApprovedSymbol[i2])
      }
    }
  }

  newgenes2[which(newgenes2 %in% setdiff(newgenes, NA))] <- NA
  ind <- intersect(which(is.na(newgenes)), which(!is.na(newgenes2)))
  newgenes[ind] <- newgenes2[ind]

  mydata <- mydata[which(!is.na(newgenes)), , drop = FALSE]
  newgenes <- newgenes[which(!is.na(newgenes))]

  # Take the median if duplicates are present
  outdata <- stats::aggregate(mydata, by = list(newgenes), FUN = stats::median)
  rownames(outdata) <- outdata[, 1]
  outdata <- outdata[, -1, drop = FALSE]
  outdata <- as.data.frame(outdata)

  outdata
}

#' Run quanTIseq Deconvolution
#'
#' @param currsig Signature matrix.
#' @param currmix Mixture matrix.
#' @param scaling Scaling factors.
#' @param method Deconvolution method: "lsei", "hampel", "huber", or "bisquare".
#'
#' @return Deconvolution results matrix.
#'
#' @keywords internal
quanTIseq <- function(currsig, currmix, scaling, method) {
  method <- rlang::arg_match(method, c("lsei", "hampel", "huber", "bisquare"))

  cgenes <- intersect(rownames(currsig), rownames(currmix))
  currsig <- as.matrix(currsig[cgenes, , drop = FALSE])
  currmix <- as.matrix(currmix[cgenes, , drop = FALSE])

  if (method == "lsei") {
    # Run deconvolution with constrained least squares
    G <- diag(ncol(currsig))
    G <- rbind(G, rep(-1, ncol(G)))
    H <- c(rep(0, ncol(currsig)), -1)

    results <- apply(currmix, 2, DClsei,
      A = currsig, G = G, H = H,
      scaling = scaling
    )
  } else {
    # Run deconvolution with robust regression
    results <- apply(currmix, 2, DCrr,
      A = currsig,
      method = method, scaling = scaling
    )
  }

  t(results)
}

#' Constrained Least Squares Deconvolution
#'
#' @param b Observation vector.
#' @param A Signature matrix.
#' @param G Constraint matrix.
#' @param H Constraint vector.
#' @param scaling Scaling factor.
#'
#' @return Estimated cell fractions.
#'
#' @keywords internal
DClsei <- function(b, A, G, H, scaling) {
  rlang::check_installed("limSolve")
  sc <- norm(A, "2")
  A <- A / sc
  b <- b / sc

  res <- limSolve::lsei(
    A = A,
    B = b,
    G = G,
    H = H,
    verbose = FALSE
  )
  est <- res$X

  est.sum <- sum(est)
  est <- est / scaling
  est <- est / sum(est) * est.sum
  est <- c(est, pmax(0, 1 - sum(est)))
  names(est)[length(est)] <- "Other"

  est
}

#' Robust Regression Deconvolution
#'
#' @param b Observation vector.
#' @param A Signature matrix.
#' @param method Robust regression method.
#' @param scaling Scaling factor.
#'
#' @return Estimated cell fractions.
#'
#' @keywords internal
DCrr <- function(b, A, method, scaling) {
  rlang::check_installed("MASS")
  m <- paste0("psi.", method)

  if (m == "psi.hampel") {
    bres <- MASS::rlm(b ~ A, psi = m, a = 1.5, b = 3.5, c = 8, maxit = 1e3)
  } else {
    bres <- MASS::rlm(b ~ A, psi = m, maxit = 1e3)
  }
  est <- bres$coefficients

  # Remove intercept
  est <- est[-1]

  # Set negative values to 0 and normalize
  est[est < 0] <- 0
  est <- est / sum(est)

  # Scale by mRNA content
  est.sum <- sum(est)
  est <- est / scaling
  est <- est / sum(est) * est.sum

  names(est) <- gsub("^A", "", names(est))

  est
}

---
title: "Multidimensional Decoding Tumor Microenvironment for Immuno-Oncology Research"
author: 
- name: Dongqiang Zeng, Yiran Fang
  email: interlaken@smu.edu.com, fyr_nate@163.com
  affiliation: Department of Oncology, Nanfang Hospital, Southern Medical University, Guangzhou, Guangdong, P. R. China
date: "`r Sys.Date()`"
output:
  prettydoc::html_pretty:
    theme: cayman
    highlight: github
  pdf_document:
    toc: true
vignette: >
  %\VignetteIndexEntry{IOBR User Manual}
  %\VignetteEngine{knitr::rmarkdown}
  %\usepackage[utf8]{inputenc}
  %\VignetteEncoding{UTF-8}
---

```{r style, echo=FALSE, results="asis", message=FALSE}
knitr::opts_chunk$set(
  tidy = FALSE,
  warning = FALSE,
  message = FALSE
)
```

# Overview

IOBR is the acronym for [Immuno-Oncology Biological Research](https://github.com/IOBR/IOBR)  to perform multi-omics immuno-oncology biological research to decipher tumour microenvironment and signatures for clinical translation.

# Module Introduction

![](Figure_1.jpg)

IOBR encompasses six functional modules: 


+ __*Transcriptome data prepare module*__ : pre-procession of transcriptome data, as well as pertinent batch statistical analyses; 
+ __*TME deconvolution and signature estimation module*__ : estimation of signature scores and identification of phenotype relevant signatures, along with decoding immune contexture; 
+ __*TME interaction module*__ : clustering TME characteristics and analyzing receptor-ligand interactions; 
+ __*Genome and TME interaction module*__ : analysis of signature associated mutations; 
+ __*TME data visualization and Statistical analysis module*__ : visual representation and statistical examination of TME data; 
+ __*TME modeling module*__ : fast model construction and the assessment of model performance.


# Methodology

IOBR integrates eight open-source deconvolution methodologies: __*CIBERSORT*__ , __*ESTIMATE*__ , __*quanTIseq*__ , __*TIMER*__ , __*IPS*__ , __*MCPCounter*__ , __*xCell*__ and __*EPIC*__ . In addition, 323 published signature gene sets have been collected by IOBR covering __*tumour microenvironment*__ , __*tumour metabolism*__ , __*m6A*__ , __*exosomes*__ , __*microsatellite instability*__ and __*tertiary lymphoid structure*__. IOBR has used three computational methods to calculate the signature score, including __*PCA*__ , __*z-score*__ and __*ssGSEA*__.

# Visualization

IOBR integrates visualization function, including __*boxplots*__ , __*heatmaps*__ , __*percentage bar charts*__ , __*scatter plots*__ , __*KM plot*__ , __*PCA plot*__ etc.

# Tutorial

Please go to <https://iobr.github.io/book/> for the full tutorial.

# Citation

If you use [IOBR](https://github.com/IOBR/IOBR) in published research, please cite:


__*Zeng D*__, Fang Y, …, Liao W (2024) IOBR2: Multidimensional Decoding of Tumor Microenvironment for Immuno-Oncology Research. **_Cell Reports Methods_**_. 4(9):100910.
doi:[10.1016/j.crmeth.2024.100910](https://doi.org/10.1016/j.crmeth.2024.100910)


__*Zeng D*__, Ye Z, Shen R, Yu G, Wu J, Xiong Y,…, Liao W (2021) IOBR: Multi-Omics Immuno-Oncology Biological Research to Decode Tumor Microenvironment and Signatures. **_Frontiers in Immunology_**. 12:687975.
doi:[10.3389/fimmu.2021.687975](https://www.frontiersin.org/journals/immunology/articles/10.3389/fimmu.2021.687975/full)



# Feedback and helps

Should any queries or concerns arise, consider checking the [IOBR primary webpage](https://github.com/IOBR/IOBR) initially. The majority of your issues are likely already addressed there. Supposing you've detected a fault, adhere to the instructions, and offer a replicable instance to be showcased on the [github issue tracker](https://github.com/IOBR/IOBR/issues).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BinomialModel.R
\name{Enet}
\alias{Enet}
\title{Elastic Net Model Fitting}
\usage{
Enet(train.x, train.y, lambdamax, nfold = 10)
}
\arguments{
\item{train.x}{Training predictors matrix.}

\item{train.y}{Training binary outcomes (0/1 or factor).}

\item{lambdamax}{Maximum lambda value for the grid search.}

\item{nfold}{Number of CV folds. Default is `10`.}
}
\value{
List containing:
\describe{
  \item{chose_alpha}{Optimal alpha value (0-1)}
  \item{chose_lambda}{Optimal lambda value}
}
}
\description{
Fits elastic net model with cross-validation to find optimal alpha and lambda.
Searches across a grid of alpha values (0 to 1) and lambda values to minimize
cross-validation error.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  set.seed(123)
  train_data <- matrix(rnorm(50 * 5), ncol = 5)
  train_outcome <- rbinom(50, 1, 0.5)
  result <- Enet(train.x = train_data, train.y = train_outcome, lambdamax = 1, nfold = 5)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xCell.R
\name{spillOver}
\alias{spillOver}
\title{Adjust Scores Using Spillover Compensation}
\usage{
spillOver(transformedScores, K, alpha = 0.5, file.name = NULL)
}
\arguments{
\item{transformedScores}{Transformed scores from transformScores.}

\item{K}{Spillover matrix.}

\item{alpha}{Spillover alpha parameter. Default is `0.5`.}

\item{file.name}{Character string for saving scores. Default is `NULL`.}
}
\value{
Adjusted xCell scores matrix.
}
\description{
Adjusts xCell scores using spillover compensation matrix.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/best_cutoff.R
\name{best_cutoff}
\alias{best_cutoff}
\title{Extract Best Cutoff and Add Binary Variable to Data Frame}
\usage{
best_cutoff(
  pdata,
  variable,
  time = "time",
  status = "status",
  print_result = TRUE
)
}
\arguments{
\item{pdata}{Data frame containing survival information and the continuous
variable.}

\item{variable}{Character string specifying the name of the continuous
variable for which the optimal cutoff should be determined.}

\item{time}{Character string specifying the column name containing
time-to-event data. Default is `"time"`.}

\item{status}{Character string specifying the column name containing event
status (censoring information). Default is `"status"`.}

\item{print_result}{Logical indicating whether to print detailed results
including cutoff value and Cox model summaries. Default is `TRUE`.}
}
\value{
Data frame identical to `pdata` with an additional binary column
  named `<variable>_binary` containing "High" and "Low" categories based on
  the optimal cutoff.
}
\description{
Determines the optimal cutoff point for a continuous variable in survival
analysis using the maximally selected rank statistics method. Creates a
binary variable based on the identified cutoff and adds it to the input data
frame.
}
\examples{
set.seed(123)
pdata <- data.frame(
  time = rexp(100),
  status = rbinom(100, 1, 0.5),
  score = rnorm(100, mean = 50, sd = 10)
)
result <- best_cutoff(pdata, variable = "score", print_result = FALSE)
table(result$score_binary)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PrognosticModel.R
\name{PrognosticResult}
\alias{PrognosticResult}
\title{Compute Prognostic Results for Survival Models}
\usage{
PrognosticResult(model, train.x, train.y, test.x, test.y)
}
\arguments{
\item{model}{A fitted survival model object (e.g., from `glmnet::cv.glmnet`).}

\item{train.x}{Matrix or data frame of training predictors.}

\item{train.y}{Training dataset survival outcomes (time and status).}

\item{test.x}{Matrix or data frame of testing predictors.}

\item{test.y}{Testing dataset survival outcomes (time and status).}
}
\value{
A list containing:
\describe{
  \item{model}{The fitted model object}
  \item{coefs}{Data frame of coefficients at `lambda.min` and `lambda.1se`}
  \item{AUC}{Data frame with AUC values for train/test at both lambda values}
}
}
\description{
Computes and compiles prognostic results from a survival model fitted with `glmnet`.
Extracts model coefficients at optimal lambda values (`lambda.min` and `lambda.1se`)
and calculates time-dependent AUC metrics for both training and testing datasets.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE) &&
  requireNamespace("survival", quietly = TRUE) &&
  requireNamespace("timeROC", quietly = TRUE)) {
  library(survival)
  set.seed(123)
  train_x <- matrix(rnorm(100 * 10), ncol = 10)
  train_y <- data.frame(time = rexp(100), status = rbinom(100, 1, 0.5))
  test_x <- matrix(rnorm(50 * 10), ncol = 10)
  test_y <- data.frame(time = rexp(50), status = rbinom(50, 1, 0.5))
  fit <- glmnet::cv.glmnet(train_x, Surv(train_y$time, train_y$status), family = "cox")
  results <- PrognosticResult(
    model = fit, train.x = train_x, train.y = train_y,
    test.x = test_x, test.y = test_y
  )
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/percent_bar.R
\name{pie_chart}
\alias{pie_chart}
\title{Create Pie or Donut Charts}
\usage{
pie_chart(
  input,
  var,
  var2 = NULL,
  type = 2,
  show_freq = FALSE,
  color = NULL,
  palette = "jama",
  title = NULL,
  text_size = 10,
  title_size = 20,
  add_sum = FALSE
)
}
\arguments{
\item{input}{Input dataframe.}

\item{var}{Variable for the chart.}

\item{var2}{Secondary variable for donut chart (type = 3).}

\item{type}{Chart type: 1 (pie), 2 (donut), 3 (PieDonut via webr).}

\item{show_freq}{Logical to show frequencies. Default is FALSE.}

\item{color}{Optional color palette.}

\item{palette}{Color palette name. Default is "jama".}

\item{title}{Plot title. Default is NULL.}

\item{text_size}{Text size. Default is 10.}

\item{title_size}{Title size. Default is 20.}

\item{add_sum}{Logical to add sum to labels. Default is FALSE.}
}
\value{
A ggplot object.
}
\description{
Generates a pie chart or donut chart from input data.
}
\examples{
# Simulate data
set.seed(123)
sim_data <- data.frame(
  Subtype = sample(c("EBV", "GS", "MSI", "CIN"), 100, replace = TRUE)
)

# Create pie chart
p1 <- pie_chart(input = sim_data, var = "Subtype", palette = "jama")
if (!is.null(p1)) print(p1)

# Create donut chart
p2 <- pie_chart(input = sim_data, var = "Subtype", type = 2)
if (!is.null(p2)) print(p2)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateRef_limma.R
\name{Construct_con}
\alias{Construct_con}
\title{Construct Contrast Matrix}
\usage{
Construct_con(pheno)
}
\arguments{
\item{pheno}{Factor with different levels representing groups to contrast.}
}
\value{
Square matrix with dimensions equal to the number of levels in
  `pheno`. Each row represents a contrast where the corresponding level
  is compared against the average of others.
}
\description{
Creates a contrast matrix for differential analysis, where each phenotype
level is contrasted against all other levels combined.
}
\examples{
pheno <- factor(c("A", "B", "C", "D"))
contrast_matrix <- Construct_con(pheno)
print(contrast_matrix)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EPIC_helper.R
\name{merge_duplicates}
\alias{merge_duplicates}
\title{Merging the duplicates from the input matrix.}
\usage{
merge_duplicates(mat, warn = TRUE, in_type = NULL)
}
\arguments{
\item{mat}{matrix}

\item{warn}{warning}

\item{in_type}{default is null}
}
\description{
In case there are some duplicate rownames in the input matrix (\emph{mat}),
this function will return a similar matrix with unique rownames and the
duplicate cases will be merged together based on the median values. When
\emph{warn} is true a warning message will be written in case there are
duplicates, this warning message will include the \emph{in_type} string to
indicate the current type of the input matrix.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assimilate_data.R
\name{assimilate_data}
\alias{assimilate_data}
\title{Harmonize Two Data Frames by Column Structure}
\usage{
assimilate_data(data_a, data_b)
}
\arguments{
\item{data_a}{Data frame. Reference data frame whose column structure
should be matched.}

\item{data_b}{Data frame. Data frame to be conformed to `data_a`.}
}
\value{
Data frame `data_b` with added missing columns (NA-filled) and
  reordered to match `data_a`.
}
\description{
Adds missing columns (filled with `NA`) to a secondary data frame so that
its column set and order match a reference data frame. This is useful when
combining data frames from different sources that should have the same
structure but may be missing some columns.
}
\examples{
# Create reference data frame
pdata_a <- data.frame(
  A = 1:5, B = 2:6, C = 3:7, D = 4:8, E = 5:9
)

# Create data frame with subset of columns
pdata_b <- data.frame(A = 1:3, C = 4:6, E = 7:9)

# Harmonize pdata_b to match pdata_a structure
pdata_b_harmonized <- assimilate_data(data_a = pdata_a, data_b = pdata_b)
print(names(pdata_b_harmonized)) # Now has A, B, C, D, E
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BinomialModel.R
\name{BinomialModel}
\alias{BinomialModel}
\title{Binomial Model Construction}
\usage{
BinomialModel(
  x,
  y,
  seed = 123456,
  scale = TRUE,
  train_ratio = 0.7,
  nfold = 10,
  plot = TRUE,
  palette = "jama",
  cols = NULL
)
}
\arguments{
\item{x}{A data frame containing sample ID and features. First column must be sample ID.}

\item{y}{A data frame where first column is sample ID and second column is outcome
(numeric or factor).}

\item{seed}{Integer for random seed. Default is `123456`.}

\item{scale}{Logical indicating whether to scale features. Default is `TRUE`.}

\item{train_ratio}{Numeric between 0 and 1 for training proportion. Default is `0.7`.}

\item{nfold}{Integer for cross-validation folds. Default is `10`.}

\item{plot}{Logical indicating whether to generate AUC plots. Default is `TRUE`.}

\item{palette}{Character string for color palette. Default is `"jama"`.}

\item{cols}{Optional color vector for ROC curves. Default is `NULL`.}
}
\value{
List containing:
\describe{
  \item{lasso_result}{Lasso model results}
  \item{ridge_result}{Ridge model results}
  \item{train.x}{Training data with IDs}
}
}
\description{
Constructs and evaluates binomial logistic regression models using Lasso and Ridge
regularization. Processes input data, scales features if specified, splits data into
training/testing sets, and fits both Lasso and Ridge models. Optionally generates
AUC plots for model evaluation.
}
\examples{
set.seed(123)
x <- data.frame(
  ID = paste0("Sample", 1:50),
  Feature1 = rnorm(50),
  Feature2 = rnorm(50),
  Feature3 = rnorm(50)
)
y <- data.frame(
  ID = x$ID,
  Outcome = factor(rbinom(50, 1, 0.5))
)
result <- BinomialModel(x = x, y = y, plot = FALSE, nfold = 5)
str(result, max.level = 1)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_method.R
\name{select_method}
\alias{select_method}
\title{Select a Signature Scoring Method Subset}
\usage{
select_method(data, method = c("ssGSEA", "PCA", "zscore"))
}
\arguments{
\item{data}{Data frame or matrix. Integrated signature score matrix.}

\item{method}{Character. One of "PCA", "ssGSEA", or "zscore" (case-insensitive).
Default is "ssGSEA".}
}
\value{
Matrix or data frame containing only the selected method's scores.
}
\description{
Filters an integrated signature score matrix to retain results from a
specified method (PCA, ssGSEA, or zscore) and strips method suffixes from
column names.
}
\examples{
# Simulate data with multiple method columns
set.seed(123)
sim_res <- data.frame(
  ID = paste0("Sample", 1:10),
  Sig1_PCA = rnorm(10),
  Sig1_ssGSEA = rnorm(10),
  Sig1_zscore = rnorm(10),
  Sig2_PCA = rnorm(10),
  Sig2_ssGSEA = rnorm(10),
  Sig2_zscore = rnorm(10)
)
# Select PCA method columns only
pca_res <- select_method(sim_res, method = "PCA")
print(colnames(pca_res))
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cell_bar_plot.R
\name{cell_bar_plot}
\alias{cell_bar_plot}
\title{Visualize Cell Fractions as Stacked Bar Chart}
\usage{
cell_bar_plot(
  input,
  id = "ID",
  title = "Cell Fraction",
  features = NULL,
  pattern = NULL,
  legend.position = "bottom",
  coord_flip = TRUE,
  palette = 3,
  show_col = FALSE,
  cols = NULL
)
}
\arguments{
\item{input}{Data frame containing deconvolution results.}

\item{id}{Character string specifying the column name containing sample
identifiers. Default is "ID".}

\item{title}{Character string specifying the plot title.
Default is "Cell Fraction".}

\item{features}{Character vector specifying column names representing cell
types to plot. If NULL, columns are selected based on `pattern`.
Default is NULL.}

\item{pattern}{Character string or regular expression to match column names
for automatic feature selection. Used when `features` is NULL.
Default is NULL.}

\item{legend.position}{Character string specifying legend position
("bottom", "top", "left", "right"). Default is "bottom".}

\item{coord_flip}{Logical indicating whether to flip plot coordinates using
`coord_flip()`. Default is TRUE.}

\item{palette}{Integer specifying the color palette to use. Default is 3.}

\item{show_col}{Logical indicating whether to display color information.
Default is FALSE.}

\item{cols}{Character vector of custom colors. If NULL, palette is used.
Default is NULL.}
}
\value{
A ggplot2 object representing the stacked bar chart.
}
\description{
Creates stacked bar charts to visualize tumor microenvironment (TME) cell
fractions. Supports batch visualization of deconvolution results from
methods such as CIBERSORT, EPIC, and quanTIseq.
}
\examples{
set.seed(123)
input_data <- data.frame(
  ID = paste0("Sample", 1:10),
  Cell_A = runif(10, 0, 0.4),
  Cell_B = runif(10, 0, 0.3),
  Cell_C = runif(10, 0, 0.3)
)
cell_bar_plot(input = input_data, id = "ID", features = c("Cell_A", "Cell_B", "Cell_C"))
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_doc.R
\docType{data}
\name{stad_group}
\alias{stad_group}
\title{Example Clinical Data for TCGA-STAD Gastric Cancer Analysis}
\format{
A data frame with patient samples as rows and clinical variables as columns:
\describe{
  \item{ID}{Unique patient identifier (TCGA barcode format)}
  \item{stage}{AJCC pathological stage (Stage_II, Stage_III, Stage_IV)}
  \item{status}{Patient vital status (Alive, Dead, NA)}
  \item{Lauren}{Lauren histological classification (Intestinal, Diffuse,
  Mixed)}
  \item{subtype}{Molecular subtype classification (EBV, GS)}
  \item{EBV}{Epstein-Barr virus status (Positive, Negative)}
  \item{time}{Overall survival time in months}
  \item{OS_status}{Overall survival status (0=alive, 1=dead)}
}
}
\usage{
data(stad_group)
}
\description{
A small example dataset demonstrating the clinical data structure required
for TCGA Stomach Adenocarcinoma (STAD) analysis using IOBR package functions.
Contains simulated clinical variables, molecular subtypes, and survival data.
}
\examples{
data(stad_group)
head(stad_group)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\name{deconvo_mcpcounter}
\alias{deconvo_mcpcounter}
\title{Deconvolve Immune Microenvironment Using MCP-Counter}
\usage{
deconvo_mcpcounter(eset, project = NULL)
}
\arguments{
\item{eset}{Gene expression matrix with HGNC symbols as row names.}

\item{project}{Optional project name. Default is `NULL`.}
}
\value{
Data frame with MCP-counter scores. Columns suffixed with
  `_MCPcounter`.
}
\description{
Estimates immune cell abundances using MCP-counter.
}
\examples{
mcp_genes <- load_data("mcp_genes")
if (!is.null(mcp_genes)) {
  set.seed(123)
  sim_eset <- matrix(rnorm(nrow(mcp_genes) * 3), nrow(mcp_genes), 3)
  rownames(sim_eset) <- mcp_genes$`HUGO symbols`
  colnames(sim_eset) <- paste0("Sample", 1:3)
  
  # Run deconvolution
  result <- deconvo_mcpcounter(eset = sim_eset, project = "TCGA-STAD")
  if (!is.null(result)) head(result)
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\name{deconvo_quantiseq}
\alias{deconvo_quantiseq}
\title{Deconvolve Using quanTIseq}
\usage{
deconvo_quantiseq(eset, project = NULL, tumor, arrays, scale_mrna)
}
\arguments{
\item{eset}{Gene expression matrix.}

\item{project}{Optional project name. Default is `NULL`.}

\item{tumor}{Logical: tumor samples. Must be specified.}

\item{arrays}{Logical: microarray data. Must be specified.}

\item{scale_mrna}{Logical: correct for mRNA content. Must be specified.}
}
\value{
Data frame with quanTIseq cell fractions. Columns suffixed with
  `_quantiseq`.
}
\description{
quanTIseq deconvolution for RNA-seq immune cell fractions.
}
\examples{
\dontrun{
quantiseq_data <- load_data("quantiseq_data")
if (!is.null(quantiseq_data)) {
  set.seed(123)
  n_sig <- nrow(quantiseq_data$TIL10_signature)
  sim_eset <- matrix(rnorm(n_sig * 2), n_sig, 2)
  rownames(sim_eset) <- rownames(quantiseq_data$TIL10_signature)
  colnames(sim_eset) <- paste0("Sample", 1:2)
  result <- deconvo_quantiseq(eset = sim_eset, project = "Example", tumor = TRUE,
                              arrays = FALSE, scale_mrna = FALSE)
  if (!is.null(result)) head(result)
}
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EPIC_helper.R
\name{scaleCounts}
\alias{scaleCounts}
\title{Scaling raw counts from each sample.}
\usage{
scaleCounts(counts, sigGenes = NULL, renormGenes = NULL, normFact = NULL)
}
\description{
Normalizing the sum of counts from each sample to 1e6.
}
\details{
Function taking a matrix (\emph{genes} x \emph{samples}) of counts as input
and returning the scaled counts for the subset signature genes (or all genes
if it sigGenes is \code{NULL}), with the scaled counts computed based on all
the 'renormGenes' (or all genes if it is NULL). The renormalization is made
independently for each sample, to have the sum of each columns over the
renormGenes equal to 1e6.

normFact, if not null, is used as the normalization factor instead of
 the colSums (used to renormalize the refProfiles.var by the same amount
 than the refProfiles).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantiseq_helper.R
\name{mapGenes}
\alias{mapGenes}
\title{Map Gene Names to Approved Symbols}
\usage{
mapGenes(mydata)
}
\arguments{
\item{mydata}{Expression matrix with gene symbols as rownames.}
}
\value{
Matrix with mapped gene symbols.
}
\description{
Maps gene symbols to approved HGNC symbols, handling withdrawn and
previous symbols.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantiseq_helper.R
\name{quanTIseq}
\alias{quanTIseq}
\title{Run quanTIseq Deconvolution}
\usage{
quanTIseq(currsig, currmix, scaling, method)
}
\arguments{
\item{currsig}{Signature matrix.}

\item{currmix}{Mixture matrix.}

\item{scaling}{Scaling factors.}

\item{method}{Deconvolution method: "lsei", "hampel", "huber", or "bisquare".}
}
\value{
Deconvolution results matrix.
}
\description{
Run quanTIseq Deconvolution
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\name{deconvo_ips}
\alias{deconvo_ips}
\title{Calculate Immunophenoscore (IPS)}
\usage{
deconvo_ips(eset, project = NULL, plot = FALSE)
}
\arguments{
\item{eset}{Gene expression matrix.}

\item{project}{Optional project name. Default is `NULL`.}

\item{plot}{Logical: generate visualization. Default is `FALSE`.}
}
\value{
Data frame with IPS scores. Columns suffixed with `_IPS`.
}
\description{
Calculates immune phenotype scores from gene expression data.
}
\examples{
\dontrun{
ips_genes <- load_data("ips_gene_set")
if (!is.null(ips_genes)) {
  set.seed(123)
  sim_eset <- matrix(rnorm(nrow(ips_genes) * 2), nrow(ips_genes), 2)
  rownames(sim_eset) <- ips_genes$GENE
  colnames(sim_eset) <- paste0("Sample", 1:2)
  result <- deconvo_ips(eset = sim_eset, project = "Example")
  if (!is.null(result)) head(result)
}
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\name{deconvo_xcell}
\alias{deconvo_xcell}
\title{Deconvolve Immune Microenvironment Using xCell}
\usage{
deconvo_xcell(eset, project = NULL, arrays = FALSE)
}
\arguments{
\item{eset}{Gene expression matrix with HGNC gene symbols as row names and
samples as columns.}

\item{project}{Optional project name to add as `ProjectID` column.
Default is `NULL`.}

\item{arrays}{Logical indicating microarray data (`TRUE`) or RNA-seq
(`FALSE`). Default is `FALSE`.}
}
\value{
Data frame with xCell enrichment scores. Cell type columns are
  suffixed with `_xCell`.
}
\description{
Estimates immune cell fractions using the xCell algorithm. xCell provides
cell type enrichment scores for 64 immune and stromal cell types from gene
expression data.
}
\examples{
\dontrun{
xcell <- load_data("xCell.data")
if (!is.null(xcell)) {
  set.seed(123)
  sim_eset <- matrix(rnorm(length(xcell$genes) * 2), length(xcell$genes), 2)
  rownames(sim_eset) <- xcell$genes
  colnames(sim_eset) <- paste0("Sample", 1:2)
  result <- deconvo_xcell(eset = sim_eset, project = "TCGA-STAD")
  if (!is.null(result)) head(result)
}
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/percent_bar.R
\name{percent_bar_plot}
\alias{percent_bar_plot}
\title{Create a Percent Bar Plot}
\usage{
percent_bar_plot(
  input,
  x,
  y,
  subset.x = NULL,
  color = NULL,
  palette = NULL,
  title = NULL,
  axis_angle = 0,
  coord_flip = FALSE,
  add_Freq = TRUE,
  Freq = "Proportion",
  size_freq = 8,
  legend.size = 0.5,
  legend.size.text = 10,
  add_sum = TRUE,
  print_result = TRUE,
  round.num = 2
)
}
\arguments{
\item{input}{Input data frame.}

\item{x}{Name of the x-axis variable.}

\item{y}{Name of the y-axis (grouping) variable.}

\item{subset.x}{Optional subset of x-axis values.}

\item{color}{Optional color palette.}

\item{palette}{Optional palette type.}

\item{title}{Optional plot title.}

\item{axis_angle}{Angle for axis labels (0-90). Default is 0.}

\item{coord_flip}{Logical to flip coordinates. Default is FALSE.}

\item{add_Freq}{Logical to add frequency count. Default is TRUE.}

\item{Freq}{Name of frequency column.}

\item{size_freq}{Size of frequency labels. Default is 8.}

\item{legend.size}{Size of legend. Default is 0.5.}

\item{legend.size.text}{Size of legend text. Default is 10.}

\item{add_sum}{Logical to add sum to x-axis labels. Default is TRUE.}

\item{print_result}{Logical to print result data frame. Default is TRUE.}

\item{round.num}{Decimal places for proportion. Default is 2.}
}
\value{
A ggplot object.
}
\description{
Generates a bar plot visualizing the percentage distribution of a variable
grouped by another variable.
}
\examples{
# Simulate data
set.seed(123)
sim_data <- data.frame(
  Subtype = sample(c("EBV", "GS", "MSI", "CIN"), 100, replace = TRUE),
  Lauren = sample(c("Diffuse", "Intestinal", "Mixed"), 100, replace = TRUE)
)

# Create percent bar plot
p <- percent_bar_plot(
  input = sim_data, x = "Subtype", y = "Lauren",
  axis_angle = 60
)
if (!is.null(p)) print(p)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_helper.R
\name{plotPurity}
\alias{plotPurity}
\title{plotPurity}
\usage{
plotPurity(
  scores,
  samples = "all_samples",
  platform = c("affymetrix", "agilent", "illumina"),
  output.dir = NULL
)
}
\arguments{
\item{scores}{A character string specifying the path to the input file containing ESTIMATE scores. The file should be a tab-separated table with appropriate headers.}

\item{samples}{A character vector specifying the sample names to plot. The default is "all_samples", which plots all samples in the input file.}

\item{platform}{A character string specifying the platform used for data collection. Can be "affymetrix", "agilent", or "illumina". Currently, only "affymetrix" is implemented.}

\item{output.dir}{A character string specifying the directory to save the output plots. If `NULL`, plots are not saved. Default is `NULL`.}
}
\value{
No return value. The function generates and saves scatterplots in the specified output directory.
}
\description{
This function generates scatterplots of tumor purity based on ESTIMATE
scores for given samples.
}
\examples{
# Create a sample ESTIMATE score matrix
scores_data <- data.frame(
  Sample1 = c(100, 200, 500, 0.80),
  Sample2 = c(120, 220, 450, 0.70),
  Sample3 = c(140, 240, 600, 0.90),
  row.names = c(
    "StromalScore", "ImmuneScore", "ESTIMATEScore",
    "TumorPurity"
  ),
  check.names = FALSE
)

# Write to a temporary GCT file
scores_file <- tempfile(fileext = ".gct")
outputGCT(scores_data, scores_file)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_eset.R
\name{check_eset}
\alias{check_eset}
\title{Check Integrity and Outliers of Expression Set}
\usage{
check_eset(eset, print_result = FALSE, estimate_sd = FALSE)
}
\arguments{
\item{eset}{Expression matrix or data frame with genes/features in rows and
samples in columns.}

\item{print_result}{Logical indicating whether to print detailed check
results to the console. Default is `FALSE`.}

\item{estimate_sd}{Logical indicating whether to check for features with
zero standard deviation. Default is `FALSE`.}
}
\value{
Invisibly returns `NULL`. Function is called for its side effects
  (printing messages and issuing warnings).
}
\description{
Performs quality checks on an expression matrix to identify missing values,
infinite values, and features with zero variance. Issues warnings when
potential problems are detected that may affect downstream analyses.
}
\examples{
set.seed(123)
eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
rownames(eset) <- paste0("Gene", 1:100)
colnames(eset) <- paste0("Sample", 1:10)
check_eset(eset, print_result = TRUE)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_wilcoxon.R
\name{batch_wilcoxon}
\alias{batch_wilcoxon}
\title{Batch Wilcoxon Rank-Sum Test Between Two Groups}
\usage{
batch_wilcoxon(
  data,
  target = "group",
  feature = NULL,
  feature_manipulation = FALSE
)
}
\arguments{
\item{data}{Data frame containing the dataset for analysis.}

\item{target}{Character string specifying the column name of the grouping
variable. Default is `"group"`.}

\item{feature}{Character vector specifying feature names to analyze. If
`NULL`, prompts for selection (interactive mode only). Default is `NULL`.}

\item{feature_manipulation}{Logical indicating whether to apply feature
manipulation filtering. Default is `FALSE`.}
}
\value{
Tibble with columns:
\describe{
  \item{sig_names}{Feature name}
  \item{p.value}{Raw p-value}
  \item{statistic}{Difference in means between groups}
  \item{p.adj}{Adjusted p-value (Benjamini-Hochberg)}
  \item{log10pvalue}{Negative log10-transformed p-value}
  \item{stars}{Significance stars: **** p<0.0001, *** p<0.001,
    ** p<0.01, * p<0.05, + p<0.5}
  \item{group1, group2}{Mean values for each group}
}
}
\description{
Performs Wilcoxon rank-sum tests (Mann-Whitney U tests) to compare the
distribution of specified features between two groups. Computes p-values,
adjusts for multiple testing, and ranks features by significance.
}
\examples{
# Create small example data
set.seed(123)
test_data <- data.frame(
  Gender = rep(c("Male", "Female"), each = 50),
  Signature1 = rnorm(100),
  Signature2 = rnorm(100)
)
# Compare features by gender
res <- batch_wilcoxon(
  data = test_data,
  target = "Gender",
  feature = c("Signature1", "Signature2")
)
head(res)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcpcounter.R
\name{MCPcounter.estimate}
\alias{MCPcounter.estimate}
\title{MCP-counter Cell Population Abundance Estimation}
\usage{
MCPcounter.estimate(
  expression,
  featuresType = c("affy133P2_probesets", "HUGO_symbols", "ENTREZ_ID", "ENSEMBL_ID"),
  probesets = read.table(system.file("extdata/probesets.txt", package = "IOBR"), sep =
    "\\t", stringsAsFactors = FALSE, colClasses = "character"),
  genes = read.table(system.file("extdata/genes.txt", package = "IOBR"), sep = "\\t",
    stringsAsFactors = FALSE, header = TRUE, colClasses = "character", check.names =
    FALSE)
)
}
\arguments{
\item{expression}{Matrix or data.frame with features in rows and samples
in columns.}

\item{featuresType}{Type of identifiers for expression features.
Options: "affy133P2_probesets", "HUGO_symbols", "ENTREZ_ID", "ENSEMBL_ID".
Default is "affy133P2_probesets".}

\item{probesets}{Probesets data table. Default loads from GitHub.}

\item{genes}{Genes data table. Default loads from GitHub.}
}
\value{
Matrix with cell populations in rows and samples in columns.
}
\description{
Estimates the abundance of different immune and stromal cell populations
using the MCP-counter method. Works with various gene identifiers including
Affymetrix probesets, HUGO gene symbols, Entrez IDs, and Ensembl IDs.
}
\examples{
expr <- matrix(runif(1000), nrow = 100, ncol = 10)
rownames(expr) <- paste0("Gene", 1:100)
estimates <- MCPcounter.estimate(expr, featuresType = "HUGO_symbols")

}
\author{
Etienne Becht
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tme_cluster.R
\name{tme_cluster}
\alias{tme_cluster}
\title{Identification of TME Cluster}
\usage{
tme_cluster(
  input,
  features = NULL,
  pattern = NULL,
  id = NULL,
  scale = TRUE,
  method = "kmeans",
  min_nc = 2,
  max.nc = 6
)
}
\arguments{
\item{input}{Data frame containing the input dataset.}

\item{features}{Vector of features to use for clustering. Default is NULL
(uses all columns or pattern-selected columns).}

\item{pattern}{Regular expression pattern for selecting features.
Default is NULL.}

\item{id}{Column name for identifiers. Default is NULL (uses row names).}

\item{scale}{Logical indicating whether to scale features. Default is TRUE.}

\item{method}{Clustering method. Default is "kmeans".}

\item{min_nc}{Minimum number of clusters to evaluate. Default is 2.}

\item{max.nc}{Maximum number of clusters to evaluate. Default is 6.}
}
\value{
Data frame with cluster assignments appended.
}
\description{
Performs TME (Tumor Microenvironment) clustering analysis using various
clustering methods. Supports feature selection, scaling, and automatic
determination of optimal cluster number.
}
\examples{
set.seed(123)
input_data <- data.frame(
  ID = paste0("Sample", 1:20),
  xCell_Tcells = rnorm(20),
  xCell_Bcells = rnorm(20),
  xCell_Macrophages = rnorm(20),
  Other_feature = rnorm(20)
)
result <- tme_cluster(
  input = input_data,
  pattern = "xCell",
  id = "ID",
  method = "kmeans"
)
table(result$cluster)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/log2eset.R
\name{log2eset}
\alias{log2eset}
\title{Log2 Transformation of Gene Expression Matrix}
\usage{
log2eset(eset)
}
\arguments{
\item{eset}{Numeric matrix. Gene expression data with genes as rows and
samples in columns.}
}
\value{
Numeric matrix. Log2-transformed gene expression data (if
  transformation was needed), or the original data otherwise.
}
\description{
Determines whether a gene expression matrix requires log2 transformation
based on the distribution of values, and applies it if necessary. This is
useful for automatically detecting raw counts or linear-scale data that
should be log-transformed for downstream analysis.
}
\examples{
set.seed(123)
eset <- matrix(rnorm(1000, mean = 10, sd = 2), nrow = 100, ncol = 10)
rownames(eset) <- paste0("Gene", 1:100)
colnames(eset) <- paste0("Sample", 1:10)
eset_transformed <- log2eset(eset)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_data.R
\name{add_iobr_mirror}
\alias{add_iobr_mirror}
\title{Add Custom Download Mirror}
\usage{
add_iobr_mirror(url, position = c("first", "last", "before_github"))
}
\arguments{
\item{url}{Character string. The mirror URL to add.}

\item{position}{Character. Where to add the mirror: "first", "last", or
"before_github". Default: "first".}
}
\value{
Invisibly returns the updated mirror list.
}
\description{
Adds a custom mirror URL to the default mirrors for the current session.
The mirror URL should be a base URL. If it contains 'github.com', it will
be treated as a GitHub proxy and the relative path to IOBR releases will
be appended. Otherwise, it will be treated as a direct repository path.
}
\examples{
\dontrun{
add_iobr_mirror("https://my-mirror.com/https://github.com")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sig_heatmap.R
\name{sig_heatmap}
\alias{sig_heatmap}
\title{Signature Heatmap with Optional Annotations}
\usage{
sig_heatmap(
  input,
  id = "ID",
  features,
  group,
  condition = NULL,
  id_condition = "vars",
  col_condition = "condition",
  cols_condition = NULL,
  scale = FALSE,
  palette = 2,
  cols_heatmap = NULL,
  palette_group = "jama",
  show_col = FALSE,
  show_palettes = FALSE,
  cols_group = NULL,
  show_plot = TRUE,
  width = 8,
  height = NULL,
  size_col = 10,
  size_row = 8,
  angle_col = 90,
  column_title = NULL,
  row_title = NULL,
  show_heatmap_col_name = FALSE,
  path = NULL,
  index = NULL
)
}
\arguments{
\item{input}{Data frame containing ID, grouping variable, and feature columns.}

\item{id}{Character string. Column name for sample identifier. Default is `"ID"`.}

\item{features}{Character vector. Feature (column) names to include in the heatmap.}

\item{group}{Character string. Grouping variable column name.}

\item{condition}{Data frame or `NULL`. Optional annotation table with
variable-condition mapping. Default is `NULL`.}

\item{id_condition}{Character string. Column name in `condition` for feature IDs.
Default is `"vars"`.}

\item{col_condition}{Character string. Column name in `condition` for condition
labels. Default is `"condition"`.}

\item{cols_condition}{Character vector. Colors for conditions.}

\item{scale}{Logical indicating whether to scale values by row. Default is `FALSE`.}

\item{palette}{Integer or character. Palette index/name for heatmap colors.
Default is `2`.}

\item{cols_heatmap}{Character vector. Custom colors for heatmap gradient.}

\item{palette_group}{Character string. Palette name for group colors.
Default is `"jama"`.}

\item{show_col}{Logical indicating whether to display the color vector.
Default is `FALSE`.}

\item{show_palettes}{Logical indicating whether to print palette options.
Default is `FALSE`.}

\item{cols_group}{Character vector. Custom colors for groups.}

\item{show_plot}{Logical indicating whether to print the heatmap. Default is `TRUE`.}

\item{width}{Numeric. Plot width in inches. Default is `8`.}

\item{height}{Numeric or `NULL`. Plot height in inches. Auto-calculated if `NULL`.}

\item{size_col}{Numeric. Font size for column labels. Default is `10`.}

\item{size_row}{Numeric. Font size for row labels. Default is `8`.}

\item{angle_col}{Numeric. Rotation angle for column labels in degrees.
Default is `90`.}

\item{column_title}{Character string or `NULL`. Title for column annotation.}

\item{row_title}{Character string or `NULL`. Title for row annotation.}

\item{show_heatmap_col_name}{Logical indicating whether to show column names.
Default is `FALSE`.}

\item{path}{Character string or `NULL`. Output directory for saving the heatmap.}

\item{index}{Integer or `NULL`. Index appended to filename. Default is `NULL`.}
}
\value{
A tidyHeatmap object. Saves PDF only when `path` is provided.
}
\description{
Generates a heatmap of selected features grouped by a categorical variable,
with optional conditional (annotation) bars. Supports palette customization,
scaling, size controls, and output saving.
}
\examples{
set.seed(123)
test_input <- data.frame(
  ID = paste0("Sample", 1:20),
  subtype = sample(c("TypeA", "TypeB"), 20, replace = TRUE),
  Sig1 = rnorm(20),
  Sig2 = rnorm(20)
)
sig_heatmap(input = test_input, features = c("Sig1", "Sig2"), group = "subtype", scale = TRUE)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CIBERSORT.R
\name{parallel_doperm}
\alias{parallel_doperm}
\title{Parallel Permutation Test for CIBERSORT}
\usage{
parallel_doperm(
  perm1,
  X1,
  Y1,
  absolute1,
  abs_method1,
  num_cores1 = 2,
  seed = NULL
)
}
\arguments{
\item{perm1}{Integer. Number of permutations to perform (\eqn{\ge}100 recommended).}

\item{X1}{Matrix or data frame. Signature matrix (cell type GEP barcode).}

\item{Y1}{Matrix. Mixture file containing gene expression profiles.}

\item{absolute1}{Logical. Whether to run in absolute mode (default: FALSE).}

\item{abs_method1}{String. Method for absolute mode: 'sig.score' or 'no.sumto1'.}

\item{num_cores1}{Integer. Number of CPU cores for parallel computation (default: 2).}

\item{seed}{Integer. Random seed for reproducibility. If NULL (default),
uses current random state. Set to a specific value (e.g., 123) for
reproducible results across runs.}
}
\value{
List containing:
\describe{
  \item{dist}{Numeric vector of correlation coefficients from permutations,
    representing the empirical null distribution.}
}
}
\description{
Parallel version of doPerm. Performs permutation-based sampling and runs
the CoreAlg function iteratively using multiple CPU cores to accelerate
computation. This function generates an empirical null distribution of
correlation coefficients for p-value calculation in CIBERSORT analysis.
}
\details{
This function utilizes the \code{foreach} and \code{doParallel} packages
to distribute permutation iterations across multiple cores. It automatically
handles cluster setup/teardown via \code{on.exit()} to prevent resource leaks.

Note: Windows users may experience slower performance due to socket-based
parallelization (PSOCK) versus forking on Unix systems.
}
\examples{
# Simulate data
set.seed(123)
X <- matrix(rnorm(1000), nrow = 100)
Y <- matrix(rnorm(500), nrow = 100)
rownames(X) <- rownames(Y) <- paste0("Gene", 1:100)
colnames(X) <- paste0("Cell", 1:10)
colnames(Y) <- paste0("Sample", 1:5)

# Run parallel permutation test
result <- parallel_doperm(
  perm1 = 10, X1 = X, Y1 = Y,
  absolute1 = FALSE, abs_method1 = "sig.score", num_cores1 = 1
)
if (!is.null(result)) str(result$dist)
}
\seealso{
\code{\link{doPerm}} for the sequential version, \code{\link{CoreAlg}}, \code{\link{CIBERSORT}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantiseq_helper.R
\name{quantiseq_helper}
\alias{quantiseq_helper}
\title{Helper Functions for quanTIseq}
\value{
Internal functions return processed data for quanTIseq deconvolution.
}
\description{
Helper functions for quanTIseq deconvolution method.
Source code adapted from https://github.com/FFinotello/quanTIseq.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_kruskal.R
\name{batch_kruskal}
\alias{batch_kruskal}
\title{Batch Kruskal-Wallis Test}
\usage{
batch_kruskal(data, group, feature = NULL, feature_manipulation = FALSE)
}
\arguments{
\item{data}{Data frame containing the dataset for analysis.}

\item{group}{Character string specifying the name of the grouping variable.}

\item{feature}{Character vector specifying the names of feature variables to
test. If `NULL`, the user is prompted to select features (interactive mode
only). Default is `NULL`.}

\item{feature_manipulation}{Logical indicating whether to apply feature
manipulation to filter valid features. Default is `FALSE`.}
}
\value{
Tibble containing:
\describe{
  \item{sig_names}{Feature name}
  \item{p.value}{Raw p-value from Kruskal-Wallis test}
  \item{statistic}{Test statistic (chi-squared)}
  \item{p.adj}{Adjusted p-value (Benjamini-Hochberg)}
  \item{log10pvalue}{Negative log10-transformed p-value}
  \item{stars}{Significance stars: **** p<0.0001, *** p<0.001,
    ** p<0.01, * p<0.05, + p<0.5}
  \item{group columns}{Mean-centered values for each group}
}
}
\description{
Performs Kruskal-Wallis rank sum tests on multiple continuous features across
different groups. Computes p-values, adjusts for multiple testing, and ranks
features by significance.
}
\examples{
# Create small example data
set.seed(123)
test_data <- data.frame(
  Gender = rep(c("Male", "Female"), each = 50),
  Signature1 = rnorm(100),
  Signature2 = rnorm(100)
)
# Test features by gender
res <- batch_kruskal(
  data = test_data,
  group = "Gender",
  feature = c("Signature1", "Signature2")
)
head(res)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xCell.R
\name{xCellAnalysis}
\alias{xCellAnalysis}
\title{The xCell Analysis Pipeline}
\usage{
xCellAnalysis(
  expr,
  signatures = NULL,
  genes = NULL,
  spill = NULL,
  rnaseq = TRUE,
  file.name = NULL,
  scale = TRUE,
  alpha = 0.5,
  save.raw = FALSE,
  cell.types.use = NULL
)
}
\arguments{
\item{expr}{Gene expression data matrix with row names as gene symbols and
columns as samples.}

\item{signatures}{GMT object of signatures. If `NULL`, uses xCell defaults.}

\item{genes}{Character vector of genes to use in the analysis. If `NULL`,
uses xCell defaults.}

\item{spill}{Spillover object for adjusting scores. If `NULL`, uses xCell defaults.}

\item{rnaseq}{Logical indicating whether to use RNA-seq (TRUE) or array
(FALSE) spillover parameters. Default is `TRUE`.}

\item{file.name}{Character string for saving scores. Default is `NULL`.}

\item{scale}{Logical indicating whether to use scaling. Default is `TRUE`.}

\item{alpha}{Numeric value to override spillover alpha parameter. Default is `0.5`.}

\item{save.raw}{Logical indicating whether to save raw scores. Default is `FALSE`.}

\item{cell.types.use}{Character vector of cell types to use. If `NULL`, uses
all available cell types. Default is `NULL`.}
}
\value{
A matrix of adjusted xCell scores.
}
\description{
Returns the xCell cell types enrichment scores for tumor microenvironment
deconvolution. Uses ssGSEA-based enrichment analysis with spillover
compensation to estimate cell type proportions from gene expression data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/creat_folder.R
\name{creat_folder}
\alias{creat_folder}
\title{Create Nested Output Folders}
\usage{
creat_folder(f1, f2 = NULL, f3 = NULL, return = NULL)
}
\arguments{
\item{f1}{Character. First-level folder name.}

\item{f2}{Character or `NULL`. Second-level folder name. Default is `NULL`.}

\item{f3}{Character or `NULL`. Third-level folder name. Default is `NULL`.}

\item{return}{Deprecated (not used). Kept for backward compatibility.}
}
\value{
List with elements:
\describe{
  \item{folder_name}{Relative path to the created folder}
  \item{abspath}{Absolute path ending with '/'}
}
}
\description{
Creates one to three nested folders (if not existing) under the current
working directory and returns their names and absolute paths.
}
\examples{
creat_folder(file.path(tempdir(), "1-result"))
creat_folder(file.path(tempdir(), "1-result"), "figures", "correlation")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CIBERSORT.R
\name{doPerm}
\alias{doPerm}
\title{Permutation Test for CIBERSORT}
\usage{
doPerm(perm, X, Y, absolute, abs_method, seed = NULL)
}
\arguments{
\item{perm}{Integer. Number of permutations to perform (\eqn{\ge}100 recommended for
reliable p-value estimation).}

\item{X}{Matrix or data frame containing signature matrix (predictor variables).}

\item{Y}{Numeric vector containing the mixture sample expression.}

\item{absolute}{Logical indicating whether to use absolute space for weights.}

\item{abs_method}{String specifying the method for absolute space weights:
`"sig.score"` or `"no.sumto1"`.}

\item{seed}{Integer. Random seed for reproducibility. If NULL (default),
uses current random state.}
}
\value{
List containing:
\describe{
  \item{dist}{Numeric vector of correlation coefficients from permutations}
}
}
\description{
Performs permutation-based sampling to generate an empirical null distribution
of correlation coefficients for p-value calculation in CIBERSORT analysis.
Randomly samples from the mixture data to create null distributions.
}
\examples{
# Simulate data
set.seed(123)
X <- matrix(rnorm(100), nrow = 10)
rownames(X) <- paste0("Gene", 1:10)
colnames(X) <- paste0("Cell", 1:10)
Y <- rnorm(10)
names(Y) <- paste0("Gene", 1:10)

# Run permutation test
result <- doPerm(10, X, Y, absolute = FALSE, abs_method = "sig.score")
if (!is.null(result)) str(result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combine_pd_eset.R
\name{combine_pd_eset}
\alias{combine_pd_eset}
\title{Combine Phenotype Data and Expression Set}
\usage{
combine_pd_eset(
  eset,
  pdata,
  id_pdata = "ID",
  feas = NULL,
  feature_manipulation = TRUE,
  scale = TRUE,
  choose_who_when_duplicate = c("eset", "pdata")
)
}
\arguments{
\item{eset}{Expression matrix with genes/features in rows and samples in
columns.}

\item{pdata}{Data frame containing phenotype/clinical data.}

\item{id_pdata}{Character string specifying the column name in `pdata`
containing sample identifiers. Default is `"ID"`.}

\item{feas}{Character vector specifying features to include from `eset`.
If `NULL`, all features are used. Default is `NULL`.}

\item{feature_manipulation}{Logical indicating whether to apply feature
manipulation to filter valid features. Default is `TRUE`.}

\item{scale}{Logical indicating whether to scale (standardize) expression
data. Default is `TRUE`.}

\item{choose_who_when_duplicate}{Character string specifying which data to
prefer when duplicate columns exist. Options are `"eset"` or `"pdata"`.
Default is `"eset"`.}
}
\value{
Data frame combining phenotype data and (transposed) expression data,
  with samples in rows and features/phenotypes in columns.
}
\description{
Merges phenotype data with an expression matrix by matching sample IDs.
Optionally filters features, applies feature manipulation, and scales
expression data before combining.
}
\examples{
set.seed(123)
eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
rownames(eset) <- paste0("Gene", 1:100)
colnames(eset) <- paste0("Sample", 1:10)
pdata <- data.frame(
  ID = colnames(eset),
  group = rep(c("A", "B"), each = 5),
  age = rnorm(10, 50, 10)
)
result <- combine_pd_eset(eset = eset, pdata = pdata, scale = FALSE)
dim(result)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timer.R
\name{GetOutlierGenes}
\alias{GetOutlierGenes}
\title{Get Outlier Genes}
\usage{
GetOutlierGenes(cancers)
}
\arguments{
\item{cancers}{Data frame. One column containing paths to gene expression files.}
}
\value{
Vector of unique gene names identified as outliers.
}
\description{
Identifies outlier genes from multiple cancer datasets. Treats the top 5
expressed genes in each sample as outliers and returns unique outlier genes.
}
\examples{
tf1 <- tempfile(fileext = ".tsv")
tf2 <- tempfile(fileext = ".tsv")

expr1 <- data.frame(
  gene = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE", "GeneF"),
  Sample1 = c(10, 50, 30, 80, 60, 20),
  Sample2 = c(15, 40, 25, 90, 55, 10)
)

expr2 <- data.frame(
  gene = c("GeneA", "GeneB", "GeneC", "GeneD", "GeneE", "GeneF"),
  Sample3 = c(100, 20, 10, 60, 30, 80),
  Sample4 = c(95, 25, 15, 70, 35, 85)
)

write.table(expr1, tf1, sep = "\t", row.names = FALSE, quote = FALSE)
write.table(expr2, tf2, sep = "\t", row.names = FALSE, quote = FALSE)

cancers <- data.frame(ExpressionFiles = c(tf1, tf2))
outlier_genes <- GetOutlierGenes(cancers)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sig_box_batch.R
\name{sig_box_batch}
\alias{sig_box_batch}
\title{Batch Signature Box Plots for Group Comparisons}
\usage{
sig_box_batch(
  input,
  vars,
  groups,
  pattern_vars = FALSE,
  path = NULL,
  index = 0,
  angle_x_text = 0,
  hjust = 0.5,
  palette = "jama",
  cols = NULL,
  jitter = FALSE,
  point_size = 5,
  size_of_font = 8,
  size_of_pvalue = 4.5,
  show_pvalue = TRUE,
  return_stat_res = FALSE,
  assay = NULL,
  slot = "scale.data",
  scale = FALSE,
  height = 5,
  width = 3.5,
  fig_type = "pdf",
  max_count_feas = 30
)
}
\arguments{
\item{input}{Data frame or Seurat object containing the data for analysis.}

\item{vars}{Character vector. Features or variables to analyze. When
`pattern_vars = TRUE`, these are treated as regular expression patterns.}

\item{groups}{Character vector. Grouping variable(s) for comparison.}

\item{pattern_vars}{Logical indicating whether to treat `vars` as regular
expression patterns for matching column names. Default is `FALSE`.}

\item{path}{Character string or `NULL`. Directory to save plots. If `NULL`,
plots are not saved. Default is `NULL`.}

\item{index}{Integer. Starting index for plot filenames. Default is `0`.}

\item{angle_x_text}{Numeric. Angle of x-axis labels in degrees. Default is `0`.}

\item{hjust}{Numeric. Horizontal justification of x-axis labels. Default is `0.5`.}

\item{palette}{Character. Color palette for plots. Default is `"jama"`.}

\item{cols}{Character vector or `NULL`. Custom colors for plot elements.}

\item{jitter}{Logical indicating whether to add jittered points. Default is `FALSE`.}

\item{point_size}{Numeric. Size of points. Default is `5`.}

\item{size_of_font}{Numeric. Base font size. Default is `8`.}

\item{size_of_pvalue}{Numeric. Size of p-value text. Default is `4.5`.}

\item{show_pvalue}{Logical indicating whether to display p-values.
Default is `TRUE`.}

\item{return_stat_res}{Logical indicating whether to return statistical
results instead of saving plots. Default is `FALSE`.}

\item{assay}{Character string or `NULL`. Assay type for Seurat objects.}

\item{slot}{Character. Data slot for Seurat objects. Default is `"scale.data"`.}

\item{scale}{Logical indicating whether to scale data before analysis.
Default is `FALSE`.}

\item{height}{Numeric. Height of plots in inches. Default is `5`.}

\item{width}{Numeric. Width of plots in inches. Default is `3.5`.}

\item{fig_type}{Character. File format for plots (e.g., `"pdf"`, `"png"`).
Default is `"pdf"`.}

\item{max_count_feas}{Integer. Maximum number of features to analyze when
`pattern_vars = TRUE`. If matched variables exceed this limit, only the
first `max_count_feas` features are used. Default is `30`.}
}
\value{
If `return_stat_res = TRUE`, returns a data frame of statistical
  results; otherwise, invisibly returns the path to saved plots.
}
\description{
Generates multiple box plots for specified features (signatures) across groups
in the input data. Supports customization of plot appearance, output path,
statistical annotation, and compatibility with Seurat objects. Plots are saved
to the specified directory or a default folder.
}
\examples{
# Simulate data
set.seed(123)
sim_pdata <- data.frame(
  ID = paste0("Sample", 1:50),
  subtype = sample(c("TypeA", "TypeB", "TypeC"), 50, replace = TRUE),
  TMEscore_plus = rnorm(50),
  GZMB = rnorm(50)
)
sig_box_batch(
  input = sim_pdata,
  vars = c("TMEscore_plus", "GZMB"),
  groups = "subtype",
  jitter = TRUE,
  palette = "jco",
  path = tempdir()
)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timer.R
\name{DrawQQPlot}
\alias{DrawQQPlot}
\title{Draw QQ Plot Comparing Cancer and Immune Expression}
\usage{
DrawQQPlot(cancer.exp, immune.exp, name = "")
}
\arguments{
\item{cancer.exp}{Vector. Gene expression data for cancer samples.}

\item{immune.exp}{Vector. Gene expression data for immune samples.}

\item{name}{Character. Optional subtitle with additional information.}
}
\value{
Generates a QQ plot.
}
\description{
Creates a quantile-quantile (QQ) plot to compare gene expression
distributions between cancer and immune samples. Points along the diagonal
indicate similar distributions.
}
\examples{
cancer_exp <- rnorm(100, mean = 5, sd = 1.5)
immune_exp <- rnorm(100, mean = 5, sd = 1.5)
DrawQQPlot(
  cancer.exp = cancer_exp,
  immune.exp = immune_exp,
  name = "Comparison of Gene Expression"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lasso_select.R
\name{lasso_select}
\alias{lasso_select}
\title{Feature Selection for Predictive or Prognostic Models Using LASSO Regression}
\usage{
lasso_select(
  x,
  y,
  type = c("binary", "survival"),
  nfold = 10,
  lambda = c("lambda.min", "lambda.1se")
)
}
\arguments{
\item{x}{A numeric matrix. Features (e.g., gene symbols or CGI) as row names and samples as column names.}

\item{y}{A response variable vector. Can be binary (0/1) or survival data (e.g., survival time and event status).}

\item{type}{Character. Model type: "binary" for binary response or "survival" for survival analysis. Default is "binary".}

\item{nfold}{Integer. Number of folds for cross-validation. Default is 10.}

\item{lambda}{Character. Regularization parameter selection: "lambda.min" (minimum mean cross-validated error) or "lambda.1se" (one standard error from minimum). Default is "lambda.min".}
}
\value{
Character vector of selected feature names with non-zero coefficients in the optimal LASSO model.
}
\description{
Applies LASSO (Least Absolute Shrinkage and Selection Operator) regression
to construct predictive or prognostic models. Supports both binary and
survival response variables, utilizing cross-validation for optimal model
selection.
}
\examples{
set.seed(123)
gene_expression <- matrix(rnorm(100 * 20), nrow = 100, ncol = 20)
rownames(gene_expression) <- paste0("Gene", 1:100)
colnames(gene_expression) <- paste0("Sample", 1:20)

# Binary response example
binary_outcome <- sample(c(0, 1), 20, replace = TRUE)
lasso_select(
  x = gene_expression,
  y = binary_outcome,
  type = "binary",
  nfold = 5
)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PrognosticModel.R
\name{CalculateTimeROC}
\alias{CalculateTimeROC}
\title{Calculate Time-Dependent ROC Curve}
\usage{
CalculateTimeROC(model, newx, s, acture.y, modelname, time_prob = 0.9)
}
\arguments{
\item{model}{A fitted survival model object.}

\item{newx}{A matrix or data frame of new data for prediction.}

\item{s}{Lambda value for prediction.}

\item{acture.y}{Data frame with `time` and `status` columns.}

\item{modelname}{Character string for model identification.}

\item{time_prob}{Numeric quantile for ROC calculation. Default is `0.9`.}
}
\value{
An object of class `timeROC` containing ROC curve information.
}
\description{
Computes time-dependent ROC curve for survival models using the `timeROC` package.
Evaluates predictive accuracy at a specified time quantile.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE) &&
  requireNamespace("survival", quietly = TRUE) &&
  requireNamespace("timeROC", quietly = TRUE)) {
  library(survival)
  dat <- na.omit(lung[, c("time", "status", "age", "sex", "ph.ecog")])
  dat$status <- dat$status - 1
  x <- as.matrix(dat[, c("age", "sex", "ph.ecog")])
  y <- Surv(dat$time, dat$status)
  fit <- glmnet::glmnet(x, y, family = "cox")
  actual_outcome <- data.frame(time = dat$time, status = dat$status)
  roc_info <- CalculateTimeROC(
    model = fit, newx = x, s = 0.01, acture.y = actual_outcome,
    modelname = "glmnet Cox Model", time_prob = 0.5
  )
  print(roc_info$AUC)
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timer.R
\name{timer_info}
\alias{timer_info}
\title{Source code for the TIMER deconvolution method.}
\usage{
timer_info(string)
}
\arguments{
\item{string}{Character. Message to be displayed.}
}
\value{
None; used for its side effect of printing a message.
}
\description{
Formats and displays informational messages for timing or logging purposes.
Useful for tracking progress or stages of execution within scripts.
}
\details{
This code is adapted from https://github.com/hanfeisun/TIMER, which
again is an adapted version of the original TIMER source code
from http://cistrome.org/TIMER/download.html.

The method is described in Li et al. Genome Biology 2016;17(1):174., PMID
27549193.
Display Timer Information Messages
}
\examples{
timer_info("Data processing started.")
}
\author{
Bo Li
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_data.R
\name{reset_iobr_cache_dir}
\alias{reset_iobr_cache_dir}
\title{Reset IOBR Cache Directory to Default}
\usage{
reset_iobr_cache_dir()
}
\value{
Invisibly returns the default cache directory path.
}
\description{
Resets the cache directory to the default session-specific
temporary directory.
}
\examples{
reset_iobr_cache_dir()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/high_var_fea.R
\name{high_var_fea}
\alias{high_var_fea}
\title{Identify High-Variance Features from Statistical Results}
\usage{
high_var_fea(
  result,
  target,
  name_padj = "padj",
  padj_cutoff = 1,
  name_logfc,
  logfc_cutoff = 0,
  n = 10,
  data_type = NULL
)
}
\arguments{
\item{result}{Data frame or tibble. Statistical results containing feature,
adjusted p-value, and logFC columns.}

\item{target}{Character. Column name of feature identifiers.}

\item{name_padj}{Character. Adjusted p-value column name. Default is `"padj"`.}

\item{padj_cutoff}{Numeric. Adjusted p-value threshold. Default is 1.}

\item{name_logfc}{Character. log2 fold-change column name.}

\item{logfc_cutoff}{Numeric. Absolute log2 fold-change threshold. Default is 0.}

\item{n}{Integer. Number of top up and top down features to select.
Default is 10.}

\item{data_type}{Character or `NULL`. If `"survival"`, adjusts logFC
interpretation. Default is `NULL`.}
}
\value{
Character vector of selected feature names (combined up and down sets).
}
\description{
Selects top variable (up- and down-regulated) features based on adjusted
p-value and log fold-change thresholds.
}
\examples{
result_data <- data.frame(
  gene = c("Gene1", "Gene2", "Gene3", "Gene4", "Gene5"),
  padj = c(0.01, 0.02, 0.05, 0.001, 0.03),
  logfc = c(-2, 1.5, -3, 2.5, 0.5)
)
high_var_fea(
  result = result_data,
  target = "gene",
  name_padj = "padj",
  name_logfc = "logfc",
  n = 2,
  padj_cutoff = 0.05,
  logfc_cutoff = 1.5
)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xCell.R
\name{xCellSignifcanceBetaDist}
\alias{xCellSignifcanceBetaDist}
\title{Calculate Significance P-values Using Beta Distribution}
\usage{
xCellSignifcanceBetaDist(
  scores,
  beta_params = NULL,
  rnaseq = TRUE,
  file.name = NULL
)
}
\arguments{
\item{scores}{xCell scores matrix.}

\item{beta_params}{Pre-calculated beta distribution parameters.}

\item{rnaseq}{Logical for RNA-seq vs array parameters. Default is `TRUE`.}

\item{file.name}{Character string for saving p-values. Default is `NULL`.}
}
\value{
Matrix of p-values.
}
\description{
Calculates FDR-adjusted p-values for the null hypothesis that a cell type
is not present in the mixture.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_data.R
\name{get_default_mirrors}
\alias{get_default_mirrors}
\title{Get Default Download Mirrors}
\usage{
get_default_mirrors()
}
\value{
Character vector of mirror URLs.
}
\description{
Returns the default list of download mirrors.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\name{deconvo_epic}
\alias{deconvo_epic}
\title{Deconvolve Immune Microenvironment Using EPIC}
\usage{
deconvo_epic(eset, project = NULL, tumor = TRUE)
}
\arguments{
\item{eset}{Gene expression matrix with genes as row names.}

\item{project}{Optional project name. Default is `NULL`.}

\item{tumor}{Logical indicating tumor (`TRUE`) or normal (`FALSE`) samples.}
}
\value{
Data frame with EPIC cell fraction estimates. Columns suffixed with
  `_EPIC`.
}
\description{
Estimates immune cell fractions using EPIC algorithm.
}
\examples{
\dontrun{
TRef <- load_data("TRef")
if (!is.null(TRef)) {
  set.seed(123)
  sim_eset <- matrix(rnorm(nrow(TRef$refProfiles) * 2), nrow(TRef$refProfiles), 2)
  rownames(sim_eset) <- rownames(TRef$refProfiles)
  colnames(sim_eset) <- paste0("Sample", 1:2)
  result <- deconvo_epic(eset = sim_eset, project = "Example", tumor = TRUE)
  if (!is.null(result)) head(result)
}
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_names.R
\name{remove_names}
\alias{remove_names}
\title{Remove Patterns from Column Names or Variables}
\usage{
remove_names(
  input_df,
  variable = "colnames",
  patterns_to_na = patterns_to_na,
  patterns_space = NULL
)
}
\arguments{
\item{input_df}{Data frame. Input data to modify.}

\item{variable}{Character. Column to modify: "colnames" for column names,
or a specific column name. Default is "colnames".}

\item{patterns_to_na}{Character vector. Patterns to replace with empty
string. Default uses [patterns_to_na].}

\item{patterns_space}{Character vector or `NULL`. Patterns to replace with
spaces. Default is `NULL`.}
}
\value{
Modified data frame with patterns replaced.
}
\description{
Modifies column names or specified variables in a data frame by replacing
specified patterns with empty strings or spaces.
}
\examples{
df <- data.frame(
  "CellA_cibersort" = 1:5,
  "CellB_xCell" = 6:10,
  "CellC_TIMER" = 11:15
)
result <- remove_names(df, variable = "colnames", patterns_to_na = patterns_to_na)
colnames(result)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_cor_matrix.R
\name{get_cor_matrix}
\alias{get_cor_matrix}
\title{Calculate and Visualize Correlation Matrix Between Two Variable Sets}
\usage{
get_cor_matrix(
  data,
  feas1,
  feas2,
  method = c("pearson", "spearman", "kendall"),
  path = NULL,
  index = 1,
  fig.type = "pdf",
  width = NULL,
  height = NULL,
  project = NULL,
  is.matrix = FALSE,
  scale = TRUE,
  font.size = 15,
  fill_by_cor = FALSE,
  round.num = 1,
  font.size.star = 8,
  cols = NULL
)
}
\arguments{
\item{data}{Input data frame or matrix. Variables should be in columns.}

\item{feas1}{Character vector of variable names for the first set.}

\item{feas2}{Character vector of variable names for the second set.}

\item{method}{Correlation method: `"pearson"`, `"spearman"`, or `"kendall"`.
Default is `"pearson"`.}

\item{path}{Directory to save the plot. If `NULL`, plot is not saved.
Default is `NULL`.}

\item{index}{Numeric prefix for output filename. Default is 1.}

\item{fig.type}{File format: `"pdf"`, `"png"`, etc. Default is `"pdf"`.}

\item{width}{Plot width in inches. Auto-calculated if `NULL`.}

\item{height}{Plot height in inches. Auto-calculated if `NULL`.}

\item{project}{Project name for plot title. Default is `NULL`.}

\item{is.matrix}{Logical: if `TRUE`, data is transposed. Default is `FALSE`.}

\item{scale}{Logical: scale variables before correlation. Default is `TRUE`.}

\item{font.size}{Font size for axis labels. Default is 15.}

\item{fill_by_cor}{Logical: show correlation values instead of stars.
Default is `FALSE`.}

\item{round.num}{Decimal places for correlation values. Default is 1.}

\item{font.size.star}{Font size for significance stars. Default is 8.}

\item{cols}{Custom colors for gradient (low, mid, high). If `NULL`, uses
blue-white-red. Default is `NULL`.}
}
\value{
ggplot object displaying the correlation matrix heatmap.
}
\description{
Calculates and visualizes the correlation matrix between two sets of variables.
Supports Pearson, Spearman, and Kendall correlation methods. The function
generates a customizable heatmap with significance stars.
}
\examples{
set.seed(123)
data <- as.data.frame(matrix(rnorm(1000), nrow = 100, ncol = 10))
colnames(data) <- paste0("Gene_", 1:10)

feas1 <- c("Gene_1", "Gene_2", "Gene_3")
feas2 <- c("Gene_4", "Gene_5", "Gene_6")

cor_plot <- get_cor_matrix(
  data = data,
  feas1 = feas1,
  feas2 = feas2,
  method = "spearman",
  project = "Example Correlation"
)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getHRandCIfromCoxph.R
\name{getHRandCIfromCoxph}
\alias{getHRandCIfromCoxph}
\title{Extract Hazard Ratio and Confidence Intervals from Cox Model}
\usage{
getHRandCIfromCoxph(coxphData)
}
\arguments{
\item{coxphData}{Fitted Cox model object from `survival::coxph()`.}
}
\value{
Data frame with p-values, HR, and confidence intervals.
}
\description{
Extracts hazard ratio (HR) and 95% confidence intervals from a fitted
Cox proportional hazards model.
}
\examples{
library(survival)
set.seed(123)
df <- data.frame(
  TTE = rexp(200, rate = 0.1),
  Cens = rbinom(200, size = 1, prob = 0.7),
  group = sample(c("Treatment", "Control"), 200, replace = TRUE)
)
coxphData <- survival::coxph(survival::Surv(TTE, Cens) ~ group, data = df)
results <- getHRandCIfromCoxph(coxphData)
print(results)
}
\author{
Dorothee Nickles

Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eset_distribution.R
\name{eset_distribution}
\alias{eset_distribution}
\title{Visualize Expression Set Distribution}
\usage{
eset_distribution(eset, quantile = 3, log = TRUE, project = NULL)
}
\arguments{
\item{eset}{Expression matrix or data frame with genes in rows and samples
in columns.}

\item{quantile}{Integer specifying the divisor for sampling columns.
Default is 3 (samples 1/3 of columns).}

\item{log}{Logical indicating whether to perform log2 transformation.
Default is `TRUE`.}

\item{project}{Optional output directory path for saving files. If `NULL`,
no files are saved. Default is `NULL`.}
}
\value{
Invisibly returns `NULL`. If `project` is provided, saves PNG files to disk.
}
\description{
Generates boxplots and density plots to analyze the distribution of
expression values in an expression set. Useful for quality control and
assessing data normalization.
}
\examples{
# Simulate data
set.seed(123)
sim_eset <- matrix(rnorm(1000 * 10, mean = 5, sd = 2), 1000, 10)
rownames(sim_eset) <- paste0("Gene", 1:1000)
colnames(sim_eset) <- paste0("Sample", 1:10)

# Run distribution plot
result <- eset_distribution(sim_eset)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BinomialModel.R
\name{ProcessingData}
\alias{ProcessingData}
\title{Process Data for Model Construction}
\usage{
ProcessingData(x, y, scale, type = c("binomial", "survival"))
}
\arguments{
\item{x}{Data frame of predictors with first column as IDs.}

\item{y}{Data frame of outcomes with first column as IDs. For survival,
expects two additional columns for time and status.}

\item{scale}{Logical indicating whether to scale predictors.}

\item{type}{Character string: `"binomial"` or `"survival"`.}
}
\value{
List containing:
\describe{
  \item{x_scale}{Processed predictor matrix}
  \item{y}{Processed outcome variable}
  \item{x_ID}{Sample IDs}
}
}
\description{
Preprocesses data for binomial or survival analysis. Aligns and filters data
based on sample IDs, optionally scales data, and ensures appropriate data types.
Handles missing values by removing columns with NA values.
}
\examples{
x <- data.frame(ID = 1:10, predictor1 = rnorm(10), predictor2 = rnorm(10))
y <- data.frame(ID = 1:10, outcome = sample(c(0, 1), 10, replace = TRUE))
result <- ProcessingData(x, y, scale = TRUE, type = "binomial")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_mytheme.R
\name{design_mytheme}
\alias{design_mytheme}
\title{Design Custom Theme for ggplot2 Plots}
\usage{
design_mytheme(
  theme = c("light", "bw", "classic", "classic2"),
  plot_title_size = 2,
  axis_title_size = 2,
  axis_text_size = 12,
  axis_angle = 60,
  hjust = 1,
  legend.position = "bottom",
  legend.direction = "horizontal",
  legend.size = 0.25,
  legend.key.height = 0.5,
  legend.key.width = 0.5,
  legend.size.text = 10,
  legend.box = "horizontal"
)
}
\arguments{
\item{theme}{Base theme: "light", "bw", "classic", "classic2".
Default is "light".}

\item{plot_title_size}{Relative size of plot title. Default is 2.}

\item{axis_title_size}{Relative size of axis titles. Default is 2.}

\item{axis_text_size}{Size of axis tick labels. Default is 12.}

\item{axis_angle}{Angle of x-axis tick labels. Default is 60.}

\item{hjust}{Horizontal justification for x-axis text. Default is 1.}

\item{legend.position}{Legend position: "none", "left", "right", "bottom",
"top". Default is "bottom".}

\item{legend.direction}{Direction of legend items: "horizontal" or "vertical".
Default is "horizontal".}

\item{legend.size}{Size of legend key. Default is 0.25.}

\item{legend.key.height}{Height of legend key in cm. Default is 0.5.}

\item{legend.key.width}{Width of legend key in cm. Default is 0.5.}

\item{legend.size.text}{Size of legend text labels. Default is 10.}

\item{legend.box}{Orientation of legend box: "horizontal" or "vertical".
Default is "horizontal".}
}
\value{
A ggplot2 theme object.
}
\description{
Creates a customized ggplot2 theme based on user-specified parameters for
plot elements such as title size, axis sizes, legend settings, and theme style.
Supports various base themes and allows fine-tuning of visual aspects.
}
\examples{
library(ggplot2)
p <- ggplot(mtcars, aes(wt, mpg)) +
  geom_point()
mytheme <- design_mytheme(theme = "bw", plot_title_size = 1.5, axis_text_size = 14)
p + mytheme + ggtitle("Example Plot")
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PrognosticModel.R
\name{PrognosticAUC}
\alias{PrognosticAUC}
\title{Calculate Time-Dependent AUC for Survival Models}
\usage{
PrognosticAUC(model, newx, s, acture.y)
}
\arguments{
\item{model}{A fitted survival model object capable of generating risk scores.}

\item{newx}{A matrix or data frame of new data for prediction.}

\item{s}{Lambda value for prediction. Can be numeric or `"lambda.min"`/`"lambda.1se"`.}

\item{acture.y}{Data frame with `time` and `status` columns.}
}
\value{
A data frame with AUC values at 30th (`probs.3`) and 90th (`probs.9`) percentiles.
}
\description{
Evaluates prognostic ability of a survival model by calculating time-dependent AUC
at the 30th and 90th percentiles of survival time. These thresholds assess
short-term and long-term predictive accuracy.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE) &&
  requireNamespace("survival", quietly = TRUE) &&
  requireNamespace("timeROC", quietly = TRUE)) {
  library(survival)
  set.seed(123)
  x <- matrix(rnorm(100 * 5), ncol = 5)
  y <- Surv(rexp(100), rbinom(100, 1, 0.5))
  fit <- glmnet::cv.glmnet(x, y, family = "cox")
  acture_y <- data.frame(time = y[, 1], status = y[, 2])
  auc_results <- PrognosticAUC(fit, newx = x, s = "lambda.min", acture.y = acture_y)
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IPS_helper.R
\name{mapbw}
\alias{mapbw}
\title{Map Score to Black and White Color}
\usage{
mapbw(x, my_palette2 = NULL)
}
\arguments{
\item{x}{Numeric value to be mapped to a color (typically between -2 and 2).}

\item{my_palette2}{Color palette vector (should have 1001 colors).
Default uses black-white gradient.}
}
\value{
A color from the black-white palette as a hex code.
}
\description{
Maps a numeric input value to a color from a black-white gradient palette.
Values are mapped to a 1001-color palette where -2 maps to black and +2 maps to white.
}
\examples{
my_palette2 <- grDevices::colorRampPalette(c("black", "white"))(1001)
color <- mapbw(1.5, my_palette2)
color <- mapbw(-1, my_palette2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sig_pheatmap.R
\name{sig_pheatmap}
\alias{sig_pheatmap}
\title{Generate Heatmap for Signature Data}
\usage{
sig_pheatmap(
  input,
  feas,
  group,
  group2 = NULL,
  group3 = NULL,
  ID = "ID",
  path = NULL,
  cols1 = "random",
  cols2 = "random",
  cols3 = "random",
  seed = 54321,
  show_col = FALSE,
  palette1 = 1,
  palette2 = 2,
  palette3 = 3,
  cluster_cols = TRUE,
  palette_for_heatmape = 6,
  scale.matrix = TRUE,
  cellwidth = 1,
  cellheight = 9,
  show_colnames = FALSE,
  fig.type = "pdf",
  width = 6,
  height = NULL,
  file_name_prefix = 1
)
}
\arguments{
\item{input}{Data frame with variables in columns.}

\item{feas}{Character vector. Feature names (columns) to include in heatmap.}

\item{group}{Character string. Column name for primary grouping variable.}

\item{group2}{Character string or `NULL`. Optional secondary grouping variable.}

\item{group3}{Character string or `NULL`. Optional tertiary grouping variable.}

\item{ID}{Character string. Column name for sample identifiers. Default is `"ID"`.}

\item{path}{Character string or `NULL`. Directory to save output files.
If `NULL`, the heatmap is not saved. Default is `NULL`.}

\item{cols1}{Character vector or `"random"` or `"normal"`. Colors for primary group.
Default is `"random"`.}

\item{cols2}{Character vector or `"random"` or `"normal"`. Colors for secondary group.
Default is `"random"`.}

\item{cols3}{Character vector or `"random"` or `"normal"`. Colors for tertiary group.
Default is `"random"`.}

\item{seed}{Integer. Random seed for color generation. Default is `54321`.}

\item{show_col}{Logical indicating whether to display colors. Default is `FALSE`.}

\item{palette1}{Integer. Palette for primary group. Default is `1`.}

\item{palette2}{Integer. Palette for secondary group. Default is `2`.}

\item{palette3}{Integer. Palette for tertiary group. Default is `3`.}

\item{cluster_cols}{Logical indicating whether to cluster columns. Default is `TRUE`.}

\item{palette_for_heatmape}{Integer. Palette number for heatmap. Default is `6`.}

\item{scale.matrix}{Logical indicating whether to scale the matrix. Default is `TRUE`.}

\item{cellwidth}{Numeric. Width of each cell in points. Default is `1`.}

\item{cellheight}{Numeric. Height of each cell in points. Default is `9`.}

\item{show_colnames}{Logical indicating whether to show column names. Default is `FALSE`.}

\item{fig.type}{Character string. File format for saving. Default is `"pdf"`.}

\item{width}{Numeric. Width of saved figure in inches. Default is `6`.}

\item{height}{Numeric or `NULL`. Height of saved figure in inches. Calculated if `NULL`.}

\item{file_name_prefix}{Character or numeric. Prefix for saved file name. Default is `1`.}
}
\value{
A list containing:
\describe{
  \item{p_anno}{Annotation data frame}
  \item{p_cols}{List of cluster colors}
  \item{plot}{ComplexHeatmap object}
  \item{eset}{Transformed expression matrix}
}
}
\description{
Creates a heatmap from signature data with grouping variables, offering
flexible options for colors, clustering, and output formats using
ComplexHeatmap.
}
\examples{
set.seed(123)
test_input <- data.frame(
  ID = paste0("Sample", 1:20),
  subtype = sample(c("TypeA", "TypeB"), 20, replace = TRUE),
  Sig1 = rnorm(20),
  Sig2 = rnorm(20)
)
sig_pheatmap(
  input = test_input,
  feas = c("Sig1", "Sig2"),
  group = "subtype"
)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xCell.R
\name{transformScores}
\alias{transformScores}
\title{Transform Raw Scores to Fractions}
\usage{
transformScores(scores, fit.vals, scale = TRUE, fn = NULL)
}
\arguments{
\item{scores}{Raw scores from rawEnrichmentAnalysis.}

\item{fit.vals}{Calibration values from spill object.}

\item{scale}{Logical indicating whether to use scaling. Default is `TRUE`.}

\item{fn}{Character string for saving scores. Default is `NULL`.}
}
\value{
Transformed xCell scores matrix.
}
\description{
Transforms raw xCell scores to estimated cell fractions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_msigdb.R
\name{format_msigdb}
\alias{format_msigdb}
\title{Format Input Signatures from MSigDB}
\usage{
format_msigdb(gmt, ont = "term", gene = "gene")
}
\arguments{
\item{gmt}{Character string. Path to a GMT file.}

\item{ont}{Character string. Name of the signature/set column in the parsed
GMT table. Default is `"term"`.}

\item{gene}{Character string. Name of the gene column in the parsed GMT
table. Default is `"gene"`.}
}
\value{
Named list of character vectors, where each element contains the
  genes belonging to one signature.
}
\description{
Reads a GMT file from MSigDB and converts it into a named list of gene sets
suitable for IOBR functions.
}
\examples{
tf <- tempfile(fileext = ".gmt")
writeLines(
  c(
    "HALLMARK_TNFA_SIGNALING_VIA_NFKB\tNA\tTNF\tNFKB1\tNFKB2",
    "HALLMARK_P53_PATHWAY\tNA\tTP53\tMDM2\tCDKN1A"
  ),
  con = tf
)

sig_list <- format_msigdb(tf, ont = "term", gene = "gene")
names(sig_list)
sig_list[[1]]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcpcounter.R
\name{.appendSignatures}
\alias{.appendSignatures}
\title{Append Signatures to Expression Matrix}
\usage{
.appendSignatures(xp, markers)
}
\arguments{
\item{xp}{Expression matrix with features in rows and samples in columns.}

\item{markers}{List of marker gene vectors for each cell population.}
}
\value{
Matrix with summarized expression values.
}
\description{
Calculates mean expression for each marker feature set.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output_sig.R
\name{output_sig}
\alias{output_sig}
\title{Save Signature Data to File}
\usage{
output_sig(signatures, format = c("csv", "rdata"), file.name)
}
\arguments{
\item{signatures}{Signature data: a list or single string of signatures.}

\item{format}{Character. Output format: "csv" or "rdata". Default is "csv".}

\item{file.name}{Character. Name of the output file without extension.}
}
\value{
Data frame containing the processed signature data, also saved to
  the specified file.
}
\description{
Saves signature data to a specified file format, supporting CSV or RData.
Handles single signatures or lists of signatures, converting them to a
data frame for storage.
}
\examples{
# Simulate signatures
set.seed(123)
sim_sigs <- list(
  Signature1 = paste0("Gene", 1:50),
  Signature2 = paste0("Gene", 51:100),
  Signature3 = paste0("Gene", 101:150)
)
tmpfile <- tempfile(fileext = ".csv")
output_sig(
  signatures = sim_sigs, format = "csv",
  file.name = tools::file_path_sans_ext(tmpfile)
)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timer.R
\name{ConvertRownameToLoci}
\alias{ConvertRownameToLoci}
\title{Convert Rowname To Loci}
\usage{
ConvertRownameToLoci(cancerGeneExpression)
}
\arguments{
\item{cancerGeneExpression}{Matrix or data frame. Gene expression data with
row names in the format 'GENE|ID'.}
}
\value{
Matrix with modified gene expression data with updated row names.
  Rows without a valid identifier are removed.
}
\description{
Processes a gene expression data matrix by modifying its row names.
Extracts the gene identifier from row names formatted as 'GENE|ID',
simplifying them to 'GENE'.
}
\examples{
example_data <- matrix(runif(20), ncol = 5)
rownames(example_data) <- c("LOC101", "LOC102", "LOC103", "LOC104")
processed_data <- ConvertRownameToLoci(example_data)
print(processed_data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CIBERSORT.R
\name{CIBERSORT}
\alias{CIBERSORT}
\title{CIBERSORT Deconvolution Algorithm}
\usage{
CIBERSORT(
  sig_matrix = NULL,
  mixture_file,
  perm,
  QN = TRUE,
  absolute = FALSE,
  abs_method = "sig.score",
  parallel = FALSE,
  num_cores = 2,
  seed = NULL
)
}
\arguments{
\item{sig_matrix}{Cell type GEP barcode matrix: row 1 = sample labels;
column 1 = gene symbols; no missing values; default = LM22.txt download
from CIBERSORT (https://cibersortx.stanford.edu/runcibersort.php)}

\item{mixture_file}{GEP matrix: row 1 = sample labels; column 1 = gene
symbols; no missing values}

\item{perm}{Set permutations for statistical analysis
(>=100 permutations recommended).}

\item{QN}{Quantile normalization of input mixture (default = TRUE)}

\item{absolute}{Run CIBERSORT in absolute mode (default = FALSE)
- note that cell subsets will be scaled by their absolute levels and
  will not be represented as fractions (to derive the default output,
  normalize absolute levels such that they sum to 1 for each mixture sample)
- the sum of all cell subsets in each mixture sample will be added to the
  output ('Absolute score'). If LM22 is used, this score will capture
  total immune content.}

\item{abs_method}{If absolute is set to TRUE, choose method:
'no.sumto1' or 'sig.score'
- sig.score = for each mixture sample, define S as the median expression
  level of all genes in the signature matrix divided by the median
  expression level of all genes in the mixture. Multiple cell subset
  fractions by S.
- no.sumto1 = remove sum to 1 constraint}

\item{parallel}{Logical. Enable parallel execution? (default = FALSE)}

\item{num_cores}{Integer. Number of cores to use when \code{parallel = TRUE}
(default = 2)}

\item{seed}{Integer. Random seed for reproducible permutation testing.
If \code{NULL} (default), uses current random state. Set to a specific
value (e.g., 123) for reproducible results across runs. Applies to both
parallel and serial permutation.}
}
\value{
A matrix object containing the estimated cibersort-cell fractions,
  p-values, correlation coefficients, and RMSE values.
}
\description{
An analytical tool to estimate cell type abundances in mixed cell populations
using gene expression data.
}
\examples{
# Simulate data
set.seed(123)
sim_sig <- matrix(rnorm(100 * 5), 100, 5)
rownames(sim_sig) <- paste0("Gene", 1:100)
colnames(sim_sig) <- paste0("Cell", 1:5)

sim_mixture <- as.data.frame(matrix(
  rnorm(100 * 3), 100, 3
))
rownames(sim_mixture) <- paste0("Gene", 1:100)
colnames(sim_mixture) <- paste0("Sample", 1:3)

# Run deconvolution
result <- CIBERSORT(
  sig_matrix = sim_sig,
  mixture_file = sim_mixture,
  perm = 10, QN = FALSE, absolute = FALSE,
  parallel = FALSE
)
if (!is.null(result)) head(result)
}
\author{
Aaron M. Newman, Stanford University (amnewman@stanford.edu)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/random_strata_cells.R
\name{random_strata_cells}
\alias{random_strata_cells}
\title{Stratified Random Sampling of Cells}
\usage{
random_strata_cells(
  input,
  group,
  proportion = 0.1,
  minimum_count_include = 300,
  minimum_count = 200,
  maximum_count = 1000,
  sub_cluster = NULL,
  cell_type = NULL
)
}
\arguments{
\item{input}{A data frame or Seurat object containing cell annotations.}

\item{group}{Character string specifying the column name for cell type
grouping.}

\item{proportion}{Numeric value between 0 and 1 specifying the sampling
proportion. Default is 0.1.}

\item{minimum_count_include}{Integer specifying the minimum count threshold
for a cell type to be included in sampling. Default is 300.}

\item{minimum_count}{Integer specifying the minimum number of cells to
sample per cell type. Default is 200.}

\item{maximum_count}{Integer specifying the maximum number of cells to
sample per cell type. Default is 1000.}

\item{sub_cluster}{Optional character string specifying a sub-cluster
column for filtering. Default is NULL.}

\item{cell_type}{Optional character string specifying the cell type value
to filter when `sub_cluster` is provided. Default is NULL.}
}
\value{
A data frame containing the sampled cells with preserved cell type
  proportions.
}
\description{
Performs stratified random sampling of cells from single-cell data,
ensuring proportional representation of each cell type while respecting
minimum and maximum count constraints.
}
\examples{
# Create simulated cell annotation data
set.seed(123)
sim_cells <- data.frame(
  cell_id = paste0("Cell", 1:500),
  cell_type = sample(c("T_cell", "B_cell", "NK_cell", "Macrophage"), 500, replace = TRUE)
)
# Sample cells with stratified random sampling
sampled <- random_strata_cells(
  input = sim_cells,
  group = "cell_type",
  proportion = 0.2,
  minimum_count_include = 50,
  minimum_count = 20,
  maximum_count = 100
)
if (!is.null(sampled)) head(sampled)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateRef_DEseq2.R
\name{Top_probe}
\alias{Top_probe}
\title{Top Probe Selector}
\usage{
Top_probe(dat, i)
}
\arguments{
\item{dat}{Data frame containing a column named "probe".}

\item{i}{Integer. Number of top probes to return.}
}
\value{
Character vector containing the names of the top `i` probes.
}
\description{
Extracts the top `i` probes based on their ordering in the provided data
frame. If the number of rows is less than or equal to `i`, returns all
probes.
}
\examples{
dat <- data.frame(
  probe = c("Probe1", "Probe2", "Probe3", "Probe4", "Probe5"),
  value = c(5, 3, 2, 4, 1)
)
top_probes <- Top_probe(dat, 3)
print(top_probes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\name{deconvo_estimate}
\alias{deconvo_estimate}
\title{Calculate ESTIMATE Scores}
\usage{
deconvo_estimate(eset, project = NULL, platform = "affymetrix")
}
\arguments{
\item{eset}{Gene expression matrix with gene symbols.}

\item{project}{Optional project name. Default is `NULL`.}

\item{platform}{Platform type: `"affymetrix"` or `"illumina"`.
Default is `"affymetrix"`.}
}
\value{
Data frame with ESTIMATE scores. Columns suffixed with `_estimate`.
}
\description{
Calculates stromal, immune, and ESTIMATE scores from gene expression.
}
\examples{
\dontrun{
common_genes <- load_data("common_genes")
if (!is.null(common_genes)) {
  set.seed(123)
  sim_eset <- matrix(rnorm(nrow(common_genes) * 2), nrow(common_genes), 2)
  rownames(sim_eset) <- common_genes$GeneSymbol
  colnames(sim_eset) <- paste0("Sample", 1:2)
  result <- deconvo_estimate(sim_eset, project = "TCGA-STAD")
  if (!is.null(result)) head(result)
}
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sig_gsea.R
\name{sig_gsea}
\alias{sig_gsea}
\title{Perform Gene Set Enrichment Analysis (GSEA)}
\usage{
sig_gsea(
  deg,
  genesets = NULL,
  path = NULL,
  gene_symbol = "symbol",
  logfc = "log2FoldChange",
  org = c("hsa", "mus"),
  msigdb = TRUE,
  category = "H",
  subcategory = NULL,
  palette_bar = "jama",
  palette_gsea = 2,
  cols_gsea = NULL,
  cols_bar = NULL,
  show_bar = 10,
  show_col = FALSE,
  show_plot = FALSE,
  show_gsea = 8,
  show_path_n = 20,
  plot_single_sig = FALSE,
  project = "custom_sig",
  minGSSize = 10,
  maxGSSize = 500,
  verbose = TRUE,
  seed = FALSE,
  fig.type = "pdf",
  print_bar = TRUE
)
}
\arguments{
\item{deg}{Data frame containing differential expression results with gene symbols
and log fold changes.}

\item{genesets}{List of custom gene sets for enrichment analysis. If `NULL`,
MSigDB gene sets are used based on `org` and `category`. Default is `NULL`.}

\item{path}{Character string specifying the directory path for saving results.
Default is `NULL`.}

\item{gene_symbol}{Character string specifying the column name in `deg`
containing gene symbols. Default is `"symbol"`.}

\item{logfc}{Character string specifying the column name in `deg` containing
log fold change values. Default is `"log2FoldChange"`.}

\item{org}{Character string specifying the organism. Options are `"hsa"`
(Homo sapiens) or `"mus"` (Mus musculus). Default is `"hsa"`.}

\item{msigdb}{Logical indicating whether to use MSigDB gene sets. Default is
`TRUE`.}

\item{category}{Character string specifying the MSigDB category (e.g., `"H"`
for Hallmark, `"C2"` for curated gene sets). Default is `"H"`.}

\item{subcategory}{Character string specifying the MSigDB subcategory to filter
gene sets. Default is `NULL`.}

\item{palette_bar}{Character string or integer specifying the color palette for
bar plots. Default is `"jama"`.}

\item{palette_gsea}{Integer specifying the color palette for GSEA plots. Default
is `2`.}

\item{cols_gsea}{Character vector specifying custom colors for GSEA enrichment
plots. If `NULL`, colors are automatically generated. Default is `NULL`.}

\item{cols_bar}{Character vector specifying custom colors for the enrichment
bar plot. If `NULL`, colors are automatically generated. Default is `NULL`.}

\item{show_bar}{Integer specifying the number of top enriched gene sets to display
in the bar plot. Default is `10`.}

\item{show_col}{Logical indicating whether to display color names in the bar plot.
Default is `FALSE`.}

\item{show_plot}{Logical indicating whether to display GSEA enrichment plots.
Default is `FALSE`.}

\item{show_gsea}{Integer specifying the number of top significant gene sets for
which to generate GSEA plots. Default is `8`.}

\item{show_path_n}{Integer specifying the number of pathways to display in GSEA
plots. Default is `20`.}

\item{plot_single_sig}{Logical indicating whether to generate separate plots for
each significant gene set. Default is `TRUE`.}

\item{project}{Character string specifying the project name for output files.
Default is `"custom_sig"`.}

\item{minGSSize}{Integer specifying the minimum gene set size for analysis.
Default is `10`.}

\item{maxGSSize}{Integer specifying the maximum gene set size for analysis.
Default is `500`.}

\item{verbose}{Logical indicating whether to display progress messages. Default
is `TRUE`.}

\item{seed}{Logical indicating whether to set a random seed for reproducibility.
Default is `FALSE`.}

\item{fig.type}{Character string specifying the file format for saving plots
(e.g., `"pdf"`, `"png"`). Default is `"pdf"`.}

\item{print_bar}{Logical indicating whether to save and print the bar plot.
Default is `TRUE`.}
}
\value{
List containing:
\describe{
  \item{up}{Data frame of up-regulated enriched gene sets}
  \item{down}{Data frame of down-regulated enriched gene sets}
  \item{all}{Complete GSEA results}
  \item{plot_top}{GSEA enrichment plot for top gene sets}
}
}
\description{
Conducts Gene Set Enrichment Analysis to identify significantly enriched gene sets
from differential gene expression data. Supports MSigDB gene sets or custom gene
signatures, and generates comprehensive visualizations and statistical results.
}
\examples{
set.seed(123)
genes <- c(
  "TP53", "BRCA1", "EGFR", "MYC", "KRAS", "PTEN", "APC", "RB1",
  "CDKN2A", "VHL", "ATM", "ATR", "CHEK2", "PALB2", "RAD51", "MDM2",
  "CDK4", "CDK6", "CCND1", "CCNE1", "CDK2", "E2F1", "E2F2", "E2F3",
  "ARF1", "ARF3", "ARF4", "ARF5", "ARF6", "GSK3B", "AKT1", "AKT2",
  "PIK3CA", "PIK3CB", "PIK3CD", "PIK3CG", "PIK3R1", "PIK3R2", "PIK3R3"
)
deg <- data.frame(
  symbol = genes,
  log2FoldChange = rnorm(length(genes), mean = 0, sd = 2),
  padj = runif(length(genes), 0, 0.1)
)
signature <- list(
  DNA_Repair = c(
    "TP53", "BRCA1", "ATM",
    "ATR", "CHEK2", "PALB2", "RAD51"
  ),
  Cell_Cycle = c(
    "TP53", "MYC",
    "RB1", "CDKN2A", "CDK4",
    "CDK6", "CCND1", "CCNE1",
    "CDK2", "E2F1", "E2F2", "E2F3"
  ),
  PI3K_AKT = c(
    "AKT1", "AKT2",
    "PIK3CA", "PIK3CB", "PIK3CD",
    "PIK3CG", "PIK3R1", "PIK3R2", "PIK3R3"
  )
)
\donttest{
res <- sig_gsea(
  deg = deg,
  genesets = signature,
  path = tempdir(),
  show_plot = FALSE,
  print_bar = FALSE
)
print(names(res))
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_sc_data.R
\name{extract_sc_data}
\alias{extract_sc_data}
\title{Extract Data Frame from Seurat Object}
\usage{
extract_sc_data(
  sce,
  vars = NULL,
  assay,
  slot = "scale.data",
  combine_meta_data = TRUE
)
}
\arguments{
\item{sce}{Seurat object.}

\item{vars}{Character vector of feature names to extract. If `NULL`, all
features are extracted.}

\item{assay}{Character vector specifying assay(s) to pull data from.}

\item{slot}{Character string specifying the assay data slot. Default is
`"scale.data"`.}

\item{combine_meta_data}{Logical indicating whether to combine metadata
with the extracted data frame. Default is `TRUE`.}
}
\value{
Data frame with cells as rows and features as columns.
}
\description{
Extracts and combines a data frame with cells as rows and features as columns
from Seurat assay data. Supports multiple assays and optional metadata
integration.
}
\examples{
if (requireNamespace("Seurat", quietly = TRUE)) {
  pbmc <- SeuratObject::pbmc_small
  vars <- c("PPBP", "IGLL5", "VDAC3", "CD1C", "AKR1C3")
  eset <- extract_sc_data(sce = pbmc, vars = vars, assay = "RNA")
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateRef_DEseq2.R
\name{generateRef_DEseq2}
\alias{generateRef_DEseq2}
\title{Generate Reference Signature Matrix Using DESeq2}
\usage{
generateRef_DEseq2(dds, pheno, FDR = 0.05, dat)
}
\arguments{
\item{dds}{Matrix. Raw count data from RNA-seq.}

\item{pheno}{Character vector. Cell type classes for samples.}

\item{FDR}{Numeric. Threshold for adjusted p-values. Default is 0.05.}

\item{dat}{Matrix. Normalized expression data (e.g., FPKM, TPM) for
calculating median expression.}
}
\value{
List containing:
  - `reference_matrix`: Data frame of median expression for significant
    genes across cell types.
  - `G`: Optimal number of probes minimizing condition number.
  - `condition_number`: Minimum condition number.
  - `whole_matrix`: Full median expression matrix.
}
\description{
Uses DESeq2 to perform differential expression analysis across cell types,
identifies significantly expressed genes, and creates a reference signature
matrix from median expression levels.
}
\examples{
set.seed(123)
dds <- matrix(sample(0:1000, 2000, replace = TRUE), nrow = 100, ncol = 20)
colnames(dds) <- paste("Sample", 1:20, sep = "_")
rownames(dds) <- paste("Gene", 1:100, sep = "_")
pheno <- rep(c("Type1", "Type2"), each = 10)
dat <- matrix(runif(2000), nrow = 100, ncol = 20)
rownames(dat) <- rownames(dds)
colnames(dat) <- colnames(dds)
\donttest{
result <- generateRef_DEseq2(dds = dds, pheno = pheno, FDR = 0.05, dat = dat)
print(result$reference_matrix)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_mut_matrix.R
\name{.create_mut_matrix}
\alias{.create_mut_matrix}
\title{Create Single Mutation Matrix}
\usage{
.create_mut_matrix(mut)
}
\arguments{
\item{mut}{Data frame with mutation data.}
}
\value{
Mutation matrix.
}
\description{
Create Single Mutation Matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sig_surv_plot.R
\name{sig_surv_plot}
\alias{sig_surv_plot}
\title{Generate Kaplan-Meier Survival Plot for Signature}
\usage{
sig_surv_plot(
  input_pdata,
  signature,
  project = "KM",
  ID = "ID",
  time = "time",
  status = "status",
  time_type = "month",
  break_month = "auto",
  cols = NULL,
  palette = "jama",
  show_col = TRUE,
  mini_sig = "score",
  fig.type = "png",
  save_path = NULL,
  index = 1
)
}
\arguments{
\item{input_pdata}{Data frame with survival data and signature scores.}

\item{signature}{Character string. Column name of the target signature.}

\item{project}{Character string. Project name for output. Default is `"KM"`.}

\item{ID}{Character string. Column name for sample IDs. Default is `"ID"`.}

\item{time}{Character string. Column name for survival time. Default is `"time"`.}

\item{status}{Character string. Column name for survival status. Default is `"status"`.}

\item{time_type}{Character string. Time unit (`"month"` or `"day"`).
Default is `"month"`.}

\item{break_month}{Numeric or `"auto"`. Time axis breaks. Default is `"auto"`.}

\item{cols}{Character vector. Optional custom colors.}

\item{palette}{Character string. Color palette if `cols` not provided.
Default is `"jama"`.}

\item{show_col}{Logical indicating whether to show colors. Default is `TRUE`.}

\item{mini_sig}{Character string. Label for low score group. Default is `"score"`.}

\item{fig.type}{Character string. File format. Default is `"png"`.}

\item{save_path}{Character string or `NULL`. Directory for saving plots.
If `NULL`, plots are not saved. Default is `NULL`.}

\item{index}{Integer. Index for multiple plots. Default is `1`.}
}
\value{
A list containing:
\describe{
  \item{data}{Processed input data with group assignments}
  \item{plots}{Combined survival plots}
}
}
\description{
Creates Kaplan-Meier survival plots for a given signature or gene, with
automatic cutoff determination. Generates three types of plots: optimal
cutoff (best cutoff), tertile-based (3 groups), and median split (2 groups).
}
\examples{
# Create small example data
set.seed(123)
test_data <- data.frame(
  time = runif(100, 0, 100),
  OS_status = sample(c(0, 1), 100, replace = TRUE),
  TMEscore_plus = rnorm(100),
  ID = paste0("S", 1:100)
)
sig_surv_plot(
  input_pdata = test_data,
  signature = "TMEscore_plus",
  time = "time",
  status = "OS_status"
)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iobr_pca.R
\name{iobr_pca}
\alias{iobr_pca}
\title{Principal Component Analysis (PCA) Visualization}
\usage{
iobr_pca(
  data,
  is.matrix = TRUE,
  scale = TRUE,
  is.log = FALSE,
  pdata,
  id_pdata = "ID",
  group = NULL,
  geom.ind = "point",
  cols = "normal",
  palette = "jama",
  repel = FALSE,
  ncp = 5,
  axes = c(1, 2),
  addEllipses = TRUE
)
}
\arguments{
\item{data}{Input data for PCA: matrix or data frame.}

\item{is.matrix}{Logical indicating if input is a matrix. Default is TRUE.}

\item{scale}{Logical indicating whether to scale the data. Default is TRUE.}

\item{is.log}{Logical indicating whether to log-transform the data. Default is FALSE.}

\item{pdata}{Data frame with sample IDs and grouping information.}

\item{id_pdata}{Column name in `pdata` for sample IDs. Default is "ID".}

\item{group}{Column name in `pdata` for grouping variable. Default is NULL.}

\item{geom.ind}{Type of geometric representation for points. Default is "point".}

\item{cols}{Color scheme for groups. Default is "normal".}

\item{palette}{Color palette for groups. Default is "jama".}

\item{repel}{Logical indicating whether to repel overlapping points. Default is FALSE.}

\item{ncp}{Number of principal components to retain. Default is 5.}

\item{axes}{Principal components to plot (e.g., c(1, 2)). Default is c(1, 2).}

\item{addEllipses}{Logical indicating whether to add concentration ellipses. Default is TRUE.}
}
\value{
A ggplot object of the PCA plot.
}
\description{
This function performs Principal Component Analysis (PCA) on gene
expression data,
reduces dimensionality while preserving variance, and generates a scatter
plot visualization.
}
\examples{
if (requireNamespace("FactoMineR", quietly = TRUE) &&
  requireNamespace("factoextra", quietly = TRUE)) {
  set.seed(123)
  eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
  rownames(eset) <- paste0("Gene", 1:100)
  colnames(eset) <- paste0("Sample", 1:10)
  pdata <- data.frame(
    ID = colnames(eset),
    group = rep(c("A", "B"), each = 5)
  )
  iobr_pca(eset, pdata = pdata, id_pdata = "ID", group = "group", addEllipses = FALSE)
}

}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timer.R
\name{check_cancer_types}
\alias{check_cancer_types}
\title{Process Batch Table and Validate Cancer Types}
\usage{
check_cancer_types(args)
}
\arguments{
\item{args}{A list containing input parameters:
\describe{
  \item{batch}{Character. Path to a CSV file (optional).}
  \item{expression}{Character vector of expression identifiers (used if
    `batch` is NULL).}
  \item{category}{Character vector of cancer types (used if `batch` is NULL).}
}}
}
\value{
A character matrix with two columns:
\describe{
  \item{Column 1}{Expression identifiers}
  \item{Column 2}{Cancer categories}
}
}
\description{
Processes input data containing cancer types and validates each category
against a predefined list of supported cancer types (`timer_available_cancers`).
}
\examples{
args <- list(
  expression = c("exp1", "exp2"),
  category = c("luad", "brca"),
  batch = NULL
)
result <- check_cancer_types(args)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateRef_seurat.R
\name{generateRef_seurat}
\alias{generateRef_seurat}
\title{Generate Reference Matrix from Seurat Object}
\usage{
generateRef_seurat(
  sce,
  celltype = NULL,
  proportion = NULL,
  assay_deg = "RNA",
  slot_deg = "data",
  adjust_assay = FALSE,
  assay_out = "RNA",
  slot_out = "data",
  verbose = FALSE,
  only.pos = TRUE,
  n_ref_genes = 50,
  logfc.threshold = 0.15,
  test.use = "wilcox"
)
}
\arguments{
\item{sce}{Seurat object containing single-cell RNA-seq data.}

\item{celltype}{Character. Cell type column name in metadata.
Default is `NULL` (uses default identity).}

\item{proportion}{Numeric. Proportion of cells to randomly select for
analysis. Default is `NULL` (use all cells).}

\item{assay_deg}{Character. Assay for finding markers. Default is `"RNA"`.}

\item{slot_deg}{Character. Slot for finding markers. Default is `"data"`.}

\item{adjust_assay}{Logical. Whether to adjust assay for SCT. Default is
`FALSE`.}

\item{assay_out}{Character. Assay for output. Default is `"RNA"`.}

\item{slot_out}{Character. Slot for output. Default is `"data"`.}

\item{verbose}{Logical. Print verbose messages. Default is `FALSE`.}

\item{only.pos}{Logical. Return only positive markers. Default is `TRUE`.}

\item{n_ref_genes}{Integer. Number of reference genes per cell type.
Default is 50.}

\item{logfc.threshold}{Numeric. Log fold change threshold. Default is 0.15.}

\item{test.use}{Character. Statistical test for marker identification.
Default is `"wilcox"`.}
}
\value{
Matrix containing aggregated expression data for reference genes.
}
\description{
Generates reference gene expression data from a Seurat object by
identifying marker genes for each cell type and aggregating expression
data.
}
\examples{
\dontrun{
if (requireNamespace("Seurat", quietly = TRUE)) {
  # Requires a Seurat object with sufficient cells and markers
  sm <- generateRef_seurat(sce = seurat_obj, celltype = "cell_type", slot_out = "data")
}
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_data.R
\name{reset_iobr_mirrors}
\alias{reset_iobr_mirrors}
\title{Reset Download Mirrors to Default}
\usage{
reset_iobr_mirrors()
}
\value{
Invisibly returns the default mirror list.
}
\description{
Resets the download mirror list to the default values.
}
\examples{
reset_iobr_mirrors()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EPIC_helper.R
\name{EPIC}
\alias{EPIC}
\title{Estimate the proportion of immune and cancer cells.}
\usage{
EPIC(
  bulk,
  reference = NULL,
  mRNA_cell = NULL,
  mRNA_cell_sub = NULL,
  sigGenes = NULL,
  scaleExprs = TRUE,
  withOtherCells = TRUE,
  constrainedSum = TRUE,
  rangeBasedOptim = FALSE
)
}
\arguments{
\item{bulk}{A matrix (\code{nGenes} x \code{nSamples}) of the genes
expression from each bulk sample (the counts should be given in TPM,
RPKM or FPKM when using the prebuilt reference profiles). This matrix
needs to have rownames telling the gene names (corresponds to the gene
symbol in the prebuilt reference profiles (e.g. CD8A, MS4A1) - no
conversion of IDs is performed at the moment).
It is advised to keep all genes in the bulk instead of a subset of
signature genes (except if \code{scaleExprs = FALSE} in which case it
doesn't make any difference).}

\item{reference}{(optional): A string or a list defining the reference cells.
It can take multiple formats:
- `NULL`: to use the default reference profiles and genes signature
\code{TRef}
- A character: either \code{"BRef"} or \code{"TRef"} to use the
reference cells and genes signature of the corresponding datasets (see
\code{BRef} and \code{TRef})
- A list containing:
  - `$refProfiles`: a matrix (\code{nGenes} x \code{nCellTypes}) of the
  reference cells genes expression (don't include a column of the 'other
  cells' (representing usually the cancer cells for which such a profile
  is usually not conserved between samples); the rownames needs to be
  defined as well as the colnames giving the names of each gene and
  reference cell types respectively. It is advised to keep all genes in
  this \code{refProfiles} matrix instead of a subset of signature genes
  - `$sigGenes`: a character vector of the gene names to use as
  signature - sigGenes can also be given as a direct input to EPIC
  function
  - `$refProfiles.var` (optional): a matrix (\code{nGenes} x
  \code{nCellTypes}) of the variability of each gene expression for each
  cell type, which is used to define weights on each gene for the
  optimization (if this is absent, we assume an identical variability
  for all genes in all cells) - it needs to have the same dimnames than
  refProfiles}

\item{mRNA_cell}{(optional): A named numeric vector: tells (in arbitrary
units) the amount of mRNA for each of the reference cells and of the
other uncharacterized (cancer) cell. Two names are of special meaning:
\emph{"otherCells"} - used for the mRNA/cell value of the "other cells"
from the sample (i.e. the cell type that don't have any reference gene
expression profile) ; and \emph{default} - used for the mRNA/cell of the
cells from the reference profiles for which no specific value is given
in mRNA_cell (i.e. if mRNA_cell=c(Bcells=2, NKcells=2.1, otherCells=3.5,
default=1), then if the refProfiles described Bcells, NKcells and Tcells,
we would use a value of 3.5 for the "otherCells" that didn't have any
reference profile and a default value of 1 for the Tcells when computing
the cell fractions).
To note: if data is in tpm, this mRNA per cell would ideally correspond
to some number of transcripts per cell.}

\item{mRNA_cell_sub}{(optional): This can be given instead of
\code{mRNA_cell} (or in addition to it). It is also a named numeric vector,
used to replace only the mRNA/cell values from some cell types (or to add
values for new cell types). The values given in mRNA_cell_sub will overwrite
the default values as well as those that might have been given by mRNA_cell.}

\item{sigGenes}{(optional): a character vector of the gene names to use as
signature for the deconvolution. In principle this is given with the
reference as the "reference$sigGenes" but if we give a value for this
input variable, it is these signature genes that will be used instead of
the ones given with the reference profile.}

\item{scaleExprs}{(optional, default is TRUE): boolean telling if the bulk
samples and reference gene expression profiles should be rescaled based on
the list of genes in common between the them (such a rescaling is
recommanded).}

\item{withOtherCells}{(optional, default is TRUE): if EPIC should allow for
an additional cell type for which no gene expression reference profile is
available or if the bulk is assumed to be composed only of the cells with
reference profiles.}

\item{constrainedSum}{(optional, default is TRUE): tells if the sum of all
cell types should be constrained to be < 1. When
\code{withOtherCells=FALSE}, there is additionally a constrain the the sum
of all cell types with reference profiles must be > 0.99.}

\item{rangeBasedOptim}{(optional): when this is FALSE (the default), the
least square optimization is performed as described in
\href{https://elifesciences.org/articles/26476}{
 \cite{Racle et al., 2017, eLife}}, which is recommanded.
When this variable is TRUE, EPIC uses the variability of each gene
from the reference profiles in another way: instead of defining weights
(based on the variability) for the fit of each gene, we define a range of
values accessible for each gene (based on the gene expression value in
the reference profile +/- the variability values). The
error that the optimization tries to minimize is by how much
the predicted gene expression is outside of this allowed range of values.}
}
\value{
A list of 3 matrices:
- `mRNAProportions`: (\code{nSamples} x (\code{nCellTypes+1})) the
proportion of mRNA coming from all cell types with a ref profile + the
uncharacterized other cell
- `cellFractions`: (\code{nSamples} x (\code{nCellTypes+1})) this gives the
proportion of cells from each cell type after accounting for the mRNA /
cell value
- `fit.gof`: (\code{nSamples} x 12) a matrix telling the quality for the
fit of the signature genes in each sample. It tells if the minimization
converged, and other info about this fit comparing the measured gene
expression in the sigGenes vs predicted gene expression in the sigGenes
}
\description{
\code{EPIC} takes as input bulk gene expression data (RNA-seq) and returns
 the proportion of mRNA and cells composing the various samples.
}
\details{
This function uses a constrained least square minimization to estimate the
proportion of each cell type with a reference profile and another
uncharacterized cell type in bulk gene expression samples.

The names of the genes in the bulk samples, the reference samples and in the
gene signature list need to be the same format (gene symbols are used in the
predefined reference profiles). The full list of gene names don't need to be
exactly the same between the reference and bulk samples: \emph{EPIC}
will use the intersection of the genes. In case of duplicate gene names,
\emph{EPIC} will use the median value per duplicate - if you want to consider
these cases differently, you can remove the duplicates before calling
\emph{EPIC}.
}
\examples{
melanoma_counts <- matrix(abs(rnorm(80)), nrow = 20, ncol = 4)
rownames(melanoma_counts) <- paste0("Gene", 1:20)
colnames(melanoma_counts) <- paste0("Sample", 1:4)
mock_ref <- list(
  refProfiles = matrix(abs(rnorm(80)), nrow = 20, ncol = 4),
  sigGenes = paste0("Gene", 1:10)
)
rownames(mock_ref$refProfiles) <- paste0("Gene", 1:20)
colnames(mock_ref$refProfiles) <- c("Bcells", "CD4T", "CD8T", "NK")
res1 <- EPIC(melanoma_counts, reference = mock_ref)
if (!is.null(res1)) head(res1$cellFractions)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IPS_helper.R
\name{ipsmap}
\alias{ipsmap}
\title{Map Score to Immunophenoscore}
\usage{
ipsmap(x)
}
\arguments{
\item{x}{Numeric value representing the aggregate z-score.}
}
\value{
Integer value between 0 and 10 representing the Immunophenoscore.
}
\description{
Maps input score to Immunophenoscore (IPS) on a 0-10 scale. Scores \eqn{\le}0 map to 0,
scores \eqn{\ge}3 map to 10, and intermediate scores are linearly scaled.
}
\examples{
ips <- ipsmap(2.5)
ips <- ipsmap(-1)
ips <- ipsmap(5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BinomialModel.R
\name{SplitTrainTest}
\alias{SplitTrainTest}
\title{Split Data into Training and Testing Sets}
\usage{
SplitTrainTest(x, y, train_ratio, type = c("binomial", "survival"), seed)
}
\arguments{
\item{x}{Predictor matrix or data frame.}

\item{y}{Outcome vector (binomial) or matrix with time/status (survival).}

\item{train_ratio}{Proportion for training (0-1). Default is `0.7`.}

\item{type}{Analysis type: `"binomial"` or `"survival"`.}

\item{seed}{Random seed for reproducibility.}
}
\value{
List containing:
\describe{
  \item{train.x}{Training predictors matrix}
  \item{train.y}{Training outcomes}
  \item{test.x}{Testing predictors matrix}
  \item{test.y}{Testing outcomes}
  \item{train_sample}{Indices of training samples}
}
}
\description{
Divides dataset into training and testing sets using random sampling.
Maintains data integrity for both binomial and survival analysis types.
}
\examples{
data_matrix <- matrix(rnorm(200), ncol = 2)
outcome_vector <- rbinom(100, 1, 0.5)
split_data <- SplitTrainTest(
  data_matrix, outcome_vector,
  train_ratio = 0.7,
  type = "binomial", seed = 123
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_pcc.R
\name{batch_pcc}
\alias{batch_pcc}
\title{Batch Calculation of Partial Correlation Coefficients}
\usage{
batch_pcc(
  input,
  interferenceid,
  target,
  features,
  method = c("pearson", "spearman", "kendall")
)
}
\arguments{
\item{input}{Data frame containing feature variables, target variable, and
interference variable.}

\item{interferenceid}{Character string specifying the column name of the
interference (confounding) variable to control for.}

\item{target}{Character string specifying the column name of the target
variable.}

\item{features}{Character vector specifying the column names of feature
variables to correlate with the target.}

\item{method}{Character string specifying the correlation method. Options are
`"pearson"`, `"spearman"`, or `"kendall"`. Default is `"pearson"`.}
}
\value{
Tibble containing the following columns for each feature:
\describe{
  \item{sig_names}{Feature name}
  \item{p.value}{Raw p-value}
  \item{statistic}{Partial correlation coefficient}
  \item{p.adj}{Adjusted p-value (Benjamini-Hochberg method)}
  \item{log10pvalue}{Negative log10-transformed p-value}
  \item{stars}{Significance stars: **** p.adj<0.0001, *** p.adj<0.001,
    ** p.adj<0.01, * p.adj<0.05, + p.adj<0.5}
}
}
\description{
Computes partial correlation coefficients between multiple features and a
target variable while controlling for an interference (confounding) variable.
Adjusts p-values for multiple testing using the Benjamini-Hochberg method.
}
\examples{
# Create small example data
set.seed(123)
test_data <- data.frame(
  TumorPurity = runif(100),
  TargetVar = rnorm(100),
  Signature1 = rnorm(100),
  Signature2 = rnorm(100)
)
# Calculate partial correlations controlling for tumor purity
res <- batch_pcc(
  input = test_data,
  interferenceid = "TumorPurity",
  target = "TargetVar",
  method = "pearson",
  features = c("Signature1", "Signature2")
)
head(res)
}
\author{
Rongfang Shen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PrognosticModel.R
\name{PlotTimeROC}
\alias{PlotTimeROC}
\title{Plot Time-Dependent ROC Curves}
\usage{
PlotTimeROC(
  train.x,
  train.y,
  test.x,
  test.y,
  model,
  modelname,
  cols = NULL,
  palette = "jama"
)
}
\arguments{
\item{train.x}{Matrix or data frame of training predictors.}

\item{train.y}{Training survival outcomes (time and status).}

\item{test.x}{Matrix or data frame of testing predictors.}

\item{test.y}{Testing survival outcomes (time and status).}

\item{model}{Fitted survival model object.}

\item{modelname}{Character string for model identification.}

\item{cols}{Optional vector of colors for plotting.}

\item{palette}{Character string specifying color palette. Default is `"jama"`.}
}
\value{
A `ggplot` object representing the ROC curve plot.
}
\description{
Generates time-dependent ROC curves for evaluating prognostic accuracy of survival models.
Plots training and testing ROC curves at the 90th percentile survival time.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE) &&
  requireNamespace("survival", quietly = TRUE) &&
  requireNamespace("timeROC", quietly = TRUE)) {
  library(survival)
  set.seed(123)
  train_x <- matrix(rnorm(100 * 5), ncol = 5)
  train_y <- data.frame(time = rexp(100), status = rbinom(100, 1, 0.5))
  test_x <- matrix(rnorm(50 * 5), ncol = 5)
  test_y <- data.frame(time = rexp(50), status = rbinom(50, 1, 0.5))
  fit <- glmnet::cv.glmnet(train_x, Surv(train_y$time, train_y$status), family = "cox")
  p <- PlotTimeROC(train_x, train_y, test_x, test_y, fit, "Cox Model")
  print(p)
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BinomialModel.R
\name{PlotAUC}
\alias{PlotAUC}
\title{Plot AUC ROC Curves}
\usage{
PlotAUC(
  train.x,
  train.y,
  test.x,
  test.y,
  model,
  modelname,
  cols = NULL,
  palette = "jama"
)
}
\arguments{
\item{train.x}{Training predictors matrix.}

\item{train.y}{Training outcomes (binary factor).}

\item{test.x}{Testing predictors matrix.}

\item{test.y}{Testing outcomes (binary factor).}

\item{model}{Fitted cv.glmnet model.}

\item{modelname}{Character string for plot title.}

\item{cols}{Optional color vector for ROC curves.}

\item{palette}{Color palette name from IOBR palettes. Default is `"jama"`.}
}
\value{
ggplot object of ROC curves.
}
\description{
Generates ROC curves for model evaluation comparing training and testing
performance at both lambda.min and lambda.1se. Creates a ggplot visualization
with AUC values in the legend.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  set.seed(123)
  train_data <- matrix(rnorm(100 * 5), ncol = 5)
  train_outcome <- rbinom(100, 1, 0.5)
  test_data <- matrix(rnorm(50 * 5), ncol = 5)
  test_outcome <- rbinom(50, 1, 0.5)
  fitted_model <- glmnet::cv.glmnet(train_data, train_outcome, family = "binomial", nfolds = 5)
  p <- PlotAUC(train_data, train_outcome, test_data, test_outcome, fitted_model, "MyModel")
  print(p)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_signatures.R
\name{format_signatures}
\alias{format_signatures}
\title{Transform Signature Data into List Format}
\usage{
format_signatures(sig_data, save_signature = FALSE, output_path = NULL)
}
\arguments{
\item{sig_data}{Data frame with signature names as columns and genes in
rows. Use `NA` for missing values.}

\item{save_signature}{Logical. Whether to save the signature list as RData.
Default is `FALSE`.}

\item{output_path}{Character. Full path (without extension) for output file.
Required if `save_signature = TRUE`. Default is `NULL`.}
}
\value{
List of signatures.
}
\description{
Converts signature data from a data frame (with signatures as columns and
genes as rows) into a list format suitable for IOBR functions. Handles
NA values appropriately.
}
\examples{
sig_data <- data.frame(
  Signature1 = c("Gene1", "Gene2", "Gene3", NA),
  Signature2 = c("Gene4", "Gene5", NA, NA)
)
format_signatures(sig_data)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_variable_genes.R
\name{find_variable_genes}
\alias{find_variable_genes}
\title{Identify Variable Genes in Expression Data}
\usage{
find_variable_genes(
  eset,
  data_type = c("count", "normalized"),
  methods = c("low", "mad"),
  prop = 0.7,
  quantile = 0.75,
  min.mad = 0.1,
  feas = NULL
)
}
\arguments{
\item{eset}{Numeric matrix. Gene expression data (genes as rows,
samples as columns).}

\item{data_type}{Character. Type of data: `"count"` or `"normalized"`.
Default is `"count"`.}

\item{methods}{Character vector. Methods for gene selection: `"low"`,
`"mad"`. Default is `c("low", "mad")`.}

\item{prop}{Numeric. Proportion of samples in which a gene must be expressed.
Default is 0.7.}

\item{quantile}{Numeric. Quantile threshold for minimum MAD (0.25, 0.5, 0.75).
Default is 0.75.}

\item{min.mad}{Numeric. Minimum allowable MAD value. Default is 0.1.}

\item{feas}{Character vector or `NULL`. Additional features to include.
Default is `NULL`.}
}
\value{
Matrix subset of `eset` containing variable genes.
}
\description{
Identifies variable genes from a gene expression dataset using specified
selection criteria. Supports multiple methods, including expression
thresholding and variability estimation via median absolute deviation (MAD).
}
\examples{
# Simulate data
set.seed(123)
sim_eset <- matrix(rnorm(100 * 20), 100, 20)
rownames(sim_eset) <- paste0("Gene", 1:100)
colnames(sim_eset) <- paste0("Sample", 1:20)

# Identify variable genes
eset_var <- find_variable_genes(
  eset = sim_eset,
  data_type = "normalized",
  methods = "mad",
  quantile = 0.25
)
head(eset_var)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iobr_deg.R
\name{iobr_deg}
\alias{iobr_deg}
\title{Differential Expression Analysis}
\usage{
iobr_deg(
  eset,
  annotation = NULL,
  id_anno = NULL,
  pdata,
  group_id = "group",
  pdata_id = "ID",
  array = FALSE,
  method = c("DESeq2", "limma"),
  contrast = c("High", "Low"),
  path = NULL,
  padj_cutoff = 0.01,
  logfc_cutoff = 0.5,
  volcano_plot = FALSE,
  col_volcano = 1,
  heatmap = TRUE,
  col_heatmap = 1,
  parallel = FALSE
)
}
\arguments{
\item{eset}{A matrix of gene expression data where rows represent genes and
columns represent samples.}

\item{annotation}{Optional data frame for mapping gene IDs to gene names.
Default is `NULL`.}

\item{id_anno}{Character string specifying the identifier column in annotation.
Default is `NULL`.}

\item{pdata}{A data frame containing sample information and grouping labels.}

\item{group_id}{Character string specifying the column name in `pdata` containing
grouping labels. Default is `"group"`.}

\item{pdata_id}{Character string specifying the column name in `pdata` for sample IDs.
Default is `"ID"`.}

\item{array}{Logical indicating whether to perform quantile normalization.
Default is `FALSE`.}

\item{method}{Character string specifying the method: `"DESeq2"` or `"limma"`.
Default is `"DESeq2"`.}

\item{contrast}{Character vector of length 2 specifying contrast groups.
Default is `c("High", "Low")`.}

\item{path}{Character string for output directory. Default is `NULL`.}

\item{padj_cutoff}{Numeric cutoff for adjusted p-values. Default is `0.01`.}

\item{logfc_cutoff}{Numeric log2 fold change cutoff. Default is `0.5`.}

\item{volcano_plot}{Logical indicating whether to generate a volcano plot.
Default is `FALSE`.}

\item{col_volcano}{Integer specifying color index for volcano plot. Default is `1`.}

\item{heatmap}{Logical indicating whether to generate a heatmap. Default is `TRUE`.}

\item{col_heatmap}{Integer specifying color index for heatmap. Default is `1`.}

\item{parallel}{Logical indicating whether to run in parallel. Default is `FALSE`.}
}
\value{
Data frame containing differentially expressed genes with statistics
  including log2 fold changes and adjusted p-values.
}
\description{
Performs differential expression analysis on gene expression data using either
DESeq2 or limma. Includes pre-processing steps like filtering low count data,
and calculates fold changes and adjusted p-values. Optionally generates
volcano plots and heatmaps.
}
\examples{
# Simulate data
set.seed(123)
sim_eset <- matrix(abs(rnorm(100 * 20)), 100, 20)
rownames(sim_eset) <- paste0("Gene", 1:100)
colnames(sim_eset) <- paste0("Sample", 1:20)

sim_pdata <- data.frame(
  ID = paste0("Sample", 1:20),
  group = rep(c("High", "Low"), each = 10)
)

# Run DEG analysis
deg <- iobr_deg(
  eset = sim_eset, pdata = sim_pdata,
  group_id = "group", pdata_id = "ID",
  method = "limma", contrast = c("High", "Low"),
  heatmap = FALSE
)
if (!is.null(deg)) head(deg)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_data.R
\name{list_github_datasets}
\alias{list_github_datasets}
\title{List Available GitHub Datasets}
\usage{
list_github_datasets()
}
\value{
Character vector of available dataset names.
}
\description{
List Available GitHub Datasets
}
\examples{
list_github_datasets()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_doc.R
\docType{data}
\name{sig_group}
\alias{sig_group}
\title{Grouped gene signatures for IOBR analysis}
\format{
A named list of length 43. Each element is a character vector of
  signature names. Representative groups include:
  \describe{
    \item{tumor_signature}{Signatures related to intrinsic tumour biology
      such as cell cycle, DNA damage repair and histone regulation.}
    \item{EMT}{Epithelial–mesenchymal transition (EMT)–associated
    signatures.}
    \item{io_biomarkers}{Immuno-oncology biomarker–related signatures.}
    \item{immu_microenvironment}{Immune microenvironment–related signatures.}
    \item{immu_suppression}{Immune suppression–related signatures.}
    \item{immu_exclusion}{Signatures associated with immune exclusion and
      stromal barriers.}
    \item{TCR_BCR}{T-cell and B-cell receptor pathway signatures.}
    \item{tme_signatures1}{Tumour microenvironment signature panel (set 1).}
    \item{tme_signatures2}{Tumour microenvironment signature panel (set 2).}
    \item{...}{Additional groups are included (43 total), but not listed
      individually here; all groups follow the same structure.}
  }
}
\usage{
data(sig_group)
}
\description{
A named list that organizes gene signatures into functional or biological
categories. Each element of the list is a character vector containing the
names of gene signatures defined in \code{signature_collection}. A total of
43 signature groups are included, covering tumour intrinsic pathways,
immune-related processes, stromal activity, TME characteristics and
immuno-oncology biomarkers. These groups are used in IOBR to conveniently
select sets of signatures for scoring and visualization.
}
\examples{
data(sig_group)
head(sig_group)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PrognosticModel.R
\name{PrognosticModel}
\alias{PrognosticModel}
\title{Build Prognostic Models Using LASSO and Ridge Regression}
\usage{
PrognosticModel(
  x,
  y,
  scale = FALSE,
  seed = 123456,
  train_ratio = 0.7,
  nfold = 10,
  plot = TRUE,
  palette = "jama",
  cols = NULL
)
}
\arguments{
\item{x}{A matrix or data frame of predictor variables (features).}

\item{y}{A data frame of survival outcomes with two columns: survival time and event status.}

\item{scale}{Logical indicating whether to scale predictor variables. Default is `FALSE`.}

\item{seed}{Integer seed for random number generation to ensure reproducibility.
Default is `123456`.}

\item{train_ratio}{Numeric proportion of data for training (e.g., 0.7). Default is `0.7`.}

\item{nfold}{Integer number of folds for cross-validation. Default is `10`.}

\item{plot}{Logical indicating whether to plot ROC curves. Default is `TRUE`.}

\item{palette}{String specifying color palette. Default is `"jama"`.}

\item{cols}{Optional vector of colors for ROC curves. If `NULL`, uses default palette.}
}
\value{
A list containing:
\describe{
  \item{lasso_result}{Results from LASSO model including coefficients and AUC}
  \item{ridge_result}{Results from Ridge model including coefficients and AUC}
  \item{train.x}{Training data with sample IDs}
}
}
\description{
Prepares data, splits it into training and testing sets, and fits LASSO and Ridge
regression models for survival analysis. Evaluates model performance using
cross-validation and optionally generates time-dependent ROC curves for visual
assessment of predictive accuracy.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE) &&
  requireNamespace("survival", quietly = TRUE)) {
  library(survival)
  set.seed(123)
  # Create small example data (first column must be ID)
  x_sim <- as.data.frame(matrix(rnorm(100 * 5), 100, 5))
  colnames(x_sim) <- paste0("Sig", 1:5)
  x_sim$ID <- paste0("S", 1:100)
  x_sim <- x_sim[, c(6, 1:5)] # Move ID to first column

  y_sim <- data.frame(
    ID = paste0("S", 1:100),
    OS_days = rexp(100, 0.01),
    OS_status = rbinom(100, 1, 0.5)
  )
  prognostic_result <- PrognosticModel(
    x = x_sim, y = y_sim,
    scale = TRUE, seed = 123456,
    train_ratio = 0.7, nfold = 3, plot = FALSE
  )
  if (!is.null(prognostic_result)) head(prognostic_result)
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantiseq_helper.R
\name{fixMixture}
\alias{fixMixture}
\title{Fix Expression Mixture Matrix}
\usage{
fixMixture(mix.mat, arrays = FALSE)
}
\arguments{
\item{mix.mat}{Expression matrix with genes as rows.}

\item{arrays}{Logical indicating if data is from arrays. Default is FALSE.}
}
\value{
Processed expression matrix.
}
\description{
Processes expression matrix by mapping genes, converting log values,
quantile normalization (if arrays), and TPM normalization.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_sig_sc.R
\name{get_sig_sc}
\alias{get_sig_sc}
\title{Extract Top Marker Genes from Single-Cell Differential Results}
\usage{
get_sig_sc(
  deg,
  cluster = "cluster",
  gene = "gene",
  avg_log2FC = "avg_log2FC",
  n = 100
)
}
\arguments{
\item{deg}{Data frame or matrix. Ranked marker statistics.}

\item{cluster}{Character. Column name containing cluster identifiers.
Default is `"cluster"`.}

\item{gene}{Character. Column name containing gene identifiers.
Default is `"gene"`.}

\item{avg_log2FC}{Character. Column name for average log2 fold change.
Default is `"avg_log2FC"`.}

\item{n}{Integer. Number of top markers per cluster. Default is 100.}
}
\value{
List of character vectors; each element contains the top N genes
  for a cluster.
}
\description{
Selects the top N marker genes per cluster from a ranked differential
expression result table.
}
\examples{
# Simulate marker data
set.seed(123)
sim_deg <- data.frame(
  cluster = rep(c("A", "B"), each = 50),
  gene = paste0("Gene", 1:100),
  avg_log2FC = rnorm(100, 2, 1)
)

# Extract top 5 markers per cluster
markers <- get_sig_sc(sim_deg, n = 5)
print(markers)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xCell.R
\name{xCellSignifcanceRandomMatrix}
\alias{xCellSignifcanceRandomMatrix}
\title{Calculate Significance Using Random Matrix}
\usage{
xCellSignifcanceRandomMatrix(
  scores,
  expr,
  spill,
  alpha = 0.5,
  nperm = 250,
  file.name = NULL
)
}
\arguments{
\item{scores}{xCell scores matrix.}

\item{expr}{Input expression matrix.}

\item{spill}{Spillover object.}

\item{alpha}{Spillover alpha parameter. Default is `0.5`.}

\item{nperm}{Number of permutations. Default is `250`.}

\item{file.name}{Character string for saving p-values. Default is `NULL`.}
}
\value{
List containing p-values, shuffled xCell scores, shuffled expression,
  and beta distributions.
}
\description{
Calculates FDR-adjusted p-values using a random shuffled matrix.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anno_eset.R
\name{anno_eset}
\alias{anno_eset}
\title{Annotate Gene Expression Matrix and Remove Duplicated Genes}
\usage{
anno_eset(
  eset,
  annotation,
  symbol = "symbol",
  probe = "probe_id",
  method = "mean"
)
}
\arguments{
\item{eset}{Expression matrix or ExpressionSet object containing gene
expression data.}

\item{annotation}{Data frame containing annotation information for probes.
Built-in options include `anno_hug133plus2`, `anno_rnaseq`, and
`anno_illumina`.}

\item{symbol}{Character string specifying the column name in `annotation`
that represents gene symbols. Default is `"symbol"`.}

\item{probe}{Character string specifying the column name in `annotation`
that represents probe identifiers. Default is `"probe_id"`.}

\item{method}{Character string specifying the aggregation method for
duplicate gene symbols. Options are `"mean"`, `"sum"`, or `"sd"`.
Default is `"mean"`.}
}
\value{
Annotated and cleaned gene expression matrix with gene symbols as
  row names.
}
\description{
Annotates an expression matrix with gene symbols using provided annotation
data,
filters out missing or invalid symbols, handles duplicate gene entries, and
removes uninformative rows. The function supports multiple aggregation
methods
for resolving duplicate gene symbols.
}
\details{
The function performs the following operations:
\enumerate{
  \item Filters probes with missing symbols or labeled as `"NA_NA"`
  \item Matches probes between expression set and annotation data
  \item Merges annotation with expression data
  \item Handles duplicate gene symbols using specified aggregation method
  \item Removes rows with all zeros, all NAs, or missing values in the first
    column
}
}
\examples{
# Create a small example expression matrix
eset_mat <- matrix(runif(100), nrow = 10, ncol = 10)
rownames(eset_mat) <- paste0("Probe", 1:10)
colnames(eset_mat) <- paste0("Sample", 1:10)

# Create a matching annotation data frame
anno_df <- data.frame(
  probe_id = paste0("Probe", 1:10),
  symbol = c("Gene1", "Gene1", "Gene2", "Gene3", "Gene4",
             "Gene5", "Gene6", "Gene7", "Gene8", "Gene9")
)

# Annotate
result <- anno_eset(eset = eset_mat, annotation = anno_df)
head(result)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timer.R
\name{GetFractions.Abbas}
\alias{GetFractions.Abbas}
\title{Constrained Regression Method (Abbas et al., 2009)}
\usage{
GetFractions.Abbas(XX, YY, w = NA)
}
\arguments{
\item{XX}{Matrix. Immune expression data with genes as rows and cell types
as columns.}

\item{YY}{Vector. Cancer expression data with gene expression levels.}

\item{w}{Vector or NA. Weights for regression. Default is NA (no weights).}
}
\value{
Vector with non-negative coefficients representing proportions of
  each cell type.
}
\description{
Implements a constrained regression approach described by Abbas et al. (2009).
Estimates proportions of immune cell types within mixed cancer tissue samples
based on gene expression data. Iteratively adjusts regression coefficients
to ensure non-negative values.
}
\examples{
XX <- matrix(runif(100), nrow = 10, ncol = 10)
colnames(XX) <- paste("CellType", 1:10, sep = "")
YY <- runif(10)
results <- GetFractions.Abbas(XX, YY)
print(results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_marker_in_bulk.R
\name{find_markers_in_bulk}
\alias{find_markers_in_bulk}
\title{Identify Marker Features in Bulk Expression Data}
\usage{
find_markers_in_bulk(
  pdata,
  eset,
  group,
  id_pdata = "ID",
  nfeatures = 2000,
  top_n = 20,
  thresh.use = 0.25,
  only.pos = TRUE,
  min.pct = 0.25,
  npcs = 30
)
}
\arguments{
\item{pdata}{Data frame. Sample metadata.}

\item{eset}{Matrix. Gene expression or signature score matrix.}

\item{group}{Character. Column name in pdata specifying grouping variable.}

\item{id_pdata}{Character. Column name for sample IDs. Default is "ID".}

\item{nfeatures}{Integer. Number of top variable features to select. Default is 2000.}

\item{top_n}{Integer. Number of top markers to retain per cluster. Default is 20.}

\item{thresh.use}{Numeric. Threshold for marker selection. Default is 0.25.}

\item{only.pos}{Logical. Whether to retain only positive markers. Default is TRUE.}

\item{min.pct}{Numeric. Minimum expression percentage threshold. Default is 0.25.}

\item{npcs}{Integer. Number of principal components to use. Default is 30.}
}
\value{
List with components: `sce` (Seurat object), `markers` (all markers), `top_markers` (top markers per group).
}
\description{
Identifies informative marker features across groups from bulk gene
expression or signature score matrices using Seurat workflows. Performs
feature selection, scaling, PCA, clustering, and marker discovery.
}
\examples{
if (requireNamespace("Seurat", quietly = TRUE) && requireNamespace("Matrix", quietly = TRUE)) {
  # Simulate data
  set.seed(123)
  sim_eset <- matrix(abs(rnorm(100 * 30)), 100, 30)
  rownames(sim_eset) <- paste0("Gene", 1:100)
  colnames(sim_eset) <- paste0("Sample", 1:30)
  
  sim_pdata <- data.frame(
    ID = paste0("Sample", 1:30),
    TMEcluster = rep(c("A", "B", "C"), each = 10)
  )
  
  res <- find_markers_in_bulk(
    pdata = sim_pdata, eset = sim_eset,
    group = "TMEcluster", npcs = 5
  )
  if (!is.null(res)) head(res$top_markers)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xCell.R
\name{microenvironmentScores}
\alias{microenvironmentScores}
\title{Calculate Microenvironment Scores}
\usage{
microenvironmentScores(adjustedScores)
}
\arguments{
\item{adjustedScores}{Adjusted xCell scores matrix.}
}
\value{
Matrix with additional microenvironment score rows.
}
\description{
Calculates combined immune and stroma scores from adjusted xCell scores.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_doc.R
\docType{data}
\name{imvigor210_pdata}
\alias{imvigor210_pdata}
\title{IMvigor210 Bladder Cancer Immunotherapy Cohort Data}
\format{
A data frame with patients as rows and variables as columns:
\describe{
  \item{ID}{Patient sample identifier}
  \item{BOR}{Best overall response (CR, PR, SD, PD, NA)}
  \item{BOR_binary}{Binary response classification (R=responder,
  NR=non-responder)}
  \item{OS_days}{Overall survival time in days}
  \item{OS_status}{Overall survival status (0=alive, 1=dead)}
  \item{Mutation_Load}{Tumor mutation burden}
  \item{Neo_antigen_Load}{Neoantigen load}
  \item{CD_8_T_effector}{CD8+ T effector signature score}
  \item{Immune_Checkpoint}{Immune checkpoint signature score}
  \item{Pan_F_TBRs}{Pan-fibroblast TGF-\eqn{\beta} response signature}
  \item{Mismatch_Repair}{Mismatch repair status or signature}
  \item{TumorPurity}{Estimated tumor purity}
}
}
\source{
IMvigor210 clinical trial (NCT02108652)
}
\usage{
data(imvigor210_pdata)
}
\description{
Clinical and biomarker data from the IMvigor210 clinical trial cohort.
Includes treatment response, survival outcomes, and immune biomarker
measurements for bladder cancer patients treated with atezolizumab.
}
\examples{
data(imvigor210_pdata)
head(imvigor210_pdata)
}
\references{
Mariathasan S et al. TGF\eqn{\beta} attenuates tumour response to PD-L1 blockade
by contributing to exclusion of T cells. Nature 554, 544-548 (2018).
doi:10.1038/nature25501
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_data.R
\name{clear_iobr_cache}
\alias{clear_iobr_cache}
\title{Clear IOBR Data Cache}
\usage{
clear_iobr_cache(cache_dir = NULL)
}
\arguments{
\item{cache_dir}{Character string. Custom cache directory. If NULL, uses
the option `IOBR.cache_dir` or the default system cache location.}
}
\value{
Invisible NULL. Called for side effects of clearing the cache.
}
\description{
Removes all cached data files downloaded from GitHub.
}
\examples{
clear_iobr_cache()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_data.R
\name{download_iobr_data}
\alias{download_iobr_data}
\title{Download IOBR Data from GitHub with Mirror Support}
\usage{
download_iobr_data(
  name,
  force = FALSE,
  verbose = TRUE,
  mirrors = get_default_mirrors(),
  cache_dir = NULL
)
}
\arguments{
\item{name}{Character string. Name of the dataset to download.}

\item{force}{Logical. Whether to force re-download even if cached. Default: FALSE.}

\item{verbose}{Logical. Whether to print progress messages. Default: TRUE.}

\item{mirrors}{Character vector. URLs of mirrors to try. Default uses
get_default_mirrors().}

\item{cache_dir}{Character string. Custom cache directory. If NULL, uses
the option `IOBR.cache_dir` or the default system cache location.}
}
\value{
The requested dataset.
}
\description{
Downloads large datasets from GitHub releases to avoid CRAN size limits.
Supports multiple download mirrors for users in different regions.
Data is cached locally after first download. Cache directory can be
customized via `options(IOBR.cache_dir = "your/path")`.
}
\examples{
\dontrun{
tcga_sig <- download_iobr_data("tcga_stad_sig")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_doc.R
\docType{data}
\name{subgroup_data}
\alias{subgroup_data}
\title{Example Dataset for Subgroup Survival Analysis}
\format{
A data frame with clinical variables and biomarker scores:
\describe{
  \item{Patient_ID}{Unique patient identifier}
  \item{ProjectID}{Study or dataset identifier (e.g., "Dataset1")}
  \item{AJCC_stage}{AJCC pathological stage (2, 3, 4)}
  \item{status}{Event status (0=censored, 1=event)}
  \item{time}{Follow-up time in months}
  \item{score}{Continuous biomarker score (numeric values)}
  \item{score_binary}{Binary biomarker classification ("High", "Low")}
}
}
\usage{
data(subgroup_data)
}
\description{
An example dataset demonstrating the data structure required for subgroup
survival analysis using the \code{\link{subgroup_survival}} function.
Contains simulated clinical and biomarker data with survival outcomes.
}
\examples{
data(subgroup_data)
head(subgroup_data)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantiseq_helper.R
\name{DCrr}
\alias{DCrr}
\title{Robust Regression Deconvolution}
\usage{
DCrr(b, A, method, scaling)
}
\arguments{
\item{b}{Observation vector.}

\item{A}{Signature matrix.}

\item{method}{Robust regression method.}

\item{scaling}{Scaling factor.}
}
\value{
Estimated cell fractions.
}
\description{
Robust Regression Deconvolution
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_cor.R
\name{exact_pvalue}
\alias{exact_pvalue}
\title{Calculate Exact P-Value for Correlation}
\usage{
exact_pvalue(x, y, method)
}
\arguments{
\item{x}{Numeric vector representing the first variable.}

\item{y}{Numeric vector representing the second variable.}

\item{method}{Character string specifying the correlation method:
`"spearman"`, `"pearson"`, or `"kendall"`.}
}
\value{
Numeric value representing the exact p-value.
}
\description{
Computes the exact p-value for the correlation between two numeric variables
using a specified correlation method.
}
\examples{
# Simulate data
set.seed(123)
x <- rnorm(100)
y <- rnorm(100)
p_val <- exact_pvalue(x = x, y = y, method = "spearman")
print(p_val)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\name{deconvo_cibersort}
\alias{deconvo_cibersort}
\title{Deconvolve Using CIBERSORT}
\usage{
deconvo_cibersort(
  eset,
  project = NULL,
  arrays = FALSE,
  perm = 1000,
  absolute = FALSE,
  abs_method = "sig.score",
  parallel = FALSE,
  num_cores = 2,
  seed = NULL
)
}
\arguments{
\item{eset}{Expression matrix with gene symbols as row names.}

\item{project}{Optional project name. Default is `NULL`.}

\item{arrays}{Logical: optimized for microarray data. Default is `FALSE`.}

\item{perm}{Permutations for statistical analysis. Default is 1000.}

\item{absolute}{Logical: run in absolute mode. Default is `FALSE`.}

\item{abs_method}{Method for absolute mode: `"sig.score"` or `"no.sumto1"`.
Default is `"sig.score"`.}

\item{parallel}{Enable parallel execution. Default is `FALSE`.}

\item{num_cores}{Number of cores for parallel mode. Default is 2.}

\item{seed}{Random seed for reproducibility. Default is `NULL`.}
}
\value{
Data frame with CIBERSORT cell fractions. Columns suffixed with `_CIBERSORT`.
}
\description{
CIBERSORT is freely available to academic users. License and binary can be
obtained from https://cibersortx.stanford.edu.
}
\examples{
\dontrun{
lm22 <- load_data("lm22")
if (!is.null(lm22)) {
  set.seed(123)
  sim_eset <- matrix(rnorm(nrow(lm22) * 2), nrow(lm22), 2)
  rownames(sim_eset) <- rownames(lm22)
  colnames(sim_eset) <- paste0("Sample", 1:2)
  result <- deconvo_cibersort(eset = sim_eset, project = "TCGA-STAD", perm = 10)
  if (!is.null(result)) head(result)
}
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timer.R
\name{deconvolute_timer.default}
\alias{deconvolute_timer.default}
\title{Deconvolute Tumor Microenvironment Using TIMER}
\usage{
deconvolute_timer.default(args)
}
\arguments{
\item{args}{List or environment containing parameters:
\describe{
  \item{outdir}{Character. Output directory path.}
  \item{batch}{Character. File containing paths to expression data and
    cancer types.}
}}
}
\value{
Matrix of abundance scores for different immune cell types across
  multiple cancer samples.
}
\description{
Performs deconvolution of the tumor microenvironment using the TIMER
algorithm. Processes multiple cancer datasets, removes batch effects,
and estimates immune cell type abundances.
}
\examples{
\dontrun{
set.seed(123)
immune <- load_data("immuneCuratedData")
cancer_genes <- load_data("cancer_type_genes")
if (!is.null(immune) && !is.null(cancer_genes)) {
  gene_names <- unique(c(head(rownames(immune$genes), 30), cancer_genes[["stad"]]))
  sample_names <- paste0("Sample", 1:2)
  n_genes <- length(gene_names)
  expr <- matrix(runif(n_genes * 2, 1, 100), n_genes, 2,
                 dimnames = list(gene_names, sample_names))
  tf <- tempfile(fileext = ".tsv")
  write.table(as.data.frame(expr), tf, sep = "\t", quote = FALSE)
  batch_tf <- tempfile(fileext = ".csv")
  write.table(data.frame(path = tf, type = "stad"), batch_tf,
              sep = ",", col.names = FALSE, row.names = FALSE, quote = FALSE)
  args <- list(outdir = tempdir(), batch = batch_tf)
  result <- deconvolute_timer.default(args)
  if (!is.null(result)) head(result)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xCell.R
\name{rawEnrichmentAnalysis}
\alias{rawEnrichmentAnalysis}
\title{Calculate Raw xCell Enrichment Scores}
\usage{
rawEnrichmentAnalysis(expr, signatures, genes, file.name = NULL)
}
\arguments{
\item{expr}{Gene expression data matrix with row names as gene symbols.}

\item{signatures}{GMT object of signatures.}

\item{genes}{Character vector of genes to use.}

\item{file.name}{Character string for saving scores. Default is `NULL`.}
}
\value{
Matrix of raw xCell scores.
}
\description{
Returns the raw xCell cell types enrichment scores using ssGSEA.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal.R
\name{gsva_use_new_api}
\alias{gsva_use_new_api}
\title{GSVA API Version Detection}
\usage{
gsva_use_new_api()
}
\value{
A list with two elements:
  - `use_new_api`: Logical indicating whether to use the new API (`TRUE`)
    or old API (`FALSE`)
  - `gsva_version`: Character string of the installed GSVA version,
    or `"not installed"` if not available
}
\description{
Detects whether the installed GSVA package supports the new
parameter-based API (gsvaParam/ssgseaParam) or the old direct argument API.
This function is used internally to ensure compatibility across different
GSVA package versions (1.50.0+ vs older versions).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sigScore.R
\name{sigScore}
\alias{sigScore}
\title{Calculate Signature Score Using PCA, Mean, or Z-score Methods}
\usage{
sigScore(eset, methods = c("PCA", "mean", "zscore"))
}
\arguments{
\item{eset}{Normalized expression matrix with genes (signature) as rows and
samples as columns.}

\item{methods}{Scoring method: `"PCA"` (default) for principal component 1,
`"mean"` for mean expression, or `"zscore"` for z-score normalized mean.}
}
\value{
Numeric vector of length `ncol(eset)`; a score summarizing the rows
  of `eset`.
}
\description{
Computes signature scores from gene expression data using Principal
Component Analysis (PCA), mean-based, or z-score approaches.
}
\examples{
# Create small example expression matrix
set.seed(123)
test_eset <- matrix(rnorm(1000), nrow = 10, ncol = 100)
rownames(test_eset) <- paste0("Gene", 1:10)
colnames(test_eset) <- paste0("Sample", 1:100)

# Calculate scores
score_pca <- sigScore(eset = test_eset, methods = "PCA")
score_mean <- sigScore(eset = test_eset, methods = "mean")
score_zscore <- sigScore(eset = test_eset, methods = "zscore")
}
\author{
Dorothee Nickles, Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_group.R
\name{calculate_break_month}
\alias{calculate_break_month}
\title{Break Time Into Blocks}
\usage{
calculate_break_month(input, block = 6, time_type = c("month", "day"))
}
\arguments{
\item{input}{Numeric vector of time durations.}

\item{block}{Number of blocks. Default is 6.}

\item{time_type}{Units: "month" or "day". Default is "month".}
}
\value{
Numeric vector of breakpoints, rounded to nearest multiple of 5.
}
\description{
Divides time duration into specified blocks for analysis.
}
\examples{
time_data <- c(24, 36, 12, 48)
blocks <- calculate_break_month(input = time_data)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_helper.R
\name{filterCommonGenes}
\alias{filterCommonGenes}
\title{filterCommonGenes}
\usage{
filterCommonGenes(input.f, output.f, id = c("GeneSymbol", "EntrezID"))
}
\arguments{
\item{input.f}{A character string specifying the path to the input file or a connection object. The file should be a tab-separated table with row names.}

\item{output.f}{A character string specifying the path to the output file.}

\item{id}{A character string indicating the type of gene identifier to use. Can be either "GeneSymbol" or "EntrezID".}
}
\value{
No return value. The function writes the merged dataset to the specified output file.
}
\description{
This function filters and merges a dataset with a set of common genes.
}
\examples{
\dontrun{
input_data <- data.frame(
  GeneSymbol = c("BRCA1", "TP53", "EGFR", "NOTCH1"),
  Value = c(10, 15, 8, 12),
  stringsAsFactors = FALSE
)
input_file <- tempfile(fileext = ".txt")
output_file <- tempfile(fileext = ".txt")
write.table(input_data, file = input_file, sep = "\t", row.names = TRUE, quote = FALSE)
filterCommonGenes(input_file, output_file, id = "GeneSymbol")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal.R
\name{load_data}
\alias{load_data}
\title{Load IOBR Datasets}
\usage{
load_data(name)
}
\arguments{
\item{name}{Character string. Name of the dataset to load. Must be a single value.
Available datasets include:
- Expression data: `"eset_stad"`, `"imvigor210_eset"`, `"melanoma_data"`
- Signatures: `"signature_tme"`, `"signature_metabolism"`, `"signature_collection"`
- Gene sets: `"hallmark"`, `"kegg"`, `"go_bp"`, `"go_cc"`, `"go_mf"`
- Cell markers: `"cellmarkers"`, `"mcp_genes"`
- Phenotype data: `"pdata_stad"`, `"pdata_sig_tme"`, `"pdata_acrg"`
- Reference data: `"xCell.data"`, `"quantiseq_data"`, `"TRef"`, `"BRef"`
- Color palettes: `"palette1"`, `"palette2"`, `"palette3"`, `"palette4"`}
}
\value{
Dataset object, typically a `list`, `data.frame`, or `matrix`.
  The exact type depends on the requested dataset.
}
\description{
Loads internal datasets from the IOBR package. Supports both sysdata (internal)
and exported data files included in the package.
}
\examples{
# Load signature collection (stored in sysdata, no download)
sig_tme <- load_data("signature_tme")

# Load color palette (stored in sysdata, no download)
colors <- load_data("palette1")

# Error handling with suggestions for similar names
try(load_data("sign_tme")) # Will suggest "signature_tme"

\dontrun{
# Load expression data (triggers download from GitHub)
eset <- load_data("eset_stad")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateRef_rnaseq.R
\name{generateRef_rnaseq}
\alias{generateRef_rnaseq}
\title{Generate Reference Gene Matrix from RNA-seq DEGs}
\usage{
generateRef_rnaseq(dds, pheno, mode = "oneVSothers", FDR = 0.05, dat)
}
\arguments{
\item{dds}{Matrix. Raw count data from RNA-seq.}

\item{pheno}{Character vector. Cell type classes.}

\item{mode}{Character. DEG identification mode: `"oneVSothers"` or
`"pairs"`. Default is `"oneVSothers"`.}

\item{FDR}{Numeric. Threshold for adjusted p-values. Default is 0.05.}

\item{dat}{Matrix. Normalized expression data (e.g., FPKM, TPM).}
}
\value{
List containing:
  - `reference_matrix`: Data frame of median expression for significant
    genes across cell types.
  - `G`: Optimal number of probes minimizing condition number.
  - `condition_number`: Minimum condition number.
  - `whole_matrix`: Full median expression matrix.
}
\description{
Uses DESeq2 to identify differentially expressed genes and create a
reference matrix from median expression levels across cell types.
}
\examples{
dds <- matrix(rpois(200 * 10, lambda = 10), ncol = 10)
rownames(dds) <- paste("Gene", 1:200, sep = "_")
colnames(dds) <- paste("Sample", 1:10, sep = "_")
pheno <- sample(c("Type1", "Type2", "Type3"), 10, replace = TRUE)
dat <- matrix(rnorm(200 * 10), ncol = 10)
rownames(dat) <- rownames(dds)
colnames(dat) <- colnames(dds)
\donttest{
results <- generateRef_rnaseq(dds = dds, pheno = pheno, FDR = 0.05, dat = dat)
}
}
\author{
Rongfang Shen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_data.R
\name{set_iobr_cache_dir}
\alias{set_iobr_cache_dir}
\title{Set IOBR Cache Directory}
\usage{
set_iobr_cache_dir(path, create = TRUE)
}
\arguments{
\item{path}{Character string. The path to the cache directory.}

\item{create}{Logical. Whether to create the directory if it doesn't exist.
Default: TRUE.}
}
\value{
Invisibly returns the cache directory path.
}
\description{
Sets a custom cache directory for IOBR downloaded data.
This is useful when you want to store cached data in a specific location,
such as a shared network drive or a custom directory.

To use the standard system cache location (persistent across sessions),
you can use: `set_iobr_cache_dir(tools::R_user_dir("IOBR", which = "cache"))`.
}
\examples{
\donttest{
# Set a custom cache directory (use tempdir() for examples)
set_iobr_cache_dir(tempdir())

# Use standard system cache (persistent)
# set_iobr_cache_dir(tools::R_user_dir("IOBR", which = "cache"))

# Check the current cache directory
get_iobr_cache_dir()

# Download data will now use the custom cache
data <- download_iobr_data("lm22")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sig_roc.R
\name{sig_roc}
\alias{sig_roc}
\title{Plot ROC Curves and Compare Them}
\usage{
sig_roc(
  data,
  response,
  variables,
  fig.path = NULL,
  main = NULL,
  file.name = NULL,
  palette = "jama",
  cols = NULL,
  alpha = 1,
  compare = FALSE,
  smooth = TRUE,
  compare_method = "bootstrap",
  boot.n = 100
)
}
\arguments{
\item{data}{Data frame containing the predictor variables and binary outcome.}

\item{response}{Character. Name of the binary outcome variable in `data`.}

\item{variables}{Character vector. Names of predictor variables for ROC curves.}

\item{fig.path}{Character or `NULL`. Directory path to save output PDF.
Default is `NULL`.}

\item{main}{Character or `NULL`. Main title for the ROC plot. Default is `NULL`.}

\item{file.name}{Character or `NULL`. Output PDF filename without extension.
Default is `"0-ROC of multiple variables"`.}

\item{palette}{Character. Color palette for ROC curves. Default is `"jama"`.}

\item{cols}{Character vector or `NULL`. Custom colors for ROC curves.
Default is `NULL`.}

\item{alpha}{Numeric. Transparency level (1 = opaque, 0 = transparent).
Default is `1`.}

\item{compare}{Logical. Whether to perform statistical comparison of AUCs.
Default is `FALSE`.}

\item{smooth}{Logical. Whether to smooth ROC curves. Default is `TRUE`.}

\item{compare_method}{Character. Method for comparing ROC curves.
Default is `"bootstrap"`.}

\item{boot.n}{Integer. Number of bootstrap replications. Default is `100`.}
}
\value{
A list containing:
\describe{
  \item{auc.out}{Data frame with AUC values and confidence intervals}
  \item{legend.name}{Vector of legend entries for the plot}
  \item{p.out}{If `compare = TRUE`, data frame with p-values from comparisons}
}
}
\description{
Generates Receiver Operating Characteristic (ROC) curves for multiple
predictors and optionally performs statistical comparisons between them.
}
\examples{
set.seed(123)
test_data <- data.frame(
  OS_status = sample(c(0, 1), 20, replace = TRUE),
  Marker1 = rnorm(20),
  Marker2 = rnorm(20)
)
result <- sig_roc(data = test_data, response = "OS_status",
                  variables = c("Marker1", "Marker2"),
                  smooth = FALSE)
if (!is.null(result)) print(result$auc.out)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BinomialModel.R
\name{CalculatePref}
\alias{CalculatePref}
\title{Calculate Performance Metrics}
\usage{
CalculatePref(model, newx, s, acture.y)
}
\arguments{
\item{model}{Fitted glmnet model.}

\item{newx}{New data matrix for prediction.}

\item{s}{Lambda value for prediction.}

\item{acture.y}{Actual binary outcomes.}
}
\value{
ROCR performance object containing TPR and FPR values.
}
\description{
Computes True Positive Rate (TPR) and False Positive Rate (FPR) for ROC
analysis using the ROCR package. Used internally for ROC curve generation.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE) &&
  requireNamespace("ROCR", quietly = TRUE)) {
  fitted_model <- glmnet::cv.glmnet(matrix(rnorm(100), ncol = 2), rbinom(50, 1, 0.5), nfolds = 3)
  perf <- CalculatePref(fitted_model, matrix(rnorm(20), ncol = 2), "lambda.min", rbinom(10, 1, 0.5))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_sig_score.R
\name{calculate_sig_score_pca}
\alias{calculate_sig_score_pca}
\title{Calculate Signature Score Using PCA Method}
\usage{
calculate_sig_score_pca(
  pdata = NULL,
  eset,
  signature,
  mini_gene_count = 3,
  column_of_sample = "ID",
  adjust_eset = FALSE
)
}
\arguments{
\item{pdata}{Data frame with phenotype data. If `NULL`, created from
`eset` column names.}

\item{eset}{Expression matrix (genes as rows, samples as columns).}

\item{signature}{List of gene signatures.}

\item{mini_gene_count}{Minimum genes required per signature. Default is 3.}

\item{column_of_sample}{Column in `pdata` with sample IDs. Default is `"ID"`.}

\item{adjust_eset}{Logical: remove problematic features. Default is `FALSE`.}
}
\value{
Tibble with signature scores.
}
\description{
Computes signature scores using Principal Component Analysis.
The first principal component is used as the signature score.
}
\examples{
set.seed(123)
eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
rownames(eset) <- paste0("Gene", 1:100)
colnames(eset) <- paste0("Sample", 1:10)
signature <- list(
  Signature1 = paste0("Gene", 1:10),
  Signature2 = paste0("Gene", 11:20)
)
\donttest{
result <- calculate_sig_score_pca(eset = eset, signature = signature)
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_doc.R
\docType{data}
\name{null_models}
\alias{null_models}
\title{NULL Model Coefficients for MCPcounter}
\format{
A `data.frame` with cell types in rows and coefficients in columns.
}
\usage{
data(null_models)
}
\description{
NULL Model Coefficients for MCPcounter
}
\examples{
data(null_models)
head(null_models)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateRef_limma.R
\name{generateRef_limma}
\alias{generateRef_limma}
\title{Generate Reference Signature Matrix Using Limma}
\usage{
generateRef_limma(dat, pheno, FDR = 0.05)
}
\arguments{
\item{dat}{Matrix or data frame. Gene probes in rows and samples in columns.}

\item{pheno}{Character vector. Cell type class of the samples.}

\item{FDR}{Numeric. Genes with BH adjusted p-value < FDR are considered
significant. Default is 0.05.}
}
\value{
List containing:
  - `reference_matrix`: Data frame of median expression values for
    significantly expressed genes.
  - `G`: Number of probes used that resulted in the lowest condition number.
  - `condition_number`: Minimum condition number obtained.
  - `whole_matrix`: Matrix of median values across all samples.
}
\description{
Performs differential expression analysis using the limma package to
identify significantly expressed genes across different cell types.
Computes median expression levels of these significant genes to create
a reference signature matrix.
}
\examples{
dat <- matrix(rnorm(2000), nrow = 100)
rownames(dat) <- paste("Gene", 1:100, sep = "_")
colnames(dat) <- paste("Sample", 1:20, sep = "_")
pheno <- sample(c("Type1", "Type2", "Type3"), 20, replace = TRUE)
results <- generateRef_limma(dat, pheno)
print(results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IPS_helper.R
\name{mapcolors}
\alias{mapcolors}
\title{Map Score to Color}
\usage{
mapcolors(x, my_palette = NULL)
}
\arguments{
\item{x}{Numeric value to be mapped to a color (typically between -3 and 3).}

\item{my_palette}{Color palette vector (should have 1001 colors).
Default uses blue-white-red gradient.}
}
\value{
A color from the palette as a hex code.
}
\description{
Maps a numeric input value to a color from a blue-white-red gradient palette.
Values are mapped to a 1001-color palette where -3 maps to blue, 0 maps to white,
and +3 maps to red.
}
\examples{
my_palette <- grDevices::colorRampPalette(c("blue", "white", "red"))(1001)
color <- mapcolors(2, my_palette)
color <- mapcolors(-2, my_palette)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BinomialModel.R
\name{BinomialAUC}
\alias{BinomialAUC}
\title{Calculate AUC for Binomial Model}
\usage{
BinomialAUC(model, newx, s, acture.y)
}
\arguments{
\item{model}{Fitted glmnet model object.}

\item{newx}{New data matrix for prediction.}

\item{s}{Lambda value for prediction (e.g., "lambda.min" or numeric).}

\item{acture.y}{Actual binary outcomes (numeric 0/1 or factor).}
}
\value{
Numeric AUC value between 0 and 1.
}
\description{
Computes Area Under the ROC Curve (AUC) for model predictions using the
ROCR package. Handles binary classification models from glmnet.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE) &&
  requireNamespace("ROCR", quietly = TRUE)) {
  set.seed(123)
  train_data <- matrix(rnorm(100 * 5), ncol = 5)
  train_outcome <- rbinom(100, 1, 0.5)
  test_data <- matrix(rnorm(50 * 5), ncol = 5)
  test_outcome <- rbinom(50, 1, 0.5)
  fitted_model <- glmnet::cv.glmnet(train_data, train_outcome, family = "binomial", nfolds = 5)
  auc_value <- BinomialAUC(fitted_model, test_data, fitted_model$lambda.min, test_outcome)
  print(auc_value)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_sig_score.R
\docType{data}
\name{signature_score_calculation_methods}
\alias{signature_score_calculation_methods}
\title{Signature Score Calculation Methods}
\format{
Named character vector:
\describe{
  \item{PCA}{Principal Component Analysis method ("pca")}
  \item{ssGSEA}{Single-sample Gene Set Enrichment Analysis ("ssgsea")}
  \item{z-score}{Z-score transformation method ("zscore")}
  \item{Integration}{Integration of multiple methods ("integration")}
}
}
\usage{
signature_score_calculation_methods
}
\description{
A named vector of supported methods for calculating signature scores.
}
\examples{
signature_score_calculation_methods
signature_score_calculation_methods["PCA"]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_batcheffect.R
\name{remove_batcheffect}
\alias{remove_batcheffect}
\title{Removing Batch Effect from Expression Sets}
\usage{
remove_batcheffect(
  eset1,
  eset2,
  eset3 = NULL,
  id_type = "ensembl",
  data_type = c("array", "count", "tpm"),
  cols = "normal",
  palette = "jama",
  log2 = TRUE,
  check_eset = TRUE,
  adjust_eset = TRUE,
  repel = FALSE,
  path = NULL
)
}
\arguments{
\item{eset1}{First expression set (matrix or data frame with genes as rows).}

\item{eset2}{Second expression set.}

\item{eset3}{Optional third expression set. Use `NULL` if not available.}

\item{id_type}{Type of gene ID in expression sets (e.g., `"ensembl"`,
`"symbol"`). Required for count data normalization.}

\item{data_type}{Type of data: `"array"`, `"count"`, or `"tpm"`.
Default is `"array"`.}

\item{cols}{Color scale for PCA plot. Default is `"normal"`.}

\item{palette}{Color palette for PCA plot. Default is `"jama"`.}

\item{log2}{Whether to perform log2 transformation. Default is `TRUE`.
Ignored for count data.}

\item{check_eset}{Whether to check expression sets for errors.
Default is `TRUE`.}

\item{adjust_eset}{Whether to adjust expression sets by removing problematic
features. Default is `TRUE`.}

\item{repel}{Whether to add repelling labels to PCA plot. Default is `FALSE`.}

\item{path}{Directory where results should be saved. Default is `NULL`
(display only).}
}
\value{
Expression matrix after batch correction.
}
\description{
Removes batch effects from expression datasets using sva::ComBat (for
microarray/TPM data) or sva::ComBat_seq (for RNA-seq count data). Generates
PCA plots to visualize data before and after correction.
}
\examples{
# Simulate data
set.seed(123)
sim_eset1 <- matrix(rnorm(100 * 5, mean = 10, sd = 2), 100, 5)
sim_eset2 <- matrix(rnorm(100 * 5, mean = 12, sd = 2), 100, 5)
rownames(sim_eset1) <- rownames(sim_eset2) <- paste0("Gene", 1:100)
colnames(sim_eset1) <- paste0("S1_", 1:5)
colnames(sim_eset2) <- paste0("S2_", 1:5)

# Run batch correction
if (requireNamespace("sva", quietly = TRUE) && requireNamespace("BiocParallel", quietly = TRUE)) {
  eset_corrected <- remove_batcheffect(sim_eset1, sim_eset2, data_type = "tpm")
  if (!is.null(eset_corrected)) head(eset_corrected)
}
}
\references{
Zhang Y, et al. ComBat-seq: batch effect adjustment for RNA-seq count data.
NAR Genomics and Bioinformatics. 2020;2(3):lqaa078.
doi:10.1093/nargab/lqaa078

Leek JT, et al. The sva package for removing batch effects and other
unwanted variation in high-throughput experiments. Bioinformatics.
2012;28(6):882-883.
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CIBERSORT.R
\name{CoreAlg}
\alias{CoreAlg}
\title{Core Algorithm for CIBERSORT Deconvolution}
\usage{
CoreAlg(X, y, absolute, abs_method)
}
\arguments{
\item{X}{Matrix or data frame containing signature matrix (predictor variables).
Rows are genes, columns are cell types.}

\item{y}{Numeric vector containing the mixture sample expression (response variable).}

\item{absolute}{Logical indicating whether to use absolute space for weights.
Default is FALSE (relative proportions).}

\item{abs_method}{String specifying the method for absolute space weights:
`"sig.score"` or `"no.sumto1"`.}
}
\value{
List containing:
\describe{
  \item{w}{Estimated cell type weights/proportions}
  \item{mix_rmse}{Root mean squared error of the deconvolution}
  \item{mix_r}{Correlation coefficient between observed and predicted mixture}
}
}
\description{
Performs nu-regression using support vector machines (SVM) to estimate
cell type proportions. This is the core computational engine of CIBERSORT,
using nu-SVR with linear kernel to decompose mixed gene expression signals.
}
\examples{
# Simulate data
set.seed(123)
X <- matrix(rnorm(100), nrow = 10)
rownames(X) <- paste0("Gene", 1:10)
colnames(X) <- paste0("Cell", 1:10)
y <- rnorm(10)
names(y) <- paste0("Gene", 1:10)

# Run core algorithm
result <- CoreAlg(X, y, absolute = FALSE, abs_method = "sig.score")
if (!is.null(result)) str(result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_doc.R
\docType{data}
\name{signature_collection}
\alias{signature_collection}
\title{Gene signature collection for pathway and immune analysis}
\format{
A named list of length 323. Each element is a character vector of
  gene symbols. Representative entries include:
  \describe{
    \item{CD_8_T_effector}{Markers of CD8\eqn{^{+}} effector T cells.}
    \item{DDR}{DNA damage response and repair genes.}
    \item{Immune_Checkpoint}{Immune checkpoint molecules.}
    \item{CellCycle_Reg}{Core regulators of cell-cycle progression.}
    \item{Mismatch_Repair}{Mismatch-repair pathway genes.}
    \item{TMEsocreA_CIR}{TME-related signature used in TMEscore analysis.}
    \item{...}{Additional signatures are included in the list but are not
      individually listed here; all follow the same structure.}
  }
}
\usage{
data(signature_collection)
}
\description{
A named list of gene signatures used in the IOBR package for immune
deconvolution, pathway scoring, functional annotation, and tumour
microenvironment (TME) characterization. Each element corresponds to a
predefined biological signature and contains a character vector of HGNC
gene symbols.
}
\examples{
data(signature_collection)
head(signature_collection)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\name{deconvo_tme}
\alias{deconvo_tme}
\title{Main TME Deconvolution Function}
\usage{
deconvo_tme(
  eset,
  project = NULL,
  method = tme_deconvolution_methods,
  arrays = FALSE,
  tumor = TRUE,
  perm = 1000,
  reference,
  scale_reference = TRUE,
  plot = FALSE,
  scale_mrna = TRUE,
  group_list = NULL,
  platform = "affymetrix",
  absolute.mode = FALSE,
  abs.method = "sig.score",
  ...
)
}
\arguments{
\item{eset}{Gene expression matrix with HGNC symbols as row names.}

\item{project}{Optional project name. Default is `NULL`.}

\item{method}{Deconvolution method. See [tme_deconvolution_methods].}

\item{arrays}{Logical: microarray-optimized mode. Default is `FALSE`.}

\item{tumor}{Logical: tumor-optimized mode (EPIC). Default is `TRUE`.}

\item{perm}{Permutations (CIBERSORT/SVR). Default is 1000.}

\item{reference}{Custom reference matrix (SVR/lsei).}

\item{scale_reference}{Logical: scale reference (SVR/lsei).}

\item{plot}{Logical: generate plots (IPS). Default is `FALSE`.}

\item{scale_mrna}{Logical: mRNA correction (quanTIseq/EPIC).}

\item{group_list}{Cancer types for TIMER (vector).}

\item{platform}{Platform for ESTIMATE. Default is `"affymetrix"`.}

\item{absolute.mode}{Logical: absolute mode (CIBERSORT/SVR).
Default is `FALSE`.}

\item{abs.method}{Absolute mode method. Default is `"sig.score"`.}

\item{...}{Additional arguments passed to method.}
}
\value{
Tibble with cell fractions and `ID` column.
}
\description{
Unified interface for multiple TME deconvolution methods.
}
\examples{
mcp_genes <- load_data("mcp_genes")
if (!is.null(mcp_genes)) {
  set.seed(123)
  sim_eset <- matrix(rnorm(nrow(mcp_genes) * 3), nrow(mcp_genes), 3)
  rownames(sim_eset) <- mcp_genes$`HUGO symbols`
  colnames(sim_eset) <- paste0("Sample", 1:3)
  
  # Run deconvolution
  result <- deconvo_tme(eset = sim_eset, method = "mcpcounter")
  if (!is.null(result)) head(result)
}
}
\references{
\enumerate{
  \item Newman et al. (2015). Robust enumeration of cell subsets from tissue
    expression profiles. Nature Methods.
  \item Vegesna et al. (2013). Inferring tumour purity and stromal/immune
    cell admixture. Nature Communications.
  \item Finotello et al. (2019). Molecular and pharmacological modulators of
    the tumor immune contexture. Genome Medicine.
  \item Li et al. (2016). Comprehensive analyses of tumor immunity.
    Genome Biology.
  \item Charoentong et al. (2017). Pan-cancer Immunogenomic Analyses.
    Cell Reports.
  \item Becht et al. (2016). Estimating population abundance of tissue-infiltrating
    immune cells. Genome Biology.
  \item Aran et al. (2017). xCell: digitally portraying tissue cellular
    heterogeneity. Genome Biology.
  \item Racle et al. (2017). Simultaneous enumeration of cancer and immune
    cell types. ELife.
}
}
\author{
Dongqiang Zeng, Rongfang Shen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_sig_surv_plot.R
\name{batch_sig_surv_plot}
\alias{batch_sig_surv_plot}
\title{Batch Signature Survival Plot}
\usage{
batch_sig_surv_plot(
  input_pdata,
  signature,
  id = "ID",
  column_of_project = "ProjectID",
  project = NULL,
  time = "time",
  status = "status",
  time_type = "day",
  break_month = "auto",
  palette = "jama",
  cols = NULL,
  mini_sig = "score",
  save_path = NULL,
  show_col = TRUE,
  fig_type = "pdf"
)
}
\arguments{
\item{input_pdata}{Data frame containing survival data and signature scores.}

\item{signature}{Character string specifying the column name of the target
signature for survival analysis.}

\item{id}{Character string specifying the column name containing unique
identifiers. Default is `"ID"`.}

\item{column_of_project}{Character string specifying the column name
containing project identifiers. Default is `"ProjectID"`.}

\item{project}{Character string or vector specifying project name(s) to
analyze. If `NULL`, all projects are analyzed. Default is `NULL`.}

\item{time}{Character string specifying the column name containing
time-to-event data. Default is `"time"`.}

\item{status}{Character string specifying the column name containing event
status. Default is `"status"`.}

\item{time_type}{Character string specifying the time unit. Options are
`"day"` or `"month"`. Default is `"day"`.}

\item{break_month}{Numeric value or `"auto"` specifying the interval for
time axis breaks in months. Default is `"auto"`.}

\item{palette}{Character string specifying the color palette. Default is
`"jama"`.}

\item{cols}{Character vector of custom colors. If `NULL`, palette is used.
Default is `NULL`.}

\item{mini_sig}{Character string for the signature label in the legend.
Default is `"score"`.}

\item{save_path}{Character string or `NULL`. Directory path for saving plots.
If `NULL`, plots are not saved. Default is `NULL`.}

\item{show_col}{Logical indicating whether to display color information.
Default is `TRUE`.}

\item{fig_type}{Character string specifying the output file format
(`"pdf"`, `"png"`, etc.). Default is `"pdf"`.}
}
\value{
Data frame containing combined survival analysis results from all
  projects.
}
\description{
Generates Kaplan-Meier survival plots for multiple projects or cohorts based
on signature scores. Automatically determines optimal cutpoints for signature
stratification and creates publication-ready survival curves.
}
\examples{
set.seed(123)
test_pdata <- data.frame(
  ID = paste0("S", 1:20),
  ProjectID = rep("P1", 20),
  OS_time = runif(20, 1, 60),
  OS_status = sample(c(0, 1), 20, replace = TRUE),
  Marker = rnorm(20)
)
result <- batch_sig_surv_plot(
  input_pdata = test_pdata, signature = "Marker",
  id = "ID", column_of_project = "ProjectID",
  time = "OS_time", status = "OS_status", time_type = "month"
)
if (!is.null(result)) head(result)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantiseq_helper.R
\name{DClsei}
\alias{DClsei}
\title{Constrained Least Squares Deconvolution}
\usage{
DClsei(b, A, G, H, scaling)
}
\arguments{
\item{b}{Observation vector.}

\item{A}{Signature matrix.}

\item{G}{Constraint matrix.}

\item{H}{Constraint vector.}

\item{scaling}{Scaling factor.}
}
\value{
Estimated cell fractions.
}
\description{
Constrained Least Squares Deconvolution
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rbind_iobr.R
\name{rbind_iobr}
\alias{rbind_iobr}
\title{Row Bind Multiple Data Sets}
\usage{
rbind_iobr(data1, data2, data3 = NULL)
}
\arguments{
\item{data1}{Data frame or matrix. First dataset to combine.}

\item{data2}{Data frame or matrix. Second dataset to combine.}

\item{data3}{Data frame or matrix or `NULL`. Optional third dataset.
Default is `NULL`.}
}
\value{
Combined data frame resulting from row binding the input datasets.
}
\description{
Combines two or three data frames or matrices vertically using `rbind`.
Ensures compatibility of input data before binding by aligning columns.
}
\examples{
data1 <- data.frame(A = 1:5, B = letters[1:5])
data2 <- data.frame(A = 6:10, B = letters[6:10])
combined_data <- rbind_iobr(data1, data2)

# With three datasets
data3 <- data.frame(A = 11:15, B = letters[11:15])
combined_data <- rbind_iobr(data1, data2, data3)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_selection.R
\name{feature_select}
\alias{feature_select}
\title{Feature Selection via Correlation or Differential Expression}
\usage{
feature_select(
  x,
  y,
  method = c("cor", "dif"),
  family = c("spearman", "pearson"),
  cutoff = NULL,
  padjcut = NULL
)
}
\arguments{
\item{x}{Numeric matrix. Features (rows) by samples (columns).}

\item{y}{Numeric or factor. Response vector (quantitative or binary).}

\item{method}{Character. "cor" (correlation) or "dif" (differential expression). Default c("cor","dif").}

\item{family}{Character. Correlation method if method = "cor": "spearman" or "pearson".}

\item{cutoff}{Numeric. Absolute correlation (for cor) or |log2FC| (for dif) threshold.}

\item{padjcut}{Numeric. Adjusted p-value cutoff.}
}
\value{
Character vector of selected feature names.
}
\description{
Selects informative features using either correlation with a quantitative
response or differential expression (limma) for binary/continuous
responses.
}
\examples{
# Simulate data
set.seed(123)
sim_eset <- matrix(rnorm(100 * 50), 100, 50)
rownames(sim_eset) <- c("PDCD1", paste0("Gene", 2:100))
colnames(sim_eset) <- paste0("Sample", 1:50)
pd1 <- as.numeric(sim_eset["PDCD1", ])
group <- ifelse(pd1 > mean(pd1), "high", "low")

# Correlation method
pd1_cor <- feature_select(
  x = sim_eset, y = pd1, method = "cor",
  family = "pearson", padjcut = 0.05, cutoff = 0.5
)

# Differential expression method
pd1_dif <- feature_select(
  x = sim_eset, y = pd1, method = "dif",
  padjcut = 0.05, cutoff = 2
)
pd1_dif_2 <- feature_select(
  x = sim_eset, y = group,
  method = "dif", padjcut = 0.05, cutoff = 2
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roc_time.R
\name{roc_time}
\alias{roc_time}
\title{Time-dependent ROC Curve for Survival Analysis}
\usage{
roc_time(
  input,
  vars,
  time = "time",
  status = "status",
  time_point = 12,
  time_type = "month",
  palette = "jama",
  cols = "normal",
  seed = 1234,
  show_col = FALSE,
  path = NULL,
  main = "PFS",
  index = 1,
  fig.type = "pdf",
  width = 5,
  height = 5.2
)
}
\arguments{
\item{input}{Data frame containing variables for analysis.}

\item{vars}{Character vector. Variable(s) to be evaluated.}

\item{time}{Character string. Name of the time variable. Default is `"time"`.}

\item{status}{Character string. Name of the status variable. Default is `"status"`.}

\item{time_point}{Integer or vector. Time point(s) for ROC analysis. Default is `12`.}

\item{time_type}{Character string. Time unit (`"day"` or `"month"`). Default is `"month"`.}

\item{palette}{Character string. Color palette for the plot. Default is `"jama"`.}

\item{cols}{Character vector or string. Color specification: `"normal"`, `"random"`,
or custom color vector.}

\item{seed}{Integer. Random seed for reproducibility. Default is `1234`.}

\item{show_col}{Logical indicating whether to display the color palette.
Default is `FALSE`.}

\item{path}{Character string or `NULL`. Path to save the plot. Default is `NULL`.}

\item{main}{Character string. Main title of the plot. Default is `"PFS"`.}

\item{index}{Integer. Index for plot filename. Default is `1`.}

\item{fig.type}{Character string. Output file type (e.g., `"pdf"`, `"png"`).
Default is `"pdf"`.}

\item{width}{Numeric. Width of the plot. Default is `5`.}

\item{height}{Numeric. Height of the plot. Default is `5.2`.}
}
\value{
A ggplot object representing the time-dependent ROC plot.
}
\description{
Generates a time-dependent Receiver Operating Characteristic (ROC) plot to evaluate
the predictive performance of one or more variables in survival analysis. Calculates
the Area Under the Curve (AUC) for each specified time point and variable, and
creates a multi-line ROC plot with annotated AUC values.
}
\examples{
# Simulate data for offline testing
set.seed(123)
sim_input <- data.frame(
  ID = paste0("Sample", 1:100),
  time = runif(100, 1, 60),
  status = sample(0:1, 100, replace = TRUE),
  Marker1 = rnorm(100),
  Marker2 = rnorm(100),
  Marker3 = rnorm(100)
)
\dontrun{
roc_time(
  input = sim_input, vars = c("Marker1", "Marker2", "Marker3"),
  time = "time", status = "status", time_point = 12, path = NULL, main = "OS"
)
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_mut_matrix.R
\name{.build_mut_matrices}
\alias{.build_mut_matrices}
\title{Build Mutation Matrices from MAF Data}
\usage{
.build_mut_matrices(mut, category)
}
\arguments{
\item{mut}{Data frame with mutation data.}

\item{category}{Character. Mutation category.}
}
\value{
List of mutation matrices.
}
\description{
Build Mutation Matrices from MAF Data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantiseq_helper.R
\name{makeQN}
\alias{makeQN}
\title{Quantile Normalization}
\usage{
makeQN(mix.mat)
}
\arguments{
\item{mix.mat}{Expression matrix.}
}
\value{
Quantile normalized matrix.
}
\description{
Quantile Normalization
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_doc.R
\docType{data}
\name{tcga_stad_pdata}
\alias{tcga_stad_pdata}
\title{TCGA-STAD Clinical and Molecular Annotation Data}
\format{
A data frame with samples as rows and variables as columns:
\itemize{
  \item ID – TCGA sample barcode
  \item stage – tumor stage (Stage_I, Stage_II, etc.)
  \item status – vital status (Alive/Dead)
  \item Lauren – histological classification
  \item subtype – molecular subtype (EBV, MSI, GS, CN, ...)
  \item EBV – EBV infection status
  \item TMEscore_plus – continuous tumor-micro-environment score
  \item TMEscore_plus_binary – High/Low TME classification
  \item time – follow-up time (months)
  \item OS_status – 0 = alive, 1 = dead
  \item ARID1A, PIK3CA – driver-gene mutation status
  \item MALAT1 – lncRNA expression
  \item remaining columns – gene-expression values and additional
  clinical/molecular annotations
}
}
\source{
The Cancer Genome Atlas Stomach Adenocarcinoma (TCGA-STAD)
}
\usage{
data(tcga_stad_pdata)
}
\description{
Clinical, molecular, and signature score data for TCGA stomach adenocarcinoma
(STAD) samples. Includes patient demographics, tumor characteristics,
molecular subtypes, and pre-computed signature scores.
}
\examples{
data(tcga_stad_pdata)
head(tcga_stad_pdata)
}
\references{
Cancer Genome Atlas Research Network. Comprehensive molecular
characterization of gastric adenocarcinoma. Nature 513, 202-209 (2014).
doi:10.1038/nature13480
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcpcounter.R
\name{test_for_infiltration}
\alias{test_for_infiltration}
\title{Test for Cell Population Infiltration}
\usage{
test_for_infiltration(MCPcounterMatrix, platform = c("133P2", "133A", "HG1"))
}
\arguments{
\item{MCPcounterMatrix}{Matrix, usually output from MCPcounter.estimate.}

\item{platform}{Expression platform: "133P2", "133A", or "HG1".
Default is "133P2".}
}
\value{
Matrix with samples in rows and cell populations in columns.
  Elements are p-values.
}
\description{
Returns p-values for the null hypothesis that samples are not infiltrated
by the corresponding cell population.
}
\examples{
# This function requires null_models data which is loaded internally
# Create example data
scores <- matrix(runif(30), nrow = 3, ncol = 10)
rownames(scores) <- c("T cells", "B cells", "NK cells")
pvals <- test_for_infiltration(scores, platform = "133P2")
}
\author{
Etienne Becht
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_mutations.R
\name{find_mutations}
\alias{find_mutations}
\title{Analyze Mutations Related to Signature Scores}
\usage{
find_mutations(
  mutation_matrix,
  signature_matrix,
  id_signature_matrix = "ID",
  signature,
  min_mut_freq = 0.05,
  plot = TRUE,
  method = "multi",
  point_alpha = 0.1,
  save_path = NULL,
  palette = "jco",
  cols = NULL,
  show_plot = TRUE,
  show_col = FALSE,
  width = 8,
  height = 4,
  oncoprint_group_by = "mean",
  oncoprint_col = "#224444",
  gene_counts = 10,
  jitter = FALSE,
  genes = NULL,
  point_size = 4.5
)
}
\arguments{
\item{mutation_matrix}{A matrix of mutation data with samples in rows and genes in columns.}

\item{signature_matrix}{A data frame with sample identifiers and signature scores.}

\item{id_signature_matrix}{Column name in `signature_matrix` for sample identifiers.}

\item{signature}{Name of the target signature for analysis.}

\item{min_mut_freq}{Minimum mutation frequency required for gene inclusion. Default is 0.05.}

\item{plot}{Logical indicating whether to generate and save plots. Default is TRUE.}

\item{method}{Statistical test method: "multi" for both Cuzick and Wilcoxon, or "Wilcoxon" only. Default is "multi".}

\item{point_alpha}{Transparency of points in box plot. Default is 0.1.}

\item{save_path}{Directory to save plots and results. If NULL, no files are saved.}

\item{palette}{Color palette for box plots(used when cols is NULL). Default is "jco".}

\item{cols}{Character vector. Custom colors for box plots. If NULL, uses palette. Default is NULL.}

\item{show_plot}{Logical indicating whether to display plots. Default is TRUE.}

\item{show_col}{Logical indicating whether to show color codes. Default is FALSE.}

\item{width}{Width of oncoprint plot. Default is 8.}

\item{height}{Height of oncoprint plot. Default is 4.}

\item{oncoprint_group_by}{Grouping method for oncoprint: "mean" or "quantile". Default is "mean".}

\item{oncoprint_col}{Color for mutations in oncoprint. Default is "#224444".}

\item{gene_counts}{Number of genes to display in oncoprint. Default is 10.}

\item{jitter}{Logical indicating whether to add jitter to box plot points. Default is FALSE.}

\item{genes}{Optional vector of gene names; if NULL, selects based on frequency.}

\item{point_size}{Size of points in box plot. Default is 4.5.}
}
\value{
A list containing statistical test results, oncoprint plots, and box plots.
}
\description{
This function identifies mutations associated with a specific signature
score, performs statistical tests
for significance, and generates oncoprints and box plots to visualize
relationships.
}
\examples{
\dontrun{
# This example requires a MAF file from TCGA or maftools
# See maftools or TCGAbiolinks documentation for obtaining MAF files
mut_list <- make_mut_matrix(
  maf = "path_to_maf_file", isTCGA = TRUE,
  category = "multi"
)
mut <- mut_list$snp
results <- find_mutations(
  mutation_matrix = mut, signature_matrix = signature_data,
  id_signature_matrix = "ID", signature = "CD_8_T_effector",
  min_mut_freq = 0.01, plot = TRUE, method = "multi"
)
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_eset.R
\name{merge_eset}
\alias{merge_eset}
\title{Merge Expression Sets by Row Names}
\usage{
merge_eset(eset1, eset2, eset3 = NULL)
}
\arguments{
\item{eset1}{First expression set (matrix or data frame with row names).}

\item{eset2}{Second expression set (matrix or data frame with row names).}

\item{eset3}{Optional third expression set. Default is `NULL`.}
}
\value{
Merged expression set (data frame) with duplicates removed.
  Row names correspond to gene symbols.
}
\description{
Merges two or three expression sets (matrices or data frames) by row names
(gene symbols), removing duplicates. The function ensures common genes
across all input expression sets are retained.
}
\examples{
# Create mock expression sets with common genes
set.seed(123)
common_genes <- c("TP53", "BRCA1", "EGFR", "MYC")
eset1 <- matrix(rnorm(12),
  nrow = 4,
  dimnames = list(common_genes, paste0("S", 1:3))
)
eset2 <- matrix(rnorm(16),
  nrow = 4,
  dimnames = list(common_genes, paste0("S", 4:7))
)
# Merge two expression sets
merged_eset <- merge_eset(eset1, eset2)
print(dim(merged_eset))
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_matrix.R
\name{scale_matrix}
\alias{scale_matrix}
\title{Scale and Manipulate a Matrix}
\usage{
scale_matrix(matrix, log2matrix = TRUE, manipulate = TRUE)
}
\arguments{
\item{matrix}{Numeric matrix with genes as rows and samples as columns.}

\item{log2matrix}{Logical indicating whether to apply log2 transformation
using [log2eset()]. Default is `TRUE`.}

\item{manipulate}{Logical indicating whether to perform feature manipulation
to remove problematic features. Default is `TRUE`.}
}
\value{
A scaled matrix (genes as rows, samples as columns).
}
\description{
Scales a gene expression matrix, optionally applies logarithmic
transformation, and performs feature manipulation to remove problematic
variables (e.g., genes with zero variance or missing values).
}
\examples{
set.seed(123)
test_matrix <- matrix(
  rlnorm(100),
  ncol = 5,
  dimnames = list(paste0("Gene", 1:20), paste0("Sample", 1:5))
)
eset2 <- scale_matrix(test_matrix, log2matrix = FALSE, manipulate = TRUE)
head(eset2)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_duplicate.R
\name{merge_duplicate}
\alias{merge_duplicate}
\title{Merge Data Frames with Duplicated Column Names}
\usage{
merge_duplicate(
  x,
  y,
  by.x,
  by.y,
  all.x = FALSE,
  all.y = FALSE,
  all = NULL,
  choose = c("x", "y")
)
}
\arguments{
\item{x}{Data frame. First data frame to merge.}

\item{y}{Data frame. Second data frame to merge.}

\item{by.x}{Character. Column name(s) in `x` used for merging.}

\item{by.y}{Character. Column name(s) in `y` used for merging.}

\item{all.x}{Logical. Include all rows from `x` in output. Default is `FALSE`.}

\item{all.y}{Logical. Include all rows from `y` in output. Default is `FALSE`.}

\item{all}{Logical or `NULL`. If not `NULL`, include all rows from both `x`
and `y`, overriding `all.x` and `all.y`.}

\item{choose}{Character. Which data frame's duplicated non-joining columns to
retain: `"x"` or `"y"`. Default is `"x"`.}
}
\value{
Data frame resulting from merging `x` and `y` according to specified
  parameters.
}
\description{
Merges two data frames, resolving duplicated column names according to user
preference. Allows selection of which data frame's duplicated columns to
retain, ensuring data integrity during merging.
}
\examples{
df1 <- data.frame(ID = 1:3, Name = c("A", "B", "C"), Value = 1:3)
df2 <- data.frame(ID = 1:3, Name = c("X", "Y", "Z"), Score = 4:6)

# Merge keeping duplicated columns from x
merged_df <- merge_duplicate(df1, df2,
  by.x = "ID", by.y = "ID",
  all.x = TRUE, choose = "x"
)
print(merged_df)

# Merge keeping duplicated columns from y
merged_df2 <- merge_duplicate(df1, df2,
  by.x = "ID", by.y = "ID",
  all = TRUE, choose = "y"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count2tpm.R
\name{count2tpm}
\alias{count2tpm}
\title{Convert Read Counts to Transcripts Per Million (TPM)}
\usage{
count2tpm(
  countMat,
  idType = "Ensembl",
  org = c("hsa", "mmus"),
  source = c("local", "biomart"),
  effLength = NULL,
  id = "id",
  gene_symbol = "symbol",
  length = "eff_length",
  check_data = FALSE
)
}
\arguments{
\item{countMat}{Numeric matrix of raw read counts with genes in rows and
samples in columns.}

\item{idType}{Character string specifying the gene identifier type. Options
are `"Ensembl"`, `"Entrez"`, or `"Symbol"`. Default is `"Ensembl"`.}

\item{org}{Character string specifying the organism. Options include
`"hsa"` (human) or `"mmus"` (mouse). Default is `"hsa"`.}

\item{source}{Character string specifying the source for gene length
information. Options are `"biomart"` (retrieve from Ensembl BioMart) or
`"local"` (use local dataset). Default is `"local"`.}

\item{effLength}{Data frame containing effective gene length information. If
`NULL`, lengths are retrieved based on `source`. Default is `NULL`.}

\item{id}{Character string specifying the column name in `effLength`
containing gene identifiers. Default is `"id"`.}

\item{gene_symbol}{Character string specifying the column name in `effLength`
containing gene symbols. Default is `"symbol"`.}

\item{length}{Character string specifying the column name in `effLength`
containing gene lengths. Default is `"eff_length"`.}

\item{check_data}{Logical indicating whether to check for missing values in
the count matrix. Default is `FALSE`.}
}
\value{
Data frame of TPM-normalized expression values with genes in rows
  and samples in columns. Gene identifiers are converted to gene symbols
  in the output, regardless of the input `idType`.
}
\description{
Transforms gene expression count data into Transcripts Per Million (TPM)
values, normalizing for gene length and library size. Supports multiple gene
ID types and can retrieve gene length information from BioMart or use local
datasets.
}
\examples{
# Simulated count data
set.seed(123)
count_matrix <- matrix(
  rpois(100, lambda = 10),
  ncol = 5,
  dimnames = list(
    paste0("ENSG00000", 101:120),
    paste0("Sample", 1:5)
  )
)
# Simulated effective length data
eff_len <- data.frame(
  id = paste0("ENSG00000", 101:120),
  symbol = paste0("Gene", 1:20),
  eff_length = runif(20, 500, 5000)
)
# Transform to TPM using local gene annotation
eset <- count2tpm(
  countMat = count_matrix, effLength = eff_len,
  id = "id", length = "eff_length", gene_symbol = "symbol"
)
head(eset)
}
\author{
Wubing Zhang, Dongqiang Zeng, Yiran Fang
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timer.R
\docType{data}
\name{timer_available_cancers}
\alias{timer_available_cancers}
\title{TIMER Available Cancer Types}
\format{
An object of class \code{character} of length 32.
}
\usage{
timer_available_cancers
}
\value{
Character vector of available cancer type abbreviations.
}
\description{
Character vector of cancer types supported by TIMER deconvolution.
TIMER signatures are cancer-specific.
}
\examples{
# List all available cancer types for TIMER
timer_available_cancers

# Check if a cancer type is supported
"brca" \%in\% timer_available_cancers
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timer.R
\name{RemoveBatchEffect}
\alias{RemoveBatchEffect}
\title{Remove Batch Effect of Expression Set}
\usage{
RemoveBatchEffect(cancer.exp, immune.exp, immune.cellType)
}
\arguments{
\item{cancer.exp}{Matrix or data frame. Cancer cell expression data with
genes as rows and samples as columns.}

\item{immune.exp}{Matrix or data frame. Immune cell expression data with
genes as rows and samples as columns.}

\item{immune.cellType}{Vector. Cell type for each column in `immune.exp`.}
}
\value{
A list containing:
\describe{
  \item{1}{Batch effect corrected cancer expression data}
  \item{2}{Batch effect corrected immune expression data}
  \item{3}{Aggregated immune expression data (median per cell type)}
}
}
\description{
Removes batch effects between two gene expression datasets, typically
representing different sample types such as cancer cells and immune cells.
Uses ComBat from the sva package for batch correction.
}
\examples{
set.seed(123)
gene_names <- paste0("Gene", 1:100)
sample_names_cancer <- paste0("CancerSample", 1:10)
cancer.exp <- matrix(runif(1000, 1, 1000),
  nrow = 100, ncol = 10,
  dimnames = list(gene_names, sample_names_cancer)
)

sample_names_immune <- paste0("ImmuneSample", 1:5)
immune.exp <- matrix(runif(500, 1, 1000),
  nrow = 100, ncol = 5,
  dimnames = list(gene_names, sample_names_immune)
)

immune.cellType <- c("T-cell", "B-cell", "T-cell", "NK-cell", "B-cell")
names(immune.cellType) <- sample_names_immune

result <- RemoveBatchEffect(cancer.exp, immune.exp, immune.cellType)
if (!is.null(result)) str(result)
}
\author{
Bo Li
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_names.R
\docType{data}
\name{patterns_to_na}
\alias{patterns_to_na}
\title{Default Pattern List for Name Cleaning}
\format{
A character vector of length 12.
}
\usage{
patterns_to_na
}
\value{
Character vector of patterns to remove.
}
\description{
A character vector of common substrings to remove from feature names.
Used in [remove_names()] and other helper functions.
}
\examples{
# View default patterns
patterns_to_na
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_duplicate_genes.R
\name{remove_duplicate_genes}
\alias{remove_duplicate_genes}
\title{Remove Duplicate Gene Symbols in Gene Expression Data}
\usage{
remove_duplicate_genes(eset, column_of_symbol, method = c("mean", "sd", "sum"))
}
\arguments{
\item{eset}{A data frame or matrix representing gene expression data, with
gene symbols as one of the columns.}

\item{column_of_symbol}{The name of the column containing gene symbols in
`eset`.}

\item{method}{The ranking method to use for selecting among duplicate gene
symbols: `"mean"` for mean expression, `"sd"` for standard deviation,
or `"sum"` for sum of expression values. Default is `"mean"`.}
}
\value{
A modified version of `eset` where duplicate gene symbols have been
  reduced to a single entry (the highest-ranking one). The gene symbols are
  set as row names in the returned data frame.
}
\description{
This function addresses duplicate gene symbols in a gene expression dataset
by selecting the highest-expressing instance among duplicates. Users can
choose between mean, standard deviation, or sum as the ranking criterion
for selection. This is useful for preparing data where duplicates can lead
to issues in downstream analyses.
}
\note{
Important: This function performs selection, not aggregation. For duplicate
genes, it retains only the highest-ranking instance (based on the specified
method) and discards others.
}
\examples{
set.seed(123)
test_eset <- data.frame(
  symbol = c("GeneA", "GeneA", "GeneB", "GeneC"),
  S1 = c(10, 5, 20, 15),
  S2 = c(12, 7, 22, 17)
)
# Remove duplicates using mean expression
test_eset_unique <- remove_duplicate_genes(
  eset = test_eset,
  column_of_symbol = "symbol",
  method = "mean"
)
print(test_eset_unique)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mouse2human_eset.R
\name{mouse2human_eset}
\alias{mouse2human_eset}
\title{Convert Mouse Gene Symbols to Human Gene Symbols}
\usage{
mouse2human_eset(
  eset,
  source = c("local", "ensembl"),
  is_matrix = TRUE,
  column_of_symbol = NULL,
  verbose = FALSE
)
}
\arguments{
\item{eset}{Matrix or data frame. Expression matrix with genes in rows.}

\item{source}{Character. Data source for conversion: "ensembl" (online) or
"local". Default is "ensembl". If Ensembl fails, use "local" which uses
the internal `mus_human_gene_symbol` dataset.}

\item{is_matrix}{Logical. Whether `eset` is a matrix with gene symbols as
row names. Default is `TRUE`. If `FALSE`, `column_of_symbol` must be
specified.}

\item{column_of_symbol}{Character or `NULL`. Column name containing gene
symbols if `eset` is not a matrix. Default is `NULL`.}

\item{verbose}{Logical. If `TRUE`, prints available Ensembl datasets.
Default is `FALSE`.}
}
\value{
Expression set with human gene symbols.
}
\description{
Converts mouse gene symbols to human gene symbols in an expression dataset.
Supports using either an online resource (Ensembl) or a local dataset for
conversion.
}
\examples{
\dontrun{
set.seed(123)
data <- matrix(runif(50 * 3), nrow = 50, ncol = 3)
rownames(data) <- c("Tpt1", "Hmgb1", "Gapdh", paste0("Gene", 4:50))
colnames(data) <- paste0("Sample", 1:3)
human_data <- mouse2human_eset(data, source = "local", is_matrix = TRUE)
if (!is.null(human_data)) head(human_data)
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tcga_rna_preps.R
\name{tcga_rna_preps}
\alias{tcga_rna_preps}
\title{Preprocess TCGA RNA-seq Data}
\usage{
tcga_rna_preps(
  eset,
  id_type = c("ensembl", "symbol"),
  input_type = c("log2count", "count"),
  output = c("tumor", "tumor_normal"),
  output_type = c("tpm", "log2tpm", "count"),
  annotation = TRUE
)
}
\arguments{
\item{eset}{Matrix or data frame. RNA-seq gene expression matrix from TCGA.}

\item{id_type}{Character. Gene identifier type: "ensembl" or "symbol".
Default is "ensembl".}

\item{input_type}{Character. Input data type: "log2count" or "count".
Default is "log2count".}

\item{output}{Character. Sample type: "tumor" or "tumor_normal".
Default is "tumor".}

\item{output_type}{Character. Output data type: "tpm", "log2tpm", or "count".
Default is "tpm".}

\item{annotation}{Logical. Whether to perform gene annotation. Default is TRUE.}
}
\value{
Preprocessed gene expression matrix.
}
\description{
Preprocesses TCGA RNA-seq data by modifying sample types, transforming data,
and annotating genes based on specified parameters.
}
\examples{
# Simulate TCGA-style data with Ensembl IDs
set.seed(123)
sim_eset <- matrix(
  abs(rnorm(500)),
  nrow = 100,
  ncol = 5,
  dimnames = list(
    paste0("ENSG00000", 101:200),
    paste0("TCGA-AB-123", 4:8, "-01A")
  )
)
# Process tumor samples only (output as count for offline testing)
eset <- tcga_rna_preps(
  eset = sim_eset, id_type = "ensembl", input_type = "count",
  output = "tumor", output_type = "count", annotation = FALSE
)
print(dim(eset))
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_mut_matrix.R
\name{make_mut_matrix}
\alias{make_mut_matrix}
\title{Construct Mutation Matrices from MAF Data}
\usage{
make_mut_matrix(
  maf = NULL,
  mut_data = NULL,
  isTCGA = TRUE,
  category = c("multi", "all", "snp", "indel", "frameshift"),
  Tumor_Sample_Barcode = "Tumor_Sample_Barcode",
  Hugo_Symbol = "Hugo_Symbol",
  Variant_Classification = "Variant_Classification",
  Variant_Type = "Variant_Type"
)
}
\arguments{
\item{maf}{Character or MAF object. Path to MAF file or an already loaded MAF object.}

\item{mut_data}{Data frame or NULL. Preloaded MAF-like data (used if 'maf' is NULL).}

\item{isTCGA}{Logical. Whether the MAF follows TCGA conventions. Default is TRUE.}

\item{category}{Character. Mutation category: "all", "snp", "indel", "frameshift", or "multi". Default is "multi".}

\item{Tumor_Sample_Barcode}{Character. Column name for tumor sample IDs. Default is "Tumor_Sample_Barcode".}

\item{Hugo_Symbol}{Character. Column name for gene symbols. Default is "Hugo_Symbol".}

\item{Variant_Classification}{Character. Column name for variant classification (e.g., Frame_Shift_Del). Default is "Variant_Classification".}

\item{Variant_Type}{Character. Column name for variant type (e.g., SNP, INS, DEL). Default is "Variant_Type".}
}
\value{
List of mutation matrices (if category = "multi") or a single matrix for the specified category.
}
\description{
Builds mutation presence/absence matrices from MAF input (file path or MAF
object). Supports multiple categories: all mutations, SNPs, indels, and
frameshift mutations. When category = "multi", returns a list of matrices
for each category. Compatible with TCGA-formatted data.
}
\note{
Some users may encounter errors from upstream data import (e.g.
"Can't combine ..$Tumor_Seq_Allele2" when using TCGAbiolinks or
TCGAmutations).
This is due to inconsistent column types in the source MAF tables,
not an issue of this function.
Please ensure your MAF or merged data frame uses consistent column types
(e.g. convert allele columns to character before input).
}
\examples{
\dontrun{
# See maftools or TCGAbiolinks documentation for obtaining MAF input
# Example: Download MAF file from TCGA portal
mut_list <- make_mut_matrix(maf = "path_to_maf_file.maf", isTCGA = TRUE, category = "multi")
}
}
\author{
Dongqian Zeng

Shixiang Huang
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_cor.R
\name{batch_cor}
\alias{batch_cor}
\title{Batch Correlation Analysis}
\usage{
batch_cor(data, target, feature, method = c("spearman", "pearson", "kendall"))
}
\arguments{
\item{data}{Data frame containing the target and feature variables.}

\item{target}{Character string specifying the name of the target variable.}

\item{feature}{Character vector specifying the names of feature variables to
correlate with the target.}

\item{method}{Character string specifying the correlation method. Options are
`"spearman"`, `"pearson"`, or `"kendall"`. Default is `"spearman"`.}
}
\value{
Tibble containing the following columns for each feature:
\describe{
  \item{sig_names}{Feature name}
  \item{p.value}{Raw p-value}
  \item{statistic}{Correlation coefficient}
  \item{p.adj}{Adjusted p-value (Benjamini-Hochberg)}
  \item{log10pvalue}{Negative log10-transformed p-value}
  \item{stars}{Significance stars: **** p<0.0001, *** p<0.001,
    ** p<0.01, * p<0.05, + p<0.5}
}
}
\description{
Performs correlation analysis between a target variable and multiple feature
variables. Computes correlation coefficients, p-values, and adjusts for
multiple testing using the Benjamini-Hochberg method.
}
\examples{
# Create a small example dataset
set.seed(123)
data_df <- as.data.frame(matrix(runif(100 * 10), 100, 10))
colnames(data_df) <- paste0("Signature", 1:10)

# Perform batch correlation
results <- batch_cor(
  data = data_df,
  target = "Signature1",
  feature = colnames(data_df)[2:10]
)
head(results)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_helper.R
\name{outputGCT}
\alias{outputGCT}
\title{outputGCT}
\usage{
outputGCT(input.f, output.f)
}
\arguments{
\item{input.f}{A data frame or a character string specifying the path to the input file. If a character string, the file should be a tab-separated table with row names.}

\item{output.f}{A character string specifying the path to the output file.}
}
\value{
No return value. The function writes the dataset to the specified output file in GCT format.
}
\description{
This function converts a gene expression dataset to a GCT format file.
}
\examples{
# Create a sample input data frame
sample_data <- data.frame(
  Gene = c("BRCA1", "TP53", "EGFR"),
  Sample1 = c(10, 15, 8),
  Sample2 = c(12, 18, 7),
  stringsAsFactors = FALSE
)
rownames(sample_data) <- sample_data$Gene
sample_data <- sample_data[, -1]

# Convert the input data frame to GCT format and save to temporary file
output_file <- tempfile(fileext = ".gct")
outputGCT(sample_data, output_file)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iobr_cor_plot.R
\name{iobr_cor_plot}
\alias{iobr_cor_plot}
\title{Integrative Correlation Analysis Between Phenotype and Features}
\usage{
iobr_cor_plot(
  pdata_group,
  id1 = "ID",
  feature_data,
  id2 = "ID",
  target = NULL,
  group = "group3",
  is_target_continuous = TRUE,
  padj_cutoff = 1,
  index = 1,
  category = "signature",
  signature_group = NULL,
  ProjectID = "TCGA",
  palette_box = "nrc",
  cols_box = NULL,
  palette_corplot = "pheatmap",
  palette_heatmap = 2,
  feature_limit = 26,
  character_limit = 60,
  show_heatmap_col_name = FALSE,
  show_col = FALSE,
  show_plot = FALSE,
  path = NULL,
  discrete_x = 20,
  discrete_width = 20,
  show_palettes = FALSE,
  fig.type = "pdf"
)
}
\arguments{
\item{pdata_group}{Data frame containing phenotype data with an identifier column.}

\item{id1}{Character string specifying the column name in `pdata_group` serving
as the sample identifier. Default is `"ID"`.}

\item{feature_data}{Data frame containing feature data with corresponding identifiers.}

\item{id2}{Character string specifying the column name in `feature_data` serving
as the sample identifier. Default is `"ID"`.}

\item{target}{Character string specifying the target variable column name for
continuous analysis. Default is `NULL`.}

\item{group}{Character string specifying the grouping variable name for categorical
analysis. Default is `"group3"`.}

\item{is_target_continuous}{Logical indicating whether the target variable is
continuous, which affects grouping strategy. Default is `TRUE`.}

\item{padj_cutoff}{Numeric value specifying the adjusted p-value cutoff for filtering
features. Default is `1`.}

\item{index}{Numeric index used for ordering output file names. Default is `1`.}

\item{category}{Character string specifying the data category: `"signature"`
or `"gene"`.}

\item{signature_group}{List specifying the grouping variable for signatures.
Options include `"sig_group"` for signature grouping or
`"signature_collection"`/`"signature_tme"` for gene grouping.}

\item{ProjectID}{Character string specifying the project identifier for file naming.}

\item{palette_box}{Character string or integer specifying the color palette for
box plots. Default is `"nrc"`.}

\item{cols_box}{Character vector of specific colors for box plots. Default is
`NULL`.}

\item{palette_corplot}{Character string or integer specifying the color palette for
correlation plots. Default is `"pheatmap"`.}

\item{palette_heatmap}{Integer specifying the color palette index for heatmaps.
Default is `2`.}

\item{feature_limit}{Integer specifying the maximum number of features to display.
Default is `26`.}

\item{character_limit}{Integer specifying the maximum number of characters for
variable labels. Default is `60`.}

\item{show_heatmap_col_name}{Logical indicating whether to display column names on
heatmaps. Default is `FALSE`.}

\item{show_col}{Logical indicating whether to display color codes for palettes.
Default is `FALSE`.}

\item{show_plot}{Logical indicating whether to display plots. Default is `FALSE`.}

\item{path}{Character string specifying the directory path for saving output files.
Default is `NULL`.}

\item{discrete_x}{Numeric threshold for character length beyond which labels are
discretized. Default is `20`.}

\item{discrete_width}{Numeric value specifying the width for label wrapping in plots.
Default is `20`.}

\item{show_palettes}{Logical indicating whether to display color palettes. Default
is `FALSE`.}

\item{fig.type}{Character string specifying the format for saving figures (`"pdf"`,
`"png"`, etc.). Default is `"pdf"`.}
}
\value{
Depending on configuration, returns ggplot2 objects (box plots, heatmaps,
  correlation plots) and/or a data frame containing statistical analysis results.
}
\description{
Performs comprehensive correlation analysis between phenotype data and feature data,
supporting both continuous and categorical phenotypes. Filters features based on
statistical significance and generates publication-ready visualizations including
box plots, heatmaps, and correlation plots.
}
\examples{
set.seed(123)

pdata_group <- data.frame(
  ID = 1:100,
  phenotype_score = rnorm(100)
)

feature_data <- data.frame(
  ID = 1:100,
  Feature1 = rnorm(100),
  Feature2 = rnorm(100),
  Feature3 = rnorm(100)
)

sig_group_example <- list(
  signature = c("Feature1", "Feature2", "Feature3")
)

results <- iobr_cor_plot(
  pdata_group = pdata_group,
  feature_data = feature_data,
  id1 = "ID",
  id2 = "ID",
  target = "phenotype_score",
  is_target_continuous = TRUE,
  category = "signature",
  signature_group = sig_group_example,
  show_plot = FALSE,
  path = tempdir()
)

print(results)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\name{deconvo_timer}
\alias{deconvo_timer}
\title{Deconvolve Using TIMER}
\usage{
deconvo_timer(eset, project = NULL, indications = NULL)
}
\arguments{
\item{eset}{Gene expression matrix.}

\item{project}{Optional project name. Default is `NULL`.}

\item{indications}{Cancer type for each sample (e.g., `"brca"`, `"stad"`).
Must match number of columns in `eset`.}
}
\value{
Data frame with TIMER cell fractions. Columns suffixed with `_TIMER`.
}
\description{
TIMER deconvolution for cancer-specific immune estimation.
}
\examples{
\dontrun{
immune <- load_data("immuneCuratedData")
cancer_genes <- load_data("cancer_type_genes")
if (!is.null(immune) && !is.null(cancer_genes)) {
  set.seed(123)
  genes <- unique(c(head(rownames(immune$genes), 100), cancer_genes[["stad"]]))
  sim_eset <- matrix(rnorm(length(genes) * 2), length(genes), 2)
  rownames(sim_eset) <- genes
  colnames(sim_eset) <- paste0("Sample", 1:2)
  result <- deconvo_timer(eset = sim_eset, project = "TCGA-STAD",
                          indications = rep("stad", 2))
  if (!is.null(result)) head(result)
}
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quantiseq.R
\name{deconvolute_quantiseq.default}
\alias{deconvolute_quantiseq.default}
\title{Use quanTIseq to Deconvolute a Gene Expression Matrix}
\usage{
deconvolute_quantiseq.default(
  mix.mat,
  arrays = FALSE,
  signame = "TIL10",
  tumor = FALSE,
  mRNAscale = TRUE,
  method = c("lsei", "hampel", "huber", "bisquare"),
  rmgenes = "unassigned"
)
}
\arguments{
\item{mix.mat}{Data frame or matrix. Gene expression matrix with gene symbols
on the first column and sample IDs on the first row. Expression data must
be on non-log scale (TPM for RNA-seq or expression values for microarrays).}

\item{arrays}{Logical. Whether expression data are from microarrays.
Default is FALSE. If TRUE, the rmgenes parameter is set to "none".}

\item{signame}{Character. Name of the signature matrix. Currently only
"TIL10" is available. Default is "TIL10".}

\item{tumor}{Logical. Whether expression data are from tumor samples.
If TRUE, signature genes with high expression in tumor samples are removed.
Default is FALSE.}

\item{mRNAscale}{Logical. Whether cell fractions must be scaled to account
for cell-type-specific mRNA content. Default is TRUE.}

\item{method}{Character. Deconvolution method: "hampel", "huber", "bisquare"
for robust regression, or "lsei" for constrained least squares.
Default is "lsei".}

\item{rmgenes}{Character. Genes to remove: "unassigned" (default), "default",
"none", or "path".}
}
\value{
Data frame with cell fractions for each sample.
}
\description{
Deconvolutes gene expression data to estimate immune cell fractions using
the quanTIseq method. Source code from https://github.com/FFinotello/quanTIseq.
}
\examples{
\dontrun{
quantiseq_data <- load_data("quantiseq_data")
if (!is.null(quantiseq_data)) {
  common_genes <- rownames(quantiseq_data$TIL10_signature)
  tpm_matrix <- as.data.frame(matrix(
    abs(rnorm(length(common_genes) * 2, mean = 5, sd = 2)),
    nrow = length(common_genes), ncol = 2
  ))
  rownames(tpm_matrix) <- common_genes
  colnames(tpm_matrix) <- paste0("Sample", 1:2)
  results <- deconvolute_quantiseq.default(mix.mat = tpm_matrix)
  if (!is.null(results)) head(results)
}
}
}
\references{
F. Finotello, C. Mayer, C. Plattner, G. Laschober, D. Rieder,
H. Hackl, A. Krogsdam, W. Posch, D. Wilflingseder, S. Sopper, M. Jsselsteijn,
D. Johnsons, Y. Xu, Y. Wang, M. E. Sanders, M. V. Estrada, P.
Ericsson-Gonzalez, J. Balko, N. F. de Miranda, Z. Trajanoski.
"quanTIseq: quantifying immune contexture of human tumors".
bioRxiv 223180. https://doi.org/10.1101/223180.
}
\author{
Finotello F, et al. (adapted for IOBR)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sig_forest.R
\name{sig_forest}
\alias{sig_forest}
\title{Forest Plot for Survival Analysis Results}
\usage{
sig_forest(
  data,
  signature,
  pvalue = "P",
  HR = "HR",
  CI_low_0.95 = "CI_low_0.95",
  CI_up_0.95 = "CI_up_0.95",
  n = 10,
  max_character = 25,
  discrete_width = 35,
  color_option = 1,
  cols = NULL,
  text.size = 13
)
}
\arguments{
\item{data}{Data frame with survival analysis results including p-values,
hazard ratios, and confidence intervals.}

\item{signature}{Character string. Column name for signatures or feature names.}

\item{pvalue}{Character string. Column name for p-values. Default is `"P"`.}

\item{HR}{Character string. Column name for hazard ratios. Default is `"HR"`.}

\item{CI_low_0.95}{Character string. Column name for lower CI bound.
Default is `"CI_low_0.95"`.}

\item{CI_up_0.95}{Character string. Column name for upper CI bound.
Default is `"CI_up_0.95"`.}

\item{n}{Integer. Maximum number of signatures to display. Default is `10`.}

\item{max_character}{Integer. Maximum characters for labels before wrapping.
Default is `25`.}

\item{discrete_width}{Integer. Width for discretizing long labels. Default is `35`.}

\item{color_option}{Integer. Color option for p-value gradient (1, 2, or 3).
Default is `1`.}

\item{cols}{Character vector. Custom colors for p-value gradient (low to high).
Default is `NULL`.}

\item{text.size}{Numeric. Text size for y-axis labels. Default is `13`.}
}
\value{
A ggplot2 object of the forest plot.
}
\description{
Generates a forest plot to visualize hazard ratios, confidence intervals,
and p-values for gene signatures or features from survival analysis.
}
\examples{
# Example with sample survival results
sample_results <- data.frame(
  ID = c("Sig1", "Sig2", "Sig3"),
  HR = c(1.5, 0.8, 2.0),
  P = c(0.01, 0.05, 0.001),
  CI_low_0.95 = c(1.1, 0.6, 1.5),
  CI_up_0.95 = c(2.0, 1.0, 2.8)
)
sig_forest(data = sample_results, signature = "ID")
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subgroup_survival.R
\name{subgroup_survival}
\alias{subgroup_survival}
\title{Subgroup Survival Analysis Using Cox Proportional Hazards Models}
\usage{
subgroup_survival(
  pdata,
  time_name = "time",
  status_name = "status",
  variables,
  object
)
}
\arguments{
\item{pdata}{Data frame containing variables, follow-up time, and outcome.}

\item{time_name}{Character. Column name of follow-up time. Default is `"time"`.}

\item{status_name}{Character. Column name of event status (1/0).
Default is `"status"`.}

\item{variables}{Character vector. Subgrouping variables (each processed
independently).}

\item{object}{Character. Variable of interest used in Cox model. If it has
levels "High" and "Low", recode "High" to 0 and "Low" to 1 before calling.}
}
\value{
Data frame summarizing subgroup Cox results (HR, CI, p-value).
}
\description{
Extracts hazard ratios (HR) and 95% confidence intervals from Cox
proportional hazards models across specified subgroups.
}
\examples{
set.seed(123)
test_pdata <- data.frame(
  time = runif(100, 1, 100),
  status = sample(c(0, 1), 100, replace = TRUE),
  subgroup = sample(c("A", "B"), 100, replace = TRUE),
  score = rnorm(100)
)
res <- subgroup_survival(
  pdata = test_pdata,
  time_name = "time",
  status_name = "status",
  variables = "subgroup",
  object = "score"
)
print(res)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_selection.R
\name{limma.dif}
\alias{limma.dif}
\title{Differential Expression Analysis Using Limma}
\usage{
limma.dif(exprdata, pdata, contrastfml)
}
\arguments{
\item{exprdata}{A matrix with rownames as features like gene symbols or cgi, and colnames as samples.}

\item{pdata}{A two-column dataframe where the first column matches the colnames of exprdata and the second column contains the grouping variable.}

\item{contrastfml}{A character vector for contrasts to be tested (see ?makeContrasts for more details).}
}
\value{
Returns a dataframe from limma::topTable, which includes genes as rows and columns like genelist, logFC, AveExpr, etc.
}
\description{
Performs differential expression analysis using the limma package on a
given gene expression dataset.
Constructs a design matrix from phenotype data, fits a linear model,
applies contrasts, and computes
statistics for differential expression.
}
\examples{
# Toy example with 100 genes and 6 samples
set.seed(123)
exprdata <- matrix(
  rnorm(100 * 6),
  nrow = 100,
  ncol = 6,
  dimnames = list(
    paste0("gene", 1:100),
    paste0("sample", 1:6)
  )
)

# Phenotype data: 3 vs 3
pdata <- data.frame(
  sample = colnames(exprdata),
  group = rep(c("group1", "group2"), each = 3),
  stringsAsFactors = FALSE
)

# Differential expression: group1 vs group2
res <- limma.dif(
  exprdata = exprdata,
  pdata = pdata,
  contrastfml = "group1 - group2"
)
head(res)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sig_box.R
\name{sig_box}
\alias{sig_box}
\title{Signature Box Plot with Statistical Comparisons}
\usage{
sig_box(
  data,
  signature,
  variable,
  palette = "nrc",
  cols = NULL,
  jitter = FALSE,
  point_size = 5,
  angle_x_text = 0,
  hjust = 0.5,
  show_pairwise_p = TRUE,
  show_overall_p = FALSE,
  return_stat_res = FALSE,
  size_of_pvalue = 6,
  size_of_font = 10,
  assay = NULL,
  slot = "scale.data",
  scale = FALSE
)
}
\arguments{
\item{data}{Data frame or Seurat object containing the signature and
grouping variable.}

\item{signature}{Character string specifying the column name (or feature name
in Seurat) for the signature values to plot on the y-axis.}

\item{variable}{Character string specifying the grouping variable column name
for the x-axis.}

\item{palette}{Character string specifying the color palette name.
Default is `"nrc"`.}

\item{cols}{Character vector of custom fill colors. If `NULL`, palette is
used. Default is `NULL`.}

\item{jitter}{Logical indicating whether to add jittered points to the box
plot. Default is `FALSE`.}

\item{point_size}{Numeric value specifying the size of jittered points.
Default is 5.}

\item{angle_x_text}{Numeric value specifying the rotation angle for x-axis
labels (in degrees). Default is 0.}

\item{hjust}{Numeric value specifying the horizontal justification of x-axis
labels. Default is 0.5.}

\item{show_pairwise_p}{Logical indicating whether to display pairwise
comparison p-values between groups. Default is `TRUE`.}

\item{show_overall_p}{Logical indicating whether to display the overall
group difference p-value. Default is `FALSE`.}

\item{return_stat_res}{Logical indicating whether to return statistical test
results instead of the plot. Default is `FALSE`.}

\item{size_of_pvalue}{Numeric value specifying the font size for p-values.
Default is 6.}

\item{size_of_font}{Numeric value specifying the base font size.
Default is 10.}

\item{assay}{Character string specifying the assay name (for Seurat objects).
Default is `NULL`.}

\item{slot}{Character string specifying the slot name (for Seurat objects).
Default is `"scale.data"`.}

\item{scale}{Logical indicating whether to scale signature values (z-score
transformation). Default is `FALSE`.}
}
\value{
If `return_stat_res = FALSE`, returns a ggplot2 object. If
  `return_stat_res = TRUE`, returns a data frame containing statistical test
  results.
}
\description{
Creates box plots to visualize signature distributions across groups with
optional statistical pairwise comparisons. Supports both data frames and
Seurat objects for single-cell data visualization.
}
\examples{
# Create small example data
set.seed(123)
test_data <- data.frame(
  subtype = rep(c("A", "B"), each = 50),
  TMEscore_plus = rnorm(100)
)
sig_box(
  data = test_data,
  signature = "TMEscore_plus",
  variable = "subtype",
  jitter = TRUE,
  palette = "jco"
)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch_surv.R
\name{batch_surv}
\alias{batch_surv}
\title{Batch Survival Analysis}
\usage{
batch_surv(
  pdata,
  variable,
  time = "time",
  status = "status",
  best_cutoff = FALSE
)
}
\arguments{
\item{pdata}{Data frame containing survival time, event status, and predictor variables.}

\item{variable}{Character vector specifying the names of predictor variables to analyze.}

\item{time}{Character string specifying the column name containing follow-up time.
Default is `"time"`.}

\item{status}{Character string specifying the column name containing event status
(1 = event occurred, 0 = censored). Default is `"status"`.}

\item{best_cutoff}{Logical indicating whether to compute optimal cutoffs for
continuous variables and analyze dichotomized versions. Default is `FALSE`.}
}
\value{
Data frame containing hazard ratios (HR), 95% confidence intervals (CI),
  and p-values for each variable, sorted by p-value.
}
\description{
Performs Cox proportional hazards regression analysis on multiple variables.
Optionally determines optimal cutoffs to dichotomize continuous predictors before
modeling. Returns hazard ratios, confidence intervals, and p-values for each variable.
}
\examples{
# Create small example data
set.seed(123)
test_data <- data.frame(
  OS_time = runif(100, 0, 100),
  OS_status = sample(c(0, 1), 100, replace = TRUE),
  Signature1 = rnorm(100),
  Signature2 = rnorm(100)
)
batch_surv(
  pdata = test_data,
  variable = c("Signature1", "Signature2"),
  time = "OS_time",
  status = "OS_status"
)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_data.R
\name{transform_data}
\alias{transform_data}
\title{Transform NA, Inf, or Zero Values in Data}
\usage{
transform_data(data, feature, data_type = c("NA", "Inf", "zero"), into = 0)
}
\arguments{
\item{data}{Data frame. Input data to be transformed.}

\item{feature}{Character vector. Column names in `data` to apply
transformation.}

\item{data_type}{Character. Type of value to replace: `"NA"`, `"Inf"`, or
`"zero"`.}

\item{into}{Value to replace specified type with. Default is 0. If `"mean"`,
replaces with column mean (excluding NA/Inf values as appropriate).}
}
\value{
Data frame with specified transformations applied to selected
  features.
}
\description{
Replaces NA, Inf, or zero values in specified columns of a data frame with a
user-defined value or the column mean.
}
\examples{
data_matrix <- data.frame(
  A = c(1, 2, NA, 4, Inf),
  B = c(Inf, 2, 3, 4, 5),
  C = c(0, 0, 0, 1, 2)
)

# Replace NAs with 0
transform_data(data_matrix, feature = c("A", "B"), data_type = "NA")

# Replace Inf values with the mean of the column
transform_data(data_matrix,
  feature = c("A", "B"),
  data_type = "Inf", into = "mean"
)

# Replace zeros with -1 in column C
transform_data(data_matrix, feature = "C", data_type = "zero", into = -1)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_data.R
\name{get_iobr_cache_dir}
\alias{get_iobr_cache_dir}
\title{Get IOBR Cache Directory}
\usage{
get_iobr_cache_dir(cache_dir = NULL)
}
\arguments{
\item{cache_dir}{Optional character string to override the current setting.}
}
\value{
Character string with the cache directory path.
}
\description{
Returns the current cache directory for IOBR downloaded data.
To comply with CRAN policies, the default cache directory is a session-specific
temporary directory. Users can opt-in to a persistent cache by setting
`options(IOBR.cache_dir = "your/path")` or using `set_iobr_cache_dir()`.

The cache directory is determined in the following priority order:
1. Function argument `cache_dir` (if provided)
2. Option `IOBR.cache_dir` (if set via `options()`)
3. Default: A session-specific temporary directory (`file.path(tempdir(), "IOBR_cache")`)
}
\examples{
# Get current cache directory (defaults to tempdir)
get_iobr_cache_dir()

# Set custom cache directory via options (use tempdir() for examples)
options(IOBR.cache_dir = tempdir())
get_iobr_cache_dir()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrichment_barplot.R
\name{enrichment_barplot}
\alias{enrichment_barplot}
\title{Enrichment Bar Plot with Two Directions}
\usage{
enrichment_barplot(
  up_terms,
  down_terms,
  terms = "Description",
  pvalue = "pvalue",
  group = "group",
  palette = "jama",
  cols = NULL,
  title = "Gene Ontology Enrichment",
  width_wrap = 30,
  font_terms = 15
)
}
\arguments{
\item{up_terms}{Data frame for up-regulated terms.}

\item{down_terms}{Data frame for down-regulated terms.}

\item{terms}{Column name for term descriptions. Default is "Description".}

\item{pvalue}{Column name for p-values. Default is "pvalue".}

\item{group}{Column name for group indicator. Default is "group".}

\item{palette}{Color palette. Default is "jama".}

\item{cols}{Character vector. Custom colors for bars. If NULL, uses palette.
Default is NULL.}

\item{title}{Plot title. Default is "Gene Ontology Enrichment".}

\item{width_wrap}{Maximum width for wrapping pathway names. Default is 30.}

\item{font_terms}{Font size for axis labels. Default is 15.}
}
\value{
A ggplot object of the enrichment bar plot.
}
\description{
Creates a bar plot visualizing enrichment results for up-regulated and
down-regulated terms, using -log10(p-values) to indicate significance.
}
\examples{
up_terms <- data.frame(
  Description = c("Pathway1", "Pathway2"),
  pvalue = c(0.001, 0.01)
)
down_terms <- data.frame(
  Description = c("Pathway4", "Pathway5"),
  pvalue = c(0.005, 0.02)
)
p <- enrichment_barplot(
  up_terms = up_terms,
  down_terms = down_terms,
  title = "Custom Enrichment Plot"
)
p
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IPS_calculation.R
\name{IPS_calculation}
\alias{IPS_calculation}
\title{Calculate Immunophenoscore (IPS)}
\usage{
IPS_calculation(eset, project = NULL, plot = FALSE)
}
\arguments{
\item{eset}{Gene expression matrix with official human gene symbols (HGNC)
as rownames. Expression values should be log2(TPM+1) or will be transformed
if max value > 100.}

\item{project}{Character string for project identifier. Default is NULL.}

\item{plot}{Logical. Whether to generate immunophenogram plots. Default is FALSE.}
}
\value{
Data frame containing:
\describe{
  \item{MHC}{MHC molecules score}
  \item{EC}{Effector cells score}
  \item{SC}{Suppressor cells score}
  \item{CP}{Checkpoints/Immunomodulators score}
  \item{AZ}{Aggregate score (sum of MHC, CP, EC, SC)}
  \item{IPS}{Immunophenoscore (0-10 scale)}
}
}
\description{
Calculates Immunophenoscore (IPS) from gene expression data. IPS is a
composite score measuring immunophenotype based on four major categories:
MHC molecules, immunomodulators, effector cells, and suppressor cells.
}
\examples{
\dontrun{
example_genes <- c(
  "HLA-A", "HLA-B", "HLA-C", "CD274", "PDCD1", "CTLA4",
  "CD8A", "CD8B", "GZMB", "PRF1", "FOXP3", "IL10"
)
sim_eset <- as.data.frame(matrix(
  rnorm(length(example_genes) * 5, mean = 5, sd = 2),
  nrow = length(example_genes), ncol = 5
))
rownames(sim_eset) <- example_genes
colnames(sim_eset) <- paste0("Sample", 1:5)
ips_result <- IPS_calculation(eset = sim_eset, project = "Example", plot = FALSE)
if (!is.null(ips_result)) head(ips_result)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_riskscore.R
\name{add_riskscore}
\alias{add_riskscore}
\title{Add Risk Score to Dataset}
\usage{
add_riskscore(
  input,
  family = c("cox", "binary"),
  target = NULL,
  time = NULL,
  status = NULL,
  vars,
  new_var_name = "riskscore"
)
}
\arguments{
\item{input}{Data frame containing the variables for analysis.}

\item{family}{Character string specifying the model family: `"cox"` for Cox
proportional hazards regression or `"binary"` for logistic regression.
Default is `"cox"`.}

\item{target}{Character string specifying the target variable name. Required
when `family = "binary"`.}

\item{time}{Character string specifying the time-to-event variable name.
Required when `family = "cox"`.}

\item{status}{Character string specifying the event status variable name.
Required when `family = "cox"`.}

\item{vars}{Character vector of variable names to include in the model.}

\item{new_var_name}{Character string specifying the name for the new risk
score column. Default is `"riskscore"`.}
}
\value{
Data frame identical to `input` with an additional column containing
  risk scores (linear predictors for Cox models or predicted probabilities
  for logistic models).
}
\description{
Computes a risk score for each observation based on Cox proportional hazards
regression or binary logistic regression. The function fits the specified
model and returns the dataset with an added risk score column.
}
\examples{
set.seed(123)
input_data <- data.frame(
  time = rexp(100),
  status = rbinom(100, 1, 0.5),
  age = rnorm(100, 60, 10),
  score1 = rnorm(100),
  score2 = rnorm(100)
)
result <- add_riskscore(
  input_data,
  time = "time", status = "status",
  vars = c("age", "score1", "score2")
)
head(result$riskscore)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_data.R
\name{list_iobr_mirrors}
\alias{list_iobr_mirrors}
\title{List Current Download Mirrors}
\usage{
list_iobr_mirrors()
}
\value{
Character vector of mirror URLs.
}
\description{
Returns the current list of download mirrors.
}
\examples{
list_iobr_mirrors()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LR_cal.R
\name{LR_cal}
\alias{LR_cal}
\title{Calculate Ligand-Receptor Interaction Scores}
\usage{
LR_cal(
  eset,
  data_type = c("count", "tpm"),
  id_type = "ensembl",
  cancer_type = "pancan"
)
}
\arguments{
\item{eset}{Gene expression matrix with genes as rows and samples as columns.}

\item{data_type}{Type of input data. Options are `"count"` or `"tpm"`.
If `"count"`, data will be converted to TPM before analysis.}

\item{id_type}{Type of gene identifier. Default is `"ensembl"`.}

\item{cancer_type}{Character string specifying the cancer type for easier.
Default is `"pancan"` for pan-cancer analysis.}
}
\value{
Data frame containing ligand-receptor interaction scores with
  sample IDs as row names.
}
\description{
Quantifies ligand-receptor interactions in the tumor microenvironment from
bulk gene expression data using the easier package. This function processes
raw counts or TPM data and computes interaction scores for each sample.
}
\examples{
# LR_cal requires HGNC gene symbols as rownames
# Create a simple example with gene symbols
example_genes <- c(
  "TGFB1", "EGFR", "VEGFA", "PDGFB", "FGF2", "CXCL12",
  "CXCR4", "IL6", "IL6R", "TNF", "TNFRSF1A", "IFNG"
)
sim_eset <- as.data.frame(matrix(
  rnorm(length(example_genes) * 10, mean = 5, sd = 2),
  nrow = length(example_genes), ncol = 10
))
rownames(sim_eset) <- example_genes
colnames(sim_eset) <- paste0("Sample", 1:10)
\dontrun{
if (requireNamespace("easier", quietly = TRUE)) {
  tryCatch({
    lr <- LR_cal(eset = sim_eset, data_type = "tpm")
    head(lr)
  }, error = function(e) {
    message("Example skipped: could not download ExperimentHub data")
  })
}
}
}
\references{
Lapuente-Santana, van Genderen, M., Hilbers, P., Finotello, F., & Eduati, F. (2021).
Interpretable systems biomarkers predict response to immune-checkpoint inhibitors.
Patterns (New York, N.Y.), 2(8), 100293. https://doi.org/10.1016/j.patter.2021.100293
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{palettes}
\alias{palettes}
\title{Select Color Palettes for Visualization}
\usage{
palettes(
  category = "box",
  palette = "nrc",
  alpha = 1,
  counts = 50,
  show_col = TRUE,
  show_message = FALSE
)
}
\arguments{
\item{category}{Character. Plot/palette category: one of `box`, `continue2`,
`continue`, `random`, `heatmap`, `heatmap3`, `tidyheatmap`.}

\item{palette}{Character or numeric. Palette name or index (varies by category).}

\item{alpha}{Numeric. Alpha (transparency) scaling factor. Default is 1.}

\item{counts}{Integer. Number of colors (for continuous palettes).
Default is 50.}

\item{show_col}{Logical. If TRUE, prints the palette. Default is TRUE.}

\item{show_message}{Logical. If TRUE, prints available options.
Default is FALSE.}
}
\value{
Character vector of hex color codes.
}
\description{
Provides curated qualitative, sequential, and diverging palettes for multiple
plot types. Supports intensity adjustment and preview.
}
\examples{
colors <- palettes(category = "box", palette = "nrc", show_col = FALSE)
heatmap_colors <- palettes(
  category = "heatmap", palette = 1, counts = 10, show_col = FALSE
)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_doc.R
\docType{data}
\name{pdata_stad}
\alias{pdata_stad}
\title{Toy STAD Phenotype Data}
\format{
A data frame with one row per TCGA-STAD sample and 8 variables:
\describe{
  \item{ID}{Character. TCGA sample barcode, matching the column names
  of \code{eset_stad}.}
  \item{stage}{Factor. Pathological stage (e.g. \code{"Stage_I"},
  \code{"Stage_II"}, \code{"Stage_III"}, \code{"Stage_IV"}).}
  \item{status}{Factor. Vital status at last follow-up
  (\code{"Alive"} or \code{"Dead"}).}
  \item{Lauren}{Factor. Lauren classification of gastric cancer
  (\code{"Intestinal"}, \code{"Diffuse"}, \code{"Mixed"} or \code{NA}).}
  \item{subtype}{Factor. Molecular subtype (e.g. \code{"CIN"}, \code{"EBV"},
  \code{"GS"}, \code{"MSI"}).}
  \item{EBV}{Factor. EBV status of the tumour (\code{"Positive"} or
  \code{"Negative"}).}
  \item{time}{Numeric. Overall survival or follow-up time, typically
  measured in months.}
  \item{OS_status}{Integer/binary. Overall survival status indicator.
  (\code{1} = death, \code{0} = censored)}
}
}
\usage{
data(pdata_stad)
}
\description{
A data frame containing clinical and pathological annotations for the
TCGA stomach adenocarcinoma (STAD) cohort. Each row corresponds to one
tumour sample and can be matched to the columns of \code{eset_stad} via
the \code{ID} column. This dataset is typically used together with
\code{eset_stad} in examples of survival analysis, subgroup comparison
and immune deconvolution in the IOBR package.
}
\examples{
data(pdata_stad)
head(pdata_stad)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_cor.R
\name{get_cor}
\alias{get_cor}
\title{Calculate and Visualize Correlation Between Two Variables}
\usage{
get_cor(
  eset,
  pdata = NULL,
  var1,
  var2,
  is.matrix = FALSE,
  id_eset = "ID",
  id_pdata = "ID",
  scale = TRUE,
  subtype = NULL,
  na.subtype.rm = FALSE,
  color_subtype = NULL,
  palette = "jama",
  index = NULL,
  method = c("spearman", "pearson", "kendall"),
  show_cor_result = TRUE,
  col_line = NULL,
  id = NULL,
  show_label = FALSE,
  point_size = 4,
  title = NULL,
  alpha = 0.5,
  title_size = 1.5,
  text_size = 10,
  axis_angle = 0,
  hjust = 0,
  show_plot = TRUE,
  save_plot = FALSE,
  path = NULL,
  fig.format = "png",
  fig.width = 7,
  fig.height = 7.3,
  add.hdr.line = FALSE
)
}
\arguments{
\item{eset}{Dataset containing the variables (data frame or matrix).}

\item{pdata}{Optional phenotype data frame. Default is `NULL`.}

\item{var1}{Name of the first variable.}

\item{var2}{Name of the second variable.}

\item{is.matrix}{Logical indicating if `eset` is a matrix with features as rows.
Default is `FALSE`.}

\item{id_eset}{ID column in `eset`. Default is `"ID"`.}

\item{id_pdata}{ID column in `pdata`. Default is `"ID"`.}

\item{scale}{Logical indicating whether to scale data. Default is `TRUE`.}

\item{subtype}{Optional grouping variable for coloring points. Default is `NULL`.}

\item{na.subtype.rm}{Logical indicating whether to remove NA in subtype.
Default is `FALSE`.}

\item{color_subtype}{Colors for subtypes. Default is `NULL`.}

\item{palette}{Color palette name. Default is `"jama"`.}

\item{index}{Plot index for filename. Default is `NULL` (uses 1).}

\item{method}{Correlation method: `"spearman"`, `"pearson"`, or `"kendall"`.
Default is `"spearman"`.}

\item{show_cor_result}{Logical indicating whether to print correlation result.
Default is `TRUE`.}

\item{col_line}{Color of regression line. Default is `NULL` (auto-determine).}

\item{id}{Column for point labels. Default is `NULL`.}

\item{show_label}{Logical indicating whether to show labels. Default is `FALSE`.}

\item{point_size}{Size of points. Default is 4.}

\item{title}{Plot title. Default is `NULL`.}

\item{alpha}{Transparency of points. Default is 0.5.}

\item{title_size}{Title font size. Default is 1.5.}

\item{text_size}{Text font size. Default is 10.}

\item{axis_angle}{Axis label angle. Default is 0.}

\item{hjust}{Horizontal justification. Default is 0.}

\item{show_plot}{Logical indicating whether to display plot. Default is `TRUE`.}

\item{save_plot}{Logical indicating whether to save plot. Default is `FALSE`.}

\item{path}{Save path. Default is `NULL`.}

\item{fig.format}{Figure format: `"png"` or `"pdf"`. Default is `"png"`.}

\item{fig.width}{Figure width in inches. Default is 7.}

\item{fig.height}{Figure height in inches. Default is 7.3.}

\item{add.hdr.line}{Logical for adding HDR (high density region) lines.
Default is `FALSE`.}
}
\value{
A ggplot object of the correlation plot.
}
\description{
Calculates and visualizes the correlation between two variables with options
for scaling, handling missing values, and incorporating grouping data.
}
\examples{
# Simulate data
set.seed(123)
sim_eset <- matrix(rnorm(100 * 20), 100, 20)
rownames(sim_eset) <- c("GZMB", "CD274", paste0("Gene", 3:100))
colnames(sim_eset) <- paste0("Sample", 1:20)

# Calculate and plot correlation
p <- get_cor(eset = sim_eset, is.matrix = TRUE, var1 = "GZMB", var2 = "CD274", show_plot = FALSE)
if (!is.null(p)) print(p)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BinomialModel.R
\name{RegressionResult}
\alias{RegressionResult}
\title{Regression Result Computation}
\usage{
RegressionResult(train.x, train.y, test.x, test.y, model)
}
\arguments{
\item{train.x}{Training predictors matrix.}

\item{train.y}{Training outcomes (binary factor).}

\item{test.x}{Testing predictors matrix.}

\item{test.y}{Testing outcomes (binary factor).}

\item{model}{Fitted cv.glmnet model object.}
}
\value{
List containing:
\describe{
  \item{model}{The fitted cv.glmnet model}
  \item{coefs}{Data frame with feature names and coefficients at lambda.min and lambda.1se}
  \item{AUC}{Matrix of AUC values for train/test sets at both lambda values}
}
}
\description{
Computes regression results with coefficients at lambda.min and lambda.1se,
and evaluates AUC for binomial outcomes. Returns a comprehensive summary
of model performance on both training and testing datasets.
}
\examples{
if (requireNamespace("glmnet", quietly = TRUE)) {
  set.seed(123)
  train_data <- matrix(rnorm(100 * 10), ncol = 10)
  train_outcome <- rbinom(100, 1, 0.5)
  test_data <- matrix(rnorm(50 * 10), ncol = 10)
  test_outcome <- rbinom(50, 1, 0.5)
  fitted_model <- glmnet::cv.glmnet(train_data, train_outcome, family = "binomial", nfolds = 5)
  results <- RegressionResult(
    train.x = train_data, train.y = train_outcome,
    test.x = test_data, test.y = test_outcome, model = fitted_model
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_cols.R
\name{get_cols}
\alias{get_cols}
\title{Set and View Color Palettes}
\usage{
get_cols(cols = "normal", palette = 1, show_col = TRUE, seed = 123)
}
\arguments{
\item{cols}{Character vector of colors, or one of:
- `"normal"`: Use standard palette
- `"random"`: Randomly shuffle the palette
Default is `"normal"`.}

\item{palette}{Numeric or character specifying the palette. Options are
1, 2, 3, 4, or palette name. Default is 1.}

\item{show_col}{Logical indicating whether to display the color palette.
Default is `TRUE`.}

\item{seed}{Integer seed for random number generator when `cols = "random"`.
Default is 123.}
}
\value{
Character vector of colors.
}
\description{
Retrieves color palettes from the IOBR package with options for randomization
and visualization. Users can specify predefined palettes or provide custom
colors.
}
\examples{
# Get default palette
mycols <- get_cols()

# Get random palette
mycols <- get_cols(cols = "random", seed = 456)

# Use custom colors
mycols <- get_cols(cols = c("red", "blue", "green"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_sig_score.R
\name{calculate_sig_score}
\alias{calculate_sig_score}
\title{Calculate Signature Score}
\usage{
calculate_sig_score(
  pdata = NULL,
  eset,
  signature = NULL,
  method = c("pca", "ssgsea", "zscore", "integration"),
  mini_gene_count = 3,
  column_of_sample = "ID",
  print_gene_proportion = FALSE,
  print_filtered_signatures = FALSE,
  adjust_eset = FALSE,
  parallel.size = 1L,
  ...
)
}
\arguments{
\item{pdata}{Phenotype data. If `NULL`, created from `eset` column names.}

\item{eset}{Expression matrix (CPM, TPM, RPKM, FPKM, etc.).}

\item{signature}{List of gene signatures. Can also be a character string
naming a built-in signature collection (e.g., `"signature_collection"`,
`"signature_tme"`, `"go_bp"`, `"kegg"`, `"hallmark"`).}

\item{method}{Scoring method: `"pca"`, `"ssgsea"`, `"zscore"`,
or `"integration"`. Default is `"pca"`.}

\item{mini_gene_count}{Minimum genes required per signature. Default is 3
(or 5 for ssGSEA).}

\item{column_of_sample}{Column with sample IDs in `pdata`. Default is `"ID"`.}

\item{print_gene_proportion}{Logical: print gene coverage. Default is
`FALSE`.}

\item{print_filtered_signatures}{Logical: print filtered signatures.
Default is `FALSE`.}

\item{adjust_eset}{Logical: clean problematic features. Default is `FALSE`.}

\item{parallel.size}{Parallel workers for ssGSEA. Default is 1.}

\item{...}{Additional arguments passed to specific methods.}
}
\value{
Tibble with phenotype data and signature scores.
}
\description{
Main interface for calculating signature scores from gene expression data.
Supports PCA, z-score, ssGSEA, and integration methods.
}
\examples{
set.seed(123)
eset <- matrix(rnorm(1000), nrow = 100, ncol = 10)
rownames(eset) <- paste0("Gene", 1:100)
colnames(eset) <- paste0("Sample", 1:10)
signature <- list(
  Signature1 = paste0("Gene", 1:10),
  Signature2 = paste0("Gene", 11:20)
)
\donttest{
result <- calculate_sig_score(eset = eset, signature = signature, method = "pca")
}
}
\references{
\enumerate{
  \item Hänzelmann S, Castelo R, Guinney J. GSVA: gene set variation
    analysis. BMC Bioinformatics. 2013;14:7.
  \item Mariathasan S, et al. TGF\eqn{\beta} attenuates tumour response to PD-L1 blockade.
    Nature. 2018;554:544-548.
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_group.R
\name{surv_group}
\alias{surv_group}
\title{Generate Kaplan-Meier Survival Plots for Categorical Groups}
\usage{
surv_group(
  input_pdata,
  target_group,
  ID = "ID",
  levels = c("High", "Low"),
  reference_group = NULL,
  project = NULL,
  time = "time",
  status = "status",
  time_type = "month",
  break_month = "auto",
  cols = NULL,
  palette = "jama",
  mini_sig = "score",
  save_path = NULL,
  fig.type = "pdf",
  index = 1,
  width = 6,
  height = 6.5,
  font.size.table = 3
)
}
\arguments{
\item{input_pdata}{Data frame containing survival data and grouping variables.}

\item{target_group}{Name of column containing the grouping variable.}

\item{ID}{Name of column with unique identifiers. Default is "ID".}

\item{levels}{Names for levels of target_group (for binary groups).
Default is c("High", "Low").}

\item{reference_group}{Reference level for binary comparison. Default is NULL.}

\item{project}{Optional title for plot. Default is NULL.}

\item{time}{Name of column with follow-up times. Default is "time".}

\item{status}{Name of column with event indicators. Default is "status".}

\item{time_type}{Units: "month" or "day". Default is "month".}

\item{break_month}{X-axis break interval. If "auto", calculated automatically.
Default is "auto".}

\item{cols}{Color vector for plot lines. Default is NULL.}

\item{palette}{Color palette name. Default is "jama".}

\item{mini_sig}{Prefix label for variables. Default is "score".}

\item{save_path}{Directory for saving plot. Default is NULL.}

\item{fig.type}{File format: "pdf" or "png". Default is "pdf".}

\item{index}{Identifier for file naming. Default is 1.}

\item{width}{Plot width. Default is 6.}

\item{height}{Plot height. Default is 6.5.}

\item{font.size.table}{Font size for risk table. Default is 3.}
}
\value{
Kaplan-Meier plot object.
}
\description{
Creates Kaplan-Meier survival plots for data grouped by a categorical variable.
Handles both binary and multi-level categorical groups with customizable plot
aesthetics.
}
\examples{
# Simulate data
set.seed(123)
sim_pdata <- data.frame(
  ID = paste0("Sample", 1:100),
  Lauren = sample(c("Intestinal", "Diffuse"), 100, replace = TRUE),
  time = runif(100, 1, 60),
  OS_status = sample(0:1, 100, replace = TRUE)
)
# Run survival analysis (survival and survminer are imported packages)
result <- surv_group(
  input_pdata = sim_pdata,
  target_group = "Lauren",
  time = "time",
  status = "OS_status",
  save_path = NULL
)
if (!is.null(result)) print(result)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iobr_deconvo_pipeline.R
\name{iobr_deconvo_pipeline}
\alias{iobr_deconvo_pipeline}
\title{Tumor Microenvironment (TME) Deconvolution Pipeline}
\usage{
iobr_deconvo_pipeline(
  eset,
  project,
  array,
  tumor_type,
  path = NULL,
  permutation = 1000
)
}
\arguments{
\item{eset}{Numeric matrix. Gene expression (TPM/log scale) with genes in rows.}

\item{project}{Character. Project name (used in output naming).}

\item{array}{Logical. Whether data originated from an array platform. Affects deconvolution choices.}

\item{tumor_type}{Character. Tumor type code (e.g., "stad") used by certain methods.}

\item{path}{Character. Output directory. Default is NULL (uses tempdir()).}

\item{permutation}{Integer. Number of permutations for CIBERSORT (and similar). Default is 1000.}
}
\value{
Data frame integrating cell fractions and signature scores (also writes intermediate outputs to disk).
}
\description{
Executes an integrated TME analysis on a gene expression matrix: performs
immune/stromal cell deconvolution using multiple algorithms, computes
signature scores, and aggregates results. Designed for exploratory
immunogenomic profiling.
}
\examples{
\dontrun{
lm22 <- load_data("lm22")
cancer_genes <- load_data("cancer_type_genes")
if (!is.null(lm22) && !is.null(cancer_genes)) {
  set.seed(123)
  genes <- rownames(lm22)
  xcell <- load_data("xCell.data")
  if (!is.null(xcell)) genes <- unique(c(genes, xcell$genes))
  genes <- unique(c(genes, cancer_genes[["stad"]]))
  eset <- matrix(runif(length(genes) * 2), nrow = length(genes), ncol = 2)
  rownames(eset) <- genes
  colnames(eset) <- paste0("Sample", 1:2)
  res <- iobr_deconvo_pipeline(
    eset = eset, project = "TEST",
    array = FALSE, tumor_type = "stad",
    path = tempdir(), permutation = 2
  )
  if (!is.null(res)) head(res)
}
}

}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\docType{data}
\name{tme_deconvolution_methods}
\alias{tme_deconvolution_methods}
\title{TME Deconvolution Methods}
\format{
A named character vector where names are display names and values
  are internal method names.
}
\usage{
tme_deconvolution_methods
}
\value{
Named character vector of available deconvolution methods.
}
\description{
A named vector of supported tumor microenvironment (TME) deconvolution
methods in the IOBR package.
}
\details{
The methods currently supported are:
\itemize{
  \item `mcpcounter`: MCP-counter for immune and stromal cell populations
  \item `epic`: EPIC for immune, stromal, and cancer cell fractions
  \item `xcell`: xCell for 64 immune and stromal cell types
  \item `cibersort`: CIBERSORT for 22 immune cell types
  \item `cibersort_abs`: CIBERSORT in absolute mode
  \item `ips`: Immunophenoscore calculation
  \item `estimate`: ESTIMATE for stromal/immune/estimate scores
  \item `svr`: Support Vector Regression (custom reference)
  \item `lsei`: Least Squares with Equality/Inequality constraints
  \item `timer`: TIMER for cancer-specific immune estimation
  \item `quantiseq`: quanTIseq for RNA-seq immune deconvolution
}
}
\examples{
# List all available TME deconvolution methods
tme_deconvolution_methods

# Get internal method name for a specific method
tme_deconvolution_methods["MCPcounter"]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_outlier_samples.R
\name{find_outlier_samples}
\alias{find_outlier_samples}
\title{Identify Outlier Samples in Gene Expression Data}
\usage{
find_outlier_samples(
  eset,
  yinter = -3,
  project = NULL,
  plot_hculst = FALSE,
  show_plot = TRUE,
  index = NULL,
  save = FALSE
)
}
\arguments{
\item{eset}{Numeric matrix. Gene expression data with genes as rows and
samples as columns.}

\item{yinter}{Numeric. Z-score threshold for identifying outliers.
Default is -3.}

\item{project}{Character or `NULL`. Output directory path for saving plots.
Required if `save = TRUE`. Default is `NULL`.}

\item{plot_hculst}{Logical. Whether to plot hierarchical clustering.
Default is `FALSE`.}

\item{show_plot}{Logical. Whether to display the connectivity plot.
Default is `TRUE`.}

\item{index}{Integer or `NULL`. Index for output file naming.
Default is `NULL`.}

\item{save}{Logical. Whether to save plots to files. Default is `FALSE`.}
}
\value{
Character vector of sample names identified as potential outliers.
}
\description{
Analyzes gene expression data to identify potential outlier samples using
connectivity analysis via the WGCNA package. Calculates normalized
adjacency and connectivity z-scores for each sample, generates connectivity
plots, and optionally performs hierarchical clustering.
}
\examples{
# Simulate data
set.seed(123)
sim_eset <- matrix(rnorm(100 * 10), 100, 10)
rownames(sim_eset) <- paste0("Gene", 1:100)
colnames(sim_eset) <- paste0("Sample", 1:10)

# Add one extreme outlier
sim_eset[, 10] <- sim_eset[, 10] + 50

# Identify outliers
if (requireNamespace("WGCNA", quietly = TRUE)) {
  outs <- find_outlier_samples(eset = sim_eset, show_plot = FALSE)
  print(outs)
}
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/feature_manipulation.R
\name{feature_manipulation}
\alias{feature_manipulation}
\title{Feature Quality Control and Filtering}
\usage{
feature_manipulation(
  data,
  feature = NULL,
  is_matrix = FALSE,
  print_result = FALSE
)
}
\arguments{
\item{data}{A matrix or data frame containing features to filter.}

\item{feature}{Character vector of feature names to check. If `is_matrix = TRUE`,
features are extracted from row names of the matrix.}

\item{is_matrix}{Logical indicating whether `data` is a gene expression matrix
(features as rows, samples as columns). If `TRUE`, the matrix is transposed
for processing. Default is `FALSE`.}

\item{print_result}{Logical indicating whether to print filtering statistics.
Default is `FALSE`.}
}
\value{
Character vector of feature names that pass all quality checks.
}
\description{
Filters features (variables) in a matrix or data frame by removing those with
missing values, non-numeric types, infinite values, or zero variance.
This is useful for preparing data for downstream statistical analyses.
}
\examples{
set.seed(123)
test_data <- data.frame(
  A = c(1, 2, 3),
  B = c(1, 1, 1), # zero variance
  C = c(1, NA, 3), # missing value
  D = c("a", "b", "c") # non-numeric
)
feas <- feature_manipulation(data = test_data, feature = colnames(test_data), print_result = TRUE)
print(feas)
}
\author{
Dongqiang Zeng
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateRef.R
\name{generateRef}
\alias{generateRef}
\title{Generate Reference Signature Matrix}
\usage{
generateRef(dds, pheno, FDR = 0.05, dat, method = "limma")
}
\arguments{
\item{dds}{Matrix. Raw count data from RNA-seq. Required if
`method = "DESeq2"`.}

\item{pheno}{Character vector. Cell type class of the samples.}

\item{FDR}{Numeric. Genes with BH adjusted p-value < FDR are considered
significant. Default is 0.05.}

\item{dat}{Matrix or data frame. Normalized transcript quantification data
(e.g., FPKM, TPM).}

\item{method}{Character. Method for differential expression: `"limma"` or
`"DESeq2"`. Default is `"limma"`.}
}
\value{
List containing:
  - `reference_matrix`: Data frame of median expression for significant
    genes across cell types.
  - `G`: Optimal number of probes minimizing condition number.
  - `condition_number`: Minimum condition number.
  - `whole_matrix`: Full median expression matrix.
}
\description{
Generates a reference signature matrix for cell types based on
differential expression analysis. Supports both limma for normalized data
and DESeq2 for raw count data.
}
\examples{
expressionData <- matrix(runif(1000 * 4, min = 0, max = 10), ncol = 4)
rownames(expressionData) <- paste("Gene", 1:1000, sep = "_")
colnames(expressionData) <- paste("Sample", 1:4, sep = "_")

phenotype <- c("celltype1", "celltype2", "celltype1", "celltype2")

rawCountData <- matrix(sample(1:100, 1000 * 4, replace = TRUE), ncol = 4)
rownames(rawCountData) <- paste("Gene", 1:1000, sep = "_")
colnames(rawCountData) <- paste("Sample", 1:4, sep = "_")

\donttest{
result <- generateRef(
  dds = rawCountData, pheno = phenotype,
  FDR = 0.05, dat = expressionData, method = "DESeq2"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/timer.R
\name{ParseInputExpression}
\alias{ParseInputExpression}
\title{Parse Input Gene Expression Data}
\usage{
ParseInputExpression(path)
}
\arguments{
\item{path}{Character. Path to a tab-delimited gene expression file.
First column should contain gene identifiers.}
}
\value{
Numeric matrix of gene expression values with genes as rows and
  samples as columns.
}
\description{
Reads gene expression data from a tab-delimited text file, using the first
column as row names. Converts data into a numeric matrix for analysis.
}
\examples{
tf <- tempfile(fileext = ".tsv")
expr <- data.frame(
  gene = c("GeneA", "GeneB", "GeneC"),
  Sample1 = c(10, 20, 30),
  Sample2 = c(15, 25, 35)
)
write.table(expr, tf, sep = "\t", row.names = FALSE, quote = FALSE)
gene_expression_data <- ParseInputExpression(tf)
print(gene_expression_data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_helper.R
\name{estimateScore}
\alias{estimateScore}
\title{estimateScore}
\usage{
estimateScore(
  input.ds,
  output.ds,
  platform = c("affymetrix", "agilent", "illumina")
)
}
\arguments{
\item{input.ds}{A character string specifying the path to the input dataset file in GCT format.
The file should have gene expression data with appropriate
headers.}

\item{output.ds}{A character string specifying the path to the output dataset file, where
the calculated scores will be written.}

\item{platform}{A character vector indicating the platform type. Must be one of "affymetrix",
"agilent", or "illumina". Platform-specific calculations
are performed
based on this parameter.}
}
\value{
This function does not return a value but writes the computed scores to the specified
        output file in GCT format.
}
\description{
This function reads a gene expression dataset in GCT format, calculates
enrichment scores
for specific gene sets, and writes the computed scores to an output file.
It supports multiple
platform types and performs platform-specific calculations if necessary.
}
\examples{
\dontrun{
set.seed(123)
si_geneset_data <- load_data("SI_geneset")
if (!is.null(si_geneset_data)) {
  gene_names <- unique(c(si_geneset_data[1, -1], si_geneset_data[2, -1]))
  gene_names <- gene_names[!is.na(gene_names) & gene_names != ""]
  gene_names <- head(gene_names, 200)
  n_genes <- length(gene_names)
  eset_sim <- as.data.frame(matrix(rnorm(n_genes * 2, mean = 5, sd = 1), n_genes, 2))
  rownames(eset_sim) <- gene_names
  colnames(eset_sim) <- c("Sample1", "Sample2")
  eset_sim <- tibble::rownames_to_column(eset_sim, var = "symbol")
  input_file <- tempfile(pattern = "estimate_", fileext = ".gct")
  output_file <- tempfile(pattern = "estimate_score_", fileext = ".gct")
  writeLines(c("#1.2", paste(nrow(eset_sim), ncol(eset_sim) - 1, sep = "\t")), input_file)
  utils::write.table(eset_sim, input_file, sep = "\t", row.names = FALSE,
                    col.names = TRUE, append = TRUE, quote = FALSE)
  score_res <- estimateScore(input.ds = input_file, output.ds = output_file,
                             platform = "affymetrix")
  if (!isFALSE(score_res) && file.exists(output_file)) {
    head(read.table(output_file, skip = 2, header = TRUE, sep = "\t"))
  }
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deconvo_tme.R
\name{deconvo_ref}
\alias{deconvo_ref}
\title{Deconvolve Using Custom Reference}
\usage{
deconvo_ref(
  eset,
  project = NULL,
  arrays = TRUE,
  method = c("svr", "lsei"),
  perm = 100,
  reference,
  scale_reference = TRUE,
  absolute.mode = FALSE,
  abs.method = "sig.score"
)
}
\arguments{
\item{eset}{Gene expression matrix.}

\item{project}{Optional project name. Default is `NULL`.}

\item{arrays}{Logical: use quantile normalization. Default is `TRUE`.}

\item{method}{Method: `"svr"` or `"lsei"`. Default is `"svr"`.}

\item{perm}{Permutations for SVR. Default is 100.}

\item{reference}{Custom reference matrix (e.g., lm22, lm6).}

\item{scale_reference}{Logical: scale reference. Default is `TRUE`.}

\item{absolute.mode}{Logical: absolute mode for SVR. Default is `FALSE`.}

\item{abs.method}{Method for absolute mode. Default is `"sig.score"`.}
}
\value{
Data frame with cell fractions. Columns suffixed with `_CIBERSORT`.
}
\description{
Cell fraction estimation using SVR or lsei methods with custom reference.
}
\examples{
# Simulate data
set.seed(123)
sim_ref <- matrix(rnorm(100 * 5), 100, 5)
rownames(sim_ref) <- paste0("Gene", 1:100)
colnames(sim_ref) <- paste0("CellType", 1:5)

sim_eset <- matrix(rnorm(100 * 3), 100, 3)
rownames(sim_eset) <- paste0("Gene", 1:100)
colnames(sim_eset) <- paste0("Sample", 1:3)

# Run deconvolution
result <- deconvo_ref(eset = sim_eset, reference = sim_ref, method = "lsei")
if (!is.null(result)) head(result)
}
\author{
Dongqiang Zeng, Rongfang Shen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/best_cutoff2.R
\name{best_cutoff2}
\alias{best_cutoff2}
\title{Extract Best Cutoff and Add Binary Variable to Data Frame}
\usage{
best_cutoff2(
  pdata,
  variable,
  time = "time",
  status = "status",
  print_result = TRUE
)
}
\arguments{
\item{pdata}{Data frame containing survival information and features.}

\item{variable}{Character string specifying the continuous variable name.}

\item{time}{Character string specifying the time-to-event column name.
Default is `"time"`.}

\item{status}{Character string specifying the event status column name.
Default is `"status"`.}

\item{print_result}{Logical indicating whether to print results.
Default is `TRUE`.}
}
\value{
List containing:
\describe{
  \item{pdata}{Data frame with binary variable added}
  \item{best_cutoff}{Numeric cutoff value}
  \item{cox_continuous_object}{Cox model summary for continuous variable}
  \item{summary_binary_variable}{Summary of binary variable}
  \item{cox_binary_object}{Cox model summary for binary variable}
}
}
\description{
Finds the best cutoff point for a continuous variable in survival analysis.
Takes input data containing a continuous variable and survival information
(time and status). Returns the modified input data with a new binary variable
created based on the best cutoff point.
}
\examples{
set.seed(123)
pdata <- data.frame(
  time = rexp(100),
  status = rbinom(100, 1, 0.5),
  score = rnorm(100, mean = 50, sd = 10)
)
result <- best_cutoff2(pdata, variable = "score", print_result = FALSE)
result$best_cutoff
}
\author{
Dongqiang Zeng
}
